@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,6 +20,7 @@
 #include "stdafx.h"
 
 #include "wb_component.h"
+#include "workbench/wb_context.h"
 #include "model/wb_model_diagram_form.h"
 #include "wb_layer_tree.h"
 #include "mdc_algorithms.h"
@@ -28,14 +29,15 @@
 
 using namespace bec;
 using namespace wb;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
+using namespace base;
 
 WBComponent::WBComponent(WBContext *context)
 : _wb(context)
 {
 }
 
+grt::GRT *WBComponent::get_grt() { return _wb->get_grt_manager()->get_grt(); }
+bec::GRTManager *WBComponent::get_grt_manager() { return _wb->get_grt_manager(); }
 
 
 grt::ValueRef WBComponent::place_object_grt(grt::GRT *grt, ModelDiagramForm *form, const Point &pos,
