@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,21 +20,36 @@
 #ifndef _WB_COMPONENT_H_
 #define _WB_COMPONENT_H_
 
-#include "workbench/wb_context.h"
+#include "workbench/wb_backend_public_interface.h"
+#include "base/trackable.h"
+#include "grtpp.h"
+#include "grtpp_util.h"
+#include "grts/structs.model.h"
+#include "base/geometry.h"
+
+namespace bec {
+  class GRTManager;
+};
+
+namespace mdc {
+  class CanvasItem;
+};
 
 namespace wb {
+  class WBContext;
+  struct WBOptions;
 
   class ModelDiagramForm;
 
-  class WBComponent : public sigc::trackable
+  class WBComponent : public base::trackable
   {
   public:
     WBComponent(WBContext *context);
     virtual ~WBComponent() {};
     
     inline WBContext *get_wb() { return _wb; }
-    inline grt::GRT *get_grt() { return _wb->get_grt_manager()->get_grt(); }
-    inline bec::GRTManager *get_grt_manager() { return _wb->get_grt_manager(); }
+    grt::GRT *get_grt();
+    bec::GRTManager *get_grt_manager();
 
     virtual std::string get_name()= 0;
     virtual std::string get_diagram_class_name() { return ""; }
@@ -73,7 +88,7 @@ namespace wb {
 
     virtual bool delete_model_object(const model_ObjectRef &object, std::map<std::string,bool> &options)= 0;
 
-    // drag&drop
+    // drag& 
     virtual std::vector<std::string> get_accepted_drop_types() const { return std::vector<std::string>(); }
     virtual bool accepts_drop(ModelDiagramForm *view, int x, int y, const std::string &type, const std::list<GrtObjectRef> &objects) { return false; }
     virtual bool accepts_drop(ModelDiagramForm *view, int x, int y, const std::string &type, const std::string &text) { return false; }
@@ -82,11 +97,11 @@ namespace wb {
     virtual bool perform_drop(ModelDiagramForm *view, int x, int y, const std::string &type, const std::string &text) { return false; }
 
   protected:
-    grt::ValueRef place_object(ModelDiagramForm *view, const MySQL::Geometry::Point &pos,
+    grt::ValueRef place_object(ModelDiagramForm *view, const base::Point &pos,
                       const std::string &object_struct,
                       const grt::DictRef &args= grt::DictRef());
     grt::ValueRef place_object_grt(grt::GRT *grt, ModelDiagramForm *view, 
-                                       const MySQL::Geometry::Point &pos, const std::string &object_struct,
+                                       const base::Point &pos, const std::string &object_struct,
                                        const grt::DictRef &args);
 
 
