@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,8 +21,9 @@
 
 #include <errno.h>
 #include "wb_component_basic.h"
+#include "wb_context_model.h"
 #include "model/wb_model_diagram_form.h"
-
+#include "workbench/wb_context.h"
 #include "grts/structs.workbench.model.h"
 #include "base/string_utilities.h"
 #include "base/file_functions.h"
@@ -31,8 +32,6 @@ using namespace wb;
 using namespace grt;
 using namespace bec;
 using namespace base;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
 
 struct HandToolContext
 {
@@ -146,9 +145,9 @@ void WBComponentBasic::setup_canvas_tool(ModelDiagramForm *view, const std::stri
     return;
   }
 
-  view->set_button_callback(sigc::bind<void*>(sigc::mem_fun(this, &WBComponentBasic::handle_button_event), data));
-  view->set_motion_callback(sigc::bind<void*>(sigc::mem_fun(this, &WBComponentBasic::handle_motion_event), data));
-  view->set_reset_tool_callback(sigc::bind<void*>(sigc::mem_fun(this, &WBComponentBasic::reset_tool), data));
+  view->set_button_callback(boost::bind(&WBComponentBasic::handle_button_event, this, _1, _2, _3, _4, _5, data));
+  view->set_motion_callback(boost::bind(&WBComponentBasic::handle_motion_event, this, _1, _2, _3, data));
+  view->set_reset_tool_callback(boost::bind(&WBComponentBasic::reset_tool, this, _1, data));
 }
 
 
@@ -248,9 +247,7 @@ void WBComponentBasic::place_layer(ModelDiagramForm *view, const Rect &rect)
   try
   {
     _wb->execute_in_grt_thread("Place new layer in model",
-                        sigc::bind<ModelDiagramForm*,Rect>(sigc::mem_fun(this, &WBComponentBasic::place_layer_grt),
-                                                           view,
-                                                           rect));
+                        boost::bind(&WBComponentBasic::place_layer_grt, this, _1, view, rect));
     _wb->show_status_text(_("Created new layer."));
   }
   catch (std::exception &exc)
@@ -293,7 +290,7 @@ void WBComponentBasic::delete_selection()
     }
 
     for (size_t c= objects.size(), i= 0; i < c; i++)
-      _wb->delete_object(objects[i], options);
+      _wb->get_model_context()->delete_object(objects[i], options);
   }
 }
 
@@ -335,7 +332,7 @@ void WBComponentBasic::delete_object(ModelDiagramForm *view, const Point &pos)
   {
     std::map<std::string, bool> options;
 
-    if (_wb->delete_object(object, options))
+    if (_wb->get_model_context()->delete_object(object, options))
       _wb->show_status_text(strfmt(_("Deleted %s"), object.get_metaclass()->get_attribute("caption").c_str()));
   }
 }
@@ -344,11 +341,11 @@ void WBComponentBasic::delete_object(ModelDiagramForm *view, const Point &pos)
 void WBComponentBasic::activate_canvas_object(const model_ObjectRef &figure, bool newwindow)
 {
   if (figure.is_instance(workbench_model_NoteFigure::static_class_name()))
-    _wb->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
+    _wb->get_grt_manager()->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
   else if (figure.is_instance(workbench_model_ImageFigure::static_class_name()))
-    _wb->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
+    _wb->get_grt_manager()->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
   else if (figure.is_instance(model_Layer::static_class_name()))
-    _wb->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
+    _wb->get_grt_manager()->open_object_editor(figure, newwindow ? bec::ForceNewWindowFlag : bec::NoFlags);
 }
 
 
@@ -592,7 +589,7 @@ bool WBComponentBasic::handle_motion_event(ModelDiagramForm *view, Point pos, md
       Point mouse_pos= hcontext->mouse_pos;
       Point viewport_pos= hcontext->viewport_pos;
 
-      Point newp= viewport_pos - wpos + mouse_pos;
+      Point newp= viewport_pos + Point((mouse_pos.x - wpos.x) / view->get_zoom(), (mouse_pos.y - wpos.y) / view->get_zoom());
 
       Rect r= cview->get_viewport();
       Size sz= cview->get_total_view_size();
