@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,7 +22,10 @@
 
 // Physical Model Handling
 
+#include "workbench/wb_backend_public_interface.h"
+
 #include "base/trackable.h"
+#include "base/notifications.h"
 
 #include "wb_config.h"
 
@@ -30,6 +33,8 @@
 
 #include "grt/icon_manager.h"
 
+#include "grts/structs.workbench.h"
+
 #include "wbcanvas/workbench_physical_model_impl.h"
 
 #define WB_DBOBJECT_DRAG_TYPE "x-mysql-wb/db.DatabaseObject"
@@ -67,7 +72,7 @@ namespace wb {
   #define WB_TOOL_PRELnm "physical/relnm"
   #define WB_TOOL_PREL_PICK "physical/relpick"
   
-  class MYSQLWBBACKEND_PUBLIC_FUNC WBComponentPhysical : virtual public WBComponent
+  class MYSQLWBBACKEND_PUBLIC_FUNC WBComponentPhysical : virtual public WBComponent, base::Observer
   {
   public:
     WBComponentPhysical(WBContext *wb);
@@ -92,9 +97,9 @@ namespace wb {
     void interactive_place_db_objects(ModelDiagramForm *vform, int x, int y, 
       const std::list<db_DatabaseObjectRef> &objects);
     
-    model_FigureRef place_db_object(ModelDiagramForm *view, const MySQL::Geometry::Point &pos, const db_DatabaseObjectRef &object,
+    model_FigureRef place_db_object(ModelDiagramForm *view, const base::Point &pos, const db_DatabaseObjectRef &object,
                          bool select_figure= true);
-    void place_new_db_object(ModelDiagramForm *view, const MySQL::Geometry::Point &pos, ObjectType type);
+    void place_new_db_object(ModelDiagramForm *view, const base::Point &pos, ObjectType type);
 
     void clone_db_object_to_schema(const db_SchemaRef &schema, const db_DatabaseObjectRef &object, grt::CopyContext &copy_context);
 
@@ -169,7 +174,7 @@ namespace wb {
       bool add_column(const db_ColumnRef &column);
       bool add_refcolumn(const db_ColumnRef &column);
 
-      void on_figure_crossed(const model_ObjectRef &owner, mdc::CanvasItem *item, bool enter, const MySQL::Geometry::Point &pos);
+      void on_figure_crossed(const model_ObjectRef &owner, mdc::CanvasItem *item, bool enter, const base::Point &pos);
       void enter_table(const workbench_physical_TableFigureRef& table);
       void leave_table(const workbench_physical_TableFigureRef& table);
             
@@ -180,11 +185,10 @@ namespace wb {
       
       void cancel();
 
-      bool button_press(ModelDiagramForm *view, const MySQL::Geometry::Point &pos);
+      bool button_press(ModelDiagramForm *view, const base::Point &pos);
     };
 
     virtual void load_app_options(bool update);
-    virtual void save_app_options();
 
     virtual void setup_context_grt(grt::GRT *grt, WBOptions *options);
 
@@ -247,9 +251,9 @@ namespace wb {
       const std::string &path);
 
     grt::ValueRef place_new_db_object_grt(grt::GRT *grt, ModelDiagramForm *view, 
-                                              const MySQL::Geometry::Point &pos, ObjectType type);
+                                              const base::Point &pos, ObjectType type);
     
-    RelationshipToolContext *start_relationship(ModelDiagramForm *view, const MySQL::Geometry::Point &pos, RelationshipType type);
+    RelationshipToolContext *start_relationship(ModelDiagramForm *view, const base::Point &pos, RelationshipType type);
     void cancel_relationship(ModelDiagramForm *view, RelationshipToolContext *rctx);
 
     bool create_nm_relationship_grt(grt::GRT *grt, ModelDiagramForm *view, workbench_physical_TableFigureRef table1, 
@@ -257,7 +261,7 @@ namespace wb {
                                     bool imandatory, bool fmandatory);
 
     grt::ValueRef place_db_object_grt(grt::GRT *grt, ModelDiagramForm *iew, 
-                                      const MySQL::Geometry::Point &pos, const db_DatabaseObjectRef &object,
+                                      const base::Point &pos, const db_DatabaseObjectRef &object,
                                       bool select_figure);
 
   private:
@@ -277,6 +281,8 @@ namespace wb {
     boost::signals2::connection _model_list_listener;
     boost::signals2::connection _catalog_object_list_listener;
 
+    virtual void handle_notification(const std::string &name, void *sender, base::NotificationInfo &info);
+    
     void refresh_ui_for_object(const GrtObjectRef &object);
 
     bool filter_catalog_tree(const bec::NodeId &node, const std::string &key, const grt::ValueRef &value, 
@@ -299,7 +305,7 @@ namespace wb {
 
     void schema_member_changed(const std::string &name, const grt::ValueRef &ovalue, const db_SchemaRef &schema);
 
-    bool handle_button_event(ModelDiagramForm*, mdc::MouseButton, bool, MySQL::Geometry::Point, mdc::EventState, void *data);
+    bool handle_button_event(ModelDiagramForm*, mdc::MouseButton, bool, base::Point, mdc::EventState, void *data);
 
   };
 
