@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,30 +27,30 @@
 #include "grts/structs.db.mysql.h"
 #include "base/string_utilities.h"
 
+#include "workbench/wb_context.h"
+#include "grtpp_undo_manager.h"
+
 #include "wbcanvas/workbench_physical_diagram_impl.h"
 #include "wbcanvas/workbench_physical_tablefigure_impl.h"
 
 using namespace wb;
 using namespace base;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
 
 WBComponentPhysical::RelationshipToolContext::RelationshipToolContext(WBComponentPhysical *owner, 
                                                                       ModelDiagramForm *form,
                                                                       RelationshipType rtype)
-: owner(owner), view(form), state(RPickingStart), floater(0), type(rtype)
+: owner(owner), view(form), state(RPickingStart), type(rtype), floater(0)
 {
   workbench_physical_Diagram::ImplData *view_bridge= workbench_physical_DiagramRef::cast_from(form->get_model_diagram())->get_data();
   if (view_bridge)
-    view_bridge->signal_item_crossed().connect(sigc::mem_fun(this, &RelationshipToolContext::on_figure_crossed));
+    scoped_connect(view_bridge->signal_item_crossed(),boost::bind(&RelationshipToolContext::on_figure_crossed, this, _1, _2, _3, _4));
     
   if (type == RelationshipPick)
   {
     floater= new RelationshipFloater(view);
     view->add_floater(floater);
     
-    floater->signal_done_clicked().
-      connect(sigc::mem_fun(this, &RelationshipToolContext::source_picking_done));
+    scoped_connect(floater->signal_done_clicked(),boost::bind(&RelationshipToolContext::source_picking_done, this));
     
     last_message= _("Select the Table to receive the Foreign Key or the Column(s) to turn into a Foreign Key.");
   }
