@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,17 +40,21 @@
 #include "base/file_utilities.h"
 #include "base/file_functions.h"
 #include "base/string_utilities.h"
+
 #include "mforms/utilities.h"
+#include "mforms/menubar.h"
+
 
 using namespace base;
 using namespace bec;
 using namespace wb;
+using namespace base;
 
 static std::map<std::string, std::string> auto_save_files;
 
 
 WBContextModel::WBContextModel(WBContextUI *wbui)
-: _wbui(wbui), _file(0), _auto_save_point(0), _last_auto_save_time(0), _auto_save_timer(NULL)
+: _wbui(wbui), _file(0), _locked_view_for_plugin_exec(0), _auto_save_point(0), _last_auto_save_time(0), _auto_save_timer(NULL)
 
 {  
   _overview= new PhysicalOverviewBE(_wbui->get_wb());
@@ -76,21 +80,25 @@ WBContextModel::WBContextModel(WBContextUI *wbui)
                              boost::bind(&WBContextModel::add_model_rgroup, this),
                              validate);
   
-  // used to update the currentDiagram field
-  scoped_connect(_wbui->signal_main_form_change(),boost::bind(&WBContextModel::update_current_diagram, this, _1));
-
+  base::NotificationCenter::get()->add_observer(this, "GNMainFormChanged");
+  
   // setup auto-save for model
   int interval = _wbui->get_wb()->get_root()->options()->options().get_int("workbench:AutoSaveModelInterval", 60);
   if (interval > 0)
     _auto_save_timer = _wbui->get_wb()->get_grt_manager()->run_every(boost::bind(&WBContextModel::auto_save_document, this), interval);
   _auto_save_interval = interval;
   
-  _wbui->get_wb()->get_root()->options()->signal_dict_changed()->connect(boost::bind(&WBContextModel::option_changed, this, _1, _2, _3));
+  scoped_connect(_wbui->get_wb()->get_root()->options()->signal_dict_changed(),boost::bind(&WBContextModel::option_changed, this, _1, _2, _3));
 }
 
 
 WBContextModel::~WBContextModel()
 {
+  base::NotificationCenter::get()->remove_observer(this);
+
+  if (_doc.is_valid() && _doc->physicalModels().is_valid() && _doc->physicalModels().count() > 0)
+    _doc->physicalModels().get(0)->get_data()->set_delegate(NULL);
+
   if (_auto_save_timer)
     _wbui->get_wb()->get_grt_manager()->cancel_timer(_auto_save_timer);
   CommandUI *cmdui = _wbui->get_command_ui();
@@ -105,6 +113,12 @@ WBContextModel::~WBContextModel()
 }
 
 
+grt::GRT *WBContextModel::get_grt()
+{
+  return _wbui->get_wb()->get_grt();
+}
+
+
 void WBContextModel::option_changed(grt::internal::OwnedDict*dict, bool, const std::string&key)
 {
   if (key == "workbench:AutoSaveModelInterval" && dict == _wbui->get_wb()->get_wb_options().valueptr())
@@ -128,12 +142,10 @@ bool WBContextModel::auto_save_document()
       && _file
       && doc.is_valid() 
       && !wb->get_grt_manager()->get_dispatcher()->get_busy()
-      && wb->get_grt()->get_undo_manager()->get_latest_closed_undo_action() != _auto_save_point
-      && !wb->get_filename().empty())
+      && wb->get_grt()->get_undo_manager()->get_latest_closed_undo_action() != _auto_save_point)
   {
     _auto_save_point = wb->get_grt()->get_undo_manager()->get_latest_closed_undo_action();
     _last_auto_save_time = now;
-    
     try
     {
       // save the document in the same directory containing the expanded mwb file
@@ -176,15 +188,19 @@ void WBContextModel::detect_auto_save_files(const std::string &autosave_dir)
 
   for (std::list<std::string>::const_iterator d = autosaves.begin(); d != autosaves.end(); ++d)
   { 
+    if (!g_file_test(d->c_str(), G_FILE_TEST_IS_DIR))
+      continue;
+
     if (base::LockFile::check(bec::make_path(*d, ModelFile::lock_filename.c_str())) != base::LockFile::NotLocked)
       continue;
     
     if (g_file_test(bec::make_path(*d, MAIN_DOCUMENT_AUTOSAVE_NAME).c_str(), G_FILE_TEST_EXISTS))
     {
+      std::string path = bec::make_path(*d, "real_path");
       gchar *orig_path;
       gsize length;
-      if (g_file_get_contents(bec::make_path(*d, "real_path").c_str(),
-                              &orig_path, &length, NULL))
+      if (g_file_test(path.c_str(), (GFileTest)(G_FILE_TEST_IS_REGULAR|G_FILE_TEST_EXISTS))
+          && g_file_get_contents(path.c_str(), &orig_path, &length, NULL))
       {
         files[std::string(orig_path, length)] = *d;
         g_free(orig_path);
@@ -435,7 +451,7 @@ mdc::CanvasView *WBContextModel::create_diagram_main(const model_DiagramRef &dia
   
   scoped_connect(diagram_reference->signal_objectActivated(),(boost::bind(&WBContextModel::activate_canvas_object, this, _1, _2)));
     
-  diagram_reference->signal_list_changed()->connect(boost::bind(&WBContextModel::diagram_object_list_changed, this, _1, _2, _3, diagram));
+  scoped_connect(diagram_reference->signal_list_changed(),boost::bind(&WBContextModel::diagram_object_list_changed, this, _1, _2, _3, diagram));
   
   register_diagram_form(diagram);
   
@@ -521,6 +537,13 @@ void WBContextModel::notify_diagram_destroyed(ModelDiagramForm *diagram)
 }
 
 
+void WBContextModel::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
+{
+  if (name == "GNMainFormChanged")
+    update_current_diagram(_wbui->get_active_main_form());
+}
+
+
 void WBContextModel::update_current_diagram(bec::UIForm *form)
 {
   ModelDiagramForm* dform= dynamic_cast<ModelDiagramForm*>(form);
@@ -541,7 +564,10 @@ void WBContextModel::diagram_object_changed(const std::string &member, const grt
   {
     if (view->get_model_diagram().is_valid())
     {
-      _wbui->get_wb()->request_refresh(RefreshViewName, view->get_model_diagram()->name(), (NativeHandle)view->get_view()->get_user_data());
+      base::NotificationInfo info;
+      info["form"] = view->form_id();
+      info["title"] = view->get_title();
+      base::NotificationCenter::get()->send("GNFormTitleDidChange", view, info);
       _wbui->get_physical_overview()->send_refresh_diagram(view->get_model_diagram());
     }
   }
@@ -585,7 +611,7 @@ void WBContextModel::add_model_schema()
 
 void WBContextModel::add_model_diagram()
 {
-  _wbui->get_wb()->add_new_diagram(get_active_model(true));
+  add_new_diagram(get_active_model(true));
 }
 
 void WBContextModel::add_model_table()
@@ -970,3 +996,254 @@ void WBContextModel::selection_changed()
 }
 
 
+
+#ifndef BasicExport____
+
+void WBContextModel::export_png(const std::string &path)
+{
+  ModelDiagramForm *form= dynamic_cast<ModelDiagramForm*>(_wbui->get_active_main_form());
+  if (form)
+  {
+    _wbui->get_wb()->show_status_text(strfmt(_("Exporting to %s..."), path.c_str()));
+    try
+    {
+      form->get_view()->export_png(path, true);
+      _wbui->get_wb()->show_status_text(strfmt(_("Exported diagram image to %s"), path.c_str()));
+    }
+    catch (const std::exception &exc)
+    {
+      _wbui->get_wb()->show_status_text(_("Could not export to PNG file."));
+      _wbui->get_wb()->show_exception(_("Export to PNG"), exc);
+    }
+  }
+  else
+    _wbui->get_wb()->show_error(_("Cannot Export Diagram"), _("Current diagram cannot be exported as image, please select a diagram first."));
+}
+
+
+void WBContextModel::export_pdf(const std::string &path)
+{
+  ModelDiagramForm *form= dynamic_cast<ModelDiagramForm*>(_wbui->get_active_main_form());
+  if (form)
+  {
+    Size size= form->get_view()->get_total_view_size();
+    double scale= _wbui->get_wb()->get_document()->pageSettings()->scale();
+    
+    size.width= size.width / scale * 2.834;
+    size.height= size.height / scale * 2.834;
+    
+    _wbui->get_wb()->show_status_text(strfmt(_("Exporting full model diagram to %s..."), path.c_str()));
+    
+    try
+    {
+      form->get_view()->export_pdf(path, size);
+      _wbui->get_wb()->show_status_text(strfmt(_("Exported PDF to %s"), path.c_str()));
+    }
+    catch (const std::exception &exc)
+    {
+      _wbui->get_wb()->show_status_text(_("Could not export to PDF"));
+      _wbui->get_wb()->show_exception(_("Export to PDF"), exc);
+    }
+  }
+  else
+    _wbui->get_wb()->show_error(_("Cannot Export Diagram"), _("Current diagram cannot be exported as image, please select a diagram first."));
+}
+
+
+void WBContextModel::export_svg(const std::string &path)
+{
+  ModelDiagramForm *form= dynamic_cast<ModelDiagramForm*>(_wbui->get_active_main_form());
+  if (form)
+  {
+    Size size= form->get_view()->get_total_view_size();
+    double scale= _wbui->get_wb()->get_document()->pageSettings()->scale();
+    
+    size.width= MM_TO_PT(size.width / scale);
+    size.height= MM_TO_PT(size.height / scale);
+    
+    _wbui->get_wb()->show_status_text(strfmt(_("Exporting full model diagram to %s..."), path.c_str()));
+    
+    try
+    {
+      form->get_view()->export_svg(path, size);
+      _wbui->get_wb()->show_status_text(strfmt(_("Exported SVG to %s"), path.c_str()));
+    }
+    catch (const std::exception &exc)
+    {
+      _wbui->get_wb()->show_status_text(_("Could not export to SVG"));
+      _wbui->get_wb()->show_exception(_("Export to SVG"), exc);
+    }
+  }
+  else
+    _wbui->get_wb()->show_error(_("Cannot Export Diagram"), _("Current diagram cannot be exported as image, please select a diagram first."));
+}
+
+
+void WBContextModel::export_ps(const std::string &path)
+{
+  ModelDiagramForm *form= dynamic_cast<ModelDiagramForm*>(_wbui->get_active_main_form());
+  if (form)
+  {
+    Size size= form->get_view()->get_total_view_size();
+    double scale= _wbui->get_wb()->get_document()->pageSettings()->scale();
+    
+    size.width= MM_TO_PT(size.width / scale);
+    size.height= MM_TO_PT(size.height / scale);
+    
+    _wbui->get_wb()->show_status_text(strfmt(_("Exporting full model diagram to %s..."), path.c_str()));
+    
+    try
+    {
+      form->get_view()->export_ps(path, size);
+      _wbui->get_wb()->show_status_text(strfmt(_("Exported PS to %s"), path.c_str()));
+    }
+    catch (std::exception &exc)
+    {
+      _wbui->get_wb()->show_status_text(_("Could not export to PS file"));
+      _wbui->get_wb()->show_exception(_("Export to PS File"), exc);
+    }
+  }
+  else
+    _wbui->get_wb()->show_error(_("Cannot Export Diagram"), _("Current diagram cannot be exported as image, please select a diagram first."));
+}
+
+#endif // BasicExport____
+
+
+#ifndef Diagrams_and_Canvas____
+
+//--------------------------------------------------------------------------------
+// Canvas Management
+
+
+void WBContextModel::add_new_diagram(const model_ModelRef &model)
+{
+  _wbui->get_wb()->show_status_text(_("Creating Diagram..."));
+  
+  model_DiagramRef view(model_DiagramRef::cast_from(_wbui->get_wb()->execute_in_grt_thread("Create new diagram",
+                                                                          boost::bind(&model_Model::addNewDiagram, &model.content(), true))));
+  
+  if (view.is_valid())
+  {
+    view->get_data()->realize();
+  }
+  
+  _wbui->get_wb()->show_status_text(_("Diagram added."));
+}
+
+
+void WBContextModel::switch_diagram(const model_DiagramRef &view)
+{
+  _wbui->get_wb()->switched_view(view->get_data()->get_canvas_view());
+}
+
+#endif // Diagrams_and_Canvas____
+#ifndef Canvas_Objects____
+/** 
+ ****************************************************************************
+ * @brief Delete a canvas object from the canvas/model
+ * 
+ * Will remove the figure from the model and delete all known
+ * references to it. There may be some references left, which will cause
+ * the GRT object and its bridge to stay alive for some more time, so the
+ * bridge is marked as invalid to prevent the canvas item from appearing
+ * on screen.
+ *
+ * @param figure to be removed
+ ****************************************************************************
+ */
+bool WBContextModel::delete_object(model_ObjectRef object, std::map<std::string, bool>& options)
+{
+  model_DiagramRef view(model_DiagramRef::cast_from(object->owner()));
+  
+  FOREACH_COMPONENT(_wbui->get_wb()->_components, iter)
+  {
+    if ((*iter)->handles_figure(object))
+      return (*iter)->delete_model_object(object, options);
+  }
+  return false;
+}
+
+
+void WBContextModel::delete_canvas_item(mdc::CanvasItem *item)
+{
+  model_DiagramRef view(get_view_with_id(item->get_layer()->get_view()->get_tag()));
+  
+  model_FigureRef figure;
+  figure= grt::find_object_in_list(view->figures(), item->get_tag());
+  if (figure.is_valid())
+  {
+    std::map<std::string, bool> map;
+    delete_object(figure, map);
+    return;
+  }
+  
+  model_ConnectionRef conn;
+  conn= grt::find_object_in_list(view->connections(), item->get_tag());
+  if (conn.is_valid())
+  {
+    std::map<std::string, bool> map;
+    delete_object(conn, map);
+    return;
+  }
+  
+  model_LayerRef layer;
+  layer= grt::find_object_in_list(view->layers(), item->get_tag());
+  if (layer.is_valid())
+  {
+    std::map<std::string, bool> map;
+    delete_object(layer, map);
+    return;
+  }
+  
+  throw std::logic_error("Could not find GRT object for canvas item");
+}
+
+#endif // Canvas_Objects____
+
+model_DiagramRef WBContextModel::get_view_with_id(const std::string &id)
+{
+  return model_DiagramRef::cast_from(get_wbui()->get_wb()->get_grt()->find_object_by_id(id, "/wb/doc"));
+}
+
+bool WBContextModel::delete_diagram(const model_DiagramRef &view)
+{
+  grt::AutoUndo undo(get_wbui()->get_wb()->get_grt());
+  view->owner()->diagrams().remove_value(view);
+  undo.end(strfmt(_("Delete Diagram '%s'"), view->name().c_str()));
+  
+#ifdef _WIN32
+  // in windows, a diagram is released as soon as the tab is closed. That means
+  // if a user closes a diagram before deleting it, the diagram is released before 
+  // it's removed from the list of diagrams in the list. That will cause the overview
+  // to be refreshed too early and with outdated contents. We force another explicit refresh
+  // here to workaround that.
+  if (get_wbui()->get_physical_overview())
+    get_wbui()->get_physical_overview()->send_refresh_diagram(model_DiagramRef());
+#endif
+  return true;
+}
+
+
+void WBContextModel::begin_plugin_exec()
+{
+  // lock the canvas so that it doesn't keep refreshing all the time.
+  // XXX we have to find some way to allow plugins control refresh freezing
+  ModelDiagramForm *view= dynamic_cast<ModelDiagramForm*>(get_wbui()->get_active_main_form());
+  _locked_view_for_plugin_exec = 0;
+  
+  if (view) 
+  {
+    _locked_view_for_plugin_exec = view->get_view();
+    _locked_view_for_plugin_exec->lock_redraw();
+  }  
+}
+
+
+void WBContextModel::end_plugin_exec()
+{
+  // form can get destroyed after a plugin is executed
+  if (_locked_view_for_plugin_exec && get_diagram_form(_locked_view_for_plugin_exec))
+    _locked_view_for_plugin_exec->unlock_redraw();
+  _locked_view_for_plugin_exec = 0;
+}
