@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,6 +29,7 @@
 #include "mdc_back_layer.h"
 #include "mforms/form.h"
 
+#include "model/wb_context_model.h"
 #include "model/wb_layer_tree.h"
 
 #include "grt/common.h"
@@ -43,15 +44,15 @@
 
 #include "wb_physical_model_diagram_features.h"
 #include "base/string_utilities.h"
-#include <boost/lambda/lambda.hpp>
+#include <boost/lambda/bind.hpp>
+
+#include "mforms/menubar.h"
 
 #define DEFAULT_TOOL "basic/select"
 
 using namespace wb;
 using namespace bec;
 using namespace base;
-using namespace MySQL::Drawing;
-using namespace MySQL::Geometry;
 
 static const double zoom_steps[]= {
   2.0,
@@ -122,6 +123,12 @@ ModelDiagramForm::~ModelDiagramForm()
   delete _features;
 }
 
+
+std::string ModelDiagramForm::get_form_context_name() const
+{
+  return WB_CONTEXT_MODEL; 
+}
+
 mforms::ToolBar *ModelDiagramForm::get_toolbar()
 {
   if (!_toolbar)
@@ -270,28 +277,28 @@ void ModelDiagramForm::revalidate_menu()
 }
 
 
-static ToolbarItemType get_toolbar_item_type(const std::string &type)
-{
-  if (type == "action")
-    return ToolbarAction;
-  else if (type == "separator")
-    return ToolbarSeparator;
-  else if (type == "toggle")
-    return ToolbarToggle;
-  else if (type == "radio")
-    return ToolbarRadio;
-  else if (type == "dropdown")
-    return ToolbarDropDown;
-  else if (type == "label")
-    return ToolbarLabel;
-  else if (type == "check")
-    return ToolbarCheck;
-  else if (type == "search")
-    return ToolbarSearch;
-  else
-    throw std::runtime_error("invalid toolbar item type "+type);
-  return ToolbarAction;
-}
+//static ToolbarItemType get_toolbar_item_type(const std::string &type)
+//{
+//  if (type == "action")
+//    return ToolbarAction;
+//  else if (type == "separator")
+//    return ToolbarSeparator;
+//  else if (type == "toggle")
+//    return ToolbarToggle;
+//  else if (type == "radio")
+//    return ToolbarRadio;
+//  else if (type == "dropdown")
+//    return ToolbarDropDown;
+//  else if (type == "label")
+//    return ToolbarLabel;
+//  else if (type == "check")
+//    return ToolbarCheck;
+//  else if (type == "search")
+//    return ToolbarSearch;
+//  else
+//    throw std::runtime_error("invalid toolbar item type "+type);
+//  return ToolbarAction;
+//}
 
 
 void ModelDiagramForm::update_options_toolbar()
@@ -970,8 +977,7 @@ void ModelDiagramForm::reset_tool(bool notify)
     _reset_tool(this);
 
   _cursor= "";
-  static const bool fval = false;
-  boost::function<bool ()> f = boost::lambda::var(fval);
+  boost::function<bool ()> f = boost::lambda::constant(false);
   _handle_button = (boost::bind(f));
   _handle_motion = (boost::bind(f));
   _reset_tool = (boost::bind(f));
@@ -1296,7 +1302,7 @@ int ModelDiagramForm::delete_selection(bool noquestions)
       options["deletedbobjects"]= false;
   }
   for (size_t c= objects.size(), i= 0; i < c; i++)
-    _owner->get_wb()->delete_object(objects[i], options);
+    _owner->get_wb()->get_model_context()->delete_object(objects[i], options);
 
   um->end_undo_group();
   um->set_action_description(strfmt(_("Delete %s"), edit_target_name.c_str()));
