@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,7 +20,7 @@
 #ifndef _WB_MODEL_DIAGRAM_FORM_H_
 #define _WB_MODEL_DIAGRAM_FORM_H_
 
-#include "workbench/wb_context.h"
+#include "workbench/wb_command_ui.h"
 #include "model/wb_layer_tree.h"
 #include "grt/icon_manager.h"
 #include "base/ui_form.h"
@@ -28,6 +28,7 @@
 #include "base/drawing.h"
 
 #include "mforms/menu.h"
+#include "mforms/toolbar.h"
 
 namespace mforms { class ToolBarItem; };
 namespace MySQL {
@@ -38,7 +39,9 @@ namespace MySQL {
 
 namespace wb {
   class Floater;  
-  
+  class WBContext;
+  class WBComponent;
+
   enum EditFinishReason
   {
     EditCancelled,
@@ -84,7 +87,7 @@ namespace wb {
     void attach_canvas_view(mdc::CanvasView *cview);
 
     virtual bool is_main_form() { return true; }
-    virtual std::string get_form_context_name() const { return WB_CONTEXT_MODEL; }
+    virtual std::string get_form_context_name() const;
 
     mdc::CanvasView *get_view() { return _view; }
     model_DiagramRef &get_model_diagram() { return _model_diagram; }
@@ -95,7 +98,7 @@ namespace wb {
     void set_closed(bool flag);
     bool is_closed();
     
-    mdc::CanvasItem *get_leaf_item_at(const MySQL::Geometry::Point &pos);
+    mdc::CanvasItem *get_leaf_item_at(const base::Point &pos);
     
     bec::Clipboard *get_clipboard();
     
@@ -133,8 +136,8 @@ namespace wb {
     void zoom_in();
     void zoom_out();
 
-    void set_button_callback(const boost::function<bool (ModelDiagramForm*, mdc::MouseButton, bool, MySQL::Geometry::Point, mdc::EventState)> &cb);
-    void set_motion_callback(const boost::function<bool (ModelDiagramForm*, MySQL::Geometry::Point, mdc::EventState)> &cb);
+    void set_button_callback(const boost::function<bool (ModelDiagramForm*, mdc::MouseButton, bool, base::Point, mdc::EventState)> &cb);
+    void set_motion_callback(const boost::function<bool (ModelDiagramForm*, base::Point, mdc::EventState)> &cb);
     void set_reset_tool_callback(const boost::function<void (ModelDiagramForm*)> &cb);
     
     std::string get_tool() { return _tool; }
@@ -165,7 +168,7 @@ namespace wb {
     bool handle_key(const mdc::KeyInfo &key, bool press, mdc::EventState state);
 
     bool current_mouse_position(int &x, int &y);
-    bool current_mouse_position(MySQL::Geometry::Point &pos);
+    bool current_mouse_position(base::Point &pos);
 
     // drag&drop
     std::vector<std::string> get_accepted_drop_types();
@@ -177,9 +180,9 @@ namespace wb {
     bool perform_drop(int x, int y, const std::string &type, const std::string &text);
 
 
-    model_LayerRef get_layer_at(const MySQL::Geometry::Point &pos, MySQL::Geometry::Point &offset);
-    model_LayerRef get_layer_bounding(const MySQL::Geometry::Rect &rect, MySQL::Geometry::Point &offset);
-    model_ObjectRef get_object_at(const MySQL::Geometry::Point &pos);
+    model_LayerRef get_layer_at(const base::Point &pos, base::Point &offset);
+    model_LayerRef get_layer_bounding(const base::Rect &rect, base::Point &offset);
+    model_ObjectRef get_object_at(const base::Point &pos);
 
 
     mdc::Layer *get_floater_layer();
@@ -195,7 +198,7 @@ namespace wb {
 
 
     // inline editing
-    void begin_editing(const MySQL::Geometry::Rect &rect, const std::string &text, float text_size, bool multiline);
+    void begin_editing(const base::Rect &rect, const std::string &text, float text_size, bool multiline);
     void stop_editing();
     boost::signals2::signal<void (std::string,EditFinishReason)>* signal_editing_done() { return &_signal_editing_done; }
 
@@ -211,7 +214,7 @@ namespace wb {
   protected:
     struct OldPosition
     {
-      MySQL::Geometry::Point pos;
+      base::Point pos;
       std::string layer_id;
     };
 
@@ -246,8 +249,8 @@ namespace wb {
     
     boost::signals2::signal<void (std::string)> _tool_argument_changed;
 
-    boost::function<bool (ModelDiagramForm*, mdc::MouseButton, bool, MySQL::Geometry::Point, mdc::EventState)> _handle_button;
-    boost::function<bool (ModelDiagramForm*, MySQL::Geometry::Point, mdc::EventState)> _handle_motion;
+    boost::function<bool (ModelDiagramForm*, mdc::MouseButton, bool, base::Point, mdc::EventState)> _handle_button;
+    boost::function<bool (ModelDiagramForm*, base::Point, mdc::EventState)> _handle_motion;
     boost::function<void (ModelDiagramForm*)> _reset_tool;
 
 
@@ -258,8 +261,8 @@ namespace wb {
     std::string _old_tool;
     std::string _old_cursor;
     boost::function<void (ModelDiagramForm*)> _old_reset_tool;
-    boost::function<bool (ModelDiagramForm*, mdc::MouseButton, bool, MySQL::Geometry::Point, mdc::EventState)> _old_handle_button;
-    boost::function<bool (ModelDiagramForm*, MySQL::Geometry::Point, mdc::EventState)> _old_handle_motion;
+    boost::function<bool (ModelDiagramForm*, mdc::MouseButton, bool, base::Point, mdc::EventState)> _old_handle_button;
+    boost::function<bool (ModelDiagramForm*, base::Point, mdc::EventState)> _old_handle_motion;
 
     void clipboard_changed();
 
@@ -279,7 +282,7 @@ namespace wb {
     void select_dropdown_item(const std::string &option, mforms::ToolBarItem *item);
     void toggle_checkbox_item(const std::string &name, const std::string &option, bool state);
     
-    //void hover_in_figure(mdc::CanvasItem *item, MySQL::Geometry::Point pos);
+    //void hover_in_figure(mdc::CanvasItem *item, base::Point pos);
     //void hover_out_figure(mdc::CanvasItem *item);
     private:
       int _update_count; // If > 0 don't refresh depending structures.
