@@ -35,6 +35,7 @@ class SqlEditorForm;
 namespace wb 
 {
   class WBContextUI;
+  class CommandUI;
   
   class MYSQLWBBACKEND_PUBLIC_FUNC WBContextSQLIDE : public base::trackable, base::Observer
   {
@@ -58,6 +59,7 @@ namespace wb
     boost::shared_ptr<SqlEditorForm> create_connected_editor(const db_mgmt_ConnectionRef &conn);
 
     WBContextUI *get_wbui() { return _wbui; }
+    CommandUI *get_cmdui();
     bec::GRTManager *get_grt_manager();
     
     SqlEditorForm* get_active_sql_editor();
