@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,17 +24,19 @@
 #include "grtpp.h"
 #include "grts/structs.app.h"
 
+#include "workbench/wb_context_names.h"
 #include "mdc_image_manager.h"
 #include "home_screen.h"
 #include "base/string_utilities.h"
 
+#include "mforms/label.h"
 #include "mforms/button.h"
 #include "mforms/imagebox.h"
 #include "mforms/menubar.h"
 
 using namespace wb;
 using namespace mforms;
-using namespace MySQL::Geometry;
+using namespace base;
 
 #ifdef __APPLE__
 #define HOME_NORMAL_FONT "Lucida Grande"
@@ -145,7 +147,7 @@ WorkbenchCentral::~WorkbenchCentral()
   delete_surface(_more_starters_icon);
   delete_surface(_default_starter_icon);
 
-  void clear_starters();
+  clear_starters();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -279,7 +281,7 @@ void WorkbenchCentral::repaint(cairo_t* cr, int areax, int areay, int areaw, int
       // Starter buttons are right aligned if they all fit into the available space. Otherwise we
       // distribute the remaining space equally to all starter distances.
       int remaining_space= 0;
-      if (starter_count == _starters.size())
+      if (starter_count == (int)_starters.size())
         current_offset += available_width - total_width;
       else
         remaining_space= available_width - total_width;
@@ -1125,7 +1127,7 @@ void ActionLinkBox::add_link(ActionLink* link)
     }
     else
     {
-      int group_separator_position = link->get_title().find("/");
+      const size_t group_separator_position = link->get_title().find("/");
      
       // When it's a grouped link it's needed to find the position where
       // it should be inserted so it is displayed inside the group on the list
@@ -1224,6 +1226,7 @@ void ActionLinkBox::select(grt::ValueRef item)
         set_selected(selected);
         break;
       case grt::IntegerType:
+      {
         int index = grt::IntegerRef::extract_from(item);
 
         if (index >= 0 && index < (int)_links.size())
@@ -1231,6 +1234,9 @@ void ActionLinkBox::select(grt::ValueRef item)
         else
           set_selected(NULL);
       
+        break;
+      }
+      default:
         break;
     }
   }
@@ -1621,7 +1627,7 @@ void GradientBox::repaint(cairo_t *cr, int areax, int areay, int areaw, int area
 
 ActionList::ActionList(HomeScreen* owner, int left_spacing, int right_spacing, WorkspaceGradient* spacer_gradient,
                        const std::string& icon_name, HomeScreenAction action, bool handle_groups)
-: Box(true), _panel(mforms::ScrollPanelDrawBackground), _panel_border(false)
+: Box(true), _panel_border(false), _panel(mforms::ScrollPanelDrawBackground)
 {
   _owner= owner;
   _action= action;
@@ -1692,7 +1698,6 @@ void ActionList::add_entry(const std::string& title, const std::string& descript
 //  bool is_group = false;
   bool group_expanded = true;
   ActionLinkGroup *link_group=NULL;
-  ActionLink* link=NULL;
 
   // When handling groups it may be needed to add not only the entry but
   // an entry for the group if it doesn't yet exist
@@ -1723,16 +1728,13 @@ void ActionList::add_entry(const std::string& title, const std::string& descript
         link_group = _link_groups[group_name];
     }
   }
-  
 
-  link = new ActionLink(icon ? icon : _icon, title, description, object, _action, true);
+  ActionLink *link = new ActionLink(icon ? icon : _icon, title, description, object, _action, true);
   if (link_group)
     link_group->add_link(link);
-  
-  
   if (!link_group || group_expanded)
     _link_box->add_link(link);
-  
+
   _link_box->set_needs_repaint();
 }
 
@@ -2066,6 +2068,8 @@ void Workspace::add_list_entry(HomeScreenListType list, const std::string& title
     case HomeScreenAdministrationList:
       _server_instances->add_entry(title, description, object, icon);
       break;
+    default:
+      break;
   }
 }
 
@@ -2084,6 +2088,8 @@ void Workspace::select_list_entry(HomeScreenListType list, grt::ValueRef item)
     case HomeScreenAdministrationList:
       _server_instances->select(item);
       break;
+    default:
+      break;
   }  
 }
 
@@ -2102,6 +2108,8 @@ void Workspace::clear_list(HomeScreenListType list)
     case HomeScreenAdministrationList:
       _server_instances->clear();
       break;
+    default:
+      break;
   }
 }
 
@@ -2120,6 +2128,8 @@ void Workspace::refresh_list(HomeScreenListType list)
     case HomeScreenAdministrationList:
       _server_instances->refresh();
       break;
+    default:
+      break;
   }
 }
 
@@ -2135,6 +2145,8 @@ void Workspace::toggle_list_group(HomeScreenListType list, const std::string &gr
     case HomeScreenAdministrationList:
       _server_instances->toggle_group(group_name);
       break;
+    default:
+      break;
   }  
 }
 
@@ -2165,6 +2177,8 @@ void Workspace::set_context_menu_items(HomeScreenListType list, const bec::MenuI
     case HomeScreenAdministrationList:
       _server_instances->set_context_menu(menu, group_menu);
       break;
+    default:
+      break;
   }
 }
 
@@ -2180,6 +2194,8 @@ void Workspace::set_collapsed_group_data(HomeScreenListType list, std::string gr
     case HomeScreenAdministrationList:
       _server_instances->set_collapsed_groups(group_data);
       break;
+    default:
+      break;
   }
 }
 
@@ -2197,6 +2213,8 @@ std::string Workspace::get_collapsed_group_data(HomeScreenListType list)
     case HomeScreenAdministrationList:
       group_data = _server_instances->get_collapsed_groups();
       break;
+    default:
+      break;
   }
   
   return group_data;
@@ -2219,6 +2237,8 @@ void Workspace::handle_command(const std::string& command, HomeScreenListType li
     case HomeScreenAdministrationList:
       object = _server_instances->selected()->get_object();
       break;
+    default:
+      break;
   }      
 
   _owner->handle_context_menu(list, object, command);
@@ -2254,7 +2274,7 @@ _workspace_section(false, "Workspace"), _callback(NULL), _user_data(0), _auto_sa
 {  
   // TODO: cannot set a minimum size here as this doesn't work on OS X (box layouter is buggy).
   //set_size(1024, 671); // Minimum size for this entire page.
-  _central_content= new WorkbenchCentral(this, grt);
+  _central_content = new WorkbenchCentral(this, grt);
   _central_content->set_size(-1, 112);
   _workbench_central_section.set_content(_central_content);
   add(&_workbench_central_section, false, true);
@@ -2273,7 +2293,6 @@ _workspace_section(false, "Workspace"), _callback(NULL), _user_data(0), _auto_sa
 
 HomeScreen::~HomeScreen()
 {
-  delete _auto_save_warning;
   delete _central_content;
   delete _workspace_content;
 }
@@ -2301,7 +2320,6 @@ void HomeScreen::show_autosave_warning(int model, int sqleditor)
     if (_auto_save_warning)
     {
       remove(_auto_save_warning);
-      delete _auto_save_warning;
       _auto_save_warning= 0;
     }
   }
@@ -2341,7 +2359,7 @@ void HomeScreen::show_autosave_warning(int model, int sqleditor)
     box->set_back_color("#eeeef7");
     add(box, false, true);
     
-    UIForm::scoped_connect(button->signal_clicked(),boost::bind(&Box::remove, this, box));
+    UIForm::scoped_connect(button->signal_clicked(),boost::bind(&HomeScreen::show_autosave_warning, this, 0, 0));
   }
 }
 
@@ -2433,3 +2451,22 @@ void HomeScreen::toggle_list_group(HomeScreenListType list, const std::string &g
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Called when the initial setup for the home screen is finished, so we can listen now for 
+ * size events.
+ */
+void HomeScreen::startup_finished()
+{
+  Box::scoped_connect(signal_resized(), boost::bind(&HomeScreen::on_resize, this));
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void HomeScreen::on_resize()
+{
+  if (get_height() < 500 && _workbench_central_section.get_expanded())
+    _workbench_central_section.set_expanded(false);
+}
+
+//--------------------------------------------------------------------------------------------------
+
