@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,9 +21,16 @@
 #define _HOME_SCREEN_H_
 
 #include "wb_backend_public_interface.h"
-#include "mforms/mforms.h"
 #include <map>
 
+#include "mforms/drawbox.h"
+#include "mforms/scrollpanel.h"
+#include "mforms/menu.h"
+#include "mforms/box.h"
+#include "mforms/appview.h"
+#include "mforms/sectionbox.h"
+#include "mforms/utilities.h"
+
 namespace wb
 {
   /**
@@ -51,7 +58,7 @@ namespace wb
     ActionNewEERModel,
     ActionNewModelFromDB,
     ActionNewModelFromScript,
-    ActionSyncScript,
+//    ActionSyncScript,
 
     ActionServerAdministration,
     ActionManageInstanceFromList,
@@ -94,12 +101,12 @@ namespace wb
     app_StarterRef starter;
 
     cairo_surface_t* icon;
-    std::string title_line1;                 // The title is split into at most two lines.
+    std::string title_line1;     // The title is split into at most two lines.
     std::string title_line2;
-    MySQL::Geometry::Rect bounds;           // Overall bounds of the entry.
-    MySQL::Geometry::Rect icon_bounds;      // Position and size of the icon relative to overall bounds.
-    MySQL::Geometry::Point title1_location; // Location of the first title line relative to overall bounds.
-    MySQL::Geometry::Point title2_location; // Dito for the second line.
+    base::Rect bounds;           // Overall bounds of the entry.
+    base::Rect icon_bounds;      // Position and size of the icon relative to overall bounds.
+    base::Point title1_location; // Location of the first title line relative to overall bounds.
+    base::Point title2_location; // Ditto for the second line.
   } StarterEntry;
 
   /**
@@ -130,13 +137,13 @@ namespace wb
     cairo_surface_t* _main_icon;
     cairo_surface_t* _pointer_icon;
     cairo_surface_t* _title_image;
-    MySQL::Geometry::Rect _title_bounds;
-    MySQL::Geometry::Rect _chapter1_title_bounds;
-    MySQL::Geometry::Rect _chapter1_details_bounds;
-    MySQL::Geometry::Rect _arrow1_bounds;
+    base::Rect _title_bounds;
+    base::Rect _chapter1_title_bounds;
+    base::Rect _chapter1_details_bounds;
+    base::Rect _arrow1_bounds;
 
     cairo_surface_t* _more_starters_icon;
-    MySQL::Geometry::Rect _more_starters_icon_bounds;
+    base::Rect _more_starters_icon_bounds;
     cairo_surface_t* _default_starter_icon;
     std::vector<StarterEntry> _starters;
     int _starter_area_height;                        // Size of the largest starter entry.
@@ -159,7 +166,7 @@ namespace wb
                const grt::ValueRef &object, HomeScreenAction action, bool enabled, ActionLinkType type = NormalLink);
     ~ActionLink();
     
-    virtual void paint(cairo_t* cr, MySQL::Geometry::Rect bounds, bool hot, bool active);
+    virtual void paint(cairo_t* cr, base::Rect bounds, bool hot, bool active);
     bool contains(double x, double y);
     HomeScreenAction get_action() { return _action; };
     grt::ValueRef get_object() { return _object; }
@@ -187,7 +194,7 @@ namespace wb
     double _title_offset;              // Vertical position of the title.
     double _description_offset;        // Ditto for description.
     double _title_width;               // Width of the (possibly shortened) title. For text decoration.
-    MySQL::Geometry::Rect _bounds;     // The link's bounds when it was drawn the last time.
+    base::Rect _bounds;     // The link's bounds when it was drawn the last time.
     int _text_height;
     HomeScreenAction _action;          // Which action does this link trigger?
     grt::ValueRef _object;
@@ -207,7 +214,7 @@ namespace wb
     
     ~ActionLinkGroup();
     
-    virtual void paint(cairo_t* cr, MySQL::Geometry::Rect bounds, bool hot, bool active);
+    virtual void paint(cairo_t* cr, base::Rect bounds, bool hot, bool active);
     
     void add_link(cairo_surface_t *icon, const std::string& title,
                                    const std::string& description, const grt::ValueRef &object, HomeScreenAction action, bool enabled);
@@ -241,9 +248,9 @@ namespace wb
     
     boost::signals2::signal<void (std::string&)> _toggled;
     
-    MySQL::Geometry::Rect _arrow_bounds;   // The group arrow icon bounds.
+    base::Rect _arrow_bounds;   // The group arrow icon bounds.
 
-    MySQL::Geometry::Rect _bar_bounds;     // The group bar bounds.
+    base::Rect _bar_bounds;     // The group bar bounds.
     double _count_width;                   // Width of the count of group items.
   };
   
@@ -271,7 +278,7 @@ namespace wb
   /**
    * Container for any number of ActionLink instances.
    */
-  class ActionLinkBox : public mforms::DrawBox, public base::trackable
+  class ActionLinkBox : public mforms::DrawBox
   {
   public:
     ActionLinkBox(HomeScreen* owner, const std::string& header_image, WorkspaceGradient* gradient,
@@ -336,6 +343,7 @@ namespace wb
     bool _can_select;
     bool _single_click;
     int _selected_index;
+    mforms::MouseButton _last_mouse_button;
     boost::signals2::signal<void ()> _signal_changed;
     boost::signals2::signal<void (const std::string&)> _group_toggled;
 
@@ -470,6 +478,7 @@ namespace wb
   private:
     mforms::SectionBox _workbench_central_section;
     WorkbenchCentral* _central_content;
+
     mforms::SectionBox _workspace_section;
     Workspace* _workspace_content;
     home_screen_action_callback _callback;
@@ -490,7 +499,6 @@ namespace wb
     void set_context_menu_items(HomeScreenListType list, const bec::MenuItemList &menu_items, bool group_menu = false);
     void set_collapsed_group_data(HomeScreenListType list, std::string group_data);
     std::string get_collapsed_group_data(HomeScreenListType list);
-    
 
     void set_callback(home_screen_action_callback callback, void* user_data);
     void trigger_callback(HomeScreenAction action, const grt::ValueRef &object);
@@ -500,6 +508,10 @@ namespace wb
     void clear_list(HomeScreenListType list);
     void refresh_list(HomeScreenListType list);
     void toggle_list_group(HomeScreenListType list, const std::string &group_name);
+
+    void startup_finished();
+
+    void on_resize();
   };
 }
 
