@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -225,7 +225,7 @@ void StarterPopup::draw_entry(cairo_t* cr, app_StarterRef starter, int x, int y,
   double button_height= image_height(_action_button_parts[0]);
 
   // First add this entry to our internal list.
-  PopupEntry entry= {starter, MySQL::Geometry::Rect(x, y, w, h + button_height / 2)};
+  PopupEntry entry= {starter, base::Rect(x, y, w, h + button_height / 2)};
   _entries.push_back(entry);
 
   cairo_surface_t* icon= Utilities::load_icon(starter->largeIcon());
@@ -420,9 +420,9 @@ void StarterPopup::handle_command(const std::string& command)
   else
     if (command == "start_plugin_external")
       action= StarterStartPluginExternal;
-    else
-      if (command == "find_plugin")
-        action= StarterFindPlugin;
+//    else
+//      if (command == "find_plugin")
+//        action= StarterFindPlugin;
       else
         if (command == "author_home")
           action= StarterAuthorHome;
