@@ -1,19 +1,39 @@
+/* 
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 
 // High-level testing for Workbench
 // This tests WBContext, which will test the integration of all components.
 
-#include "stdafx.h"
+#include "tut_stdafx.h"
 
 #include <memory>
-#include "test.h"
 #include "wb_helpers.h"
 #include "grtdb/db_object_helpers.h"
 #include "grt_test_utility.h"
 
 using namespace wb;
-using namespace MySQL::Geometry;
+using namespace base;
 
 BEGIN_TEST_DATA_CLASS(wb_context)
+protected:
+  WBTester wb;
 END_TEST_DATA_CLASS;
 
 TEST_MODULE(wb_context, "high-level tests for Workbench");
@@ -22,8 +42,6 @@ TEST_FUNCTION(2)
 {
   // test creating new document
   
-  WBTester wb;
-
   wb.wb->new_document();
 
   ensure("doc", wb.wb->get_document().is_valid());
@@ -35,8 +53,6 @@ TEST_FUNCTION(2)
 TEST_FUNCTION(3)
 {
   // test loading documents (saved as xml)
-  WBTester wb;
-
   // check if the test file is correct
   {
     FILE *f= fopen("data/workbench/test_model1.mwb", "r");
@@ -120,22 +136,20 @@ TEST_FUNCTION(3)
 TEST_FUNCTION(5)
 {
   // bug: opening a model with selection will cause a crash
-  WBTester tester;
+  wb.wb->new_document();
 
-  tester.wb->new_document();
+  wb.add_view();
 
-  tester.add_view();
-
-  tester.add_table_figure("table", 10, 10);
-  model_DiagramRef view(tester.get_pmodel()->diagrams()[0]);
+  wb.add_table_figure("table", 10, 10);
+  model_DiagramRef view(wb.get_pmodel()->diagrams()[0]);
 
   ensure_equals("check selection", view->selection().count(), 1U);
 
-  tester.wb->save_as("test4.mwb");
+  wb.wb->save_as("test4.mwb");
 
-  tester.wb->open_document("test4.mwb");
+  wb.wb->open_document("test4.mwb");
 
-  view= tester.get_pmodel()->diagrams()[0];
+  view= wb.get_pmodel()->diagrams()[0];
 
   ensure("check loaded doc", view->figures().count()==1);
   ensure_equals("check loaded doc", view->selection().count(),1U);
@@ -266,49 +280,47 @@ TEST_FUNCTION(7)
   // bug: deleting tables with relationship won't delete the connection
 
   {
-    WBTester tester;
+    wb.wb->open_document("data/workbench/2tables_1fk.mwb");
+    ensure("load doc", wb.wb->get_document().is_valid());
+    ensure_equals("contains tables", wb.get_schema()->tables().count(), 2U);
+    ensure_equals("no figures", wb.get_pview()->figures().count(), 0U);
+    ensure_equals("no connections", wb.get_pview()->connections().count(), 0U);
 
-    tester.wb->open_document("data/workbench/2tables_1fk.mwb");
-    ensure("load doc", tester.wb->get_document().is_valid());
-    ensure_equals("contains tables", tester.get_schema()->tables().count(), 2U);
-    ensure_equals("no figures", tester.get_pview()->figures().count(), 0U);
-    ensure_equals("no connections", tester.get_pview()->connections().count(), 0U);
-
-    tester.sync_view();
+    wb.sync_view();
 
     std::list<db_DatabaseObjectRef> objects;
-    objects.push_back(grt::find_named_object_in_list(tester.get_schema()->tables(), "table1"));
-    objects.push_back(grt::find_named_object_in_list(tester.get_schema()->tables(), "table2"));
+    objects.push_back(grt::find_named_object_in_list(wb.get_schema()->tables(), "table1"));
+    objects.push_back(grt::find_named_object_in_list(wb.get_schema()->tables(), "table2"));
 
-    tester.interactive_place_db_objects(10, 150, objects);
+    wb.interactive_place_db_objects(10, 150, objects);
 
-    ensure_equals("place 2 tables", tester.get_pview()->figures().count(), 2U);
-    ensure_equals("connections created", tester.get_pview()->connections().count(), 1U);
+    ensure_equals("place 2 tables", wb.get_pview()->figures().count(), 2U);
+    ensure_equals("connections created", wb.get_pview()->connections().count(), 1U);
 
     // Delete 1 of the tables and see if connection is gone.
     // Don't use silent mode to ensure the underlying db object stays intact.
     std::map<std::string, bool> opts;
     opts["silent-mode"]= false;
 
-    model_FigureRef figure= tester.get_pview()->figures()[0];
+    model_FigureRef figure= wb.get_pview()->figures()[0];
 
-    ensure("root global", tester.wb->get_root()->is_global());
-    ensure("doc is global", tester.get_pview()->is_global());
+    ensure("root global", wb.wb->get_root()->is_global());
+    ensure("doc is global", wb.get_pview()->is_global());
 
     ensure("figure is global", figure->is_global());
 
-    tester.add_response_for_confirm_dialog(1);
+    wb.add_response_for_confirm_dialog(1);
     
-    tester.wb->delete_object(figure, opts);
+    wb.wb->get_model_context()->delete_object(figure, opts);
 
-    ensure_equals("delete 1 table", tester.get_pview()->figures().count(), 1U);
-    ensure_equals("connection count after expected auto-delete", tester.get_pview()->connections().count(), 0U);
+    ensure_equals("delete 1 table", wb.get_pview()->figures().count(), 1U);
+    ensure_equals("connection count after expected auto-delete", wb.get_pview()->connections().count(), 0U);
 
     // undo
-    tester.grt->get_undo_manager()->undo();
+    wb.grt->get_undo_manager()->undo();
 
-    ensure_equals("undo delete table", tester.get_pview()->figures().count(), 2U);
-    ensure_equals("undo delete table (connection)", tester.get_pview()->connections().count(), 1U);
+    ensure_equals("undo delete table", wb.get_pview()->figures().count(), 2U);
+    ensure_equals("undo delete table (connection)", wb.get_pview()->connections().count(), 1U);
   }
 }
 
@@ -326,7 +338,8 @@ TEST_FUNCTION(8)
   //TODO: Investigate what was the purpose of this and if it continues needed
   //callbacks.confirm_action= boost::bind(&delegated_confirm_action, &result);
 
-  WBTester tester(Size(800, 800), callbacks);
+//  WBTester tester(Size(800, 800), callbacks);
+  WBTester& tester = wb;
 
   tester.wb->new_document();
   tester.add_view();
@@ -343,7 +356,7 @@ TEST_FUNCTION(8)
   
   std::map<std::string, bool> opts;
 
-  tester.wb->delete_object(tester.get_pview()->figures()[0], opts);
+  tester.wb->get_model_context()->delete_object(tester.get_pview()->figures()[0], opts);
 
   ensure_equals("table object", tester.get_schema()->tables().count(), 0U);
   ensure_equals("table figure", tester.get_pview()->figures().count(), 0U);
@@ -358,7 +371,7 @@ TEST_FUNCTION(8)
 
   opts["deletedbobjects"]= false;
 
-  tester.wb->delete_object(tester.get_pview()->figures()[0], opts);
+  tester.wb->get_model_context()->delete_object(tester.get_pview()->figures()[0], opts);
 
   opts.clear();
 
@@ -374,7 +387,7 @@ TEST_FUNCTION(8)
   // cancel deletion (-1)
   result= -1;
 
-  tester.wb->delete_object(tester.get_pview()->figures()[0], opts);
+  tester.wb->get_model_context()->delete_object(tester.get_pview()->figures()[0], opts);
 
   ensure_equals("table object", tester.get_schema()->tables().count(), 1U);
   ensure_equals("table figure", tester.get_pview()->figures().count(), 1U);
@@ -384,62 +397,60 @@ TEST_FUNCTION(9)
 {
   // make sure connections are deleted and recreated with undo/redo
 
-  WBTester tester;
+  wb.wb->new_document();
+  wb.add_view();
 
-  tester.wb->new_document();
-  tester.add_view();
+  grt::UndoManager *um= wb.grt->get_undo_manager();
 
-  grt::UndoManager *um= tester.grt->get_undo_manager();
-
-  db_mysql_TableRef table1= tester.add_table_figure("table1", 10, 10);
-  db_mysql_ColumnRef column1(tester.grt);
+  db_mysql_TableRef table1= wb.add_table_figure("table1", 10, 10);
+  db_mysql_ColumnRef column1(wb.grt);
   column1->owner(table1);
   column1->name("pk");
-  db_mysql_TableRef table2= tester.add_table_figure("table2", 100, 10);
+  db_mysql_TableRef table2= wb.add_table_figure("table2", 100, 10);
 
-  db_mysql_ColumnRef column2(tester.grt);
+  db_mysql_ColumnRef column2(wb.grt);
   column2->owner(table2);
   column2->name("pk");
 
-  ensure_equals("table object", tester.get_schema()->tables().count(), 2U);
-  ensure_equals("table figure", tester.get_pview()->figures().count(), 2U);
+  ensure_equals("table object", wb.get_schema()->tables().count(), 2U);
+  ensure_equals("table figure", wb.get_pview()->figures().count(), 2U);
 
-  tester.grt->start_tracking_changes();
+  wb.grt->start_tracking_changes();
   table1->addPrimaryKeyColumn(column1);
   table2->addPrimaryKeyColumn(column2);
-  tester.grt->stop_tracking_changes();
+  wb.grt->stop_tracking_changes();
 
   // create 1:n rel and test undo
 
-  grt::AutoUndo undo(tester.grt);
+  grt::AutoUndo undo(wb.grt);
   bec::TableHelper::create_foreign_key_to_table(table2, table1, true, true, true, true,
-                                                tester.get_rdbms(),
-                                                grt::DictRef(tester.grt),
-                                                grt::DictRef(tester.grt));
+                                                wb.get_rdbms(),
+                                                grt::DictRef(wb.grt),
+                                                grt::DictRef(wb.grt));
   undo.end("create fk");
 
   ensure_equals("table fks", table2->foreignKeys().count(), 1U);
 
-  grt::BaseListRef connections(tester.get_pview()->connections());
+  grt::BaseListRef connections(wb.get_pview()->connections());
 
-  tester.flush_until(2, boost::bind(&BaseListRef::count, connections), 1);
+  wb.flush_until(2, boost::bind(&BaseListRef::count, connections), 1);
 
   ensure_equals("connection created", connections.count(), 1U);
 
   um->undo();
 
-  tester.flush_until(2, boost::bind(&BaseListRef::count, connections), 0);
+  wb.flush_until(2, boost::bind(&BaseListRef::count, connections), 0);
 
   ensure_equals("connection undone", connections.count(), 0U);
 
   um->redo();
 
-  tester.flush_until(2, boost::bind(&BaseListRef::count, connections), 1);
+  wb.flush_until(2, boost::bind(&BaseListRef::count, connections), 1);
 
   ensure_equals("connection redone", connections.count(), 1U);
 
   um->undo();
-  tester.flush_until(2, boost::bind(&BaseListRef::count, connections), 0);
+  wb.flush_until(2, boost::bind(&BaseListRef::count, connections), 0);
 
 }
 
@@ -449,8 +460,6 @@ TEST_FUNCTION(10)
 {
   // bug: copy/paste object across schemas are not updating the owner
 
-  WBTester wb;
-
   wb.wb->new_document();
 
   WBComponentPhysical *ph= wb.wb->get_component<WBComponentPhysical>();
@@ -496,8 +505,6 @@ TEST_FUNCTION(15)
 {
   // bug: deleting an identifying relationship doesnt delete indexes
 
-  WBTester wb;
-
   wb.wb->open_document("data/workbench/identifying_relationship.mwb");
 
   wb.wb->flush_idle_tasks();
@@ -518,7 +525,7 @@ TEST_FUNCTION(15)
 
   std::map<std::string,bool> opts;
   wb.add_response_for_confirm_dialog(0);
-  wb.wb->delete_object(doc->physicalModels()[0]->diagrams()[0]->connections()[0], opts);
+  wb.wb->get_model_context()->delete_object(doc->physicalModels()[0]->diagrams()[0]->connections()[0], opts);
 
   ensure_equals("connections", doc->physicalModels()[0]->diagrams()[0]->connections().count(), 0U);
   ensure_equals("check index count", table1->indices().count(), 1U);
@@ -527,8 +534,6 @@ TEST_FUNCTION(15)
 
 TEST_FUNCTION(16)
 {
-  WBTester wb;
-
   wb.wb->open_document("data/workbench/identifying_relationship.mwb");
 
   wb.wb->flush_idle_tasks();
@@ -551,51 +556,47 @@ TEST_FUNCTION(17)
   // test whether relationship is created when a FK is added and its details
   // are set in steps (non-atomically)
 
-  WBTester tester;
+  wb.wb->new_document();
+  wb.add_view();
 
-  tester.wb->new_document();
-  tester.add_view();
-
-  db_mysql_TableRef table1= tester.add_table_figure("table1", 10, 10);
-  db_mysql_ColumnRef column1(tester.grt);
+  db_mysql_TableRef table1= wb.add_table_figure("table1", 10, 10);
+  db_mysql_ColumnRef column1(wb.grt);
   column1->owner(table1);
   column1->name("pk");
-  db_mysql_TableRef table2= tester.add_table_figure("table2", 100, 10);
+  db_mysql_TableRef table2= wb.add_table_figure("table2", 100, 10);
 
-  db_mysql_ColumnRef column2(tester.grt);
+  db_mysql_ColumnRef column2(wb.grt);
   column2->owner(table2);
   column2->name("fkcol");
 
-  ensure_equals("table object", tester.get_schema()->tables().count(), 2U);
-  ensure_equals("table figure", tester.get_pview()->figures().count(), 2U);
+  ensure_equals("table object", wb.get_schema()->tables().count(), 2U);
+  ensure_equals("table figure", wb.get_pview()->figures().count(), 2U);
 
-  tester.grt->start_tracking_changes();
+  wb.grt->start_tracking_changes();
   table1->addPrimaryKeyColumn(column1);
-  tester.grt->stop_tracking_changes();
+  wb.grt->stop_tracking_changes();
   
-  db_mysql_ForeignKeyRef fk(tester.grt);
+  db_mysql_ForeignKeyRef fk(wb.grt);
   fk->owner(table2);
   fk->name("fk");
   
   table2->foreignKeys().insert(fk);
-  tester.flush_until(2);
+  wb.flush_until(2);
 
   fk->columns().insert(column2);
   fk->referencedColumns().insert(column1);
   fk->referencedTable(table1);
 
-  grt::BaseListRef connections(tester.get_pview()->connections());
-  tester.flush_until(2, boost::bind(&grt::BaseListRef::count, connections), 1);
+  grt::BaseListRef connections(wb.get_pview()->connections());
+  wb.flush_until(2, boost::bind(&grt::BaseListRef::count, connections), 1);
 
-  ensure_equals("rel count", tester.get_pview()->connections().count(), 1U);
+  ensure_equals("rel count", wb.get_pview()->connections().count(), 1U);
 }
 
 TEST_FUNCTION(19)
 {
   // test if new view has layer size set 
 
-  WBTester wb;
-
   wb.wb->new_document();
   wb.add_view();
 
@@ -608,18 +609,16 @@ TEST_FUNCTION(19)
 TEST_FUNCTION(20)
 { // test creation of a simple model with a relationship
   
-  std::auto_ptr<WBTester> wb(new WBTester());
-  
-  wb->wb->new_document();
-  wb->add_view();
+  wb.wb->new_document();
+  wb.add_view();
 
-  wb->add_table_figure("table1", 100, 100);
+  wb.add_table_figure("table1", 100, 100);
 
-  wb->add_table_figure("table2", 300, 100);
+  wb.add_table_figure("table2", 300, 100);
   
   
   
-  wb->export_png("test20_dump.png");
+  wb.export_png("test20_dump.png");
 }
 
 
@@ -627,12 +626,11 @@ TEST_FUNCTION(21)
 {
   // bug: loading a model with layers and then hitting new crashes
 
-  WBTester wb;
-
   wb.wb->open_document("data/workbench/2tables_conn_layer.mwb");
 
   wb.wb->new_document();
-  
+
+  ensure("Hangin forefer", false);
   while (dynamic_cast<ModelDiagramForm*>(wb.wbui->get_active_main_form())!=0)
     wb.wb->flush_idle_tasks();
 }
@@ -642,8 +640,6 @@ TEST_FUNCTION(22)
 {
   // bug: loading model twice causes bad internal state in GUI
   
-  WBTester wb;
-
   wb.wb->open_document("data/workbench/sakila.mwb");
 
   wb.wb->open_document("data/workbench/sakila.mwb");
@@ -653,8 +649,6 @@ TEST_FUNCTION(22)
 TEST_FUNCTION(23)
 {
   // bug: loading a model with selection won't reselect the items in the canvas
-  WBTester wb;
-
   wb.wb->open_document("data/workbench/selected_table.mwb");
 
 
@@ -700,8 +694,6 @@ static std::string get_note_content(grt::GRT *grt, const GrtStoredNoteRef &note)
 TEST_FUNCTION(30)
 {
   // check stored note management
-  WBTester wb;
-
   wb.wb->new_document();
 
   // add a note
@@ -741,7 +733,6 @@ TEST_FUNCTION(31)
   // check undo for stored notes
 
   // create note
-  WBTester wb;
   wb.wb->new_document();
 
   // add a note
