@@ -1,11 +1,29 @@
+/* 
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
 
-#include "stdafx.h"
+#include "tut_stdafx.h"
 
-#include "test.h"
 #include "wb_helpers.h"
 #include "grtdb/db_object_helpers.h"
 #include "grt_test_utility.h"
 #include "model/wb_model_diagram_form.h"
+#include "model/wb_context_model.h"
 #include "workbench/wb_history_tree.h"
 #include "grtpp_undo_manager.h"
 #include "grtpp_util.h"
@@ -13,7 +31,7 @@
 using namespace bec;
 using namespace wb;
 using namespace grt;
-using namespace MySQL::Geometry;
+using namespace base;
 
 
 BEGIN_TEST_DATA_CLASS(wb_undo_diagram)
@@ -37,27 +55,6 @@ TEST_DATA_CONSTRUCTOR(wb_undo_diagram)
 
   last_undo_stack_height= 0;
   last_redo_stack_height= 0;
-}
-
-#include "wb_undo_methods.h"
-
-void place_figure_with_tool(const std::string &tool, double x= 10, double y= 10)
-{
-  view->set_tool(tool);
-
-  view->handle_mouse_button(mdc::ButtonLeft, true, x, y, (mdc::EventState)0);
-}
-
-
-END_TEST_DATA_CLASS;
-
-
-TEST_MODULE(wb_undo_diagram, "undo tests for diagram actions in Workbench");
-
-
-// setup
-TEST_FUNCTION(1)
-{
   bool flag= wb.wb->open_document("data/workbench/undo_test_model1.mwb");
   ensure("open_document", flag);
   
@@ -77,13 +74,34 @@ TEST_FUNCTION(1)
   
   ensure_equals("layers", view->layers().count(), 1U);
   
-  wb.sync_view();
+//  wb.sync_view();
   
   this->view= wb.wb->get_model_context()->get_diagram_form_for_diagram_id(wb.get_pview().id());
   ensure("viewform", this->view != 0);
   
   this->mview= view;
-  
+
+}
+
+#include "wb_undo_methods.h"
+
+void place_figure_with_tool(const std::string &tool, double x= 10, double y= 10)
+{
+  view->set_tool(tool);
+
+  view->handle_mouse_button(mdc::ButtonLeft, true, static_cast<int>(x), static_cast<int>(y), (mdc::EventState)0);
+}
+
+
+END_TEST_DATA_CLASS;
+
+
+TEST_MODULE(wb_undo_diagram, "undo tests for diagram actions in Workbench");
+
+
+// setup
+TEST_FUNCTION(1)
+{
   wbui->set_active_form(this->view);
   
   ensure_equals("undo stack is empty", um->get_undo_stack().size(), 0U);
@@ -386,7 +404,7 @@ TEST_FUNCTION(20) // Move Object
   x= figure->left();
   y= figure->top();
 
-  view->handle_mouse_button(mdc::ButtonLeft, true, x+5, y+5, (mdc::EventState)0);
+  view->handle_mouse_button(mdc::ButtonLeft, true, static_cast<int>(x+5), static_cast<int>(y+5), (mdc::EventState)0);
   view->handle_mouse_move(50, 50, mdc::SLeftButtonMask);
   view->handle_mouse_button(mdc::ButtonLeft, false, 50, 50, mdc::SLeftButtonMask);
   check_only_one_undo_added();
@@ -418,7 +436,7 @@ TEST_FUNCTION(22) // Move into and out of Layer
   ensure_equals("layer begins empty", layer->figures().count(), 0U);
 
   // move object into layer
-  view->handle_mouse_button(mdc::ButtonLeft, true, x+5, y+5, (mdc::EventState)0);
+  view->handle_mouse_button(mdc::ButtonLeft, true, static_cast<int>(x+5), static_cast<int>(y+5), (mdc::EventState)0);
   view->handle_mouse_move(150, 400, mdc::SLeftButtonMask);
   view->handle_mouse_button(mdc::ButtonLeft, false, 150, 400, mdc::SLeftButtonMask);
   check_only_one_undo_added();
@@ -448,7 +466,7 @@ TEST_FUNCTION(22) // Move into and out of Layer
   new_x= *figure->left();
   new_y= *figure->top();
 
-  view->handle_mouse_button(mdc::ButtonLeft, true, new_x+5, new_y+5, (mdc::EventState)0);
+  view->handle_mouse_button(mdc::ButtonLeft, true, static_cast<int>(new_x+5), static_cast<int>(new_y+5), (mdc::EventState)0);
   view->handle_mouse_move(10, 10, mdc::SLeftButtonMask);
   view->handle_mouse_button(mdc::ButtonLeft, false, 10, 10, mdc::SLeftButtonMask);
   check_only_one_undo_added();
@@ -599,7 +617,7 @@ TEST_FUNCTION(32) // Delete Table
   // delete the figure
   wb.add_response_for_confirm_dialog(1);
   std::map<std::string,bool> opt;
-  wbui->get_wb()->delete_object(figure, opt);
+  wbui->get_wb()->get_model_context()->delete_object(figure, opt);
   check_only_one_undo_added();
 
   ensure("table delete", !find_named_object_in_list(mview->figures(), "table1").is_valid());
@@ -696,9 +714,9 @@ TEST_FUNCTION(40) // Create Relationship (in diagram)
   ensure_equals("rel count", mview->connections().count(), 1U);
   
   // click table1
-  view->handle_mouse_button(mdc::ButtonLeft, true, *table1->left()+20, *table1->top()+20, (mdc::EventState)0);
+  view->handle_mouse_button(mdc::ButtonLeft, true, static_cast<int>(*table1->left()+20), static_cast<int>(*table1->top()+20), (mdc::EventState)0);
   // click table2
-  view->handle_mouse_button(mdc::ButtonLeft, true, *table2->left()+20, *table2->top()+20, (mdc::EventState)0);
+  view->handle_mouse_button(mdc::ButtonLeft, true, static_cast<int>(*table2->left()+20), static_cast<int>(*table2->top()+20), (mdc::EventState)0);
 
   check_only_one_undo_added();
 
