@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,6 +19,7 @@
 
 #include "stdafx.h"
 
+#include <string>
 #include "wb_context_ui.h"
 #include "wb_command_ui.h"
 #include "wb_module.h"
@@ -752,9 +753,8 @@ mforms::MenuBar *CommandUI::create_menubar_for_context(const std::string &contex
     
     if (mitem->name() == "edit")
       connect_validate_to_signal(_validate_edit_menu_items, item->get_subitems());
-    
-    mitem->reset_references();
   }
+  main_menu->reset_references();
   
   return menubar;
 }
