@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,41 +30,44 @@
 
 #include "base/ui_form.h"
 #include "grt/grt_manager.h"
-#include "grt/plugin_manager.h"
-#include "grt/icon_manager.h"
+#include "base/notifications.h"
+
+#include "wb_context_names.h"
 
 #include "grts/structs.workbench.h"
 #include "wb_backend_public_interface.h"
 
-#include "sqlide/wb_context_sqlide.h"
-
-#include "model/wb_context_model.h"
-#include "sqlide/wb_context_sqlide.h"
-
 #include <grtpp_undo_manager.h>
 
-#include "mdc.h"
 #include "mforms/utilities.h"
 #include "base/trackable.h"
 
 
 #define APP_MAJOR_NUMBER 5
 #define APP_MINOR_NUMBER 2
-#define APP_RELEASE_NUMBER 35
-#define APP_REVISION_NUMBER 7915
+#define APP_RELEASE_NUMBER 44
+#define APP_REVISION_NUMBER 9933
 #define APP_RELEASE_TYPE ""
 
 
 #define WBContext_VERSION 5
 
+namespace mdc {
+  class CanvasView;
+  class CanvasItem;
+};
 
 namespace bec {
   class Clipboard;
+  class IconManager;
 };
 
+class SqlEditorForm;
+
 namespace wb {
   class WBContextUI;
-
+  class WBContextModel;
+  class WBContextSQLIDE;
   class WorkbenchImpl;
   class WBComponent;
 
@@ -78,6 +81,7 @@ namespace wb {
     RefreshNothing,
     RefreshSchemaList,
     RefreshSchema,
+    RefreshSchemaNoReload,
     RefreshNewDiagram,
     RefreshSelection,
     RefreshCloseEditor, // argument: object-id (close all if "")
@@ -89,7 +93,6 @@ namespace wb {
     RefreshOverviewNodeChildren,// argument: node-id, overview ptr 
 
     RefreshCatalogTreeNode,
-    RefreshViewName,
     RefreshDocument,
     RefreshCloseDocument,
     RefreshZoom,
@@ -98,16 +101,6 @@ namespace wb {
     RefreshFinishEdits // Force all ongoing edit operations (eg in TreeView cells) to be commited
   };
 
-  struct WBShortcut
-  {
-    std::string shortcut;
-    mdc::KeyInfo key;
-    mdc::EventState modifiers;
-
-    std::string name;
-    std::string command;
-  };
-
   struct MYSQLWBBACKEND_PUBLIC_FUNC WBPaperSize
   {
     std::string name;
@@ -122,18 +115,6 @@ namespace wb {
     std::string description;
   };
 
-  // context names
-  #define WB_CONTEXT_GLOBAL  "*global"
-  #define WB_CONTEXT_HOME_GLOBAL "home"
-  #define WB_CONTEXT_MODEL_GLOBAL  "*model"
-  #define WB_CONTEXT_QUERY_GLOBAL  "*query"
-  #define WB_CONTEXT_ADMIN_GLOBAL  "*admin"
-
-  #define WB_CONTEXT_MODEL   "model"
-  #define WB_CONTEXT_EDITOR "editor" // object editors
-  #define WB_CONTEXT_PHYSICAL_OVERVIEW "overview.physical"
-  #define WB_CONTEXT_QUERY "dbquery"
-  
   // basic toolbar names
   #define WB_TOOLBAR_OPTIONS "options"
 
@@ -164,10 +145,6 @@ namespace wb {
 
     // Show some text in the application's status bar: must be thread-safe
     boost::function<void (std::string)> show_status_text;
-    // Displays a progress bar
-    // Args: title, status, percentage completed
-    // Must return false if the action must be cancelled
-    boost::function<bool (std::string,std::string,float)> show_progress;
 
     // ok/cancel, title, flags, returnval
     boost::function<bool (std::string, int, std::string&)> request_input;
@@ -222,18 +199,18 @@ namespace wb {
     std::string run_language; // language of the script in run_at_startup
     bool force_sw_rendering;
     bool force_opengl_rendering;
-    bool use_log_file;
     bool verbose;
     bool quit_when_done;
+    bool no_logo;
 
-    WBOptions() : force_sw_rendering(false), force_opengl_rendering(false), use_log_file(false), 
-         verbose(false), quit_when_done(false) {}
+    WBOptions();
+    bool parse_args(char **argv, int argc, int *retval = NULL);
   };
 
   
 #define FOREACH_COMPONENT(list, iter) for (std::vector<WBComponent*>::iterator iter= list.begin(); iter != list.end(); ++iter)
 
-  class MYSQLWBBACKEND_PUBLIC_FUNC WBContext : public base::trackable
+  class MYSQLWBBACKEND_PUBLIC_FUNC WBContext : public base::trackable, base::Observer
   {
     friend class WorkbenchImpl;
     friend class WBComponent;
@@ -243,8 +220,8 @@ namespace wb {
     WBContext(WBContextUI *ui, bool verbose = false);
     ~WBContext();
 
-    bool use_opengl() { return _use_opengl; }
-    bool is_opengl_available();
+    bool software_rendering_enforced();
+    bool opengl_rendering_enforced();
 
     bool init_(WBFrontendCallbacks *callbacks, WBOptions *options);
     void init_finish_(WBOptions *options);
@@ -256,12 +233,12 @@ namespace wb {
     bec::UIForm *get_active_main_form();
 
     WBContextModel *get_model_context() { return _model_context; }
-    WBContextSQLIDE *get_sqlide_context() { return &_sqlide_context; }
+    WBContextSQLIDE *get_sqlide_context() { return _sqlide_context; }
     
     // Document handling.
     void new_document();
-    bool can_close_document();
-    bool close_document(); // returns false for cancelled
+    bool can_close_document(); // returns false for cancelled
+    bool close_document(); 
     
     void close_document_finish();
     void new_model_finish();
@@ -272,11 +249,7 @@ namespace wb {
 
     std::string get_filename() const;
 
-    void export_png(const std::string &path);
-    void export_pdf(const std::string &path);
-    void export_ps(const std::string &path);
-    void export_svg(const std::string &path);
-
+    void report_bug(const std::string &errorInfo);
 
     // plugins
     void execute_plugin(const std::string &plugin_name, 
@@ -284,14 +257,8 @@ namespace wb {
 
     void update_plugin_arguments_pool(bec::ArgumentPool &args);
 
-    // Diagrams
-    void add_new_diagram(const model_ModelRef &model);
-    bool delete_diagram(const model_DiagramRef &view);
-
-    void switch_diagram(const model_DiagramRef &view);
-    
     // DB Querying
-    boost::shared_ptr<Db_sql_editor> add_new_query_window(const db_mgmt_ConnectionRef &target);
+    boost::shared_ptr<SqlEditorForm> add_new_query_window(const db_mgmt_ConnectionRef &target);
     
     // Admin
     void add_new_admin_window(const db_mgmt_ServerInstanceRef &target);
@@ -301,14 +268,12 @@ namespace wb {
 
     void close_gui_plugin(NativeHandle handle);
 
-    bool delete_object(model_ObjectRef object, std::map<std::string, bool>& options);
-
 
     //
     void request_refresh(RefreshType type, const std::string &str, NativeHandle ptr= (NativeHandle)0);
 
     const std::string &get_user_datadir() const { return _user_datadir; }
-    //TODO: Temoprarly solution need to make ModelFile grt class
+    // TODO: Temporary solution need to make ModelFile grt class
     workbench_DocumentRef openModelFile(const std::string &file);
     std::string getTempDir();
     int closeModelFile();
@@ -336,7 +301,7 @@ namespace wb {
 
     bec::GRTManager *get_grt_manager() const { return _manager; }
     grt::GRT *get_grt() const { return _manager->get_grt(); }
-    WorkbenchImpl *get_workbench() { return _workbench; };
+    boost::shared_ptr<WorkbenchImpl> get_workbench() { return _workbench; };
 
     bec::Clipboard *get_clipboard() const { return _clipboard; }
 
@@ -376,15 +341,10 @@ namespace wb {
     void execute_async_in_grt_thread(const std::string &name, 
                                      const boost::function<grt::ValueRef (grt::GRT*)> &function) THROW (grt::grt_runtime_error);
 
-    void open_object_editor(const GrtObjectRef &object, bec::GUIPluginFlags flags=bec::NoFlags);
     bool activate_live_object(const GrtObjectRef &object);
     bool create_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name);
     bool drop_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name);
 
-    model_DiagramRef get_view_with_id(const std::string &id);
-
-    std::string get_object_tooltip(const model_ObjectRef &object, mdc::CanvasItem *item);
-
     std::string create_attached_file(const std::string &group, const std::string &tmpl);
     void save_attached_file_contents(const std::string &name, const char *data, size_t size);
     std::string get_attached_file_contents(const std::string &name);
@@ -410,19 +370,16 @@ namespace wb {
   protected:
     friend class WBContextModel; // to access _components
     
-    ModelFile* _model_file;
     bec::GRTManager *_manager;
     bec::PluginManager *_plugin_manager;
 
     workbench_WorkbenchRef _wb_root;
 
-    std::string _opengl_version;
     int _user_interaction_blocked;
-    int _opengl_available; // -1 don't know yet, 0 no, 1 yes
-    bool _use_opengl;
     bool _send_messages_to_shell;
     bool _asked_for_saving;
     bool _initialization_finished;
+    bool _attachments_changed;
     
     std::string _datadir;
     std::string _user_datadir;
@@ -440,11 +397,11 @@ namespace wb {
 
     WBContextUI *_uicontext;
     WBContextModel *_model_context;
-    WBContextSQLIDE _sqlide_context;
+    WBContextSQLIDE *_sqlide_context;
     
     std::vector<WBComponent*> _components;
 
-    WorkbenchImpl *_workbench;
+    boost::shared_ptr<WorkbenchImpl> _workbench;
 
     bec::Clipboard *_clipboard;
 
@@ -454,11 +411,14 @@ namespace wb {
     grt::UndoAction *_save_point;
     
     TunnelManager *_tunnel_manager;
-    
-    boost::signals2::signal<void ()> _signal_app_closing;
-    
-    void delete_canvas_item(mdc::CanvasItem *item);
 
+    ModelFile* _model_file;
+    
+    bool _force_sw_rendering;     // Command line switch.
+    bool _force_opengl_rendering; // Command line switch.
+
+    //boost::signals2::signal<void ()> _signal_app_closing;
+    
     grt::ListRef<app_PaperType> get_paper_types(grt::GRT *grt);
 
     // setup
@@ -468,6 +428,7 @@ namespace wb {
     void init_plugin_groups_grt(grt::GRT *grt, WBOptions *options);
     void init_object_listeners_grt(grt::GRT *grt);
     void init_properties_grt(workbench_DocumentRef &doc);
+    void init_rdbms_modules(grt::GRT *grt);
 
     void do_close_document(bool destroying);
 
@@ -481,11 +442,14 @@ namespace wb {
     std::string get_auto_save_dir();
   public:
     void save_app_options();
-    void add_recent_file(const std::string &file);
+    void save_connections();
+    void save_instances();
 
-    boost::signals2::signal<void ()>* signal_app_closing() { return &_signal_app_closing; }
+//    boost::signals2::signal<void ()>* signal_app_closing() { return &_signal_app_closing; }
     
   protected:
+    void add_recent_file(const std::string &file);
+
     void load_app_state();
     void save_app_state();
     void load_starters();
@@ -497,13 +461,17 @@ namespace wb {
     grt::ValueRef execute_plugin_grt(grt::GRT *grt, const app_PluginRef &plugin, const grt::BaseListRef &args);
     void plugin_finished(const grt::ValueRef &result, const app_PluginRef &plugin);
 
-    void handle_message(const grt::Message &msg);
+    bool handle_message(const grt::Message &msg);
 
     void reset_document();
     void reset_listeners();
 
     void option_dict_changed(grt::internal::OwnedDict*dict=0, bool added=false, const std::string& key="");
 
+  private:
+    // for base::Observer
+    virtual void handle_notification(const std::string &name, void *sender, std::map<std::string, std::string> &info);
+
   public:
     ModelFile *get_file() { return _file; }
 
@@ -513,15 +481,15 @@ namespace wb {
   private:
     bool find_connection_password(const db_mgmt_ConnectionRef &conn, std::string &password);
 
-    bool do_request_password(const std::string &title, const std::string &service, std::string &account, bool reset_password, std::string *ret_password);
-    bool do_find_connection_password(const std::string &hostId, const std::string &username, std::string *ret_password);
+    void *do_request_password(const std::string &title, const std::string &service, std::string &account, bool reset_password, std::string *ret_password);
+    void *do_find_connection_password(const std::string &hostId, const std::string &username, std::string *ret_password);
 
     void attempt_options_upgrade(xmlDocPtr xmldoc, const std::string &version);
 
-    std::string request_connection_password(const db_mgmt_ConnectionRef &conn, bool force_asking);
     bool show_error(const std::string& title, const std::string& message);
 
-    void determine_opengl_support(WBOptions* command_line_options);
+  public:
+    std::string request_connection_password(const db_mgmt_ConnectionRef &conn, bool force_asking);
 
   public: // front end callbacks
     boost::function<std::string (std::string,std::string,std::string)> show_file_dialog;
