@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,11 +19,15 @@
 
 #include "stdafx.h"
 
+#include <string>
+
 #include "wb_context_ui.h"
+#include "wb_context.h"
 #include "model/wb_component.h"
 #include "model/wb_component_physical.h"
-
+#include "model/wb_context_model.h"
 #include "model/wb_model_diagram_form.h"
+#include "sqlide/wb_context_sqlide.h"
 
 #include "wb_overview.h"
 #include "model/wb_overview_physical.h"
@@ -40,9 +44,11 @@
 #include "output_view.h"
 #include "webbrowser_view.h"
 
-#include "mforms/utilities.h"
-#include "mforms/app.h"
+#include "mforms/mforms.h"
+
 #include "base/string_utilities.h"
+#include "base/log.h"
+#include "base/notifications.h"
 
 #include "plugin_install_window.h"
 
@@ -50,13 +56,17 @@ using namespace wb;
 using namespace bec;
 using namespace base;
 
+DEFAULT_LOG_DOMAIN(DOMAIN_WB_CONTEXT_UI)
 
 WBContextUI::WBContextUI(bool verbose)
   : _wb(new WBContext(this, verbose)), _usertypes_tree(_wb), _command_ui(_wb)
 {
   NodeId n; // workaround a bug causing crash when output tab is shown with GLib-ERROR **: The thread system is not yet initialized.
 
-
+  base::NotificationCenter::get()->add_observer(this, "GNAppClosing");
+  base::NotificationCenter::get()->add_observer(this, "GNTextSelectionChanged");
+  base::NotificationCenter::get()->add_observer(this, "GNClipboardChanged");
+  
   _catalog_tree= 0;
 
   _history_tree= 0;
@@ -66,7 +76,7 @@ WBContextUI::WBContextUI(bool verbose)
   _find_dialog= new FindDialogBE(_wb);
   _find_dialog->set_activate_slot(boost::bind(&WBContextUI::activate_find_result, this, _1, _2, _3));
 
-  _output_view= new OutputView(_wb->get_grt_manager()->get_messages_list());
+  _output_view= new OutputView(_wb);
   scoped_connect(_output_view->get_be()->signal_show(),boost::bind(&WBContextUI::show_output, this));
     
   _active_form= 0;
@@ -91,8 +101,11 @@ WBContextUI::WBContextUI(bool verbose)
 
 WBContextUI::~WBContextUI()
 {
+  base::NotificationCenter::get()->remove_observer(this);
+  
   _wb->do_close_document(true);
 
+  // TODO: is it really necessary to set all member variables to NULL in a dtor?
   delete _addon_download_window;
   _addon_download_window = 0;
   delete _plugin_install_window;
@@ -101,8 +114,11 @@ WBContextUI::~WBContextUI()
   _find_dialog = 0;
   delete _history_tree;
   _history_tree = 0;
-  delete _home_screen;
-  _home_screen = 0;
+  if (_home_screen != NULL)
+  {
+    _home_screen->release();
+    _home_screen = NULL;
+  }
   delete _shell_window;
   _shell_window = 0;
 
@@ -110,10 +126,32 @@ WBContextUI::~WBContextUI()
   _wb = 0;
 }
 
+
+void WBContextUI::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
+{
+  if (name == "GNAppClosing")
+    home_screen_closing();
+  else if (name == "GNTextSelectionChanged" || name == "GNClipboardChanged")
+    get_command_ui()->revalidate_edit_menu_items();
+}
+
 //--------------------------------------------------------------------------------------------------
 
 bool WBContextUI::init(WBFrontendCallbacks *callbacks, WBOptions *options)
 {
+  // Log set folders.
+  log_info("Initializing workbench context UI with these values:\n"
+    "\tbase dir: %s\n\tplugin path: %s\n\tstruct path: %s\n\tmodule path: %s\n\t"
+    "library path: %s\n\tuser data dir: %s\n\topen at start: %s\n\topen type: %s\n\trun at startup: %s\n\t"
+    "run type: %s\n\tForce SW rendering: %s\n\tForce OpenGL: %s\n\tquit when done: %s\n",
+    options->basedir.c_str(), options->plugin_search_path.c_str(), options->struct_search_path.c_str(),
+    options->module_search_path.c_str(), options->library_search_path.c_str(),
+    options->user_data_dir.c_str(), options->open_at_startup.c_str(), options->open_at_startup_type.c_str(),
+    options->run_at_startup.c_str(), options->run_language.c_str(),
+    options->force_sw_rendering ? "Yes" : "No",
+    options->force_opengl_rendering ? "Yes" : "No",
+    options->quit_when_done ? "Yes" : "No");
+
   _initializing = true;
   bool flag = false;
   try
@@ -135,9 +173,15 @@ bool WBContextUI::init(WBFrontendCallbacks *callbacks, WBOptions *options)
     show_home_screen();
     _initializing = false;
   }
+  catch (const std::exception& e)
+  {
+    _initializing = false;
+    log_error("WBContextUI::init, exception '%s'\n", e.what()); // log_error logs to stderr too in debug mode.
+  }
   catch (...)
   {
     _initializing = false;
+    log_error("Some exception has happened. It was caught at WBContextUI::init.\n");
   }
 
   return flag;
@@ -173,22 +217,8 @@ bool WBContextUI::request_quit()
   if (!_wb->get_grt_manager()->in_main_thread())
     g_warning("request_quit() called in worker thread");
 
-  if (_wb->has_unsaved_changes())
-  {
-    int answer= mforms::Utilities::show_message(_("Quit Application"),
-      _("Do you want to save pending changes to the document?\n\n"
-      "If you don't save your changes, they will be lost."),
-      _("Save"), _("Cancel"), _("Don't Save"));
-    if (answer == mforms::ResultOk)
-    {
-      if (!_wb->save_changes())
-        return false;
-    }
-    else if (answer == mforms::ResultCancel)
-      return false;
-
-    _wb->do_close_document(true);
-  }
+  if (!_wb->can_close_document())
+    return false;
 
   if (_wb->get_sqlide_context() && !_wb->get_sqlide_context()->request_quit())
     return false;
@@ -203,7 +233,10 @@ bool WBContextUI::request_quit()
 void WBContextUI::perform_quit()
 {
   _quitting= true;
+  _wb->do_close_document(true);
   _wb->quit_application();
+
+  mforms::ControlFactory::get_instance()->shutdown();
 }
 
 
@@ -254,7 +287,7 @@ void WBContextUI::overview_selection_changed()
   if (get_active_main_form() == get_physical_overview())
   {
     _wb->request_refresh(RefreshSelection, "", (NativeHandle)get_physical_overview()->get_frontend_data());
-    (*_command_ui.signal_validate_edit_menu_items())();
+    get_command_ui()->revalidate_edit_menu_items();
   }
 }
 
@@ -331,7 +364,7 @@ void WBContextUI::add_backend_builtin_commands()
     boost::bind(add_script_file, this));
   _command_ui.add_builtin_command("add_note_file", 
     boost::bind(add_note_file, this));
-
+/*
   // handle some url opening stuff ourselves through mforms
   _command_ui.add_builtin_command("community_blog",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/", true));
@@ -343,21 +376,22 @@ void WBContextUI::add_backend_builtin_commands()
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/?page_id=16", true));
   _command_ui.add_builtin_command("community_contribute",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/?page_id=17", true));
+ */
   _command_ui.add_builtin_command("web_mysql_home",
     boost::bind(&WBContextUI::show_web_page, this, "http://mysql.com/", true));
   _command_ui.add_builtin_command("web_home",
     boost::bind(&WBContextUI::show_web_page, this, "http://mysql.com/products/tools/workbench/", true));
-  _command_ui.add_builtin_command("report_a_bug",
-    boost::bind(&WBContextUI::show_web_page, this, "http://bugs.mysql.com/", true));
   _command_ui.add_builtin_command("list_bugs",
     boost::bind(&WBContextUI::show_web_page, this, "http://bugs.mysql.com/saved/WB%20bugs", true));
-  _command_ui.add_builtin_command("plugin_site",
-    boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/addons/", true));
-  _command_ui.add_builtin_command("check_update",
-    boost::bind(&WBContextUI::show_web_page, this, 
-               base::strfmt("http://wb.mysql.com/workbench/version-check.php?version=%i.%i.%i&revision=%i", 
-                            APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, APP_REVISION_NUMBER), false));
+ // _command_ui.add_builtin_command("plugin_site",
+ //   boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/addons/", true));
+//  _command_ui.add_builtin_command("check_update",
+//    boost::bind(&WBContextUI::show_web_page, this, 
+//               base::strfmt("http://wb.mysql.com/workbench/version-check.php?version=%i.%i.%i&revision=%i", 
+//                            APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, APP_REVISION_NUMBER), false));
   _command_ui.add_builtin_command("help_index", boost::bind(&WBContextUI::show_help_index, this));
+  _command_ui.add_builtin_command("locate_log_file", boost::bind(&WBContextUI::locate_log_file, this));
+  _command_ui.add_builtin_command("show_log_file", boost::bind(&WBContextUI::show_log_file, this));
 }
 
 #ifndef ___specialforms
@@ -394,6 +428,7 @@ void WBContextUI::show_output()
 {
   mforms::App::get()->dock_view(_output_view, "maintab");
   mforms::App::get()->set_view_title(_output_view, "Output");
+  _output_view->setup_ui();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -428,6 +463,21 @@ void WBContextUI::show_help_index()
 
 //--------------------------------------------------------------------------------------------------
 
+void WBContextUI::locate_log_file()
+{
+  if (!base::Logger::log_dir().empty())
+    mforms::Utilities::open_url(base::Logger::log_dir());
+}
+
+
+void WBContextUI::show_log_file()
+{
+  if (!base::Logger::log_filename().empty())
+    mforms::Utilities::open_url(base::Logger::log_filename());
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void WBContextUI::activate_figure(const grt::ValueRef &value)
 {
   ModelDiagramForm *form= 0;
@@ -560,7 +610,7 @@ bec::ValueInspectorBE *WBContextUI::create_inspector_for_selection(std::vector<s
   return 0;
 }
 
-
+/* unused
 static std::list<GrtObjectRef> get_selected_objects_from_form(WBContext *wb, bec::UIForm *form)
 {
   std::list<GrtObjectRef> objects;
@@ -603,7 +653,7 @@ static std::list<GrtObjectRef> get_selected_objects_from_form(WBContext *wb, bec
     }
   }
   return objects;
-}
+}*/
 
 
 std::string WBContextUI::get_description_for_selection(grt::ListRef<GrtObject> &activeObjList, std::vector<std::string> &items)
@@ -997,21 +1047,21 @@ void WBContextUI::set_wb_options_value(const std::string &model, const std::stri
   case grt::DoubleType:
     {
       grt::DoubleRef v(atof(value.c_str()));
-      if (options.get_double(key) != *v)
+      if (!options.has_key(key) || options.get_double(key) != *v)
         options.set(key, v);
       break;
     }
   case grt::IntegerType:
     {
       grt::IntegerRef v(atoi(value.c_str()));
-      if (options.get_int(key) != *v)
+      if (!options.has_key(key) || options.get_int(key) != *v)
         options.set(key, v);
       break;
     }
   case grt::StringType:
     {
       grt::StringRef v(value);
-      if (options.get_string(key) != *v)
+      if (!options.has_key(key) || options.get_string(key) != *v)
         options.set(key, v);
       break;
     }
@@ -1088,7 +1138,10 @@ void WBContextUI::set_active_form(bec::UIForm *form)
     if (_active_main_form != form)
     {
       _active_main_form= form;
-      _main_form_change_signal(form);
+
+      base::NotificationInfo info;
+      info["form"] = form ? form->form_id() : "";
+      NotificationCenter::get()->send("GNMainFormChanged", 0, info);
     }
   }
 
