@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,10 +21,13 @@
 #include <glib/gstdio.h>
 
 #include "wb_context_ui.h"
+#include "model/wb_context_model.h"
+#include "sqlide/wb_context_sqlide.h"
 
 #include "base/file_functions.h"
 #include "base/file_utilities.h"
 #include "base/string_utilities.h"
+#include "base/log.h"
 
 #include "server_instance_editor.h"
 #include "grtui/grtdb_connection_editor.h"
@@ -32,11 +35,13 @@
 #include "new_connection_wizard.h"
 #include "edit_sql_script_wizard.h"
 #include "edit_table_data_wizard.h"
+#include "select_option_dialog.h"
 
 
 #define MODEL_AUTOSAVED_ICON_FILE "db_model_warning.png"
 #define SQLEDITOR_AUTOSAVED_ICON_FILE "db_connection_warning.png"
 
+DEFAULT_LOG_DOMAIN("WBContextUI");
 
 using namespace bec;
 using namespace wb;
@@ -84,27 +89,195 @@ std::string get_server_info(db_mgmt_ServerInstanceRef instance)
 
 //--------------------------------------------------------------------------------------------------
 
-static void save_state(WBContext *wb, HomeScreen *home)
+static void save_state(WBContextUI *owner, HomeScreen *home)
 {
   if (!home->get_wb_central_expanded())
-    wb->save_state("WBCentralCollapsedInVersion", "home",
+    owner->get_wb()->save_state("WBCentralCollapsedInVersion", "home",
                    strfmt("%i.%i.%i", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER));
   else
-    wb->save_state("WBCentralCollapsedInVersion", "home", std::string(""));  
+    owner->get_wb()->save_state("WBCentralCollapsedInVersion", "home", std::string(""));  
   
-  wb->save_state("WBHomeGroupStateAdministration", "home", home->get_collapsed_group_data(HomeScreenAdministrationList));  
-  wb->save_state("WBHomeGroupStateSQLDev", "home", home->get_collapsed_group_data(HomeScreenSQLDevList));  
+  owner->get_wb()->save_state("WBHomeGroupStateAdministration", "home", home->get_collapsed_group_data(HomeScreenAdministrationList));  
+  owner->get_wb()->save_state("WBHomeGroupStateSQLDev", "home", home->get_collapsed_group_data(HomeScreenSQLDevList));  
 }
 
 //--------------------------------------------------------------------------------------------------
 
-static bool on_home_closing(WBContextUI* owner, HomeScreen *home)
-{
-  owner->home_screen_closing();
-  save_state(owner->get_wb(), home);
-  return true;
-}
 
+  template <class T>
+  void get_groups_for_movement(grt::ListRef<T> items, const grt::ValueRef &object, std::vector<std::string> &groups)
+  {
+    grt::Ref<T> item = grt::Ref<T>::cast_from(object);
+
+    std::string item_name = item->name();
+    size_t current_group_separator_position = item_name.find("/");
+
+    std::string current_group = "";
+    
+    if (current_group_separator_position != std::string::npos)
+    {
+      current_group = item_name.substr(0, current_group_separator_position);
+      groups.push_back("*Ungrouped*");
+    }
+
+    for (typename grt::ListRef<T>::const_iterator end = items.end(),
+          inst = items.begin(); inst != end; ++inst)
+    {
+      std::string name = (*inst)->name();
+      size_t group_separator_position = name.find("/");
+
+      if (group_separator_position != std::string::npos)
+      {
+        std::string group_name = name.substr(0, group_separator_position);
+
+        bool found = false;
+        for (std::vector<std::string>::iterator end = groups.end(),
+            index = groups.begin(); index != end && !found; index++)
+        {
+          found = (*index).compare(0, group_separator_position, group_name) == 0;
+        }
+
+        if (!found && group_name != current_group)
+          groups.push_back(group_name);
+      }
+    }
+  }
+
+  template <class T>
+  bool validate_group_for_movement(grt::ListRef<T> items, const grt::ValueRef &object, std::string group)
+  {
+    bool ret_val = false;
+
+    // Validates the group is valid...
+    size_t group_position = group.find("/");
+
+    if( group.length() == 0 )
+      Utilities::show_warning(_("Move To Group"), _("You must select the target group from the list or type a new group."),  _("Ok"));
+    else if (group_position != std::string::npos)
+      Utilities::show_warning(_("Move To Group"), _("The selected group is invalid, should not contain the \"/\" character."),  _("Ok"));
+    else
+    {
+      // Gets the connection and connection list...
+      grt::Ref<T> item = grt::Ref<T>::cast_from(object);
+
+      // Creates the new connection name...
+      std::string item_name = item->name();
+      std::string new_item_name = "";
+
+      group_position = item_name.find("/");
+
+      if (group == "*Ungrouped*")
+        new_item_name = item_name.substr(group_position + 1);
+      else
+      {
+        if (group_position != std::string::npos)
+          new_item_name = group + "/" + item_name.substr(group_position + 1);
+        else
+          new_item_name = group + "/" + item_name;
+      }
+
+      size_t item_index = bec::find_list_ref_item_position<T>(items, new_item_name, MatchAny, NULL, FindFull);
+
+      if (item_index != grt::BaseListRef::npos)
+        Utilities::show_warning(_("Move To Group"), _("Unable to perform the movement as there's an entry with the same name in the target group"),  _("Ok"));
+      else
+        ret_val = true;
+    }
+
+    return ret_val;
+  }
+
+  template <class T>
+  void update_item_group(const grt::ValueRef &object, std::string group)
+      // At this point the movement is considered valid so we just do it...
+  {
+    grt::Ref<T> item = grt::Ref<T>::cast_from(object);
+    std::string item_name = item->name();
+    size_t current_group_separator_position = item_name.find("/");
+
+    std::string new_item_name = "";
+    if (group == "*Ungrouped*")
+      new_item_name = item_name.substr(current_group_separator_position + 1);
+    else
+    {
+      if (current_group_separator_position != std::string::npos)
+        new_item_name = group + "/" + item_name.substr(current_group_separator_position + 1);
+      else
+        new_item_name = group + "/" + item_name;
+    }
+
+    item->name(new_item_name);
+  }
+
+  template <class T>
+  void move_item_to_group(std::string group, grt::ListRef<T> items, const grt::ValueRef &object)
+  {
+    size_t current_item_index, current_group_index, sibling_index, target_index;
+    bool sibling_move = false;
+    bool item_move = false;
+    
+    // To ensure the grouping order is maintained after a move to group operation it is needed to consider
+    // - If the item being moved is in a group or not
+    // - If the item is being moved to a group or not
+    // - Removing the first item from a group may affect the group ordering, to prevent that, the next item in the group 
+    //   is being moved to the position of the first item
+    // - When moved to a group, the item will be placed after the last element in the group
+    // - Moving to a new group or ungrouping the item doesn't affect it's position on the list
+
+    //grt::Type object_type = object.type();
+    
+    grt::Ref<T> item = grt::Ref<T>::cast_from(object);
+    std::string item_name = item->name();
+
+    std::string item_group = "";
+    size_t group_indicator_position = item_name.find("/");
+
+    // Gets the position of the item being moved...
+    current_item_index = find_list_ref_item_position<T>(items, item_name);
+    if (group_indicator_position != std::string::npos)
+    {
+      item_group = item_name.substr(0, group_indicator_position + 1);
+
+      // Gets the position of the first element on the group
+      current_group_index = find_list_ref_item_position<T>(items, item_group);
+
+      if (current_item_index == current_group_index)
+      {
+        sibling_index = find_list_ref_item_position<T>(items, item_group, MatchAfter, &item);
+
+        if (sibling_index != grt::BaseListRef::npos)
+          sibling_move = true;
+      }
+    }
+
+    if (group != "*Ungrouped*")
+    {
+      // Gets the position of the last element on the target group if exists ( happily the function does that when passing MatchBefore without a reference )
+      std::string tmp_group = group + "/";
+      target_index = find_list_ref_item_position<T>(items, tmp_group , MatchLast);
+
+      if (target_index != grt::BaseListRef::npos)
+      {
+        item_move = true;
+
+        if (target_index < current_item_index)
+          target_index++;
+      }
+    }
+
+    if (sibling_move)
+    {
+      items.reorder(sibling_index, current_item_index);
+
+      if (sibling_index > current_item_index)
+        current_item_index++;
+    }
+
+    if (item_move)
+      items.reorder(current_item_index, target_index);
+
+    update_item_group<T>(object, group);
+  }  
 
 /**
  * Creates the main home screen (Workbench Central, Workspace) if not yet done and docks it to 
@@ -115,15 +288,10 @@ void WBContextUI::show_home_screen()
   if (_home_screen == NULL)
   {    
     _home_screen= mforms::manage(new HomeScreen(&_command_ui, _wb->get_grt()));
-    _home_screen->set_on_close(boost::bind(on_home_closing, this, _home_screen));
+    _home_screen->set_on_close(boost::bind(&WBContextUI::home_screen_closing, this));
     _home_screen->set_callback((home_screen_action_callback)&WBContextUI::home_action_callback, this);
     _home_screen->handle_context_menu = boost::bind(&WBContextUI::handle_home_context_menu, this, _1, _2, _3);
     
-#if defined(__APPLE__) || defined(_WIN32)
-    // in Linux, the Home screen is closed/deleted when the app closes, so state should be
-    // saved when the home screen is closed
-    scoped_connect(_wb->signal_app_closing(),boost::bind(save_state, _wb, _home_screen));
-#endif
     // setup context menus
     bec::MenuItemList items, pitems;
     bec::MenuItem item, sep;
@@ -160,8 +328,12 @@ void WBContextUI::show_home_screen()
     item.caption= "Move Down";
     items.push_back(item);
     
-    items.push_back(sep);
+    item.name= "move_connection_to_group";
+    item.caption= "Move to Group...";
+    items.push_back(item);
     
+    items.push_back(sep);
+
     item.name= "delete_connection";
     item.caption= "Delete Connection...";
     items.push_back(item);
@@ -249,6 +421,10 @@ void WBContextUI::show_home_screen()
     item.caption= "Move Down";
     items.push_back(item);
 
+    item.name= "move_instance_to_group";
+    item.caption= "Move to Group...";
+    items.push_back(item);
+
     items.push_back(sep);
     
     item.name= "delete_instance";
@@ -282,13 +458,16 @@ void WBContextUI::show_home_screen()
     
 
   }
+
   mforms::App::get()->dock_view(_home_screen, "maintab");
-  mforms::App::get()->set_view_title(_home_screen, "Home");
+//  mforms::App::get()->set_view_title(_home_screen, "Home"); we use an icon for that page.
   
   refresh_home_models();
   refresh_home_instances(0);
   refresh_home_connections();
   refresh_home_starters();
+
+  _home_screen->startup_finished();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -296,9 +475,15 @@ void WBContextUI::show_home_screen()
 /**
  * Called when the home screen is closed by the UI. We have to clear our reference then.
  */
-void WBContextUI::home_screen_closing()
+bool WBContextUI::home_screen_closing()
 {  
+  if (_home_screen)
+  {
+    save_state(this, _home_screen);
+    _home_screen->release();
+  }
   _home_screen= NULL;
+  return true;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -343,15 +528,38 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
   else if (action == "delete_connection")
   {
     db_mgmt_ConnectionRef connection(db_mgmt_ConnectionRef::cast_from(object));
+    grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
     
     std::string name= connection->name();
     std::string text= strfmt(_("Do you want to delete connection %s?"), name.c_str());
     int answer= Utilities::show_warning(_("Delete Connection"), text,  _("Delete"), _("Cancel"));
     if (answer == mforms::ResultOk)
     {
-      grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+      // Remove password associated with this connection (if stored in keychain/vault). Check first
+      // this service/username combination isn't used anymore by other connections.
+      bool credentials_still_used = false;
+      grt::DictRef parameter_values = connection->parameterValues();
+      std::string host = connection->hostIdentifier();
+      std::string user = parameter_values.get_string("userName");
+      for (grt::ListRef<db_mgmt_Connection>::const_iterator i = connections.begin();
+        i != connections.end(); ++i)
+      {
+        if (*i != connection)
+        {
+          grt::DictRef current_parameters = (*i)->parameterValues();
+          if (host == *(*i)->hostIdentifier() && user == current_parameters.get_string("userName"))
+          {
+            credentials_still_used = true;
+            break;
+          }
+        }
+      }
+      if (!credentials_still_used)
+        mforms::Utilities::forget_password(host, user);
+
       connections->remove(connection);
       refresh_home_connections();
+      _wb->save_connections();
     }
   }
   else if (action == "move_connection_up")
@@ -362,6 +570,7 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
     
     refresh_home_connections();
     _home_screen->select_list_entry(HomeScreenSQLDevList, object);
+    _wb->save_connections();
   }
   else if (action == "move_connection_down")
   {
@@ -371,6 +580,28 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
     
     refresh_home_connections();
     _home_screen->select_list_entry(HomeScreenSQLDevList, object);
+    _wb->save_connections();
+  }
+  else if (action == "move_connection_to_group")
+  {
+    grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+    
+    // Create the available groups for the movement...
+    std::vector<std::string> groups;
+    get_groups_for_movement<db_mgmt_Connection>(connections, object, groups);
+
+    SelectOptionDialog dialog("Move To Group", "Pick a group to move the selected connection to\nor type a name to move it to a new one.", groups);
+    dialog.set_validation_function(boost::bind(&validate_group_for_movement<db_mgmt_Connection>, connections, object, _1));
+    std::string result = dialog.run();
+
+    // At this point the movement is considered valid so we just do it...
+    if( result != "")
+    {
+      move_item_to_group<db_mgmt_Connection>(result, connections, object);
+      refresh_home_connections();
+      _home_screen->select_list_entry(HomeScreenSQLDevList, object);
+      _wb->save_connections();
+    }
   }
   else if (action == "delete_connection_group")
   {
@@ -394,7 +625,7 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
 
         index--;
       }
-
+      _wb->save_connections();
       refresh_home_connections();
     }
   }
@@ -405,9 +636,18 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
     if (answer == mforms::ResultOk)
     {
       grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+      for (grt::ListRef<db_mgmt_Connection>::const_iterator i = connections.begin();
+        i != connections.end(); ++i)
+      {
+        grt::DictRef parameter_values = (*i)->parameterValues();
+        std::string host = (*i)->hostIdentifier();
+        std::string user = parameter_values.get_string("userName");
+        mforms::Utilities::forget_password(host, user);
+      }
       while (connections->count() > 0)
         connections->remove(0);
       refresh_home_connections();
+      _wb->save_connections();
     }
   }
   else if (action == "open_model")
@@ -417,6 +657,41 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
   else if (action == "remove_model")
   {
     _wb->get_root()->options()->recentFiles()->remove(object);
+
+    bool remove_auto_save = false;
+    std::string file = grt::StringRef::cast_from(object);
+    if (file.size() > 5 && file.substr(file.size()-5) == ".mwbd")
+      remove_auto_save = true;
+    else
+    {
+      std::map<std::string, std::string> auto_save_models(WBContextModel::auto_save_files());
+
+      if (auto_save_models.find(file) != auto_save_models.end())
+      {
+        file = auto_save_models[file];
+        remove_auto_save = true;
+      }
+      else if (auto_save_models.find(g_basename(file.c_str())) != auto_save_models.end())
+      {
+        file = auto_save_models[g_basename(file.c_str())];
+        remove_auto_save = true;
+      }
+    }
+    
+    if (remove_auto_save)
+    {
+      try
+      {
+        base::remove_recursive(file);
+
+        // Refreshes the list of autosaved files...
+        WBContextModel::detect_auto_save_files(_wb->get_auto_save_dir());
+      }
+      catch (std::exception &exc)
+      {
+        log_error("Error removing model file %s: %s\n", file.c_str(), exc.what());
+      }
+    }
     refresh_home_models();
   }
   else if (action == "remove_model_all")
@@ -450,6 +725,7 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
       grt::ListRef<db_mgmt_ServerInstance> instances(_wb->get_root()->rdbmsMgmt()->storedInstances());
       instances->remove(instance);
       refresh_home_instances();
+      _wb->save_instances();
     }
   }
   else if (action == "move_instance_up")
@@ -460,6 +736,7 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
     
     refresh_home_instances();
     _home_screen->select_list_entry(HomeScreenAdministrationList, object);
+    _wb->save_instances();
   }
   else if (action == "move_instance_down")
   {
@@ -469,7 +746,29 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
     
     refresh_home_instances();
     _home_screen->select_list_entry(HomeScreenAdministrationList, object);
+    _wb->save_instances();
   }
+  else if (action == "move_instance_to_group")
+  {
+    grt::ListRef<db_mgmt_ServerInstance> instances(_wb->get_root()->rdbmsMgmt()->storedInstances());
+    
+    // Create the available groups for the movement...
+    std::vector<std::string> groups;
+    get_groups_for_movement<db_mgmt_ServerInstance>(instances, object, groups);
+
+    SelectOptionDialog dialog("Move To Group", "Pick a group to move the selected server instance to\nor type a name to move it to a new one.", groups);
+    dialog.set_validation_function(boost::bind(&validate_group_for_movement<db_mgmt_ServerInstance>, instances, object, _1));
+    std::string result = dialog.run();
+
+    if( result != "")
+    {
+      move_item_to_group<db_mgmt_ServerInstance>(result, instances, object);
+      refresh_home_instances();
+      _home_screen->select_list_entry(HomeScreenAdministrationList, object);
+      _wb->save_instances();
+    }
+
+  }  
   else if (action == "delete_instance_group")
   {
     std::string group = object.repr();
@@ -494,6 +793,7 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
       }
       
       refresh_home_instances();
+      _wb->save_instances();
     }
   }
   
@@ -507,6 +807,7 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
       while (instances->count() > 0)
         instances->remove(0);
       refresh_home_instances();
+      _wb->save_instances();
     }
   }
   else 
@@ -524,7 +825,9 @@ void WBContextUI::handle_home_context_menu(HomeScreenListType list, const grt::V
         break;
       case HomeScreenAdministrationList:
         argument_pool.add_entries_for_object("selectedInstance", db_mgmt_ServerInstanceRef::cast_from(object));
-        break;        
+        break;
+      default:
+        break;
     }
 
     get_command_ui()->activate_command(action, argument_pool);
@@ -570,7 +873,7 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
     {
       GUILock lock(_wb, _("Opening 'What's New' web page"), _("The MySQL Workbench news web page is being opened "
                                                               "and should be available in a moment.\n\nPlease stand by..."));
-      show_web_page("http://wb.mysql.com/?page_id=49", true);
+      show_web_page("http://dev.mysql.com/doc/workbench/en/wb-change-history.html", true);
     }
       break;
       
@@ -629,7 +932,7 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
       NewConnectionWizard wizard(_wb->get_root()->rdbmsMgmt());
       wizard.set_title("Setup New Connection");
       wizard.run();
-      _wb->save_app_options();
+      _wb->save_connections();
       refresh_home_connections();
       break;
     }
@@ -660,7 +963,7 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
       editor.run();
       _wb->show_status_text("");
       // save connection list now
-      _wb->save_app_options();
+      _wb->save_connections();
       refresh_home_connections();        
       break;
     }
@@ -780,7 +1083,8 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
       _wb->show_status_text(_("Creating new server instance"));
       if (wizard.run_modal())
       {
-        _wb->save_app_options();
+        _wb->save_instances();
+        _wb->save_connections();
         refresh_home_connections();
         refresh_home_instances((int)_wb->get_root()->rdbmsMgmt()->storedInstances().count()-1);
         _wb->show_status_text(_("New server instance created"));
@@ -835,7 +1139,8 @@ void WBContextUI::handle_home_action(HomeScreenAction action, const grt::ValueRe
       db_mgmt_ServerInstanceRef instance(editor.run());
       _wb->show_status_text("");
       // save instance list now
-      _wb->save_app_options();
+      _wb->save_instances();
+      _wb->save_connections();
       refresh_home_connections();
       refresh_home_instances((int)_wb->get_root()->rdbmsMgmt()->storedInstances().get_index(instance));        
       break;
@@ -850,6 +1155,9 @@ void WBContextUI::handle_starter_action(StarterAction action, const app_StarterR
   int index= 0;
   switch (action)
   {
+    case StarterNone:
+      break;
+
     case StarterPluginCentral:
     {
       GUILock lock(_wb, _("Opening Plugin Central"), _("The MySQL Workbench Plugin Central web page "
@@ -960,7 +1268,7 @@ void WBContextUI::refresh_home_instances(int selected)
     _home_screen->add_list_entry(HomeScreenAdministrationList, (*inst)->name(), text, *inst);
   }
   
-  grt::ValueRef selected_item = ((int) instances.count() > selected) ? instances[selected] : db_mgmt_ServerInstanceRef();
+  grt::ValueRef selected_item = ((int) instances.count() > selected && selected >= 0) ? instances[selected] : db_mgmt_ServerInstanceRef();
 
   _home_screen->select_list_entry(HomeScreenAdministrationList, selected_item);
 
@@ -1026,7 +1334,7 @@ void WBContextUI::refresh_home_connections(int selected)
       _home_screen->add_list_entry(HomeScreenSQLDevList, (*inst)->name(), text, *inst);
   }
   
-  grt::ValueRef selected_item = ((int) connections.count() > selected) ? connections[selected] : db_mgmt_ConnectionRef();
+  grt::ValueRef selected_item = ((int) connections.count() > selected && selected >= 0) ? connections[selected] : db_mgmt_ConnectionRef();
   _home_screen->select_list_entry(HomeScreenSQLDevList, selected_item);
 
   _home_screen->refresh_list(HomeScreenSQLDevList);
@@ -1119,7 +1427,7 @@ void WBContextUI::refresh_home_models(int selected)
     struct _stat stbuf;
     
     if (base_stat((*f).c_str(), &stbuf) < 0)
-      _home_screen->add_list_entry(HomeScreenModellingList, *f, "File not accessible", grt::ValueRef());
+      _home_screen->add_list_entry(HomeScreenModellingList, *f, "File not accessible", *f);
     else
     {
       struct tm t;
