@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,15 +27,15 @@
 #include <algorithm>
 #include <set>
 #include <stdexcept>
+#include <errno.h>
 #include <glib/gstdio.h>
 
 #include "grtpp.h"
 #include "base/string_utilities.h"
 #include "base/file_utilities.h"
+#include "base/file_functions.h"
 #include "mforms/utilities.h"
 
-#include "base/util_public_interface.h"
-
 #include "grt/common.h"
 #include "grt/grt_manager.h"
 
@@ -51,7 +51,8 @@
 // updated to 1.4.0 in 5.2.0
 // updated to 1.4.1 in 5.2.19
 // updated to 1.4.2 in 5.2.33
-#define DOCUMENT_VERSION "1.4.2"
+// updated to 1.4.3 in 5.2.40
+#define DOCUMENT_VERSION "1.4.3"
 
 #define ZIP_FILE_FORMAT "1.0"
 
@@ -240,9 +241,16 @@ time_t get_file_timestamp(const std::string &file)
 void ModelFile::open(const std::string &path, GRTManager *grtm)
 {
   bool file_is_zip;
+  bool file_is_autosave = false;
 
   AutoLock lock(_mutex);
 
+  if (base::is_directory(path) && path[path.size()-1] == 'd')
+  {
+    file_is_zip = false;
+    file_is_autosave = true;
+  }
+  else
   {
     FILE *f= base_fopen(path.c_str(), "rb");
     if (!f)
@@ -281,7 +289,7 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
     throw std::runtime_error("Invalid path "+path);
   }
 
-  std::string auto_save_dir = bec::make_path(_temp_dir, basename).append("d"); // default
+  std::string auto_save_dir = file_is_autosave ? path : bec::make_path(_temp_dir, basename).append("d"); // default
   std::list<std::string> possible_autosaves = base::scan_for_files_matching(auto_save_dir+"*");
   for (std::list<std::string>::const_iterator d = possible_autosaves.begin(); d != possible_autosaves.end(); ++d)
   {
