@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,8 +26,12 @@
 #include "grtdb/db_object_helpers.h"
 #include "grts/structs.db.mysql.h"
 #include "sqlide/table_inserts_loader_be.h"
+#include "base/log.h"
 
 #include "base/string_utilities.h"
+#include "mforms/utilities.h"
+
+DEFAULT_LOG_DOMAIN("ModelUpgrader");
 
 using namespace wb;
 using namespace grt;
@@ -328,7 +332,7 @@ bool ModelFile::attempt_xml_document_upgrade(xmlDocPtr xmldoc, const std::string
       }
       
       
-      xml.traverse_subtree("/physicalModels/0/catalog/schemata/0/tables", sigc::ptr_fun(fix_user_datatypes));
+      xml.traverse_subtree("/physicalModels/0/catalog/schemata/0/tables", std::ptr_fun(fix_user_datatypes));
       
       // add boolean UDT to the userTypes list
       typesList= xml.get_object_by_path("/physicalModels/0/catalog/userDatatypes");
@@ -535,7 +539,7 @@ workbench_DocumentRef ModelFile::attempt_document_upgrade(const workbench_Docume
 
   // in 1.4.1 (after 5.2.19) a new index field was added to db.ForeignKey pointing to the
   // matching index created for the FK and the FOREIGN index type was dropped (just uses INDEX now)
-  if (major <= 1 && (minor < 4 || (minor == 4 && revision == 0)))
+  if (major <= 1 && (minor < 4 || (minor == 4 && revision < 2)))
   {
     ListRef<db_Schema> schemata(doc->physicalModels()[0]->catalog()->schemata());
     for (size_t sc= schemata.count(), si= 0; si < sc; si++)
@@ -587,7 +591,7 @@ workbench_DocumentRef ModelFile::attempt_document_upgrade(const workbench_Docume
           {
             fk->index(found_index);
 
-            g_message("Document Upgrade: ForeignKey %s.%s was assigned the index %s by column matching", 
+            log_info("Document Upgrade: ForeignKey %s.%s was assigned the index %s by column matching\n", 
                       table->name().c_str(), fk->name().c_str(), 
                       found_index->name().c_str());
           }
@@ -598,14 +602,52 @@ workbench_DocumentRef ModelFile::attempt_document_upgrade(const workbench_Docume
             fk->index(new_index);
             table->indices().insert(new_index);
 
-            g_warning("ForeignKey %s.%s has no matching index, created one", 
+            log_warning("ForeignKey %s.%s has no matching index, created one\n", 
                       table->name().c_str(), fk->name().c_str());            
           }
         }
       }
     }
   }
-    
+
+
+  if (major <= 1 && (minor < 4 || (minor == 4 && revision < 3)))
+  {
+      bool ask_confirmation = true;
+      bool aborted = false;
+      ListRef<db_Schema> schemata(doc->physicalModels()[0]->catalog()->schemata());
+      for (size_t sc= schemata.count(), si= 0; (si < sc) && !aborted; si++)
+      {
+          ListRef<db_Table> tables= schemata[si]->tables();
+          for (size_t c= tables.count(), i= 0; (i < c) && !aborted; i++)
+          {
+              ListRef<db_ForeignKey> fks(tables[i]->foreignKeys());
+              ListRef<db_Index> indexes(tables[i]->indices());
+              for (size_t ic= indexes.count(), ii= 0; (ii < ic) && !aborted; ii++)
+                  for (size_t fc= fks.count(), fi= 0; (fi < fc) && !aborted; fi++)
+                      if(fks[fi]->name() == indexes[ii]->name())
+                      {
+                          if (ask_confirmation &&
+                              mforms::Utilities::show_message("Fix Index Names", "Index names identical to FK names were found in the model, which is not allowed for MySQL 5.5 and later. \r\nWould you like to rename the indexes?",
+                              "Rename", "Ignore") != mforms::ResultOk)
+                          {
+                              aborted = true;
+                              break;
+                          }
+                          ask_confirmation = false;
+                          std::string idx_name = indexes[ii]->name();
+                          if (idx_name.length() > 59)
+                              idx_name.resize(59);
+                          indexes[ii]->name(grt::get_name_suggestion_for_list_object(indexes,idx_name.append("_idx"),false));
+                          log_info("Document Upgrade: Index %s.%s was renamed to %s.%s to avoid name conflict with FK\n",
+                              tables[i]->name().c_str(), fks[fi]->name().c_str(),
+                              tables[i]->name().c_str(), indexes[ii]->name().c_str());
+                          continue;
+                      }
+          }
+      }
+  }
+
   // Version * -> X.X.X //!
   if (!has_file(get_rel_db_file_path()))
   {
@@ -813,7 +855,7 @@ static int fix_duplicate_uuid_bug(xmlNodePtr node,
           std::string remapped = strfmt("%s%i", id.c_str(), (int)remapped_ids.size());
           if (remapped_ids.find(id) != remapped_ids.end()
               || remapped_ids[id].find(sname) != remapped_ids[id].end())
-            g_warning("Found more than 1 duplicate entry for same UUID, will be unable to correctly fix file (%s %s)",
+            log_warning("Found more than 1 duplicate entry for same UUID, will be unable to correctly fix file (%s %s)\n",
                       sname.c_str(), id.c_str());
           if (remapped_ids[id].size() == 0)
           {
@@ -860,11 +902,11 @@ static void fix_duplicate_uuid_bug_references(grt::GRT *grt, xmlNodePtr node,
             if (XMLTraverser::node_prop(node, "type") == "list" && XMLTraverser::node_prop(node, "content-type") == "object"
                 && !(sname = XMLTraverser::node_prop(node, "content-struct-name")).empty())
             {
-              g_message("link %s is in list of %s", id.c_str(), sname.c_str());
+              log_info("link %s is in list of %s\n", id.c_str(), sname.c_str());
             }
             else
             {
-              g_message("link %s has no struct-name", id.c_str());
+              log_info("link %s has no struct-name\n", id.c_str());
               continue;
             }
           }
@@ -906,7 +948,7 @@ static void fix_duplicate_uuid_bug_references(grt::GRT *grt, xmlNodePtr node,
                 if (mc->name() == sname)
                 {
                   count++;
-                  g_message("found match %i %s for %s:%s (%s -> %s)", count, opt->first.c_str(), sname.c_str(), target_key.c_str(),
+                  log_info("found match %i %s for %s:%s (%s -> %s)\n", count, opt->first.c_str(), sname.c_str(), target_key.c_str(),
                             id.c_str(), opt->second.c_str());
                   xmlNodeSetContent(n, (xmlChar*)opt->second.c_str());
                   ok = true;
@@ -917,7 +959,7 @@ static void fix_duplicate_uuid_bug_references(grt::GRT *grt, xmlNodePtr node,
             }
 
             if (!ok)
-              g_message("WARNING: Could not resolve link for %s %s (%i) key='%s'. Report to the developers to get a fix.",
+              log_warning("WARNING: Could not resolve link for %s %s (%i) key='%s'. Report to the developers to get a fix.\n",
                         id.c_str(), sname.c_str(), (int)structs.size(),
                         XMLTraverser::node_prop(n, "key").c_str());
           }
