@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,20 +28,18 @@
 #include "model/wb_model_diagram_form.h"
 #include "wb_context_ui.h"
 #include "wb_context.h"
+#include "model/wb_context_model.h"
 #include "model/wb_component.h"
 
 #include "mdc_back_layer.h"
 
 #include "grt/common.h"
 
-#include "base/util_public_interface.h"
-
 #include "wbcanvas/model_figure_impl.h"
 
-#include <mforms/mforms.h>
-
 #include "user_defined_type_editor.h"
 #include "preferences_form.h"
+#include "bug_report_form.h"
 #include "document_properties_form.h"
 #include "grt_shell_window.h"
 #include "plugin_manager_window.h"
@@ -49,12 +47,13 @@
 #include "server_instance_editor.h"
 #include "grtui/grtdb_connection_editor.h"
 #include "base/string_utilities.h"
+#include "base/util_functions.h"
 #include "grtdb/db_helpers.h"
 
 using namespace wb;
 using namespace grt;
 using namespace base;
-using namespace MySQL::Geometry;
+using namespace base;
 
 //--------------------------------------------------------------------------------------------------
 
@@ -82,6 +81,86 @@ void WorkbenchImpl::set_context(WBContext *wb)
   _wb= wb;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Returns a number of system parameters for use in the log and the debug output.
+ */
+std::string WorkbenchImpl::getSystemInfo(bool indent)
+{
+  #ifdef EDITION_OSS
+    #define EDITION_NAME "CE"
+  #elif defined(EDITION_SE)
+    #define EDITION_NAME "SE"
+  #else
+    #define EDITION_NAME "unknown edition"
+  #endif
+
+  #ifdef _WIN32
+    #define PLATFORM_NAME "Windows"
+  #elif defined(__APPLE__)
+    #define PLATFORM_NAME "Mac OS X"
+  #else
+    #define PLATFORM_NAME "Linux/Unix"
+  #endif
+
+  const char* tab = indent ? "\t" : "";
+  std::string result = strfmt("%sMySQL Workbench "EDITION_NAME" for "PLATFORM_NAME" version %i.%i.%i %s revision %i\n",
+    tab, APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, APP_RELEASE_TYPE, APP_REVISION_NUMBER);
+  result += strfmt("%sConfiguration Directory: %s\n", tab, _wb->get_grt_manager()->get_user_datadir().c_str());
+  result += strfmt("%sData Directory: %s\n", tab, _wb->get_grt_manager()->get_basedir().c_str());
+
+  int cver= cairo_version();
+  result += strfmt("%sCairo Version: %i.%i.%i\n", tab, (cver / 10000) % 100, (cver / 100) % 100, cver % 100);
+
+  gchar* tmp;
+  result += strfmt("%sOS: %s\n", tab, tmp = get_local_os_name());
+  g_free(tmp);
+  result += strfmt("%sCPU: %s\n", tab, tmp = get_local_hardware_info());
+  g_free(tmp);
+
+#ifdef _WIN32
+  result += getFullVideoAdapterInfo(indent);
+
+  int locale_buffer_size = GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLANGUAGE, NULL, 0);
+  if (locale_buffer_size > 0)
+  {
+    TCHAR* buffer = new TCHAR[locale_buffer_size];
+    GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLANGUAGE, buffer, locale_buffer_size);
+
+    CW2A converted_buffer(buffer, CP_UTF8);
+    delete [] buffer;
+
+    result += strfmt("%sCurrent user language: %s\n", tab, converted_buffer);
+  }
+  else
+  {
+    // Very unlikely the system cannot return the current user language, but just in case.
+    result += strfmt("%sUnable to determine current user language.\n", tab);
+  }
+
+#elif defined(__APPLE__)
+#else
+  // not sure about how to fetch video adapter info, but could be useful
+  
+  // get distro name/version from lsb_release
+  {
+    int rc;
+    char *stdo;
+    if (g_spawn_command_line_sync("lsb_release -d", &stdo, NULL, &rc, NULL) && stdo)
+    {
+      char *d = strchr(stdo, ':');
+      if (d)
+        result += strfmt("%sDistribution: %s\n", tab, g_strchug(d+1));
+      g_free(stdo);
+    }
+  }
+#endif
+
+  return result;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 #define def_plugin(group, aName, type, aCaption, descr)\
   {\
@@ -253,10 +332,10 @@ ListRef<app_Plugin> WorkbenchImpl::getPluginInfo()
   def_file_plugin ("file", "saveModelAs", "internal", "Save As", "Save Model to a New File", "Save Model", "save", "mwb");
   def_plugin      ("file", "exit", "internal", "Exit", "Exit Workbench");
     
-  def_file_plugin ("export", "exportPNG", "normal", "Export as PNG", "Export Current Diagram as PNG", "Export as PNG", "save", "png");
-  def_file_plugin ("export", "exportPDF", "normal", "Export as PDF", "Export Current Diagram as PDF", "Export as PDF", "save", "pdf");
-  def_file_plugin ("export", "exportSVG", "normal", "Export as SVG", "Export Current Diagram as SVG", "Export as SVG", "save", "svg");
-  def_file_plugin ("export", "exportPS", "normal", "Export as PS", "Export Current Diagram as PostScript", "Export as PostScript", "save", "ps");
+  def_file_plugin ("export", "exportPNG", "standalone", "Export as PNG", "Export Current Diagram as PNG", "Export as PNG", "save", "png");
+  def_file_plugin ("export", "exportPDF", "standalone", "Export as PDF", "Export Current Diagram as PDF", "Export as PDF", "save", "pdf");
+  def_file_plugin ("export", "exportSVG", "standalone", "Export as SVG", "Export Current Diagram as SVG", "Export as SVG", "save", "svg");
+  def_file_plugin ("export", "exportPS", "standalone", "Export as PS", "Export Current Diagram as PostScript", "Export as PostScript", "save", "ps");
 
   def_plugin("edit", "selectAll", "internal", "Select All", "Select All Objects in Diagram");
   def_plugin("edit", "selectSimilar", "internal", "Select Similar Figures", "Select Similar Figures in Diagram");
@@ -304,6 +383,7 @@ ListRef<app_Plugin> WorkbenchImpl::getPluginInfo()
   def_plugin("form", "newGRTFile", "standalone", "New Script File...", "Create a new Workbench script/plugin file");
   def_plugin("form", "openGRTFile", "standalone", "Open Script File...", "Open an existing Workbench script/plugin file");
   def_form_plugin("form", "showPluginManager", "Plugin Manager...", "Show Workbench Plugin Manager Window");
+  def_arg_plugin("form", "reportBug", "standalone", "Report Bug...", "Show Report Bug Window");
   
   def_plugin("debug", "debugValidateGRT", "normal", "Validate GRT Tree", "Validate Consistency of GRT Tree");
   def_plugin("debug", "debugShowInfo", "normal", "Show Debugging Info", "Show various system and application information");
@@ -387,14 +467,14 @@ int WorkbenchImpl::saveModelAs(const std::string &filename)
 
 int WorkbenchImpl::exportPNG(const std::string &filename)
 {
-  _wb->export_png(bec::append_extension_if_needed(filename, ".png"));
+  _wb->get_model_context()->export_png(bec::append_extension_if_needed(filename, ".png"));
 
   return 0;
 }
 
 int WorkbenchImpl::exportPDF(const std::string &filename)
 {
-  _wb->export_pdf(bec::append_extension_if_needed(filename, ".pdf"));
+  _wb->get_model_context()->export_pdf(bec::append_extension_if_needed(filename, ".pdf"));
 
   return 0;
 }
@@ -402,7 +482,7 @@ int WorkbenchImpl::exportPDF(const std::string &filename)
 
 int WorkbenchImpl::exportSVG(const std::string &filename)
 {
-  _wb->export_svg(bec::append_extension_if_needed(filename, ".svg"));
+  _wb->get_model_context()->export_svg(bec::append_extension_if_needed(filename, ".svg"));
 
   return 0;
 }
@@ -410,7 +490,7 @@ int WorkbenchImpl::exportSVG(const std::string &filename)
 
 int WorkbenchImpl::exportPS(const std::string &filename)
 {
-  _wb->export_ps(bec::append_extension_if_needed(filename, ".ps"));
+  _wb->get_model_context()->export_ps(bec::append_extension_if_needed(filename, ".ps"));
 
   return 0;
 }
@@ -581,14 +661,14 @@ int WorkbenchImpl::editSelectedFigureInNewWindow(const model_DiagramRef &view)
 
 int WorkbenchImpl::editObject(const GrtObjectRef &object)
 {
-  _wb->open_object_editor(object, bec::NoFlags);
+  _wb->get_grt_manager()->open_object_editor(object, bec::NoFlags);
   return 0;
 }
 
 
 int WorkbenchImpl::editObjectInNewWindow(const GrtObjectRef &object)
 {
-  _wb->open_object_editor(object, bec::ForceNewWindowFlag);
+  _wb->get_grt_manager()->open_object_editor(object, bec::ForceNewWindowFlag);
   return 0;
 }
 
@@ -763,7 +843,7 @@ int WorkbenchImpl::goToMarker(const std::string &marker)
       diagram->y(mk->y());
 
       _wb->get_grt_manager()->get_dispatcher()->call_from_main_thread<void>(
-        boost::bind(&WBContext::switch_diagram, _wb, diagram), false, false);
+        boost::bind(&WBContextModel::switch_diagram, _wb->get_model_context(), diagram), false, false);
     }
   }
 
@@ -807,6 +887,7 @@ int WorkbenchImpl::setFigureNotation(const std::string &name, workbench_physical
 
   if (model.is_valid() && model.is_instance<workbench_physical_Model>())
     workbench_physical_ModelRef::cast_from(model)->figureNotation(name);
+  _wb->get_wb_options().set("DefaultFigureNotation", grt::StringRef(name));
 
   return 0;
 }
@@ -818,7 +899,7 @@ int WorkbenchImpl::setRelationshipNotation(const std::string &name, workbench_ph
 
   if (model.is_valid() && model.is_instance<workbench_physical_Model>())
     workbench_physical_ModelRef::cast_from(model)->connectionNotation(name);
-
+  _wb->get_wb_options().set("DefaultConnectionNotation", grt::StringRef(name));
   return 0;
 }
 
@@ -1121,98 +1202,8 @@ int WorkbenchImpl::debugGrtStats()
 int WorkbenchImpl::debugShowInfo()
 {
   grt::GRT *grt= get_grt();
-
   grt->make_output_visible();
-
-#ifdef EDITION_OSS
-#define EDITION_NAME "CE"
-#elif defined(EDITION_SE)
-#define EDITION_NAME "SE"
-#else
-#define EDITION_NAME "unknown edition"
-#endif
-
-#ifdef _WIN32
-#define PLATFORM_NAME "Windows"
-#define NATIVE_RENDERER "GDI"
-#elif defined(__APPLE__)
-#define PLATFORM_NAME "Mac OS X"
-#define NATIVE_RENDERER "Quartz"
-#else
-#define PLATFORM_NAME "Linux/Unix"
-#define NATIVE_RENDERER "Native"
-#endif
-
-  grt->send_output(strfmt("MySQL Workbench "EDITION_NAME" for "PLATFORM_NAME" version %i.%i.%i\n", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER));
-
-  grt->send_output(strfmt("Configuration Directory: %s\n", _wb->get_grt_manager()->get_user_datadir().c_str()));
-  grt->send_output(strfmt("Data Directory: %s\n", _wb->get_grt_manager()->get_basedir().c_str()));
-  
-  int cver= cairo_version();
-  grt->send_output(strfmt("Cairo Version: %i.%i.%i\n", (cver/10000)%100, (cver/100)%100, cver%100));
-
-  gchar *tmp;
-  const gchar *render_mode = "Unknown";
-  const gchar *ogl_state;
-  if (_wb->is_opengl_available())
-    ogl_state= "OpenGL is available on this system";
-  else
-    ogl_state= "OpenGL is not available on this system";
-
-  if (_wb->use_opengl())
-  {
-    if (_wb->is_opengl_available())
-      render_mode= "so OpenGL is used for rendering.";
-    else
-      render_mode= "but there is an INTERNAL ERROR, contact the authors."; // Should never happen. Must be a coding mistake.
-  }
-  else
-  {
-    if (_wb->is_opengl_available())
-      render_mode=  ("but "NATIVE_RENDERER" was requested. Using "NATIVE_RENDERER" for rendering.");
-    else
-      render_mode=  ("so "NATIVE_RENDERER" is used for rendering.");
-  }
-
-  grt->send_output(strfmt("Rendering Mode: %s, %s\n", ogl_state, render_mode));
-  if (_wb->use_opengl())
-  {
-    if (_wb->is_opengl_available())
-      grt->send_output(strfmt("OpenGL Driver Version: %s\n", _wb->_opengl_version.c_str()));
-    else
-      grt->send_output(strfmt("OpenGL Driver Version: Not Detected\n"));
-  }
-  grt->send_output(strfmt("OS: %s\n", tmp= get_local_os_name()));
-  g_free(tmp);
-  grt->send_output(strfmt("CPU: %s\n", tmp= get_local_hardware_info()));
-  g_free(tmp);
-
-#ifdef _WIN32
-  grt->send_output(getFullVideoAdapterInfo());
-
-  int locale_buffer_size = GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLANGUAGE, NULL, 0);
-  if (locale_buffer_size > 0)
-  {
-    TCHAR* buffer = new TCHAR[locale_buffer_size];
-    GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLANGUAGE, buffer, locale_buffer_size);
-
-    CW2A converted_buffer(buffer, CP_UTF8);
-    delete [] buffer;
-
-    grt->send_output(strfmt("Current user language: %s\n", converted_buffer));
-  }
-  else
-  {
-    // Very unlikely the system cannot return the current user language, but just in case.
-    grt->send_output("Unable to determine current user language.\n");
-  }
-
-#elif defined(__APPLE__)
-//not needed? maybe get just machine model info #warning NOT IMPLEMENTED
-#else
-#warning not implemented
-#endif
-
+  grt->send_output(getSystemInfo(false));
   grt->send_output("\n");
 
   return 0;
@@ -1330,6 +1321,33 @@ int WorkbenchImpl::showOptions()
   return 0;
 }
 
+int WorkbenchImpl::reportBug(const std::string error_info)
+{
+  grt::Module *module;
+
+  module = _wb->get_ui()->get_wb()->get_grt()->get_module("WbBugReport");
+
+  if (module == NULL)
+    throw std::logic_error("Internal error: can't find Workbench Bug Report module.");
+
+  // Calls the plugin
+  grt::ValueRef resultRef = module->call_function("test_connection",grt::BaseListRef());
+
+  // Gets the information returned by the plugin
+  std::string result = resultRef.repr();
+
+  std::vector<std::string> result_info = base::split(result, "|");
+
+  if (result_info[0] != "success")
+    mforms::Utilities::show_error("Connection Failed", result_info[1], "OK");
+  else
+  {
+    BugReportForm report(_wb->get_ui(), getSystemInfo(false), error_info);
+    report.show();
+  }
+  
+  return 0;
+}
 
 int WorkbenchImpl::showConnectionManager()
 {
@@ -1338,6 +1356,7 @@ int WorkbenchImpl::showConnectionManager()
   editor.run();
   _wb->show_status_text("");  
   _wb->get_ui()->refresh_home_connections();
+  _wb->save_connections();
   
   return 0;
 }
@@ -1350,12 +1369,27 @@ int WorkbenchImpl::showInstanceManager()
   db_mgmt_ServerInstanceRef instance(editor.run());
   _wb->show_status_text("");
   // save instance list now
-  _wb->save_app_options();
+  _wb->save_instances();
   _wb->get_ui()->refresh_home_instances((int)_wb->get_root()->rdbmsMgmt()->storedInstances().get_index(instance));
   return 0;
 }
 
 
+int WorkbenchImpl::saveConnections()
+{
+  _wb->save_connections();
+  return 0;
+}
+
+
+int WorkbenchImpl::saveInstances()
+{
+  _wb->save_instances();
+  return 0;
+}
+
+
+
 int WorkbenchImpl::showQueryConnectDialog()
 {
   _wb->add_new_query_window(db_mgmt_ConnectionRef());
@@ -1901,9 +1935,10 @@ std::string WorkbenchImpl::getVideoAdapter()
 /**
  * Returns all available info about the currently active video adapter in human readable format.
  */
-std::string WorkbenchImpl::getFullVideoAdapterInfo()
+std::string WorkbenchImpl::getFullVideoAdapterInfo(bool indent)
 {
   std::stringstream result;
+  std::string tab = indent ? "\t" : "";
   try
   {
 #ifdef _WIN32
@@ -1919,29 +1954,26 @@ std::string WorkbenchImpl::getFullVideoAdapterInfo()
       if (entry.get_string("Availability", "0") != "3")
         continue;
 
-      // Return the first active adapter we find.
-      result << "Active video adapter: " << entry.get_string("Caption", "") << '\n';
+      result << tab << "Active video adapter (" << i << "): " << entry.get_string("Caption", "") << '\n';
       std::string value = entry.get_string("AdapterRAM", "");
       if (value.size() > 0)
       {
         int size = atoi(value.c_str()) / 1024 / 1024;
-        result << "Installed video RAM: " << size << " MB\n";
+        result << tab << "Installed video RAM: " << size << " MB\n";
       }
       else
-        result << "Installed video RAM: unknown\n";
+        result << tab << "Installed video RAM: unknown\n";
 
-      result << "Current video mode: " << entry.get_string("VideoModeDescription", "unknown") << '\n';
+      result << tab << "Current video mode: " << entry.get_string("VideoModeDescription", "unknown") << '\n';
 
       value = entry.get_string("CurrentBitsPerPixel", "");
       if (value.size() > 0)
-        result << "Used bit depth: " << value << '\n';
+        result << tab << "Used bit depth: " << value << '\n';
       else
-        result << "Used bit depth: unknown\n";
+        result << tab << "Used bit depth: unknown\n";
       
-      result << "Driver version: " << entry.get_string("DriverVersion", "unknown") << '\n';
-      result << "Installed display drivers: " << entry.get_string("InstalledDisplayDrivers", "unknown") << '\n';
-
-      break;
+      result << tab << "Driver version: " << entry.get_string("DriverVersion", "unknown") << '\n';
+      result << tab << "Installed display drivers: " << entry.get_string("InstalledDisplayDrivers", "unknown") << '\n';
     }
 #else
     // TODO: other OSes go here.
