@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -45,6 +45,7 @@ public:
   ~WorkbenchImpl();
 
   void set_context(WBContext *wb);
+  std::string getSystemInfo(bool indent);
 
   DEFINE_INIT_MODULE(WBModule_VERSION, "Oracle Corporation", grt::ModuleImplBase,
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::getPluginInfo),
@@ -140,10 +141,14 @@ public:
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::showConnectionManager),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::showInstanceManager),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::showQueryConnectDialog),
+    DECLARE_MODULE_FUNCTION(WorkbenchImpl::saveConnections),
+    DECLARE_MODULE_FUNCTION(WorkbenchImpl::saveInstances),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::showGRTShell),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::newGRTFile),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::openGRTFile),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::showPluginManager),                     
+    DECLARE_MODULE_FUNCTION(WorkbenchImpl::reportBug),
+
     // Utilities
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::input),
     DECLARE_MODULE_FUNCTION(WorkbenchImpl::confirm),
@@ -284,10 +289,13 @@ private:
   int showConnectionManager();
   int showInstanceManager();
   int showQueryConnectDialog();
+  int saveConnections();
+  int saveInstances();
   int showGRTShell();
   int showPluginManager();
   int newGRTFile();
   int openGRTFile();
+  int reportBug(const std::string error_info = "");
   // UI
   std::string request_input_m(const std::string &caption);
 
@@ -318,7 +326,7 @@ private:
   int createInstancesFromLocalServers();
 
   std::string getVideoAdapter();
-  std::string getFullVideoAdapterInfo();
+  std::string getFullVideoAdapterInfo(bool indent);
 };
 
 };
