@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,6 @@
 
 #include "wb_tunnel.h"
 #include "wb_context.h"
-#include "mforms/mforms.h"
 
 #include "base/string_utilities.h"
 
@@ -41,7 +40,8 @@
 #include "boost/scoped_array.hpp"
 
 #include "base/log.h"
-ENABLE_LOG("wbprivate")
+
+DEFAULT_LOG_DOMAIN("SSH tunnel")
 
 using namespace wb;
 using namespace base;
@@ -56,29 +56,30 @@ class SSHTunnel : public sql::TunnelConnection
 {
   TunnelManager *_tm;
   int _port;
-  
+
 public:
   SSHTunnel(TunnelManager *tm, int port)
   : _tm(tm), _port(port)
-  {    
+  {
   }
-  
+
   virtual ~SSHTunnel()
   {
     disconnect();
   }
-  
+
   virtual int get_port()
   {
     return _port;
   }
-  
+
   virtual void connect(db_mgmt_ConnectionRef connectionProperties)
-  {    
+  {
     if (_port == 0)
       throw std::runtime_error("Could not connect SSH tunnel");
-    
+
     _tm->wait_tunnel(_port);
+
     /*
     if (!g_str_has_prefix(result.c_str(), "OK"))
     {
@@ -98,7 +99,7 @@ public:
       throw std::runtime_error("Could not connect SSH tunnel: "+result);
     }*/
   }
-  
+
   virtual void disconnect()
   {
     _tm->close_tunnel(_port);
@@ -115,11 +116,11 @@ TunnelManager::TunnelManager(wb::WBContext *wb)
 
 void TunnelManager::start()
 {
-  std::string progpath = _wb->get_grt_manager()->get_basedir()+"/sshtunnel.py";
-  
+  std::string progpath = bec::make_path(_wb->get_grt_manager()->get_basedir(), "sshtunnel.py");
+
   grt::WillEnterPython lock;
   grt::PythonContext *py = grt::PythonContext::get();
-  if (py->run_file(progpath.c_str(), false) < 0)
+  if (py->run_file(progpath, false) < 0)
   {
     g_warning("Tunnel manager could not be executed");
     throw std::runtime_error("Cannot start SSH tunnel manager");
@@ -131,7 +132,7 @@ void TunnelManager::start()
 int TunnelManager::lookup_tunnel(const char *server, const char *username, const char *target)
 {
   grt::WillEnterPython lock;
-  
+
   // Note: without the (char*) cast gcc will complain about passing a const char* to a char*.
   //       Ideally the function signature should be changed to take a const char*.
   PyObject *ret = PyObject_CallMethod(_tunnel, (char*) "lookup_tunnel", (char*) "sss", server, username, target);
@@ -181,7 +182,7 @@ int TunnelManager::open_tunnel(const char *server, const char *username, const c
     Py_XDECREF(ret);
     throw std::runtime_error("TunnelManager.open_tunnel returned invalid value");
   }
-  
+
   PyObject *status = PyTuple_GetItem(ret, 0);
   PyObject *value = PyTuple_GetItem(ret, 1);
 
@@ -189,7 +190,7 @@ int TunnelManager::open_tunnel(const char *server, const char *username, const c
   {
     char *error = PyString_AsString(value);
     Py_XDECREF(ret);
-    
+
     if (g_str_has_prefix(error, "Authentication error"))
       throw tunnel_auth_error(error);
 
@@ -199,7 +200,7 @@ int TunnelManager::open_tunnel(const char *server, const char *username, const c
   {
     int port = PyInt_AsLong(value);
     Py_XDECREF(ret);
-    return port;    
+    return port;
   }
 }
 
@@ -220,10 +221,16 @@ void TunnelManager::wait_tunnel(int port)
   }
   std::string str = PyString_AsString(ret);
   Py_XDECREF(ret);
-  
-  if (g_str_has_prefix(str.c_str(), "Authentication error"))
+
+  if ( g_str_has_prefix(str.c_str(), "Bad authentication type") ||
+       g_str_has_prefix(str.c_str(), "Private key file is encrypted") ||
+       g_str_has_prefix(str.c_str(), "Authentication failed")              )
+    throw tunnel_auth_error("Authentication error. Please check that your username and password are correct and try again."
+            "\nDetails (Original exception message):\n" + str);
+
+  if ( g_str_has_prefix(str.c_str(), "Authentication error") )
     throw tunnel_auth_error(str);
-  
+
   throw std::runtime_error("Error connecting SSH tunnel: "+str);
 }
 
@@ -241,15 +248,15 @@ void TunnelManager::close_tunnel(int port)
 }
 
 
-sql::TunnelConnection* TunnelManager::create_tunnel(db_mgmt_ConnectionRef connectionProperties)
+boost::shared_ptr<sql::TunnelConnection> TunnelManager::create_tunnel(db_mgmt_ConnectionRef connectionProperties)
 {
   if (!_tunnel)
   {
-    log_info("Starting tunnel");
+    log_info("Starting tunnel\n");
     start();
   }
 
-  sql::TunnelConnection* tunnel = 0;
+  boost::shared_ptr<sql::TunnelConnection> tunnel;
   grt::DictRef parameter_values= connectionProperties->parameterValues();
 
   if (connectionProperties->driver()->name() == "MysqlNativeSSH")
@@ -270,19 +277,23 @@ sql::TunnelConnection* TunnelManager::create_tunnel(db_mgmt_ConnectionRef connec
     if (tunnel_port > 0)
     {
       _wb->get_grt_manager()->replace_status_text("Existing SSH tunnel found, connecting...");
-      tunnel = new ::SSHTunnel(this, tunnel_port);
+      log_info("Existing SSH tunnel found, connecting\n");
+      tunnel = boost::shared_ptr<sql::TunnelConnection>(new ::SSHTunnel(this, tunnel_port));
     }
     else
-    { 
+    {
       bool reset_password = false;
     retry:
-      
+
       _wb->get_grt_manager()->replace_status_text("Existing SSH tunnel not found, opening new one...");
+      log_info("Existing SSH tunnel not found, opening new one\n");
+      std::string service;
       if (keyfile.empty() && password.empty())
       {
         // interactively ask user for password
+        service = strfmt("ssh@%s", server.c_str());
         if (!mforms::Utilities::find_or_ask_for_password(_("Open SSH Tunnel"),
-                                                        strfmt("ssh@%s", server.c_str()),
+                                                        service,
                                                         username,
                                                         reset_password,
                                                         password))
@@ -302,23 +313,29 @@ sql::TunnelConnection* TunnelManager::create_tunnel(db_mgmt_ConnectionRef connec
         }
 
         // interactively ask user for SSH key passphrase
+        service = strfmt("ssh_keyfile@%s", keyfile.c_str());
         if (encrypted && !mforms::Utilities::find_or_ask_for_password(_("Open SSH Tunnel"),
-                                                                     strfmt("ssh_keyfile@%s", keyfile.c_str()),
+                                                                     service,
                                                                      username,
                                                                      reset_password,
                                                                      password))
           // we need to throw an exception to signal that tunnel could not be opened (and not that it was not needed)
           throw std::runtime_error("SSH key passphrase input cancelled by user");
       }
-      
+
       _wb->get_grt_manager()->replace_status_text("Opening SSH tunnel to "+server+"...");
+      log_info("Opening SSH tunnel to %s\n", server.c_str());
+
       try
       {
         tunnel_port = open_tunnel(server.c_str(), username.c_str(), password.c_str(), keyfile.c_str(), target.c_str());
-        
+
         _wb->get_grt_manager()->replace_status_text("SSH tunnel opened, connecting...");
-        
-        tunnel = new ::SSHTunnel(this, tunnel_port);
+
+        tunnel = boost::shared_ptr<sql::TunnelConnection>(new ::SSHTunnel(this, tunnel_port));
+
+        if (tunnel)
+          tunnel->connect(connectionProperties);
       }
       catch (tunnel_auth_error &exc)
       {
@@ -326,6 +343,8 @@ sql::TunnelConnection* TunnelManager::create_tunnel(db_mgmt_ConnectionRef connec
         if (mforms::Utilities::show_error("Could not connect the SSH Tunnel", exc.what(), _("Retry"), _("Cancel")) == mforms::ResultOk)
         {
           reset_password= true;
+          mforms::Utilities::forget_password(service, username);
+          password = "";
           goto retry;
         }
         else
@@ -337,11 +356,8 @@ sql::TunnelConnection* TunnelManager::create_tunnel(db_mgmt_ConnectionRef connec
         throw std::runtime_error(std::string("Cannot open SSH Tunnel: ").append(exc.what()));
       }
     }
-  }
-
-  if (tunnel)
-  {
-    tunnel->connect(connectionProperties);
+    
+    _wb->get_grt_manager()->replace_status_text("Using SSH tunnel to "+server);
   }
 
   return tunnel;
