@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,7 +24,7 @@
 
 #include "base/string_utilities.h"
 #include "wmi.h"
-#include "grt/common.h"
+#include "base/threading.h"
 
 using namespace wmi;
 using namespace grt;
@@ -172,12 +172,6 @@ WmiMonitor::WmiMonitor(IWbemServices* services, const std::string& parameter)
   }
 
   CComBSTR path = args[0].c_str();
-  //if (parameter == "LoadPercentage")
-  //{
-  //  path = L"Win32_PerfFormattedData_PerfOS_Processor";
-  //  _propertyName = L"PercentProcessorTime";
-  //  _findTotal = true;
-  //}
   _propertyName = args[1].c_str();
 
   if (path.Length() == 0)
@@ -357,7 +351,7 @@ WmiServices::~WmiServices()
 
 void WmiServices::allocate_locator()
 {
-  bec::GStaticMutexLock lock(_locatorMutex_);
+  base::GStaticMutexLock lock(_locatorMutex_);
   if (_locator_refcount_ == 0)
   {
     HRESULT hr = CoCreateInstance(CLSID_WbemLocator, NULL, CLSCTX_INPROC_SERVER, IID_IWbemLocator,
@@ -374,7 +368,7 @@ void WmiServices::allocate_locator()
 
 void WmiServices::deallocate_locator()
 {
-  bec::GStaticMutexLock lock(_locatorMutex_);
+  base::GStaticMutexLock lock(_locatorMutex_);
   if (_locator_refcount_ > 0)
   {
     _locator_refcount_--;
@@ -404,7 +398,7 @@ grt::DictListRef WmiServices::query(grt::GRT* grt, const std::string& query)
   // Making this function explicitly thread-safe might be unnecessary as we don't have
   // any data which is allocated/deallocated concurrently. But since we know we will be called
   // from different threads we play safe here, as it does not harm either.
-  bec::GStaticMutexLock lock(_locatorMutex_);
+  base::GStaticMutexLock lock(_locatorMutex_);
 
   grt::DictListRef queryResult(grt);
 
@@ -465,7 +459,7 @@ grt::DictListRef WmiServices::query(grt::GRT* grt, const std::string& query)
   }
   else
   {
-    throw std::runtime_error("WMI query execution failed.\n\n" + wmiResultToString(hr));
+    throw std::runtime_error("WMI query execution failed with this error:" + wmiResultToString(hr));
   }
 
   return queryResult;
@@ -488,7 +482,7 @@ grt::DictListRef WmiServices::query(grt::GRT* grt, const std::string& query)
  */
 std::string WmiServices::serviceControl(const std::string& service, const std::string& action)
 {
-  bec::GStaticMutexLock lock(_locatorMutex_);
+  base::GStaticMutexLock lock(_locatorMutex_);
 
   CComBSTR instancePath = string2Bstr("Win32_Service.Name='" + service + "'");
 
@@ -635,7 +629,7 @@ std::string WmiServices::serviceControl(const std::string& service, const std::s
  */
 std::string WmiServices::systemStat(const std::string& what)
 {
-  bec::GStaticMutexLock lock(_locatorMutex_);
+  base::GStaticMutexLock lock(_locatorMutex_);
 
   CComBSTR instancePath = string2Bstr("Win32_OperatingSystem");
 
