@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,9 +20,12 @@
 
 #include "common.h"
 #include "base/string_utilities.h"
+#include "base/threading.h"
 
 #include <cstdlib>
 
+using namespace base;
+
 // XXX: most of the code here is not utf-8 safe, check that.
 // TODO: The content here should all move the base library.
 namespace bec {
@@ -91,15 +94,6 @@ namespace bec {
     }
   }
 
-
-  std::string rtrim(const std::string &value)
-  {
-    size_t trailing_spaces_count= 0;
-    for (std::string::const_reverse_iterator i= value.rbegin(), i_end= value.rend(); (i != i_end) && std::isspace(*i); ++i, ++trailing_spaces_count);
-    return value.substr(0, value.size()-trailing_spaces_count);
-  }
-
-
   std::string replace_variable(const std::string &format, const std::string &variable, const std::string &value)
   {
 
@@ -187,30 +181,6 @@ namespace bec {
     return prefix+G_DIR_SEPARATOR+file;
   }  
 
-  GStaticMutexLock::GStaticMutexLock(GStaticMutex& mtx) 
-  : mutex(mtx) 
-  {   
-    g_static_mutex_lock(&mutex);  
-  }
-  
-  GStaticMutexLock::~GStaticMutexLock()                               
-  {
-    g_static_mutex_unlock(&mutex); 
-  }
-  
-  
-  GStaticRecMutexLock::GStaticRecMutexLock(GStaticRecMutex& mtx) 
-  : mutex(mtx) 
-  {   
-    g_static_rec_mutex_lock(&mutex);  
-  }
-  
-  GStaticRecMutexLock::~GStaticRecMutexLock()
-  {
-    g_static_rec_mutex_unlock(&mutex); 
-  }  
-
-
   TimerActionThread::TimerActionThread(const Action &action, gulong milliseconds) : _action(action), _microseconds(milliseconds * 1000)
   {
     _action_mutex= g_mutex_new();
