@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -47,7 +47,8 @@ namespace bec {
   {
     MatchAny,
     MatchBefore,
-    MatchAfter
+    MatchAfter,
+    MatchLast
   };
   
   enum MoveType
@@ -56,6 +57,12 @@ namespace bec {
     MoveDown
   };
 
+  enum FindType
+  {
+    FindPrefix,
+    FindFull
+  };
+
 #define DATETIME_FMT "%Y-%m-%d %H:%M"
   std::string WBPUBLICBACKEND_PUBLIC_FUNC fmttime(time_t t= 0, const char *fmt= "%b %d, %Y");
 
@@ -65,8 +72,6 @@ namespace bec {
   void WBPUBLICBACKEND_PUBLIC_FUNC replace_string_inplace(std::string &text,
                             const std::string &from,
                             const std::string &to);
-  std::string WBPUBLICBACKEND_PUBLIC_FUNC rtrim(const std::string &value);
-
   
   // replaces a variable from a string in format %variable%
   // a filter can be passed to the variable as in %variable|filter%
@@ -109,11 +114,15 @@ namespace bec {
   }
   
   template <class T>
-  size_t find_list_ref_item_position(grt::ListRef<T> &item_data, std::string& name, MatchType match = MatchAny, grt::Ref<T>* reference = NULL)
+  size_t find_list_ref_item_position(grt::ListRef<T> &item_data, std::string& name, MatchType match = MatchAny, grt::Ref<T>* reference = NULL, FindType find_mode = FindPrefix)
   {
+
+    if ((match == MatchBefore || match == MatchAfter) && !reference)
+        throw std::invalid_argument("A reference must be specified for MatchBefore and MatchAfter");
+
     bool search_enabled = match != MatchAfter;
     bool exit = false;
-    
+
     size_t index = grt::BaseListRef::npos;
     
     for ( grt::TypedListConstIterator<T> end = item_data.end(),
@@ -129,14 +138,17 @@ namespace bec {
         {
           std::string item_name = (*inst)->name();
           
+          int compare_result = (find_mode == FindPrefix) ? item_name.compare(0, name.length(), name) : item_name.compare(name);
+
           // index will contain always the position of the last matched entry
-          if (item_name.compare(0, name.length(), name) == 0)
+          if (compare_result == 0)
           {
             index = item_data.get_index(*inst);
             
-            // MatchBefore needs to search until the reference is found, the others are done
-            // as soon as a match is found
-            if (match != MatchBefore)
+            // MatchBefore needs to search until the reference is found
+            // MatchLast needs to search until the whole list has been searched to get the last match
+            // MatchAfter and MatchAny are done as soon as a match is found
+            if (match != MatchBefore && match != MatchLast)
               exit = true;
           }
         }
@@ -245,63 +257,6 @@ namespace bec {
     
   }  
 
-  //! \brief RAII style GMutex locker/unlocker  
-  class WBPUBLICBACKEND_PUBLIC_FUNC GMutexLock
-  {
-    GMutex *mutex;
-  public:
-    GMutexLock(GMutex *mtx) : mutex(mtx)
-    {
-      if (mutex)
-        g_mutex_lock(mutex);
-    }
-    GMutexLock(const GMutexLock &lock)
-    {
-      GMutexLock *lock_= const_cast<GMutexLock*>(&lock);
-      operator=(*lock_);
-    }
-    ~GMutexLock()
-    {
-      reset();
-    }
-    GMutex * release()
-    {
-      GMutex *res= mutex;
-      mutex= NULL;
-      return res;
-    }
-    void reset()
-    {
-      if (mutex)
-      {
-        g_mutex_unlock(mutex);
-        mutex= NULL;
-      }
-    }
-    GMutexLock & operator=(GMutexLock& lock)
-    {
-      mutex= lock.release();
-      return *this;
-    }
-  };
-
-  //! \brief RAII style GStaticMutex locker/unlocker  
-  class WBPUBLICBACKEND_PUBLIC_FUNC GStaticMutexLock // Note: must compile with strict-aliasing disabled (-fno-strict-aliasing in gcc)
-  {
-      GStaticMutex    &mutex;
-    public:
-    GStaticMutexLock(GStaticMutex& mtx);
-    ~GStaticMutexLock();
-  };
-
-  class WBPUBLICBACKEND_PUBLIC_FUNC GStaticRecMutexLock // Note: must compile with strict-aliasing disabled (-fno-strict-aliasing in gcc)
-  {
-    GStaticRecMutex    &mutex;
-  public:
-    GStaticRecMutexLock(GStaticRecMutex& mtx);
-    ~GStaticRecMutexLock();
-  };
-
   class WBPUBLICBACKEND_PUBLIC_FUNC TimerActionThread
   {
   public:
