@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,11 +23,6 @@
 
 using namespace bec;
 
-
-
-
-
-
 UndoObjectChangeGroup::UndoObjectChangeGroup(const std::string &object_id, const std::string &member)
   : _object_id(object_id), _member(member)
 {
@@ -48,8 +43,9 @@ bool UndoObjectChangeGroup::matches_group(UndoGroup *group) const
 
 
 BaseEditor::BaseEditor(GRTManager *grtm, const grt::Ref<GrtObject> &object)
-: _grtm(grtm), _object(object)
+: _grtm(grtm), _ignore_object_changes_for_ui_refresh(false), _object(object)
 {
+  _ignored_object_changes_for_ui_refresh = 0;
   if (object.is_valid())
     add_listeners(object);
 }
@@ -80,26 +76,63 @@ void BaseEditor::object_member_changed(const std::string &member, const grt::Val
 }
 
 
+void BaseEditor::freeze_refresh_on_object_change()
+{
+  _ignore_object_changes_for_ui_refresh++;
+}
+
+
+void BaseEditor::thaw_refresh_on_object_change(bool discard_pending)
+{
+  _ignore_object_changes_for_ui_refresh--;
+  if (_ignore_object_changes_for_ui_refresh == 0)
+  {
+    if (_ignored_object_changes_for_ui_refresh > 0 && !discard_pending)
+      on_object_changed();
+    _ignored_object_changes_for_ui_refresh = 0;
+  }
+}
+
+void BaseEditor::apply_changes_to_live_object()
+{
+  commit_changes();
+}
+
+/**
+ * Checks if the editor can be closed and returns true if so. In case there are pending changes in 
+ * the editor control the user is asked to apply or cancel them first prior to closing.
+ * Must be called in the context of the main thread.
+ */
+bool BaseEditor::can_close()
+{
+  if (!UIForm::can_close())
+    return false;
+
+  if (is_editor_dirty())
+    return false;
+  return true;
+}
+
 void BaseEditor::on_object_changed()
 {
-  // calling ui_refresh from here will cause refresh to be called from the GRT thread
-  // which must not happen. delaying it to be executing when idle will make it 
-  // get called on main thread
-
-//  do_ui_refresh();
-  //if (_refresh_ui)
-  //  _refresh_ui();
-
-  if (_grtm->in_main_thread())
-    do_ui_refresh();
+  if (_ignore_object_changes_for_ui_refresh == 0)
+  {
+    // calling ui_refresh from here will cause refresh to be called from the GRT thread
+    // which must not happen. delaying it to be executing when idle will make it 
+    // get called on main thread
+    if (_grtm->in_main_thread())
+      do_ui_refresh();
+    else
+      _ui_refresh_conn = _grtm->run_once_when_idle(boost::bind(&RefreshUI::do_ui_refresh, this));
+  }
   else
-    _ui_refresh_conn = _grtm->run_once_when_idle(boost::bind(&RefreshUI::do_ui_refresh, this));
+    _ignored_object_changes_for_ui_refresh++;
 }
 
 
 void BaseEditor::undo_applied()
 {
-    _ui_refresh_conn = _grtm->run_once_when_idle(boost::bind(&RefreshUI::do_ui_refresh, this));
+  _ui_refresh_conn = _grtm->run_once_when_idle(boost::bind(&RefreshUI::do_ui_refresh, this));
 }
 
 
