@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,6 +29,8 @@ using namespace bec;
 static bool debug_dispatcher= false;
 static GStaticRecMutex retain_mutex = G_STATIC_REC_MUTEX_INIT;
 
+DEFAULT_LOG_DOMAIN("GRTDispatcher");
+
 #ifdef __GNUC__
 #define DPRINT(fmt, ...) if (debug_dispatcher) g_message(fmt,##__VA_ARGS__)
 #else
@@ -90,7 +92,8 @@ private:
 //---------------------------------------------------------------------------
 
 GRTTaskBase::GRTTaskBase(const std::string &name, GRTDispatcher *disp)
-  : _dispatcher(disp), _exception(0), _name(name), _refcount(1), _cancelled(false), _finished(false)
+  : _dispatcher(disp), _exception(0), _name(name), _refcount(1), _cancelled(false), _finished(false),
+_messages_to_main_thread(true)
 {
 }
 
@@ -203,31 +206,29 @@ private:
 };
 
 
-void GRTTaskBase::process_message(const grt::Message &msg)
+bool GRTTaskBase::process_message(const grt::Message &msg)
 {
   retain();
-
-  _dispatcher->call_from_main_thread<void>(boost::bind(&GRTTaskBase::process_message_m, this, msg), false/*was true*/, false);
+  if (_messages_to_main_thread)
+    _dispatcher->call_from_main_thread<void>(boost::bind(&GRTTaskBase::process_message_m, this, msg), false/*was true*/, false);
+  else
+    process_message_m(msg);
+  
+  return true;
 }
 
 
 void GRTTaskBase::process_message_m(const grt::Message &msg)
 {
-  // fallback to the original handlerf
-  if(_default_callbacks.message_cb)
-    _default_callbacks.message_cb(msg, this);
+  // if there's no message handler, the default one will be called
+//  // fallback to the original handler
+//  if (_default_callbacks.message_cb)
+//    _default_callbacks.message_cb(msg, this);
 
   release();
 }
 
 
-int GRTTaskBase::perform_status_query()
-{
-  // calling cancel() on the task will cause it to flag the GRT to cancel the task
-  return is_cancelled() ? 1 : 0;
-}
-
-
 //---------------------------------------------------------------------------
 
 
@@ -267,14 +268,17 @@ void GRTTask::failed_m(const std::exception &error)
 }
 
 
-void GRTTask::process_message_m(const grt::Message &msgs)
+bool GRTTask::process_message(const grt::Message &msg)
 {
   if (_message.empty())
-  {
-    GRTTaskBase::process_message_m(msgs);
-    return;
-  }
+    return false;
+  
+  return GRTTaskBase::process_message(msg);
+}
 
+
+void GRTTask::process_message_m(const grt::Message &msgs)
+{
   _message(msgs);
 
   release();
@@ -306,6 +310,16 @@ void GRTShellTask::finished_m(const grt::ValueRef &result)
   GRTTaskBase::finished_m(result);
 }
 
+
+bool GRTShellTask::process_message(const grt::Message &msg)
+{
+  if (_message.empty())
+    return false;
+  
+  return GRTTaskBase::process_message(msg);
+}
+
+
 void GRTShellTask::process_message_m(const grt::Message &msg)
 {
   _message(msg);
@@ -323,7 +337,7 @@ static void sleep_2ms()
 static GThread *_main_thread= 0;
 
 GRTDispatcher::GRTDispatcher(grt::GRT *grt, bool threaded, bool is_main_dispatcher)
-  : _busy(false), _threading_disabled(!threaded), _is_main_dispatcher(is_main_dispatcher), _grt(grt), _current_task(NULL), _shut_down(false)
+  : _busy(false), _threading_disabled(!threaded), _is_main_dispatcher(is_main_dispatcher), _shut_down(false), _grt(grt), _current_task(NULL)
 {
   _worker_running= false;
   _worker_shutdown= false;
@@ -377,10 +391,8 @@ void GRTDispatcher::start(boost::shared_ptr<GRTDispatcher> self)
 
   if (_is_main_dispatcher)
   {
-    DispatcherCallbacks cb;
-    cb.message_cb= _grt->set_message_handler(boost::bind(&GRTDispatcher::message_callback, this, _1, _2));
-    cb.status_query_cb= _grt->set_status_query_handler(boost::bind(&GRTDispatcher::status_query_callback, this));
-    _default_callbacks= cb;
+    _grt->push_message_handler(boost::bind(&GRTDispatcher::message_callback, this, _1, _2));
+//    _grt->push_status_query_handler(boost::bind(&GRTDispatcher::status_query_callback, this));
   }
 }
 
@@ -392,9 +404,8 @@ void GRTDispatcher::shutdown()
   _shut_down = true;
   if (_is_main_dispatcher)
   {
-    DispatcherCallbacks &cb(_default_callbacks);
-    _grt->set_message_handler(cb.message_cb);
-    _grt->set_status_query_handler(cb.status_query_cb);
+    _grt->pop_message_handler();
+//    _grt->pop_status_query_handler();
   }
 
   _worker_shutdown= true;
@@ -494,6 +505,8 @@ gpointer GRTDispatcher::worker_thread(gpointer data)
       task->release();
       continue;
     }
+    
+    int count = self->grt()->message_handler_count();
 
     // do pre-execution preparations
     self->prepare_task(task);
@@ -501,6 +514,17 @@ gpointer GRTDispatcher::worker_thread(gpointer data)
     // execute the task
     self->execute_task(task);
 
+    if (task->get_error())
+    {
+      DPRINT("%s", std::string(("worker: task '"+task->name()+"' has failed with error:.")+task->get_error()->what()).c_str());
+      task->release();
+      continue;
+    }
+
+    if (count != self->grt()->message_handler_count())
+      log_error("INTERNAL ERROR: Message handler count mismatch after executing task '%s' (%i vs %i)", 
+                task->name().c_str(), count, self->grt()->message_handler_count());
+
     // cleanup
     task->release();
 
@@ -522,14 +546,14 @@ gpointer GRTDispatcher::worker_thread(gpointer data)
 
 void GRTDispatcher::execute_now(GRTTaskBase *task)
 {
-    _busy= true;
-    prepare_task(task);
-    
-    execute_task(task);
-
-    task->release();
-    _busy= false;
-  }
+  _busy= true;
+  prepare_task(task);
+  
+  execute_task(task);
+  
+  task->release();
+  _busy= false;
+}
 
 
 void GRTDispatcher::add_task(GRTTaskBase *task)
@@ -582,7 +606,10 @@ void GRTDispatcher::flush_pending_callbacks()
   {
     while ((callback= static_cast<DispatcherCallbackBase*>(g_async_queue_try_pop(_callback_queue))))
     {
-      callback->execute();
+      // Don't run any task, but clear the queue if we are shutting down.
+      // This way the NullTask can surface up in the worker thread.
+      if (!_worker_shutdown)
+        callback->execute();
       callback->signal();
       callback->release();  // this release <--------------------------------------------------+
     }                                                                                     //   |
@@ -639,35 +666,34 @@ void GRTDispatcher::worker_thread_iteration()
 }
 
 
-void GRTDispatcher::message_callback(const grt::Message &msgs, void *sender)
+bool GRTDispatcher::message_callback(const grt::Message &msgs, void *sender)
 {
   if (GRTTaskBase *task= (sender) ? static_cast<GRTTaskBase*>(sender) : _current_task)
-    task->process_message(msgs);
-  else
-    _default_callbacks.message_cb(msgs, sender);
+    return task->process_message(msgs);
+  return false; // let it bubble up by default
 }
 
-
+/*
 bool GRTDispatcher::status_query_callback()
 {
   if (_current_task)
     return _current_task->perform_status_query() != 0;
   else
-    return _default_callbacks.status_query_cb() != 0;
+    return _grt->query_status();
 }
+*/
 
-
-static void call_process_message(const grt::Message &msgs, void *sender, GRTTaskBase *task)
+static bool call_process_message(const grt::Message &msgs, void *sender, GRTTaskBase *task)
 {
   if (sender)
     task= static_cast<GRTTaskBase*>(sender);
-  task->process_message(msgs);
+  return task->process_message(msgs);
 }
-
+/*
 static bool call_status_query(GRTTaskBase *task)
 {
   return task->perform_status_query() != 0;
-}
+}*/
 
 
 void GRTDispatcher::prepare_task(GRTTaskBase *gtask)
@@ -679,8 +705,8 @@ void GRTDispatcher::prepare_task(GRTTaskBase *gtask)
   // directly set the task callbacks
   if (_is_main_dispatcher)
   {
-    _grt->set_message_handler(boost::bind(call_process_message, _1, _2, gtask));
-    _grt->set_status_query_handler(boost::bind(call_status_query, gtask));
+    _grt->push_message_handler(boost::bind(call_process_message, _1, _2, gtask));
+//    _grt->push_status_query_handler(boost::bind(call_status_query, gtask));
   }
 }
 
@@ -690,8 +716,8 @@ void GRTDispatcher::restore_callbacks(GRTTaskBase *task)
   // restore originally set msg callbacks
   if (_is_main_dispatcher)
   {
-    _grt->set_message_handler(boost::bind(&GRTDispatcher::message_callback, this, _1, _2));
-    _grt->set_status_query_handler(boost::bind(&GRTDispatcher::status_query_callback, this));
+    _grt->pop_message_handler();
+//    _grt->pop_status_query_handler();
   }
  
 //  _grt->flush_messages();
