@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,9 +26,11 @@
 #include <python_context.h>
 
 #include "grt/grt_manager.h"
-#include "grt/common.h"
+#include "base/threading.h"
 #include "glib/gstdio.h"
+#include "base/log.h"
 
+#include "base/notifications.h"
 #include "base/file_functions.h"
 #include "base/string_utilities.h"
 #include "mforms/utilities.h"
@@ -37,6 +39,7 @@ using namespace grt;
 using namespace bec;
 using namespace base;
 
+DEFAULT_LOG_DOMAIN("GRTManager");
 
 static GThread *main_thread= 0;
 
@@ -75,14 +78,11 @@ GRTManager::GRTManager(bool threaded, bool verbose)
   _idle_mutex= g_mutex_new();
   _timer_mutex= g_mutex_new();
   _disp_map_mutex= g_mutex_new();
-  _tmp_output_cb_stack_mutex= g_mutex_new();
 
   _grt= new GRT();
   
   _grt->set_verbose(verbose);
   
-  _grt->set_message_handler(boost::bind(&GRTManager::grt_default_msg_cb, this, _1, _2));
-
   _terminated= false;
   _idle_blocked= false;
   _clipboard= 0;
@@ -101,8 +101,6 @@ GRTManager::GRTManager(bool threaded, bool verbose)
   _plugin_manager= _grt->get_native_module<PluginManagerImpl>();
 
   _value_tree= 0;
-  _structs_tree= 0;
-  _modules_tree= 0;
   _messages_list= new MessageListStorage(this);
 }
 
@@ -216,8 +214,6 @@ GRTManager::~GRTManager()
     _instances.erase(_grt);
   }  
 
-  _grt->set_message_handler(MessageSlot());
-
   _dispatcher->shutdown();
   _dispatcher.reset();
 
@@ -225,10 +221,6 @@ GRTManager::~GRTManager()
   _shell = 0;
   delete _value_tree;
   _value_tree = 0;
-  delete _structs_tree;
-  _structs_tree = 0;
-  delete _modules_tree;
-  _modules_tree = 0;
   delete _messages_list;
   _messages_list = 0;
 
@@ -241,7 +233,6 @@ GRTManager::~GRTManager()
   g_mutex_free(_disp_map_mutex);
   g_mutex_free(_timer_mutex);
   g_mutex_free(_idle_mutex);
-  g_mutex_free(_tmp_output_cb_stack_mutex);
 }
 
 
@@ -312,179 +303,23 @@ ValueInspectorBE *GRTManager::get_new_grouped_object_inspector(const grt::ValueR
 }
 
 
-StructsTreeBE *GRTManager::get_shared_structs_tree()
-{
-  if (!_structs_tree)
-    _structs_tree= new StructsTreeBE(_grt);
-
-  return _structs_tree;
-}
-
-
-ModulesTreeBE *GRTManager::get_shared_modules_tree()
-{
-  if (!_modules_tree)
-    _modules_tree= new ModulesTreeBE(_grt);
-
-  return _modules_tree;
-}
-
-
-void GRTManager::set_message_callback(const boost::function<void (grt::Message)> &cb)
-{
-  _message_cb= cb;
-}
-
-void GRTManager::set_progress_callback(const boost::function<bool (std::string,std::string,float)> &cb)
-{
-  _progress_cb= cb;
-}
-
-
-void GRTManager::push_output_callback(const boost::function<bool (std::string)> &slot)
-{
-  GMutexLock lock(_tmp_output_cb_stack_mutex);
-  _tmp_output_cb_stack.push_back(slot);
-}
-
-
-void GRTManager::pop_output_callback()
-{
-  GMutexLock lock(_tmp_output_cb_stack_mutex);
-  _tmp_output_cb_stack.pop_back();
-}
-
-
-void GRTManager::task_started_cb(const std::string &title)
-{
-  if (_progress_cb)
-    _progress_cb(title, "", 0.0);
-}
-
-
-void GRTManager::task_finished_cb(const grt::ValueRef &result)
-{
-  if (_progress_cb)
-    _progress_cb("", "", -1);
-}
-
-
 void GRTManager::task_error_cb(const std::exception &error, const std::string &title)
 {
   mforms::Utilities::show_error(title, error.what(), _("Close"));
 }
 
 
-void GRTManager::task_msg_cb(const grt::Message &msg, const std::string &title, bool show_progress)
-{
-  // some day only OuputMsg will exist in here and everything else will be optional for parts that need it
-  // (like in modeling). progress messaging should also maybe be separate
-  if (msg.type == OutputMsg)
-  {
-    GMutexLock lock(_tmp_output_cb_stack_mutex);
-    
-    if (!_tmp_output_cb_stack.empty())
-    {
-      std::list<boost::function<bool (std::string)> >::reverse_iterator iter = _tmp_output_cb_stack.rbegin();
-      while (iter != _tmp_output_cb_stack.rend())
-      {
-        if ((*iter)(msg.text))
-          return;
-      }
-    }
-  }
-  
-  switch (msg.type)
-  {
-  case ErrorMsg:
-  case WarningMsg:
-    _grt->make_output_visible();
-    // fall through
-  case InfoMsg:
-  case OutputMsg:
-  case ControlMsg:
-    if (_message_cb)
-      _message_cb(msg);
-    break;
-  case VerboseMsg:
-    if (_verbose && _message_cb)
-      _message_cb(msg);
-    break;
-  case ProgressMsg:
-    if (show_progress && _progress_cb)
-      _progress_cb(msg.text, msg.detail, msg.progress);
-    break;
-  default:
-    g_message("unhandled message %i: %s", msg.type, msg.format().c_str());
-    break;
-  }
-}
-
-
-void GRTManager::grt_default_msg_cb(const grt::Message &msg, void *sender)
-{
-  if (msg.type == OutputMsg)
-  {
-    GMutexLock lock(_tmp_output_cb_stack_mutex);
-    
-    if (!_tmp_output_cb_stack.empty())
-    {
-      _tmp_output_cb_stack.back()(msg.text);
-      return;
-    }
-  }
-
-  switch (msg.type)
-  {
-    case ProgressMsg:
-      if (_progress_cb)
-        _progress_cb(msg.text, msg.detail, msg.progress);
-      break;
-      
-    default:
-    case ErrorMsg:
-    case WarningMsg:
-    case InfoMsg:
-    case ControlMsg:
-      if (_message_cb)
-        _message_cb(msg);
-      break;
-  }
-}
-
-
 void GRTManager::execute_grt_task(const std::string &title,
                                   const boost::function<grt::ValueRef (grt::GRT*)> &function,
-                                  const boost::function<void (grt::ValueRef)> &finished_cb,
-                                  bool show_progress)
+                                  const boost::function<void (grt::ValueRef)> &finished_cb)
 {
   GRTTask *task(new GRTTask(title, _dispatcher.get(), function));
 
-  if (show_progress)
-  {
-    scoped_connect(task->signal_started(),boost::bind(&GRTManager::task_started_cb, this, title));
-    scoped_connect(task->signal_finished(),boost::bind(&GRTManager::task_finished_cb, this, _1));
-  }
-
   // connect finished_cb provided by caller (after ours)
   task->signal_finished()->connect(finished_cb);
 
   scoped_connect(task->signal_failed(),boost::bind(&GRTManager::task_error_cb, this, _1, title));
 
-  scoped_connect(task->signal_message(),boost::bind(&GRTManager::task_msg_cb, this, _1, title, show_progress));
-
-  _dispatcher->add_task(task);
-}
-
-
-void GRTManager::dispatch_task(GRTTask *task)
-{
-  if (!task->signal_message()->empty())
-  {
-    // make msgs be sent to output window as well
-    scoped_connect(task->signal_message(), boost::bind(&GRTManager::task_msg_cb, this, _1, "", false));
-  }
-
   _dispatcher->add_task(task);
 }
 
@@ -603,12 +438,16 @@ void GRTManager::perform_idle_tasks()
 
   if (!_idle_blocked)
   {
+    block_idle_tasks();
+
     int emited_group = _current_idle_serial;
     ++_current_idle_serial;
     // emit everything
     _idle_signal();
 
     _idle_signal.disconnect(emited_group);
+
+    unblock_idle_tasks();
   }
 }
 
@@ -936,13 +775,6 @@ bool GRTManager::init_loaders(const std::string &loader_module_path)
     if (grt::init_python_support(_grt, loader_module_path))
     {
       if (_verbose) _shell->write_line(_("Python loader initialized."));
-
-      PythonModuleLoader *loader = dynamic_cast<PythonModuleLoader*>(_grt->get_module_loader("python"));
-      if (loader)
-      {
-        loader->get_python_context()->push_output_handler = boost::bind(&GRTManager::push_output_callback, this, _1);
-        loader->get_python_context()->pop_output_handler = boost::bind(&GRTManager::pop_output_callback, this);
-      }
     }
   }
   catch (std::exception &exc)
@@ -1185,3 +1017,47 @@ bool GRTManager::check_plugin_runnable(const app_PluginRef &plugin, const bec::A
   }
   return true;
 }
+
+//--------------------------------------------------------------------------------------------------
+
+void GRTManager::open_object_editor(const GrtObjectRef &object, bec::GUIPluginFlags flags)
+{
+  try 
+  {
+    grt::BaseListRef args(_grt, AnyType);
+    args.ginsert(object);
+    
+    app_PluginRef plugin(_plugin_manager->select_plugin_for_input("catalog/Editors", args));
+    if (!plugin.is_valid())
+      plugin= _plugin_manager->select_plugin_for_input("model/Editors", args);
+    
+    if (plugin.is_valid())
+      _plugin_manager->open_gui_plugin(plugin, args, flags);
+    else
+    {
+      log_error("No suitable editor found for object of type '%s'.", 
+                object.get_metaclass()->get_attribute("caption").c_str());
+
+      mforms::Utilities::show_error(_("Edit Object"), 
+                                    strfmt(_("No suitable editor found for object of type '%s'."), 
+                                           object.get_metaclass()->get_attribute("caption").c_str()), 
+                                    "OK");
+    }
+  }
+  catch (grt::grt_runtime_error &exc) 
+  {
+    log_error("Exception in Open object editor: %s\n%s", exc.what(), exc.detail.c_str());
+
+    mforms::Utilities::show_error(_("Edit Object"), 
+                                  strfmt("%s\n%s", exc.what(), exc.detail.c_str()),
+                                  "OK");
+  }
+  catch (std::exception &exc) 
+  {
+    log_exception("Open object editor", exc);
+    mforms::Utilities::show_error(_("Edit Object"), 
+                                  strfmt("%s", exc.what()),
+                                  "OK");    
+  }
+}
+
