@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,20 +21,18 @@
 
 #include <grtpp.h>
 
+#include "base/trackable.h"
 #include "common.h"
 #include "grt_dispatcher.h"
 #include "grt_shell.h"
 #include "grt_value_tree.h"
 #include "grt_value_inspector.h"
-#include "grt_structs_tree.h"
-#include "grt_modules_tree.h"
 #include "grt_message_list.h"
 
 #include "wbpublic_public_interface.h"
 
 #include "plugin_manager.h"
-#include <boost/lambda/bind.hpp>
-#include <boost/signals2/signal.hpp>
+#include <boost/signals2/connection.hpp>
 
 #ifdef _MSC_VER
 #pragma make_public(::bec::GRTManager)
@@ -91,10 +89,6 @@ namespace bec {
     std::string get_unique_tmp_subdir();
     void cleanup_tmp_dir();
 
-    void add_log_file_entry(const char *text, size_t size);
-    void add_log_file_text(const char *line);
-    void set_use_log_file(bool value);
-
     void set_module_extensions(const std::list<std::string> &extensions);
 
     void rescan_modules();
@@ -121,22 +115,14 @@ namespace bec {
     void push_status_text(const std::string &message);
     void replace_status_text(const std::string &message);
     void pop_status_text();
-    void set_status_slot(const boost::function<void (const std::string&)> &slot);
-    
-    void push_output_callback(const boost::function<bool (const std::string&)> &slot);
-    void pop_output_callback();
+    void set_status_slot(const boost::function<void (std::string)> &slot);
 
     void push_cancel_query_callback(const boost::function<bool ()> &slot);
     void pop_cancel_query_callback();
 
     void set_canceled();
-  
-    void set_message_callback(const boost::function<void (const grt::Message&)> &handler);
-
-    void set_progress_callback(const boost::function<bool (const std::string&,const std::string&,float)> &cb);
-
+  public:
     GRTDispatcher *get_dispatcher() const { return _dispatcher.get(); };
-    void dispatch_task(GRTTask *task);
 
     grt::GRT *get_grt() const { return _grt; };
 
@@ -155,8 +141,7 @@ namespace bec {
 
     void execute_grt_task(const std::string &title,
                           const boost::function<grt::ValueRef (grt::GRT*)> &function,
-                          const boost::function<void (grt::ValueRef)> &finished_cb,
-                          bool show_progress);
+                          const boost::function<void (grt::ValueRef)> &finished_cb);
 
 
     // message displaying (as dialogs)
@@ -172,8 +157,6 @@ namespace bec {
     MessageListStorage *get_messages_list();
 
     // these always return the same instance
-    StructsTreeBE *get_shared_structs_tree();
-    ModulesTreeBE *get_shared_modules_tree();
     ValueTreeBE *get_shared_value_tree(const std::string &valuespec);
 
     //
@@ -216,11 +199,15 @@ namespace bec {
     
     
   public:
+    boost::function<void (bec::ArgumentPool&)> update_plugin_arguments_pool; // set by WBContext
+    
     bec::MenuItemList get_plugin_context_menu_items(const std::list<std::string> &groups,
                                                     const bec::ArgumentPool &argument_pool);
     bool check_plugin_runnable(const app_PluginRef &plugin, const bec::ArgumentPool &argpool,
                                bool debug_output= false);
     
+    void open_object_editor(const GrtObjectRef &object, bec::GUIPluginFlags flags = bec::NoFlags);
+    
   protected:
     grt::GRT *_grt;
     bool _has_unsaved_changes;
@@ -243,28 +230,17 @@ namespace bec {
 
     ShellBE *_shell;
     ValueTreeBE *_value_tree;
-    StructsTreeBE *_structs_tree;
-    ModulesTreeBE *_modules_tree;
 
     MessageListStorage *_messages_list;
 
     boost::function<void (std::string)> _status_text_slot;
     
-    // some day this should be changed so that grt::Message is specific to modeling validation
-    // everything else should just use plain text output
-    boost::function<void (const grt::Message&)> _message_cb;
-    
-    // stack of text output handlers
-    GMutex *_tmp_output_cb_stack_mutex;
-    std::list<boost::function<bool (std::string)> > _tmp_output_cb_stack;
-
-    boost::function<bool (std::string,std::string, float)> _progress_cb;
-
     std::list<Timer*> _timers;
     std::set<Timer*> _cancelled_timers;
     boost::function<void ()> _timeout_request;
 
-    boost::signals2::signal<void ()> *_idle_signal;
+    boost::signals2::signal<void ()> _idle_signal;
+    int _current_idle_serial;
     bool _idle_blocked;
 
     std::list<std::string> _module_extensions;
@@ -298,23 +274,15 @@ namespace bec {
     
     void flush_shell_output();
 
-    FILE *_log_file;
-
   private:
     bool _terminated; // true if application termination was requested by the BE or a plugin.
     static std::map<grt::GRT*,GRTManager*> _instances;
     
     grt::ValueRef setup_grt(grt::GRT *grt);
 
-    void grt_default_msg_cb(const grt::Message &msg, void *sender= NULL);
-
     void shell_write(const std::string &text);
 
-    void task_started_cb(const std::string &title);
-    void task_finished_cb(const grt::ValueRef &result);
     void task_error_cb(const std::exception &error, const std::string &title);
-
-    void task_msg_cb(const grt::Message &msg, const std::string &title, bool show_progress);
   };
 
 };
