@@ -259,7 +259,7 @@ db_TableRef TableHelper::create_associative_table(const db_SchemaRef &schema,
   // when the 1st FK is created, the PK and FK will have the same columns, so the index will be reused
   // when the 2nd FK is created, the PK will have both columns so it can't be used by the 1st FK, so we have
   // to create a new one for it
-  db_IndexRef index = create_index_for_fk(schema->get_grt(), first_fk);
+  db_IndexRef index = create_index_for_fk(schema->get_grt(), first_fk, rdbms->maximumIdentifierLength());
   first_fk->index(index);
   atable->indices().insert(index);
   
@@ -268,11 +268,11 @@ db_TableRef TableHelper::create_associative_table(const db_SchemaRef &schema,
   return atable;
 }
 
-
-static std::string format_ident_with_table(const std::string &fmt, const db_TableRef &table)
-{
-  return replace_variable(fmt, "%table%", table->name().c_str());
-}
+//
+//static std::string format_ident_with_table(const std::string &fmt, const db_TableRef &table)
+//{
+//  return replace_variable(fmt, "%table%", table->name().c_str());
+//}
 
 static std::string format_ident_with_stable_dtable(const std::string &fmt, const db_TableRef &stable, const db_TableRef &dtable)
 {
@@ -286,10 +286,13 @@ static std::string format_ident_with_column(const std::string &fmt, const db_Col
 }
 
 
-db_IndexRef TableHelper::create_index_for_fk(grt::GRT *grt, const db_ForeignKeyRef &fk)
+db_IndexRef TableHelper::create_index_for_fk(grt::GRT *grt, const db_ForeignKeyRef &fk, const size_t max_len)
 {
   std::string index_name(fk->name().c_str());
-  //index_name.append("_index");
+  if (index_name.length() > (max_len-5))
+      index_name.resize(max_len-5);
+  index_name.append("_idx");
+  index_name = grt::get_name_suggestion_for_list_object(fk->owner()->indices(),index_name,false);
 
   // add the corresponding index
   db_IndexRef index= grt->create_object<db_Index>(db_TableRef::cast_from(fk->owner()).get_metaclass()->get_member_type("indices").content.object_class.c_str());
@@ -581,7 +584,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   // check if an index already exists for the FK columns and if not, create one
   if (!find_index_usable_by_fk(new_fk).is_valid())
   {
-    index = create_index_for_fk(grt, new_fk);
+    index = create_index_for_fk(grt, new_fk, rdbms->maximumIdentifierLength());
     new_fk->index(index);
     // index is inserted later down
   }
@@ -671,7 +674,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   // check if an index already exists for the FK columns and if not, create one
   if (!find_index_usable_by_fk(new_fk).is_valid())
   {
-    db_IndexRef index(create_index_for_fk(grt, new_fk));
+    db_IndexRef index(create_index_for_fk(grt, new_fk,rdbms->maximumIdentifierLength()));
     new_fk->index(index);
     
     // add the FK to the source table
@@ -778,18 +781,15 @@ gchar* find_comment_split(const std::string &comment,const size_t &db_comment_le
   return g_utf8_offset_to_pointer(comment.c_str(), res);
 };
 
-std::string TableHelper::get_sync_comment(const std::string &comment)
+std::string TableHelper::get_sync_comment(const std::string &comment, const int max_len)
 {
-  //TODO add comment size depending on server version
-  gchar* split_position = find_comment_split(comment, 60);
-  
+  gchar* split_position = find_comment_split(comment, max_len);
   return std::string(comment.c_str(), split_position - comment.c_str());
 };
 
-std::string TableHelper::get_document_comment(const std::string &comment)
+std::string TableHelper::get_document_comment(const std::string &comment, const int max_len)
 {
-  //TODO add comment size depending on server version
-  return std::string(find_comment_split(comment, 60));
+  return std::string(find_comment_split(comment, max_len));
 };
 
 void ColumnHelper::copy_column(const db_ColumnRef &from, db_ColumnRef &to)
@@ -851,47 +851,47 @@ void ColumnHelper::set_default_value(db_ColumnRef column, const std::string &val
 }
 
 //------------------------------------------------------------------------------------
-
-
-//XXX the following code must be moved to db.mysql module
-
-static enum_field_types get_mysql_type_from_string(const char *type_name)
-{
-  if(g_strcasecmp(type_name, "INT") == 0)
-    return MYSQL_TYPE_LONG;
-  if(g_strcasecmp(type_name, "TINYINT") == 0)
-    return MYSQL_TYPE_TINY;
-  if(g_strcasecmp(type_name, "SMALLINT") == 0)
-    return MYSQL_TYPE_SHORT;
-  if(g_strcasecmp(type_name, "BIGINT") == 0)
-    return MYSQL_TYPE_LONGLONG;
-  if(g_strcasecmp(type_name, "FLOAT") == 0)
-    return MYSQL_TYPE_FLOAT;
-  if(g_strcasecmp(type_name, "DOUBLE") == 0)
-    return MYSQL_TYPE_DOUBLE;
-  if(g_strcasecmp(type_name, "TIME") == 0)
-    return MYSQL_TYPE_TIME;
-  if(g_strcasecmp(type_name, "DATE") == 0)
-    return MYSQL_TYPE_DATE;
-  if(g_strcasecmp(type_name, "DATETIME") == 0)
-    return MYSQL_TYPE_DATETIME;
-  if(g_strcasecmp(type_name, "TIMESTAMP") == 0)
-    return MYSQL_TYPE_TIMESTAMP;
-  if(g_strcasecmp(type_name, "TEXT") == 0)
-    return MYSQL_TYPE_STRING;
-  if(g_strcasecmp(type_name, "CHAR") == 0)
-    return MYSQL_TYPE_STRING;
-  if(g_strcasecmp(type_name, "VARCHAR") == 0)
-    return MYSQL_TYPE_STRING;
-  if(g_strcasecmp(type_name, "BLOB") == 0)
-    return MYSQL_TYPE_BLOB;
-  if(g_strcasecmp(type_name, "BINARY") == 0)
-    return MYSQL_TYPE_BLOB;
-  if(g_strcasecmp(type_name, "VARBINARY") == 0)
-    return MYSQL_TYPE_BLOB;
-
-  return MYSQL_TYPE_NULL;
-}
+//
+//
+////XXX the following code must be moved to db.mysql module
+//
+//static enum_field_types get_mysql_type_from_string(const char *type_name)
+//{
+//  if(g_strcasecmp(type_name, "INT") == 0)
+//    return MYSQL_TYPE_LONG;
+//  if(g_strcasecmp(type_name, "TINYINT") == 0)
+//    return MYSQL_TYPE_TINY;
+//  if(g_strcasecmp(type_name, "SMALLINT") == 0)
+//    return MYSQL_TYPE_SHORT;
+//  if(g_strcasecmp(type_name, "BIGINT") == 0)
+//    return MYSQL_TYPE_LONGLONG;
+//  if(g_strcasecmp(type_name, "FLOAT") == 0)
+//    return MYSQL_TYPE_FLOAT;
+//  if(g_strcasecmp(type_name, "DOUBLE") == 0)
+//    return MYSQL_TYPE_DOUBLE;
+//  if(g_strcasecmp(type_name, "TIME") == 0)
+//    return MYSQL_TYPE_TIME;
+//  if(g_strcasecmp(type_name, "DATE") == 0)
+//    return MYSQL_TYPE_DATE;
+//  if(g_strcasecmp(type_name, "DATETIME") == 0)
+//    return MYSQL_TYPE_DATETIME;
+//  if(g_strcasecmp(type_name, "TIMESTAMP") == 0)
+//    return MYSQL_TYPE_TIMESTAMP;
+//  if(g_strcasecmp(type_name, "TEXT") == 0)
+//    return MYSQL_TYPE_STRING;
+//  if(g_strcasecmp(type_name, "CHAR") == 0)
+//    return MYSQL_TYPE_STRING;
+//  if(g_strcasecmp(type_name, "VARCHAR") == 0)
+//    return MYSQL_TYPE_STRING;
+//  if(g_strcasecmp(type_name, "BLOB") == 0)
+//    return MYSQL_TYPE_BLOB;
+//  if(g_strcasecmp(type_name, "BINARY") == 0)
+//    return MYSQL_TYPE_BLOB;
+//  if(g_strcasecmp(type_name, "VARBINARY") == 0)
+//    return MYSQL_TYPE_BLOB;
+//
+//  return MYSQL_TYPE_NULL;
+//}
 
 void CatalogHelper::apply_defaults(db_mysql_ColumnRef column)
 {
