@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,14 +24,12 @@
 #include "grt/editor_base.h"
 
 #include "grt/tree_model.h"
-#include "sqlide/sql_editor_be.h"
 #include "grtsqlparser/invalid_sql_parser.h"
 #include "grts/structs.db.h"
 #include "grts/structs.db.mgmt.h"
 
 #include <memory>
 
-
 namespace bec {
 
   class WBPUBLICBACKEND_PUBLIC_FUNC DBObjectEditorBE : public BaseEditor
@@ -59,6 +57,7 @@ namespace bec {
     virtual void delete_custom_data(const std::string &name);
 
     virtual Sql_editor::Ref get_sql_editor();
+    virtual bool is_editor_dirty();
 
   public: // helper functions
 
@@ -70,6 +69,7 @@ namespace bec {
     db_SchemaRef get_schema_with_name(const std::string &schema_name);
 
     virtual std::vector<std::string> get_all_table_names();
+    virtual std::vector<std::string> get_all_schema_names();
     virtual std::vector<std::string> get_schema_table_names();
     virtual std::vector<std::string> get_table_column_names(const std::string &table_name);
     virtual std::vector<std::string> get_table_column_names(const db_TableRef &table);
@@ -81,8 +81,6 @@ namespace bec {
 
     void update_change_date();
 
-    virtual ~DBObjectEditorBE();
-
   public:
     void sql_mode(const std::string &value);
 
@@ -134,7 +132,7 @@ namespace bec {
     virtual bool can_close();
 
   private:
-    boost::signals2::connection    _val_notify_conn;
+    boost::signals2::scoped_connection _val_notify_conn;
     void notify_from_validation(const grt::Validator::Tag& tag, const grt::ObjectRef&, const std::string&, const int level);//level is grt::MessageType
     // Real-time validation part
     grt::MessageType    _last_validation_check_status;
