@@ -241,7 +241,7 @@ void RoutineGroupEditorBE::set_routines_sql(const std::string &sql, bool sync)
 {
   if (get_sql() != sql)
   {
-    set_sql_parser_task_cb(sigc::mem_fun(this, &RoutineGroupEditorBE::parse_sql));
+    set_sql_parser_task_cb(boost::bind(&RoutineGroupEditorBE::parse_sql, this, _1, _2));
     set_sql(sql, sync, _group);
   }
 }
@@ -277,3 +277,9 @@ Sql_editor::Ref RoutineGroupEditorBE::get_sql_editor()
   }
   return sql_editor;
 }
+
+std::string RoutineGroupEditorBE::get_title()
+{
+  return base::strfmt("%s - Group", get_name().c_str()); 
+}
+
