@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,6 +28,11 @@
 #include "grt/validation_manager.h"
 
 #include "base/string_utilities.h"
+#include "base/file_utilities.h"
+
+#ifdef __APPLE__
+#include "base/log.h"
+#endif
 
 #include "mforms/utilities.h"
 
@@ -35,6 +40,8 @@ using namespace bec;
 using namespace grt;
 using namespace base;
 
+DEFAULT_LOG_DOMAIN("Table Editor backend")
+
 /*
  The big mess of user datatypes, synonyms and column flags.
  
@@ -408,7 +415,7 @@ bool TableColumnsListBE::set_column_type_from_string(db_ColumnRef &col, const st
   {
     if (col->simpleType().is_valid())
     {
-      // Remove flags that dont exist in new datatype.
+      // Remove flags that don't exist in new datatype.
       if (col->flags().count() > 0)
       {
         grt::StringListRef valid_flags(col->simpleType()->flags());
@@ -426,35 +433,12 @@ bool TableColumnsListBE::set_column_type_from_string(db_ColumnRef &col, const st
       
       // Read top of file for note on column flag handling
       
-      /* don't set flags from userType or we have trouble when userType is updated, that was
-       incorrect behaviour
-      // set flags list to the ones defined in userType
-      if (col->userType()->actualType().is_valid())
-      {
-        gchar **tokens= g_strsplit(col->userType()->flags().c_str(), " ", 0);
-        if (tokens)
-        {
-          grt::StringListRef flags(col->userType()->actualType()->flags());
-          
-          for (size_t j= 0; tokens[j] != NULL; j++)
-          {
-            for (size_t c= flags.count(), i= 0; i < c; i++)
-            {
-              if (g_strcasecmp(flags.get(i).c_str(), tokens[j]) == 0)
-              {
-                col->flags().insert(flags[i]);
-                break;
-              }
-            }
-          }
-          g_strfreev(tokens);
-        }
-      }
-       */
+      // Don't set flags from userType or we have trouble when userType is updated, that was
+      // incorrect behavior.
     }
   }
   else
-    g_message("%s is not a valid column type", type.c_str());
+    log_warning("%s is not a valid column type", type.c_str());
   return flag;
 }
 
@@ -560,7 +544,7 @@ bool TableColumnsListBE::set_field(const NodeId &node, int column, const std::st
 
     if ((ColumnListColumns)column == Name)
     {
-      col_name= rtrim(value);
+      col_name= base::trim_right(value);
       _owner->add_column(col_name);
 
       db_ColumnRef col(_owner->get_table()->columns()[node[0]]);
@@ -607,7 +591,7 @@ bool TableColumnsListBE::set_field(const NodeId &node, int column, const std::st
   {
   case Name:
     {
-      std::string value_= rtrim(value);
+      std::string value_= base::trim_right(value);
       if (old != value_)
         _owner->rename_column(col, value_);
     }
@@ -646,7 +630,7 @@ bool TableColumnsListBE::set_field(const NodeId &node, int column, const std::st
     {
       AutoUndoEdit undo(_owner, col, "defaultValue");
 
-      std::string value_= rtrim(value);
+      std::string value_= base::trim_right(value);
       bec::ColumnHelper::set_default_value(col, value_.empty() ? "" : quote_value_if_needed(col, value_));
       _owner->update_change_date();
 
@@ -760,6 +744,7 @@ bool TableColumnsListBE::set_field(const NodeId &node, int column, int value)
     return true;
   case IsNotNull:
     {
+      FreezeRefresh frz(_owner);
       AutoUndoEdit undo(_owner);
       col->isNotNull(value != 0);
 
@@ -775,13 +760,25 @@ bool TableColumnsListBE::set_field(const NodeId &node, int column, int value)
     }
     return true;
   case IsBinary:
-    return set_column_flag(node, "BINARY", value!=0);
+    {
+      FreezeRefresh frz(_owner);
+      return set_column_flag(node, "BINARY", value!=0);
+    }
   case IsUnsigned:
-    return set_column_flag(node, "UNSIGNED", value!=0);
+    {
+      FreezeRefresh frz(_owner);
+      return set_column_flag(node, "UNSIGNED", value!=0);
+    }
   case IsZerofill:
-    return set_column_flag(node, "ZEROFILL", value!=0);
+    {
+      FreezeRefresh frz(_owner);
+      return set_column_flag(node, "ZEROFILL", value!=0);
+    }
   case IsUnique:
-    return make_unique_index(col, value!=0);
+    {
+      FreezeRefresh frz(_owner);
+      return make_unique_index(col, value!=0);
+    }
   case Flags:
   case Default:
     return false;
@@ -887,7 +884,10 @@ bool TableColumnsListBE::get_field_grt(const NodeId &node, int column, grt::Valu
 
     }
     else
-      value= grt::StringRef("");
+    {
+      value = grt::StringRef("");
+      return false;
+    }
     return true;
   }
   return false;
@@ -1505,7 +1505,7 @@ bool IndexListBE::index_editable(const db_IndexRef &index)
 }
 
 
-bool IndexListBE::index_belongs_to_fk(const db_IndexRef &index)
+db_ForeignKeyRef IndexListBE::index_belongs_to_fk(const db_IndexRef &index)
 {
   // check if the index belongs to a FK, if so it cannot be edited
   if (index.is_valid())
@@ -1513,21 +1513,24 @@ bool IndexListBE::index_belongs_to_fk(const db_IndexRef &index)
     ListRef<db_ForeignKey> fks(db_TableRef::cast_from(index->owner())->foreignKeys());
     for (size_t c= fks.count(), i= 0; i < c; i++)
       if (fks[i]->index() == index)
-        return true;
+        return fks[i];
   }
-  return false;
+  return db_ForeignKeyRef();
 }
 
 
 MenuItemList IndexListBE::get_popup_items_for_nodes(const std::vector<NodeId> &nodes)
 {
-  db_IndexRef index(get_selected_index());
+  db_IndexRef index;
+
+  if (!nodes.empty() && nodes[0][0] < (int)_owner->get_table()->indices().count())
+    index = _owner->get_table()->indices()[nodes[0][0]];
   
   MenuItemList items;
   MenuItem item;
   item.caption = _("Delete Selected Indices");
   item.name    = "deleteIndices";
-  item.enabled = nodes.size() > 0 && !index_belongs_to_fk(index) && index_editable(index);
+  item.enabled = index.is_valid() && nodes.size() > 0 && /*!index_belongs_to_fk(index) && */index_editable(index);
   items.push_back(item);
 
   return items;
@@ -1541,7 +1544,21 @@ bool IndexListBE::activate_popup_item_for_nodes(const std::string &name, const s
   if (name == "deleteIndices")
   {
     for (std::vector<NodeId>::reverse_iterator node= nodes.rbegin(); node != nodes.rend(); ++node)
+    {
+      if ((*node)[0] < (int)_owner->get_table()->indices().count())
+      {
+        db_IndexRef index(_owner->get_table()->indices().get((*node)[0]));
+        db_ForeignKeyRef fk;
+        if (index.is_valid() && (fk = index_belongs_to_fk(index)).is_valid())
+        {
+          mforms::Utilities::show_message("Cannot Delete Index",
+                                          base::strfmt("The index '%s' belongs to the Foreign Key '%s'.\nYou must delete the Foreign Key to delete the index.", index->name().c_str(), fk->name().c_str()),
+                                          "OK", "", "");
+          continue;
+        }
+      }
       _owner->remove_index(*node);
+    }
   }
   else
     return false;
@@ -1574,7 +1591,7 @@ NodeId IndexListBE::add_column(const db_ColumnRef &column, const db_IndexRef &aI
   if (!index.is_valid())
     return NodeId();
 
-  if (!index_editable(index) || index_belongs_to_fk(index))
+  if (!index_editable(index) || index_belongs_to_fk(index).is_valid())
     return NodeId();
 
   std::string column_struct= index.get_metaclass()->get_member_type("columns").content.object_class;
@@ -1602,7 +1619,7 @@ void IndexListBE::remove_column(const NodeId &node)
   RefreshUI::Blocker __centry(*_owner);
 
   db_IndexRef index= get_selected_index();
-  if (!index_editable(index) || index_belongs_to_fk(index))
+  if (!index_editable(index) || index_belongs_to_fk(index).is_valid())
     return;
 
   db_ColumnRef column(_owner->get_table()->columns().get(node[0]));
@@ -2447,13 +2464,23 @@ bool FKConstraintListBE::get_field_grt(const NodeId &node, int column, grt::Valu
     return true;
   case OnDelete:
     if (fk.is_valid())
-      value= fk->deleteRule();
+    {
+      if (fk->deleteRule().empty())
+        value = grt::StringRef("RESTRICT"); // empty means default means RESTRICT
+      else
+        value= fk->deleteRule();
+    }
     else
       value= grt::StringRef("");
     return true;
   case OnUpdate:
     if (fk.is_valid())
-      value= fk->updateRule();
+    {
+      if (fk->updateRule().empty())
+        value = grt::StringRef("RESTRICT"); // empty means default means RESTRICT
+      else
+        value= fk->updateRule();
+    }
     else
       value= grt::StringRef("");
     return true;
@@ -2570,7 +2597,7 @@ void TableEditorBE::set_name(const std::string &name)
 
     AutoUndoEdit undo(this, get_object(), "name");
     bec::ValidationManager::validate_instance(_table, CHECK_NAME);
-    std::string name_= rtrim(name);
+    std::string name_= base::trim_right(name);
     //DBObjectEditorBE::set_name(name_);
     get_dbobject()->name(name_);
     update_change_date();
@@ -2646,6 +2673,12 @@ void TableEditorBE::remove_column(const NodeId &node)
 
 NodeId TableEditorBE::add_fk(const std::string &name)
 {
+
+   if(!get_table()->columns().count())
+   {
+      mforms::Utilities::show_warning("FK Creation", "Cannot add FK on empty table, add some columns first", "OK");
+      return NodeId();
+   }
   ListRef<db_ForeignKey> fklist= get_table()->foreignKeys();
   db_ForeignKeyRef fk;
 
@@ -2700,6 +2733,12 @@ bool TableEditorBE::remove_fk(const NodeId &fk)
 
 NodeId TableEditorBE::add_index(const std::string &name)
 {
+   if(!get_table()->columns().count())
+   {
+      mforms::Utilities::show_warning("Index Creation", "Cannot add Index on empty table, add some columns first", "OK");
+      return NodeId();
+   }
+
   ListRef<db_Index> indices= get_table()->indices();
   db_IndexRef index;
 
@@ -2739,7 +2778,7 @@ bool TableEditorBE::remove_index(const NodeId &index)
 
   // do not allow removal of PRIMARY or FOREIGN indices by user
   if (!get_indexes()->index_editable(indexobj) ||
-      get_indexes()->index_belongs_to_fk(indexobj))
+      get_indexes()->index_belongs_to_fk(indexobj).is_valid())
     return false;
 
   AutoUndoEdit undo(this);
@@ -2846,12 +2885,12 @@ void TableEditorBE::set_triggers_sql(const std::string &sql, bool sync)
 
 grt::ValueRef TableEditorBE::parse_triggers_sql(grt::GRT* grt, grt::StringRef sql)
 {
-  //grt::AutoUndo undo(grt);
   AutoUndoEdit undo(this);
 
   if (!_sql_parser.get())
     throw std::logic_error("SQL parser is not initialzed");
 
+  _table->customData().set("NonTriggerSQLFound", grt::IntegerRef(0));
   grt::IntegerRef res= _sql_parser->parse_triggers(_table, sql.c_str());
 
   undo.end(strfmt(_("Edit triggers of table `%s`.`%s`"), get_schema_name().c_str(), get_name().c_str()));
@@ -2874,9 +2913,9 @@ Recordset::Ref TableEditorBE::get_inserts_model()
     _inserts_storage->table(_table);
 
     _inserts_model= Recordset::create(_grtm);
-    _inserts_model->empty_info_message_on_reset(true);
+    _inserts_model->show_apply_buttons(true);
     _inserts_model->data_storage(_inserts_storage);
-    _inserts_model->export_wizard = boost::bind(&TableEditorBE::show_export_wizard, this);
+    _inserts_model->export_wizard = boost::bind(&TableEditorBE::show_export_wizard, this, (mforms::Form*)0);
     _inserts_model->refresh();
   }
   return _inserts_model;
@@ -2884,10 +2923,32 @@ Recordset::Ref TableEditorBE::get_inserts_model()
 
 //--------------------------------------------------------------------------------------------------
 
-void TableEditorBE::show_export_wizard()
+void TableEditorBE::show_export_wizard(mforms::Form *owner)
 {
-  InsertsExportForm exporter(_inserts_model);
-  exporter.run();  
+  grt::ValueRef option(_grtm->get_app_option("TableEditor:LastExportDirectory"));
+  std::string path = option.is_valid() ? grt::StringRef::cast_from(option) : "";
+  option = _grtm->get_app_option("TableEditor:LastExportExtension");
+  std::string extension = option.is_valid() ? grt::StringRef::cast_from(option) : "";
+  InsertsExportForm exporter(owner, _inserts_model, extension);
+  exporter.set_title(strfmt(_("Export Inserts for %s"), get_name().c_str()));
+  if (!path.empty())
+  {
+    path = bec::make_path(path, get_name());
+    exporter.set_path(path);
+  }  
+  path = exporter.run();
+  if (path.empty())
+    _grtm->replace_status_text(_("Export inserts canceled"));
+  else
+  {
+    _grtm->replace_status_text(strfmt(_("Exported inserts to %s"), path.c_str()));
+    _grtm->set_app_option("TableEditor:LastExportDirectory", grt::StringRef(exporter.get_directory()));
+    extension = base::extension(path);
+    if (!extension.empty() && extension[0] == '.')
+      extension = extension.substr(1);
+    if (!extension.empty())
+      _grtm->set_app_option("TableEditor:LastExportExtension", grt::StringRef(extension));
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -2906,3 +2967,9 @@ Sql_editor::Ref TableEditorBE::get_sql_editor()
   }
   return sql_editor;
 }
+
+std::string TableEditorBE::get_title()
+{
+  return base::strfmt("%s - Table", get_name().c_str()); 
+}
+
