@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,7 +30,7 @@
 
 #include "mforms/scrollpanel.h"
 #include "mforms/imagebox.h"
-#include "mforms/treeview.h"
+#include "mforms/treenodeview.h"
 #include "mforms/code_editor.h"
 #include "mforms/filechooser.h"
 
@@ -73,7 +73,7 @@ class HexDataViewer : public BinaryDataViewer
 {
 public:
   HexDataViewer(BinaryDataEditor *owner, bool read_only)
-  : BinaryDataViewer(owner), _tree((mforms::TreeOptions)(mforms::TreeShowColumnLines|mforms::TreeShowRowLines)), _box(true)
+  : BinaryDataViewer(owner), _tree(mforms::TreeShowColumnLines|mforms::TreeShowRowLines|mforms::TreeFlatList), _box(true)
   {
     _offset = 0;
     _block_size = 8 * 1024;
@@ -144,17 +144,17 @@ public:
     suspend_layout();
 
     unsigned char *ptr = (unsigned char*)_owner->data() + _offset;
-    _tree.clear_rows();
+    _tree.clear();
     size_t offs;
     size_t length = std::min<int>(_offset + _block_size, _owner->length());
     for (offs = _offset; offs < length; offs += 16)
     {
-      int row = _tree.add_row();
-      _tree.set(row, 0, base::strfmt("0x%08x", (unsigned int) offs));
+      mforms::TreeNodeRef row = _tree.add_node();
+      row->set_string(0, base::strfmt("0x%08x", (unsigned int) offs));
 
       for (int i= offs, min= std::min<int>(offs+16, length); i < min; ++i)
       {
-        _tree.set(row, i+1-offs, base::strfmt("%02x", *ptr));
+        row->set_string(i+1-offs, base::strfmt("%02x", *ptr));
         ptr++;
       }
     }
@@ -185,7 +185,7 @@ public:
   }
   
 private:
-  mforms::TreeView _tree;
+  mforms::TreeNodeView _tree;
   mforms::Box _box;
   mforms::Button _first;
   mforms::Button _back;
@@ -195,9 +195,9 @@ private:
   size_t _offset;
   size_t _block_size;
 
-  void set_cell_value(int row, int column, const std::string &value)
+  void set_cell_value(mforms::TreeNodeRef node, int column, const std::string &value)
   {
-    size_t offset = _offset + row * 16 + (column-1);
+    size_t offset = _offset + _tree.row_for_node(node) * 16 + (column-1);
     
     if (offset < _owner->length())
     {
@@ -206,7 +206,7 @@ private:
         return;
       if (i < 0 || i > 255)
         return;
-      _tree.set(row, column, base::strfmt("%02x", i));
+      node->set_string(column, base::strfmt("%02x", i));
       
       *(unsigned char*)(_owner->data() + offset) = i;
       _owner->notify_edit();
@@ -228,9 +228,9 @@ public:
     add(&_message, false, true);
     add(&_text, true, true);
     
-    _text.set_read_only(read_only);
     _text.set_language(mforms::LanguageNone);
     _text.set_features(mforms::FeatureWrapText, true);
+    _text.set_features(mforms::FeatureReadOnly, read_only);
     
     scoped_connect(_text.signal_changed(),boost::bind(&TextDataViewer::edited, this));
   }
@@ -254,22 +254,25 @@ public:
       if (_owner->length())
       {
         _message.set_text(message);
-        _text.set_read_only(true);
+        _text.set_features(mforms::FeatureReadOnly, true);
       }
       else
       {
-        _text.set_read_only(false);
+        _text.set_features(mforms::FeatureReadOnly, false);
       }
       _text.set_value("");
     }
     else
     {
       _message.set_text("");
-      _text.set_read_only(false);
+      _text.set_features(mforms::FeatureReadOnly, false);
       _text.set_value(std::string(converted, bwritten));
       if (_owner == NULL || _owner->read_only())
-        _text.set_read_only(true);
+        _text.set_features(mforms::FeatureReadOnly, true);
     }
+
+    if (converted != NULL)
+      g_free(converted);
   }
   
 private:
