@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,7 +23,7 @@
 
 #include "base/geometry.h"
 
-using namespace MySQL::Geometry;
+using namespace base;
 
 #include "grtdb/db_helpers.h"
 #include "db_conn_be.h"
@@ -41,6 +41,7 @@ using namespace MySQL::Geometry;
 
 #undef max
 
+DEFAULT_LOG_DOMAIN("DbConnectPanel");
 
 static const std::string control_name_prefix= "ctrl__";
 
@@ -73,6 +74,8 @@ DbDriverParam::ParamType DbDriverParam::decode_param_type(std::string type_name)
     result= ptPassword;
   else if (0 == type_name.compare("keychain"))
     result= ptKeychainPassword;
+  else if (0 == type_name.compare("enum"))
+    result= ptEnum;
   else
     g_warning("Unkown DB driver parameter type '%s'", type_name.c_str());
 
@@ -88,7 +91,7 @@ _type(ptUnknown)
 
   if (_type == ptKeychainPassword)
     set_value(driver_param->name());
-  else if (stored_conn.is_valid())
+  else if (stored_conn.is_valid() && !(*stored_conn->name()).empty())
     set_value(stored_conn->parameterValues().get(driver_param->name(), driver_param->defaultValue()));
   else
     set_value(driver_param->defaultValue());
@@ -108,9 +111,11 @@ ControlType DbDriverParam::get_control_type() const
       return ctFileSelector;
     case DbDriverParam::ptKeychainPassword:
       return ctKeychainPassword;
+    case DbDriverParam::ptEnum:
+      return ctEnumSelector;
     case DbDriverParam::ptInt:
     case DbDriverParam::ptString:
-    case DbDriverParam::ptPassword:
+    case DbDriverParam::ptPassword:      
     default:
       return ctTextBox;
   }
@@ -151,6 +156,12 @@ void DbDriverParam::set_value(const grt::ValueRef &value)
 
       break;
     }
+      
+  case ptEnum:
+    {
+      _value= grt::StringRef::cast_from(value);
+      break;
+    }
 
   case ptUnknown:
   default:
@@ -161,6 +172,39 @@ void DbDriverParam::set_value(const grt::ValueRef &value)
 }
 
 
+std::vector<std::pair<std::string, std::string> > DbDriverParam::get_enum_options()
+{
+  std::vector<std::pair<std::string, std::string> > options;
+  grt::Module *module = _inner.get_grt()->get_module(*_inner->lookupValueModule());
+  if (module)
+  {
+    grt::BaseListRef args(_inner.get_grt());
+    grt::ValueRef result = module->call_function(*_inner->lookupValueMethod(), args);
+    if (result.is_valid() && grt::StringListRef::can_wrap(result))
+    {
+      grt::StringListRef list = grt::StringListRef::cast_from(result);
+      for (int i= 0; i < (int)list.count(); i++)
+      {
+        std::string s = list[i];
+        std::string::size_type pos;
+        if ((pos = s.find('|')) != std::string::npos)
+          options.push_back(std::make_pair(s.substr(0, pos), s.substr(pos+1)));
+        else
+          options.push_back(std::make_pair(s, s));
+      }
+    }
+    else
+      log_warning("Error calling enum value lookup method %s.%s for DriverParameter %s",
+                  _inner->lookupValueModule().c_str(), _inner->lookupValueMethod().c_str(),
+                  _inner->name().c_str());
+  }
+  else
+    log_warning("Error searching module for enum value lookup method %s.%s for DriverParameter %s",
+                _inner->lookupValueModule().c_str(), _inner->lookupValueMethod().c_str(),
+                _inner->name().c_str());  
+  return options;
+}
+
 //----------------------------------------------------------------------
 
 
@@ -236,25 +280,28 @@ void DbDriverParams::init(
   const db_mgmt_DriverRef &driver,
   const db_mgmt_ConnectionRef &stored_conn,
   const boost::function<void (bool)> &suspend_layout,
-  const boost::function<void ()> &clear_param_controls,
+  const boost::function<void ()> &begin_layout,
   const boost::function<void (DbDriverParam*, ControlType, const ControlBounds&, const std::string &)> &create_control,
+  const boost::function<void ()> &end_layout,
   bool skip_schema,
   int first_row_label_width,
   int hmargin,
   int vmargin)
 {
-  suspend_layout(true);
+  if (begin_layout)
+    begin_layout();
 
-  clear_param_controls();
   free_dyn_mem();
 
+  if (suspend_layout)
+    suspend_layout(true);
+
   _driver= driver;
   grt::ListRef<db_mgmt_DriverParameter> params= driver->parameters();
   size_t param_count= params.count();
 
   _collection.resize(param_count);
   _control_name_index.clear();
-
   DbDriverParam *param_handle;
   bool trim_schema = false;
   // create param handles
@@ -266,21 +313,17 @@ void DbDriverParams::init(
       trim_schema = true;
       continue;
     }
-
     param_handle= new DbDriverParam(param, stored_conn);
     _collection[_control_name_index.size()]= param_handle;
     _control_name_index[control_name_prefix + *param->name()]= param_handle;
   }
-  
   if (trim_schema)
     _collection.resize(param_count-1);
-
   for (int layout_type= 0; layout_type < 2; ++layout_type) // separate cycle for controls tagged as advanced layout
   {
     LayoutRows rows;
     LayoutRow row(0, hmargin);
     int y_offset= vmargin;
-
     for (Collection::iterator i= _collection.begin(); i != _collection.end(); ++i)
     {
       param_handle= *i;
@@ -347,7 +390,8 @@ void DbDriverParams::init(
       for (LayoutRow::LayoutControls::iterator c= controls->begin(); c != controls->end(); ++c)
       {
         c->bounds.top = row_index;
-        create_control(c->param_handle, c->type, c->bounds, c->caption);
+        if(create_control)
+            create_control(c->param_handle, c->type, c->bounds, c->caption);
       }
 
       // row description control
@@ -355,13 +399,17 @@ void DbDriverParams::init(
         LayoutControl ctrl= r->desc_control();
         ctrl.bounds.top = row_index;
 
-        create_control(ctrl.param_handle, ctrl.type, ctrl.bounds, ctrl.caption);
+        if(create_control)
+            create_control(ctrl.param_handle, ctrl.type, ctrl.bounds, ctrl.caption);
       }
       row_index++;
     }
   }
-
-  suspend_layout(false);
+  if (suspend_layout)
+    suspend_layout(false);
+  
+  if (end_layout)
+    end_layout();
 }
 
 
@@ -374,14 +422,18 @@ void DbDriverParams::free_dyn_mem()
 
 grt::DictRef DbDriverParams::get_params() const
 {
-  grt::DictRef params(_driver.get_grt());
-  for (Collection::const_iterator i= _collection.begin(); i != _collection.end(); ++i)
+  if (_driver.is_valid())
   {
-    DbDriverParam *param_handle= *i;
-    if (param_handle->get_value().is_valid())
-      params.set(param_handle->object()->name(), param_handle->get_value());
+    grt::DictRef params(_driver.get_grt());
+    for (Collection::const_iterator i= _collection.begin(); i != _collection.end(); ++i)
+    {
+      DbDriverParam *param_handle= *i;
+      if (param_handle->get_value().is_valid())
+        params.set(param_handle->object()->name(), param_handle->get_value());
+    }
+    return params;
   }
-  return params;
+  return grt::DictRef();
 }
 
 
@@ -420,54 +472,57 @@ std::string DbDriverParams::validate() const
 
 static const char *DEFAULT_RDBMS_ID= "com.mysql.rdbms.mysql";
 
-
-void DbConnection::init(const db_mgmt_ManagementRef &mgmt, bool skip_schema)
+DbConnection::DbConnection(const db_mgmt_ManagementRef &mgmt, const db_mgmt_DriverRef &driver, bool skip_schema)
+: _mgmt(mgmt), _active_driver(driver), _skip_schema(skip_schema) 
 {
-  _mgmt= mgmt;
-  _skip_schema = skip_schema;
 }
 
+
 void DbConnection::set_control_callbacks(
   const boost::function<void (bool)> &suspend_layout,
-  const boost::function<void ()> &clear_param_controls,
-  const boost::function<void (DbDriverParam*, ControlType, const ControlBounds&, const std::string &)> &create_control)
+  const boost::function<void ()> &begin_layout,
+  const boost::function<void (DbDriverParam*, ControlType, const ControlBounds&, const std::string &)> &create_control,
+  const boost::function<void ()> &end_layout)
 {
   _suspend_layout = suspend_layout;
-  _clear_param_controls= clear_param_controls;
+  _begin_layout = begin_layout;
+  _end_layout = end_layout;
   _create_control= create_control;
 }
 
 
 db_mgmt_ConnectionRef DbConnection::get_connection()
 {
+  save_changes();
   return _connection;
 }
 
 
 void DbConnection::save_changes()
 {
-  _connection->driver(get_active_driver());
+  _connection->driver(_active_driver);
+  
   grt::replace_contents(_connection->parameterValues(), _db_driver_param_handles.get_params());
   _connection->hostIdentifier(bec::get_host_identifier_for_connection(_connection));
 }
 
 DbConnection::~DbConnection()
 {
-};
+}
 
-void DbConnection::set_connection(const db_mgmt_ConnectionRef &connection)
+void DbConnection::set_connection_and_update(const db_mgmt_ConnectionRef &connection)
 {
   if (_connection != connection)
   {
     _connection= connection;
-    if (_connection.is_valid() && !_connection->driver().is_valid())
-      _connection->driver(get_active_driver());
+    _active_driver = connection->driver();
 
-    _db_driver_param_handles.init(connection->driver(),
+    _db_driver_param_handles.init(_active_driver,
                                   _connection,
                                   _suspend_layout,
-                                  _clear_param_controls,
+                                  _begin_layout,
                                   _create_control,
+                                  _end_layout,
                                   _skip_schema);
   }
 }
@@ -478,31 +533,30 @@ void DbConnection::set_connection_keeping_parameters(const db_mgmt_ConnectionRef
   if (_connection != connection)
   {
     _connection= connection;
-    _connection->driver(get_active_driver());
-    grt::replace_contents(_connection->parameterValues(), _db_driver_param_handles.get_params());
+    _connection->driver(_active_driver);
+    grt::DictRef curparams(_db_driver_param_handles.get_params());
+    if (curparams.is_valid())
+      grt::merge_contents(_connection->parameterValues(), curparams, true);
   }
 }
 
 
-void DbConnection::set_active_driver(int driver_index)
+void DbConnection::set_driver_and_update(db_mgmt_DriverRef driver)
 {
-  //! if (driver_index != _active_db_driver_index)
-  {    
-    _active_db_driver_index= driver_index;
-    
-    if (_connection.is_valid())
-      _connection->driver(get_active_driver());
-
-    _db_driver_param_handles.init(get_active_driver(),
-                                  _connection,
-                                  _suspend_layout,
-                                  _clear_param_controls,
-                                  _create_control,
-                                  _skip_schema);
-    
-    if (_connection.is_valid())
-      save_changes();
-  }
+  _active_driver = driver;
+  if (_connection.is_valid())
+    _connection->driver(driver);
+  
+  _db_driver_param_handles.init(_active_driver,
+                                _connection,
+                                _suspend_layout,
+                                _begin_layout,
+                                _create_control,
+                                _end_layout,
+                                _skip_schema);
+  
+  if (_connection.is_valid())
+    save_changes();
 }
 
 
@@ -533,16 +587,18 @@ void DbConnection::init_dbc_connection(sql::Connection* dbc_conn, const db_mgmt_
 
 sql::ConnectionWrapper DbConnection::get_dbc_connection()
 {
+  save_changes();
+  
   sql::ConnectionWrapper dbc_conn= sql::DriverManager::getDriverManager()->getConnection(
     get_connection(),
     boost::bind(&DbConnection::init_dbc_connection, this, _1, _2));
 
   if (dbc_conn.get() != NULL)
   {
-    if (_rdbms.is_valid() && (_rdbms.id() == DEFAULT_RDBMS_ID))
+    if (_active_driver.is_valid() && _active_driver->owner().is_valid() && (_active_driver->owner().id() == DEFAULT_RDBMS_ID))
     {
       // set SQL_MODE variable to be consistent with SQL stored in the model
-      bec::GRTManager *grtm= bec::GRTManager::get_instance_for(_rdbms->get_grt());
+      bec::GRTManager *grtm= bec::GRTManager::get_instance_for(_active_driver->get_grt());
       grt::ValueRef sql_mode_value= grtm->get_app_option("SqlMode");
       if (sql_mode_value.is_valid() && grt::StringRef::can_wrap(sql_mode_value))
       {
@@ -556,44 +612,10 @@ sql::ConnectionWrapper DbConnection::get_dbc_connection()
   return dbc_conn;
 }
 
+
 std::string DbConnection::validate_driver_params() const
 {
   return _db_driver_param_handles.validate();
 }
 
 
-void DbConnection::set_active_rdbms(int index)
-{
-  _rdbms= _mgmt->rdbms().get(index);
-}
-
-
-grt::ListRef<db_mgmt_Driver> DbConnection::get_driver_list()
-{
-  return _rdbms->drivers();
-}
-
-
-int DbConnection::get_default_rdbms_index() const
-{
-  size_t index= find_object_index_in_list(_mgmt->rdbms(), DEFAULT_RDBMS_ID);
-  return (int)index;
-}
-
-
-int DbConnection::get_rdbms_default_driver_index() const
-{
-  size_t result= -1;
-  if (_rdbms.is_valid())
-  {
-    db_mgmt_DriverRef defaultDriver= _rdbms->defaultDriver();
-    result= find_object_index_in_list(_rdbms->drivers(), defaultDriver.id());
-  }
-  return (int)result;
-}
-
-
-grt::ListRef<db_mgmt_Rdbms> DbConnection::get_rdbms_list()
-{
-  return _mgmt->rdbms();
-}
