@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,10 +31,15 @@
 #include "mforms/utilities.h"
 #include "mforms/checkbox.h"
 
-using namespace MySQL::Geometry;
+#define MYSQL_RDBMS_ID "com.mysql.rdbms.mysql"
+
+DEFAULT_LOG_DOMAIN("DbConnectPanel");
+
+using namespace base;
 using namespace grtui;
 using namespace mforms;
 
+/* Todo: Remove if not used.
 static bool is_ssh_driver(const std::string &driver_name)
 {
   char *name = g_strdown(g_strdup(driver_name.c_str()));
@@ -46,25 +51,36 @@ static bool is_ssh_driver(const std::string &driver_name)
   g_free(name);
   return false;
 }
+*/
 
-
-DbConnectPanel::DbConnectPanel(bool show_connection_combo)
-: Box(false), _connection(0), _show_connection_combo(show_connection_combo), _tab(mforms::TabViewSystemStandard)
+DbConnectPanel::DbConnectPanel(DbConnectPanelFlags flags)
+: Box(false), _connection(0), 
+_tab(mforms::TabViewSystemStandard),
+_params_panel(mforms::TransparentPanel), _params_table(0), 
+_advanced_panel(mforms::TransparentPanel), _advanced_table(0),
+_show_connection_combo((flags & DbConnectPanelShowConnectionCombo) != 0), 
+_show_manage_connections((flags & DbConnectPanelShowManageConnections) != 0)
 {
-  _rdbms_index= -1;
-  _driver_index= -1;
+  _allow_edit_connections = false;
   _initialized= false;
   _updating= false;
   
+  
   _skip_schema_name= false;
   _delete_connection_be= false;
   
   set_spacing(4);
 
-  if (show_connection_combo)
+  if (_show_connection_combo)
+  {
+    _allow_edit_connections = false;
     _label1.set_text(_("Stored Connection:"));
+  }
   else
+  {
+    _allow_edit_connections = true;
     _label1.set_text(_("Connection Name:"));
+  }
   _label2.set_text(_("Database System:"));
   _label3.set_text(_("Connection Method:"));
   
@@ -72,7 +88,7 @@ DbConnectPanel::DbConnectPanel(bool show_connection_combo)
   _label2.set_text_align(MiddleRight);
   _label3.set_text_align(MiddleRight);
   
-  if (show_connection_combo)
+  if (_show_connection_combo)
     _desc1.set_text(_("Select from saved connection settings"));
   else
     _desc1.set_text(_("Type a name for the connection"));
@@ -82,48 +98,54 @@ DbConnectPanel::DbConnectPanel(bool show_connection_combo)
   _desc3.set_text(_("Method to use to connect to the RDBMS"));
   _desc3.set_style(mforms::SmallHelpTextStyle);
 
-  if (show_connection_combo)
+  if (_show_connection_combo)
     scoped_connect(_stored_connection_sel.signal_changed(),boost::bind(&DbConnectPanel::change_active_stored_conn, this));
   scoped_connect(_rdbms_sel.signal_changed(),boost::bind(&DbConnectPanel::change_active_rdbms, this));
   scoped_connect(_driver_sel.signal_changed(),boost::bind(&DbConnectPanel::change_active_driver, this));
-  
-  _table.set_row_count(2);
+
+  _table.set_name("connect_panel:table");
+  _table.set_row_count(flags & DbConnectPanelShowRDBMSCombo ? 4 : 2);
   _table.set_column_count(3);
   
   _table.set_column_spacing(4);
   _table.set_row_spacing(4);
 
-  if (show_connection_combo)
+  int row = 0;
+  if (flags & DbConnectPanelShowRDBMSCombo)
   {
-    _table.add(&_label1, 0, 1, 0, 1, mforms::HFillFlag);
-    _table.add(&_stored_connection_sel, 1, 2, 0, 1, mforms::HExpandFlag | mforms::HFillFlag);
-    _table.add(&_desc1, 2, 3, 0, 1, mforms::HFillFlag);
+    _table.add(&_label2, 0, 1, row, row+1, mforms::HFillFlag);
+    _table.add(&_rdbms_sel, 1, 2, row, row+1, mforms::HExpandFlag | mforms::HFillFlag);
+    _table.add(&_desc2, 2, 3, row, row+1, mforms::HFillFlag);
+    row++;
+    _table.add(mforms::manage(new mforms::Label()), 0, 1, row, row+1, mforms::HFillFlag);
+    row++;
+  }
+  
+  if (_show_connection_combo)
+  {
+    _table.add(&_label1, 0, 1, row, row+1, mforms::HFillFlag);
+    _table.add(&_stored_connection_sel, 1, 2, row, row+1, mforms::HExpandFlag | mforms::HFillFlag);
+    _table.add(&_desc1, 2, 3, row, row+1, mforms::HFillFlag);
   }
   else
   {
-    _table.add(&_label1, 0, 1, 0, 1, mforms::HFillFlag);
-    _table.add(&_name_entry, 1, 2, 0, 1, mforms::HExpandFlag | mforms::HFillFlag);
-    _table.add(&_desc1, 2, 3, 0, 1, mforms::HFillFlag);    
+    _table.add(&_label1, 0, 1, row, row+1, mforms::HFillFlag);
+    _table.add(&_name_entry, 1, 2, row, row+1, mforms::HExpandFlag | mforms::HFillFlag);
+    _table.add(&_desc1, 2, 3, row, row+1, mforms::HFillFlag);    
   }
+  row++;
   
-  _table.add(&_label3, 0, 1, 1, 2, mforms::HFillFlag);
-  _table.add(&_driver_sel, 1, 2, 1, 2, mforms::HExpandFlag | mforms::HFillFlag);
-  _table.add(&_desc3, 2, 3, 1, 2, mforms::HFillFlag);
+  _table.add(&_label3, 0, 1, row, row+1, mforms::HFillFlag);
+  _table.add(&_driver_sel, 1, 2, row, row+1, mforms::HExpandFlag | mforms::HFillFlag);
+  _table.add(&_desc3, 2, 3, row, row+1, mforms::HFillFlag);
 
-  _tab.add_page(&_params_table, _("Parameters"));
-
-  _params_table.set_column_count(3);
-  _params_table.set_row_spacing(MF_TABLE_ROW_SPACING);
-  _params_table.set_column_spacing(MF_TABLE_COLUMN_SPACING);
-  _params_table.set_padding(MF_PANEL_PADDING);
-
-  _tab.add_page(&_advanced_table, _("Advanced"));
-
-  _advanced_table.set_column_count(3);
-  _advanced_table.set_row_spacing(MF_TABLE_ROW_SPACING);
-  _advanced_table.set_column_spacing(MF_TABLE_COLUMN_SPACING);
-  _advanced_table.set_padding(MF_PANEL_PADDING);
+  _tab.set_name("connect_panel:tab");
+  _params_panel.set_name("params_panel");
+  _advanced_panel.set_name("advanced_panel");
+  _tab.add_page(&_params_panel, _("Parameters"));
+  _tab.add_page(&_advanced_panel, _("Advanced"));
   
+  set_name("connect_panel");
   add(&_table, false, false);
   add(&_tab, true, true);
 }
@@ -155,34 +177,57 @@ void DbConnectPanel::init(DbConnection *conn, const db_mgmt_ConnectionRef &defau
 {
   _connection= conn;
   _delete_connection_be= false;
-  
+    
   _connection->set_control_callbacks(boost::bind(&DbConnectPanel::suspend_view_layout, this, _1),
-                                     boost::bind(&DbConnectPanel::clear_param_controls, this),
-                                     boost::bind(&DbConnectPanel::create_control, this, _1, _2, _3, _4));
-  
-  if (_show_connection_combo)
-    refresh_stored_connections();
-  
-  _anonymous_connection= default_conn.is_valid() ? default_conn : db_mgmt_ConnectionRef(_connection->get_grt());
-
-  grt::ListRef<db_mgmt_Rdbms> rdbms(_connection->get_rdbms_list());
-  _rdbms_sel.clear();
-  for (grt::ListRef<db_mgmt_Rdbms>::const_iterator iter= rdbms.begin();
-       iter != rdbms.end(); ++iter)
-    _rdbms_sel.add_item((*iter)->caption());
-  _rdbms_sel.set_selected(-1);
-
-  
-  int index= _connection->get_default_rdbms_index();
-  set_active_rdbms(index, -1);
-  _rdbms_sel.set_selected(index);
+                                     boost::bind(&DbConnectPanel::begin_layout, this),
+                                     boost::bind(&DbConnectPanel::create_control, this, _1, _2, _3, _4),
+                                     boost::bind(&DbConnectPanel::end_layout, this));
   
   if (default_conn.is_valid())
-    _connection->set_connection(_anonymous_connection);
+    _anonymous_connection= default_conn;
+  else
+  {
+    _anonymous_connection = db_mgmt_ConnectionRef(_connection->get_grt());
+    _anonymous_connection->owner(_connection->get_db_mgmt());
+  }
+
+  if (!_allowed_rdbms.is_valid())
+  {
+    _allowed_rdbms = grt::ListRef<db_mgmt_Rdbms>(_connection->get_grt());
+    _allowed_rdbms.ginsert(_connection->get_db_mgmt()->rdbms()[0]);
+  }
+
+  _rdbms_sel.clear();
+  for (grt::ListRef<db_mgmt_Rdbms>::const_iterator iter= _allowed_rdbms.begin();
+       iter != _allowed_rdbms.end(); ++iter)
+    _rdbms_sel.add_item((*iter)->caption());
+  _rdbms_sel.set_selected(0);
+  
+  _initialized= true;
+  change_active_rdbms();
+  
+  if (!_anonymous_connection->driver().is_valid())
+    _anonymous_connection->driver(selected_driver());
+  
+  if (default_conn.is_valid())
+    _connection->set_connection_and_update(_anonymous_connection);
   else
     _connection->set_connection_keeping_parameters(_anonymous_connection);
+}
 
-  _initialized= true;
+
+void DbConnectPanel::init(const db_mgmt_ManagementRef &mgmt, const grt::ListRef<db_mgmt_Rdbms> &allowed_rdbms, const db_mgmt_ConnectionRef &default_conn)
+{  
+  if (!mgmt.is_valid())
+    throw std::invalid_argument("DbConnectPanel::init() called with invalid db mgmt object");
+
+  _allowed_rdbms = allowed_rdbms;
+  
+  DbConnection *conn= new DbConnection(mgmt, default_conn.is_valid() ? default_conn->driver() : _allowed_rdbms[0]->defaultDriver(),
+                                       _skip_schema_name);
+  
+  init(conn, default_conn);
+  _delete_connection_be= true;
 }
 
 
@@ -190,13 +235,8 @@ void DbConnectPanel::init(const db_mgmt_ManagementRef &mgmt, const db_mgmt_Conne
 {  
   if (!mgmt.is_valid())
     throw std::invalid_argument("DbConnectPanel::init() called with invalid db mgmt object");
-
-  DbConnection *conn= new DbConnection();
-  
-  conn->init(mgmt, _skip_schema_name);
-
-  init(conn, default_conn);
-  _delete_connection_be= true;
+ 
+  init(mgmt, mgmt->rdbms(), default_conn);
 }
 
 
@@ -205,9 +245,32 @@ db_mgmt_ConnectionRef DbConnectPanel::get_connection()
   return _connection->get_connection();
 }
 
+
+grt::ListRef<db_mgmt_Connection> DbConnectPanel::connection_list()
+{
+  if (_rdbms_sel.get_item_count() > 0)
+  {
+    int i = _rdbms_sel.get_selected_index();
+    if (i >= 0 && i < (int)_allowed_rdbms->count())
+    {
+      if (_allowed_rdbms[i]->id() == MYSQL_RDBMS_ID)
+        return _connection->get_db_mgmt()->storedConns();
+      else
+        return _connection->get_db_mgmt()->otherStoredConns();
+    }
+  }
+  
+  db_mgmt_ConnectionRef conn(get_connection());
+  if (conn.is_valid() && conn->driver().is_valid() && conn->driver()->owner().is_valid() && conn->driver()->owner().id() == MYSQL_RDBMS_ID)
+    return _connection->get_db_mgmt()->storedConns();
+  else
+    return _connection->get_db_mgmt()->otherStoredConns();
+}
+
+
 void DbConnectPanel::set_connection(const db_mgmt_ConnectionRef& conn)
 {
-  const grt::ListRef<db_mgmt_Connection> list(_connection->get_db_mgmt()->storedConns());
+  const grt::ListRef<db_mgmt_Connection> list(connection_list());
 
   int count = 0;
   grt::ListRef<db_mgmt_Connection>::const_iterator iter = list.begin();
@@ -220,7 +283,6 @@ void DbConnectPanel::set_connection(const db_mgmt_ConnectionRef& conn)
       change_active_stored_conn();
       break;
     }
-
     ++count;
   }
 }
@@ -263,7 +325,7 @@ void DbConnectPanel::param_value_changed(mforms::View *sender)
 {
   std::string param_name= sender->get_name();
   
-  if (_show_connection_combo && !_updating)
+  if (!_allow_edit_connections && !_updating)
   {
     // if stored connections combo is shown, copy the current connection params to the
     // to anonymous connection and select it
@@ -286,49 +348,105 @@ void DbConnectPanel::param_value_changed(mforms::View *sender)
 }
 
 
+void DbConnectPanel::enum_param_value_changed(mforms::Selector *sender, std::vector<std::string> options)
+{
+  std::string param_name= sender->get_name();
+  
+  if (!_allow_edit_connections && !_updating)
+  {
+    // if stored connections combo is shown, copy the current connection params to the
+    // to anonymous connection and select it
+    // since stored connections are not editable in this case
+    _connection->set_connection_keeping_parameters(_anonymous_connection);
+    if (_stored_connection_sel.get_selected_index() != 0)
+      _stored_connection_sel.set_selected(0);
+  }
+  
+  DbDriverParam *param= _connection->get_db_driver_param_handles()->get(param_name);
+  
+  int i = sender->get_selected_index();
+  if (i >= 0)
+    param->set_value(grt::StringRef(options[i]));
+  else
+    param->set_value(grt::StringRef(""));
+  
+  _connection->save_changes();
+  
+  std::string error= _connection->validate_driver_params();
+  if (error != _last_validation)
+    _signal_validation_state_changed(error, error.empty());
+  _last_validation= error;
+}
+
+
 void DbConnectPanel::change_active_rdbms()
 {
   if (_initialized && !_updating)
-  {
-    set_active_rdbms(_rdbms_sel.get_selected_index(), -1);
-    
-    if (_show_connection_combo)
+  { 
+    if (!_allow_edit_connections)
     {
-      // if stored connections combo is shown, copy the current connection params to the
-      // to anonymous connection and select it
-      // since stored connections are not editable in this case
       _connection->set_connection_keeping_parameters(_anonymous_connection);
       if (_stored_connection_sel.get_selected_index() != 0)
         _stored_connection_sel.set_selected(0);
-    }    
+    }
+    db_mgmt_RdbmsRef active_rdbms(selected_rdbms());
+    if (active_rdbms.is_valid())
+    {
+      int i = 0;
+      int default_driver = -1;
+      _updating = true;
+      // refresh list of drivers
+      grt::ListRef<db_mgmt_Driver> drivers(active_rdbms->drivers());
+      _driver_sel.clear();
+      for (grt::ListRef<db_mgmt_Driver>::const_iterator iter= drivers.begin();
+           iter != drivers.end(); ++iter, ++i)
+      {
+        _driver_sel.add_item((*iter)->caption());
+        if ((*iter) == active_rdbms->defaultDriver())
+          default_driver = i;
+      }
+      
+      if (_show_connection_combo)
+      {
+        // refresh list of stored connections
+        // this will select the driver for the default connection
+        refresh_stored_connections();
+
+        if (_stored_connection_sel.get_selected_index() > 0)
+          change_active_stored_conn();
+        else
+          _connection->set_driver_and_update(selected_driver());
+      }
+      else
+      {
+        // select the default driver for the rdbms
+        if (default_driver >= 0)
+          _driver_sel.set_selected(default_driver);
+        _connection->set_driver_and_update(selected_driver());
+      }
+      _updating = false;      
+    }
+    else
+      log_warning("DbConnectPanel: no active rdbms\n");
   }
 }
 
 
-void DbConnectPanel::set_active_rdbms(int rdbms_index, int driver_index)
-{  
-  if (_rdbms_index != rdbms_index)
-  {
-    _connection->set_active_rdbms(rdbms_index);
+db_mgmt_RdbmsRef DbConnectPanel::selected_rdbms()
+{
+  int i = _rdbms_sel.get_selected_index();
+  if (i >= 0 && i < (int)_allowed_rdbms.count())
+    return _allowed_rdbms[i];
+  return db_mgmt_RdbmsRef();
+}
 
-    grt::ListRef<db_mgmt_Driver> drivers(_connection->get_driver_list());
 
-    _driver_sel.clear();
-    for (grt::ListRef<db_mgmt_Driver>::const_iterator iter= drivers.begin();
-         iter != drivers.end(); ++iter)
-    {
-      // only the default driver is supported ATM
-      if ((*iter)->driverLibraryName() == "mysqlcppconn")
-        _driver_sel.add_item((*iter)->caption());
-    }
-    _driver_sel.set_selected(-1);
-
-    _rdbms_index= rdbms_index;
-  }
-
-  if (driver_index == -1)
-    driver_index= _connection->get_rdbms_default_driver_index();
-  set_active_driver(driver_index);
+db_mgmt_DriverRef DbConnectPanel::selected_driver()
+{
+  int i = _driver_sel.get_selected_index();
+  if (i >= 0 && i < (int)selected_rdbms()->drivers().count())
+    return selected_rdbms()->drivers()[i];
+  return db_mgmt_DriverRef();
 }
 
 
@@ -336,98 +454,142 @@ void DbConnectPanel::change_active_driver()
 {
   if (_initialized && !_updating)
   {
-    db_mgmt_ConnectionRef conn(_connection->get_connection());
-    grt::DictRef current_params(conn->parameterValues());
-    // save current driver params
-    for (grt::DictRef::const_iterator iter = current_params.begin(); iter != current_params.end(); ++iter)
-      _parameters_per_driver[conn->driver()->name()]= grt::DictRef::cast_from(grt::copy_value(current_params, false));
-
-    db_mgmt_DriverRef new_driver(_connection->get_rdbms()->drivers()[_driver_sel.get_selected_index()]);
-    // update params to driver-specific params
-    if (_parameters_per_driver.find(new_driver->name()) == _parameters_per_driver.end())
+    if (!_allow_edit_connections)
     {
-      grt::DictRef params(grt::DictRef::cast_from(grt::copy_value(current_params, false)));
-      
-      if (!_default_host_name.empty())
-      {
-        if (is_ssh_driver(new_driver->name()))
-          params.gset("sshHost", _default_host_name);
-        else
-          params.gset("hostName", _default_host_name);
-      }
-
-      _parameters_per_driver[new_driver->name()]= params;
-    }
-    grt::replace_contents(conn->parameterValues(), _parameters_per_driver[new_driver->name()]);
-
-    set_active_driver(_driver_sel.get_selected_index());
-    
-    if (_show_connection_combo)
-    {
-      // if stored connections combo is shown, copy the current connection params to the
-      // to anonymous connection and select it
-      // since stored connections are not editable in this case
       _connection->set_connection_keeping_parameters(_anonymous_connection);
       if (_stored_connection_sel.get_selected_index() != 0)
         _stored_connection_sel.set_selected(0);
+    }
+    db_mgmt_DriverRef current_driver = _connection->driver();
+    db_mgmt_DriverRef new_driver = selected_driver();
+    if (new_driver == current_driver)
+      return;
+
+    show(false);
+    
+    // When switching to/from ssh based connections the value for the host name gets another
+    // semantic. In ssh based connections the sshHost is the remote server name (what is otherwise
+    // the host name) and the host name is relative to the ssh host (usually localhost).
+    db_mgmt_ConnectionRef actual_connection = get_connection();
+    if (*current_driver->name() == "MysqlNativeSSH")
+    {
+      std::string machine = actual_connection->parameterValues().get_string("sshHost");
+      if (machine.find(':') != std::string::npos)
+        machine = machine.substr(0, machine.find(':'));
+      actual_connection->parameterValues().gset("hostName", machine);
+    }
+    else
+      if (*new_driver->name() == "MysqlNativeSSH")
+      {
+        std::string machine = actual_connection->parameterValues().get_string("hostName");
+        actual_connection->parameterValues().gset("sshHost", machine + ":22");
+        actual_connection->parameterValues().gset("hostName", "127.0.0.1");
+      }
+    
+    _connection->set_driver_and_update(new_driver);
+    show();
+    
+    //db_mgmt_ConnectionRef conn(_connection->get_connection());
+//    grt::DictRef current_params(conn->parameterValues());
+//    // save current driver params
+//    for (grt::DictRef::const_iterator iter = current_params.begin(); iter != current_params.end(); ++iter)
+//      _parameters_per_driver[conn->driver()->name()]= grt::DictRef::cast_from(grt::copy_value(current_params, false));
+//    db_mgmt_DriverRef new_driver(selected_driver());
+//    // update params to driver-specific params
+//    if (_parameters_per_driver.find(new_driver->name()) == _parameters_per_driver.end())
+//    {
+//      grt::DictRef params(grt::DictRef::cast_from(grt::copy_value(current_params, false)));
+//      
+//      if (!_default_host_name.empty())
+//      {
+//        if (is_ssh_driver(new_driver->name()))
+//          params.gset("sshHost", _default_host_name);
+//        else
+//          params.gset("hostName", _default_host_name);
+//      }
+//
+//      _parameters_per_driver[new_driver->name()]= params;
+//    }
+//    grt::replace_contents(conn->parameterValues(), _parameters_per_driver[new_driver->name()]);
+
+    {
+      // we update the validation msg
+      _last_validation= _connection->validate_driver_params();
+      // notify the frontend that the state has changed but don't show any error
+      // even if there is one
+      _signal_validation_state_changed("", _last_validation.empty());
     }    
   }
 }
 
 
-void DbConnectPanel::set_active_driver(int driver_index)
-{
-  // backend will update the parameter UI
-  show(false);
-  _connection->set_active_driver(driver_index);
-
-  _driver_index= driver_index;
-  if (_driver_sel.get_selected_index() != driver_index)
-    _driver_sel.set_selected(driver_index);
-
-  // we update the validation msg
-  _last_validation= _connection->validate_driver_params();
-  // notify the frontend that the state has changed but don't show any error
-  // even if there is one
-  _signal_validation_state_changed("", _last_validation.empty());
-  show();
-}
-
-
 void DbConnectPanel::refresh_stored_connections()
 {
-  grt::ListRef<db_mgmt_Connection> list(_connection->get_db_mgmt()->storedConns());
+  grt::ListRef<db_mgmt_Connection> list(connection_list());
+  db_mgmt_RdbmsRef rdbms = selected_rdbms();
 
+  int selected_index = 0, i = 1;
+  
   _stored_connection_sel.clear();
   _stored_connection_sel.add_item("");
   for (grt::ListRef<db_mgmt_Connection>::const_iterator iter= list.begin(); iter != list.end(); ++iter)
   {
-    _stored_connection_sel.add_item((*iter)->name());
+    if (!rdbms.is_valid() || ((*iter)->driver().is_valid() && (*iter)->driver()->owner() == rdbms))
+    {
+      _stored_connection_sel.add_item((*iter)->name());
+      if (*(*iter)->isDefault())
+        selected_index = i;
+      i++;
+    }
   }
-  
-  _stored_connection_sel.add_item("-");
-  _stored_connection_sel.add_item(_("Manage Stored Connections..."));
-  
-  if (_stored_connection_sel.get_selected_index() != 0)
-    _stored_connection_sel.set_selected(0);
+
+  if (_show_manage_connections)
+  {
+    _stored_connection_sel.add_item("-");
+    _stored_connection_sel.add_item(_("Manage Stored Connections..."));
+  }
+  if (_stored_connection_sel.get_selected_index() != selected_index)
+    _stored_connection_sel.set_selected(selected_index);
 }
 
 
-
-
-void DbConnectPanel::get_connection_details(int &rdbms_index, int &driver_index)
-{  
-  db_mgmt_DriverRef driver= _connection->get_connection()->driver();
-  if (driver.is_valid())
+/**
+ Save the current connection with the given name.
+ */
+void DbConnectPanel::save_connection_as(const std::string &name)
+{
+  _connection->save_changes();
+  
+  db_mgmt_ConnectionRef conn(_connection->get_connection());
+  
+  grt::ListRef<db_mgmt_Connection> list(_connection->get_db_mgmt()->storedConns());
+  if (list.get_index(conn) != grt::BaseListRef::npos)
+    throw std::invalid_argument("The connection cannot be saved because it is already stored");
+  
+  db_mgmt_ConnectionRef dup;
+  if ((dup = find_named_object_in_list(list, name, true, "name")).is_valid())
   {
-    rdbms_index= (int)find_object_index_in_list(_connection->get_rdbms_list(), driver->owner().id());
-    driver_index= (int)find_object_index_in_list(db_mgmt_RdbmsRef::cast_from(driver->owner())->drivers(), driver.id());
+    list->remove(dup);
+    //throw std::invalid_argument(strfmt("The connection cannot be saved. There already is a connection named '%s'", name.c_str()));
   }
-  else
+
+  list = _connection->get_db_mgmt()->otherStoredConns();
+  if (list.get_index(conn) != grt::BaseListRef::npos)
+    throw std::invalid_argument("The connection cannot be saved becasue it is already stored");
+
+  if ((dup = find_named_object_in_list(list, name, true, "name")).is_valid())
   {
-    rdbms_index= -1;
-    driver_index= -1;
+    list->remove(dup);
+    //throw std::invalid_argument(strfmt("The connection cannot be saved. There already is a connection named '%s'", name.c_str()));
   }
+
+  conn->name(name);
+  conn->owner(_connection->get_db_mgmt());
+  
+  connection_list().insert(conn);
+  
+  refresh_stored_connections();
+  change_active_stored_conn();
 }
 
 
@@ -457,91 +619,134 @@ bool DbConnectPanel::test_connection()
 }
 
 
-void DbConnectPanel::set_active_stored_conn(int stored_conn_index)
+void DbConnectPanel::set_active_stored_conn(const std::string &name)
 {
-  int rdbms_index;
-  int driver_index;
-  
-  grt::ListRef<db_mgmt_Connection> conns(_connection->get_db_mgmt()->storedConns());
-  if (stored_conn_index <= 0 || stored_conn_index > (int) conns.count())
-    _connection->set_connection(_anonymous_connection);
+  if (name.empty())
+    _connection->set_connection_keeping_parameters(_anonymous_connection);
   else
-    _connection->set_connection(conns[stored_conn_index-1]);
-  
-  get_connection_details(rdbms_index, driver_index);
+    set_active_stored_conn(find_named_object_in_list(connection_list(), name, true, "name"));
+}
 
-  if (rdbms_index == -1)
-    rdbms_index= _rdbms_sel.get_selected_index();
-  if (driver_index == -1)
-    driver_index= _driver_sel.get_selected_index();
 
-  set_active_rdbms(rdbms_index, driver_index);
+void DbConnectPanel::set_active_stored_conn(db_mgmt_ConnectionRef connection)
+{
+  if (!connection.is_valid())
+    connection = _anonymous_connection;
+
+  db_mgmt_DriverRef driver = connection->driver();
+  db_mgmt_RdbmsRef rdbms = db_mgmt_RdbmsRef::cast_from(connection->driver()->owner());
+  // check if the rdbms of the connection is not the selected one (usually should be)
+  if (rdbms.is_valid() && selected_rdbms() != rdbms)
+  {
+    int rdbms_index = find_object_index_in_list(_allowed_rdbms, rdbms->id());
+    _rdbms_sel.set_selected(rdbms_index);
+    change_active_rdbms();
+  }
   
+  // ensure the correct driver is selected in the selector
+  int driver_index = find_object_index_in_list(rdbms->drivers(), driver->id());
+  if (driver_index >= 0 && driver_index < _driver_sel.get_item_count())
+    _driver_sel.set_selected(driver_index);
+
+  // mark this connection as the active one for this rdbms type
+  {
+    grt::ListRef<db_mgmt_Connection> conns(connection_list());
+    for (size_t c = conns->count(), i = 0; i < c; i++)
+    {
+      db_mgmt_ConnectionRef conn(conns[i]);
+      if (conn->driver().is_valid() && conn->driver()->owner() == rdbms)
+        conn->isDefault(0);
+    }
+    connection->isDefault(1);
+  }
+
+  // set the connection as the one being edited, updating the UI for it
+  _connection->set_connection_and_update(connection);
+
   if (!_show_connection_combo)
-    _name_entry.set_value(_connection->get_connection()->name());
+    _name_entry.set_value(connection->name());
 }
 
 
 void DbConnectPanel::change_active_stored_conn()
 {
   static bool choosing = false;
-  show(false);
   if (_initialized && !choosing)
   {
     _updating= true;
 
-    if (_stored_connection_sel.get_selected_index() == _stored_connection_sel.get_item_count()-1)
+    if (_show_manage_connections && _stored_connection_sel.get_selected_index() == _stored_connection_sel.get_item_count()-1)
     {   
       choosing = true;
-      int index= open_editor();
+      db_mgmt_ConnectionRef connection = open_editor();
       refresh_stored_connections();
-      _stored_connection_sel.set_selected(index);
-      set_active_stored_conn(index);
+      if (connection.is_valid())
+        _stored_connection_sel.set_selected(_stored_connection_sel.index_of_item_with_title(*connection->name()));
+      else
+        _stored_connection_sel.set_selected(0);
+      show(false);
+      set_active_stored_conn(connection);
+      show();
       choosing = false;
     }
     else
-      set_active_stored_conn(_stored_connection_sel.get_selected_index());
-    
+    {
+      std::string name = _stored_connection_sel.get_string_value();
+      show(false);
+      set_active_stored_conn(name);
+      show();
+    }
     _updating= false;
   }
-  show();
 }
 
 
-int DbConnectPanel::open_editor()
+db_mgmt_ConnectionRef DbConnectPanel::open_editor()
 {
+  grt::ListRef<db_mgmt_Rdbms> rdbms_list(_connection->get_grt());
+  rdbms_list.ginsert(selected_rdbms());
   DbConnectionEditor editor(_connection->get_db_mgmt());
-  db_mgmt_ConnectionRef selected;
 
-  selected= editor.run(_connection->get_connection());
-
-  if (selected.is_valid())
-  {
-    grt::ListRef<db_mgmt_Connection> conns(_connection->get_db_mgmt()->storedConns());
-    
-    size_t index= conns.get_index(selected);
-    if (index != grt::BaseListRef::npos)
-      return index+1;
-  }
-  return 0;
+  return editor.run(_connection->get_connection());
 }
 
 
-void DbConnectPanel::clear_param_controls()
+void DbConnectPanel::begin_layout()
 {
-  for (std::list<View*>::reverse_iterator iter= _views.rbegin();
-       iter != _views.rend(); ++iter)
+  if (_params_table)
   {
-    Box *box= dynamic_cast<Box*> ((*iter)->get_parent());
-    if (box)
-      box->remove(*iter);
-    else
-      dynamic_cast<Table*> ((*iter)->get_parent())->remove(*iter);
-    (*iter)->release();
+    //_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF | _CRTDBG_CHECK_ALWAYS_DF );
+
+    _params_panel.remove(_params_table);
+  }  
+  if (_advanced_table)
+  {
+    _advanced_panel.remove(_advanced_table);
   }
+  _params_table = mforms::manage(new mforms::Table());
+  _params_table->set_name("params_table");
+  _params_table->set_column_count(3);
+  _params_table->set_row_spacing(MF_TABLE_ROW_SPACING);
+  _params_table->set_column_spacing(MF_TABLE_COLUMN_SPACING);
+  _params_table->set_padding(MF_PANEL_PADDING);
+
+  _advanced_table = mforms::manage(new mforms::Table());
+  _advanced_table->set_name("advanced_table");
+  _advanced_table->set_column_count(3);
+  _advanced_table->set_row_spacing(MF_TABLE_ROW_SPACING);
+  _advanced_table->set_column_spacing(MF_TABLE_COLUMN_SPACING);
+  _advanced_table->set_padding(MF_PANEL_PADDING);
+  
   _views.clear();
   _param_rows.clear();
-  _advanced_rows.clear();
+  _advanced_rows.clear();  
+}
+
+
+void DbConnectPanel::end_layout()
+{
+  _params_panel.add(_params_table);
+  _advanced_panel.add(_advanced_table);
 }
 
 
@@ -620,7 +825,7 @@ void DbConnectPanel::create_control(::DbDriverParam *driver_param, const ::Contr
       {
         is_new_line= true;
 
-        _params_table.set_row_count(_param_rows.size() + 1);
+        _params_table->set_row_count(_param_rows.size() + 1);
         _param_rows.push_back(box= new Box(true));
 
         mforms::TableItemFlags flags;
@@ -629,14 +834,14 @@ void DbConnectPanel::create_control(::DbDriverParam *driver_param, const ::Contr
 #else
         flags = mforms::HExpandFlag | mforms::HFillFlag;
 #endif
-        _params_table.add(mforms::manage(box), 1, 2, bounds.top, bounds.top + 1, flags);
+        _params_table->add(mforms::manage(box), 1, 2, bounds.top, bounds.top + 1, flags);
         box->set_spacing(4);
         _views.push_back(box);
       }
       else
         box= _param_rows[bounds.top];
 
-      table= &_params_table;
+      table= _params_table;
       break;
     }
   case 1:
@@ -645,7 +850,7 @@ void DbConnectPanel::create_control(::DbDriverParam *driver_param, const ::Contr
       {
         is_new_line= true;
 
-        _advanced_table.set_row_count(_advanced_rows.size() + 1);
+        _advanced_table->set_row_count(_advanced_rows.size() + 1);
         if (ctrl_type == ::ctCheckBox)
         {
           _advanced_rows.push_back(box= new Box(false));
@@ -658,15 +863,15 @@ void DbConnectPanel::create_control(::DbDriverParam *driver_param, const ::Contr
         }
         _views.push_back(box);
 #ifdef __APPLE__ // TODO: HExpand is messing layout in mac, needs fix
-        _advanced_table.add(mforms::manage(box), 1, 2, bounds.top, bounds.top + 1, mforms::HFillFlag);
+        _advanced_table->add(mforms::manage(box), 1, 2, bounds.top, bounds.top + 1, mforms::HFillFlag);
 #else
-        _advanced_table.add(mforms::manage(box), 1, 2, bounds.top, bounds.top + 1, mforms::HExpandFlag|mforms::HFillFlag);
+        _advanced_table->add(mforms::manage(box), 1, 2, bounds.top, bounds.top + 1, mforms::HExpandFlag|mforms::HFillFlag);
 #endif
       }
       else
         box= _advanced_rows[bounds.top];
 
-      table= &_advanced_table;
+      table= _advanced_table;
       break;
     }
   default:
@@ -790,6 +995,30 @@ void DbConnectPanel::create_control(::DbDriverParam *driver_param, const ::Contr
       
       break;
     }
+    case ::ctEnumSelector:
+    {
+      mforms::Selector *ctrl = new Selector();
+      ctrl->set_name(driver_param->get_control_name());
+      std::vector<std::pair<std::string, std::string> > options = driver_param->get_enum_options();
+      std::vector<std::string> option_ids;
+      std::string value = driver_param->get_value_repr();
+      int idx = -1;
+      for (size_t i = 0; i < options.size(); i++)
+      {
+        ctrl->add_item(options[i].second);
+        option_ids.push_back(options[i].first);
+        if (value == options[i].first)
+          idx = i;
+      }
+      if (idx >= 0)
+        ctrl->set_selected(idx);
+      enum_param_value_changed(ctrl, option_ids);
+
+      scoped_connect(ctrl->signal_changed(),boost::bind(&DbConnectPanel::enum_param_value_changed, this, ctrl, option_ids));
+      box->add(mforms::manage(ctrl), true, true);
+      _views.push_back(ctrl);
+      break;
+    }
     default:
       g_message("Unknown param type for %s", driver_param->get_control_name().c_str());
       break;
