@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,19 +21,51 @@
 #include "grtdb_connection_editor.h"
 
 #include "cppdbc.h"
-#include "base/string_utilities.h"
 #include <grt/common.h>
+#include "base/string_utilities.h"
+
+#include "mforms/uistyle.h"
+#include "mforms/utilities.h"
 
 //------------------------------------------------------------------------------
-grtui::DbConnectionEditor::DbConnectionEditor(const db_mgmt_ManagementRef &mgmt)
+/*
+grtui::DbConnectionEditor::DbConnectionEditor(const db_mgmt_ManagementRef &mgmt, const grt::ListRef<db_mgmt_Rdbms> &allowed_rdbms)
   : mforms::Form(0)
   , _mgmt(mgmt)
+  , _connection_list(mgmt->otherStoredConns())
   , _panel(false)
   , _top_vbox(false)
   , _top_hbox(true)
   , _conn_list_buttons_hbox(true)
-  , _stored_connection_list(mforms::TreeDefault)
+  , _stored_connection_list(mforms::TreeDefault|mforms::TreeFlatList)
   , _bottom_hbox(true)
+{
+  _panel.init(_mgmt, allowed_rdbms);
+
+  init();
+}*/
+
+// for MySQL only connection editors
+grtui::DbConnectionEditor::DbConnectionEditor(const db_mgmt_ManagementRef &mgmt)
+: mforms::Form(0)
+, _mgmt(mgmt)
+, _connection_list(mgmt->storedConns())
+, _panel(false)
+, _top_vbox(false)
+, _top_hbox(true)
+, _conn_list_buttons_hbox(true)
+, _stored_connection_list(mforms::TreeDefault|mforms::TreeFlatList)
+, _bottom_hbox(true)
+{  
+  grt::ListRef<db_mgmt_Rdbms> default_allowed_rdbms(mgmt.get_grt());
+  default_allowed_rdbms.ginsert(find_object_in_list(mgmt->rdbms(), "com.mysql.rdbms.mysql"));
+  _panel.init(_mgmt, default_allowed_rdbms);
+  
+  init();
+}
+
+
+void grtui::DbConnectionEditor::init()
 {
   set_title(_("Manage DB Connections"));
   
@@ -44,9 +76,7 @@ grtui::DbConnectionEditor::DbConnectionEditor(const db_mgmt_ManagementRef &mgmt)
   _top_vbox.add(&_bottom_hbox, false, true);
   
   _bottom_hbox.set_spacing(12);
-  
-  _panel.init(_mgmt);
-  
+    
   scoped_connect(_stored_connection_list.signal_changed(),boost::bind(&grtui::DbConnectionEditor::change_active_stored_conn, this));
 
   _conn_name= _panel.get_name_entry();
@@ -111,10 +141,10 @@ db_mgmt_ConnectionRef grtui::DbConnectionEditor::run(const db_mgmt_ConnectionRef
 { 
   size_t index;
   // check if the connection is a pre-saved one and if so, just select it in the list
-  if ((index = _panel.get_be()->get_db_mgmt()->storedConns().get_index(connection)) != grt::BaseListRef::npos)
+  if ((index = _connection_list.get_index(connection)) != grt::BaseListRef::npos)
   {
     reset_stored_conn_list();
-    _stored_connection_list.set_selected(index);
+    _stored_connection_list.select_node(_stored_connection_list.node_at_row(index));
     change_active_stored_conn();
   }
   else
@@ -123,7 +153,7 @@ db_mgmt_ConnectionRef grtui::DbConnectionEditor::run(const db_mgmt_ConnectionRef
 
     // set the displayed params to the one given to us
     if (connection.is_valid())
-      _panel.get_be()->set_connection(connection);
+      _panel.get_be()->set_connection_and_update(connection);
     // switch the connection object with the anon connection from the panel object
     _panel.get_be()->set_connection_keeping_parameters(_panel.get_default_connection());
     _conn_name->set_value(_("Press New to save these settings"));
@@ -136,7 +166,7 @@ db_mgmt_ConnectionRef grtui::DbConnectionEditor::run(const db_mgmt_ConnectionRef
 }
 
 
-//------------------------------------------------------------------------------
+//--------------------------------------------------------------------------------------------------
 void grtui::DbConnectionEditor::run()
 {
   reset_stored_conn_list();
@@ -144,44 +174,42 @@ void grtui::DbConnectionEditor::run()
   run_modal(&_ok_button, &_cancel_button);
 }
 
-//------------------------------------------------------------------------------
+//--------------------------------------------------------------------------------------------------
+
 void grtui::DbConnectionEditor::reset_stored_conn_list()
 {
-  grt::ListRef<db_mgmt_Connection> list(_panel.get_be()->get_db_mgmt()->storedConns());
+  grt::ListRef<db_mgmt_Connection> list(_connection_list);
   std::string selected_name;
   
   if (_panel.get_be()->get_connection().is_valid())
     selected_name= _panel.get_be()->get_connection()->name();
   
-  _stored_connection_list.clear_rows();
+  _stored_connection_list.clear();
 
-  int selected_idx= -1;
-  int added_row_idx = -1;
+  mforms::TreeNodeRef selected_node;
+  mforms::TreeNodeRef added_row;
   grt::ListRef<db_mgmt_Connection>::const_iterator iter = list.begin();
   grt::ListRef<db_mgmt_Connection>::const_iterator last = list.end();
   for (; iter != last; ++iter)
   {
-    added_row_idx = _stored_connection_list.add_row();
-    if (added_row_idx >= 0)
+    added_row = _stored_connection_list.root_node()->add_child();
+    if (added_row)
     {
-      _stored_connection_list.set(added_row_idx, 0, (*iter)->name());
+      added_row->set_string(0, (*iter)->name());
       if ((*iter)->name() == selected_name)
-        selected_idx= added_row_idx;
+        selected_node= added_row;
     }
   }
 
-  if (selected_idx >= 0)
-  {
-    selected_idx = selected_idx >= 0 ? selected_idx : 0;
-    _stored_connection_list.set_selected(selected_idx);
-  }
+  if (selected_node)
+    _stored_connection_list.select_node(selected_node);
   change_active_stored_conn();
 }
 
 //------------------------------------------------------------------------------
 void grtui::DbConnectionEditor::add_stored_conn(bool copy)
 {
-  grt::ListRef<db_mgmt_Connection> list(_panel.get_be()->get_db_mgmt()->storedConns());
+  grt::ListRef<db_mgmt_Connection> list(_connection_list);
   const int length = (std::string("New connection 1")).length() - 1;
   int max_conn_nr = 0;
   for (size_t i = 0; i < list.count(); ++i)
@@ -202,15 +230,13 @@ void grtui::DbConnectionEditor::add_stored_conn(bool copy)
   
   new_connection->owner(_panel.get_be()->get_db_mgmt());
   new_connection->name(buf);
+  new_connection->driver(_panel.selected_driver());
   list.insert(new_connection);
   
   if (copy)
     _panel.get_be()->set_connection_keeping_parameters(new_connection);
   else
-  {
-    _panel.set_active_driver(0);
-    _panel.get_be()->set_connection(new_connection);
-  }
+    _panel.set_connection(new_connection);
   reset_stored_conn_list();
   change_active_stored_conn();
 }
@@ -218,10 +244,10 @@ void grtui::DbConnectionEditor::add_stored_conn(bool copy)
 //------------------------------------------------------------------------------
 void grtui::DbConnectionEditor::del_stored_conn()
 {
-  const int idx = _stored_connection_list.get_selected();
-  if (idx >= 0 && idx < (int)_mgmt->storedConns().count())
+  const int idx = _stored_connection_list.get_selected_row();
+  if (idx >= 0 && idx < (int)_connection_list.count())
   {
-    grt::ListRef<db_mgmt_Connection> conns(_mgmt->storedConns());
+    grt::ListRef<db_mgmt_Connection> conns(_connection_list);
     db_mgmt_ConnectionRef conn(conns[idx]);
     bool found = false;
     grt::ListRef<db_mgmt_ServerInstance> instances(_mgmt->storedInstances());
@@ -243,6 +269,28 @@ void grtui::DbConnectionEditor::del_stored_conn()
       return;
     }
 
+    // Remove password associated with this connection (if stored in keychain/vault). Check first
+    // this service/username combination isn't used anymore by other connections.
+    bool credentials_still_used = false;
+    grt::DictRef parameter_values = conn->parameterValues();
+    std::string host = conn->hostIdentifier();
+    std::string user = parameter_values.get_string("userName");
+    for (grt::ListRef<db_mgmt_Connection>::const_iterator i = conns.begin();
+      i != conns.end(); ++i)
+    {
+      if (*i != conn)
+      {
+        grt::DictRef current_parameters = (*i)->parameterValues();
+        if (host == *(*i)->hostIdentifier() && user == current_parameters.get_string("userName"))
+        {
+          credentials_still_used = true;
+          break;
+        }
+      }
+    }
+    if (!credentials_still_used)
+      mforms::Utilities::forget_password(host, user);
+    
     if (idx < (int)conns.count())
       conns.remove(idx);
     
@@ -254,12 +302,12 @@ void grtui::DbConnectionEditor::del_stored_conn()
 //------------------------------------------------------------------------------
 void grtui::DbConnectionEditor::change_active_stored_conn()
 {
-  const int selected = _stored_connection_list.get_selected();
-  if (selected >= 0)
+  mforms::TreeNodeRef selected = _stored_connection_list.get_selected_node();
+  if (selected)
   {
     _panel.set_enabled(true);
     _panel.suspend_layout();
-    _panel.set_active_stored_conn(selected+1);
+    _panel.set_active_stored_conn(selected->get_string(0));
     _panel.resume_layout();
     
     _test_button.set_enabled(true);
@@ -283,11 +331,15 @@ void grtui::DbConnectionEditor::change_active_stored_conn()
 void grtui::DbConnectionEditor::name_changed()
 {
   std::string name= _conn_name->get_string_value();
+  mforms::TreeNodeRef selnode(_stored_connection_list.get_selected_node());
   
-  std::string oname= _stored_connection_list.get_string(_stored_connection_list.get_selected(), 0);
-  
-  if (rename_stored_conn(oname, name))
-    _stored_connection_list.set(_stored_connection_list.get_selected(), 0, name);
+  if (selnode)
+  {
+    std::string oname= selnode->get_string(0);
+ 
+    if (rename_stored_conn(oname, name))
+      selnode->set_string(0, name);
+  }
 }
 
 //------------------------------------------------------------------------------
@@ -306,7 +358,7 @@ bool grtui::DbConnectionEditor::rename_stored_conn(const std::string &oname, con
 {
   if (name == oname) return true;
   
-  grt::ListRef<db_mgmt_Connection> conns= _mgmt->storedConns();
+  grt::ListRef<db_mgmt_Connection> conns= _connection_list;
   db_mgmt_ConnectionRef conn= find_named_object_in_list(conns, oname);
   
   if (conn.is_valid())
@@ -325,32 +377,32 @@ bool grtui::DbConnectionEditor::rename_stored_conn(const std::string &oname, con
 
 void grtui::DbConnectionEditor::reorder_conn(bool up)
 {
-  grt::ListRef<db_mgmt_Connection> conns= _mgmt->storedConns();
-  int i= _stored_connection_list.get_selected();
+  grt::ListRef<db_mgmt_Connection> conns= _connection_list;
+  int row = _stored_connection_list.get_selected_row();
   
-  if (i < 0)
+  if (row < 0)
     return;
   
   if (up)
   {
-    if (i > 0)
+    if (row > 0)
     {
-      conns.reorder(i, i-1);
-      _stored_connection_list.set_selected(i-1);
+      conns.reorder(row, row-1);
+      _stored_connection_list.select_node(_stored_connection_list.node_at_row(row-1));
     }
   }
   else
   {
-    if (i < _stored_connection_list.count()-1)
+    if (row < _stored_connection_list.root_node()->count()-1)
     {
-      conns.reorder(i, i+1);
-      _stored_connection_list.set_selected(i+1);
+      conns.reorder(row, row+1);
+      _stored_connection_list.select_node(_stored_connection_list.node_at_row(row+1));
     }
   }
   
-  i= 0;
+  row= 0;
   GRTLIST_FOREACH(db_mgmt_Connection, conns, inst)
   {
-    _stored_connection_list.set(i++, 0, (*inst)->name().c_str());
+    _stored_connection_list.root_node()->get_child(row++)->set_string(0, (*inst)->name().c_str());
   }    
 }
