@@ -108,14 +108,15 @@ grt::StringRef db_Column::formattedType() const
 
   if (simpleType.is_valid())
   {
+    int ptype = simpleType->parameterFormatType();
     caption= simpleType->name();
 
     if (simpleType->numericPrecision() != bec::EMPTY_TYPE_PRECISION)
     {
       std::string tmp;
-      if (this->scale() != bec::EMPTY_COLUMN_SCALE)
+      if (this->precision() != bec::EMPTY_COLUMN_PRECISION && this->scale() != bec::EMPTY_COLUMN_SCALE && (ptype == 3 || ptype == 4 || ptype == 5 || ptype == 6))
         tmp= strfmt("(%i,%i)", (int)this->precision(), (int)this->scale());
-      else if (this->precision() != bec::EMPTY_COLUMN_PRECISION)
+      else if (this->precision() != bec::EMPTY_COLUMN_PRECISION && (ptype == 1 || ptype == 2 || ptype == 4 || ptype == 6))
         tmp= strfmt("(%i)", (int)this->precision());
       caption.append(tmp);
     }
@@ -124,7 +125,7 @@ grt::StringRef db_Column::formattedType() const
       if (*simpleType->characterMaximumLength() != bec::EMPTY_TYPE_MAXIMUM_LENGTH 
           || *simpleType->characterOctetLength() != bec::EMPTY_TYPE_OCTET_LENGTH)
       {
-        if (this->length() != bec::EMPTY_COLUMN_LENGTH)
+        if (this->length() != bec::EMPTY_COLUMN_LENGTH && (ptype == 1 || ptype == 2 || ptype == 4 || ptype == 6))
         {
           caption.append(strfmt("(%i)", (int)this->length()));
         }
@@ -411,13 +412,17 @@ static bool parseTypeDefinition(const std::string &type,
 grt::IntegerRef db_Column::setParseType(const std::string &type, const grt::ListRef<db_SimpleDatatype> &typeList)
 {
   grt::ListRef<db_UserDatatype> user_types;
+  grt::ListRef<db_SimpleDatatype> default_type_list;
   if (!owner().is_valid() || !owner()->owner().is_valid() || !owner()->owner()->owner().is_valid())
     ;
   else
   {
     db_CatalogRef catalog= db_CatalogRef::cast_from(owner()->owner()->owner());
     if (catalog.is_valid())
+    {
       user_types= catalog->userDatatypes();
+      default_type_list = catalog->simpleDatatypes();
+    }
   }
 
   db_UserDatatypeRef userType;
@@ -475,7 +480,7 @@ grt::IntegerRef db_Column::setParseType(const std::string &type, const grt::List
     }
     
     // parse usertype definition    
-    if (!parseTypeDefinition(finalType, typeList, simpleType, 
+    if (!parseTypeDefinition(finalType, typeList.is_valid() ? typeList : default_type_list, simpleType, 
                              precision, scale, length, datatypeExplicitParams))
       return false;
     
@@ -498,7 +503,7 @@ grt::IntegerRef db_Column::setParseType(const std::string &type, const grt::List
   }
   else
   {
-    if (!parseTypeDefinition(type, typeList, simpleType, 
+    if (!parseTypeDefinition(type, typeList.is_valid() ? typeList : default_type_list, simpleType, 
                              precision, scale, length, datatypeExplicitParams))
       return false;
     
