@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,6 +24,7 @@
 #include "recordset_be.h"
 #include "grtsqlparser/sql_facade.h"
 #include "base/string_utilities.h"
+#include "base/sqlstring.h"
 #include "base/boost_smart_ptr_helpers.h"
 #include <sqlite/query.hpp>
 #include <boost/foreach.hpp>
@@ -58,7 +59,7 @@ std::string PrimaryKeyPredicate::operator()(std::vector<boost::shared_ptr<sqlite
     size_t partition;
     ColumnId partition_column= Recordset::translate_data_swap_db_column(col, &partition);
     boost::shared_ptr<sqlite::result> &data_row_rs= data_row_results[partition];
-
+    
     data_row_rs->get_variant(partition_column, v);
     predicate+= "`" + (*_column_names)[col] + "`=" + boost::apply_visitor(*_qv, (*_column_types)[col], v) + " and";
   }
@@ -107,6 +108,7 @@ void Recordset_sql_storage::init_variant_quoter(sqlide::QuoteVar &qv) const
   }
   else
   {
+    // used for sqlite storage and others
     qv.escape_string= boost::bind(sqlide::QuoteVar::escape_ansi_sql_string, _1);
     // swap db (sqlite) stores unknown values as quoted strings
     qv.store_unknown_as_string= true;
@@ -171,7 +173,9 @@ void Recordset_sql_storage::do_unserialize(Recordset *recordset, sqlite::connect
   }
   else
   {
-    _readonly= !_sql_query.empty();
+    // Is read only if it is a non editable select in which case
+    // schema name and table name will be empty
+    _readonly= !_sql_query.empty() && full_table_name().empty();
     _valid= false;
   }
 }
@@ -333,6 +337,8 @@ void Recordset_sql_storage::get_pkey_predicate_for_data_cache_rowid(Recordset *r
 
   sqlide::QuoteVar qv;
   init_variant_quoter(qv);
+  // turn blob values into hex strings when building a primary key, since we can't bind those
+  //  qv.blob_to_string= boost::bind(sqlide::QuoteVar::blob_to_hex_string, _1, _2);
   PrimaryKeyPredicate pkey_pred(&column_types, &column_names, &_pkey_columns, &qv);
   pkey_predicate= pkey_pred(data_row_results);
 }
@@ -340,6 +346,8 @@ void Recordset_sql_storage::get_pkey_predicate_for_data_cache_rowid(Recordset *r
 
 void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqlite::connection *data_swap_db, Sql_script &sql_script, bool is_update_script)
 {
+  // this one is used mostly internally, for eg.: sqlite inserts storage
+
   if (_is_sql_script_substitute_enabled)
   {
     sql_script= _sql_script_substitute;
@@ -357,6 +365,12 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
   ColumnId editable_col_count= recordset->get_column_count();
   sqlide::QuoteVar qv;
   init_variant_quoter(qv);
+
+  sqlide::QuoteVar pk_qv;
+  init_variant_quoter(pk_qv);
+  // turn blob values into hex strings when building a primary key, since we can't bind those
+  pk_qv.blob_to_string= boost::bind(sqlide::QuoteVar::blob_to_hex_string, _1, _2);
+
   std::string full_table_name= this->full_table_name();
 
   if (0 == editable_col_count)
@@ -368,7 +382,7 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
 
   if (is_update_script)
   {
-    PrimaryKeyPredicate pkey_pred(&column_types, &column_names, &_pkey_columns, &qv);
+    PrimaryKeyPredicate pkey_pred(&real_column_types, &column_names, &_pkey_columns, &pk_qv);
 
     std::list<boost::shared_ptr<sqlite::query> > data_row_queries(partition_count);
     Recordset::prepare_partition_queries(data_swap_db, "select * from `data%s` where id=?", data_row_queries);
@@ -454,10 +468,9 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
                 boost::shared_ptr<sqlite::result> &data_row_rs= data_row_results[partition];
 
                 data_row_rs->get_variant(partition_column, v);
+
                 values+= strfmt("%s, ",
-                  column_quoting[column] || sqlide::is_var_null(v)?
-                  boost::apply_visitor(qv, column_types[column], v).c_str():
-                  boost::apply_visitor(var_to_str, v).c_str());
+                    boost::apply_visitor(qv, column_types[column], v).c_str());
                 if (blob_columns[column])
                   sql_bindings.push_back(v);
               }
@@ -466,7 +479,6 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
                 col_names.resize(col_names.size()-2);
               if (!values.empty())
                 values.resize(values.size()-2);
-
               sql= strfmt("INSERT INTO %s (%s) VALUES (%s)", _omit_schema_qualifier ? (std::string("`")+table_name()+std::string("`")).c_str() : full_table_name.c_str(), col_names.c_str(), values.c_str());
             }
           }
@@ -501,16 +513,13 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
                 data_row_rs->get_variant(partition_column, v);
                 values+= strfmt("`%s`=%s, ",
                   column_names[column].c_str(),
-                  column_quoting[column] || sqlide::is_var_null(v)?
-                  boost::apply_visitor(qv, column_types[column], v).c_str():
-                  boost::apply_visitor(var_to_str, v).c_str());
+                  boost::apply_visitor(qv, column_types[column], v).c_str());
                 if (blob_columns[column] && _binding_blobs)
                   sql_bindings.push_back(v);
               }
               while (changed_row_columns_rs->next_row());
               if (!values.empty())
                 values.resize(values.size()-2);
-
               sql= strfmt("UPDATE %s SET %s WHERE %s", full_table_name.c_str(), values.c_str(), pkey_pred(data_row_results).c_str());
             }
           }
@@ -572,6 +581,8 @@ void Recordset_sql_storage::generate_sql_script(const Recordset *recordset, sqli
 
 void Recordset_sql_storage::generate_inserts(const Recordset *recordset, sqlite::connection *data_swap_db, Sql_script &sql_script)
 {
+  // This one is used for generating inserts for export
+
   if (_is_sql_script_substitute_enabled)
   {
     sql_script= _sql_script_substitute;
@@ -623,10 +634,19 @@ void Recordset_sql_storage::generate_inserts(const Recordset *recordset, sqlite:
             data_rs->get_variant(partition_column, v);
             sqlide::VarToStr var_to_str;
 
-            values+= strfmt("%s, ",//boost::apply_visitor(var_to_str, v).c_str());
-                            column_quoted[col] || sqlide::is_var_null(v)?
-                            boost::apply_visitor(qv, column_types[col], v).c_str():
-                            boost::apply_visitor(var_to_str, v).c_str());
+            std::string value;
+            if (column_quoted[col] || sqlide::is_var_null(v))
+              value = boost::apply_visitor(qv, column_types[col], v);
+            else
+            {
+              value = boost::apply_visitor(var_to_str, v);
+              static std::string func_escape = "\\func ";
+              static std::string func_escape_ape = "\\\\func ";
+              if (value.substr(0, func_escape.size()) == func_escape
+                  || value.substr(0, func_escape_ape.size()) == func_escape_ape)
+                value = boost::apply_visitor(qv, column_types[col], v);
+            }
+            values+= strfmt("%s, ", value.c_str());//boost::apply_visitor(var_to_str, v).c_str());
           }
         }
         if (!values.empty())
