@@ -26,10 +26,10 @@
 #include <sqlite/database_exception.hpp>
 #include <sqlite/variant.hpp>
 #include <sqlite/connection.hpp>
-#include <sigc++/sigc++.h>
 #include <limits>
 #include <ctime>
 #include <base/string_utilities.h>
+#include <boost/function.hpp>
 
 
 namespace sqlide
@@ -171,9 +171,9 @@ class WBPUBLICBACKEND_PUBLIC_FUNC QuoteVar : public VarConvBase
 {
 public:
   QuoteVar() : store_unknown_as_string(true), allow_func_escaping(false) {}
-  typedef sigc::slot<std::string, const std::string&> Escape_sql_string;
+  typedef boost::function<std::string (const std::string&)> Escape_sql_string;
   Escape_sql_string escape_string;
-  typedef sigc::slot<std::string, const unsigned char*, size_t> Blob_to_string;
+  typedef boost::function<std::string (const unsigned char*, size_t)> Blob_to_string;
   Blob_to_string blob_to_string;
   bool store_unknown_as_string;
   bool allow_func_escaping;
@@ -232,7 +232,7 @@ public:
   template<typename T>
   result_type operator()(const T&, const Null &) const { return "NULL"; }
   template<typename T>
-  result_type operator()(const T&, const std::string &v) const
+  result_type operator()(const T&x, const std::string &v) const
   {
     if (allow_func_escaping)
     {
@@ -256,11 +256,14 @@ public:
   {
     return (blob_to_string.empty()) ? "?" /*bind variable placeholder*/ : blob_to_string(&(*v)[0], v->size());
   }
-  template<typename V>
   result_type operator()(const BlobRef&, const BlobRef& v) const
   {
     return (blob_to_string.empty()) ? "?" /*bind variable placeholder*/ : blob_to_string(&(*v)[0], v->size());
   }
+  result_type operator()(const BlobRef&, const std::string& v) const
+  {
+    return (blob_to_string.empty()) ? "?" /*bind variable placeholder*/ : blob_to_string((const unsigned char*)v.data(), v.size());
+  }
   result_type operator()(const BlobRef&, const Null&) const
   {
     return (blob_to_string.empty()) ? "?" /*bind variable placeholder*/ : "NULL";
