@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -75,8 +75,10 @@ public:
 
 public:
   virtual bool is_readonly() const { return _readonly; }
+  virtual std::string readonly_reason() const { return _readonly_reason; }
 protected:
   bool _readonly;
+  std::string _readonly_reason;
 
 public:
   typedef std::vector<std::string> Column_names;
@@ -100,6 +102,7 @@ public:
   virtual bool set_field(const bec::NodeId &node, int column, long long value);
   virtual bool get_field(const bec::NodeId &node, int column, std::string &value);
   virtual bool get_field_repr(const bec::NodeId &node, int column, std::string &value);
+  bool get_field_repr_no_truncate(const bec::NodeId &node, int column, std::string &value);
   virtual bool get_field(const bec::NodeId &node, int column, int &value);
   virtual bool get_field(const bec::NodeId &node, int column, long long &value);
   virtual bool get_field(const bec::NodeId &node, int column, double &value);
