@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,8 +23,7 @@
 #include "base/string_utilities.h"
 
 using namespace wbfig;
-using namespace MySQL::Drawing;
-using namespace MySQL::Geometry;
+using namespace base;
 
 mdc::FontSpec parse_font_spec(const std::string &font)
 {
@@ -80,7 +79,7 @@ Titlebar::Titlebar(mdc::Layer *layer, FigureEventHub *hub, BaseFigure *owner, bo
   if (expander)
   {
     _expander= new mdc::Button(layer, mdc::ExpanderButton);
-    _expander->signal_activate().connect(sigc::mem_fun(this, &Titlebar::expand_toggled));
+    scoped_connect(_expander->signal_activate(),boost::bind(&Titlebar::expand_toggled, this));
     _expander->set_pen_color(Color(0.4, 0.4, 0.4));
     
     add(_expander, false, false);
@@ -175,7 +174,7 @@ bool Titlebar::get_expanded()
 
 void Titlebar::expand_toggled()
 {
-  _signal_expand_toggle.emit(_expander->get_active());
+  _signal_expand_toggle(_expander->get_active());
 }
 
 
@@ -445,7 +444,7 @@ void BaseFigure::invalidate_min_sizes(mdc::CanvasItem *item)
 {
   item->set_needs_relayout();
   if (dynamic_cast<mdc::Layouter*>(item))
-    dynamic_cast<mdc::Layouter*>(item)->foreach(sigc::ptr_fun(&BaseFigure::invalidate_min_sizes));
+    dynamic_cast<mdc::Layouter*>(item)->foreach(std::ptr_fun(&BaseFigure::invalidate_min_sizes));
 }
 
 
@@ -472,7 +471,7 @@ bool BaseFigure::on_drag_handle(mdc::ItemHandle *handle, const Point &pos, bool
   if (!dragging)
   {
     _resizing= false;
-    _signal_resize.emit(_initial_bounds);
+    _signal_resize(_initial_bounds);
   }
   if (flag)
     set_fixed_size(get_size());
@@ -551,7 +550,7 @@ BaseFigure::ItemList::iterator BaseFigure::sync_next(mdc::Box &box, ItemList &li
 
     list.insert(iter, item); // insert before
 
-    _signal_item_added.emit(item);
+    _signal_item_added(item);
 
     return iter; // return same
   }
