@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,17 +36,17 @@ class FigureEventHub
 {
 public:
   virtual bool figure_click(const model_ObjectRef &owner, mdc::CanvasItem *target, 
-    const MySQL::Geometry::Point &point, mdc::MouseButton button, mdc::EventState state)= 0;
+    const base::Point &point, mdc::MouseButton button, mdc::EventState state)= 0;
   virtual bool figure_double_click(const model_ObjectRef &owner, mdc::CanvasItem *target,
-    const MySQL::Geometry::Point &point, mdc::MouseButton button, mdc::EventState state)= 0;
+    const base::Point &point, mdc::MouseButton button, mdc::EventState state)= 0;
   virtual bool figure_button_press(const model_ObjectRef &owner, mdc::CanvasItem *target,
-    const MySQL::Geometry::Point &point, mdc::MouseButton button, mdc::EventState state)= 0;
+    const base::Point &point, mdc::MouseButton button, mdc::EventState state)= 0;
   virtual bool figure_button_release(const model_ObjectRef &owner, mdc::CanvasItem *target,
-    const MySQL::Geometry::Point &point, mdc::MouseButton button, mdc::EventState state)= 0;
+    const base::Point &point, mdc::MouseButton button, mdc::EventState state)= 0;
   virtual bool figure_enter(const model_ObjectRef &owner, mdc::CanvasItem *target,
-    const MySQL::Geometry::Point &point)= 0;
+    const base::Point &point)= 0;
   virtual bool figure_leave(const model_ObjectRef &owner, mdc::CanvasItem *target,
-    const MySQL::Geometry::Point &point)= 0;
+    const base::Point &point)= 0;
 };
 
 class BaseFigure;
@@ -63,12 +63,12 @@ public:
   
   inline const std::string &get_title() const { return _icon_text.get_text(); }
 
-  void set_color(const MySQL::Drawing::Color &color);
-  void set_text_color(const MySQL::Drawing::Color &color);
+  void set_color(const base::Color &color);
+  void set_text_color(const base::Color &color);
   void set_font(const mdc::FontSpec &font);
   const mdc::FontSpec &get_font() { return _icon_text.get_font(); }
   void set_rounded(mdc::CornerMask corners);
-  void set_border_color(const MySQL::Drawing::Color &color);
+  void set_border_color(const base::Color &color);
 
   void set_expanded(bool flag);
   bool get_expanded();
@@ -86,9 +86,9 @@ protected:
   mdc::IconTextFigure _icon_text;
   mdc::Button *_expander;
 
-  MySQL::Drawing::Color _back_color;
+  base::Color _back_color;
   mdc::CornerMask _corners;
-  MySQL::Drawing::Color _border_color;
+  base::Color _border_color;
 
   boost::signals2::signal<void (bool)> _signal_expand_toggle; 
   
@@ -96,16 +96,16 @@ protected:
 
   virtual void render(mdc::CairoCtx *cr);
 
-  virtual bool on_click(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_click(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_double_click(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_double_click(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_button_press(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_button_press(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_button_release(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_button_release(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_enter(mdc::CanvasItem *target, const MySQL::Geometry::Point &point);
-  virtual bool on_leave(mdc::CanvasItem *target, const MySQL::Geometry::Point &point);
+  virtual bool on_enter(mdc::CanvasItem *target, const base::Point &point);
+  virtual bool on_leave(mdc::CanvasItem *target, const base::Point &point);
 };
 
 
@@ -116,16 +116,16 @@ class WBPUBLICBACKEND_PUBLIC_FUNC CaptionFigure : public mdc::TextFigure
   FigureEventHub *_hub; 
   model_Object *_owner_object;
 
-  virtual bool on_click(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_click(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_double_click(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_double_click(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_button_press(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_button_press(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_button_release(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_button_release(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_enter(mdc::CanvasItem *target, const MySQL::Geometry::Point &point);
-  virtual bool on_leave(mdc::CanvasItem *target, const MySQL::Geometry::Point &point);
+  virtual bool on_enter(mdc::CanvasItem *target, const base::Point &point);
+  virtual bool on_leave(mdc::CanvasItem *target, const base::Point &point);
 
 public:
   CaptionFigure(mdc::Layer *layer, FigureEventHub *hub, model_Object *owner);
@@ -143,20 +143,20 @@ class WBPUBLICBACKEND_PUBLIC_FUNC FigureItem : public mdc::IconTextFigure
   std::string _object_id;
   bool _dirty;
 
-  virtual bool on_click(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_click(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_double_click(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_double_click(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_button_press(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_button_press(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_button_release(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_button_release(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_enter(mdc::CanvasItem *target, const MySQL::Geometry::Point &point);
-  virtual bool on_leave(mdc::CanvasItem *target, const MySQL::Geometry::Point &point);
+  virtual bool on_enter(mdc::CanvasItem *target, const base::Point &point);
+  virtual bool on_leave(mdc::CanvasItem *target, const base::Point &point);
   
   virtual void draw_state(mdc::CairoCtx *cr);
-  virtual MySQL::Geometry::Point get_intersection_with_line_to(const MySQL::Geometry::Point &p);
-  virtual MySQL::Geometry::Rect get_root_bounds() const;
+  virtual base::Point get_intersection_with_line_to(const base::Point &p);
+  virtual base::Rect get_root_bounds() const;
 
 public:
   FigureItem(mdc::Layer *layer, FigureEventHub *hub, BaseFigure *owner);
@@ -177,17 +177,17 @@ public:
 
   // default implementation just sets background color
   virtual void unset_color();
-  virtual void set_color(const MySQL::Drawing::Color &color);
+  virtual void set_color(const base::Color &color);
 
   virtual void set_title_font(const mdc::FontSpec &font) {}
   virtual void set_content_font(const mdc::FontSpec &font);
   
-  virtual void highlight(const MySQL::Drawing::Color *color= 0);
+  virtual void highlight(const base::Color *color= 0);
   virtual void unhighlight();
 
   virtual void set_allow_manual_resizing(bool flag);
 
-  boost::signals2::signal<void (const MySQL::Geometry::Rect&)>* signal_interactive_resize() { return &_signal_resize; }
+  boost::signals2::signal<void (base::Rect)>* signal_interactive_resize() { return &_signal_resize; }
 
   boost::signals2::signal<void (FigureItem*)>* signal_item_added() { return &_signal_item_added; }
 
@@ -204,10 +204,10 @@ protected:
   FigureEventHub *_hub;
   model_Object *_represented_object;
   
-  boost::signals2::signal<void (const MySQL::Geometry::Rect&)> _signal_resize;
+  boost::signals2::signal<void (base::Rect)> _signal_resize;
   boost::signals2::signal<void (FigureItem*)> _signal_item_added;
 
-  MySQL::Geometry::Rect _initial_bounds;
+  base::Rect _initial_bounds;
   mdc::FontSpec _content_font;
   bool _manual_resizing;
   bool _resizing;
@@ -230,18 +230,18 @@ protected:
 
   virtual void end_sync(mdc::Box &box, ItemList &list, ItemList::iterator iter);
 
-  virtual bool on_click(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_click(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_double_click(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_double_click(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_button_press(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_button_press(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_button_release(mdc::CanvasItem *target, const MySQL::Geometry::Point &point,
+  virtual bool on_button_release(mdc::CanvasItem *target, const base::Point &point,
     mdc::MouseButton button, mdc::EventState state);
-  virtual bool on_enter(mdc::CanvasItem *target, const MySQL::Geometry::Point &point);
-  virtual bool on_leave(mdc::CanvasItem *target, const MySQL::Geometry::Point &point);
+  virtual bool on_enter(mdc::CanvasItem *target, const base::Point &point);
+  virtual bool on_leave(mdc::CanvasItem *target, const base::Point &point);
   
-  virtual bool on_drag_handle(mdc::ItemHandle *handle, const MySQL::Geometry::Point &pos, bool dragging);
+  virtual bool on_drag_handle(mdc::ItemHandle *handle, const base::Point &pos, bool dragging);
 };
 
   
@@ -255,10 +255,10 @@ class WBPUBLICBACKEND_PUBLIC_FUNC ShrinkableBox : public mdc::Box
   bool _manual_resizing;
   
   virtual void render(mdc::CairoCtx *cr);
-  virtual void resize_to(const MySQL::Geometry::Size &size);
+  virtual void resize_to(const base::Size &size);
 public:
   ShrinkableBox(mdc::Layer *layer, mdc::Box::Orientation orientation);
-  virtual MySQL::Geometry::Size calc_min_size();
+  virtual base::Size calc_min_size();
   
   void set_item_count_limit(int limit);
   
