@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,15 +30,13 @@
 #include "base/string_utilities.h"
 
 using namespace base;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
 
 model_Figure::ImplData::ImplData(model_Figure *owner)
   : model_Object::ImplData(owner), _realizing(false)
 {
   _connected_update_options= false;
   
-  owner->signal_changed().connect(sigc::mem_fun(this, &model_Figure::ImplData::member_changed));
+  scoped_connect(owner->signal_changed(),boost::bind(&model_Figure::ImplData::member_changed, this, _1, _2));
 }
 
 
@@ -99,7 +97,7 @@ void model_Figure::ImplData::member_changed(const std::string &name, const grt::
   {
     if (_in_view)
     {
-      self()->owner()->signal_refreshDisplay().emit(self());
+      (*self()->owner()->signal_refreshDisplay())(self());
     }
   }
   else if (name == "color")
@@ -289,13 +287,13 @@ void model_Figure::ImplData::finish_realize()
   if (!_connected_update_options)
   {
     _connected_update_options= true;
-    self()->owner()->owner()->get_data()->signal_options_changed().connect(sigc::mem_fun(this, &model_Figure::ImplData::update_options));
+    scoped_connect(self()->owner()->owner()->get_data()->signal_options_changed(),boost::bind(&model_Figure::ImplData::update_options, this, _1));
   }
 
   figure->set_tag(self()->id());
 
-  figure->signal_bounds_changed().connect(sigc::mem_fun(this, &model_Figure::ImplData::figure_bounds_changed));
-  figure->signal_interactive_resize().connect(sigc::mem_fun(this, &model_Figure::ImplData::figure_resized));
+  scoped_connect(figure->signal_bounds_changed(),boost::bind(&model_Figure::ImplData::figure_bounds_changed, this, _1));
+  scoped_connect(figure->signal_interactive_resize(),boost::bind(&model_Figure::ImplData::figure_resized, this, _1));
 
   pos.x= *self()->_left;
   pos.y= *self()->_top;
