@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -60,9 +60,9 @@ protected:
   boost::shared_ptr<boost::signals2::shared_connection_block> _refresh_conn_block;
   boost::shared_ptr<boost::signals2::shared_connection_block> _changed_conn_block;
 
-  boost::signals2::connection _table_fk_conn;
-  boost::signals2::connection _refresh_conn;
-  boost::signals2::connection _changed_conn;
+  boost::signals2::scoped_connection _table_fk_conn;
+  boost::signals2::scoped_connection _refresh_conn;
+  boost::signals2::scoped_connection _changed_conn;
   
   std::vector<int> _list_listeners;
   
@@ -93,7 +93,7 @@ public:
   db_ColumnRef get_column_at(mdc::CanvasItem *item);
   db_IndexRef get_index_at(mdc::CanvasItem *item);
   
-  void set_column_highlighted(const db_ColumnRef &column, const MySQL::Drawing::Color *color= 0);
+  void set_column_highlighted(const db_ColumnRef &column, const base::Color *color= 0);
   void set_column_unhighlighted(const db_ColumnRef &column);
   
   virtual mdc::CanvasItem* get_canvas_item() const { return _figure; }
