@@ -24,6 +24,7 @@
 #include "workbench/wb_context.h"
 #include "workbench/wb_context_ui.h"
 #include "model/wb_model_diagram_form.h"
+#include "model/wb_context_model.h"
 #include "workbench/wb_history_tree.h"
 #include "model/wb_diagram_options.h"
 #include "model/wb_user_datatypes.h"
@@ -132,69 +133,35 @@ public:
 
 // ----------------------------------------------------------------------------
 
-
 public ref class WbOptions
 {
-  String^ basedir;
-  String^ plugin_search_path;
-  String^ struct_search_path;
-  String^ module_search_path;
-  String^ library_search_path;
-  String^ user_data_dir;
-  String^ open_at_startup;
-  String^ open_at_startup_type;
-  String^ run_at_startup;
-  String^ run_at_startup_lang;
-  bool force_sw_rendering;
-  bool force_opengl_rendering;
-  bool quit_when_done;
+private:
+  wb::WBOptions* inner; 
 
 public:
-  WbOptions(String^ basedir, String^ plugin_search_path, String^ struct_search_path,
-    String^ module_search_path, String^ library_search_path, String^ user_data_dir, String^ open_at_startup, 
-    String^ open_at_startup_type, String^ run_at_startup, String^ run_at_startup_lang,
-    bool force_sw_rendering, bool force_opengl_rendering, bool quit_when_done) 
-    : basedir(basedir),
-    plugin_search_path(plugin_search_path),
-    struct_search_path(struct_search_path),
-    module_search_path(module_search_path),
-    library_search_path(library_search_path),
-    user_data_dir(user_data_dir),
-    open_at_startup(open_at_startup),
-    open_at_startup_type(open_at_startup_type),
-    run_at_startup(run_at_startup),
-    run_at_startup_lang(run_at_startup_lang),
-    force_sw_rendering(force_sw_rendering),
-    force_opengl_rendering(force_opengl_rendering),
-    quit_when_done(quit_when_done)
+  WbOptions(String^ baseDir, String^ userDir);
+
+  wb::WBOptions* get_unmanaged_object() { return inner; };
+  bool parse_args(array<String^>^ args, String^ app_path);
+
+  property bool ShowSplash
   {
-    Logger::LogDebug(DOMAIN_WB_CONTEXT_WRAPPER, 1, String::Format("Creating options wrapper "
-      "with these values:\n\tbase dir: {0}\n\tplugin path: {1}\n\tstruct path: {2}\n\tmodule path: {3}\n\t"
-      "library path: {4}\n\tuser data dir: {5}\n\topen at start: {6}\n\topen type: {7}\n\trun at startup: {8}\n\t"
-      "run type: {9}\n\tForce SW rendering: {10}\n\tForce OpenGL: {11}\n\tquit when done: {12}\n",
-      basedir, plugin_search_path, struct_search_path, module_search_path, library_search_path,
-      user_data_dir, open_at_startup, open_at_startup_type, run_at_startup, run_at_startup_lang,
-      force_sw_rendering, force_opengl_rendering, quit_when_done
-    ));
+    bool get() { return !inner->no_logo; }
   }
 
-  ::wb::WBOptions *get_unmanaged_object()
+  property bool Verbose
   {
-    ::wb::WBOptions *opt= new ::wb::WBOptions;
-    opt->basedir= NativeToCppString(basedir);
-    opt->plugin_search_path= NativeToCppString(plugin_search_path);
-    opt->struct_search_path= NativeToCppString(struct_search_path);
-    opt->module_search_path= NativeToCppString(module_search_path);
-    opt->library_search_path= NativeToCppString(library_search_path);
-    opt->user_data_dir= NativeToCppString(user_data_dir);
-    opt->open_at_startup= NativeToCppString(open_at_startup);
-    opt->open_at_startup_type= NativeToCppString(open_at_startup_type);
-    opt->run_at_startup= NativeToCppString(run_at_startup);
-    opt->run_language= NativeToCppString(run_at_startup_lang);
-    opt->force_sw_rendering= force_sw_rendering;
-    opt->force_opengl_rendering= force_opengl_rendering;
-    opt->quit_when_done= quit_when_done;
-    return opt;
+    bool get() { return !inner->verbose; }
+  }
+
+  property String^ OpenAtStartup
+  {
+    String^ get();
+  }
+
+  property String^ OpenAtStartupType
+  {
+    String^ get();
   }
 };
 
