@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,7 +22,7 @@
 #define __ACTION_LIST_WR_H__
 
 #include "grt/action_list.h"
-#include "SigCSlot.h"
+#include "DelegateWrapper.h"
 
 namespace MySQL {
 namespace Grt {
