@@ -1,3 +1,22 @@
+/* 
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 #ifndef __DB_OBJECT_FILTER_BE_WRAPPER_H__
 #define __DB_OBJECT_FILTER_BE_WRAPPER_H__
 
@@ -6,7 +25,6 @@
 #ifdef _MSC_VER
 using namespace System;
 using namespace System::Collections::Generic;
-using namespace System::Runtime::InteropServices;
 #endif
 
 //#pragma make_public(::bec::DBObjectFilterBE)(removed because it throws some metadata error on link -alfredo)
