@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,8 +24,6 @@
 #include "GrtShell.h"
 #include "GrtValueTree.h"
 #include "GrtValueInspector.h"
-#include "GrtStructsTree.h"
-#include "GrtModulesTree.h"
 
 namespace MySQL {
   namespace Grt {
@@ -92,16 +90,6 @@ namespace MySQL {
           NativeToCppString(libs_sp));
       }
 
-      void set_progress_callback(BoolStringStringFloatDelegate^ dt)
-      {
-        progress_callback_delegate= dt;
-        progress_callback_wrapper_delegate = 
-          gcnew BoolStringStringFloatWrapperDelegate(this, &GrtManager::progress_callback_wrapper);
-        IntPtr ip = Marshal::GetFunctionPointerForDelegate(progress_callback_wrapper_delegate);
-        BOOL_STRING_STRING_FLOAT_HANDLER_CB cb = static_cast<BOOL_STRING_STRING_FLOAT_HANDLER_CB>(ip.ToPointer());
-        inner->set_progress_callback(boost::bind(cb, _1, _2, _3));
-      }
-
       GRT^ get_grt() 
       { 
         if(managed_grt == nullptr)
@@ -135,12 +123,6 @@ namespace MySQL {
         return gcnew GrtValueInspector(inner->get_new_value_inspector(value->get_unmanaged_object(), process_editas_flag), false);
       }
 
-      GrtStructsTree^ get_shared_structs_tree()
-      { return gcnew GrtStructsTree(inner->get_shared_structs_tree()); }
-
-      GrtModulesTree^ get_shared_modules_tree()
-      { return gcnew GrtModulesTree(inner->get_shared_modules_tree()); }
-
       void terminate() { inner->terminate(); };
       bool terminated() { return inner->terminated(); };
       void resetTermination() { inner->reset_termination(); };
