@@ -1,13 +1,28 @@
 /*
-   Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
 
-   The MySQL Connector/C++ is licensed under the terms of the GPL
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPL as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
+
+
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -338,8 +353,9 @@ MySQL_Prepared_Statement::MySQL_Prepared_Statement(
 			boost::shared_ptr< NativeAPI::NativeStatementWrapper > & s, sql::Connection * conn,
 			sql::ResultSet::enum_type rset_type, boost::shared_ptr< MySQL_DebugLogger > & log
 		)
-	:connection(conn), proxy(s), isClosed(false), warningsHasBeenLoaded(true), logger(log),
-		resultset_type(rset_type), result_bind(new MySQL_ResultBind(proxy, logger))
+	:connection(conn), proxy(s), isClosed(false), warningsHaveBeenLoaded(true), logger(log),
+		resultset_type(rset_type), result_bind(new MySQL_ResultBind(proxy, logger)),
+		warningsCount(0)
 
 {
 	CPP_ENTER("MySQL_Prepared_Statement::MySQL_Prepared_Statement");
@@ -407,7 +423,9 @@ MySQL_Prepared_Statement::do_query()
 		sql::mysql::util::throwSQLException(*proxy.get());
 	}
 
-	warningsHasBeenLoaded= false;
+	warningsCount= proxy->warning_count();
+
+	warningsHaveBeenLoaded= false;
 }
 /* }}} */
 
@@ -1143,10 +1161,11 @@ MySQL_Prepared_Statement::getWarnings()
 	CPP_INFO_FMT("this=%p", this);
 	checkClosed();
 
-	if (!warningsHasBeenLoaded)
+	if (!warningsHaveBeenLoaded)
 	{
-		warnings.reset( loadMysqlWarnings(connection) );
-		warningsHasBeenLoaded= true;
+		if (warningsCount)
+		warnings.reset( loadMysqlWarnings(connection, warningsCount) );
+		warningsHaveBeenLoaded= true;
 	}
 
 	return warnings.get();
