@@ -1488,7 +1488,7 @@ private:
   unsigned int maxItemWidth;
   unsigned int aveCharWidth;
   unsigned int maxIconWidth;
-  Font font;
+  FontAlias font;
   int maxWidth;
 
   NSTableView* table;
@@ -1506,7 +1506,7 @@ public:
     doubleClickAction(NULL), doubleClickActionData(NULL)
   {
   }
-  ~ListBoxImpl() {}
+  ~ListBoxImpl() { ClearRegisteredImages(); }
 
   // ListBox methods
   void SetFont(Font& font);
@@ -1608,6 +1608,7 @@ void ListBoxImpl::Create(Window& /*parent*/, int /*ctrlID*/, Scintilla::Point pt
     defer: NO];
   [winLB setLevel:NSFloatingWindowLevel];
   [winLB setHasShadow:YES];
+  [winLB setAlphaValue: 0.9];
   scroller = [NSScrollView alloc];
   NSRect scRect = NSMakeRect(0, 0, lbRect.size.width, lbRect.size.height);
   [scroller initWithFrame: scRect];
@@ -1640,14 +1641,15 @@ void ListBoxImpl::Create(Window& /*parent*/, int /*ctrlID*/, Scintilla::Point pt
 
 void ListBoxImpl::SetFont(Font& font_)
 {
-  font.SetID(font_.GetID());
+  font.MakeAlias(font_);
+  
   // NSCell setFont takes an NSFont* rather than a CTFontRef but they
   // are the same thing toll-free bridged.
   QuartzTextStyle* style = reinterpret_cast<QuartzTextStyle*>(font_.GetID());
   NSFont *pfont = (NSFont *)style->getFontRef();
   [[colText dataCell] setFont: pfont];
-  CGFloat itemHeight = lround([pfont ascender] - [pfont descender]);
-  [table setRowHeight:itemHeight];
+  CGFloat itemHeight = ceil([pfont boundingRectForFont].size.height);
+  [table setRowHeight: itemHeight];
 }
 
 void ListBoxImpl::SetAverageCharWidth(int width)
