@@ -67,13 +67,12 @@ static const KeyToCommand macMapDefault[] =
   {SCK_LEFT,      SCI_ALT,    SCI_WORDLEFT},
   {SCK_LEFT,      SCI_META,   SCI_WORDLEFT},
   {SCK_LEFT,      SCI_SMETA,  SCI_WORDLEFTEXTEND},
-  {SCK_LEFT,      SCI_ASHIFT, SCI_CHARLEFTRECTEXTEND},
+  {SCK_LEFT,      SCI_ASHIFT, SCI_WORDLEFTEXTEND},
   {SCK_RIGHT,     SCI_NORM,   SCI_CHARRIGHT},
   {SCK_RIGHT,     SCI_SHIFT,  SCI_CHARRIGHTEXTEND},
   {SCK_RIGHT,     SCI_ALT,    SCI_WORDRIGHT},
   {SCK_RIGHT,     SCI_META,   SCI_WORDRIGHT},
-  {SCK_RIGHT,     SCI_SMETA,  SCI_WORDRIGHTEXTEND},
-  {SCK_RIGHT,     SCI_ASHIFT, SCI_CHARRIGHTRECTEXTEND},
+  {SCK_RIGHT,     SCI_ASHIFT, SCI_WORDRIGHTEXTEND},
   {'/',           SCI_CTRL,   SCI_WORDPARTLEFT},
   {'/',           SCI_CSHIFT, SCI_WORDPARTLEFTEXTEND},
   {'\\',          SCI_CTRL,   SCI_WORDPARTRIGHT},
@@ -107,7 +106,7 @@ static const KeyToCommand macMapDefault[] =
   {SCK_BACK,      SCI_NORM,   SCI_DELETEBACK},
   {SCK_BACK,      SCI_SHIFT,  SCI_DELETEBACK},
   {SCK_BACK,      SCI_CTRL,   SCI_DELWORDLEFT},
-  {SCK_BACK,      SCI_ALT,    SCI_UNDO},
+  {SCK_BACK,      SCI_ALT,    SCI_DELWORDLEFT},
   {SCK_BACK,      SCI_CSHIFT, SCI_DELLINELEFT},
   {'z',           SCI_CMD,    SCI_UNDO},
   {'z',           SCI_SCMD,   SCI_REDO},
@@ -156,6 +155,7 @@ static const KeyToCommand macMapDefault[] =
 
 - (void) dealloc
 {
+  [[NSNotificationCenter defaultCenter] removeObserver: self];
   [notificationQueue release];
   [super dealloc];
 }
@@ -208,8 +208,9 @@ static const KeyToCommand macMapDefault[] =
 
 ScintillaCocoa::ScintillaCocoa(NSView* view)
 {
-  wMain= [view retain];
-  timerTarget = [[[TimerTarget alloc] init: this] retain];
+  //wMain= [view retain];// Don't retain since we're owned by view, which would cause a cycle
+  wMain = view;
+  timerTarget = [[TimerTarget alloc] init: this];
   Initialise();
 }
 
@@ -219,8 +220,9 @@ ScintillaCocoa::~ScintillaCocoa()
 {
   SetTicking(false);
   [timerTarget release];
-  NSView* container = ContentView();
-  [container release];
+  //container is no longer retained as usual pattern is that container owns content, not the opposite
+  //NSView* container = ContentView();
+  //[container release];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -831,8 +833,10 @@ NSPoint ScintillaCocoa::GetCaretPosition()
 {
   NSPoint result;
 
-  result.y = pdoc->LineFromPosition(sel.RangeMain().caret.Position());
-  result.x = sel.RangeMain().caret.Position() - pdoc->LineStart(result.y);
+  int position = sel.RangeMain().caret.Position();
+  result.y = pdoc->LineFromPosition(position);
+  result.x = pdoc->GetColumn(position);
+  
   return result;
 }
 
