@@ -4,7 +4,8 @@
  *
  * Created by Mike Lischke.
  *
- * Copyright 2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright 2009, 2011 Sun Microsystems, Inc. All rights reserved.
  * This file is dual licensed under LGPL v2.1 and the Scintilla license (http://www.scintilla.org/License.txt).
  */
 
@@ -20,6 +21,11 @@
 @class ScintillaView;
 
 extern NSString *SCIUpdateUINotification;
+extern NSString *SCIAutoCompleteNotification;
+
+@protocol ScintillaNotificationProtocol
+- (void)notification: (Scintilla::SCNotification*)notification;
+@end
 
 /**
  * InnerView is the Cocoa interface to the Scintilla backend. It handles text input and
@@ -39,6 +45,9 @@ extern NSString *SCIUpdateUINotification;
   int mLastPosition;
 }
 
+@property (nonatomic, assign) ScintillaView* owner;
+
+
 - (void) dealloc;
 - (void) removeMarkedText;
 - (void) setCursor: (Scintilla::Window::Cursor) cursor;
@@ -46,7 +55,6 @@ extern NSString *SCIUpdateUINotification;
 - (BOOL) canUndo;
 - (BOOL) canRedo;
 
-@property (retain) ScintillaView* owner;
 @end
 
 @interface ScintillaView : NSView <InfoBarCommunicator>
@@ -69,8 +77,14 @@ extern NSString *SCIUpdateUINotification;
   NSView <InfoBarCommunicator>* mInfoBar;
   BOOL mInfoBarAtTop;
   int mInitialInfoBarWidth;
+
+  id<ScintillaNotificationProtocol> mDelegate;
 }
 
+@property (nonatomic, assign) Scintilla::ScintillaCocoa* backend;
+@property (nonatomic, assign) NSObject* owner;
+@property (nonatomic, assign) id<ScintillaNotificationProtocol> delegate;
+
 - (void) dealloc;
 - (void) layout;
 
@@ -144,6 +158,4 @@ extern NSString *SCIUpdateUINotification;
                  wholeWord: (BOOL) wholeWord
                      doAll: (BOOL) doAll;
 
-@property Scintilla::ScintillaCocoa* backend;
-@property (retain) NSObject* owner;
 @end
