@@ -4,6 +4,7 @@
  *
  * Created by Mike Lischke.
  *
+ * Copyright 2011, 2012, Oracle and/or its affiliates. All rights reserved.
  * Copyright 2009, 2011 Sun Microsystems, Inc. All rights reserved.
  * This file is dual licensed under LGPL v2.1 and the Scintilla license (http://www.scintilla.org/License.txt).
  */
@@ -21,6 +22,7 @@ static NSCursor* waitCursor;
 
 NSString *SCIUpdateUINotification = @"SCIUpdateUI";
 NSString *SCIMarginClickNotification = @"SCIMarginClick";
+NSString *SCIAutoCompleteNotification = @"SCIAutoCompleteNotification";
 
 @implementation InnerView
 
@@ -214,8 +216,8 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
 
 - (void) doCommandBySelector: (SEL) selector
 {
-  if ([self respondsToSelector: @selector(selector)])
-    [self performSelector: selector withObject: nil];
+ // if ([self respondsToSelector: selector])
+//    [self performSelector: selector withObject: nil];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -622,6 +624,7 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
 
 @synthesize backend = mBackend;
 @synthesize owner   = mOwner;
+@synthesize delegate = mDelegate;
 
 /**
  * ScintiallView is a composite control made from an NSView and an embedded NSView that is
@@ -726,6 +729,8 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
 /**
  * Notification function used by Scintilla to call us back (e.g. for handling clicks on the 
  * folder margin or changes in the editor).
+ * A delegate can be set to receive all notifications. If set no handling takes place here, except
+ * for action pertaining to internal stuff (like the info bar).
  */
 static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wParam, uintptr_t lParam)
 {
@@ -740,6 +745,14 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
       SCNotification* scn = reinterpret_cast<SCNotification*>(lParam);
       ScintillaCocoa *psc = reinterpret_cast<ScintillaCocoa*>(scn->nmhdr.hwndFrom);
       editor = reinterpret_cast<InnerView*>(psc->ContentView()).owner;
+
+      if (editor.delegate != nil)
+      {
+        [editor.delegate notification: scn];
+        if (scn->nmhdr.code != SCN_ZOOM && scn->nmhdr.code != SCN_UPDATEUI)
+          return;
+      }
+      
       switch (scn->nmhdr.code)
       {
         case SCN_MARGINCLICK:
@@ -794,7 +807,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
           [editor sendNotification: SCIUpdateUINotification];
           [editor sendNotification: NSTextViewDidChangeSelectionNotification];
           break;
-      }
+        }
       }
       break;
     }
@@ -877,7 +890,9 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
 
 - (void) dealloc
 {
-  [mInfoBar release];
+  [[NSNotificationCenter defaultCenter] removeObserver: self];
+  // infobar is a subview and there are no extra retains
+  //[mInfoBar release];
   delete mBackend;
   [super dealloc];
 }
