@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2012 Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,9 +24,13 @@
 #include "base/string_utilities.h"
 #include "base/geometry.h"
 #include "base/drawing.h"
+#include "base/log.h"
 
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
+#include "mforms/utilities.h"
+
+using namespace base;
+
+DEFAULT_LOG_DOMAIN("AdvancedSidebar");
 
 #include "advanced_sidebar.h"
 
@@ -37,6 +41,7 @@ using namespace std;
 using namespace wb;
 using namespace mforms;
 
+
 #ifdef _WIN32
   #define SIDEBAR_FONT "Segoe UI"
   #define SIDEBAR_TITLE_FONT_SIZE 11
@@ -60,12 +65,12 @@ using namespace mforms;
 
 //----------------- SidebarSection -----------------------------------------------------------------
 
-SidebarEntry::SidebarEntry(const string& title, const string& icon, const string& command, bool as_link)
+SidebarEntry::SidebarEntry(const string& title, const string& icon, const string& command, TaskEntryType type)
 {
   _title = title;
   _icon = Utilities::load_icon(icon);
   _command = command;
-  _is_link = as_link;
+  _type = type;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -76,6 +81,17 @@ SidebarEntry::~SidebarEntry()
     cairo_surface_destroy(_icon);
 }
 
+
+//--------------------------------------------------------------------------------------------------
+
+void SidebarEntry::set_title(const std::string &title, const std::string &icon)
+{
+  _title = title;
+  if (_icon)
+    cairo_surface_destroy(_icon);
+  _icon = Utilities::load_icon(icon);
+}
+
 //--------------------------------------------------------------------------------------------------
 
 void SidebarEntry::paint(cairo_t* cr, Rect bounds, bool hot, bool active, const Color& selection_color)
@@ -149,6 +165,9 @@ SidebarSection::SidebarSection(AdvancedSidebar* owner, const std::string& title,
   _expand_text_visible = false;
   _expand_text_width = 0;
   _expand_text_active = false;
+  _refresh_hot = false;
+  _refresh_down = false;
+  _show_busy = false;
 
   if (refreshable)
   {
@@ -160,6 +179,12 @@ SidebarSection::SidebarSection(AdvancedSidebar* owner, const std::string& title,
   }
   else
     _refresh_icon = NULL;
+
+#ifdef __APPLE__
+  _busy_icon = Utilities::load_icon("busy_sidebar_mac.png");
+#else
+  _busy_icon = Utilities::load_icon("busy_sidebar.png");
+#endif
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -169,6 +194,8 @@ SidebarSection::~SidebarSection()
   clear();
   if (_refresh_icon != NULL)
     cairo_surface_destroy(_refresh_icon);
+  if (_busy_icon != NULL)
+    cairo_surface_destroy(_busy_icon);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -278,13 +305,13 @@ void SidebarSection::toggle_expand()
 //--------------------------------------------------------------------------------------------------
 
 int SidebarSection::add_entry(const std::string& title, const std::string& icon,
-  const std::string& command, bool as_link)
+  const std::string& command, TaskEntryType type)
 {
   int result = find_entry(title);
   if (result > -1)
     return result;
 
-  SidebarEntry* entry = new SidebarEntry(title, icon, command, as_link);
+  SidebarEntry* entry = new SidebarEntry(title, icon, command, type);
   _entries.push_back(entry);
   set_layout_dirty(true);
 
@@ -293,6 +320,22 @@ int SidebarSection::add_entry(const std::string& title, const std::string& icon,
 
 //--------------------------------------------------------------------------------------------------
 
+void SidebarSection::set_entry_text(int index, const std::string& title, const std::string& icon)
+{
+  if (index >= 0 && index < (int)_entries.size())
+    _entries[index]->set_title(title, icon);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void SidebarSection::mark_busy(bool busy)
+{
+  _show_busy = busy;
+  set_needs_repaint();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void SidebarSection::remove_entry(const std::string& entry)
 {
   int index = find_entry(entry);
@@ -340,13 +383,6 @@ bool SidebarSection::select(const std::string& title)
 
 //--------------------------------------------------------------------------------------------------
 
-//void SidebarSection::set_selection_color(const Color* color)
-//{
-//  _selection_color = color;
-//}
-
-//--------------------------------------------------------------------------------------------------
-
 void draw_header_text(cairo_t* cr, Rect &bounds, const std::string& text, bool active)
 {
   // Draw heading twice. Once like a white shadow and once normal.
@@ -379,10 +415,41 @@ void SidebarSection::repaint(cairo_t *cr, int areax, int areay, int areaw, int a
   cairo_set_font_size(cr, SIDEBAR_TITLE_FONT_SIZE);
   draw_header_text(cr, bounds, _title, false);
 
-  if (_refresh_icon != NULL)
+  cairo_surface_t *icon = _show_busy ? _busy_icon : _refresh_icon;
+  if (icon != NULL)
   {
     int image_width = cairo_image_surface_get_width(_refresh_icon);
-    cairo_set_source_surface(cr, _refresh_icon, bounds.size.width - image_width, SECTION_TOP_SPACING);
+    int image_height = cairo_image_surface_get_height(_refresh_icon);
+    double image_left = bounds.size.width - image_width + 0.5;
+    double image_top = SECTION_TOP_SPACING + 0.5;
+    if (_refresh_hot || _refresh_down)
+    {
+      // Draw a bezel shape under the icon.
+      double radius = 3;
+      cairo_new_sub_path(cr);
+      cairo_arc(cr, image_left + image_width - radius, image_top + radius, radius, -M_PI / 2, 0);
+      cairo_arc(cr, image_left + image_width - radius, image_top + image_height - radius, radius, 0, M_PI / 2);
+      cairo_arc(cr, image_left + radius, image_top + image_height - radius, radius, M_PI / 2, M_PI);
+      cairo_arc(cr, image_left + radius, image_top + radius, radius, M_PI, 3 * M_PI / 2);
+      cairo_close_path(cr);
+
+      cairo_set_line_width(cr, 1);
+      if (_refresh_down)
+      {
+        cairo_set_source_rgba(cr, 0, 0, 0, 0.1);
+        cairo_fill_preserve(cr);
+        cairo_set_source_rgba(cr, 0, 0, 0, 0.15);
+      }
+      else
+      {
+        cairo_set_source_rgba(cr, 0, 0, 0, 0.05);
+        cairo_fill_preserve(cr);
+        cairo_set_source_rgba(cr, 0, 0, 0, 0.1);
+      }
+      cairo_stroke(cr);
+    }
+
+    cairo_set_source_surface(cr, icon, image_left + 0.5, image_top + 0.5);
     cairo_paint(cr);
 
     width -= image_width + SECTION_HEADER_REFRESH_SPACING;
@@ -419,11 +486,13 @@ void SidebarSection::repaint(cairo_t *cr, int areax, int areay, int areaw, int a
 
 void SidebarSection::mouse_leave()
 {
-  if (_hot_entry != NULL || _expand_text_visible || _expand_text_active)
+  if (_hot_entry != NULL || _expand_text_visible || _expand_text_active || _refresh_hot || _refresh_down)
   {
     _hot_entry= NULL;
     _expand_text_visible = false;
     _expand_text_active = false;
+    _refresh_down = false;
+    _refresh_hot = false;
     set_needs_repaint();
   }
 }
@@ -434,21 +503,48 @@ void SidebarSection::mouse_move(int x, int y)
 {
   bool need_refresh = false;
   
-  if (_expandable && (_expand_text_visible != (y < SECTION_TOP_SPACING + SECTION_HEADER_HEIGHT)))
+  if (y < SECTION_TOP_SPACING + SECTION_HEADER_HEIGHT)
   {
-    _expand_text_visible = y < SECTION_TOP_SPACING + SECTION_HEADER_HEIGHT;
-    _hot_entry= NULL;
-     need_refresh = true;
+    // Header area.
+    if (_expandable && !_expand_text_visible)
+    {
+      need_refresh = true;
+      _expand_text_visible = true;
+    }
+    if (_hot_entry != NULL)
+    {
+      need_refresh = true;
+      _hot_entry= NULL;
+    }
+
+    // Check refresh button.
+    if (_refresh_icon != NULL)
+    {
+      int width = get_width() - SECTION_SIDE_SPACING;
+      bool isHot = (x >= (width - cairo_image_surface_get_width(_refresh_icon)) && x < width);
+      if (isHot != _refresh_hot)
+      {
+        _refresh_hot = isHot;
+        need_refresh = true;
+      }
+    }
   }
   else
   {
+    if (_expand_text_visible || _refresh_hot)
+    {
+      _expand_text_visible = false;
+      _refresh_hot = false;
+      need_refresh = true;
+    }
+
     SidebarEntry* entry = entry_from_point(x, y);
     if (entry != _hot_entry)
     {
-      if (_hot_entry != NULL || entry->is_link())
+      if (_hot_entry != NULL || entry->type() == mforms::TaskEntryLink)
         need_refresh = true;
 
-      if (entry != NULL && entry->is_link())
+      if (entry != NULL && entry->type() == mforms::TaskEntryLink)
         _hot_entry = entry; // Only links can appear as hot entries.
       else
         _hot_entry = NULL;
@@ -465,20 +561,23 @@ void SidebarSection::mouse_down(int button, int x, int y)
 {
   if (button == MouseButtonLeft)
   {
-    int width = get_width() - SECTION_SIDE_SPACING;
-    if (_refresh_icon != NULL)
-      width -= cairo_image_surface_get_width(_refresh_icon) + SECTION_HEADER_REFRESH_SPACING;
-    if (_expand_text_visible && x >= (width - _expand_text_width) && x < width)
+    if (_refresh_hot)
     {
-      _expand_text_active = true;
+      _refresh_down = true;
       set_needs_repaint();
     }
     else
-    {
-      SidebarEntry* entry = entry_from_point(x, y);
-      if (entry && !entry->is_link())
-        set_selected(entry);
-    }
+      if (_expand_text_visible)
+      {
+        _expand_text_active = true;
+        set_needs_repaint();
+      }
+      else
+      {
+        SidebarEntry* entry = entry_from_point(x, y);
+        if (entry && entry->type() == mforms::TaskEntrySelectableItem)
+          set_selected(entry);
+      }
   }
 }
 
@@ -492,27 +591,19 @@ void SidebarSection::mouse_click(int button, int x, int y)
     {
       bool handled = false;
 
-      // _expand_text_visible implicitly indicates that the mouse is in the header area.
-      if (_expand_text_visible)
+      if (_expand_text_active)
       {
-        if (_expand_text_active)
+        handled = true;
+        toggle_expand();
+        _expand_text_active = false;
+        set_needs_repaint();
+      }
+      else
+        if (_refresh_down)
         {
           handled = true;
-          toggle_expand();
-          _expand_text_active = false;
-          set_needs_repaint();
+          _owner->refresh_clicked(this);
         }
-        else
-          if (_refresh_icon != NULL)
-          {
-            int width = get_width() - SECTION_SIDE_SPACING;
-            if (x >= (width - cairo_image_surface_get_width(_refresh_icon)) && x < width)
-            {
-              handled = true;
-              _owner->refresh_clicked(this);
-            }
-          }
-      }
 
       if (!handled)
       {
@@ -534,8 +625,18 @@ void SidebarSection::mouse_click(int button, int x, int y)
 
 //--------------------------------------------------------------------------------------------------
 
-void SidebarSection::mouse_double_click(int button, int x, int y)
+void SidebarSection::mouse_up(int button, int x, int y)
 {
+  switch (button)
+  {
+  case MouseButtonLeft:
+    if (_refresh_down)
+    {
+      _refresh_down = false;
+      set_needs_repaint();
+    }
+    break;
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -562,15 +663,36 @@ void SidebarSection::clear_selection()
 
 //----------------- AdvancedSidebar ----------------------------------------------------------------
 
-AdvancedSidebar::AdvancedSidebar()
-  : _schema_tree(TreeNoColumns | TreeNoBorder | TreeSidebar | TreeNoHeader), _is_model_owner(false),
-    _schema_box(false), _schema_tree_heading(NULL)
-{
-  setup_schema_tree();
+// implicitly initialize the adv. sidebar
+bool AdvancedSidebar::__init = init_factory_method();
 
-  //_schema_model = new LiveSchemaTree(_connection->get_grt());
-  //_schema_tree.set_model(_schema_model);
-  // _is_model_owner = true;
+bool AdvancedSidebar::init_factory_method()
+{
+  log_debug3("Initializing AdvancedSidebar factory method\n");
+  register_factory(&create_instance);
+  return true;
+}
+
+
+AdvancedSidebar::AdvancedSidebar()
+  : _schema_tree_heading(NULL), 
+    _new_schema_tree(TreeNoColumns | TreeNoBorder | TreeSidebar | TreeNoHeader),
+    _filtered_schema_tree(TreeNoColumns | TreeNoBorder | TreeSidebar | TreeNoHeader),
+    _schema_search_box(true),
+    _schema_search_text(mforms::SmallSearchEntry),
+    _schema_search_warning(_("Showing loaded schemas only")),
+    _schema_model(0),
+    _base_model(0),
+    _is_model_owner(false),
+    _schema_box(false),
+    _remote_search_enabled(false)
+{
+  _remote_search.set_text(_("Search on Server..."));
+
+  _schema_search_warning.set_style(mforms::SmallHelpTextStyle);
+  _schema_search_warning.set_text_align(mforms::MiddleCenter);
+
+  setup_schema_tree();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -580,7 +702,6 @@ AdvancedSidebar::~AdvancedSidebar()
   for (size_t i = 0; i < _sections.size(); i++)
     delete _sections[i];
 
-  _schema_tree.set_model(0);
   if (_is_model_owner)
     delete _schema_model;
 }
@@ -618,28 +739,111 @@ int AdvancedSidebar::find_section(const std::string& title)
  */
 void AdvancedSidebar::setup_schema_tree()
 {
-#if defined(_WIN32) || defined(__APPLE__)
-  _schema_tree.add_column(IconStringGRTColumnType, 0, _("Schema"));
-#else
-  _schema_tree.add_column(IconStringGRTColumnType, 4242, _("Schema"));
-#endif
-  _schema_tree.set_back_color("#d9e2ef");
+  _new_schema_tree.add_column(mforms::IconStringColumnType, _("Schema"), 100, false, true);
+  _new_schema_tree.set_selection_mode(mforms::TreeSelectMultiple);
+  _new_schema_tree.set_back_color("#d9e2ef");
+  scoped_connect(_new_schema_tree.signal_changed(), boost::bind(&AdvancedSidebar::on_tree_node_selected, this));
+
+  _filtered_schema_tree.add_column(mforms::IconStringColumnType, _("Schema"), 100, false, true);
+  _filtered_schema_tree.set_selection_mode(mforms::TreeSelectMultiple);
+  _filtered_schema_tree.set_back_color("#d9e2ef");
+  scoped_connect(_filtered_schema_tree.signal_changed(), boost::bind(&AdvancedSidebar::on_tree_node_selected, this));
+
   set_back_color("#d9e2ef");
-  _schema_tree.set_context_menu(&_tree_context_menu);
+
+  _new_schema_tree.set_context_menu(&_tree_context_menu);
+  _new_schema_tree.end_columns();
+
+  _filtered_schema_tree.set_context_menu(&_tree_context_menu);
+  _filtered_schema_tree.end_columns();
+
   scoped_connect(_tree_context_menu.signal_will_show(),boost::bind(&AdvancedSidebar::on_show_menu, this));
   _tree_context_menu.set_handler(boost::bind(&AdvancedSidebar::handle_menu_command, this, _1));
 
+  _schema_search_box.set_back_color("#d9e2ef");
+  _schema_search_text.set_placeholder_text("Search objects");
+  _schema_search_box.add(&_schema_search_text, true, true);
+  _schema_search_box.set_padding(8, 0, 8, 2);
+  scoped_connect(_schema_search_text.signal_changed(), boost::bind(&AdvancedSidebar::on_search_text_changed, this));
+  scoped_connect(_remote_search.signal_clicked(),boost::bind(&AdvancedSidebar::on_remote_search_clicked, this));
   // Add the tree itself and its section caption to the container.
-  _schema_tree_heading = new SidebarSection(this, _("SCHEMAS"), true, true);
+  _schema_tree_heading = new SidebarSection(this, _("SCHEMAS"), false, true);
+  _schema_tree_heading->set_back_color("#d9e2ef");
   scoped_connect(_schema_tree_heading->expanded_changed(),boost::bind(&AdvancedSidebar::on_expand_changed, this, _1));
   _sections.push_back(_schema_tree_heading);
   _schema_box.set_back_color("#d9e2ef");
   _schema_box.add(_schema_tree_heading, false, true);
-  _schema_box.add(&_schema_tree, true, true);
-
+  _schema_box.add(&_schema_search_box, false, true);
+  _schema_box.add(&_new_schema_tree, true, true);
+//  _schema_box.set_font("Trebuchet MS bold 9");
   _schema_box.show(false);
   add_end(&_schema_box, true, true);
 }
+//--------------------------------------------------------------------------------------------------
+
+void AdvancedSidebar::on_search_text_changed()
+{
+  std::string filter = _schema_search_text.get_string_value();
+
+  // Updates the current schema model to filtered/base
+  // based on the content of filter
+  if ( filter.length() > 0 )
+  {
+    _filtered_schema_model->set_filter(_schema_search_text.get_string_value());
+    _filtered_schema_model->filter_data();
+
+    if (_schema_model == _base_model)
+    {
+      //_base_model->filter_data(LiveSchemaTree::Any, _schema_search_text.get_string_value(), *_filtered_schema_model);
+      _schema_box.remove(&_new_schema_tree);
+      set_schema_model(_filtered_schema_model);
+      _schema_box.add(&_filtered_schema_tree, true, true);
+      
+      if (_remote_search_enabled)
+        _schema_box.add(&_remote_search, false, true);
+      else
+        _schema_box.add(&_schema_search_warning, false, true);
+        
+      _base_model->enable_events(false);
+      _filtered_schema_model->enable_events(true);
+    }
+  }
+  else
+  {
+    _schema_box.remove(&_filtered_schema_tree);
+    
+    if (_remote_search_enabled)
+      _schema_box.remove(&_remote_search);
+    else
+      _schema_box.remove(&_schema_search_warning);
+      
+    set_schema_model(_base_model);
+    _schema_box.add(&_new_schema_tree, true, true);
+    _base_model->enable_events(true);
+    _filtered_schema_model->enable_events(false);
+  }
+
+  // Raises a signal indicating the filter has changed
+  _search_box_changed_signal(filter);
+}
+
+void AdvancedSidebar::on_tree_node_selected()
+{
+  _tree_node_selected();
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void AdvancedSidebar::on_remote_search_clicked()
+{
+  std::vector<std::string> filter = base::split(_schema_search_text.get_string_value(), ".", 2);
+  std::string schema_filter = filter[0];
+  std::string object_filter = "";
+  if (filter.size() == 2 )
+    object_filter = filter[1];
+
+  _filtered_schema_model->load_data_for_filter(schema_filter, object_filter);
+}
 
 //--------------------------------------------------------------------------------------------------
 
@@ -647,8 +851,11 @@ void AdvancedSidebar::on_show_menu()
 {
   _tree_context_menu.clear();
 
-  std::vector<bec::NodeId> nodes;
-  _schema_tree.get_selection(nodes);
+  std::list<mforms::TreeNodeRef> nodes = _new_schema_tree.get_selection();
+  //std::vector<bec::NodeId> nodes;
+  //_schema_tree.get_selection(nodes);
+  
+  //TODO: mforms::TreeNodeRef node = _new_schema_tree.get_selected_node();
   bec::MenuItemList items = _schema_model->get_popup_items_for_nodes(nodes);
   _tree_context_menu.add_items_from_list(items);
 }
@@ -657,15 +864,16 @@ void AdvancedSidebar::on_show_menu()
 
 void AdvancedSidebar::on_expand_changed(SidebarSection* section)
 {
-  _schema_tree.show(section->expanded());
+//  _schema_tree.show(section->expanded());
+  // TODO: _new_schema_tree.show(section->expanded());
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void AdvancedSidebar::handle_menu_command(const std::string& command)
 {
-  std::vector<bec::NodeId> nodes;
-  _schema_tree.get_selection(nodes);
+  std::list<mforms::TreeNodeRef> nodes = _new_schema_tree.get_selection();
+  
   _schema_model->activate_popup_item_for_nodes(command, nodes);
 }
 
@@ -675,7 +883,7 @@ void AdvancedSidebar::refresh_clicked(SidebarSection* section)
 {
   if (section == _schema_tree_heading)
   {
-    std::vector<bec::NodeId> nodes;
+    std::list<mforms::TreeNodeRef> nodes;
     _schema_model->activate_popup_item_for_nodes("refresh", nodes);
   }
 }
@@ -684,18 +892,20 @@ void AdvancedSidebar::refresh_clicked(SidebarSection* section)
 
 void AdvancedSidebar::refresh_model()
 {
-  _schema_model->refresh();
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void AdvancedSidebar::set_schema_model(LiveSchemaTree* model)
 {
-  _schema_tree.set_model(model);
-  _changed_conn.disconnect();
-  if (model)
-    _changed_conn = *scoped_connect(model->tree_changed_signal(),boost::bind(&GRTTreeView::row_count_changed, &_schema_tree, _1, _2)).get();
-  
+  // Sets the given model as the base model if none assigned already
+  if(!_base_model)
+  {
+    _base_model = model;
+    _base_model->set_model_view(&_new_schema_tree);
+    _base_model->enable_events(true);
+  }
+
   if (_is_model_owner)
   {
     delete _schema_model;
@@ -703,21 +913,23 @@ void AdvancedSidebar::set_schema_model(LiveSchemaTree* model)
   }
   _schema_model = model;
   _schema_box.show(_schema_model != NULL);
-  
-  _activate_conn.disconnect();
-  if (_schema_model)
-    _activate_conn = *scoped_connect(_schema_tree.signal_row_activate(),boost::bind(&wb::LiveSchemaTree::activate_node, _schema_model, _1)).get();
+}
+
+void AdvancedSidebar::set_filtered_schema_model(wb::LiveSchemaTree* model) 
+{
+  _filtered_schema_model = model;
+  _filtered_schema_model->set_model_view(&_filtered_schema_tree);
 }
 
 //--------------------------------------------------------------------------------------------------
 
-int AdvancedSidebar::add_section(const string& title)
+int AdvancedSidebar::add_section(const string& title, bool collapsible)
 {
   int result = find_section(title);
   if (result > -1)
     return result;
 
-  SidebarSection* box = new SidebarSection(this, title, true, false);
+  SidebarSection* box = new SidebarSection(this, title, collapsible, false);
   box->set_back_color("#d9e2ef");
   _sections.push_back(box);
   add(box, false, true);
@@ -741,13 +953,42 @@ void AdvancedSidebar::remove_section(const std::string& section)
 //--------------------------------------------------------------------------------------------------
 
 int AdvancedSidebar::add_section_entry(const std::string& section, const std::string& title,
-  const std::string& icon, const std::string& command, bool as_link)
+  const std::string& icon, const std::string& command, TaskEntryType type)
 {
   int index = find_section(section);
   if (index < 0)
     return - 1;
 
-  return _sections[index]->add_entry(title, icon, command, as_link);
+  return _sections[index]->add_entry(title, icon, command, type);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void AdvancedSidebar::set_section_entry_text(const std::string& section, int entry_index, const std::string& title, const std::string& icon)
+{
+  int index = find_section(section);
+  if (index < 0)
+    return;
+  
+  if (entry_index >= 0 && entry_index < _sections[index]->entry_count())
+    _sections[index]->set_entry_text(entry_index, title, icon);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void AdvancedSidebar::mark_section_busy(const std::string& section, bool busy)
+{
+  if (section.empty())
+  {
+    _schema_tree_heading->mark_busy(busy);
+    return;
+  }
+
+  int index = find_section(section);
+  if (index < 0)
+    return;
+
+  _sections[index]->mark_busy(busy);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -855,7 +1096,8 @@ void AdvancedSidebar::set_selection_color(const mforms::SystemColor color)
  */
 void AdvancedSidebar::expand_schema(int schema_index)
 {
-  _schema_tree.set_expanded(schema_index, true);
+  // TODO: _new_schema_tree
+  //_schema_tree.set_expanded(schema_index, true);
 }
 
 //--------------------------------------------------------------------------------------------------
