@@ -46,10 +46,10 @@ static mforms::Label *new_label(const std::string &text, bool right_align=false,
 //--------------------------------------------------------------------------------------------------
 
 BugReportForm::BugReportForm(wb::WBContextUI *wbui, const std::string& sys_info, const std::string &error_info):
-Form(NULL, mforms::FormResizable), _top_box(false), _bottom_box(true), 
-  _button_box(true),_credentials_box(false), _user_box(true), _description(mforms::BothScrollBars), 
-  _how_to_repeat(mforms::BothScrollBars), _internal_info(mforms::BothScrollBars), _email(mforms::NormalEntry), 
-  _password(mforms::PasswordEntry), _stored_user(""), _stored_password(""), _logged_in(false)
+Form(NULL, mforms::FormResizable), _top_box(false), 
+  _credentials_box(false), _user_box(true), _email(mforms::NormalEntry), _password(mforms::PasswordEntry), _description(mforms::BothScrollBars), 
+  _how_to_repeat(mforms::BothScrollBars), _dev_notes(mforms::BothScrollBars), _bottom_box(true), _button_box(true),
+  _stored_user(""), _stored_password(""), _logged_in(false)
 {
   _wbui= wbui;
 
@@ -71,19 +71,22 @@ Form(NULL, mforms::FormResizable), _top_box(false), _bottom_box(true),
   create_text_section(_("Description"), &_description);
   mforms::Box* box = create_text_section(_("How To Repeat"), &_how_to_repeat);
   box->add(new_label(_("It is very important that repeat instructions are clear and detailed as the developers need to reproduce the bug to be able to fix it"), false, true), false);
-  create_text_section(_("Private Notes to Developer"), &_internal_info);
+  create_text_section(_("Notes to Developer"), &_dev_notes);
+
+  _private_dev_notes.set_text(_("Make notes to developer private"));
+  _private_dev_notes.set_active(false);
+  _top_box.add(&_private_dev_notes, false);
 
   _include_log.set_text(_("Attach Workbench log file to bug report (File will not be public)"));
   _include_log.set_active(true);
-
   _top_box.add(&_include_log, false);
 
-  _internal_info.append_text(_sys_info);
+  _dev_notes.append_text(_sys_info);
 
   if (_error_info != "from_menu")
   {
-    _internal_info.append_text("\n\n");
-    _internal_info.append_text(_error_info);
+    _dev_notes.append_text("\n\n");
+    _dev_notes.append_text(_error_info);
   }
 
   _top_box.add_end(&_bottom_box, false);
@@ -197,6 +200,7 @@ void BugReportForm::create_details_box()
   _category.add_item("MySQL Workbench: SQL Editor");
   _category.add_item("MySQL Workbench: Modeling");
   _category.add_item("MySQL Workbench: MySQL Utilities");
+  _category.add_item("MySQL Workbench: Migration");
   _category.add_item("MySQL Workbench: Docs");
 
   table->add(new_label(_("Category :"), true), 0, 1, 1, 2, mforms::HFillFlag);
@@ -260,13 +264,15 @@ void BugReportForm::parse_sys_info()
     pcre_get_substring(_sys_info_list[0].c_str(), patres, rc, 1, &value);
     
     std::string os(value);
-    
+    pcre_free_substring(value);
+
     if (os == "Linux/Unix")    _os = "5";
     else if (os == "Mac OS X") _os = "6";
     else if (os == "Windows")  _os = "7"; 
 
     pcre_get_substring(_sys_info_list[0].c_str(), patres, rc, 2, &value);
     _version = value;
+    pcre_free_substring(value);
   }
 
   bool found = false;
@@ -325,7 +331,7 @@ void BugReportForm::form_closed()
       throw std::logic_error("Internal error: can't find Workbench module.");
 
     // Fills a map with all the information needed for the bug submission.
-    grt::internal::Dict data(_wbui->get_wb()->get_grt(), false);
+    grt::DictRef data(_wbui->get_wb()->get_grt());
     data.set("in[sdesc]",grt::StringRef(_synopsis.get_string_value()));
     data.set("in[bug_type]",grt::StringRef(_category.get_string_value()));
     data.set("in[severity]",grt::StringRef(grt::IntegerRef(5 - _severity.get_selected_index()).repr()));
@@ -342,20 +348,31 @@ void BugReportForm::form_closed()
     data.set("in[reviewer2]",grt::StringRef(""));
     data.set("in[lead]",grt::StringRef(""));
     data.set("in[eta]",grt::StringRef(""));
-    data.set("in[ldesc]",grt::StringRef(_description.get_string_value()));
     data.set("in[howtorepeat]",grt::StringRef(_how_to_repeat.get_string_value()));
     data.set("in[suggestedfix]",grt::StringRef(""));
-    data.set("in[internalinfo]",grt::StringRef(_internal_info.get_string_value()));
     data.set("in[really]",grt::StringRef(_really));
 
     if (_include_log.get_active())
       data.set("log_file", grt::StringRef(base::Logger::log_filename()));
+      
+    if (_private_dev_notes.get_active())
+    {
+      data.set("in[ldesc]",grt::StringRef(_description.get_string_value()));
+      data.set("in[internalinfo]",grt::StringRef(_dev_notes.get_string_value()));
+    }
+    else
+    {
+      data.set("in[ldesc]",grt::StringRef(_description.get_string_value() + 
+                                          "\n\n\n----- Developer Notes -----\n\n" + 
+                                          _dev_notes.get_string_value()));
+      data.set("in[internalinfo]",grt::StringRef(""));
+    }
 
     // Sets the parameters for the python plugin.
     grt::BaseListRef args(_wbui->get_wb()->get_grt());
     args.ginsert(grt::StringRef(_email.get_string_value()));
     args.ginsert(grt::StringRef(_password.get_string_value()));
-    args.ginsert(grt::DictRef(&data));
+    args.ginsert(data);
 
     // Calls the plugin
     std::string result;
@@ -368,8 +385,8 @@ void BugReportForm::form_closed()
     }
     catch(std::exception &e)
     {
-      result = _("error|An error occurred while submitting the report.\n Please verify the proper submition by logging in at http:\\bugs.mysql.com");
-      log_error("WB Bug Report","An error occurred while submitting the report: %s", e.what());
+      result = _("error|An error occurred while submitting the report.\n Please verify the proper submition by logging in at http://bugs.mysql.com");
+      log_error("An error occurred while submitting the report: %s", e.what());
     }
 
     // The plugin will return the information as follows:
@@ -394,7 +411,16 @@ void BugReportForm::form_closed()
       return_code = mforms::Utilities::show_message(_("Bug Report Confirmation"), message, _("Go to Bug Report"), _("OK"));
       
       if (return_code == 1)
-        mforms::Utilities::open_url("http://bugs.mysql.com/" + result_details[1].substr(1));
+      {
+        std::string bug_number = "";
+        for(size_t index = 0; index < result_details.size() && bug_number == ""; index++)
+        {
+          if (result_details[index][0] == '#')
+            bug_number = result_details[index].substr(1);
+        }
+        mforms::Utilities::open_url("http://bugs.mysql.com/bug.php?id=" + bug_number);
+      
+      }
       
       mforms::Form::end_modal(true);
     }
@@ -492,4 +518,4 @@ std::string BugReportForm::validate_fields()
   }
 
   return error;
-}
\ No newline at end of file
+}
