@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,6 +24,8 @@
 #include "grt/grt_manager.h"
 #include "grtui/grtdb_connection_editor.h"
 
+#include "mforms/uistyle.h"
+
 #define INTRO_TEXT \
   "This wizard will guide you through the creation of a Server Profile to manage a MySQL server. "\
   "To fully support management of a remote MySQL server, an SSH daemon must be running "\
@@ -142,6 +144,10 @@ HostMachinePage::HostMachinePage(WizardForm* host, wb::WBContext *wb)
 
 void HostMachinePage::host_selection_changed()
 {
+  // Mark a local connection only if the user selected localhost. This way we can allow
+  // an IP address like 127.0.0.1 to be used like a remote one (e.g. for port forwarding).
+  values().gset("isLocal", _localhost_button.get_active() ? 1 : 0);
+
   bool flag= _networkhost_button.get_active();
   _ip_address_block.set_enabled(flag);
 
@@ -260,23 +266,21 @@ bool DatabaseServerPage::skip_page()
 
 void DatabaseServerPage::enter(bool advancing)
 {
+  wizard()->clear_problem();
+
   if (advancing)
   {
-    db_mgmt_ConnectionRef conn(_context->get_grt());
-    conn->name(values().get_string("host_name", "new_connection"));
-    conn->parameterValues().gset("hostName", values().get_string("host_name", "new_connection"));
-    
     if (!_conn_name)
     {
+      db_mgmt_ConnectionRef conn(_context->get_grt());
+      conn->name(values().get_string("host_name", "new_connection"));
+      conn->parameterValues().gset("hostName", values().get_string("host_name", "new_connection"));
+
       _panel.init(_context->get_root()->rdbmsMgmt(), conn);
     
-      _conn_name= _panel.get_name_entry();
+      _conn_name = _panel.get_name_entry();
+      _conn_name->set_value(conn->name());    
     }
-    else
-    {
-      _panel.get_be()->set_connection(conn);
-    }
-    _conn_name->set_value(conn->name());    
   }
 }
 
@@ -284,6 +288,17 @@ void DatabaseServerPage::enter(bool advancing)
 
 bool DatabaseServerPage::advance()
 {
+  // Check if the instance name already exists.
+  grt::ListRef<db_mgmt_Connection> connections = _context->get_root()->rdbmsMgmt()->storedConns();
+  std::string name = base::trim(_conn_name->get_string_value());
+  GRTLIST_FOREACH(db_mgmt_Connection, connections, connection)
+  {
+    if ((*connection)->name() == name)
+    {
+      wizard()->set_problem(strfmt(_("A connection with the name %s exists already. Please choose another one."), name.c_str()));
+      return false;
+    }
+  }
   return true;
 }
 
@@ -340,7 +355,7 @@ TestDatabaseSettingsPage::TestDatabaseSettingsPage(WizardForm* host)
            boost::bind(&TestDatabaseSettingsPage::get_server_platform, this),
            _("Querying server OS type..."));
 
-/*XXX TODO
+/*XXX Improve rights checks.
   add_task(_("Check Account Privileges"),
            boost::bind(&TestDatabaseSettingsPage::get_server_platform, this),
            _("Checking if account being used has enough privileges..."));
@@ -657,6 +672,8 @@ void HostAndRemoteTypePage::enter(bool advancing)
 
 void HostAndRemoteTypePage::refresh_profile_list()
 {
+  wizard()->clear_problem();
+
   std::string system = _os_selector.get_string_value();
   
   _type_selector.clear();
@@ -669,6 +686,8 @@ void HostAndRemoteTypePage::refresh_profile_list()
 
 void HostAndRemoteTypePage::toggle_remote_admin()
 {
+  wizard()->clear_problem();
+
   // Clear out any OS value if we don't use remote management.
   if (_no_remote_admin.get_active() && !wizard()->is_local())
   {
@@ -849,7 +868,7 @@ SSHConfigurationPage::SSHConfigurationPage(WizardForm* host)
   scoped_connect(_use_ssh_key.signal_clicked(),boost::bind(&SSHConfigurationPage::use_ssh_key_changed, this));
   _ssh_settings_table.add(&_use_ssh_key, 1, 5, 4, 5, HFillFlag);
   
-  _ssh_path_label.set_text(_("SSH Public Key Path:"));
+  _ssh_path_label.set_text(_("SSH Private Key Path:"));
   _ssh_settings_table.add(&_ssh_path_label, 1, 2, 5, 6, HFillFlag);
   _ssh_settings_table.add(&_ssh_key_path, 2, 3, 5, 6, HFillFlag | HExpandFlag);
   _ssh_settings_table.add(&_ssh_key_browse_button, 3, 4, 5, 6, 0);
@@ -2043,8 +2062,10 @@ bool NewServerInstanceWizard::is_admin_enabled()
  */
 bool NewServerInstanceWizard::is_local()
 {
-  std::string host= values().get_string("host_name", "localhost");
-  return (host == "localhost") || (host == "127.0.0.1") || (base::trim(host).empty());
+  // A local connection is not defined by an IP address (as port forwarding could be active)
+  // but by the user himself, as he should know if it is a local host or not.
+  int isLocal = values().get_int("isLocal");
+  return isLocal != 0;
 }
 
 //--------------------------------------------------------------------------------------------------
