@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,29 +19,36 @@
 
 #include "stdafx.h"
 
+#include "wb_context.h"
 #include "output_view.h"
 #include "base/string_utilities.h"
 
-OutputView::OutputView(bec::MessageListStorage *storage)
-  : mforms::AppView(true, "output", true), _splitter(true), _message_list(mforms::TreeDefault),
+using namespace wb;
+
+OutputView::OutputView(WBContext* context)
+  : mforms::AppView(true, "output", true), _splitter(true), _message_list(mforms::TreeFlatList),
   _output_text(mforms::BothScrollBars)
 {
+  _wb = context;
+  _can_track_changes = false; // True when the UI setup is done and we can track further UI changes.
+
   add(&_splitter, true, true);
 
   _splitter.add(&_message_list);
   _splitter.add(&_output_text);
+  Box::scoped_connect(_splitter.signal_position_changed(), boost::bind(&OutputView::splitter_moved, this));
   
-  _message_list.add_column(mforms::IconStringGRTColumnType, 0, "");
-  _message_list.set_column_width(0, 100);
-  _message_list.add_column(mforms::StringGRTColumnType, 1, "Message");
-  _message_list.set_column_width(1, 500);
+  _message_list.add_column(mforms::IconStringColumnType, "", 100, false);
+  _message_list.add_column(mforms::StringColumnType, "Message", 500, false);
+  _message_list.add_column(mforms::StringColumnType, "Detail", 200, false);
+  _message_list.end_columns();
   
-  storage->set_output_handler(boost::bind(&mforms::TextBox::append_text, &_output_text, _1, true));
+  _storage = _wb->get_grt_manager()->get_messages_list();
+  _storage->set_output_handler(boost::bind(&mforms::TextBox::append_text, &_output_text, _1, true));
   
-  _storage = storage;
-  _messages = storage->create_list();
-  _message_list.set_model(_messages);
-  _message_list.set_allow_multi_selection(true);
+  _messages = _storage->create_list();
+  _message_list.set_selection_mode(mforms::TreeSelectMultiple);
+  refresh();
   
   UIForm::scoped_connect(_messages->signal_row_added(),boost::bind(&OutputView::row_added, this));
   
@@ -60,7 +67,6 @@ OutputView::OutputView(bec::MessageListStorage *storage)
 OutputView::~OutputView()
 {
   _storage->set_output_handler(boost::function<void (std::string)>());
-  _message_list.set_model(0);
   delete _messages;
   _messages = 0;
 }
@@ -71,14 +77,15 @@ void OutputView::handle_command(const std::string& command)
 {
   if (command == "copy")
   {
-    std::vector<bec::NodeId> selection;
-    _message_list.get_selection(selection);
+    std::list<mforms::TreeNodeRef> selection;
+    selection = _message_list.get_selection();
     std::string result;
-    for (std::vector<bec::NodeId>::const_iterator iterator = selection.begin(); iterator != selection.end(); 
+    for (std::list<mforms::TreeNodeRef>::const_iterator iterator = selection.begin(); iterator != selection.end(); 
       iterator++)
     {
       std::string type_string;
-      grt::MessageType type = _messages->get_message_type(*iterator);
+      int row = _message_list.row_for_node(*iterator);
+      grt::MessageType type = _messages->get_message_type(row);
       switch (type)
       {
         case grt::WarningMsg:
@@ -93,11 +100,11 @@ void OutputView::handle_command(const std::string& command)
       std::string time;
       std::string message;
       std::string details;
-      _messages->get_field(*iterator, bec::MessageListBE::Time, time);
-      _messages->get_field(*iterator, bec::MessageListBE::Message, message);
+      _messages->get_field(row, bec::MessageListBE::Time, time);
+      _messages->get_field(row, bec::MessageListBE::Message, message);
       if (message.empty())
         message = _("none");
-      _messages->get_field(*iterator, bec::MessageListBE::Detail, details);
+      _messages->get_field(row, bec::MessageListBE::Detail, details);
       if (details.empty())
         details = _("n/a");
       result.append(base::strfmt(_("%s:\tTime: %s\tMessage: %s\tDetails: %s\n"),
@@ -109,9 +116,7 @@ void OutputView::handle_command(const std::string& command)
     if (command == "clear")
     {
       _messages->clear();
-      // TODO: the model should send a notification about the clearance, so that the viewer
-      //       can implicitly update itself.
-      _message_list.refresh(bec::NodeId());
+      refresh();
     }
 }
 
@@ -119,16 +124,62 @@ void OutputView::handle_command(const std::string& command)
 
 void OutputView::row_added()
 {
-  _message_list.row_count_changed(bec::NodeId(), _messages->count()-1);
-  _message_list.refresh(bec::NodeId()); // TODO: Must be support for row_inserted!
+  mforms::TreeNodeRef node;
+  if (_message_list.root_node())
+  {
+    for (int c = _messages->count(), i = _message_list.root_node()->count(); i < c; i++)
+    {
+      std::string s;
+      bec::IconId icon = _messages->get_field_icon(i, 0, bec::Icon16);
+      node = _message_list.add_node();
+      _messages->get_field(i, bec::MessageListBE::Time, s);
+      node->set_string(0, s);
+      if (icon >= 0)
+        node->set_icon_path(0, bec::IconManager::get_instance()->get_icon_path(icon));
+      _messages->get_field(i, bec::MessageListBE::Message, s);
+      node->set_string(1, s);
+      _messages->get_field(i, bec::MessageListBE::Detail, s);
+      node->set_string(2, s);
+    }
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void OutputView::refresh()
+{
+  _message_list.clear();
+  
+  row_added();
 }
 
 //--------------------------------------------------------------------------------------------------
 
 bool OutputView::will_close()
 {
+  _can_track_changes = false;
   return true;
 }
 
 //--------------------------------------------------------------------------------------------------
 
+void OutputView::setup_ui()
+{
+  int splitter_position = _wb->read_state("message_width", "output_view", 500);
+  _splitter.set_position(splitter_position);
+  _can_track_changes = true;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void OutputView::splitter_moved()
+{
+  if (!_can_track_changes)
+    return;
+
+  int splitter_position = _splitter.get_position();
+  _wb->save_state("message_width", "output_view", splitter_position);
+}
+
+//--------------------------------------------------------------------------------------------------
+
