@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,13 +20,15 @@
 #include "stdafx.h"
 
 #include "preferences_form.h"
-#include "mforms/mforms.h"
 #include "mforms/widgets.h"
 #include "mforms/sectionbox.h"
 
 #include "base/string_utilities.h"
 #include "base/util_functions.h"
 
+#include "snippet_popover.h"
+#include "mforms/textbox.h"
+
 #if defined(_WIN32) || defined(__APPLE__)
 #define HAVE_BUNDLED_MYSQLDUMP
 #endif
@@ -115,7 +117,7 @@ public:
     _table.add(entry, 1, 2, _rows-1, _rows, mforms::HFillFlag);
   }
   
-  void add_checkbox_option(const std::string &option, const std::string &caption, const std::string &tooltip)
+  mforms::CheckBox *add_checkbox_option(const std::string &option, const std::string &caption, const std::string &tooltip)
   {
     _table.set_row_count(++_rows);
     
@@ -129,14 +131,16 @@ public:
     int start_column = 1;
 #endif
     _table.add(cb, start_column, 2, _rows - 1, _rows, mforms::HFillFlag);
+    
+    return cb;
   }
 };
 
 
 
 PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const std::string &model_id)
-  : Form(NULL, mforms::FormResizable), _top_box(false), _bottom_box(true), _button_box(true), 
-  _font_list(mforms::TreeDefault), _tabview(mforms::TabViewDocument)
+  : Form(NULL, mforms::FormResizable), _top_box(false), _bottom_box(true), 
+  _tabview(mforms::TabViewDocument), _button_box(true), _font_list(mforms::TreeFlatList)
 {
   _wbui= wbui;
   _model_id= model_id;
@@ -190,6 +194,7 @@ PreferencesForm::PreferencesForm(wb::WBContextUI *wbui, const std::string &model
     create_general_page();
     create_admin_page();
     create_sqlide_page();
+    create_query_page();
   }
   create_model_page();
   create_mysql_page();
@@ -328,8 +333,9 @@ void PreferencesForm::update_checkbox_option(const std::string &option_name, mfo
 #ifdef _WIN32
   // On Windows we have to write the following value also to the registry as our options are not
   // available yet when we need that value.
-  if (option_name == "SingleInstance")
-    set_value_to_registry(HKEY_CURRENT_USER, "Software\\Oracle\\MySQL Workbench", "SingleInstance", value.c_str());
+  if (option_name == "DisableSingleInstance")
+    set_value_to_registry(HKEY_CURRENT_USER, "Software\\Oracle\\MySQL Workbench",
+      "DisableSingleInstance", value.c_str());
 #endif
 }
 
@@ -538,12 +544,134 @@ void PreferencesForm::create_admin_page()
 
 void PreferencesForm::create_sqlide_page()
 {
+  // General options for the SQL Editor
+  
   mforms::Box *box= mforms::manage(new mforms::Box(false));
   box->set_padding(12);
   box->set_spacing(8);
   
   _tabview.add_page(box, _("SQL Editor"));
 
+  {
+    mforms::Panel *frame= mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
+    frame->set_title(_("General"));
+    box->add(frame, false);
+    
+    mforms::Box *vbox= mforms::manage(new mforms::Box(false));
+    vbox->set_padding(8);
+    vbox->set_spacing(4);
+    frame->add(vbox);
+    
+    {
+      mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:ShowSchemaTreeSchemaContents");
+      check->set_text(_("Show Schema Contents in Schema Tree"));
+      check->set_tooltip(_(
+                           "Whether to show schema contents (tables, views and routine names) in schema tree."));
+      vbox->add(check, false);
+    }
+    
+    {
+      mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:ShowMetadataSchemata");
+      check->set_text(_("Show Metadata Schemata"));
+      check->set_tooltip(_(
+                           "Whether to show metadata/internal schemata in schema tree (eg INFORMATION_SCHEMA, mysql and schemas starting with '.')."));
+      vbox->add(check, false);
+    }
+    
+    {
+      mforms::Box *tbox= mforms::manage(new mforms::Box(true));
+      tbox->set_spacing(4);
+      vbox->add(tbox, false);
+      
+      tbox->add(new_label(_("DBMS connection keep-alive interval (in seconds):"), true), false, false);
+      mforms::TextEntry *entry= new_entry_option("DbSqlEditor:KeepAliveInterval", false);
+      entry->set_max_length(5);
+      entry->set_size(50, -1);
+      entry->set_tooltip(_(
+                           "Time interval between sending keep-alive messages to DBMS.\n"
+                           "Set to 0 to not send keep-alive messages."));
+      tbox->add(entry, false, false);
+    }    
+    
+    {
+      mforms::Box *tbox= mforms::manage(new mforms::Box(true));
+      tbox->set_spacing(4);
+      vbox->add(tbox, false);
+      
+      tbox->add(new_label(_("DBMS connection read time out (in seconds):"), true), false, false);
+      mforms::TextEntry *entry= new_entry_option("DbSqlEditor:ReadTimeOut", false);
+      entry->set_max_length(5);
+      entry->set_size(50, -1);
+      entry->set_tooltip(_(
+                           "Max time the a query can take to return data from the DBMS"));
+      tbox->add(entry, false, false);
+    }
+  }
+  
+  {
+    mforms::Panel *frame= mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
+    frame->set_title(_("SQL Editor"));
+    box->add(frame, false);
+    
+    mforms::Box *vbox= mforms::manage(new mforms::Box(false));
+    vbox->set_padding(8);
+    vbox->set_spacing(4);
+    frame->add(vbox);
+    
+    // Code completion settings is a horizontal box with two check boxes.
+    {
+      mforms::Box *cc_box = mforms::manage(new mforms::Box(true));
+      vbox->add(cc_box, false);
+      mforms::CheckBox *auto_start_check = new_checkbox_option("DbSqlEditor:AutoStartCodeCompletion");
+      {
+        mforms::CheckBox *check = new_checkbox_option("DbSqlEditor:CodeCompletionEnabled");
+        scoped_connect(check->signal_clicked(), boost::bind(&PreferencesForm::code_completion_changed,
+                                                            this, check, auto_start_check));
+        
+        check->set_text(_("Enable Code Completion in Editors"));
+        check->set_tooltip(_(
+                             "If enabled SQL editors display a code completion list when pressing the defined hotkey"));
+        cc_box->add(check, false);
+        
+        // Set initial enabled state of the auto-cc checkbox depending on whether
+        // code completion is enabled at all.
+        std::string value;
+        _wbui->get_wb_options_value(_model_id, "DbSqlEditor:CodeCompletionEnabled", value);
+        auto_start_check->set_enabled(atoi(value.c_str()) != 0);
+      }
+      {
+        auto_start_check->set_text(_("Automatically Start Code Completion"));
+        auto_start_check->set_tooltip(_(
+                                        "Available only if code completion is enabled. By activating this option code completion "
+                                        "will be started automatically when you type something and wait a moment"));
+        cc_box->add_end(auto_start_check, false);
+      }
+    }
+    
+    {
+      mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:ReformatViewDDL");
+      check->set_text(_("Reformat DDL for Views"));
+      check->set_tooltip(_("Whether to automatically reformat View DDL returned by the server. The MySQL server does not store the formatting information for View definitions."));
+      vbox->add(check, false);
+    }
+    
+    {
+      mforms::Box *tbox= mforms::manage(new mforms::Box(true));
+      tbox->set_spacing(4);
+      vbox->add(tbox, false);
+      
+      tbox->add(new_label(_("Max syntax error count:"), true), false, false);
+      mforms::TextEntry *entry= new_entry_option("SqlEditor::SyntaxCheck::MaxErrCount", false);
+      entry->set_max_length(5);
+      entry->set_size(50, -1);
+      entry->set_tooltip(_(
+                           "Maximum number of errors to stop syntax check.\n"
+                           "Syntax errors aren't highlighted beyond this threshold.\n"
+                           "Set to 0 to show all errors."));
+      tbox->add(entry, false, false);
+    }
+  }
+  
   {
     mforms::Panel *frame= mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
     frame->set_title(_("SQL"));
@@ -553,7 +681,7 @@ void PreferencesForm::create_sqlide_page()
     vbox->set_padding(8);
     vbox->set_spacing(4);
     frame->add(vbox);
-
+    
     {
       mforms::Box *tbox= mforms::manage(new mforms::Box(true));
       tbox->set_spacing(4);
@@ -562,22 +690,22 @@ void PreferencesForm::create_sqlide_page()
       tbox->add(new_label(_("Default SQL_MODE:"), true), false, false);
       mforms::TextEntry *entry= new_entry_option("SqlMode", false);
       entry->set_tooltip(_(
-        "Value of SQL_MODE DBMS session variable customizes the rules and restrictions for SQL syntax and semantics. See MySQL Server reference for details.\n"
-        "This globally defined parameter determines initial value for same named parameter in each newly created model. "
-        "Model scoped same named parameter in its turn affects SQL parsing within the model, and defines the value of SQL_MODE session variable when connecting to DBMS.\n"
-        "Note: Empty value for this parameter will cause Workbench to treat SQL_MODE as empty string when parsing SQL within the model, but will leave DBMS session variable at its default value.\n"
-        "To force Workbench to reset SQL_MODE session variable as well, this parameter needs to be set to a whitespace symbol."));
+                           "Value of SQL_MODE DBMS session variable customizes the rules and restrictions for SQL syntax and semantics. See MySQL Server reference for details.\n"
+                           "This globally defined parameter determines initial value for same named parameter in each newly created model. "
+                           "Model scoped same named parameter in its turn affects SQL parsing within the model, and defines the value of SQL_MODE session variable when connecting to DBMS.\n"
+                           "Note: Empty value for this parameter will cause Workbench to treat SQL_MODE as empty string when parsing SQL within the model, but will leave DBMS session variable at its default value.\n"
+                           "To force Workbench to reset SQL_MODE session variable as well, this parameter needs to be set to a whitespace symbol."));
       tbox->add(entry, true, true);
     }
-
+    
     {
       mforms::CheckBox *check= new_checkbox_option("SqlIdentifiersCS");
       check->set_text(_("SQL Identifiers are Case Sensitive"));
       check->set_tooltip(_(
-        "Whether to treat identifiers separately if their names differ only in letter case."));
+                           "Whether to treat identifiers separately if their names differ only in letter case."));
       vbox->add(check, false);
     }
-
+    
     {
       mforms::Box *tbox= mforms::manage(new mforms::Box(true));
       tbox->set_spacing(4);
@@ -587,45 +715,48 @@ void PreferencesForm::create_sqlide_page()
       mforms::TextEntry *entry= new_entry_option("SqlDelimiter", false);
       entry->set_size(50, -1);
       entry->set_tooltip(_(
-        "SQL statement delimiter different from the normally used one (ie, shouldn't be ;). Change this only if the delimiter you normally use, specially in stored routines, happens to be the current setting."));
+                           "SQL statement delimiter different from the normally used one (ie, shouldn't be ;). Change this only if the delimiter you normally use, specially in stored routines, happens to be the current setting."));
       tbox->add(entry, false, false);
     }
-  }
+  }  
+}
 
+
+void PreferencesForm::create_query_page()
+{
+  // Options specific for the query/script execution aspect of the SQL Editor
+  
+  mforms::Box *box= mforms::manage(new mforms::Box(false));
+  box->set_padding(12);
+  box->set_spacing(8);
+  
+  _tabview.add_page(box, _("SQL Queries"));
+  
   {
     mforms::Panel *frame= mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
-    frame->set_title(_("Query Editor"));
+    frame->set_title(_("General"));
     box->add(frame, false);
-
+    
     mforms::Box *vbox= mforms::manage(new mforms::Box(false));
     vbox->set_padding(8);
     vbox->set_spacing(4);
     frame->add(vbox);
     
     {
-      mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:ShowLivePhysicalOverview");
-      check->set_text(_("Show Live Schema Overview"));
-      check->set_tooltip(_(
-        "Whether to show the overview panel for the schemata from the connected database server."));
-      vbox->add(check, false);
+      mforms::Box *tbox= mforms::manage(new mforms::Box(true));
+      tbox->set_spacing(4);
+      vbox->add(tbox, false);
+      
+      tbox->add(new_label(_("Max. query length to store in history (in bytes):"), true), false, false);
+      mforms::TextEntry *entry= new_entry_option("DbSqlEditor:MaxQuerySizeToHistory", false);
+      entry->set_size(50, -1);
+      entry->set_tooltip(_(
+                           "Queries beyond specified size will not be saved in the history when executed.\n"
+                           "Set to 0 to save any executed query or script"));
+      tbox->add(entry, false, false);
     }
-
-    {
-      mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:ShowSchemaTreeSchemaContents");
-      check->set_text(_("Show Schema Contents in Schema Tree"));
-      check->set_tooltip(_(
-        "Whether to show schema contents (tables, views and routine names) in schema tree."));
-      vbox->add(check, false);
-    }
-
-    {
-      mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:ShowMetadataSchemata");
-      check->set_text(_("Show Metadata Schemata"));
-      check->set_tooltip(_(
-        "Whether to show metadata/internal schemata in schema tree and overview panel (eg INFORMATION_SCHEMA and mysql)."));
-      vbox->add(check, false);
-    }
-
+    
+    
     {
       mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:ContinueOnError");
       check->set_text(_("Continue on SQL Script Error (by default)"));
@@ -646,24 +777,8 @@ void PreferencesForm::create_sqlide_page()
                            "or delete a large number of rows. \n"
                            "Changing this option requires a reconnection (Query -> Reconnect to Server)"));
       vbox->add(check, false);
-    }    
-    
-    {
-      mforms::Box *tbox= mforms::manage(new mforms::Box(true));
-      tbox->set_spacing(4);
-      vbox->add(tbox, false);
-      
-      tbox->add(new_label(_("Max syntax error count:"), true), false, false);
-      mforms::TextEntry *entry= new_entry_option("SqlEditor::SyntaxCheck::MaxErrCount", false);
-      entry->set_max_length(5);
-      entry->set_size(50, -1);
-      entry->set_tooltip(_(
-                           "Maximum number of errors to stop syntax check.\n"
-                           "Syntax errors aren't highlighted beyond this threshold.\n"
-                           "Set to 0 to show all errors."));
-      tbox->add(entry, false, false);
     }
-
+    
     {
       mforms::Box *tbox= mforms::manage(new mforms::Box(true));
       tbox->set_spacing(4);
@@ -677,52 +792,8 @@ void PreferencesForm::create_sqlide_page()
                            "Time interval between UI updates when running SQL script."));
       tbox->add(entry, false, false);
     }
-
-    {
-      mforms::Box *tbox= mforms::manage(new mforms::Box(true));
-      tbox->set_spacing(4);
-      vbox->add(tbox, false);
-      
-      tbox->add(new_label(_("DBMS connection keep-alive interval (in seconds):"), true), false, false);
-      mforms::TextEntry *entry= new_entry_option("DbSqlEditor:KeepAliveInterval", false);
-      entry->set_max_length(5);
-      entry->set_size(50, -1);
-      entry->set_tooltip(_(
-                           "Time interval between sending keep-alive messages to DBMS.\n"
-                           "Set to 0 to not send keep-alive messages."));
-      tbox->add(entry, false, false);
-    }    
-
-    {
-      mforms::Box *tbox= mforms::manage(new mforms::Box(true));
-      tbox->set_spacing(4);
-      vbox->add(tbox, false);
-      
-      tbox->add(new_label(_("DBMS connection read time out (in seconds):"), true), false, false);
-      mforms::TextEntry *entry= new_entry_option("DbSqlEditor:ReadTimeOut", false);
-      entry->set_max_length(5);
-      entry->set_size(50, -1);
-      entry->set_tooltip(_(
-                           "Max time the a query can take to return data from the DBMS"));
-      tbox->add(entry, false, false);
-    }    
-
-    {
-      mforms::Box *tbox= mforms::manage(new mforms::Box(true));
-      tbox->set_spacing(4);
-      vbox->add(tbox, false);
-      
-      tbox->add(new_label(_("Max. query length to store in history (in bytes):"), true), false, false);
-      mforms::TextEntry *entry= new_entry_option("DbSqlEditor:MaxQuerySizeToHistory", false);
-      entry->set_size(50, -1);
-      entry->set_tooltip(_(
-        "Queries beyond specified size will not be saved in the history when executed.\n"
-        "Set to 0 to save any executed query or script"));
-      tbox->add(entry, false, false);
-    }
-
   }
-
+  
   {
     mforms::Panel *frame= mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
     frame->set_title(_("Query Results"));
@@ -737,11 +808,11 @@ void PreferencesForm::create_sqlide_page()
       mforms::CheckBox *check= new_checkbox_option("SqlEditor:LimitRows");
       check->set_text(_("Limit Rows"));
       check->set_tooltip(_(
-        "Whether every select query to be implicitly adjusted to limit result set to specified number of rows by appending the LIMIT keyword to the query.\n"
-        "If enabled it's still possible to load entire result set by pressing \"Fetch All\" button."));
+                           "Whether every select query to be implicitly adjusted to limit result set to specified number of rows by appending the LIMIT keyword to the query.\n"
+                           "If enabled it's still possible to load entire result set by pressing \"Fetch All\" button."));
       vbox->add(check, false);
     }
-
+    
     {
       mforms::Box *tbox= mforms::manage(new mforms::Box(true));
       tbox->set_spacing(4);
@@ -752,10 +823,10 @@ void PreferencesForm::create_sqlide_page()
       entry->set_max_length(5);
       entry->set_size(50, -1);
       entry->set_tooltip(_(
-        "Every select query to be implicitly adjusted to limit result set to specified number of rows."));
+                           "Every select query to be implicitly adjusted to limit result set to specified number of rows."));
       tbox->add(entry, false, false);
     }
-
+    
     {
       mforms::Box *tbox= mforms::manage(new mforms::Box(true));
       tbox->set_spacing(4);
@@ -766,39 +837,66 @@ void PreferencesForm::create_sqlide_page()
       entry->set_max_length(5);
       entry->set_size(50, -1);
       entry->set_tooltip(_(
-        "Symbols beyond specified threashold will be truncated when showing in the grid. Doesn't affect editing field values.\n"
-        "Set to -1 to disable truncation."));
+                           "Symbols beyond specified threashold will be truncated when showing in the grid. Doesn't affect editing field values.\n"
+                           "Set to -1 to disable truncation."));
       tbox->add(entry, false, false);
     }
-
+    
     {
       mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:MySQL:TreatBinaryAsText");
       check->set_text(_("Treat BINARY/VARBINARY as nonbinary character string"));
       check->set_tooltip(_(
-        "Whether to treat binary byte strings as nonbinary character strings.\n"
-        "Binary byte string values do not appear in results grid and are marked as a BLOB values that are supposed to be viewed/edited by means of BLOB editor.\n"
-        "Nonbinary character string values are shown right in results grid and can be edited with either cell's in-place editor or BLOB editor.\n"
-        "Warning: Since binary byte strings tend to contain zero-bytes in their values, turning this option on may lead to data truncation when viewing/editing.\n"
-        "Note: Application restart is needed to get new option value in affect."));
+                           "Whether to treat binary byte strings as nonbinary character strings.\n"
+                           "Binary byte string values do not appear in results grid and are marked as a BLOB values that are supposed to be viewed/edited by means of BLOB editor.\n"
+                           "Nonbinary character string values are shown right in results grid and can be edited with either cell's in-place editor or BLOB editor.\n"
+                           "Warning: Since binary byte strings tend to contain zero-bytes in their values, turning this option on may lead to data truncation when viewing/editing.\n"
+                           "Note: Application restart is needed to get new option value in affect."));
       vbox->add(check, false);
     }
-
+    
     {
       mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:IsDataChangesCommitWizardEnabled");
-      check->set_text(_("Enable Data Changes Commit Wizard"));
-      check->set_tooltip(_(
-        "Whether to use wizard providing more control over applying changes to table data."));
+      check->set_text(_("Confirm Data Changes"));
+      check->set_tooltip(_("Whether to show a dialog confirming changes to be made to a table recordset."));
       vbox->add(check, false);
     }
-
+    
     /*{
-      mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:IsLiveObjectAlterationWizardEnabled");
-      check->set_text(_("Enable Live Object Alteration Wizard"));
-      check->set_tooltip(_(
-        "Whether to use wizard providing more control over applying changes to live database object."));
-      vbox->add(check, false);
-    }*/
+     mforms::CheckBox *check= new_checkbox_option("DbSqlEditor:IsLiveObjectAlterationWizardEnabled");
+     check->set_text(_("Enable Live Object Alteration Wizard"));
+     check->set_tooltip(_(
+     "Whether to use wizard providing more control over applying changes to live database object."));
+     vbox->add(check, false);
+     }*/
+  }
+}
 
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Triggered when the user switches the code completion enabled state. We use this to enable
+ * the auto start code completion check box which is passed in as parameter.
+ */
+void PreferencesForm::code_completion_changed(mforms::CheckBox *cc_box, mforms::CheckBox *auto_cc_box)
+{
+  auto_cc_box->set_enabled(cc_box->get_active());
+  if (!cc_box->get_active())
+    auto_cc_box->set_active(false);
+}
+
+//--------------------------------------------------------------------------------------------------
+static void force_checkbox_on_toggle(mforms::CheckBox *value, mforms::CheckBox *target, bool same_value, bool disable_on_active)
+{
+  if (value->get_active())
+  {
+    target->set_active(!same_value);
+    target->set_enabled(!disable_on_active);
+  }
+  else
+  {
+    //    target->set_active(same_value);
+    target->set_enabled(disable_on_active);
   }
 }
 
@@ -842,7 +940,9 @@ void PreferencesForm::create_general_page()
   table = mforms::manage(new OptionTable(this, _("SQL Editor"), true));
   top_box->add(table, false, true);
   {
-    table->add_checkbox_option("workbench:SaveSQLWorkspaceOnClose",
+    mforms::CheckBox *save_workspace, *discard_unsaved;
+    
+    save_workspace = table->add_checkbox_option("workbench:SaveSQLWorkspaceOnClose",
                                _("Save snapshot of open editors on close"),
                                _("A snapshot of all open scripts is saved when the SQL Editor is closed. Next time it is opened to the same connection that state is restored. Unsaved files will remain unsaved, but their contents will be preserved."));
 
@@ -852,9 +952,15 @@ void PreferencesForm::create_general_page()
       
       table->add_option(sel, _("Auto-save scripts interval:"), 
                         _("Interval to perform auto-saving of all open script tabs.\nThe scripts will be restored from the last auto-saved version\nif Workbench unexpectedly quits."));
-    }    
+    }
+    
+    discard_unsaved = table->add_checkbox_option("DbSqlEditor:DiscardUnsavedQueryTabs",
+                               _("Discard query tabs that were never saved"),
+                               _("Enable if you'd like to automatically discard query tabs that are not stored in a file when closing them.\n"
+                                 "If Snapshot saving is enabled, query tabs are always autosaved to temporary files when the connection is closed."));
+    save_workspace->signal_clicked()->connect(boost::bind(force_checkbox_on_toggle, save_workspace, discard_unsaved, true, true));
+    (*save_workspace->signal_clicked())();
   }
-
   
   table = mforms::manage(new OptionTable(this, _("Others"), true));
   top_box->add(table, false, true);
@@ -864,9 +970,9 @@ void PreferencesForm::create_general_page()
                                  "Clear this checbox to place sidebar on the left side."));
     
 #ifdef _WIN32
-    table->add_checkbox_option("SingleInstance", _("Only allow one instance of MySQL Workbench to run"), 
-      _("Determines if only one or multiple instances of MySQL Workbench can run at the same time for "
-      "a single user.\nThis option requires a restart."));
+    table->add_checkbox_option("DisableSingleInstance", _("Allow more than one instance of MySQL Workbench to run"), 
+      _("By default only one instance of MySQL Workbench can run at the same time.\nThis is more resource friendly "
+        "and necessary as multiple instances share the same files (settings etc.). Change at your own risk."));
 #endif
 
     mforms::Selector *combo= new_selector_option("grtshell:ShellLanguage");
@@ -874,6 +980,14 @@ void PreferencesForm::create_general_page()
                       _("Select the language to use in the interactive GRT shell.\n"
                         "Scripts, modules and plugins will work regardless of this setting.\n"
                         "This option requires a restart."));
+
+    mforms::TextEntry *entry= new_entry_option("workbench:InternalSchema", false);
+    entry->set_max_length(100);
+    entry->set_size(100, -1);
+      
+    table->add_option(entry, _("Internal Workbench Schema:"), 
+                      _("This schema will be used by Workbench.\nto store information required on\ncertain operations."));
+
   }
 
   _tabview.add_page(top_box, _("General"));
@@ -923,14 +1037,14 @@ void PreferencesForm::create_model_page()
 
     mforms::TextEntry *entry;
     
-    table->add(new_label(_("PK Name:"), true), 0, 1, 0, 1, mforms::HFillFlag);
+    table->add(new_label(_("PK Column Name:"), true), 0, 1, 0, 1, mforms::HFillFlag);
     entry= new_entry_option("PkColumnNameTemplate", false);
     entry->set_tooltip(_("Substitutions:\n"
                          "%table% - name of the table\n"
                          "May be used as %table|upper% %table|lower% or %table|capitalize%"));
     table->add(entry, 1, 2, 0, 1, mforms::HFillFlag|mforms::HExpandFlag);
 
-    table->add(new_label(_("PK Type:"), true), 2, 3, 0, 1, mforms::HFillFlag);
+    table->add(new_label(_("PK Column Type:"), true), 2, 3, 0, 1, mforms::HFillFlag);
     entry= new_entry_option("DefaultPkColumnType", false);
     entry->set_tooltip(_("Default type for use in newly added primary key columns.\nSpecify a column type name or a user defined type.\nFlags such as UNSIGNED are not accepted."));
     table->add(entry, 3, 4, 0, 1, mforms::HFillFlag|mforms::HExpandFlag);
@@ -1031,6 +1145,22 @@ void PreferencesForm::create_mysql_page()
     
     box->add(frame, false);
   }
+  
+  {
+    mforms::Panel *frame= mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
+    frame->set_title(_("Forward Engineering and Synchronization"));
+    
+    mforms::Box *tbox= mforms::manage(new mforms::Box(true));
+    mforms::TextEntry *entry;
+    tbox->set_padding(8);
+    
+    frame->add(tbox);
+    tbox->add(new_label(_("SQL_MODE to be used in generated scripts:"), true), false, false);    
+    tbox->add(entry = new_entry_option("SqlGenerator.Mysql:SQL_MODE", false), true, true);
+    entry->set_tooltip(_("The default value of TRADITIONAL is recommended."));
+    
+    box->add(frame, false);
+  }
 
   _tabview.add_page(box, _("Model: MySQL"));
 }
@@ -1228,7 +1358,7 @@ void PreferencesForm::create_appearance_page()
     mforms::TextBox *text;
     
     table->add(new_label(_("Colors available for tables, views etc")), 0, 1, 0, 1, mforms::HFillFlag);
-    text= new mforms::TextBox(mforms::VerticalScrollBar);
+    text= mforms::manage(new mforms::TextBox(mforms::VerticalScrollBar));
     text->set_size(200, 100);
     table->add(text, 0, 1, 1, 2, mforms::FillAndExpand);
     
@@ -1239,7 +1369,7 @@ void PreferencesForm::create_appearance_page()
     option->update_value= boost::bind(update_text_option, get_options(), "workbench.model.ObjectFigure:ColorList", text);
     
     table->add(new_label(_("Colors available for layers, notes etc")), 1, 2, 0, 1, mforms::HFillFlag);
-    text= new mforms::TextBox(mforms::VerticalScrollBar);
+    text= mforms::manage(new mforms::TextBox(mforms::VerticalScrollBar));
     text->set_size(200, 100);
     table->add(text, 1, 2, 1, 2, mforms::FillAndExpand);
 
@@ -1289,27 +1419,36 @@ void PreferencesForm::create_test_page()
 {
   Box* content= manage(new Box(false));
 
-  Box* file_path = manage(new Box(true));
-  file_path->set_spacing(4);
-  Label* labelfile = manage(new Label("   File Path"));
-  TextEntry* file_te = manage(new TextEntry());
-  file_te->set_value("asdlkjasfd");  
-  file_path->add(labelfile, false, false);
-  file_path->add(file_te, true, true);
+  _test_button = manage(new Button());
+  content->add(_test_button, false, true);
 
-  Button* file_btn = manage(new Button());
-  file_btn->set_text("...");
-  file_btn->enable_internal_padding(false);
- // file_btn->signal_clicked().connect(boost::bind(&PreferencesForm::test_clicked, this));
-  file_path->add(file_btn, false, false);
+  _test_button->set_text("popover");
+  scoped_connect(_test_button->signal_clicked(), boost::bind(&PreferencesForm::test_clicked, this));
   
-  content->add(file_path, false, true);
   _tabview.add_page(content, _("Test page"));
 }
 
-void PreferencesForm::test_clicked(const std::string& url)
+void PreferencesForm::test_clicked()
 {
-  Utilities::show_message("Link clicked", url, "OK");
+  static wb::SnippetPopover* popover = NULL;
+
+  if (popover == NULL)
+  {
+    popover = new wb::SnippetPopover();
+    popover->set_size(376, 257);
+    popover->set_read_only(true);
+    int x = _test_button->get_x();
+    int y = _test_button->get_y();
+    _test_button->client_to_screen(x, y);
+    popover->show(x + _test_button->get_width() / 2, y + _test_button->get_height() / 2, mforms::Below);
+  }
+  else
+  {
+    popover->close();
+    delete popover;
+    popover = NULL;
+  }
+  
 }
 
 #endif
@@ -1334,7 +1473,7 @@ void PreferencesForm::show_colors_and_fonts()
   std::vector<std::string> options= _wbui->get_wb_options_keys("");
 
   _font_options.clear();
-  _font_list.clear_rows();
+  _font_list.clear();
   
   for (std::vector<std::string>::const_iterator iter= options.begin();
        iter != options.end(); ++iter)
@@ -1358,11 +1497,11 @@ void PreferencesForm::show_colors_and_fonts()
           
           caption= separate_camel_word(figure) + " " + part;
           
-          int row= _font_list.add_row();
+          mforms::TreeNodeRef node= _font_list.add_node();
           std::string value;
           _wbui->get_wb_options_value("", *iter, value);
-          _font_list.set(row, 0, caption);
-          _font_list.set(row, 1, value);
+          node->set_string(0, caption);
+          node->set_string(1, value);
           
           _font_options.push_back(*iter);
         }
@@ -1378,7 +1517,7 @@ void PreferencesForm::update_colors_and_fonts()
 {
   for (int c= _font_list.count(), i= 0; i < c; i++)
   {
-    std::string value= _font_list.get_string(i, 1);
+    std::string value= _font_list.root_node()->get_child(i)->get_string(1);
 
     _wbui->set_wb_options_value("", _font_options[i], value);
   }
