@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,6 +26,10 @@
 #include <grt/common.h>
 #include "base/string_utilities.h"
 
+#include "mforms/uistyle.h"
+#include "mforms/panel.h"
+#include "mforms/filechooser.h"
+
 #define SYSTEM_STAT_HEADER "# Customize this script to suit your system if necessary\n"\
 "# Shell function names and output format must be maintained\n\n"
 
@@ -42,15 +46,15 @@ static struct SystemStatScript {
 "\n"
 },
 {"Linux", 
-"get_cpu_info = /usr/bin/uptime | wba_token(' ', 10)"
+"get_cpu_info = /usr/bin/uptime | wba_token(' ', -3)"
 "\n"
-"get_mem_info = free | wba_filter(Mem:) | wba_token(' ', 3)"
+"get_mem_info = free | wba_filter(-/+ buffers/cache:) | wba_token(' ', 3)"
 "\n"
 "get_mem_total = free | wba_filter(Mem:) | wba_token(' ', 1)"
 "\n"
 },
 {"MacOS X",
-"get_cpu_info = /usr/bin/uptime | wba_token(' ', 10)"
+"get_cpu_info = /usr/bin/uptime | wba_token(' ', -3)"
 "\n"
 "get_mem_info = vm_stat | wba_filter(free) | wba_token(':', 1)"
 "\n"
@@ -58,7 +62,7 @@ static struct SystemStatScript {
 "\n"
 },
 {"OpenSolaris",
-"get_cpu_info = /usr/bin/uptime | wba_token(' ', 10)"
+"get_cpu_info = /usr/bin/uptime | wba_token(' ', -3)"
 "\n"
 "get_mem_info = free | wba_filter(Mem:) | wba_token(' ', 1, 2)"
 "\n"
@@ -66,7 +70,7 @@ static struct SystemStatScript {
 "\n"
 },
 {"FreeBSD",
-"get_cpu_info = /usr/bin/uptime | wba_token(' ', 10)"
+"get_cpu_info = /usr/bin/uptime | wba_token(' ', -3)"
 "\n"
 "get_mem_info = free | wba_filter(Mem:) | wba_token(' ', 1, 2)"
 "\n"
@@ -112,7 +116,8 @@ static const char *filter_help =
 "wba_line(<line number>)\n"
 "  Skips all lines in the output except the given one\n"
 "wba_token(<separator>, <token number>)\n"
-"  Splits line into pieces using separator and returns given numbered token\n"
+"  Splits line into pieces using separator and returns given numbered token.\n"
+"  token number can have negative values to refer to a token counting in reverse order\n"
 "wba_filter(<pattern>)\n"
 "  Removes all lines which do not contain <pattern>\n"
 "wba_arithm(<op>, <arg>)\n"
@@ -181,9 +186,12 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
 , _top_hbox(true)
 , _content_box(false)
 , _inst_list_buttons_hbox(true)
-, _stored_instance_list(TreeDefault)
+, _stored_instance_list(TreeDefault|TreeFlatList)
 , _tabview(mforms::TabViewSystemStandard)
 , _conn_box(false)
+, _no_remote_admin(RadioButton::new_id())
+, _win_remote_admin(_no_remote_admin.group_id())
+, _ssh_remote_admin(_no_remote_admin.group_id())
 , _remote_param_box(false)
 , _password_box(true)
 , _sys_box(false)
@@ -191,9 +199,6 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
 , _sudo_cmd(mforms::SelectorCombobox)
 , _script_text(VerticalScrollBar)
 , _bottom_hbox(true)
-, _no_remote_admin(RadioButton::new_id())
-, _win_remote_admin(_no_remote_admin.group_id())
-, _ssh_remote_admin(_no_remote_admin.group_id())
 {
   _mgmt= mgmt;
   _instances= mgmt->storedInstances();
@@ -464,7 +469,12 @@ ServerInstanceEditor::ServerInstanceEditor(bec::GRTManager *grtm, const db_mgmt_
     _sys_box.add(&_details_tabview, true, true);
     
     table = NewTable(6, 2);
+#if defined(_WIN32) || defined(__APPLE__)
     _details_tabview.add_page(manage(table), _("MySQL Management"));
+#else
+    // XXX tmp workaround for crash caused by changed destruction routine
+    _details_tabview.add_page(table, _("MySQL Management"));
+#endif
     
     table->add(manage(RLabel(_("Start MySQL:"))), 0, 1, 0, 1, HFillFlag);
     table->add(&_start_cmd, 1, 2, 0, 1, HFillFlag | HExpandFlag);
@@ -749,7 +759,7 @@ db_mgmt_ServerInstanceRef ServerInstanceEditor::run()
   refresh_options();
   refresh_instance_list();
     
-  _stored_instance_list.set_selected(0);
+  _stored_instance_list.select_node(_stored_instance_list.node_at_row(0));
   instance_changed();
   run_modal(&_ok_button, NULL);
 
@@ -873,23 +883,21 @@ void ServerInstanceEditor::refresh_profile_list()
 
 void ServerInstanceEditor::refresh_instance_list()
 {
-  _stored_instance_list.clear_rows();
+  _stored_instance_list.clear();
 
   GRTLIST_FOREACH(db_mgmt_ServerInstance, _instances, inst)
   {
-    int i = _stored_instance_list.add_row();
-    if (i >= 0)
-    {
-      _stored_instance_list.set(i, 0, (*inst)->name().c_str());
-    }
+    TreeNodeRef node = _stored_instance_list.root_node()->add_child();
+    node->set_string(0, *(*inst)->name());
   }
 }
 
 db_mgmt_ServerInstanceRef ServerInstanceEditor::selected_instance()
 {
-  int i= _stored_instance_list.get_selected();
-  if (i >= 0)
-    return _instances[i];
+  TreeNodeRef node= _stored_instance_list.get_selected_node();
+  int row = _stored_instance_list.row_for_node(node);
+  if (row >= 0)
+    return _instances[row];
 
   return db_mgmt_ServerInstanceRef();
 }
@@ -975,8 +983,9 @@ void ServerInstanceEditor::add_instance()
 {
   db_mgmt_ServerInstanceRef instance(_instances.get_grt());
   std::string name= "new instance";
-  int i= 1;
+  TreeNodeRef node;
   bool dupe;
+  int i = 1;
   do
   {
     dupe= false;
@@ -991,10 +1000,10 @@ void ServerInstanceEditor::add_instance()
     }
   } while (dupe);
   
-  i = _stored_instance_list.add_row();
-  if (i >= 0)
+  node = _stored_instance_list.root_node()->add_child();
+  if (node)
   {
-    _stored_instance_list.set(i, 0, name);
+    node->set_string(0, name);
 
     instance->owner(_mgmt);
     instance->name(name);
@@ -1018,7 +1027,7 @@ void ServerInstanceEditor::add_instance()
     }
     
     _instances.insert(instance);
-    _stored_instance_list.set_selected(i);
+    _stored_instance_list.select_node(node);
   }
   else
     g_warning("add_row returned -1");
@@ -1034,10 +1043,11 @@ void ServerInstanceEditor::add_instance()
 
 void ServerInstanceEditor::delete_instance()
 {
-  int i= _stored_instance_list.get_selected();
-  if (i >= 0 && i < (int)_instances.count())
+  TreeNodeRef node(_stored_instance_list.get_selected_node());
+  int row = _stored_instance_list.row_for_node(node);
+  if (row >= 0 && row < (int)_instances.count())
   {
-    _instances.remove(i);
+    _instances.remove(row);
     refresh_instance_list();
   }
 }
@@ -1052,8 +1062,8 @@ void ServerInstanceEditor::duplicate_instance()
     return;
   
   std::string name= orig->name();
-  int i= 1;
   bool dupe;
+  int i = 1;
   do
   {
     dupe= false;
@@ -1068,10 +1078,10 @@ void ServerInstanceEditor::duplicate_instance()
     }
   } while (dupe);
   
-  i = _stored_instance_list.add_row();
-  if (i >= 0)
+  TreeNodeRef node(_stored_instance_list.root_node()->add_child());
+  if (node)
   {
-    _stored_instance_list.set(i, 0, name);
+    node->set_string(0, name);
     
     instance->owner(_mgmt);
     instance->name(name);
@@ -1080,7 +1090,7 @@ void ServerInstanceEditor::duplicate_instance()
     grt::merge_contents(instance->serverInfo(), orig->serverInfo(), true);
     
     _instances.insert(instance);
-    _stored_instance_list.set_selected(i);
+    _stored_instance_list.select_node(node);
   }
   else
     g_warning("add_row returned -1");
@@ -1091,32 +1101,32 @@ void ServerInstanceEditor::duplicate_instance()
 
 void ServerInstanceEditor::reorder_instance(bool up)
 {
-  int i= _stored_instance_list.get_selected();
+  int row = _stored_instance_list.get_selected_row();
 
-  if (i < 0)
+  if (row < 0)
     return;
 
   if (up)
   {
-    if (i > 0)
+    if (row > 0)
     {
-      _instances.reorder(i, i-1);
-      _stored_instance_list.set_selected(i-1);
+      _instances.reorder(row, row-1);
+      _stored_instance_list.select_node(_stored_instance_list.node_at_row(row-1));
     }
   }
   else
   {
-    if (i < _stored_instance_list.count()-1)
+    if (row < _stored_instance_list.root_node()->count()-1)
     {
-      _instances.reorder(i, i+1);
-      _stored_instance_list.set_selected(i+1);
+      _instances.reorder(row, row+1);
+      _stored_instance_list.select_node(_stored_instance_list.node_at_row(row+1));
     }
   }
 
-  i= 0;
+  row= 0;
   GRTLIST_FOREACH(db_mgmt_ServerInstance, _instances, inst)
   {
-    _stored_instance_list.set(i++, 0, (*inst)->name().c_str());
+    _stored_instance_list.root_node()->get_child(row++)->set_string(0, (*inst)->name().c_str());
   }  
 }
 
@@ -1162,8 +1172,10 @@ void ServerInstanceEditor::entry_changed(mforms::TextEntry *sender)
   {
     if (&_name_entry == sender)
     {
+      TreeNodeRef node(_stored_instance_list.get_selected_node());
       instance->name(value);
-      _stored_instance_list.set(_stored_instance_list.get_selected(), 0, value);
+      if (node)
+        node->set_string(0, value);
     }
     else if (&_server_version == sender)
       instance->serverInfo().gset("serverVersion", value);
@@ -1437,9 +1449,9 @@ void ServerInstanceEditor::refresh_options()
 
 void ServerInstanceEditor::instance_changed()
 {
-  int i= _stored_instance_list.get_selected();
+  TreeNodeRef node(_stored_instance_list.get_selected_node());
   
-  if (i >= 0)
+  if (node)
   {
     _content_box.set_enabled(true);
     _move_up_button.set_enabled(true);
@@ -1461,12 +1473,12 @@ void ServerInstanceEditor::instance_changed()
 
 void ServerInstanceEditor::show_instance()
 {
-  const int i = _stored_instance_list.get_selected();
+  int row = _stored_instance_list.get_selected_row();
   db_mgmt_ServerInstanceRef instance;
 
-  if (i >= 0)
+  if (row >= 0)
   {
-    instance= _instances[i];
+    instance= _instances[row];
     _del_inst_button.set_enabled(true);
     _dup_inst_button.set_enabled(true);
   }
