@@ -16,7 +16,7 @@ protected:
 public:
   sigc::signal<void,std::string> signal_title_changed() { return _title_changed; }
 
-  virtual std::string get_title()= 0;
+  std::string get_title() { return get_form()->get_title(); }
   virtual Gtk::Widget *get_panel()= 0;
 
   virtual bec::UIForm *get_form() const= 0;
@@ -33,12 +33,16 @@ public:
 
   virtual void dispose() {}
 
+  virtual bool perform_command(const std::string &cmd) { return false; }
+
 private:
   bool close_plugin_tab(PluginEditorBase *editor);
   
 protected:
   sigc::slot<void, PluginEditorBase*> _close_editor;
 
+  virtual void plugin_tab_added(PluginEditorBase *plugin) {};
+
 public:
   void set_close_editor_callback(const sigc::slot<void, PluginEditorBase*> &handler);
 
