@@ -2,7 +2,6 @@
 #include "gtk_helpers.h"
 #include <gtkmm/cellrenderercombo.h>
 #include <gtkmm/icontheme.h>
-#include "custom_renderers.h"
 #include "grt/common.h"
 
 //#define DEBUG
@@ -33,14 +32,14 @@ static void run_menu_and_forward_action(const bec::MenuItemList& items
                                         ,bec::ListModel* model
                                         ,const std::vector<bec::NodeId>& nodes
                                         ,const sigc::slot<void, const std::string&, const std::vector<bec::NodeId>&> fe_menu_handler
-                                        )
+                                        ,Gtk::Menu *menu)
 {
   if (items.size() > 0)
   {
     run_popup_menu(items
                    ,time
                    ,sigc::bind(sigc::ptr_fun(process_menu_actions), model, nodes, fe_menu_handler)
-                   );
+                   ,menu);
   }
 }
 
@@ -76,7 +75,7 @@ Index::Index(GtkTreeIter* it, const bec::NodeId& node)
   }
   else if (m == ListNode)
   {
-    it->user_data = (void*)(node[0]);
+    it->user_data = (void*)(intptr_t)(node[0]);
   }
 }
 
@@ -506,6 +505,16 @@ void ColumnsModel::add_model_column(Gtk::TreeModelColumnBase *col, int bec_tm_id
 
 //------------------------------------------------------------------------------
 //------------------------------------------------------------------------------
+
+struct LMPtr {
+  ListModelWrapper *ptr;
+  int count;
+
+  LMPtr(ListModelWrapper *p) : ptr(p), count(1) {}
+  LMPtr *retain() { count++; return this; }
+  void release() { if (--count == 0) delete this; }
+};
+
 ListModelWrapper::ListModelWrapper(bec::ListModel* tm, Gtk::TreeView *treeview, const std::string& name)
                     : Glib::ObjectBase(typeid(ListModelWrapper))
                     , Glib::Object()
@@ -513,41 +522,45 @@ ListModelWrapper::ListModelWrapper(bec::ListModel* tm, Gtk::TreeView *treeview,
                     , _tm(tm)
                     , _treeview(treeview)
                     , _iconview(0)
+                    , _context_menu(0)
                     , _stamp(1)
                     , _columns(this, treeview)
                     , _icon_size(bec::Icon16)
-                    , _self_ref(new ListModelWrapper*(this))
+                    , _self_ref(new LMPtr(this))
                     , _name(name)
 {
-  tm->tree_changed_signal()->connect(sigc::mem_fun(*this, &::ListModelWrapper::model_changed));
-  tm->add_destroy_notify_callback(_self_ref, &ListModelWrapper::on_bec_model_destroyed);
+  scoped_connect(tm->tree_changed_signal(), boost::bind(&::ListModelWrapper::model_changed, this, _1, _2));
+  tm->add_destroy_notify_callback(_self_ref->retain(), &ListModelWrapper::on_bec_model_destroyed);
   #ifndef NO_MENU_MANAGER
   if (_treeview)
     _treeview->signal_event().connect(sigc::mem_fun(this, &ListModelWrapper::handle_popup_event));
   #endif
 }
 
-
+//------------------------------------------------------------------------------
 void* ListModelWrapper::on_bec_model_destroyed(void* data)
 {
-  ListModelWrapper** self = (ListModelWrapper**)data;
-  if (self && *self)
+  LMPtr* self = (LMPtr*)data;
+  if (self)
   {
-    (*self)->_tm = 0;
+    self->ptr->_tm = 0;
+    self->release();
   }
-  delete self;
   return 0;
 }
 
+//------------------------------------------------------------------------------
 ListModelWrapper::~ListModelWrapper()
 {
+  delete _context_menu;
   if (_tm)
     _tm->remove_destroy_notify_callback(_self_ref);
-  *_self_ref= 0;
+  _self_ref->release();
 
   _tm= 0;
 }
 
+//------------------------------------------------------------------------------
 void ListModelWrapper::set_iconview(Gtk::IconView* iv)
 {
   _iconview = iv;
@@ -557,41 +570,43 @@ void ListModelWrapper::set_iconview(Gtk::IconView* iv)
   #endif
 }
 
+//------------------------------------------------------------------------------
 ListModelWrapper::NodeIdArray ListModelWrapper::get_selection() const
 {
-  NodeIdArray nodes;
+  NodeIdArray                 nodes;
+  std::vector<Gtk::TreePath>  paths;
 
   if (_treeview)
-  {
-    const std::vector<Gtk::TreePath> paths = _treeview->get_selection()->get_selected_rows();
-    const int size = paths.size();
-    nodes.reserve(size);
-    for (int i = 0; i < size; ++i)
-      nodes.push_back(get_node_for_path(paths[i]));
-  }
+    paths = _treeview->get_selection()->get_selected_rows();
   else if (_iconview)
-  {
-    const std::vector<Gtk::TreePath> paths = _iconview->get_selected_items();
-    const int size = paths.size();
-    nodes.reserve(size);
-    for (int i = 0; i < size; ++i)
-      nodes.push_back(get_node_for_path(paths[i]));
-  }
-  
+    paths = _iconview->get_selected_items();
+
+  const int size = paths.size();
+  nodes.reserve(size);
+  for (int i = 0; i < size; ++i)
+    nodes.push_back(get_node_for_path(paths[i]));
+
   return nodes;
 }
 
+//------------------------------------------------------------------------------
 void ListModelWrapper::set_be_model(bec::ListModel* tm)
 {
   if (_tm)
+  {
     _tm->remove_destroy_notify_callback(_self_ref);
+    _self_ref->release();
+  }
   _tm = tm;
   if (_tm)
-    _tm->add_destroy_notify_callback(_self_ref, &ListModelWrapper::on_bec_model_destroyed);
+  {
+    _tm->add_destroy_notify_callback(_self_ref->retain(), &ListModelWrapper::on_bec_model_destroyed);
+  }
 }
 
 #ifndef NO_MENU_MANAGER
-void ListModelWrapper::handle_popup(const int x, const int y, const int time)
+//------------------------------------------------------------------------------
+void ListModelWrapper::handle_popup(const int x, const int y, const int time, GdkEventButton* evb)
 {
   Gtk::TreeModel::Path   path;
   Gtk::TreeView::Column *column(0);
@@ -608,11 +623,12 @@ void ListModelWrapper::handle_popup(const int x, const int y, const int time)
     path = _iconview->get_path_at_pos(x, y);
     there_is_path_at_pos = path.gobj() && !path.empty();
   }
-  
-  if ( there_is_path_at_pos )
+
+  if ( !there_is_path_at_pos )
+    list.clear();
+  else
   {
-    // Check that popup is requested on selection, otherwise unselect all and select
-    // path_at_pos
+    // Check that @path is on selection, otherwise add @path to selection
     bec::NodeId node = get_node_for_path(path);
     // list stores current selection
     bool path_at_pos_is_in_selection = false;
@@ -625,36 +641,42 @@ void ListModelWrapper::handle_popup(const int x, const int y, const int time)
       }
     }
 
-    // If path is not in current selection unselect old selection and
-    // select path at pos 
     if (!path_at_pos_is_in_selection)
     {
-      if (_treeview)
+      // Add it, if user holds Ctrl while clicking right mouse btn
+      // Otherwise clear selection, and select only @path
+      const bool clear_selection = evb ? (!(evb->state & GDK_CONTROL_MASK)) : false;
+      if (clear_selection)
       {
-        _treeview->get_selection()->unselect_all();
-        _treeview->get_selection()->select(path);
-        // update list of selected nodes as well
-      }
-      else if (_iconview)
-      {
-        _iconview->unselect_all();
-        _iconview->select_path(path);
+        if (_treeview)
+          _treeview->get_selection()->unselect_all();
+        if (_iconview)
+          _iconview->unselect_all();
       }
 
-      list.clear();
-      list.push_back(node);
+      if (_treeview)
+        _treeview->get_selection()->select(path);
+      if (_iconview)
+        _iconview->select_path(path);
+
+      list = get_selection();
     }
 
-    run_menu_and_forward_action(_tm->get_popup_items_for_nodes(list), x, y, time, _tm, list, _fe_menu_handler);
+    if (!_context_menu)
+      _context_menu = new Gtk::Menu();
+
+    run_menu_and_forward_action(_tm->get_popup_items_for_nodes(list), x, y, time, _tm, list, _fe_menu_handler, _context_menu);
   }
 }
 
+//------------------------------------------------------------------------------
 bool ListModelWrapper::handle_popup_event(GdkEvent* event)
 {
   bool ret = false;
   if ( event->type == GDK_BUTTON_PRESS && event->button.button == 3 )
   {
-    handle_popup((int)event->button.x, (int)event->button.y, event->button.time);
+    GdkEventButton* evb = (GdkEventButton*)event;
+    handle_popup((int)event->button.x, (int)event->button.y, event->button.time, evb);
     // Return true to preserve selection. In handle_popup check that popup is requested
     // on selection.
     ret = true;
@@ -663,11 +685,13 @@ bool ListModelWrapper::handle_popup_event(GdkEvent* event)
 }
 #endif
 
+//------------------------------------------------------------------------------
 Gtk::TreeModelFlags ListModelWrapper::get_flags_vfunc() const
 {
   return Gtk::TREE_MODEL_ITERS_PERSIST | Gtk::TREE_MODEL_LIST_ONLY;
 }
 
+//------------------------------------------------------------------------------
 bec::NodeId ListModelWrapper::get_node_for_path(const Gtk::TreeModel::Path &path) const
 {
   if (path.empty())
@@ -877,7 +901,7 @@ Gtk::TreeModel::Path ListModelWrapper::get_path_vfunc(const iterator& iter) cons
   return path;
 }
 
-
+//------------------------------------------------------------------------------
 void ListModelWrapper::get_icon_value(const iterator& iter, int column, const bec::NodeId &node, Glib::ValueBase& value) const
 {
   if (!_tm)
@@ -1057,23 +1081,26 @@ void ListModelWrapper::get_value_impl(const iterator& row, int column, Glib::Val
   get_value_vfunc(row, column, value);
 }
 
-
+//------------------------------------------------------------------------------
 void ListModelWrapper::set_row_draggable_slot(const sigc::slot<bool, Gtk::TreeModel::Path> &slot)
 {
   _row_draggable= slot;
 }
 
+//------------------------------------------------------------------------------
 bool ListModelWrapper::drag_data_get_vfunc(const Gtk::TreeModel::Path& path, Gtk::SelectionData& selection_data) const
 {
   selection_data.set("text/path", path.to_string());
   return true;
 }
 
+//------------------------------------------------------------------------------
 bool ListModelWrapper::drag_data_delete_vfunc (const Gtk::TreeModel::Path& path)
 {
   return true;
 }
 
+//------------------------------------------------------------------------------
 bool ListModelWrapper::row_draggable_vfunc(const TreeModel::Path &path) const
 {
   if (_row_draggable)
@@ -1082,11 +1109,13 @@ bool ListModelWrapper::row_draggable_vfunc(const TreeModel::Path &path) const
   return true;
 }
 
+//------------------------------------------------------------------------------
 bool ListModelWrapper::row_drop_possible_vfunc(const Gtk::TreeModel::Path& dest, const Gtk::SelectionData& selection_data) const
 {
   return true;
 }
 
+//------------------------------------------------------------------------------
 bool ListModelWrapper::drag_data_received_vfunc(const Gtk::TreeModel::Path& dest, const Gtk::SelectionData& selection_data)
 {
   bool ret = false;
