@@ -3,6 +3,10 @@
 #include "linux_utilities/gtk_helpers.h"
 #include "base/string_utilities.h"
 #include "linux_utilities/toolbar_manager.h"
+#include "mforms/utilities.h"
+#include "base/log.h"
+
+DEFAULT_LOG_DOMAIN("RecordsetView");
 
 using base::strfmt;
 
@@ -18,7 +22,7 @@ RecordsetView * RecordsetView::create(Recordset::Ref model, Gtk::Container *pare
 
 RecordsetView::RecordsetView(Recordset::Ref model)
 :
-_grid(NULL)
+_grid(NULL), _single_row_height(-1)
 {
   _filter_entry= 0;
   this->model(model);
@@ -38,22 +42,14 @@ void RecordsetView::init()
   Gtk::ScrolledWindow *recordset_placeholder = 0;
   xml->get_widget("recordset_placeholder", recordset_placeholder);
 
-  _grid= GridView::create(_model);
-  
-  // get_selected() doesn't work work with SELECTION_MULTIPLE
-  _grid->get_selection()->set_mode(Gtk::SELECTION_SINGLE);
-  
+  _grid= GridView::create(_model, false); // XXX put back to true when column-autosizing is fixed
+  _grid->get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);
+ 
   recordset_placeholder->add(*_grid);
 
   recordset_view->reparent(*this);
   set_shadow_type(Gtk::SHADOW_NONE);
 
-  //WIDGET_ (xml, messages_view) // auto-replaced
-  xml->get_widget("messages_view", _messages_view);
-  //WIDGET_ (xml, messages_window) // auto-replaced
-  xml->get_widget("messages_window", _messages_window);
-  _messages_window->property_height_request()= 1;
-
   _toolbar_box = 0;
   xml->get_widget("recordset_toolbar", _toolbar_box);
   
@@ -63,7 +59,7 @@ void RecordsetView::init()
   action_list.register_action("record_next", sigc::mem_fun(this, &RecordsetView::on_record_next));
   action_list.register_action("record_last", sigc::mem_fun(this, &RecordsetView::on_goto_last_row_btn_clicked));
   //action_list.register_action("record_fetch_all", sigc::mem_fun(this, &RecordsetView::));
-  //action_list.register_action("record_wrap_vertical", sigc::mem_fun(this, &RecordsetView::));
+  action_list.register_action("record_wrap_vertical", sigc::mem_fun(this, &RecordsetView::on_toggle_vertical_sizing));
   action_list.register_action("record_sort_asc", sigc::mem_fun(this, &RecordsetView::on_record_sort_asc));
   action_list.register_action("record_sort_desc", sigc::mem_fun(this, &RecordsetView::on_record_sort_desc));
   action_list.register_action("record_edit", sigc::mem_fun(this, &RecordsetView::on_record_edit));
@@ -78,6 +74,7 @@ void RecordsetView::init()
   update_toolbar();
   
   show();
+
 }
 
 void RecordsetView::model(Recordset::Ref value)
@@ -85,36 +82,12 @@ void RecordsetView::model(Recordset::Ref value)
   _model= value;
   _model->refresh_ui_cb= sigc::mem_fun(this, &RecordsetView::refresh);
   _model->refresh_ui_status_bar_signal.connect(sigc::mem_fun(this, &RecordsetView::update_toolbar));
-  _model->task->msg_cb(sigc::mem_fun(this, &RecordsetView::process_task_msg));
+  //_model->task->msg_cb(sigc::mem_fun(this, &RecordsetView::process_task_msg));
 
   if (_grid)
     _grid->model(_model);
 }
 
-int RecordsetView::process_task_msg(int msg_type, const std::string &message, const std::string &detail)
-{
-  _messages_view->get_buffer()->set_text(message);
-
-  _messages_window->property_height_request()= message.empty() ? 1 :
-    ((1 < _messages_window->get_height()) ? _messages_window->get_height() : 26);
-
-  switch ((grt::MessageType)msg_type)
-  {
-    case grt::ErrorMsg:
-      _messages_view->modify_text(Gtk::STATE_NORMAL, Gdk::Color("red"));
-      break;
-    case grt::WarningMsg:
-    case grt::InfoMsg:
-      _messages_view->modify_text(Gtk::STATE_NORMAL, Gdk::Color("black"));
-      break;
-    default:
-      break;
-  }
-
-  return 0;
-}
-
-
 Gtk::Widget *RecordsetView::create_toolbar_item(const bec::ToolbarItem &item)
 {
   if (item.command == "filter")
@@ -136,7 +109,6 @@ void RecordsetView::update_toolbar()
   // force a repaint of the records in case the contents have changed too
   _grid->queue_draw();
 
-
   if (_filter_entry)
     text = _filter_entry->get_text();
   
@@ -153,10 +125,19 @@ void RecordsetView::update_toolbar()
 
 bool RecordsetView::activate_toolbar_item(const std::string &action)
 {
-  bool r = _model->action_list().trigger_action(action);
-  if (r)
-    update_toolbar();
-  return r;
+  try
+  {
+    bool r = _model->action_list().trigger_action(action);
+    if (r)
+      update_toolbar();
+    return r;
+  }
+  catch (const std::exception &exc)
+  {
+    log_error("Unhandled exception in activate_toolbar_item(%s): %s", action.c_str(), exc.what());
+    mforms::Utilities::show_error(_("Unhandled Error"), exc.what(), "OK", "", "");
+  }
+  return false;
 }
 
 
@@ -170,9 +151,34 @@ int RecordsetView::refresh()
   _grid->refresh(true);
   update_toolbar();
 
+  // calculate the height of a row with single line of text
+  if (_grid->view_model()->row_numbers_visible())
+  {
+    Gtk::TreeViewColumn *col = _grid->get_column(0);
+    Gtk::CellRenderer *rend = col ? col->get_first_cell_renderer() : 0;
+    if (rend)
+    {
+      int x, y, w, h;
+      rend->get_size(*_grid, x, y, w, h);
+      _single_row_height = h;
+    }
+  }
+ 
+  if (_grid->get_fixed_height_mode())
+    set_fixed_row_height(_single_row_height);
+  else
+    set_fixed_row_height(-1);
   return 0;
 }
 
+
+void RecordsetView::activate_popup_menu_item(const std::string &action, const std::vector<int> &rows, int clicked_column)
+{
+  if (_model.get()->activate_popup_menu_item(action, rows, clicked_column))
+    _grid->sync_row_count();
+}
+
+
 bool RecordsetView::on_event(GdkEvent *event)
 {
   bool processed= false;
@@ -182,34 +188,65 @@ bool RecordsetView::on_event(GdkEvent *event)
     if (3 == event->button.button)
     {
       Gtk::StateType state= _model->is_readonly() ? Gtk::STATE_INSENSITIVE : Gtk::STATE_NORMAL;
-      std::vector<int> rows;
+      std::vector<int> rows = _grid->get_selected_rows();
       Gtk::TreePath path;
       Gtk::TreeViewColumn *tcolumn;
-      int cell_x, cell_y;
 
-      if (_grid->get_path_at_pos(event->button.x, event->button.y, path, tcolumn, cell_x, cell_y))
+      if (rows.size() <= 1)
       {
-        _grid->get_selection()->select(path);
-        int column = 0;
+        int cell_row, cell_col;
+        _grid->current_cell(cell_row, cell_col);
 
-        std::vector<Gtk::TreeViewColumn*> columns(_grid->get_columns());
-        for (int i = 0; i < (int)columns.size(); i++)
+        int cell_x, cell_y;
+        // select the item under the cursor
+        if (_grid->get_path_at_pos(event->button.x, event->button.y, path, tcolumn, cell_x, cell_y))
         {
-          if (columns[i] == tcolumn)
+          int column = 0;
+
+          std::vector<Gtk::TreeViewColumn*> columns(_grid->get_columns());
+          for (int i = 0; i < (int)columns.size(); i++)
           {
-            column = i;
-            break;
+            if (columns[i] == tcolumn)
+            {
+              column = i;
+              break;
+            }
+          }
+
+          if (cell_row < 0 || cell_col < 0)
+            _grid->get_selection()->select(path);
+          else
+          {
+            // if a cell is already selected, then reselect the cell that was right-clicked
+            rows.push_back(path[0]);
+            _grid->select_cell(path[0], column-1);
+            _grid->get_selection()->unselect_all();
+          }
+
+          run_popup_menu(_model->get_popup_menu_items(rows, column-1),
+                         event->button.time, sigc::bind(sigc::mem_fun(this, &RecordsetView::activate_popup_menu_item), rows, column-1),
+			 &_context_menu);
+        }
+      }
+      else
+      {
+        int column = 0;
+        int dummy;
+        if (_grid->get_path_at_pos(event->button.x, event->button.y, path, tcolumn, dummy, dummy))
+        {
+          std::vector<Gtk::TreeViewColumn*> columns(_grid->get_columns());
+          for (int i = 0; i < (int)columns.size(); i++)
+          {
+            if (columns[i] == tcolumn)
+            {
+              column = i;
+              break;
+            }
           }
         }
-        int row = path.front();//_grid->get_selection()->get_selected();
-        if (row >= 0 && column > 0)
-        {
-          rows.push_back(row);
-          
-          run_popup_menu(_model->get_popup_menu_items(rows, column-1),
-                         event->button.time,
-                         sigc::hide_return(sigc::bind(sigc::mem_fun(_model.get(), &Recordset::activate_popup_menu_item), rows, column-1)));
-        }
+        run_popup_menu(_model->get_popup_menu_items(rows, column-1),
+                         event->button.time, sigc::bind(sigc::mem_fun(this, &RecordsetView::activate_popup_menu_item), rows, column-1),
+			 &_context_menu);
       }
       processed= true;
     }
@@ -337,7 +374,13 @@ void RecordsetView::on_record_del()
 {
   if (_model->is_readonly())
     return;
-  _grid->delete_selected_rows();
+  std::vector<int> rows = _grid->get_selected_rows();
+  std::vector<bec::NodeId> nodes;
+  for (size_t i = 0; i < rows.size(); i++)
+    nodes.push_back(rows[i]);
+
+//  _grid->delete_selected_rows();
+  _model->delete_nodes(nodes);
 }
 
 void RecordsetView::on_record_sort_asc()
@@ -357,3 +400,33 @@ void RecordsetView::on_record_sort_desc()
     return;
   _grid->sort_by_column(col, 1, true);
 }
+
+void RecordsetView::on_toggle_vertical_sizing()
+{
+  if (!_grid->get_fixed_height_mode())
+  {
+    std::vector<Gtk::TreeViewColumn*> columns = _grid->get_columns();
+    for (std::vector<Gtk::TreeViewColumn*>::iterator iter = columns.begin(); iter != columns.end(); ++iter)
+      (*iter)->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
+  }
+  _grid->set_fixed_height_mode(!_grid->get_fixed_height_mode());
+  refresh();
+}
+
+void RecordsetView::set_fixed_row_height(int height)
+{
+  if (_grid && _grid->view_model())
+  {
+    std::vector<Gtk::TreeViewColumn*> columns = _grid->get_columns();
+    if (_grid->view_model()->row_numbers_visible())
+      columns.erase(columns.begin());
+
+    for (std::vector<Gtk::TreeViewColumn*>::iterator iter = columns.begin(); iter != columns.end(); ++iter)
+    {
+      std::vector<Gtk::CellRenderer*> cells((*iter)->get_cell_renderers());
+      for (std::vector<Gtk::CellRenderer*>::iterator cell = cells.begin(); cell != cells.end(); ++cell)
+        (*cell)->set_fixed_size(-1, height);
+    }
+  }
+}
+
