@@ -12,7 +12,9 @@
 #include <gtkmm/progressbar.h>
 #include <gtkmm/builder.h>
 
-#include <mforms.h>
+#include "mforms.h"
+#include "mforms/dockingpoint.h"
+#include "base/notifications.h"
 
 namespace bec
 {
@@ -30,7 +32,7 @@ namespace Gtk
 }
 
 
-class SqlEditorFE;
+class ActiveLabel;
 class FormViewBase;
 class ModelPanel;
 class OverviewPanel;
@@ -45,7 +47,7 @@ class PluginEditorBase;
 //==============================================================================
 //
 //==============================================================================
-class MainForm : public sigc::trackable
+class MainForm : public sigc::trackable, base::Observer, public mforms::DockingPointDelegate
 {
 public:
   MainForm(wb::WBContextUI* ctx);
@@ -82,6 +84,8 @@ public:
   wb::WBContextUI* get_wbui_context() const { return _wbui_context; }
 
 private:
+  virtual void handle_notification(const std::string &name, void *sender, base::NotificationInfo &info);
+
   void register_commands();
   ModelDiagramPanel *get_panel_for_view(mdc::CanvasView *view);
   
@@ -98,11 +102,16 @@ private:
   // 
 
   void close_active_tab();
+  void close_main_tab();
+  void close_inner_tab();
   
   void close_tab(Gtk::Notebook *note, Gtk::Widget *widget);
   void append_tab_page(Gtk::Notebook *note, Gtk::Widget *widget, const std::string &title,
                        bool auto_switch, 
-                       Gtk::Label **title_label_ret= 0);
+                       ActiveLabel **title_label_ret= 0);
+
+  void init_tab_menu(mforms::Menu* m);
+  void tab_menu_handler(const std::string& action, ActiveLabel* label, Gtk::Widget* widget, Gtk::Notebook* note);
 
   void show_about();
   void show_find();
@@ -115,6 +124,8 @@ private:
   
   bool close_window(GdkEventAny *ev);
   void on_focus_widget(Gtk::Widget* focus);
+  void on_configure_window(GdkEventConfigure *conf);
+  void on_window_state(GdkEventWindowState *conf);
   
   void prepare_close_document();
 
@@ -155,14 +166,17 @@ private:
   // mforms integration
   void setup_mforms_app();
 
-  static void dock_view(mforms::App *app, mforms::AppView *view, const std::string &position);
-  static bool select_view(mforms::App *app, const std::string &id);
-  static void undock_view(mforms::App *app, mforms::AppView *view);
-  static void set_status_text(mforms::App *app, const std::string &text);
-  static void set_view_title(mforms::App *app, mforms::AppView *view, const std::string &title);
+  virtual std::string get_type() { return "MainWindow"; }
+  virtual void dock_view(mforms::AppView *view, const std::string &position, int arg);
+  virtual bool select_view(mforms::AppView *view);
+  virtual void undock_view(mforms::AppView *view);
+  virtual void get_size(int *w, int *h);
+  virtual void set_view_title(mforms::AppView *view, const std::string &title);
 
-  bool find_callback(const std::string &search, const std::string &replace, 
-                     mforms::SearchFlags flags, SqlEditorFE *editor);
+  static void set_status_text(mforms::App *app, const std::string &text);
+
+//@@@  bool find_callback(const std::string &search, const std::string &replace, 
+//                     mforms::SearchFlags flags, SqlEditorFE *editor);
   
   Gtk::Widget *decorate_widget(Gtk::Widget *panel, bec::UIForm *form);
 private:
