@@ -3,6 +3,7 @@
 #include "model_diagram_panel.h"
 #include "image_cache.h"
 #include "grtdb/db_object_helpers.h"
+#include "model/wb_context_model.h"
 #include "workbench/wb_context_ui.h"
 #include "mforms/../gtk/lf_toolbar.h"
 #include "mforms/../gtk/lf_menubar.h"
@@ -24,7 +25,7 @@
 #include <gdk/gdkkeysyms.h>
 
 #include "base/log.h"
-ENABLE_LOG("fe")
+DEFAULT_LOG_DOMAIN("fe")
 
 class CanvasViewer : public mdc::GtkCanvas
 {
@@ -348,11 +349,11 @@ void ModelDiagramPanel::init(const std::string &view_id)
     }
 
     mforms::ToolBar *tools_toolbar = _be->get_tools_toolbar();
-    log_debug3("ModelDiagramPanel::init got tools toolbar %p", tools_toolbar);
+    log_debug3("ModelDiagramPanel::init got tools toolbar %p\n", tools_toolbar);
     if (tools_toolbar)
     {
       _tools_toolbar = dynamic_cast<Gtk::VBox*>(mforms::gtk::widget_for_toolbar(tools_toolbar));
-      log_debug3("ModelDiagramPanel::init cast _tools_toolbar = %p", _tools_toolbar);
+      log_debug3("ModelDiagramPanel::init cast _tools_toolbar = %p\n", _tools_toolbar);
       if (_tools_toolbar)
       {
         _diagram_hbox->pack_start(*_tools_toolbar, false, true);
@@ -391,12 +392,8 @@ void ModelDiagramPanel::init(const std::string &view_id)
   
   show_all();
 
-  // must show before attaching signal because this initial show shouldn't be accounted
-  // for changnig the closed state of the diagram
-  signal_show().connect(sigc::bind(sigc::mem_fun(_be, &wb::ModelDiagramForm::set_closed), false));
-  signal_hide().connect(sigc::bind(sigc::mem_fun(_be, &wb::ModelDiagramForm::set_closed), true));
+  get_panel()->signal_show().connect_notify((sigc::bind(sigc::mem_fun(_be, &wb::ModelDiagramForm::set_closed), false)));
 
-  
   _catalog_tree->refresh();
   _layer_tree->refresh();
 }
@@ -554,22 +551,20 @@ void ModelDiagramPanel::update_tool_cursor()
 }
 
 
-void ModelDiagramPanel::set_title(const std::string &title)
-{
-  _title_changed.emit(title);
-}
-
-
 bool ModelDiagramPanel::on_close()
 {
+  _be->set_closed(true);
   get_parent()->hide(); // hide the container
 
   return false; // don't close
 }
 
-void ModelDiagramPanel::refresh_catalog()
+void ModelDiagramPanel::refresh_catalog(bool hard)
 {
-  _catalog_tree->refresh();
+  if (hard)
+    _catalog_tree->refresh();
+  else
+    _catalog_tree->queue_draw();
 }
 
 
