@@ -1,7 +1,6 @@
 
 #include "model_panel.h"
 #include "overview_panel.h"
-#include "linux_utilities/MGGladeXML.h"
 #include "linux_utilities/gtk_helpers.h"
 #include <gtkmm/alignment.h>
 #include <gtkmm/notebook.h>
@@ -14,23 +13,20 @@
 
 #define _(s) s
 
-bool ModelPanel::will_close()
+bool ModelPanel::on_close()
 { 
-  _overview->will_close(); 
+  _overview->on_close(); 
   return false;
 }
 
 
 ModelPanel *ModelPanel::create(wb::WBContextUI *wb, wb::OverviewBE *overview)
 {
-  MGGladeXML *xml= new MGGladeXML(wb->get_wb()->get_grt_manager()->get_data_file_path("model_view.glade").c_str(), "", "");
-  GtkHPaned *paned;
+  Glib::RefPtr<Gtk::Builder> xml= Gtk::Builder::create_from_file(wb->get_wb()->get_grt_manager()->get_data_file_path("model_view.glade"));
 
-  paned = GTK_HPANED(xml->get_unwrapped("model_pane"));
-  
-  ModelPanel *panel = new ModelPanel(xml, paned, wb, overview);
-
-  delete xml;
+  ModelPanel *panel = 0;
+  xml->get_widget_derived<ModelPanel>("top_vbox", panel);
+  panel->post_construct(wb, overview, xml);
   
   return panel;
 }
@@ -42,38 +38,42 @@ static void fix_size(Gtk::Paned *paned)
   paned->set_position(paned->get_parent()->get_width() - paned->get_position());
 }
 
-ModelPanel::ModelPanel(MGGladeXML *xml, GtkHPaned *paned, wb::WBContextUI *wb, wb::OverviewBE *overview)
-  : Gtk::HPaned(paned), _wb(wb)
+ModelPanel::ModelPanel(GtkVBox *paned, Glib::RefPtr<Gtk::Builder> xml)
+  : Gtk::VBox(paned), _wb(0)
 {
-  _top_box = new Gtk::VBox(false, 0);
+}
+
+void ModelPanel::post_construct(wb::WBContextUI *wb, wb::OverviewBE *overview, Glib::RefPtr<Gtk::Builder> xml)
+{
+  _wb= wb;
   {
     mforms::MenuBar *menubar = overview->get_menubar();
     if (menubar)
-      _top_box->pack_start(*mforms::gtk::widget_for_menubar(menubar), false, true);
+      pack_start(*mforms::gtk::widget_for_menubar(menubar), false, true);
     mforms::ToolBar *toolbar = overview->get_toolbar();
     if (toolbar)
-      _top_box->pack_start(*mforms::gtk::widget_for_toolbar(toolbar), false, true);
+      pack_start(*mforms::gtk::widget_for_toolbar(toolbar), false, true);
   }
-  _top_box->show_all();
-  _top_box->pack_start(*this, true, true);
-  show();
+  show_all();
 
   if (wb->get_wb()->get_wb_options().get_int("Sidebar:RightAligned", 0))
   {
-    swap_panned_children(this, true);
-    signal_realize().connect(sigc::bind(sigc::ptr_fun(fix_size), this));
+    Gtk::Paned *p;
+    xml->get_widget("model_pane", p);
+    swap_panned_children(p, true);
+    signal_realize().connect(sigc::bind(sigc::ptr_fun(fix_size), p));
   }
 
-  _overview = new OverviewPanel(wb, overview);
+  _overview = Gtk::manage(new OverviewPanel(wb, overview));
   
-  xml->get("editor_tab", &_editor_note);
-  xml->get("content", &_editor_paned);
+  xml->get_widget("editor_tab", _editor_note);
+  xml->get_widget("content", _editor_paned);
   
-  xml->get("model_sidebar", &_sidebar);
+  xml->get_widget("model_sidebar", _sidebar);
   
   Gtk::Alignment *placeholder;
   
-  xml->get("overview_placeholder", &placeholder);
+  xml->get_widget("overview_placeholder", placeholder);
   
   placeholder->add(*_overview);
   _overview->show();
@@ -81,13 +81,13 @@ ModelPanel::ModelPanel(MGGladeXML *xml, GtkHPaned *paned, wb::WBContextUI *wb, w
   Gtk::Notebook *note;
   Gtk::Label *label;
 
-  xml->get("side_model_note0", &note);
+  xml->get_widget("side_model_note0", note);
   _documentation_box= Gtk::manage(new DocumentationBox(wb));
   label = Gtk::manage(new Gtk::Label(_("<small>Description</small>")));
   note->append_page(*_documentation_box, *label);
   label->set_use_markup(true);
 
-  xml->get("side_model_note1", &note);
+  xml->get_widget("side_model_note1", note);
   _usertypes_box= Gtk::manage(new UserTypesBox(wb));
   label = Gtk::manage(new Gtk::Label(_("<small>User Types</small>")));
   note->append_page(*_usertypes_box, *label);
@@ -107,25 +107,12 @@ void ModelPanel::selection_changed()
 }
 
 
-std::string ModelPanel::get_title()
-{
-  return "Model";
-}
-
-
 bec::UIForm *ModelPanel::get_form() const
 {
   return (bec::UIForm*)_overview->get_be();
 }
 
 
-ModelPanel::~ModelPanel()
-{
-  delete _overview;
-  delete _top_box;
-}
-
-
 void ModelPanel::toggle_sidebar()
 {
   if (_sidebar->is_visible())
