@@ -6,26 +6,24 @@
 #include <gtkmm/paned.h>
 #include <gtkmm/notebook.h>
 #include <gtkmm/box.h>
+#include <gtkmm/builder.h>
 #include "linux_utilities/form_view_base.h"
 
 #include "workbench/wb_overview.h"
 #include "workbench/wb_context_ui.h"
 
 class OverviewPanel;
-class MGGladeXML;
 class HistoryTree;
 class UserTypesBox;
 class DocumentationBox;
 
-class ModelPanel : public Gtk::HPaned, public FormViewBase
+class ModelPanel : public Gtk::VBox, public FormViewBase
 {
 public:
   static ModelPanel *create(wb::WBContextUI *wb, wb::OverviewBE *overview);
-  virtual ~ModelPanel();
   
-  virtual bool will_close();
-  virtual std::string get_title();
-  virtual Gtk::Widget *get_panel() { return _top_box; }
+  virtual bool on_close();
+  virtual Gtk::Widget *get_panel() { return this; }
   virtual bec::UIForm *get_form() const;
   virtual void toggle_sidebar();
 
@@ -39,7 +37,6 @@ public:
 private:
   wb::WBContextUI *_wb;
   OverviewPanel *_overview;
-  Gtk::VBox *_top_box;
   Gtk::Paned *_editor_paned;
   Gtk::Widget *_sidebar;
   bec::NodeId                  _last_found_node;
@@ -48,7 +45,9 @@ private:
   UserTypesBox                *_usertypes_box;
   DocumentationBox            *_documentation_box;
 
-  ModelPanel(MGGladeXML *xml, GtkHPaned *paned, wb::WBContextUI *wb, wb::OverviewBE *overview);
+  friend class Gtk::Builder;
+  ModelPanel(GtkVBox *vb, Glib::RefPtr<Gtk::Builder> xml);
+  void post_construct(wb::WBContextUI *wb, wb::OverviewBE *overview, Glib::RefPtr<Gtk::Builder> xml);
 };
 
 
