@@ -1,5 +1,7 @@
 #include "grtpp_module_python.h"
 
+#include "gtk/lf_mforms.h"
+
 // linux ui includes
 #include "gtk_helpers.h"
 #include "program.h"
@@ -102,7 +104,6 @@ Program::Program(wb::WBOptions wboptions)
   // Assign those callback methods
   wbcallbacks.show_file_dialog= sigc::mem_fun(this, &Program::show_file_dialog_becb);
   wbcallbacks.show_status_text= sigc::mem_fun(_main_form, &MainForm::show_status_text_becb);
-  wbcallbacks.show_progress= sigc::mem_fun(_main_form, &MainForm::show_progress_becb);
   wbcallbacks.request_input= sigc::mem_fun(this, &Program::request_input_becb);
   wbcallbacks.open_editor= sigc::mem_fun(_main_form, &MainForm::open_plugin_becb);
   wbcallbacks.show_editor= sigc::mem_fun(_main_form, &MainForm::show_plugin_becb);
