@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,6 @@
  * 02110-1301  USA
  */
 
-#import "sigobjc++.h"
 #import "MCanvasViewer.h"
 #include "mdc.h"
 #include "mdc_canvas_view_macosx.h"
@@ -140,7 +139,7 @@ static void canvas_view_needs_repaint(int x, int y, int w, int h, MCanvasViewer
     
     _view= new mdc::QuartzCanvasView((CGContextRef)[[NSGraphicsContext currentContext] graphicsPort], 
                                      NSWidth(frame), NSHeight(frame));
-    _view->signal_repaint().connect(sigc::bind(sigc::ptr_fun(canvas_view_needs_repaint), self));
+    _view->signal_repaint()->connect(boost::bind(canvas_view_needs_repaint, _1, _2, _3, _4, self));
   }
 }
 
@@ -153,13 +152,13 @@ static void canvas_view_needs_repaint(int x, int y, int w, int h, MCanvasViewer
 
 - (void)scrollToPoint:(NSPoint)offset
 {
-  _view->set_offset(MySQL::Geometry::Point(offset.x, offset.y));
+  _view->set_offset(base::Point(offset.x, offset.y));
 }
 
 
 - (NSRect)documentRect
 {
-  MySQL::Geometry::Size size(_view->get_total_view_size());
+  base::Size size(_view->get_total_view_size());
 
   return NSMakeRect(0, 0, size.width, size.height);
 }
@@ -167,7 +166,7 @@ static void canvas_view_needs_repaint(int x, int y, int w, int h, MCanvasViewer
 
 - (NSRect)documentVisibleRect
 {
-  MySQL::Geometry::Rect rect(_view->get_viewport());
+  base::Rect rect(_view->get_viewport());
 
   return NSMakeRect(rect.pos.x, rect.pos.y, rect.size.width, rect.size.height);
 }
