@@ -62,7 +62,7 @@ static NSImage *descendingSortIndicator= nil;
     ::bec::GridModel::SortColumns sort_columns= (*mData)->sort_columns();
     for (::bec::GridModel::SortColumns::const_iterator i= sort_columns.begin(), end= sort_columns.end(); i != end; ++i)
     {
-      NSTableColumn *tableColumn= [mTableView tableColumnWithIdentifier:[NSString stringWithFormat:@"%i", i->first]];
+      NSTableColumn *tableColumn= [mTableView tableColumnWithIdentifier:[NSString stringWithFormat:@"%i", (int)i->first]];
       [mTableView setIndicatorImage:((i->second == 1) ? ascendingSortIndicator : descendingSortIndicator) inTableColumn:tableColumn];
     }
   }
@@ -101,9 +101,25 @@ static int processTaskMessage(int msgType, const std::string &message, const std
   return 0;
 }*/
 
+- (void)refreshGrid
+{
+  mPendingRefresh = NO;
+  [mTableView reloadData];
+}
+
+static int onRefreshWhenIdle(MResultsetViewer *self)
+{
+  if (!self->mPendingRefresh)
+  {
+    self->mPendingRefresh = YES;
+    [self performSelector: @selector(refreshGrid) withObject:nil afterDelay: 0];
+  }
+  return 0;
+}
+
 - (void)clickedTable:(id)sender
 {
-  (*self->mData)->set_edited_field([mTableView clickedRow], [mTableView clickedColumn]) ;
+  (*self->mData)->set_edited_field([mTableView clickedRow], [mTableView clickedColumn]);
   
   [mTableView editColumn:[mTableView clickedColumn]
                      row:[mTableView clickedRow]
@@ -137,13 +153,16 @@ static int processTaskMessage(int msgType, const std::string &message, const std
     
     [mTableView setRecordset: mData->get()];
     
+    (*mData)->tree_changed_signal()->connect(boost::bind(onRefreshWhenIdle, self));
+    
     (*mData)->refresh_ui_cb= boost::bind(onRefresh, self);
     mSigConns.push_back((*mData)->refresh_ui_status_bar_signal.connect(boost::bind(updateToolbar, self)));
     //(*mData)->task->msg_cb(boost::bind(processTaskMessage, _1, _2, _3, self));
     
+    [mTableView setIntercellSpacing: NSMakeSize(0, 1)];
     [mTableView selectionChangedActionTarget:self];
     [mTableView setSelectionChangedAction:@selector(handleNSTableViewSelectionIsChangingNotification:)];
-    
+    [mTableView setAllowsMultipleSelection: YES];
     [mView setExpandSubviewsByDefault: YES]; // makes the resultset part expand inside the VerticalLayouter
 
     [mView tile];
@@ -267,6 +286,9 @@ static int processTaskMessage(int msgType, const std::string &message, const std
           [view setEnabled: iter->enabled];
         }
         break;
+        
+      default:
+        break;
     }
   }
   [mToolbar tile];
@@ -488,27 +510,17 @@ static int onRefresh(MResultsetViewer *self)
 {
   if ([aTableColumn identifier] != nil)
   {
-    int columnIndex = [[aTableColumn identifier] intValue];
+    int columnIndex = [[aTableColumn identifier] intValue];    
     
-    int selectedColumnIndex = [(MGridView*)aTableView selectedColumnIndex];
-    int selectedRowIndex = [(MGridView*)aTableView selectedRowIndex];
-    
-    [aCell setIsNull: (*mData)->is_field_null(rowIndex, columnIndex)];
-    [aCell setIsBlob: (*mData)->get_column_type(columnIndex) == bec::GridModel::BlobType];
-    
-    if ( (columnIndex+1 == selectedColumnIndex) && (rowIndex == selectedRowIndex) ) 
+    if (columnIndex > 0)
     {
-      [aCell setDrawsBackground: YES];
-      if ([[aTableView window] firstResponder] != aTableView)
-        [aCell setBackgroundColor: [NSColor lightGrayColor]];
-      else
-        [aCell setBackgroundColor: [NSColor alternateSelectedControlColor]];// selectedMenuItemColor]];
-     // [aCell setTextColor: [NSColor whiteColor]];
+      [aCell setIsNull: (*mData)->is_field_null(rowIndex, columnIndex)];
+      [aCell setIsBlob: (*mData)->get_column_type(columnIndex) == bec::GridModel::BlobType];
     }
-    else 
+    else
     {
-      [aCell setDrawsBackground: NO];
-      [aCell setTextColor: [NSColor blackColor]];
+      [aCell setIsNull: NO];
+      [aCell setIsBlob: NO];
     }
   }
   else
@@ -518,13 +530,6 @@ static int onRefresh(MResultsetViewer *self)
 }
 
 
-- (BOOL) selectionShouldChangeInTableView: (NSTableView*) aTableView;
-{
-  // Supress row highlighting.
-  return NO;
-}
-
-
 - (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView
 {
   if (mData)
@@ -595,7 +600,7 @@ static int onRefresh(MResultsetViewer *self)
     int sort_order= 1; // ascending (1) on first click, descending (-1) on second, then toggling
     for (::bec::GridModel::SortColumns::const_iterator i= sort_columns.begin(), end= sort_columns.end(); i != end; ++i)
     {
-      if (i->first == column_index)
+      if ((int)i->first == column_index)
       {
         sort_order= (1 == i->second) ? -1 : 0;
         break;
