@@ -23,7 +23,6 @@
 // for _NSGetArg*
 #include <crt_externs.h>
 
-#import "WBPluginBase.h"
 #import "WBPluginPanel.h"
 #import "WBPluginEditorBase.h"
 #import "WBPluginWindowBase.h"
@@ -32,7 +31,7 @@
 #import "WBModelDiagramPanel.h"
 
 #include "workbench/wb_context_ui.h"
-
+#include "model/wb_context_model.h"
 #include "grtui/gui_plugin_base.h"
 
 #import "WBExceptions.h"
@@ -43,13 +42,14 @@
 #import "WBDiagramSizeController.h"
 #import "WBModelDiagramPanel.h"
 
+#import "ScintillaView.h"
+
 #include "mforms/toolbar.h"
-#include "mforms/../dialogs/search_replace.h"
+#import "MFCodeEditor.h"
 
 #include "workbench/wb_module.h"
 #include "base/log.h"
 
-
 static GThread *mainthread= 0;
 
 DEFAULT_LOG_DOMAIN("Workbench")
@@ -154,12 +154,6 @@ static void windowShowStatusText(const std::string &text, WBMainWindow *window)
 }
 
 
-static bool windowShowProgress(const std::string &,const std::string &,float, WBMainWindow *window)
-{
-  return true;
-}
-
-
 static NativeHandle windowOpenPlugin(bec::GRTManager *grtm, 
                                      grt::Module *ownerModule, const std::string &shlib, const std::string &class_name,
                                      const grt::BaseListRef &args, bec::GUIPluginFlags flags, WBMainWindow *window)
@@ -399,6 +393,9 @@ static void quitApplication()
 
 - (void)applicationWillTerminate:(NSNotification *)aNotification
 {
+  [[mainWindow window] orderOut: nil];
+  [mainWindow closeAllPanelsNoConfirm];
+
   _wbui->get_wb()->finalize();
   // is crashing delete _wbui;
 }
@@ -451,12 +448,36 @@ static bool validate_copy(wb::WBContextUI *wbui)
     NSRange textRange = [(id)[[NSApp keyWindow] firstResponder] selectedRange];
     return textRange.length > 0;
   }
+  if (/*[[[NSApp keyWindow] firstResponder] isKindOfClass: [NSTableView class]]
+      &&*/ [[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(copy:)])
+    return true; //[[(NSTableView*)[[NSApp keyWindow] firstResponder] selectedRowIndexes] count] > 0;
   
   return (wbui->get_active_form() && wbui->get_active_form()->can_copy());
 }
 
 //--------------------------------------------------------------------------------------------------
 
+static bool validate_cut(wb::WBContextUI *wbui)
+{
+  if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(selectedRange)])
+  {
+    NSRange textRange = [(id)[[NSApp keyWindow] firstResponder] selectedRange];
+    if (textRange.length > 0)
+    {
+      if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(isEditable)])
+        return [(id)[[NSApp keyWindow] firstResponder] isEditable];
+      return true;
+    }
+    return false;
+  }
+  else if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(cut:)])
+    return true;
+  
+  return (wbui->get_active_form() && wbui->get_active_form()->can_cut());
+}
+
+//--------------------------------------------------------------------------------------------------
+
 static bool validate_paste(wb::WBContextUI *wbui)
 {
   if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(isEditable)])
@@ -468,9 +489,11 @@ static bool validate_paste(wb::WBContextUI *wbui)
     // 2) The pasteboard contains text.
     NSArray* supportedTypes = [NSArray arrayWithObjects: NSStringPboardType, nil];
     NSString *bestType = [[NSPasteboard generalPasteboard] availableTypeFromArray: supportedTypes];
- 
+
     return isEditable && (bestType != nil);
   }
+  else if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(paste:)])
+    return true;
   
   return (wbui->get_active_form() && wbui->get_active_form()->can_paste());
 }
@@ -504,24 +527,6 @@ static bool validate_delete(wb::WBContextUI *wbui)
 
 //--------------------------------------------------------------------------------------------------
 
-static bool validate_cut(wb::WBContextUI *wbui)
-{
-  if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(isEditable)])
-  {
-    BOOL isEditable = [(id)[[NSApp keyWindow] firstResponder] isEditable];
-    
-    if (isEditable && [[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(selectedRange)])
-    {
-      NSRange textRange = [(id)[[NSApp keyWindow] firstResponder] selectedRange];
-      return textRange.length > 0;
-    }
-  }
-  
-  return (wbui->get_active_form() && wbui->get_active_form()->can_cut());
-}
-
-//--------------------------------------------------------------------------------------------------
-
 // XXX deprecated, remove it eventually
 static void call_closetab_old(WBMainWindow *mainWindow)
 {
@@ -604,85 +609,11 @@ static void call_search(WBMainWindow *mainWindow)
 
 //--------------------------------------------------------------------------------------------------
 
-static bool search_callback(const std::string &find, const std::string &replace_with,
-                            mforms::SearchFlags flags, ScintillaView *editor)
-{
-  bool doReplace = (flags & mforms::SearchDoReplace) != 0;
-  bool doAll = (flags & mforms::SearchAll) != 0;
-  bool matchCase = (flags & mforms::SearchMatchCase) != 0;
-  bool matchWholeWord = (flags & mforms::SearchMatchWholeWord) != 0;
-  bool backwards = (flags & mforms::SearchPrevious) != 0;
-  
-  // Searching all occurences without replacing them doesn't make sense so we force replace here
-  // if the SearchAll flag is set.
-  if (doAll)
-    doReplace = true;
-  
-  NSString* text = [NSString stringWithCPPString: find];
-  
-  if (doReplace)
-  {
-    [editor findAndReplaceText: text
-                        byText: [NSString stringWithCPPString: replace_with]
-                     matchCase: matchCase
-                     wholeWord: matchWholeWord
-                         doAll: doAll];
-  }
-  else
-  {
-    // Simply find and select the first hit starting from the current location in the editor.
-    [editor findAndHighlightText: text
-                       matchCase: matchCase
-                       wholeWord: matchWholeWord
-                        scrollTo: YES
-                            wrap: YES
-                       backwards: backwards];
-  }
-  
-  return false;
-}
-
-//--------------------------------------------------------------------------------------------------
-
-static void call_find_replace(wb::WBContextUI *wbui, bool no_replace)
+static bool validate_find_replace()
 {
   id firstResponder = [[NSApp keyWindow] firstResponder];
-  id superView = [firstResponder respondsToSelector: @selector(superview)] ? [firstResponder superview] : nil;
   
-  if ([firstResponder isKindOfClass: [MScintillaView class]] && [[firstResponder delegate] respondsToSelector: @selector(showFindPanel:)])
-  {
-    [[firstResponder delegate] performSelector: @selector(showFindPanel:) withObject: firstResponder];
-    [firstResponder enableReplaceInFindPanel: !no_replace];
-  }
-  else if ([superView isKindOfClass: [MScintillaView class]] && [[superView delegate] respondsToSelector: @selector(showFindPanel:)])
-  {
-    [[superView delegate] performSelector: @selector(showFindPanel:) withObject: superView];
-    [superView enableReplaceInFindPanel: !no_replace];
-  }
-  else if ([firstResponder isKindOfClass: [ScintillaView class]]
-      || [[firstResponder superview] isKindOfClass: [ScintillaView class]]
-      || [firstResponder isKindOfClass: [NSTextView class]])
-  {
-    if (![firstResponder isKindOfClass: [ScintillaView class]])
-      firstResponder = [firstResponder superview];
-    
-    mforms::SearchReplace dialog;
-    
-    dialog.set_callback(boost::bind(search_callback, _1, _2, _3, firstResponder));
-    dialog.show(false,
-                (mforms::SearchFlags)(mforms::SearchNone),
-                !no_replace);
-  }
-  else
-    NSBeep();
-}
-
-//--------------------------------------------------------------------------------------------------
-
-static bool validate_find_replace(wb::WBContextUI *wbui)
-{
-  id firstResponder = [[NSApp keyWindow] firstResponder];
-  if ([firstResponder isKindOfClass: [ScintillaView class]] ||
+  if ([firstResponder isKindOfClass: [ScintillaView class]] || [firstResponder isKindOfClass: [NSTextView class]] ||
       ([firstResponder isKindOfClass: [NSView class]] && [[firstResponder superview] isKindOfClass: [ScintillaView class]]))
     return true;
   return false;
@@ -690,16 +621,29 @@ static bool validate_find_replace(wb::WBContextUI *wbui)
 
 //--------------------------------------------------------------------------------------------------
 
+static void call_find_replace(bool do_replace)
+{
+  if (validate_find_replace())
+  {
+    id firstResponder = [[NSApp keyWindow] firstResponder];
+    if ([firstResponder isKindOfClass: [InnerView class]])
+      firstResponder = [firstResponder superview];
+    
+    if ([firstResponder isKindOfClass: [MFCodeEditor class]])
+      [firstResponder showFindPanel: do_replace];
+  }
+  else
+    NSBeep();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 static void call_find(WBMainWindow *mainWindow)
 {
-  // if we're in a scintilla editor that is not the main editor of the form,
-  // open the find dialog
-  wb::WBContextUI *wbui = [mainWindow context];
-  
-  if (validate_find_replace(wbui))
-    call_find_replace(wbui, true);
+  if (validate_find_replace())
+    call_find_replace(false);
   else
-    [mainWindow focusSearchField:nil];
+    [mainWindow focusSearchField: nil];
 }
 
 static bool validate_find(WBMainWindow *mainWindow)
@@ -707,7 +651,7 @@ static bool validate_find(WBMainWindow *mainWindow)
   bec::UIForm *form = [mainWindow context]->get_active_main_form();
   if (form && form->get_toolbar() && form->get_toolbar()->find_item("searchbox"))
     return true;
-  return validate_find_replace([mainWindow context]);
+  return validate_find_replace();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -814,8 +758,8 @@ static bool validate_redo(WBMainWindow *mainWindow)
   _wbui->get_command_ui()->add_builtin_command("searchbox", boost::bind(call_search, mainWindow));
                                                //boost::bind(validate_closetab), mainWindow));
  
-  _wbui->get_command_ui()->add_builtin_command("search_replace", boost::bind(call_find_replace, _wbui, false),
-                                               boost::bind(validate_find_replace, _wbui));
+  _wbui->get_command_ui()->add_builtin_command("search_replace", boost::bind(call_find_replace, true),
+                                               boost::bind(validate_find_replace));
 
   _wbui->get_command_ui()->add_builtin_command("undo", boost::bind(call_undo, mainWindow),
                                                boost::bind(validate_undo, mainWindow));
@@ -836,12 +780,12 @@ static bool validate_redo(WBMainWindow *mainWindow)
 }
 
 
-static void set_clipboard_text(const std::string &text)
-{
-  [[NSPasteboard generalPasteboard] declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:nil];
-  [[NSPasteboard generalPasteboard] setString:[NSString stringWithCPPString:text]
-                                      forType:NSStringPboardType];
-}
+//static void set_clipboard_text(const std::string &text)
+//{
+//  [[NSPasteboard generalPasteboard] declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:nil];
+//  [[NSPasteboard generalPasteboard] setString:[NSString stringWithCPPString:text]
+//                                      forType:NSStringPboardType];
+//}
 
 
 static void flush_main_thread()
@@ -894,6 +838,7 @@ static void flush_main_thread()
                                              object: nil];
 }
 
+
 #include "mforms/../cocoa/MFMenuBar.h"
 
 - (void)windowDidBecomeKey:(NSNotification*)notification
@@ -946,7 +891,6 @@ static NSString *applicationSupportFolder()
     // Assign those callback methods
     wbcallbacks.show_file_dialog= boost::bind(showFileDialog, _1, _2, _3);
     wbcallbacks.show_status_text= boost::bind(windowShowStatusText, _1, mainWindow);
-    wbcallbacks.show_progress= boost::bind(windowShowProgress, _1, _2, _3, mainWindow);
     wbcallbacks.request_input= boost::bind(requestInput, _1, _2, _3, self);
     wbcallbacks.open_editor= boost::bind(windowOpenPlugin, _1, _2, _3, _4, _5, _6, mainWindow);
     wbcallbacks.show_editor= boost::bind(windowShowPlugin, _1, mainWindow);
@@ -997,11 +941,11 @@ static NSString *applicationSupportFolder()
 
 /**
  * Instantiates the option class for the backend and parses the command line.
- * @result Returns the passed log level to the caller or an empty string if none was given.
  */
-- (std::string) setupOptionsAndParseCommandline
+- (void)setupOptionsAndParseCommandline
 {
   _options= new wb::WBOptions();
+  
   _options->basedir = [[[NSBundle mainBundle] resourcePath] fileSystemRepresentation];
   _options->struct_search_path = _options->basedir + "/grt";
   _options->plugin_search_path = std::string([[[NSBundle mainBundle] builtInPlugInsPath] fileSystemRepresentation]);
@@ -1010,100 +954,21 @@ static NSString *applicationSupportFolder()
   _options->cdbc_driver_search_path = std::string([[[NSBundle mainBundle] privateFrameworksPath] fileSystemRepresentation]);
   _options->user_data_dir= [[applicationSupportFolder() stringByAppendingString: @"/MySQL/Workbench"] fileSystemRepresentation];
 
-  // handle some command line options
-  // --model=<modelfile>
-  // --query=<connection>
-  // --admin=<admin>
-  // --script=<scriptfile>
-  // --run=<script>
-  // --run-python=<script>
-  // --run-lua=<script>
-  // --quit-when-done
-  // --log-level
-  std::string logLevel = "";
   int argc= *_NSGetArgc();
   char **argv= *_NSGetArgv();
-  for (int i= 1; i < argc; i++)
+  
+  int rc = 0;
+  if (!_options->parse_args(argv, argc, &rc))
   {
-    if (strncmp(argv[i], "--model=", strlen("--model=")) == 0
-        || strncmp(argv[i], "--query=", strlen("--query=")) == 0
-        || strncmp(argv[i], "--admin=", strlen("--admin=")) == 0
-        || strncmp(argv[i], "--script=", strlen("--script=")) == 0)
-    {
-      char *eq = strchr(argv[i], '=');
-      _options->open_at_startup_type= std::string(argv[i]+2, eq);
-      
-      if (strlen(eq+1) > 0)
-        _options->open_at_startup= eq+1;
-      else
-      {
-        NSLog(@"Missing argument for %s option", argv[i]);
-        exit(1);
-      }
-    }
-    else if (strncmp(argv[i], "--run=", strlen("--run=")) == 0)
-    {
-      char *eq = strchr(argv[i], '=');
-      if (strlen(eq+1) > 0)
-        _options->run_at_startup= eq+1;
-      else
-      {
-        NSLog(@"Missing argument for --run option");
-        exit(1);
-      }
-    }
-    else if (strncmp(argv[i], "--run-", strlen("--run-")) == 0)
-    {
-      char *eq = strchr(argv[i], '=');
-      if (!eq || strlen(eq+1) == 0)
-      {
-        NSLog(@"Missing argument for %s option", argv[i-1]);
-        exit(1);
-      }
-      
-      _options->run_language= std::string(argv[i]+strlen("--run-"), eq);
-      _options->run_at_startup= eq+1;
-    }        
-    else if (strcmp(argv[i], "--quit-when-done") == 0)
-      _options->quit_when_done= true;
-    else if (strncmp(argv[i], "--log-level=", strlen("--log-level=")) == 0)
-    {
-      char *eq = strchr(argv[i], '=');
-      if (!eq || strlen(eq+1) == 0)
-      {
-        NSLog(@"Missing argument for %s option", argv[i - 1]);
-        exit(1);
-      }
-      
-      logLevel = std::string(eq + 1);
-    }        
-    else if (strcmp(argv[i], "--help") == 0)
-    {
-      NSLog(@"%s [<options>] [<model file>]", argv[0]);
-      NSLog(@"--model=<model file>             Open a model file at startup");
-      NSLog(@"--query=<connection name>        Starts a query tab to the named connection");
-      NSLog(@"--admin=<instance name>          Starts the administrator for the named instance");
-      NSLog(@"--script=<script file>           Executes the script file");
-      NSLog(@"--run=<script code>              Executes the script code, in the default scripting language");
-      NSLog(@"--run-python=<python code>       Executes the given Python code");
-      NSLog(@"--run-lua=<lua code>             Executes the given Lua code");
-      NSLog(@"--quit-when-done                 Quits Workbench when the script finished executing");
-      NSLog(@"--log-level=<level>              Determines the amount of logging (error, warning, info, debug1/2/3)");
-      NSLog(@"--version                        Print version information and exit");
-      NSLog(@"--help                           Print this message and exit");
-      exit(0);
-    }
-    else if (strcmp(argv[i], "--version") == 0)
-    {
-#ifdef EDITION_SE
-      NSLog(@"MySQL Workbench SE %i.%i.%i rev %i %s", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, APP_REVISION_NUMBER, APP_RELEASE_TYPE);
-#else
-      NSLog(@"MySQL Workbench CE %i.%i.%i rev %i %s", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, APP_REVISION_NUMBER, APP_RELEASE_TYPE);
-#endif
-      exit(0);
-    }
+    log_info("Exiting with rc %i after parsing arguments\n", rc);
+    exit(rc);
   }
-  return logLevel;
+
+  // no dock icon when the app will quit when finished running script 
+  /* supported in 10.6+ only
+  if (_options->quit_when_done && [NSApp respondsToSelector: @selector(setActivationPolicy:)])
+    [NSApp setActivationPolicy: NSApplicationActivationPolicyAccessory];
+   */
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1183,21 +1048,10 @@ static void init_mforms()
 - (void)awakeFromNib
 { 
   // Prepare the logger to be ready as first part.
-  base::Logger logger = base::Logger([[applicationSupportFolder() stringByAppendingString: @"/MySQL/Workbench"] fileSystemRepresentation]);
-#if DEBUG
-  std::string logLevel = "debug2";
-#else
-  std::string logLevel = "info";
-#endif
+  base::Logger([[applicationSupportFolder() stringByAppendingString: @"/MySQL/Workbench"] fileSystemRepresentation]);
   
-  char* environmentLevel = getenv("WB_LOG_LEVEL");
-  if (environmentLevel != NULL)
-    logLevel = environmentLevel;
+  [self setupOptionsAndParseCommandline];
   
-  std::string passedLevel = [self setupOptionsAndParseCommandline];
-  if (passedLevel.size() > 0)
-    logLevel = passedLevel;
-  base::Logger::active_level(logLevel);
   log_info("Starting up Workbench\n");
   
   if (!mainWindow)
@@ -1226,7 +1080,11 @@ static void init_mforms()
     
     setupSQLQueryUI(self, mainWindow, _wbui);
     
-    [mainWindow showWindow:nil];
+    // don't show the main window if we'll quit after running a script
+    if ((_options->quit_when_done && !_options->run_at_startup.empty()))
+      [[mainWindow window] orderOut: nil];
+    else
+      [mainWindow showWindow:nil];
 
     // do the final setup for after the window is shown
     [mainWindow setupReady];    
@@ -1239,7 +1097,7 @@ static void init_mforms()
       NSDirectoryEnumerator *dirEnum = [[NSFileManager defaultManager] enumeratorAtPath:pluginsPath];
       NSString *file;
       
-      while (file = [dirEnum nextObject]) {
+      while ((file = [dirEnum nextObject])) {
         if ([[file pathExtension] isEqualToString: @"mwbplugin"]) {
           NSString *path= [pluginsPath stringByAppendingPathComponent:file];
           
@@ -1315,7 +1173,7 @@ static void init_mforms()
       break;
       
     case APP_MENU_QUIT:
-      _wbui->get_command_ui()->activate_command("plugin:wb.file.exit");
+      quitApplication(); 
       break;
   }
 }
@@ -1351,6 +1209,7 @@ static void init_mforms()
   WBDiagramSizeController *controller= [[WBDiagramSizeController alloc] initWithWBContext:_wbui];
   
   [controller showModal];
+  [controller release];
 }
 
 
