@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,11 +29,11 @@
 #import "WBSplitView.h"
 #import "WBSplitViewUnbrokenizerDelegate.h"
 #import "WBSQLQueryPanel.h"
-#import "WBPluginBase.h"
 #import "WBPluginEditorBase.h"
 #import "MCPPUtilities.h"
 #import "MTabSwitcher.h"
 #include "workbench/wb_context_ui.h"
+#include "model/wb_context_model.h"
 #include "model/wb_model_diagram_form.h"
 #include "model/wb_overview_physical.h"
 
@@ -60,6 +60,8 @@ DEFAULT_LOG_DOMAIN("Workbench")
 }
 @end
 
+
+
 class MacNotificationObserver : base::Observer
 {
   WBMainWindow *mainWindow;
@@ -87,6 +89,8 @@ public:
   }
 };
 
+
+
 @implementation WBWindow
 
 - (id)initWithContentRect:(NSRect)contentRect styleMask:(NSUInteger)aStyle backing:(NSBackingStoreType)bufferingType defer:(BOOL)flag
@@ -97,7 +101,7 @@ public:
     // setup pointer to main window in mforms
     NSInteger rc = [self retainCount];
     mforms::Form::main_form()->set_data(self);
-    if ([self retainCount] > rc) // don't let the mforms placeholder retain us
+    if ((int)[self retainCount] > rc) // don't let the mforms placeholder retain us
       [self release];
     
     //[self setBackgroundColor: [NSColor whiteColor]];
@@ -141,6 +145,8 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 - (void)setWBContext:(wb::WBContextUI*)wbui
 {
+  _backendObserver = new MacNotificationObserver(self);
+  
   _wbui= wbui;
   
   setup_mforms_app(self);
@@ -358,21 +364,6 @@ void setup_mforms_app(WBMainWindow *mwin);
       break;
     }
       
-    case wb::RefreshViewName:
-    {
-      id diagramPanel= (id)arg2;
-      id identifier= [diagramPanel identifier];
-      for (NSTabViewItem *item in [topTabView tabViewItems])
-      {
-        if ([item identifier] == identifier)
-        {
-          [item setLabel: [diagramPanel title]];
-          break;
-        }
-      }
-      break;
-    } 
-      
     case wb::RefreshSchemaList:
     case wb::RefreshSchema:
     {
@@ -893,7 +884,7 @@ void setup_mforms_app(WBMainWindow *mwin);
     {
       id form= [_panels objectForKey: [item identifier]];
       if (!form)
-        NSLog(@"Item in toplevel tabview is not a known form", [item label]);
+        NSLog(@"Item in toplevel tabview is not a known form %@", [item label]);
 
       return form;
     }
@@ -979,7 +970,6 @@ void setup_mforms_app(WBMainWindow *mwin);
         if (subpanel)
           return subpanel;
         
-        previous = responder;
         responder = [responder nextResponder];
       }
     }
@@ -1072,13 +1062,6 @@ void setup_mforms_app(WBMainWindow *mwin);
     _defaultMainMenu = [[NSApp mainMenu] retain];
     
     _homeIcon = [[NSImage imageNamed: @"WB_Home_mac.png"] retain];
-    
-    [[NSNotificationCenter defaultCenter] addObserver: self
-                                             selector: @selector(applicationWillTerminate:)
-                                                 name: NSApplicationWillTerminateNotification
-                                               object: nil];
-	
-	_backendObserver = new MacNotificationObserver(self);
   }
   return self;
 }
@@ -1103,12 +1086,20 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 //--------------------------------------------------------------------------------------------------
 
-- (void)applicationWillTerminate:(NSNotification*)notification
+- (void)closeAllPanelsNoConfirm
 {
   id panelsCopy = [_panels copy];
   for (id panel in [panelsCopy objectEnumerator])
-    [self closePanel: panel];
+  {
+    id identifier = [[[panel identifier] retain] autorelease];
+
+    if ([topTabView indexOfTabViewItemWithIdentifier: identifier] != NSNotFound)
+      [self closeTopPanelWithIdentifier: identifier];
+    else
+      [self closeBottomPanelWithIdentifier: identifier];    
+  }
   [panelsCopy release];
+  [_panels removeAllObjects];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1116,7 +1107,7 @@ void setup_mforms_app(WBMainWindow *mwin);
 - (void)dealloc
 {
   delete _backendObserver;
-	
+  
   [_closedTopTabs release];
   [_closedBottomTabs release];
   
@@ -1150,77 +1141,76 @@ void setup_mforms_app(WBMainWindow *mwin);
 // mforms integration
 #import <mforms/../cocoa/MFMForms.h>
 
-static void dock_view(mforms::App *app, mforms::AppView *view, const std::string &position)
+class MainWindowDockingPoint : public mforms::DockingPointDelegate
 {
-  WBMainWindow* self= (WBMainWindow*)app->get_data();
-  WBMFormsPluginPanel *panel = [WBMFormsPluginPanel panelOfAppView: view];
-  view->set_managed();
-
-  if (position == "maintab")
-  {
-    if (panel)
-      [self->topTabView selectTabViewItemWithIdentifier: [panel identifier]]; 
-    else
-    {
-      panel = [[WBMFormsPluginPanel alloc] initWithAppView: view];
-      if (!view->get_menubar())
-        [panel setDefaultMenuBar: [self context]->get_command_ui()->create_menubar_for_context("")];
-        
-      [self addTopPanelAndSwitch: [panel autorelease]];
-    }
-  }
-  else if (position == "editortab")
-  {
-    /*XXX
-    if (panel)
-      [self->bottomTabView selectTabViewItemWithIdentifier: [panel identifier]]; 
-    else
-    {      
-      panel = [[WBMFormsPluginPanel alloc] initWithAppView: view];
-      [self addBottomPanel: [panel autorelease]];
-    }*/
-  }  
-}
-
-//--------------------------------------------------------------------------------------------------
-
-static bool select_view(mforms::App *app, const std::string &identifier)
-{
-  WBMainWindow* self= (WBMainWindow*)app->get_data();
-  NSString *ident = [NSString stringWithUTF8String: identifier.c_str()];
-
-  if ([self->topTabView indexOfTabViewItemWithIdentifier: ident] < 0)
-    return false;
-  [self->topTabView selectTabViewItemWithIdentifier: ident];
-  return false;
-}
-
-//--------------------------------------------------------------------------------------------------
-
-static void undock_view(mforms::App *app, mforms::AppView *view)
-{
-  WBMainWindow* self= (WBMainWindow*)app->get_data();
-  WBMFormsPluginPanel *panel = [WBMFormsPluginPanel panelOfAppView: view];
+  WBMainWindow *self;
   
-  if (panel) 
-    [self closePanel: panel];
-}
-
-//--------------------------------------------------------------------------------------------------
-
-static void set_view_title(mforms::App *app, mforms::AppView *view, const std::string &title)
-{
-  WBMFormsPluginPanel *panel = [WBMFormsPluginPanel panelOfAppView: view];
-  if (panel)
+public:
+  MainWindowDockingPoint(WBMainWindow *mainWindow) : self(mainWindow) {}
+  
+  virtual std::string get_type()
   {
-    WBMainWindow *mwin = app->get_data();
-    
-    [panel setTitle: [NSString stringWithCPPString: title]];
-    [mwin setTitle: [panel title] forPanel: panel];
+    return "MainWindow";
   }
-  else
-    NSLog(@"set_view_title called for undocked view");
-}
+  
+  virtual void dock_view(mforms::AppView *view, const std::string &arg1, int arg2)
+  {
+    WBMFormsPluginPanel *panel = [WBMFormsPluginPanel panelOfAppView: view];
+    view->set_managed();
+    
+    if (arg1 == "maintab" || arg1.empty())
+    {
+      if (panel)
+        [self->topTabView selectTabViewItemWithIdentifier: [panel identifier]]; 
+      else
+      {
+        panel = [[WBMFormsPluginPanel alloc] initWithAppView: view];
+        if (!view->get_menubar())
+          [panel setDefaultMenuBar: [self context]->get_command_ui()->create_menubar_for_context(view->is_main_form() ?
+                                                                                                 view->get_form_context_name()
+                                                                                                 : "")];
+        [self addTopPanelAndSwitch: [panel autorelease]];
+      }
+    }    
+  }
+  
+  virtual bool select_view(mforms::AppView *view)
+  {
+    NSString *ident = [NSString stringWithUTF8String: view->identifier().c_str()];
+    
+    if ([self->topTabView indexOfTabViewItemWithIdentifier: ident] < 0)
+      return false;
+    [self->topTabView selectTabViewItemWithIdentifier: ident];
+    return false;
+  }
+  
+  virtual void undock_view(mforms::AppView *view)
+  {
+    WBMFormsPluginPanel *panel = [WBMFormsPluginPanel panelOfAppView: view];
+    
+    if (panel) 
+      [self closePanel: panel];
+  }
+  
+  virtual void set_view_title(mforms::AppView *view, const std::string &title)
+  {
+    WBMFormsPluginPanel *panel = [WBMFormsPluginPanel panelOfAppView: view];
+    if (panel)
+    {
+      [panel setTitle: [NSString stringWithCPPString: title]];
+      [self setTitle: [panel title] forPanel: panel];
+    }
+    else
+      NSLog(@"set_view_title called for undocked view");
+  }
+  
+  virtual void get_size(int *w, int *h)
+  {
+    NSRect rect = [[self window] frame];
+    *w = NSWidth(rect);
+    *h = NSHeight(rect);
+  }
+};
 
 //--------------------------------------------------------------------------------------------------
 
@@ -1265,6 +1255,28 @@ static std::string get_resource_path(mforms::App *app, const std::string &file)
   return "";
 }
 
+//--------------------------------------------------------------------------------------------------
+
+static std::string get_executable_path(mforms::App *app, const std::string &file)
+{
+  if (file.empty())
+    return [[[[NSBundle mainBundle] executablePath] stringByDeletingLastPathComponent] UTF8String];
+  
+  if (!file.empty() && file[0] == '/')
+    return file;
+  
+  {
+    char *fn = g_path_get_basename(file.c_str());
+    NSString *filename = [NSString stringWithUTF8String:fn];
+    g_free(fn);
+    
+    NSString *path = [[[NSBundle mainBundle] executablePath] stringByDeletingLastPathComponent];
+    return [[path stringByAppendingPathComponent: filename] UTF8String];
+  }
+  return "";
+}
+
+
 //--------------------------------------------------------------------------------------------------
 
 static std::string get_main_window_bounds(mforms::App *app)
@@ -1331,17 +1343,27 @@ static void exit_event_loop(mforms::App *app, int retcode)
   }
 }
 
-static MySQL::Drawing::Color get_system_color(mforms::SystemColor color)
+static base::Color get_system_color(mforms::SystemColor color)
 {
   switch (color)
   {
-    case mforms::SystemHighlight:
+    case mforms::SystemColorHighlight:
     {
       NSColor *c = [[NSColor alternateSelectedControlColor] colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
-      return MySQL::Drawing::Color([c redComponent], [c greenComponent], [c blueComponent]);
+      return base::Color([c redComponent], [c greenComponent], [c blueComponent]);
+    }
+    case mforms::SystemColorEditor:
+    {
+      NSColor *c = [[NSColor controlBackgroundColor] colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
+      return base::Color([c redComponent], [c greenComponent], [c blueComponent]);
+    }
+    case mforms::SystemColorDisabled:
+    {
+      NSColor *c = [[NSColor controlHighlightColor] colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
+      return base::Color([c redComponent], [c greenComponent], [c blueComponent]);
     }
   }
-  return MySQL::Drawing::Color::Color(1, 0, 0);
+  return base::Color::Color(1, 0, 0);
 }
 
 void setup_mforms_app(WBMainWindow *mwin)
@@ -1349,13 +1371,11 @@ void setup_mforms_app(WBMainWindow *mwin)
   mforms::ControlFactory *cf = mforms::ControlFactory::get_instance();
   g_assert(cf);
   
+  mforms::App::instantiate(new MainWindowDockingPoint(mwin), true);
   mforms::App::get()->set_data(mwin);
-
-  cf->_app_impl.dock_view= dock_view;
-  cf->_app_impl.select_view= select_view;
-  cf->_app_impl.undock_view= undock_view;
-  cf->_app_impl.set_view_title= set_view_title;
+  
   cf->_app_impl.get_resource_path= get_resource_path;
+  cf->_app_impl.get_executable_path= get_executable_path;
   cf->_app_impl.set_status_text= set_status_text;
   cf->_app_impl.get_bounds= get_main_window_bounds;
   cf->_app_impl.begin_thread_loop= alloc_ar_pool;
