@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -88,7 +88,7 @@ sharedSidebarController:(WBModelSidebarController*) controller
     [layerDS setHidesRootNode:YES];
 
     // setup navigator
-    for (int i= 0; i < sizeof(zoom_levels)/sizeof(int); i++)
+    for (int i= 0; i < (int)(sizeof(zoom_levels)/sizeof(int)); i++)
       [zoomCombo addItemWithObjectValue:[NSNumber numberWithFloat:zoom_levels[i]]];
     [navigatorViewer setupQuartz];
     [navigatorViewer setPostsFrameChangedNotifications:YES];
@@ -332,6 +332,7 @@ static NSPoint loadCursorHotspot(const std::string &path)
     [layerOutline selectRowIndexes: selectedIndexes byExtendingSelection: NO];
     
     _formBE->get_wb()->get_grt_manager()->soft_unlock_globals_tree();
+    [selectedIndexes release];
   }
 
 /*
@@ -518,7 +519,7 @@ static void refresh_layer_tree(const bec::NodeId &node, int old_count, WBModelDi
 {  
   NSPasteboard *pboard= [sender draggingPasteboard];
   NSString *data= [pboard stringForType:@"x-mysql-wb/db.DatabaseObject"];
-  NSPoint pos= [_viewer convertPointFromBase:[sender draggingLocation]];
+  NSPoint pos= [_viewer convertPoint:[sender draggingLocation] fromView: nil];
 
   return _formBE->perform_drop(pos.x, pos.y, "x-mysql-wb/db.DatabaseObject", [data UTF8String]);
 }
