@@ -1,10 +1,21 @@
-//
-//  WBModelSidebarController.h
-//  MySQLWorkbench
-//
-//  Created by Alfredo Kojima on 15/Dec/08.
-//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
-//
+/*
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
 
 #import <Cocoa/Cocoa.h>
 #import "GRTTreeDataSource.h"
@@ -19,7 +30,11 @@ namespace wb {
 @class GRTListDataSource;
 
 
-@interface WBModelSidebarController : NSObject<GRTDragDelegate>
+@interface WBModelSidebarController : NSObject<GRTDragDelegate
+#ifdef MAC_OS_X_VERSION_10_6
+   , NSOutlineViewDelegate
+#endif
+>
 {
   IBOutlet NSOutlineView *catalogOutline;
   IBOutlet NSTableView *userTypeTable;
