@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -158,7 +158,7 @@ static void refresh_user_types(const bec::NodeId &parent, int orow, WBModelSideb
   if (nodes.size() > 0)
   {
     // Copy only selected items if there are any.
-    for (int i = 0; i < nodes.size(); i++)
+    for (int i = 0; i < (int)nodes.size(); i++)
     {
       std::string label;
       model->get_field(nodes[i], 0, label);
@@ -239,6 +239,7 @@ static void refresh_user_types(const bec::NodeId &parent, int orow, WBModelSideb
     [[catalogOutline enclosingScrollView] reflectScrolledClipView: [[catalogOutline enclosingScrollView] contentView]];
     
     [catalogOutline selectRowIndexes: selectedIndexes byExtendingSelection: NO];
+    [selectedIndexes release];
   }
 }
 
