@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,7 +25,6 @@ using System.Reflection;
 using System.Windows.Forms;
 
 using Aga.Controls.Tree;
-using ScintillaNet;
 
 using MySQL.Controls;
 using MySQL.Forms;
@@ -70,6 +69,8 @@ namespace MySQL.GUI.Workbench
     private bool shuttingDown = false;
     private ImageList tabImageList = null;
 
+    public MainPageDockDelegate dockDelegate = null;
+
     #endregion
 
     #region Constructors
@@ -92,6 +93,8 @@ namespace MySQL.GUI.Workbench
       wbContext = WbContext;
       grtManager = wbContext.get_grt_manager();
 
+      dockDelegate = new MainPageDockDelegate(this, null);
+
       tabImageList = new ImageList();
       tabImageList.ColorDepth = ColorDepth.Depth32Bit;
       tabImageList.ImageSize = new Size(16, 16);
@@ -282,39 +285,6 @@ namespace MySQL.GUI.Workbench
       String result = "";
       switch (command) 
       {
-        case AppCommand.AppDockView:
-          if (str == "maintab")
-            DockDocument(view.GetHost(), true, true);
-          else
-            if (str == "editortab")
-              DockDocument(view.GetHost(), false, true);
-          break;
-        case AppCommand.AppSelectView:
-          foreach (ITabDocument document in contentTabControl.Documents)
-          {
-            if (document is AppViewDockContent)
-            {
-              AppViewDockContent content = (document as AppViewDockContent);
-              if (content.GetAppViewIdentifier() == str)
-              {
-                content.Activate();
-                result = "1";
-                break;
-              }
-            }
-          }
-          result = "0";
-          break;
-        case AppCommand.AppUndockView:
-          UndockDocument(view.GetHost());
-          break;
-        case AppCommand.AppSetViewTitle:
-          {
-            ITabDocument document = view.GetHost();
-            if (contentTabControl.IndexFromDocument(document) > 0)
-              view.GetHost().TabText = str;
-          }
-          break;
         case AppCommand.AppQuit: // Not yet used.
           Close();
           break;
@@ -504,37 +474,35 @@ namespace MySQL.GUI.Workbench
                 else
                 {
                   // If so, try to change the current GRT Object and exit
-                  if ((plugin as ObjectEditorView).EditorPlugin.ChangeGrtList(grtManager, GrtList))
+                  if ((plugin as ObjectEditorView).EditorPlugin.ReinitWithArguments(GrtList))
                   {
-                    // Release the current editor in the BE.
-                    PluginFormClosed(plugin, new FormClosedEventArgs(CloseReason.None));
+                    // Unregister plugin from back end.
+                    wbContext.close_gui_plugin(plugin.GetFixedPtr());
 
                     // return old Ptr as Ptr for the new editor
                     return plugin.GetFixedPtr();
                   }
                   else
-                    System.Diagnostics.Debug.Print(String.Format("Object editor for {0} does not support reuse", ClassName));
+                    Logger.LogDebug("Plugins .NET", 1, String.Format("Object editor for {0} does not support reuse", ClassName));
                 }
               }
 
               ObjectEditorPlugin objectEditorPlugin = Activator.CreateInstance(type, args) as ObjectEditorPlugin;
               if (objectEditorPlugin != null)
               {
+                objectEditorPlugin.Context = wbContext;
+
                 ObjectEditorView pluginForm;
                 if ((GUIPluginFlags.StandaloneWindowFlag & flags) != 0)
                   pluginForm = new StandaloneWindowPlugin(objectEditorPlugin);
                 else
-                  pluginForm = new DockedWindowPlugin(objectEditorPlugin);
-                //pluginForm.Show();
-
-                // Make sure form is removed from GRT editor list when closed
-                pluginForm.EditorPlugin.FormClosed += new FormClosedEventHandler(PluginFormClosed);
+                  pluginForm = new ObjectEditorView(objectEditorPlugin);
 
                 // Get fixed ptr
                 ptr = pluginForm.GetFixedPtr();
 
                 if (ptr == IntPtr.Zero)
-                  throw new Exception("Class not found");
+                  throw new Exception("Internal error: could not pin memory for plugin.");
               }
 
               break;
@@ -571,13 +539,13 @@ namespace MySQL.GUI.Workbench
         switch (ViewName)
         {
           case "dbquery":
-            DbSqlEditor db_sql_editor = new DbSqlEditor(wbContext, viewBE);
-            DockDocument(db_sql_editor, true, true);
+            form = new SqlIdeForm(wbContext, viewBE);
             break;
         }
-        if (null != form)
+        if (form != null)
         {
-          form.FormClosed += new FormClosedEventHandler(PluginFormClosed);
+          form.Context = wbContext;
+          DockDocument(form, true, true);
         }
       }
       catch (Exception e)
@@ -598,7 +566,7 @@ namespace MySQL.GUI.Workbench
             ObjectEditorView objectEditorView = (pluginForm as ObjectEditorPlugin).ContainerForm;
             if (objectEditorView is StandaloneWindowPlugin)
               objectEditorView.Show();
-            else if (objectEditorView is DockedWindowPlugin)
+            else if (objectEditorView is ObjectEditorView)
               DockDocument(objectEditorView, false, true);
           }
           else if (pluginForm is DockablePlugin && !(pluginForm is WizardPlugin))
@@ -622,17 +590,44 @@ namespace MySQL.GUI.Workbench
           editorForm.Hide();
       }
     }
+#endregion
+ 
+    #region DockingPointDelegate
+
+    public bool select_view(AppViewImpl view)
+    {
+      foreach (ITabDocument document in contentTabControl.Documents)
+      {
+        if (document is AppViewDockContent)
+        {
+          AppViewDockContent content = (document as AppViewDockContent);
+          if (content == view.GetHost())
+          {
+            content.Activate();
+            return true;
+          }
+        }
+      }
+      return false;
+    }
+
+    public void set_view_title(AppViewImpl view, String str)
+    {
+      ITabDocument document = view.GetHost();
+      if (contentTabControl.IndexFromDocument(document) > 0)
+        view.GetHost().TabText = str;
+    }
+
+    public void get_size(out int w, out int h)
+    {
+      w = contentTabControl.Width;
+      h = contentTabControl.Height;
+    }
+
+    #endregion
 
     #region Edit Menu Handlers
 
-    // Note: the EditCan* functions are called to determine the enabled state of the edit menu
-    //       items. However this is always interpreted as edit actions for WB items.
-    //       Hence, if a normal edit control is focused currently we have to return false.
-    //       This will make the menu entries disabled unfortunately, but allows us to do clipboard
-    //       operations in the edit controls.
-    //       We, nonetheless, handle the actions triggered by the shortcuts (which are processed
-    //       by WB).
-    // TODO: edit menu update must be reworked to also take normal platform specific actions into account.
     private void EditUndo()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
@@ -642,8 +637,8 @@ namespace MySQL.GUI.Workbench
         if (activeControl is ComboBox)
           Win32Api.Undo(activeControl as ComboBox);
         else
-          if (activeControl is Scintilla)
-            (activeControl as Scintilla).UndoRedo.Undo();
+          if (activeControl is ScintillaControl)
+            (activeControl as ScintillaControl).Undo();
           else
             if (wbContext.edit_can_undo())
               wbContext.edit_undo();
@@ -655,8 +650,8 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase || activeControl is ComboBox)
         return false;
       else
-        if (activeControl is Scintilla)
-          return (activeControl as Scintilla).UndoRedo.CanUndo;
+        if (activeControl is ScintillaControl)
+          return (activeControl as ScintillaControl).CanUndo;
         else
           return wbContext.edit_can_undo();
     }
@@ -667,8 +662,8 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase || activeControl is ComboBox)
         return;
       else
-        if (activeControl is Scintilla)
-          (activeControl as Scintilla).UndoRedo.Redo();
+        if (activeControl is ScintillaControl)
+          (activeControl as ScintillaControl).Redo();
         else
           if (wbContext.edit_can_redo())
             wbContext.edit_redo();
@@ -680,13 +675,12 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase || activeControl is ComboBox)
         return false;
       else
-        if (activeControl is Scintilla)
-          return (activeControl as Scintilla).UndoRedo.CanRedo;
+        if (activeControl is ScintillaControl)
+          return (activeControl as ScintillaControl).CanRedo;
         else
           return wbContext.edit_can_redo();
     }
 
-
     private void EditCopy()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
@@ -696,8 +690,8 @@ namespace MySQL.GUI.Workbench
         if (activeControl is ComboBox)
           Win32Api.Copy(activeControl as ComboBox);
         else
-          if (activeControl is Scintilla)
-            (activeControl as Scintilla).Clipboard.Copy();
+          if (activeControl is ScintillaControl)
+            (activeControl as ScintillaControl).Copy();
           else
             if (activeControl is GridView)
               (activeControl as GridView).Copy();
@@ -712,8 +706,8 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase || activeControl is ComboBox)
         return false;
       else
-        if (activeControl is Scintilla)
-          return (activeControl as Scintilla).Selection.Length > 0;
+        if (activeControl is ScintillaControl)
+          return (activeControl as ScintillaControl).CanCopy;
         else
           if (activeControl is GridView)
             return (activeControl as GridView).CanCopy;
@@ -730,8 +724,8 @@ namespace MySQL.GUI.Workbench
         if (activeControl is ComboBox)
           Win32Api.Cut(activeControl as ComboBox);
         else
-          if (activeControl is Scintilla)
-              (activeControl as Scintilla).Clipboard.Cut();
+          if (activeControl is ScintillaControl)
+            (activeControl as ScintillaControl).Cut();
             else
               if (wbContext.edit_can_copy())
                 wbContext.edit_cut();
@@ -743,8 +737,8 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase || activeControl is ComboBox)
         return false;
       else
-        if (activeControl is Scintilla)
-          return (activeControl as Scintilla).Selection.Length > 0;
+        if (activeControl is ScintillaControl)
+          return (activeControl as ScintillaControl).CanCut;
         else
           if ((activeControl is TreeViewAdv) || (activeControl != null && activeControl.Parent is TreeViewAdv))
             return false; // Returning false tells the backend not to take over the operation.
@@ -761,8 +755,8 @@ namespace MySQL.GUI.Workbench
         if (activeControl is ComboBox)
           Win32Api.Paste(activeControl as ComboBox);
         else
-          if (activeControl is Scintilla)
-            (activeControl as Scintilla).Clipboard.Paste();
+          if (activeControl is ScintillaControl)
+            (activeControl as ScintillaControl).Paste();
           else
             if (wbContext.edit_can_paste())
               wbContext.edit_paste();
@@ -774,8 +768,8 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase || activeControl is ComboBox || activeControl is GridView)
         return false;
       else
-        if (activeControl is Scintilla)
-          return (activeControl as Scintilla).Clipboard.CanPaste;
+        if (activeControl is ScintillaControl)
+          return (activeControl as ScintillaControl).CanPaste;
         else
           return wbContext.edit_can_paste();
     }
@@ -783,14 +777,17 @@ namespace MySQL.GUI.Workbench
     private void EditSelectAll()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is TextBoxBase)
-        (activeControl as TextBoxBase).SelectAll();
+      if (activeControl is ComboBox)
+        (activeControl as ComboBox).SelectAll();
       else
-        if (activeControl is Scintilla)
-          (activeControl as Scintilla).Selection.SelectAll();
+        if (activeControl is TextBoxBase)
+          (activeControl as TextBoxBase).SelectAll();
         else
-          if (wbContext.edit_can_select_all())
-            wbContext.edit_select_all();
+          if (activeControl is ScintillaControl)
+            (activeControl as ScintillaControl).SelectAll();
+          else
+            if (wbContext.edit_can_select_all())
+              wbContext.edit_select_all();
     }
 
     private bool EditCanSelectAll()
@@ -799,7 +796,7 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase || activeControl is ComboBox)
         return false;
       else
-        if (activeControl is Scintilla)
+        if (activeControl is ScintillaControl)
           return true;
         else
           return wbContext.edit_can_select_all();
@@ -811,8 +808,8 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase)
         (activeControl as TextBoxBase).SelectedText = "";
       else
-        if (activeControl is Scintilla)
-          (activeControl as Scintilla).Selection.Text = "";
+        if (activeControl is ScintillaControl)
+          (activeControl as ScintillaControl).Delete();
         else
           if (wbContext.edit_can_delete())
             wbContext.edit_delete();
@@ -824,19 +821,17 @@ namespace MySQL.GUI.Workbench
       if (activeControl is TextBoxBase || activeControl is ComboBox)
         return false;
       else
-        if (activeControl is Scintilla)
-          return (activeControl as Scintilla).Selection.Length > 0;
+        if (activeControl is ScintillaControl)
+          return (activeControl as ScintillaControl).CanDelete;
         else
           return wbContext.edit_can_delete();
     }
 
     private void EditFind()
     {
-      // TODO: unify search/replace calls, currently they take two paths (via main menu and
-      // via SQL editor partial ui refresh (if an editor is active).
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is Scintilla)
-        (activeControl as Scintilla).Commands.Execute(BindableCommand.ShowFind);
+      if (activeControl is ScintillaControl)
+        (activeControl as ScintillaControl).ShowFindPanel(false);
       else
       {
         // Not an editor control. So try focusing the search box in the toolbar (if there's any).
@@ -852,28 +847,20 @@ namespace MySQL.GUI.Workbench
       if (wbDocument == null)
         return false;
 
-      if (wbDocument is DbSqlEditor)
-      {
-        Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-        if (activeControl is Scintilla)
-          return true;
-        return false;
-      }
-
       return true;
     }
 
     private void EditFindReplace()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is Scintilla)
-        (activeControl as Scintilla).Commands.Execute(BindableCommand.ShowReplace);
+      if (activeControl is ScintillaControl)
+        (activeControl as ScintillaControl).ShowFindPanel(true);
     }
 
     private bool EditCanFindReplace()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is Scintilla)
+      if (activeControl is ScintillaControl)
         return true;
       return false;
     }
@@ -896,9 +883,7 @@ namespace MySQL.GUI.Workbench
       }
     }
     
-    #endregion
-
-    private void RegisterCommands()
+    public void RegisterCommands()
     {
       List<String> commands= new List<string>();
 
@@ -1085,15 +1070,18 @@ namespace MySQL.GUI.Workbench
 
     #endregion
 
-    #region Standard Forms Handling
+    #region Other Application Logic
 
     /// <summary>
-    /// Try to find the given file in known locations. For now we only search the image paths.
-    /// That will get improved later to find any resource in any location (including resource DLLs etc.).
+    /// Try to find the given file in known locations. If name is empty return the absolute application
+    /// path (as common root for all resource subfolders).
     /// </summary>
     /// <param name="name"></param>
     static public string GetIconPath(string name)
     {
+      if (name == "")
+        return Environment.CurrentDirectory;
+
       string result = "";
       if (File.Exists(name))
         return name;
@@ -1142,11 +1130,6 @@ namespace MySQL.GUI.Workbench
       workbenchPhysicalOverviewForm.Activate();
     }
 
-    public void SetupUI()
-    {
-      RegisterCommands();
-    }
-
     public void LockGUI(bool value)
     {
       if (value)
@@ -1160,6 +1143,115 @@ namespace MySQL.GUI.Workbench
       // Ignore.
     }
 
+    /// <summary>
+    /// Restores the form to a usable state and activates it.
+    /// </summary>
+    public void Activate(bool restoreIfMinimized)
+    {
+      if (restoreIfMinimized && WindowState == FormWindowState.Minimized)
+        WindowState = FormWindowState.Normal;
+      Activate();
+    }
+
+    /// <summary>
+    /// Make the upper part of the window extend the glass frame (if we are on Aero).
+    /// </summary>
+    /// <param name="toolbarHeight"></param>
+    private void AdjustGlassFrame(int toolbarHeight)
+    {
+      if (ControlUtilities.IsCompositionEnabled())
+      {
+        int topAreaHeight = contentTabControl.ItemSize.Height + contentTabControl.Margin.Top +
+          toolbarHeight + Margin.Top;
+
+        Win32.MARGINS margins = new Win32.MARGINS(2, topAreaHeight, 2, 0);
+        Win32.DwmExtendFrameIntoClientArea(Handle, margins);
+      }
+    }
+
+    /// <summary>
+    /// Prepares the given document to conform to the overall layout. This requires to draw an
+    /// intermediate set of controls (panels) and add menu + toolbar.
+    /// </summary>
+    /// <param name="document">The document to adjust, which comes with the actual content.</param>
+    /// <param name="backend">The backend UI form used to get toolbar and menu.</param>
+    private void SetupDockLayout(ITabDocument document, UIForm backend)
+    {
+      // Setting up the layout requires some additional nesting of controls.
+      // A tab document should have only one container as top level control, which we re-host.
+      Control root = document.Content;
+      Control content = (root.Controls.Count > 0) ? root.Controls[0] : null;
+
+      root.Controls.Clear();
+
+      // Content area.
+      DrawablePanel contentPanel = new DrawablePanel();
+      contentPanel.BackColor = Color.FromArgb(255, 40, 55, 82);
+      contentPanel.CustomBackground = false;
+      contentPanel.Dock = DockStyle.Fill;
+      contentPanel.Padding = new Padding(7);
+      root.Controls.Add(contentPanel);
+      if (content != null)
+      {
+        contentPanel.Controls.Add(content);
+        content.Dock = DockStyle.Fill;
+      }
+
+      // Menu + toolbar area. We have two types of tab documents:
+      // - appview based (and hence backend-managed)
+      // - native Form (with interface via WBContext).
+      // Appview based documents have own methods to return their menu and toolbar because
+      // retrieval via managed code requires a managed UIForm. However appview classes are
+      // already managed via mforms and adding another reference management via UIForm
+      // will lead to unpredictable results.
+      ToolStrip toolbar;
+      MenuStrip menu;
+      if (backend != null)
+      {
+        // Retrieval via WBContext will return a default toolbar if no other is defined.
+        toolbar = wbContext.toolbar_for_form(backend);
+        menu = wbContext.menu_for_form(backend);
+      }
+      else
+      {
+        AppViewDockContent dockContent = document as AppViewDockContent;
+        toolbar = dockContent.GetAppView().GetToolBar();
+        if (toolbar == null)
+          toolbar = wbContext.toolbar_for_form(null);
+        menu = dockContent.GetAppView().GetMenuBar();
+        if (menu == null)
+          menu = wbContext.menu_for_form(null);
+      }
+      DrawablePanel menuPanel = new DrawablePanel();
+      if (toolbar != null || menu != null)
+      {
+        // For menu + toolbar we have a special gradient background, which we implement by
+        // placing them on another panel.
+        menuPanel.BackColor = Color.Transparent;
+        menuPanel.CustomBackground = true;
+        menuPanel.Dock = DockStyle.Top;
+        menuPanel.Padding = new Padding(5, 0, 5, 0);
+        menuPanel.AutoSize = true;
+        menuPanel.PaintBackground += new EventHandler<PaintEventArgs>(menuPanel_PaintBackground);
+
+        root.Controls.Add(menuPanel);
+        if (toolbar != null)
+        {
+          menuPanel.Controls.Add(toolbar);
+          toolbar.Dock = DockStyle.Top;
+          toolbar.AutoSize = true;
+          toolbar.Padding = new Padding(0, 0, 0, 3);
+        }
+
+        if (menu != null)
+        {
+          menuPanel.Controls.Add(menu);
+          menu.Dock = DockStyle.Top;
+          menu.AutoSize = true;
+        }
+      }
+    }
+
     #endregion
 
     #region Document Handling
@@ -1301,23 +1393,6 @@ namespace MySQL.GUI.Workbench
       AdjustGlassFrame(topAreaHeight);
     }
 
-    /// <summary>
-    /// Remove plugin form from WbContext editor cache.
-    /// </summary>
-    /// <param name="sender">The form</param>
-    /// <param name="e">The close event arguments</param>
-    internal void PluginFormClosed(object sender, FormClosedEventArgs e)
-    {
-      DockablePlugin plugin = sender as DockablePlugin;
-
-      if (plugin != null)
-      {
-        wbContext.close_gui_plugin(plugin.GetFixedPtr());
-      }
-
-      Program.CollectGarbageOnIdle();
-    }
-
     private void ForwardRefreshToActivDocument(RefreshType refresh, String str, IntPtr ptr)
     {
       if (contentTabControl.ActiveDocument is IWorkbenchDocument)
@@ -1351,7 +1426,7 @@ namespace MySQL.GUI.Workbench
     /// <param name="document">The document to dock.</param>
     /// <param name="main">If true then dock that document to our own tab control, otherwise
     /// dock it to the currently active main document.</param>
-    private void DockDocument(ITabDocument document, bool main, bool activate)
+    public void DockDocument(ITabDocument document, bool main, bool activate)
     {
       if (main)
       {
@@ -1386,8 +1461,8 @@ namespace MySQL.GUI.Workbench
           if (host is ModelOverviewForm)
             (host as ModelOverviewForm).DockDocument(document, activate);
           else
-            if (host is DbSqlEditor)
-              (host as DbSqlEditor).DockDocument(document, activate);
+            if (host is SqlIdeForm)
+              (host as SqlIdeForm).DockDocument(document, activate);
       }
     }
 
@@ -1396,7 +1471,7 @@ namespace MySQL.GUI.Workbench
     /// If not forward the call to the currently active document.
     /// </summary>
     /// <param name="document"></param>
-    private void UndockDocument(ITabDocument document)
+    public void UndockDocument(ITabDocument document)
     {
       if (contentTabControl.HasDocument(document))
         contentTabControl.RemoveDocument(document);
@@ -1409,8 +1484,8 @@ namespace MySQL.GUI.Workbench
           if (host is ModelOverviewForm)
             (host as ModelOverviewForm).UndockDocument(document);
           else
-            if (host is DbSqlEditor)
-              (host as DbSqlEditor).UndockDocument(document);
+            if (host is SqlIdeForm)
+              (host as SqlIdeForm).UndockDocument(document);
       }
     }
 
@@ -1432,23 +1507,75 @@ namespace MySQL.GUI.Workbench
 
     private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
     {
+      // Temporarily remove the focus from what ever control has it currently.
+      // This way controls that save content on leave get their chance.
+      Focus();
+
       shuttingDown = true;
 
-      e.Cancel = !wbContext.request_quit();
+      // Query all documents if they can close in advance, so they stay open if one says nay.
+      // For historical reasons diagram and model overview pages are on the same tab level
+      // (the main one) even though diagrams are subordinated to model tabs. This means we have
+      // to do two rounds to ensure diagram tabs are checked before the model tabs are.
+      foreach (ITabDocument document in contentTabControl.Documents)
+      {
+        if (document is ModelDiagramForm)
+        {
+          e.Cancel = !(document as IWorkbenchDocument).CanCloseDocument();
+          
+          // Make the page active that refused to close.
+          if (e.Cancel)
+          {
+            document.Activate();
+            break;
+          }
+        }
+      }
+
+      if (!e.Cancel)
+      {
+        foreach (ITabDocument document in contentTabControl.Documents)
+        {
+          if (document is ModelDiagramForm)
+            continue;
+
+          if (document is IWorkbenchDocument)
+            e.Cancel = !(document as IWorkbenchDocument).CanCloseDocument();
+          else
+            if (document is MySQL.Forms.AppViewDockContent)
+            {
+              MySQL.Forms.AppViewDockContent content = document as MySQL.Forms.AppViewDockContent;
+              e.Cancel = !content.GetAppView().DocumentClosing();
+            }
+
+          if (e.Cancel)
+          {
+            // Make the page active that refused to close.
+            document.Activate();
+            break;
+          }
+        }
+      }
+
+      if (!e.Cancel)
+        e.Cancel = !wbContext.request_quit();
 
-      // Reset termination flag, in case it was set.
       if (e.Cancel)
       {
+        // Reset termination flag, in case it was set.
         grtManager.resetTermination();
         shuttingDown = false;
         return;
       }
 
+      // Restore previous clipboard chain.
+      Win32.ChangeClipboardChain(this.Handle, nextClipboardViewer);
       RemoveFocusChangeHandlers(this);
       SaveFormState();
 
       Hide();
-      // Go through each workbench document and close it unconditionally.
+
+      // Go through each workbench document and close it, now unconditionally.
       contentTabControl.SuspendLayout();
       try
       {
@@ -1503,11 +1630,6 @@ namespace MySQL.GUI.Workbench
       // Don't restore into minimized state. Confusing for the user and makes trouble for layouting.
       if (WindowState == FormWindowState.Minimized)
         WindowState = FormWindowState.Normal;
-
-      /* Disabled by order.
-      if (wbContext.read_state("outputvisible", "mainform", 0) == 1)
-        wbContext.show_output();
-      */
     }
 
     private void SaveFormState()
@@ -1546,6 +1668,9 @@ namespace MySQL.GUI.Workbench
       LoadFormState();
       InstallFocusChangeHandlers(this);
 
+      // Register the form as clipboard listener so we get notified about changes there.
+      nextClipboardViewer = Win32.SetClipboardViewer(this.Handle);
+
       SplashScreen.CloseSplash();
 
       BringToFront();
@@ -1613,8 +1738,6 @@ namespace MySQL.GUI.Workbench
         InstallFocusChangeHandlers(sub);
     }
 
-    #endregion
-
     private void tabControl_TabClosing(object sender, TabClosingEventArgs e)
     {
       e.canClose = true;
@@ -1623,159 +1746,31 @@ namespace MySQL.GUI.Workbench
       if (document == null)
         return;
 
-      if (document is DbSqlEditor)
-        e.canClose = (document as DbSqlEditor).Backend.can_close();
-      if (document is Plugins.DockablePlugin)
-      {
-        Plugins.DockablePlugin plugin = document as Plugins.DockablePlugin;
-        plugin.RunOnFormClosing(null);
-      }
+      if (document is IWorkbenchDocument)
+        e.canClose = (document as IWorkbenchDocument).CanCloseDocument();
       else
-        if (document is IWorkbenchDocument)
-          e.canClose = (document as IWorkbenchDocument).BackendForm.can_close();
-        else
-          if (document is MySQL.Forms.AppViewDockContent)
-          {
-            MySQL.Forms.AppViewDockContent content = document as MySQL.Forms.AppViewDockContent;
-            e.canClose = content.GetAppView().DocumentClosing();
-          }
+        if (document is MySQL.Forms.AppViewDockContent)
+        {
+          MySQL.Forms.AppViewDockContent content = document as MySQL.Forms.AppViewDockContent;
+          e.canClose = content.GetAppView().DocumentClosing();
+        }
 
       if (e.canClose)
         RemoveFocusChangeHandlers(document as Control);
-
-      // Because of the order of processing in the UI we have to anticipate notification handling.
-      // It is too late when the regular messages come in (then the page for which they are meant
-      // is already gone, leaving so for instance plugins registered, which don't exist anymore).
-      if (e.canClose && document is ModelOverviewForm)
-        (document as ModelOverviewForm).RefreshGUI(RefreshType.RefreshCloseEditor, "", IntPtr.Zero);
     }
 
     private void contentTabControl_TabClosed(object sender, TabClosedEventArgs e)
     {
       ITabDocument document = (sender as FlatTabControl).DocumentFromPage(e.page);
-      if (document is DbSqlEditor)
-        document.Close();
-      else if (document is Plugins.DockablePlugin)
-        PluginFormClosed(sender, null);
+      if (document is IWorkbenchDocument)
+        (document as IWorkbenchDocument).CloseDocument();
       else
-        if (document is IWorkbenchDocument)
-          (document as IWorkbenchDocument).BackendForm.close();
-        else
-          if (document is MySQL.Forms.AppViewDockContent)
-          {
-            // Remove the appview from the page, otherwise it gets disposed with the page,
-            // what we don't want.
-            e.page.Controls.Clear();
-          }
-    }
-
-    /// <summary>
-    /// Restores the form to a usable state and activates it.
-    /// </summary>
-    public void Activate(bool restoreIfMinimized)
-    {
-      if (restoreIfMinimized && WindowState == FormWindowState.Minimized)
-        WindowState = FormWindowState.Normal;
-      Activate();
-    }
-
-    /// <summary>
-    /// Make the upper part of the window extend the glass frame (if we are on Aero).
-    /// </summary>
-    /// <param name="toolbarHeight"></param>
-    private void AdjustGlassFrame(int toolbarHeight)
-    {
-      if (ControlUtilities.IsCompositionEnabled())
-      {
-        int topAreaHeight = contentTabControl.ItemSize.Height + contentTabControl.Margin.Top +
-          toolbarHeight + Margin.Top;
-
-        Win32.MARGINS margins = new Win32.MARGINS(2, topAreaHeight, 2, 0);
-        Win32.DwmExtendFrameIntoClientArea(Handle, margins);
-      }
-    }
-
-    /// <summary>
-    /// Prepares the given document to conform to the overall layout. This requires to draw an
-    /// intermediate set of controls (panels) and add menu + toolbar.
-    /// </summary>
-    /// <param name="document">The document to adjust, which comes with the actual content.</param>
-    /// <param name="backend">The backend UI form used to get toolbar and menu.</param>
-    private void SetupDockLayout(ITabDocument document, UIForm backend)
-    {
-      // Setting up the layout requires some additional nesting of controls.
-      // A tab document should have only one container as top level control, which we re-host.
-      Control root = document.Content;
-      Control content = (root.Controls.Count > 0) ? root.Controls[0] : null;
-
-      root.Controls.Clear();
-
-      // Content area.
-      DrawablePanel contentPanel = new DrawablePanel();
-      contentPanel.BackColor = Color.FromArgb(255, 40, 55, 82);
-      contentPanel.CustomBackground = false;
-      contentPanel.Dock = DockStyle.Fill;
-      contentPanel.Padding = new Padding(7);
-      root.Controls.Add(contentPanel);
-      if (content != null)
-      {
-        contentPanel.Controls.Add(content);
-        content.Dock = DockStyle.Fill;
-      }
-
-      // Menu + toolbar area. We have two types of tab documents:
-      // - appview based (and hence backend-managed)
-      // - native Form (with interface via WBContext).
-      // Appview based documents have own methods to return their menu and toolbar because
-      // retrieval via managed code requires a managed UIForm. However appview classes are
-      // already managed via mforms and adding another reference management via UIForm
-      // will lead to unpredictable results.
-      ToolStrip toolbar;
-      MenuStrip menu;
-      if (backend != null)
-      {
-        // Retrieval via WBContext will return a default toolbar if no other is defined.
-        toolbar = wbContext.toolbar_for_form(backend);
-        menu = wbContext.menu_for_form(backend);
-      }
-      else
-      {
-        AppViewDockContent dockContent = document as AppViewDockContent;
-        toolbar = dockContent.GetAppView().GetToolBar();
-        if (toolbar == null)
-          toolbar = wbContext.toolbar_for_form(null);
-        menu = dockContent.GetAppView().GetMenuBar();
-        if (menu == null)
-          menu = wbContext.menu_for_form(null);
-      }
-      DrawablePanel menuPanel = new DrawablePanel();
-      if (toolbar != null || menu != null)
-      {
-        // For menu + toolbar we have a special gradient background, which we implement by
-        // placing them on another panel.
-        menuPanel.BackColor = Color.Transparent;
-        menuPanel.CustomBackground = true;
-        menuPanel.Dock = DockStyle.Top;
-        menuPanel.Padding = new Padding(5, 0, 5, 0);
-        menuPanel.AutoSize = true;
-        menuPanel.PaintBackground += new EventHandler<PaintEventArgs>(menuPanel_PaintBackground);
-
-        root.Controls.Add(menuPanel);
-        if (toolbar != null)
+        if (document is MySQL.Forms.AppViewDockContent)
         {
-          menuPanel.Controls.Add(toolbar);
-          toolbar.Dock = DockStyle.Top;
-          toolbar.AutoSize = true;
-          toolbar.Padding = new Padding(0, 0, 0, 3);
+          // Remove the appview from the page, otherwise it gets disposed with the page,
+          // what we don't want.
+          e.page.Controls.Clear();
         }
-
-        if (menu != null)
-        {
-          menuPanel.Controls.Add(menu);
-          menu.Dock = DockStyle.Top;
-          menu.AutoSize = true;
-        }
-      }
     }
 
     private Bitmap logo = null;
@@ -1845,7 +1840,7 @@ namespace MySQL.GUI.Workbench
             closeOtherTabsMenuItem.Enabled = contentTabControl.TabCount > 1;
 
             ITabDocument document = contentTabControl.DocumentFromIndex(clickedIndex);
-            closeTabsOfSameTypeItem.Enabled = document is DbSqlEditor || document is ModelDiagramForm
+            closeTabsOfSameTypeItem.Enabled = document is SqlIdeForm || document is ModelDiagramForm
               || document is ModelOverviewForm;
 
             // Keep the found index for later handling in the item click handler.
@@ -1857,6 +1852,82 @@ namespace MySQL.GUI.Workbench
           break;
       }
     }
+    
+    #endregion
+
+    #region Windows Message Handling
+
+    private IntPtr nextClipboardViewer = IntPtr.Zero;
+
+    protected override void WndProc(ref Message m)
+    {
+      switch ((WM)m.Msg)
+      {
+        case WM.DRAWCLIPBOARD:
+          // Content of the clipboard changed. Notify next viewer in the chain.
+          if (nextClipboardViewer != IntPtr.Zero)
+            Win32.SendMessage(nextClipboardViewer, m.Msg, m.WParam, m.LParam);
+
+          ManagedNotificationCenter.Send("GNClipboardChanged", this.Handle);
+          break;
+
+        case WM.CHANGECBCHAIN:
+          // The clipboard chain changed. Update our next pointer.
+          if (m.WParam == nextClipboardViewer)
+            nextClipboardViewer = m.LParam;
+          else
+            Win32.SendMessage(nextClipboardViewer, m.Msg, m.WParam, m.LParam);
+          break;
+
+        default:
+          base.WndProc(ref m);
+          break;
+      }
+    }
+
+    #endregion
 
   }
+
+  public class MainPageDockDelegate : DockingPointDelegateWrapper
+  {
+    private MainForm _owner = null;
+
+    public MainPageDockDelegate(MainForm owner, Object representedObject)
+      : base(representedObject)
+    {
+      _owner = owner;
+    }
+
+    public override String get_type(Object representedObject)
+    {
+      return "MainWindow";
+    }
+
+    public override void dock_view(Object representedObject, AppViewImpl view, String arg1, int arg2)
+    {
+      _owner.DockDocument(view.GetHost(), true, true);
+    }
+
+    public override void undock_view(Object representedObject, AppViewImpl view)
+    {
+      _owner.UndockDocument(view.GetHost());
+    }
+
+    public override bool select_view(Object representedObject, AppViewImpl view)
+    {
+      return _owner.select_view(view);
+    }
+
+    public override void set_view_title(Object representedObject, AppViewImpl view, String str)
+    {
+      _owner.set_view_title(view, str);
+    }
+
+    public override void get_size(Object representedObject, out int w, out int h)
+    {
+      _owner.get_size(out w, out h);
+    }
+  };
+
 }
