@@ -17,6 +17,7 @@
  * 02110-1301  USA
  */
 
+
 using System;
 using System.Collections.Generic;
 using System.Drawing;
@@ -24,21 +25,20 @@ using System.IO;
 using System.Windows.Forms;
 
 using MySQL.Controls;
-using MySQL.Forms;
 using MySQL.Grt;
 using MySQL.GUI.Mdc;
 using MySQL.GUI.Workbench.Plugins;
-using MySQL.Utilities.SysUtils;
+using MySQL.Forms;
 using MySQL.Workbench;
+using MySQL.Utilities.SysUtils;
 
 namespace MySQL.GUI.Workbench
 {
-  public partial class ModelDiagramForm : TabDocument, IWorkbenchDocument
+  public partial class ModelDiagramForm : TabDocument, IWorkbenchDocument, IWorkbenchObserver
   {
     #region Member Variables
     
     private WbContext wbContext;
-    private WorkbenchDocumentClosing onWorkbenchDocumentClosing = null;
     private SortedDictionary<String, Cursor> cursors = new SortedDictionary<String, Cursor>();
     private MySQL.Workbench.ModelDiagramFormWrapper formBE = null;
 
@@ -98,6 +98,8 @@ namespace MySQL.GUI.Workbench
       optionsToolStrip.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right;
       optionsToolStrip.Hide();
 
+      ManagedNotificationCenter.AddObserver(this, "GNFormTitleDidChange");
+
       FocusCanvasControl();
     }
 
@@ -105,26 +107,18 @@ namespace MySQL.GUI.Workbench
     {
       formBE.Dispose();
 
-      // Explicitly close all editors to ensure plugins are unregistered from the backend.
-      foreach (ITabDocument content in bottomTabControl.Documents)
-        CloseTabDocument(content);
+      ManagedNotificationCenter.RemoveObserver(this, "GNFormTitleDidChange");
+
+      // No need to again close all docked editors here. This happened already in CloseDocument().
 
       canvasViewer.FinalizeCanvas();
 
-      if (onWorkbenchDocumentClosing != null)
-        onWorkbenchDocumentClosing(this, new FormClosingEventArgs(CloseReason.FormOwnerClosing, false));
     }
 
     #endregion
     
     #region IWorkbenchDocument Interface
-    
-    public WorkbenchDocumentClosing OnWorkbenchDocumentClosing
-    {
-      get { return onWorkbenchDocumentClosing; }
-      set { onWorkbenchDocumentClosing = value; }
-    }
-    
+
     public UIForm BackendForm
     {
       get { return formBE; }
@@ -176,6 +170,18 @@ namespace MySQL.GUI.Workbench
       }
     }
 
+    /// <summary>
+    /// Notifications coming in from the notification center.
+    /// </summary>
+    /// <param name="name">The name of the notification. We only get called for notifications we want.</param>
+    /// <param name="sender">The object that caused the notification to be sent.</param>
+    /// <param name="info">Name/Value string pairs for data to be passed.</param>
+    public void HandleNotification(string name, IntPtr sender, Dictionary<string, string> info)
+    {
+      if (name == "GNFormTitleDidChange" && info["form"] == formBE.form_id())
+        TabText = info["title"];
+    }
+
     public void PerformCommand(String command)
     {
       switch (command)
@@ -224,6 +230,9 @@ namespace MySQL.GUI.Workbench
       foreach (ITabDocument content in bottomTabControl.Documents)
         if (content is ObjectEditorView && (content as ObjectEditorView).EditorPlugin.GetType() == type)
         {
+          // Unregister plugin from back end.
+          wbContext.close_gui_plugin((content as ObjectEditorView).EditorPlugin.GetFixedPtr());
+
           content.Close();
           if (bottomTabControl.TabCount == 0)
             contentSplitContainer.Panel2Collapsed = true;
@@ -233,6 +242,26 @@ namespace MySQL.GUI.Workbench
       return false;
     }
 
+    public bool CanCloseDocument()
+    {
+      foreach (ITabDocument document in bottomTabControl.Documents)
+        if (document is IWorkbenchDocument)
+          if (!(document as IWorkbenchDocument).CanCloseDocument())
+            return false;
+
+      if (!BackendForm.can_close())
+        return false;
+
+      return true;
+    }
+
+    public void CloseDocument()
+    {
+      foreach (ITabDocument document in bottomTabControl.Documents)
+        CloseTabDocument(document);
+      BackendForm.close();
+    }
+
     #endregion
 
     #region Properties
@@ -367,7 +396,7 @@ namespace MySQL.GUI.Workbench
     {
       ITabDocument document = (sender as FlatTabControl).DocumentFromPage(e.page);
       if (document is IWorkbenchDocument)
-        e.canClose = (document as IWorkbenchDocument).BackendForm.can_close();
+        e.canClose = (document as IWorkbenchDocument).CanCloseDocument();
       else
         if (document is MySQL.Forms.AppViewDockContent)
         {
@@ -387,21 +416,14 @@ namespace MySQL.GUI.Workbench
 
     private void CloseTabDocument(ITabDocument document)
     {
-      if (document is Plugins.DockablePlugin)
-      {
-        Plugins.DockablePlugin plugin = document as Plugins.DockablePlugin;
-        wbContext.close_gui_plugin(plugin.GetFixedPtr());
-        plugin.Dispose();
-      }
+      if (document is IWorkbenchDocument)
+        (document as IWorkbenchDocument).CloseDocument();
       else
-        if (document is IWorkbenchDocument)
-          (document as IWorkbenchDocument).BackendForm.close();
-        else
-          if (document is MySQL.Forms.AppViewDockContent)
-          {
-            // This type of document is already closed (in TabClosing).
-            // TODO: adjust code to support OnClosing/OnClose duality also in AppView.
-          }
+        if (document is MySQL.Forms.AppViewDockContent)
+        {
+          // This type of document is already closed (in TabClosing).
+          // TODO: adjust code to support OnClosing/OnClose duality also in AppView.
+        }
     }
     private void ModelDiagramForm_Shown(object sender, EventArgs e)
     {
