@@ -21,17 +21,17 @@ using System;
 using System.Collections;
 using System.Collections.Generic;
 using System.Drawing;
-using System.IO;
 using System.Windows.Forms;
-
 using MySQL.Controls;
-using MySQL.Forms;
+
 using MySQL.Grt;
 using MySQL.GUI.Workbench.Plugins;
 using MySQL.GUI.Workbench.Properties;
 using MySQL.Utilities;
 using MySQL.Utilities.SysUtils;
+using MySQL.Forms;
 using MySQL.Workbench;
+using System.IO;
 
 namespace MySQL.GUI.Workbench
 {
@@ -165,7 +165,7 @@ namespace MySQL.GUI.Workbench
               control = control.Parent;
             if (control != null)
             {
-              Win32.SendMessage(control.Handle, (uint) m.Msg, m.WParam, m.LParam);
+              Win32.SendMessage(control.Handle, m.Msg, m.WParam, m.LParam);
               return true;
             }
           }
@@ -179,12 +179,6 @@ namespace MySQL.GUI.Workbench
 
     #region IWorkbenchDocument Interface
 
-    public WorkbenchDocumentClosing OnWorkbenchDocumentClosing
-    {
-      get { return null; }
-      set { ; }
-    }
-
     public UIForm BackendForm
     {
       get { return wbOverview.get_uiform(); }
@@ -272,6 +266,9 @@ namespace MySQL.GUI.Workbench
       foreach (ITabDocument content in bottomTabControl.Documents)
         if (content is ObjectEditorView && (content as ObjectEditorView).EditorPlugin.GetType() == type)
         {
+          // Unregister plugin from back end.
+          wbContext.close_gui_plugin((content as ObjectEditorView).EditorPlugin.GetFixedPtr());
+
           content.Close();
           if (bottomTabControl.TabCount == 0)
             contentSplitContainer.Panel2Collapsed = true;
@@ -285,11 +282,11 @@ namespace MySQL.GUI.Workbench
 
     #region Event Handling
 
-    private void tabControl_TabClosing(object sender, TabClosingEventArgs e)
+    private void bottomTabControl_TabClosing(object sender, TabClosingEventArgs e)
     {
       ITabDocument document = (sender as FlatTabControl).DocumentFromPage(e.page);
       if (document is IWorkbenchDocument)
-        e.canClose = (document as IWorkbenchDocument).BackendForm.can_close();
+        e.canClose = (document as IWorkbenchDocument).CanCloseDocument();
       else
         if (document is MySQL.Forms.AppViewDockContent)
         {
@@ -304,21 +301,14 @@ namespace MySQL.GUI.Workbench
         contentSplitContainer.Panel2Collapsed = true;
 
       ITabDocument document = (sender as FlatTabControl).DocumentFromPage(e.page);
-      if (document is Plugins.DockablePlugin)
-      {
-        Plugins.DockablePlugin plugin = document as Plugins.DockablePlugin;
-        wbContext.close_gui_plugin(plugin.GetFixedPtr());
-        plugin.Dispose();
-      }
+      if (document is IWorkbenchDocument)
+        (document as IWorkbenchDocument).CloseDocument();
       else
-        if (document is IWorkbenchDocument)
-          (document as IWorkbenchDocument).BackendForm.close();
-        else
-          if (document is MySQL.Forms.AppViewDockContent)
-          {
-            // This type of document is already closed (in TabClosing).
-            // TODO: adjust code to support OnClosing/OnClose duality also in AppView.
-          }
+        if (document is MySQL.Forms.AppViewDockContent)
+        {
+          // This type of document is already closed (in TabClosing).
+          // TODO: adjust code to support OnClosing/OnClose duality also in AppView.
+        }
       e.page.Controls.Clear();
       e.page.Dispose();
       GC.Collect();
@@ -1752,6 +1742,32 @@ namespace MySQL.GUI.Workbench
       }
     }
 
+    /// <summary>
+    /// Determines if this document can be closed, i.e. all sub documents are queried for closing
+    /// as well. Returns false if this form cannot be closed.
+    /// </summary>
+    /// <returns></returns>
+    public bool CanCloseDocument()
+    {
+      foreach (ITabDocument document in bottomTabControl.Documents)
+        if (document is IWorkbenchDocument)
+          if (!(document as IWorkbenchDocument).CanCloseDocument())
+            return false;
+
+      if (!BackendForm.can_close())
+        return false;
+
+      return true;
+    }
+
+    public void CloseDocument()
+    {
+      foreach (ITabDocument document in bottomTabControl.Documents)
+        if (document is IWorkbenchDocument)
+          (document as IWorkbenchDocument).CloseDocument();
+      BackendForm.close();
+    }
+
     private void ShowUserDatatypes()
     {
       mainSplitContainer.Panel2Collapsed = false;
@@ -1851,7 +1867,7 @@ namespace MySQL.GUI.Workbench
     private void mainSplitContainer_SplitterMoving(object sender, SplitterCancelEventArgs e)
     {
       // This event only comes up when the splitter is moved with the mouse (i.e. by the user).
-      // We can use it to differentiate between user initiated and programatic splitter changes.
+      // We can use it to differentiate between user initiated and programmatic splitter changes.
       splitterMovePending = true;
     }
 
