@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,12 +18,12 @@
  */
 
 using System;
+using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 using System.Reflection;
 using System.Threading;
 using System.Windows.Forms;
-using System.Collections;
-using System.Collections.Generic;
 
 using Microsoft.Win32;
 
@@ -70,60 +70,30 @@ namespace MySQL.GUI.Workbench
     [STAThread]
     static void Main(string[] Args)
     {
+      // Connect the application to console to have proper output there if requested.
+      bool consoleRedirectionWorked = Win32Api.RedirectConsole();
+
       // Start with command line parsing.
-      Arguments CommandLine = new Arguments();
-
-      CommandLine.AddOptionName("-verbose", true);
-      CommandLine.AddOptionName("-v", true);
-      CommandLine.AddOptionName("-help", true);
-      CommandLine.AddOptionName("-h", true);
-      CommandLine.AddOptionName("-open", false);
-      CommandLine.AddOptionName("-nologo", true);
-      CommandLine.AddOptionName("-version", true);
-      CommandLine.AddOptionName("-grtversion", true);
-      CommandLine.AddOptionName("-swrendering", false);
-      CommandLine.AddOptionName("-log", true);
-      CommandLine.AddOptionName("-query", false);
-      CommandLine.AddOptionName("-model", false);
-      CommandLine.AddOptionName("-admin", false);
-      CommandLine.AddOptionName("-script", false);
-      CommandLine.AddOptionName("-run", false);
-      CommandLine.AddOptionName("-run-python", false);
-      CommandLine.AddOptionName("-run-lua", false);
-      CommandLine.AddOptionName("-quit-when-done", true);
-      CommandLine.AddOptionName("-log-level", false);
-
-      CommandLine.Parse(Args);
-
-      // Command line handling
-      bool exitApplication = ProcessCommandLineArguments(CommandLine);
-
-      // Initialize the logger. Enabled log levels are by default error, warning and info
-      // (+ debug1 and 2 in debug builds).
       string userDir = System.IO.Path.Combine(System.IO.Path.Combine(
         Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),
         "MySQL"), "Workbench");
       Logger.InitLogger(userDir);
-#if DEBUG
-      string logLevel = "debug2";
-#else
-      string logLevel = "info";
-#endif
-      string environmentLevel = Environment.GetEnvironmentVariable("WB_LOG_LEVEL");
-      if (environmentLevel != null)
-        logLevel = environmentLevel;
-      if (CommandLine["log-level"] != null)
-        logLevel = CommandLine["log-level"];
 
-      Logger.ActiveLevel = logLevel;
-      PrintInitialLogInfo();
+      if (!consoleRedirectionWorked)
+        Logger.LogError("Workbench", "Console redirection failed.\n");
 
-      if (exitApplication)
+      System.Reflection.Assembly asm = System.Reflection.Assembly.GetEntryAssembly();
+      string baseDir = System.IO.Path.GetDirectoryName(asm.Location);
+      WbOptions wbOptions = new WbOptions(baseDir, userDir);
+
+      if (!wbOptions.parse_args(Args, asm.Location))
       {
         Logger.LogInfo("Workbench", "Command line params told us to shut down.\n");
         return;
       }
 
+      PrintInitialLogInfo();
+
       Application.EnableVisualStyles();
       Application.SetCompatibleTextRenderingDefault(false);
 
@@ -136,7 +106,7 @@ namespace MySQL.GUI.Workbench
 
       // Read some early values which cannot be stored in the preferences (since they are loaded
       // later) from registry.
-      bool singleInstance = false;
+      bool singleInstance = true;
       string lastVersion = "";
       string currentVersion = GetApplicationMetaInfo(ApplicationMetaInfo.Version);
       Logger.LogInfo("Workbench", "Current version given by meta info is: " + currentVersion + '\n');
@@ -146,7 +116,8 @@ namespace MySQL.GUI.Workbench
         wbKey = wbKey.OpenSubKey(@"Software\Oracle\MySQL Workbench", false);
         if (wbKey != null)
         {
-          singleInstance = wbKey.GetValue("SingleInstance", 0).ToString() == "1";
+          if (wbKey.GetValue("DisableSingleInstance", 0).ToString() == "1")
+            singleInstance = false;
           lastVersion = wbKey.GetValue("LastStartedAs", "").ToString();
         }
         else
@@ -178,11 +149,9 @@ namespace MySQL.GUI.Workbench
       // Give the main thread a proper name, so we can later check for it when needed.
       Thread.CurrentThread.Name = "mainthread";
 
-      // Get the base directory of the executable and change the working dir to it
-      // TODO: the working dir should be untouched and the app should work
-      // from any basedir
-      System.Reflection.Assembly asm = System.Reflection.Assembly.GetEntryAssembly();
-      string baseDir = System.IO.Path.GetDirectoryName(asm.Location);
+      // Change the working dir to to application path.
+      // This is necessary because all our internal data files etc. are located under the app dir
+      // and WB could have been called from a different dir.
       string workdir = System.IO.Directory.GetCurrentDirectory();
       System.IO.Directory.SetCurrentDirectory(baseDir);
 
@@ -205,11 +174,12 @@ namespace MySQL.GUI.Workbench
 
       String []paths= cleanedPath.Split(new char[]{';'});
       cleanedPath = "";
+
       // Strip all python related dirs from PATH to avoid conflicts with other Python installations.
       foreach (String path in paths)
       {
         if (!path.ToLower().Contains("python"))
-          cleanedPath = cleanedPath+";"+path;
+          cleanedPath = cleanedPath + ";" + path;
       }
       Environment.SetEnvironmentVariable("PATH", systemFolder + cleanedPath);
       Logger.LogInfo("Workbench", "Setting PATH to: " + systemFolder + cleanedPath + '\n');
@@ -220,10 +190,7 @@ namespace MySQL.GUI.Workbench
       Environment.SetEnvironmentVariable("PYTHONHOME", workdir + "\\python");
 
       // Initialize forms stuff.
-      MySQL.Forms.Manager formsManager = new MySQL.Forms.Manager();
-
-      // Connect application to console to have proper output there if requested.
-      Win32.RedirectConsole();
+      MySQL.Forms.Manager formsManager = MySQL.Forms.Manager.get_instance(); // Creates the singleton.
 
       #region Runtime path check
 
@@ -274,7 +241,7 @@ namespace MySQL.GUI.Workbench
       #region Variables and Splashscreen
 
       // Show splash screen if not turned off
-      if (CommandLine["nologo"] == null)
+      if (wbOptions.ShowSplash)
         SplashScreen.Show(typeof(WorkbenchSplashScreen));
 
       #endregion
@@ -285,7 +252,7 @@ namespace MySQL.GUI.Workbench
       try
       {
         // Create Workbench Context
-        wbContext = new WbContext((CommandLine["verbose"] != null || CommandLine["v"] != null));
+        wbContext = new WbContext(wbOptions.Verbose);
 
         if (wbContext != null)
         {
@@ -300,12 +267,12 @@ namespace MySQL.GUI.Workbench
 
       #endregion
 
-      #region Initialize Callbacks and Mainform
-
       // If the Workbench Context and GRT Manager were successfully created, 
       // initialize the application
       if (wbContext != null && grtManager != null)
       {
+        #region Initialize Callbacks and Mainform
+
         mainForm = new MainForm(wbContext);
 
         // Initialize the Workbench context
@@ -326,92 +293,8 @@ namespace MySQL.GUI.Workbench
           new AppImpl.VoidBoolDelegate(mainForm.LockGUI),
           new AppImpl.VoidStrDelegate(mainForm.PerformCommand),
           new AppImpl.VoidDelegate(mainForm.QuitApplication),
-          new AppImpl.StringAppCommandAppViewStringDelegate(mainForm.ApplicationCommand));
-
-
-        string openAtStartupType = "";
-        string openAtStartup = null;
-
-        if (CommandLine["open"] != null)
-        {
-          openAtStartup = CommandLine["open"];
-          openAtStartupType = "";
-        }
-        else if (CommandLine["model"] != null)
-        {
-          openAtStartup = CommandLine["model"];
-          openAtStartupType = "model";
-        }
-        else if (CommandLine["query"] != null)
-        {
-          openAtStartup = CommandLine["query"];
-          openAtStartupType = "query";
-        }
-        else if (CommandLine["admin"] != null)
-        {
-          openAtStartup = CommandLine["admin"];
-          openAtStartupType = "admin";
-        }
-        else if (CommandLine["script"] != null)
-        {
-          openAtStartup = CommandLine["script"];
-          openAtStartupType = "script";
-        }
-
-        if (openAtStartup == null)
-        {
-          openAtStartup = "";
-          if (CommandLine.Leftovers.Count > 0)
-            openAtStartup = CommandLine.Leftovers[CommandLine.Leftovers.Count-1];
-        }
-        if (!string.IsNullOrEmpty(openAtStartup) && (openAtStartupType == "" || openAtStartupType == "model" || openAtStartupType == "script"))
-        {
-          if (!System.IO.Path.IsPathRooted(openAtStartup))
-            openAtStartup = System.IO.Path.Combine(workdir, openAtStartup);
-
-          // Since manually assigning an application to a file type does not automatically add
-          // our self defined keywords (like script) to the command line we better check what
-          // file type we got to open, based on the extension.
-          if (string.IsNullOrEmpty(openAtStartupType))
-          {
-            switch (Path.GetExtension(openAtStartup).ToLowerInvariant())
-            {
-              case ".sql":
-              case ".txt":
-              case ".bak":
-                openAtStartupType = "query";
-                break;
-            }
-          }
-        }
-
-        String runScript = "";
-        String runScriptLanguage = "";
-        if (CommandLine["run"] != null)
-          runScript = CommandLine["run"];
-        else if (CommandLine["run-python"] != null)
-        {
-          runScript = CommandLine["run-python"];
-          runScriptLanguage = "python";
-        }
-        else if (CommandLine["run-lua"] != null)
-        {
-          runScript = CommandLine["run-lua"];
-          runScriptLanguage = "lua";
-        }
-
-        WbOptions wbOptions = new WbOptions(baseDir, // basedir
-          baseDir, // plugin
-          "",       // struct
-          baseDir + "/modules", // modules
-          baseDir, // library
-          userDir,
-          openAtStartup,
-          openAtStartupType,
-          runScript, runScriptLanguage,
-          (CommandLine["swrendering"] != null),
-          (CommandLine["openglrendering"] != null),
-          (CommandLine["quit-when-done"] != null || CommandLine["quitwhendone"] != null));
+          new AppImpl.StringAppCommandAppViewStringDelegate(mainForm.ApplicationCommand),
+          mainForm.dockDelegate);
 
         // TODO: check return value and show error message.
         // Currently the return value is always true. In case of an error an exception is raised.
@@ -420,6 +303,9 @@ namespace MySQL.GUI.Workbench
           new WbContext.VoidStrUIFormDelegate(mainForm.CreateMainFormView)
         );
 
+        // command registration must be done after WBContext init
+        mainForm.RegisterCommands();
+
         // Set the Application.Idle event handler
         Application.Idle += new EventHandler(OnApplicationIdle);
 
@@ -433,8 +319,7 @@ namespace MySQL.GUI.Workbench
         grtManager.perform_idle_tasks();
         
         // Setup Menus
-        Logger.LogInfo("Workbench", "Setting up main UI\n");
-        mainForm.SetupUI();
+        wbContext.validate_edit_menu();
         mainForm.Show();
         Logger.LogInfo("Workbench", "UI is up\n");
 
@@ -474,8 +359,13 @@ namespace MySQL.GUI.Workbench
           }
         }
 
+        #endregion
+
         Logger.LogInfo("Workbench", "Shutting down Workbench\n");
 
+        timer.Stop();
+        timer.Dispose();
+
         // shutdown wb context
         if (wbContext != null)
         {
@@ -486,12 +376,12 @@ namespace MySQL.GUI.Workbench
           wbContext.Dispose();
         }
         formsApplication.Dispose();
+        formsManager.Dispose();
+
         GC.Collect();
       }
       
-      #endregion
-
-      Win32.ReleaseConsole();
+      Win32Api.ReleaseConsole();
 
       Logger.LogInfo("Workbench", "Done\n");
     }
@@ -551,14 +441,14 @@ namespace MySQL.GUI.Workbench
     }
 
     /// <summary>
-    /// Handler that gets called if this another instance of the application was started. It passes
+    /// Handler that gets called if another instance of the application was started. It passes
     /// the command line arguments from the other instance on so we can act accordingly here.
     /// </summary>
     /// <param name="sender">The sender.</param>
     /// <param name="args">The instance containing the event data.</param>
     private static void SingleInstanceCallback(object sender, InstanceCallbackEventArgs args)
     {
-      if (args == null || mainForm == null)
+      if (args == null || mainForm == null || !mainForm.IsHandleCreated)
         return;
 
       Action<bool> d = (bool x) =>
@@ -566,57 +456,10 @@ namespace MySQL.GUI.Workbench
         mainForm.Activate(x);
 
         // Parse command line and extract the model file name if there is one.
-        Arguments CommandLine = new Arguments();
-
-        CommandLine.AddOptionName("-verbose", true);
-        CommandLine.AddOptionName("-v", true);
-        CommandLine.AddOptionName("-help", true);
-        CommandLine.AddOptionName("-h", true);
-        CommandLine.AddOptionName("-open", false);
-        CommandLine.AddOptionName("-nologo", true);
-        CommandLine.AddOptionName("-version", true);
-        CommandLine.AddOptionName("-grtversion", true);
-        CommandLine.AddOptionName("-swrendering", false);
-        CommandLine.AddOptionName("-log", true);
-        CommandLine.AddOptionName("-query", false);
-        CommandLine.AddOptionName("-model", false);
-        CommandLine.AddOptionName("-admin", false);
-        CommandLine.AddOptionName("-script", false);
-        CommandLine.AddOptionName("-run", false);
-        CommandLine.AddOptionName("-run-python", false);
-        CommandLine.AddOptionName("-run-lua", false);
-        CommandLine.AddOptionName("-quit-when-done", true);
-
-        CommandLine.Parse(args.CommandLineArgs);
-
-        string fileType = "";
-        string fileToOpen = null;
-
-        if (CommandLine["open"] != null)
-        {
-          fileToOpen = CommandLine["open"];
-          fileType = "";
-        }
-        else if (CommandLine["model"] != null)
-        {
-          fileToOpen = CommandLine["model"];
-          fileType = "model";
-        }
-
-        if (fileToOpen == null)
-        {
-          fileToOpen = "";
-          if (CommandLine.Leftovers.Count > 0)
-            fileToOpen = CommandLine.Leftovers[CommandLine.Leftovers.Count - 1];
-        }
-        if (fileToOpen != null && fileToOpen != "" && (fileType == "" || fileType == "model"))
-        {
-          if (!System.IO.Path.IsPathRooted(fileToOpen))
-            fileToOpen = System.IO.Path.Combine(System.IO.Directory.GetCurrentDirectory(), fileToOpen);
-        }
-
-        if (!string.IsNullOrEmpty(fileToOpen) && !wbContext.get_filename().Equals(fileToOpen))
-          wbContext.open_document(fileToOpen);
+        WbOptions options = new WbOptions("", "");
+        System.Reflection.Assembly asm = System.Reflection.Assembly.GetEntryAssembly();
+        options.parse_args(args.CommandLineArgs, asm.Location);
+        wbContext.finished_loading(options);
       };
       mainForm.Invoke(d, true);
     }
@@ -706,70 +549,6 @@ namespace MySQL.GUI.Workbench
       return value;
     }
 
-    /// <summary>
-    /// Deals with the given command line arguments
-    /// </summary>
-    /// <param name="CommandLine">The command line arguments passed to the application</param>
-    /// <returns></returns>
-    static bool ProcessCommandLineArguments(Arguments CommandLine)
-    {
-      if (CommandLine["help"] != null || CommandLine["h"] != null)
-      {
-        Console.WriteLine("MySQL Workbench {0} {1}. " 
-          + "(C) {2} by {3}. All rights reserved." + Environment.NewLine,
-          GetApplicationMetaInfo(ApplicationMetaInfo.Version), 
-          GetApplicationMetaInfo(ApplicationMetaInfo.Configuration),
-          GetApplicationMetaInfo(ApplicationMetaInfo.Copyright),
-          GetApplicationMetaInfo(ApplicationMetaInfo.Company));
-        Console.WriteLine("Usage: MySQLWorkbench [options] [model file]" + Environment.NewLine);
-        CommandLine.AddOptionName("-query", false);
-        CommandLine.AddOptionName("-model", false);
-        CommandLine.AddOptionName("-admin", false);
-        CommandLine.AddOptionName("-script", false);
-        CommandLine.AddOptionName("-run", false);
-        CommandLine.AddOptionName("-run-python", false);
-        CommandLine.AddOptionName("-run-lua", false);
-        CommandLine.AddOptionName("-quit-when-done", true);
-
-        Console.WriteLine("Options" + Environment.NewLine
-          + "  -admin instance .... Open an admin tab to the named server instance at startup" + Environment.NewLine
-          + "  -open filename ..... Open the given filename at startup" + Environment.NewLine
-          + "  -query server ...... Open a DB query tab to the named server connection at startup" + Environment.NewLine
-          + "  -run script ........ Executes the given Workbench script at startup"  + Environment.NewLine
-          + "  -run-python script . Executes the given Workbench Python script at startup"  + Environment.NewLine
-          + "  -run-lua script .... Executes the given Workbench Lua script at startup"  + Environment.NewLine
-          + "  -script scriptfile . Executes the given Workbench script file at startup"  + Environment.NewLine
-          + "  -quit-when-done .... Quits Workbench once the given script finishes executing"  + Environment.NewLine
-          + "  -swrendering ....... Force the canvas to use software rendering instead of OpenGL" + Environment.NewLine
-          + "  -nologo ............ Do not display the splash screen" + Environment.NewLine
-          + "  -log-level ......... Determines the amount of logging (error, warning, info, debug1/2/3)" + Environment.NewLine
-          + "  -verbose (-v) ...... Print verbose output in the GRT Shell" + Environment.NewLine
-          + "  -version ........... Print the version information" + Environment.NewLine
-          + "  -grtversion ........ Print the GRT version information" + Environment.NewLine
-          + "  -help (-h) ......... Print this output" + Environment.NewLine
-        );
-
-        return true;
-      }
-      else
-        if (CommandLine["version"] != null)
-        {
-          Console.WriteLine("MySQL Workbench {0}.{1} {2}",
-            GetApplicationMetaInfo(ApplicationMetaInfo.Version), 
-            GetApplicationMetaInfo(ApplicationMetaInfo.Revision),
-            GetApplicationMetaInfo(ApplicationMetaInfo.Configuration));
-          return true;
-        }
-        else
-          if (CommandLine["grtversion"] != null)
-          {
-            Console.WriteLine("GRT Environment {0}", GRT.version());
-            return true;
-          }
-
-      return false;
-    }
-
     static public bool CheckForNewVersion()
     {
       try
@@ -796,12 +575,17 @@ namespace MySQL.GUI.Workbench
       String message;
       String info;
 
+      bool isFontProblem = false;
       if (e != null)
-      { // Report System.Exception info
+      {
+        // Report System.Exception info
         message = e.Message;
+
         info = "Exception = " + e.GetType() + "\n";
         info += "Message = " + e.Message + "\n";
         info += "FullText = " + e.ToString();
+
+        isFontProblem = (e is ArgumentException) && message.Contains("'Regular'") && info.Contains(".CreateNativeFont");
       }
       else
       { // Report exception Object info
@@ -826,7 +610,21 @@ namespace MySQL.GUI.Workbench
         Win32Api.UnblockWorkbenchFiles(System.IO.Directory.GetCurrentDirectory());
       }
       else
-        ExceptionDialog.Show(message, info, wbContext);
+      {
+        // There's a relatively common error with missing 'Regular' style even on system
+        // standard fonts, which is extremely weird, but not caused by WB. However, we show
+        // a hint and close WB, so the user knows he can fix it and doesn't enter new bug reports for this.
+        if (isFontProblem)
+        {
+          MessageBox.Show("There was a problem with one of the system fonts, which indicates this font" +
+          "is corrupt. The original message is:\n" + message + "\nMySQL Workbench needs to close now. You " +
+          "should re-install the font in question to avoid this problem in the future.",
+          "Invalid Font Reported", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
+          Application.Exit();
+        }
+        else
+          ExceptionDialog.Show(message, info, wbContext);
+      }
     }
 
     /// <summary>
