@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,6 +19,8 @@
 
 #include "base/string_utilities.h"
 #include "base/file_functions.h"
+#include "base/log.h"
+
 #include <stdexcept>
 #include <functional>
 #include <locale>
@@ -29,6 +31,8 @@
 #include <boost/function.hpp>
 #include <boost/bind.hpp>
 
+DEFAULT_LOG_DOMAIN(DOMAIN_BASE);
+
 using namespace std;
 
 namespace base
@@ -194,12 +198,13 @@ string trim(const string& s, const string& t)
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Simple case conversion routine, which returns a new string. Works well only for ANSI characters.
+ * Simple case conversion routine, which returns a new string.
  */
 string tolower(const string& s)
 {
-  std::string result = s;
-  std::transform(result.begin(), result.end(), result.begin(), ::tolower);
+  char *str_down= g_utf8_strdown(s.c_str(), g_utf8_strlen(s.c_str(), -1));
+  std::string result(str_down);
+  g_free(str_down);
   return result;
 }
 
@@ -207,8 +212,9 @@ string tolower(const string& s)
 
 string toupper(const string& s)
 {
-  std::string result = s;
-  std::transform(result.begin(), result.end(), result.begin(), ::toupper);
+  char *str_up= g_utf8_strup(s.c_str(), g_utf8_strlen(s.c_str(), -1));
+  std::string result(str_up);
+  g_free(str_up);
   return result;
 }
 
@@ -232,6 +238,16 @@ std::string truncate_text(const std::string& s, int max_length)
   
 //--------------------------------------------------------------------------------------------------
 
+std::string sanitize_utf8(const std::string& s)
+{
+  const char *end = 0;
+  if (!g_utf8_validate(s.data(), s.size(), &end))
+    return std::string(s.data(), end);
+  return s;
+}
+  
+//--------------------------------------------------------------------------------------------------
+
 std::vector<std::string> split(const std::string &s, const std::string &sep, int count)
 {
   std::vector<std::string> parts;
@@ -259,6 +275,150 @@ std::vector<std::string> split(const std::string &s, const std::string &sep, int
   return parts;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+std::vector<std::string> split_by_set(const std::string &s, const std::string &separator_set, int count)
+{
+  std::vector<std::string> parts;
+  std::string ss= s;
+
+  std::string::size_type p;
+
+  if (s.empty())
+    return parts;
+
+  if (count == 0)
+    count= -1;
+
+  p= ss.find_first_of(separator_set);
+  while (!ss.empty() && p != std::string::npos && (count < 0 || count > 0))
+  {
+    parts.push_back(ss.substr(0, p));
+    ss = ss.substr(p + 1);
+
+    --count;
+    p = ss.find_first_of(separator_set);
+  }
+  parts.push_back(ss);
+
+  return parts;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+std::vector<std::string> split_token_list(const std::string &s, int sep)
+{
+  std::vector<std::string> parts;
+  std::string ss= s;
+  
+  std::string::size_type end = s.size(), pe, p = 0;
+  
+  {
+    bool done;
+    bool empty_pending = true;
+
+    while (p < end)
+    {
+      empty_pending = false;
+      switch (s[p])
+      {
+        case '\'':
+          pe = p+1;
+          done = false;
+          // keep going until we find closing '
+          while (pe < end && !done)
+          {
+            switch (s[pe++])
+            {
+              case '\'':
+                if (pe < end && s[pe] == '\'')
+                  pe++;
+                else
+                  done = true;
+                break;
+              case '\\':
+                if (pe < end)
+                  pe++;
+                break;
+            }
+          }
+          parts.push_back(s.substr(p, pe-p));
+          p = pe;
+          // skip whitespace
+          while (p < end && (s[p] == ' ' || s[p] == '\t' || s[p] == '\r' || s[p] == '\n')) p++;
+          if (p < end)
+          {
+            if (s[p] != sep)
+              log_debug("Error splitting string list");
+            else
+              p++;
+          }
+          break;
+
+        case '"':
+          pe = p+1;
+          done = false;
+          // keep going until we find closing "
+          while (pe < end && !done)
+          {
+            switch (s[pe++])
+            {
+              case '"':
+                if (pe < end && s[pe] == '"')
+                  pe++;
+                else
+                  done = true;
+                break;
+              case '\\':
+                if (pe < end)
+                  pe++;
+                break;
+            }
+          }
+          parts.push_back(s.substr(p, pe-p));
+          p = pe;
+          // skip whitespace
+          while (p < end && (s[p] == ' ' || s[p] == '\t' || s[p] == '\r' || s[p] == '\n')) p++;
+          if (p < end)
+          {
+            if (s[p] != sep)
+              log_debug("Error splitting string list");
+            else
+              p++;
+          }
+          break;
+
+        case ' ':
+        case '\t':
+          p++;
+          break;
+
+        default:
+          // skip until separator
+          pe = p;
+          while (pe < end)
+          {
+            if (s[pe] == sep)
+            {
+              empty_pending = true;
+              break;
+            }
+            pe++;
+          }
+          parts.push_back(trim_right(s.substr(p, pe-p)));
+          p = pe+1;
+          // skip whitespace
+          while (p < end && (s[p] == ' ' || s[p] == '\t' || s[p] == '\r' || s[p] == '\n')) p++;
+          break;
+      }
+    }
+    if (empty_pending)
+      parts.push_back("");
+  }
+
+  return parts;
+}
+  
 //--------------------------------------------------------------------------------------------------
   
 bool partition(const std::string &s, const std::string &sep, std::string &left, std::string &right)
@@ -276,7 +436,18 @@ bool partition(const std::string &s, const std::string &sep, std::string &left,
 }
   
 //--------------------------------------------------------------------------------------------------
-  
+
+/**
+ * Returns the index of the given string in the given vector or -1 if not found.
+ */
+int index_of(std::vector<std::string> list, const std::string &s)
+{
+  std::vector<string>::iterator location = std::find(list.begin(), list.end(), s);
+  if (location == list.end())
+    return -1;
+  return location - list.begin();
+}
+
 /**
  * Returns a string containing all characters beginning at "start" from the given string "id", which form
  * a valid, unqualified identifier. The returned identifier does not contain any quoting anymore.
@@ -589,6 +760,25 @@ std::string expand_tilde(const std::string &path)
   
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Checks the input for characters not allowed in the file system and converts them to underscore.
+ */
+std::string make_valid_filename(const std::string &name)
+{
+  std::string result;
+  std::string illegal_chars = "\\/:?\"<>|*";
+  for (std::string::const_iterator iterator = name.begin(); iterator != name.end(); ++iterator)
+  {
+    if (illegal_chars.find(*iterator) != string::npos)
+      result += '_';
+    else
+      result += *iterator;
+  }
+  return result;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Tests if t begins with part.
  */
@@ -693,10 +883,12 @@ std::string get_text_file_contents(const std::string &filename)
 
 //--------------------------------------------------------------------------------------------------
 
-/** Escape a string to be used in a SQL query
+/**
+ * Escape a string to be used in a SQL query
  * Same code as used by mysql. Handles null bytes in the middle of the string.
+ * If wildcards is true then _ and % are masked as well.
  */
-std::string escape_sql_string(const std::string &s)
+std::string escape_sql_string(const std::string &s, bool wildcards)
 {
   std::string result;
   result.reserve(s.size());
@@ -728,6 +920,14 @@ std::string escape_sql_string(const std::string &s)
       case '\032':                        /* This gives problems on Win32 */
         escape= 'Z';
         break;
+      case '_':
+        if (wildcards)
+          escape = '_';
+        break;
+      case '%':
+        if (wildcards)
+          escape = '%';
+        break;
     }
     if (escape)
     {
@@ -740,8 +940,46 @@ std::string escape_sql_string(const std::string &s)
   return result;
 }
 
+
+std::string unescape_sql_string(const std::string &s)
+{
+  std::string result;
+  result.reserve(s.size());
+  
+  for (std::string::const_iterator ch= s.begin(); ch != s.end(); ++ch)
+  {
+    int out = *ch;
+    if (out == '\'' && (ch+1) != s.end() && *(ch+1) == '\'')
+      continue; // skip ' in ''
+    else if (out == '\\')
+    {
+      if ((ch+1) != s.end())
+      {
+        ++ch;
+        switch (*ch)
+        {
+        case 'n': out = '\n'; break;
+        case 't': out = '\t'; break;
+        case 'r': out = '\r'; break;
+        case 'b': out = '\b'; break;
+        case '0': out = 0; break;         // Ascii null
+        case 'Z': out = '\032'; break;         // Win32 end of file
+     //   case 'N': found_null=1;
+        /* fall through */
+        default: out = *ch; break;
+        }
+      }
+    }
+    result.push_back(out);
+  }
+  return result;
+}
+
+
+
 //--------------------------------------------------------------------------------------------------
 
+// NOTE: This is not the same as escape_sql_string, as embedded ` are escaped as ``, not \`
 std::string escape_backticks(const std::string &s)
 {
   std::string result;
@@ -775,7 +1013,8 @@ std::string escape_backticks(const std::string &s)
         escape= 'Z';
         break;
       case '`':
-        escape= '`';
+        // special case
+        result.push_back('`');
         break;        
     }
     if (escape)
@@ -860,13 +1099,13 @@ bool parse_font_description(const std::string &fontspec, std::string &font, int
   
   for (int i= 0; i < 2 && !parts.empty(); i++)
   {
-    if (g_strcasecmp(parts.back().c_str(), "bold")==0)
+    if (g_ascii_strcasecmp(parts.back().c_str(), "bold")==0)
     {
       bold = true;
       parts.pop_back();
     }
     
-    if (g_strcasecmp(parts.back().c_str(), "italic")==0)
+    if (g_ascii_strcasecmp(parts.back().c_str(), "italic")==0)
     {
       italic = true;
       parts.pop_back();
@@ -882,8 +1121,57 @@ bool parse_font_description(const std::string &fontspec, std::string &font, int
   return true;  
 }
 
+//--------------------------------------------------------------------------------------------------
+
+std::string unquote_identifier(const std::string& identifier)
+{
+  int start = 0;
+  int size = identifier.size();
+
+  if (identifier[0] == '"' || identifier[0] == '`')
+    start++;
+
+  if (identifier[size-1] == '"' || identifier[size-1] == '`')
+    size--;
+
+  size-=start;
+
+  return identifier.substr(start, size);
+}
+
 //--------------------------------------------------------------------------------------------------
   
+std::string quote_identifier(const std::string& identifier, const char quote_char)
+{
+  return quote_char + identifier + quote_char;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Quotes the given identifier, but only if it needs to be quoted.
+ * http://dev.mysql.com/doc/refman/5.1/en/identifiers.html specifies what is allowed in unquoted identifiers.
+ * Leading numbers are not strictly forbidden but discouraged as they may lead to ambiguous behavior.
+ */
+std::string quote_identifier_if_needed(const std::string &ident, const char quote_char)
+{
+  bool needs_quotation= false;
+  for (std::string::const_iterator i= ident.begin(); i != ident.end(); ++i)
+  {
+    if ((*i >= 'a' && *i <= 'z') || (*i >= 'A' && *i <= 'Z') || (*i >= '0' && *i <= '9')
+      || (*i == '_') || (*i == '$') || ((unsigned char)(*i) > 0x7F))
+      continue;
+    needs_quotation = true;
+    break;
+  }
+  if (needs_quotation)
+    return quote_char + ident + quote_char;
+  else
+    return ident;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 EolHelpers::Eol_format EolHelpers::detect(const std::string &text)
 {
   std::string::size_type pos= text.find_first_of("\r\n");
