@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,9 +40,10 @@
 
 #include "mdc_line.h"
 
+#include "base/file_utilities.h"
+
 using namespace mdc;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
+using namespace base;
 
 // XXX: use the values defined by the platform!
 #define DOUBLE_CLICK_TIME 0.5
@@ -112,8 +113,6 @@ static CanvasView *destroyed= 0;
 
 CanvasView::~CanvasView()
 {
-  g_static_rec_mutex_free(&_lock);
-
   destroyed= this;
 
   delete _blayer;
@@ -136,6 +135,7 @@ CanvasView::~CanvasView()
 
   if (_crsurface)
     cairo_surface_destroy(_crsurface);
+  g_static_rec_mutex_free(&_lock);
 }
 
 
@@ -942,7 +942,7 @@ void CanvasView::export_png(const std::string &filename, bool crop)
 {
   CanvasAutoLock lock(this);
 
-  FileHandle fh(filename.c_str(), "wb");
+  base::FileHandle fh(filename.c_str(), "wb");
   Size vsize= get_total_view_size();
   
   Rect bounds= get_content_bounds();
@@ -1615,11 +1615,12 @@ bool CanvasView::handle_key(const KeyInfo &key, bool press, EventState state)
 }
 
 
-void *CanvasView::last_click_item_destroyed(void *data)
+void *CanvasView::canvas_item_destroyed(void *data)
 {
   CanvasView *view= (CanvasView*)data;
 
-  view->_last_click_item= 0;
+  view->_last_click_item = NULL;
+  view->_last_over_item = NULL;
 
   return 0;
 }
@@ -1629,36 +1630,30 @@ void CanvasView::set_last_click_item(CanvasItem *item)
 {
   if (_last_click_item != item)
   {
-    if (_last_click_item)
+    // Remove the notification callback for this item if it isn't also stored as hot item
+    // (otherwise we still need the notification).
+    if (_last_click_item && _last_click_item != _last_over_item)
       _last_click_item->remove_destroy_notify_callback(this);
 
     _last_click_item= item;
-    if (_last_click_item)
-      _last_click_item->add_destroy_notify_callback(this, last_click_item_destroyed);
+
+    // Register the notification callback if not yet done (i.e. for the hot item tracking).
+    if (_last_click_item && _last_click_item != _last_over_item)
+      _last_click_item->add_destroy_notify_callback(this, canvas_item_destroyed);
   }
 }
 
 
-void *CanvasView::last_over_item_destroyed(void *data)
-{
-  CanvasView *view= (CanvasView*)data;
-
-  view->_last_over_item= 0;
-
-  return 0;
-}
-
-
 void CanvasView::set_last_over_item(CanvasItem *item)
 {
   if (_last_over_item != item)
   {
-    if (_last_over_item)
+    if (_last_over_item && _last_over_item != _last_click_item)
       _last_over_item->remove_destroy_notify_callback(this);
 
     _last_over_item= item;
-    if (_last_over_item)
-      _last_over_item->add_destroy_notify_callback(this, last_over_item_destroyed);
+    if (_last_over_item && _last_over_item != _last_click_item)
+      _last_over_item->add_destroy_notify_callback(this, canvas_item_destroyed);
   }
 }
 
