@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,14 +28,14 @@
 #include "mdc_group.h"
 
 using namespace mdc;
-using namespace MySQL::Geometry;
+using namespace base;
 
 Selection::Selection(CanvasView *view)
 : _view(view)
 {
   _block_signals= 0;
 
-//  _view->get_interaction_layer()->signal_custom_repaint().connect(sigc::mem_fun(this, &Selection::render_drag_images));
+//  _view->get_interaction_layer()->signal_custom_repaint().connect(boost::bind(&Selection::render_drag_images, this));
 
   g_static_rec_mutex_init(&_mutex);
 }
@@ -140,7 +140,7 @@ void Selection::add(CanvasItem *item)
   }
   unlock();
   if (notify)
-    _signal_changed.emit(true, item);
+    _signal_changed(true, item);
 }
 
 
@@ -161,7 +161,7 @@ void Selection::remove(CanvasItem *item)
   unlock();
 
   if (notify)
-    _signal_changed.emit(false, item);
+    _signal_changed(false, item);
 }
 
 
@@ -240,7 +240,7 @@ void Selection::remove_items_outside(const Rect &rect)
 
 void Selection::begin_moving(const Point &mouse_pos)
 {
-  _signal_begin_drag.emit();
+  _signal_begin_drag();
 
   lock();
   for (ContentType::iterator it= _items.begin();
@@ -322,7 +322,7 @@ bool Selection::is_moving()
 
 void Selection::end_moving()
 {
-  _signal_end_drag.emit();
+  _signal_end_drag();
 
   lock();  
   //for (std::list<CanvasItem*>::const_iterator i= _items.begin(); i!= _items.end(); ++i)
@@ -388,6 +388,6 @@ void Selection::clear(bool keep_move_info)
   unlock();
 
   if (!was_empty)
-    _signal_changed.emit(false, 0);
+    _signal_changed(false, 0);
 }
 
