@@ -1,9 +1,27 @@
+/* 
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
 
-#include "stdafx.h"
+#include "tut_stdafx.h"
 
 #include "connection_helpers.h"
 
-#define VERBOSE_TESTING 0
+//XXX these tests are useless without calls to ensure
 
 using namespace std;
 
@@ -37,19 +55,18 @@ TEST_FUNCTION(2)
 
   while (rset->next())
   {
-#if VERBOSE_TESTING
-    std::cout << rset->getString("schema_name") << std::endl;
-    std::cout << "  Schema Objects:" << std::endl;
-
-#endif
+    if (!getenv("QUIET"))
+    {
+      std::cout << rset->getString("Database") << std::endl;
+      std::cout << "  Schema Objects:" << std::endl;
+    }
     
     std::auto_ptr<sql::ResultSet> rset2(meta->getSchemaObjects("", rset->getString("database")));
     while (rset2->next())
     {
-#if VERBOSE_TESTING
-      std::cout << rset2->getString("object_type") << ": " << rset2->getString("name") << "," << 
-        rset2->getString("ddl") << std::endl;
-#endif
+      if (!getenv("QUIET"))
+        std::cout << rset2->getString("object_type") << ": " << rset2->getString("name") << "," << 
+          rset2->getString("ddl") << std::endl;
     }
   }
 }
@@ -71,53 +88,49 @@ TEST_FUNCTION(3)
 
   connection->setAutoCommit(0);
 
-#if VERBOSE_TESTING
-  std::cout << "Insert Data." << std:endl;
-#endif
+  if (!getenv("QUIET"))
+    std::cout << "Insert Data." << std::endl;
 
   std::auto_ptr<sql::PreparedStatement> prepStmt(connection->prepareStatement("INSERT INTO test.product(idproduct, name)  VALUES(?, ?)"));
   prepStmt->setInt(1, 1);
   prepStmt->setString(2, "Harry Potter");
   prepStmt->executeUpdate();
 
-#if VERBOSE_TESTING
-  std::cout << "Display Data." << std::endl;
-#endif
+  if (!getenv("QUIET"))
+    std::cout << "Display Data." << std::endl;
 
   std::auto_ptr<sql::ResultSet> rset1(stmt->executeQuery("SELECT * FROM test.product"));
 
   int i = 0;
   while (rset1->next())
   {
-#if VERBOSE_TESTING
-    std::cout << rset1->getString(2) << ", " << rset1->getString("name") << std::endl;
-#endif
+    if (!getenv("QUIET"))
+      std::cout << rset1->getString(2) << ", " << rset1->getString("name") << std::endl;
     i++;
   }
-#if VERBOSE_TESTING
-  printf("%d row(s)", i);
+  if (!getenv("QUIET"))
+  {
+    printf("%d row(s)", i);
 
-  printf("Rollback");
-#endif
+    printf("Rollback");
+  }
 
   connection->rollback();
 
-#if VERBOSE_TESTING
-  printf("Display Data Again.\n");
-#endif
+  if (!getenv("QUIET"))
+    printf("Display Data Again.\n");
+
   std::auto_ptr<sql::ResultSet> rset2(stmt->executeQuery("SELECT * FROM test.product"));
 
   i = 0;
   while (rset2->next())
   {
-#if VERBOSE_TESTING
-    std::cout << rset2->getString(2) << ", " << rset2->getString("name") << std::endl;
-#endif
+    if (!getenv("QUIET"))
+      std::cout << rset2->getString(2) << ", " << rset2->getString("name") << std::endl;
     i++;
   }
-#if VERBOSE_TESTING
-  std::cout << i << " row(s)" << std::endl;
-#endif
+  if (!getenv("QUIET"))
+    std::cout << i << " row(s)" << std::endl;
 }
 
 END_TESTS
