@@ -1,5 +1,23 @@
+/* 
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
 
-#include "stdafx.h"
+#include "tut_stdafx.h"
 
 #include "connection_helpers.h"
 #include "grtsqlparser/sql_facade.h"
@@ -48,7 +66,7 @@ TEST_MODULE(module_dbc_statement_test, "DBC: statement tests");
 
 TEST_FUNCTION(1)
 {
-
+  sql_splitter = NULL;
   std::string path = "../../Bin/Debug";
   grtm.set_search_paths(path, path, path);
   grtm.initialize(path);
@@ -645,6 +663,7 @@ TEST_FUNCTION(16)
 			"CREATE TABLE dbc_statement_test_15.table3 (id int);"
 			"SELECT 1;";
 		std::list<std::string> statements;
+        ensure("failed to get sqlparser module", (NULL != sql_splitter));
 		sql_splitter->splitSqlScript(sql_script, statements);
 		sql::SqlBatchExec()(stmt.get(), statements);
 
