@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -177,7 +177,7 @@ enum
     
     subframe.origin.x+= NSWidth(subframe);
     if (expand && !fill)
-      subframe.origin.x+ extraSpace;
+      subframe.origin.x+= extraSpace;
   }
 }
 
@@ -293,8 +293,18 @@ enum
 }
 
 
+- (void)subviewMinimumSizeChanged
+{
+  if (!mOwner->is_destroying())
+    [super subviewMinimumSizeChanged];
+}
+
+
 - (NSSize)minimumSize
 {
+  if (mOwner == NULL || mOwner->is_destroying())
+    return NSZeroSize;
+  
   NSSize size;
   float maxSize= 0;
   float frameWidth= NSWidth([self frame]);
@@ -306,7 +316,7 @@ enum
   for (id subview in [self subviews])
   {
     NSSize minSize;
-    if (![subview isHidden])
+    if (![subview isHidden] && (([subview tag] & RemovingFlag) == 0))
     {
       visibleSubviews++;
       
