@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,19 +20,25 @@
 
 #import "MFView.h"
 #include "mforms/view.h"
+#include "mforms/label.h"
 
 @interface MFContainerBase : NSView {
   NSInteger mTag;
-  float mPadding;
+  float mLeftPadding;
+  float mTopPadding;
+  float mBottomPadding;
+  float mRightPadding;
   int mFreezeRelayout;
   NSColor* mBackColor;
   NSColor* mDefaultBackColor;
+  NSImage* mBackImage;
+  mforms::Alignment mBackImageAlignment;
 }
 
-- (void)setPadding:(int)padding;
+- (void)setPaddingLeft:(float)lpad right:(float)rpad top:(float)tpad bottom:(float)bpad;
 - (void)setTag:(NSInteger)tag;
 - (void)subviewMinimumSizeChanged;
 - (void)setFreezeRelayout:(BOOL)flag;
 - (void)setBackgroundColor: (NSColor*) color;
-- (void)setBackgroundImage: (NSString*) path withLayout: (mforms::ImageLayout) layout;
+- (void)setBackgroundImage: (NSString*) path withAlignment: (mforms::Alignment) align;
 @end
