@@ -1,13 +1,25 @@
-//
-//  cf_form.mm
-//  MySQLWorkbench
-//
-//  Created by Alfredo Kojima on 21/Jan/09.
-//  Copyright 2009 Sun Microsystems Inc. All rights reserved.
-//
+/* 
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
 
+#import "MFMForms.h"
 #import "MFForm.h"
-#include <mforms/mforms.h>
+#import "MFDrawBox.h"
 
 @implementation MFFormImpl
 
@@ -24,7 +36,14 @@
     [self setHidesOnDeactivate: NO];
     [self setReleasedWhenClosed: NO];
 
-    mParentWindow = ownerWindow;
+    if (ownerWindow)
+    {
+      id owner = ownerWindow->get_data();
+      if ([owner isKindOfClass: [NSWindowController class]])
+        mParentWindow = [owner window];
+      else
+        mParentWindow = owner;
+    }
     
     mOwner= form;
     mOwner->set_data(self);
@@ -101,8 +120,8 @@
   // automatically set parent to key window status
   if (mParentWindow)
   {
-    if ([mParentWindow->get_data() isVisible])
-      [mParentWindow->get_data() makeKeyWindow];
+    if ([mParentWindow isVisible])
+      [mParentWindow makeKeyWindow];
   }    
 }
 
@@ -127,7 +146,7 @@
 
   if (mParentWindow)
   {
-    NSRect rect = [mParentWindow->get_data() frame];
+    NSRect rect = [mParentWindow frame];
     NSSize size = [self frame].size;
     NSPoint pos;
     
@@ -156,6 +175,11 @@
 
 - (void)setFixedFrameSize:(NSSize)size
 {
+  NSSize curSize = [self frame].size;
+  if (size.width < 0)
+    size.width = curSize.width;
+  if (size.height < 0)
+    size.height = curSize.height;
   [self setFrameSize: size];
 }
 
