@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -80,7 +80,13 @@
 
 //--------------------------------------------------------------------------------------------------
 
-@interface ComboStyleSelector : NSComboBox {
+@interface ComboStyleSelector :
+#ifdef MAC_OS_X_VERSION_10_6
+    NSComboBox < NSComboBoxDelegate >
+#else
+    NSComboBox
+#endif
+{
   mforms::Selector *mOwner;
 }
 
