@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
  */
 
 #import "MFTabView.h"
-#include <mforms/mforms.h>
+#import "MFMForms.h"
 
 @implementation MFTabViewItemView
 
@@ -63,28 +63,49 @@
 
 - (id)initWithObject:(::mforms::TabView*)aTabView tabType:(mforms::TabViewType)tabType
 {
-  self= [super initWithFrame:NSMakeRect(10, 10, 100, 50)];
+  self= [super initWithFrame:NSMakeRect(10, 10, 100, 100)];
   if (self)
-  { 
+  {     
+    mTabView = [[[NSTabView alloc] initWithFrame: NSMakeRect(0, 0, 100, 100)] autorelease];
+
     switch (tabType)
     {
       case mforms::TabViewTabless:
-        [self setTabViewType: NSNoTabsNoBorder];
+        [mTabView setTabViewType: NSNoTabsNoBorder];
         break;
       case mforms::TabViewMainClosable:
         //[self doCustomize];
         break;
       case mforms::TabViewPalette:
-        [self setControlSize: NSSmallControlSize];
-        [self setFont: [NSFont systemFontOfSize: [NSFont smallSystemFontSize]]];
+        [mTabView setControlSize: NSSmallControlSize];
+        [mTabView setFont: [NSFont systemFontOfSize: [NSFont smallSystemFontSize]]];
+        break;
+        
+      case mforms::TabViewSelectorSecondary:
+        [mTabView setTabViewType: NSNoTabsNoBorder];
+        mTabSwitcher = [[MTabSwitcher alloc] initWithFrame: NSMakeRect(0, 0, 100, 26)];
+        [mTabSwitcher setTabStyle: MPaletteTabSwitcher];
+        [self addSubview: mTabSwitcher];
+        [mTabSwitcher setTabView: mTabView];
+        break;
+      default:
         break;
     }
+    [mTabView setDrawsBackground: NO];
+    [self addSubview: mTabView];
 
-    mExtraSize = [super minimumSize];
-
+    mExtraSize = [mTabView minimumSize];
+    {
+      NSRect contentRect = [mTabView contentRect];
+      mExtraSize.width -= NSWidth(contentRect);
+      mExtraSize.height -= NSHeight(contentRect);
+    }
     mOwner= aTabView;
     mOwner->set_data(self);
-    [self setDelegate:self];
+    if (mTabSwitcher)
+      [mTabSwitcher setDelegate: self];
+    else
+      [mTabView setDelegate:self];
   }
   return self;
 }
@@ -95,11 +116,20 @@
 }
 
 
+- (NSTabView*)tabView
+{
+  return mTabView;
+}
+
+
 - (NSSize)minimumSize
 {
+  if (mOwner == NULL || mOwner->is_destroying())
+    return NSZeroSize;
+  
   NSSize minSize= NSZeroSize;
   
-  for (NSTabViewItem *item in [self tabViewItems])
+  for (NSTabViewItem *item in [mTabView tabViewItems])
   {
     NSSize size= [[item view] minimumSize];
     
@@ -116,14 +146,28 @@
 // necessary or rebuilding the UI won't work (test case: connection editor)
 - (void)resizeSubviewsWithOldSize:(NSSize)oldBoundsSize
 {
-  for (NSTabViewItem *item in [self tabViewItems])
+  [super resizeSubviewsWithOldSize: oldBoundsSize];
+  if (mTabSwitcher)
+  {
+    NSRect srect = [mTabSwitcher frame];
+    NSRect rect = [self bounds];
+
+    srect.size.width = NSWidth(rect);
+    srect.origin.y = NSHeight(rect) - NSHeight(srect);
+    [mTabSwitcher setFrame: srect];
+
+    rect.size.height -= NSHeight(srect);
+    [mTabView setFrame: rect];
+  }
+  else
+    [mTabView setFrame: [self bounds]];
+  for (NSTabViewItem *item in [mTabView tabViewItems])
     [[item view] resizeSubviewsWithOldSize: oldBoundsSize];
 }
 
-
 - (void)setEnabled:(BOOL)flag
 {
-  for (NSTabViewItem *item in [self tabViewItems])
+  for (NSTabViewItem *item in [mTabView tabViewItems])
     [[item view] setEnabled: flag];
 }
 
@@ -132,7 +176,6 @@
   mTag= tag;
 }
 
-
 - (NSInteger)tag
 {
   return mTag;
@@ -161,7 +204,7 @@ static void tabview_set_active_tab(::mforms::TabView *self, int tab)
     
     if ( tabView )
     {
-      [tabView selectTabViewItem: [tabView tabViewItemAtIndex:tab]];
+      [tabView->mTabView selectTabViewItem: [tabView->mTabView tabViewItemAtIndex:tab]];
     }
   }
 }
@@ -175,7 +218,7 @@ static void tabview_set_tab_title(::mforms::TabView *self, int tab, const std::s
     
     if ( tabView )
     {
-      [[tabView tabViewItemAtIndex:tab] setLabel: wrap_nsstring(title)];
+      [[tabView->mTabView tabViewItemAtIndex:tab] setLabel: wrap_nsstring(title)];
     }
   }
 }
@@ -189,7 +232,7 @@ static int tabview_get_active_tab(::mforms::TabView *self)
     
     if ( tabView )
     {
-      return [tabView indexOfTabViewItem: [tabView selectedTabViewItem]];
+      return [tabView->mTabView indexOfTabViewItem: [tabView->mTabView selectedTabViewItem]];
     }
   }
   return 0;
@@ -207,16 +250,16 @@ static int tabview_add_page(::mforms::TabView *self, mforms::View *tab, const st
       NSTabViewItem *item= [[[NSTabViewItem alloc] initWithIdentifier: [NSString stringWithFormat:@"%p", tab]] autorelease];
       MFTabViewItemView *view= [[MFTabViewItemView alloc] init];
       
-      view->mTabView= tabView;
+      view->mTabView= tabView->mTabView;
       
       [item setLabel: wrap_nsstring(label)];
       [item setView: view];
       
       [view addSubview: tab->get_data()];
       
-      [tabView addTabViewItem: item];
+      [tabView->mTabView addTabViewItem: item];
       
-      return [tabView numberOfTabViewItems]-1;
+      return [tabView->mTabView numberOfTabViewItems]-1;
     }
   }
   return -1;
@@ -231,17 +274,17 @@ static void tabview_remove_page(::mforms::TabView *self, mforms::View *tab)
     
     if (tabView)
     {
-      int i= [tabView indexOfTabViewItemWithIdentifier: [NSString stringWithFormat:@"%p", tab]];
+      int i= [tabView->mTabView indexOfTabViewItemWithIdentifier: [NSString stringWithFormat:@"%p", tab]];
       if (i != NSNotFound)
       {
-        NSTabViewItem *item= [tabView tabViewItemAtIndex: i];
+        NSTabViewItem *item= [tabView->mTabView tabViewItemAtIndex: i];
         if (item)
         {
           MFTabViewItemView *view= [item view];
           [[[view subviews] lastObject] removeFromSuperview];
           [view release];
           
-          [tabView removeTabViewItem: item];
+          [tabView->mTabView removeTabViewItem: item];
         }      
       }
     }
