@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,7 +32,11 @@
 
 @end
 
-@interface MFTextBoxImpl : NSScrollView {
+@interface MFTextBoxImpl : NSScrollView
+#ifdef MAC_OS_X_VERSION_10_6
+    <NSTextViewDelegate>
+#endif
+{
   NSTextView *mContentView;
   mforms::TextBox *mOwner;
   NSInteger mTag;
