@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -44,6 +44,46 @@ static const float EMPTY_TOOLBAR_HEIGHT = 6;
 using namespace mforms;
 
 
+@interface MFColorMenuItem : NSMenuItem
+{
+}
+@end
+
+@implementation MFColorMenuItem
+
+static NSColor* colorFromHexString(const char* hexcolor)
+{
+  int r, g, b;
+  
+  if (sscanf(hexcolor, "#%02x%02x%02x", &r, &g, &b) != 3)
+    return nil;
+  
+  return [NSColor colorWithDeviceRed:r / 255.0 green:g / 255.0 blue:b / 255.0 alpha: 1.0];
+}
+
+- (id)initWithColorName: (NSString*)color
+{
+  self = [super init];
+  if (self)
+  {
+    NSImage *image= [[NSImage alloc] initWithSize: NSMakeSize(24, 16)];
+    [image lockFocus];
+    [[NSColor lightGrayColor] set];
+    NSFrameRect(NSMakeRect(1, 1, 22, 14));
+    [colorFromHexString([color UTF8String]) set];
+    NSRectFill(NSMakeRect(2, 2, 20, 12));
+    [image unlockFocus];
+    [self setImage: image];
+    [self setTitle: @""];
+    [self setRepresentedObject: color];
+    [image release];
+  }
+  return self;
+}
+
+@end
+
+
 
 @interface MFToolBarSeparatorImpl : NSView
 {
@@ -109,7 +149,7 @@ using namespace mforms;
       NSBezierPath *vdotted;
       
       CGFloat pattern[] = {1.0, 2.0};
-      vdotted = [[NSBezierPath bezierPath] retain];
+      vdotted = [NSBezierPath bezierPath];
       [vdotted setLineWidth: 1.0];
       [vdotted setLineDash: pattern count: 2 phase: 0.0];
       
@@ -224,6 +264,16 @@ using namespace mforms;
   }
 }
 
+- (void)setStringValue: (NSString*)value
+{
+  [self setTitle: value];
+}
+
+- (NSString*)stringValue
+{
+  return [self title];
+}
+
 @end
 
 //--------------------------------------------------------------------------------------------------
@@ -264,6 +314,8 @@ using namespace mforms;
     case OptionsToolBar:
       [self setFont: [NSFont systemFontOfSize: [NSFont smallSystemFontSize]]];
       break;
+    default:
+      break;
   }
 }
 
@@ -388,6 +440,8 @@ using namespace mforms;
     case OptionsToolBar:
       [self setBordered: NO];
       break;
+    default:
+      break;
   }
 }
 
@@ -519,7 +573,7 @@ using namespace mforms;
     wasEmpty = YES;
 
   NSView *view = item;
-  if (index >= [[self subviews] count])
+  if (index >= (int)[[self subviews] count])
     [self addSubview: view];
   else
     [self addSubview: view positioned: NSWindowBelow relativeTo: [[self subviews] objectAtIndex: index]];
@@ -833,19 +887,13 @@ static void set_item_alt_icon(ToolBarItem *item, const std::string &image)
 static void set_item_text(ToolBarItem *item, const std::string &text)
 {
   id tbitem = item->get_data();
-  if ([tbitem isKindOfClass: [NSButton class]])
-    [tbitem setTitle: wrap_nsstring(text)];
-  else
-    [tbitem setStringValue: wrap_nsstring(text)];
+  [tbitem setStringValue: wrap_nsstring(text)];
 }
 
 static std::string get_item_text(ToolBarItem *item)
 {
   id tbitem = item->get_data();
-  if ([tbitem isKindOfClass: [NSButton class]])
-    return [[tbitem title] UTF8String];
-  else    
-    return [[tbitem stringValue] UTF8String] ?: "";
+  return [[tbitem stringValue] UTF8String] ?: "";
 }
 
 static void set_item_enabled(ToolBarItem *item, bool flag)
@@ -880,17 +928,6 @@ static void set_item_tooltip(ToolBarItem *item, const std::string &text)
 }
 
 
-static NSColor* colorFromHexString(std::string hexcolor)
-{
-  int r, g, b;
-  
-  if (sscanf(hexcolor.c_str(), "#%02x%02x%02x", &r, &g, &b) != 3)
-    return nil;
-  
-  return [NSColor colorWithDeviceRed:r / 255.0 green:g / 255.0 blue:b / 255.0 alpha: 1.0];
-}
-
-
 static void set_selector_items(ToolBarItem *item, const std::vector<std::string> &items)
 {
   id tbitem = item->get_data();
@@ -902,19 +939,7 @@ static void set_selector_items(ToolBarItem *item, const std::vector<std::string>
       [[tbitem cell] setControlSize: NSSmallControlSize];
       for (std::vector<std::string>::const_iterator color= items.begin(); color != items.end(); ++color)
       {
-        NSMenuItem *item= [[[NSMenuItem alloc] init] autorelease];
-        NSImage *image= [[NSImage alloc] initWithSize: NSMakeSize(24, 16)];
-        [image lockFocus];
-        [[NSColor lightGrayColor] set];
-        NSFrameRect(NSMakeRect(1, 1, 22, 14));
-        [colorFromHexString(*color) set];
-        NSRectFill(NSMakeRect(2, 2, 20, 12));
-        [image unlockFocus];
-        [item setImage: image];
-        [item setTitle: @""];
-        [item setRepresentedObject: wrap_nsstring(*color)];
-        [image release];
-        [menu addItem: item];
+        [menu addItem: [[[MFColorMenuItem alloc] initWithColorName: wrap_nsstring(*color)] autorelease]];
       }
       [tbitem setMenu: menu];
       [tbitem sizeToFit];
