@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,13 +17,16 @@
  * 02110-1301  USA
  */
 
+#include "../lf_mforms.h"
 #include "../lf_label.h"
 
 mforms::gtk::LabelImpl::LabelImpl(::mforms::Label *self)
   : ViewImpl(self)
 {
+  _style = mforms::NormalStyle;
   _label = Gtk::manage(new Gtk::Label(""));
   _label->set_alignment(0, 0.5);
+  _label->set_use_underline(false);
 
   setup();    
   _label->signal_expose_event().connect(sigc::bind(sigc::ptr_fun(mforms::gtk::expose_event_slot), _label), false);
