@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,13 +20,22 @@
 #ifndef _MFORMS_MENU_H_
 #define _MFORMS_MENU_H_
 
+#include <map>
+#include "boost/signals2.hpp"
+
 #include "base/trackable.h"
 #include "mforms/base.h"
 #include "base/ui_form.h"
-#include <map>
 
 namespace mforms {
   class Menu;
+}
+
+#ifdef _WIN32
+  #pragma make_public(mforms::Menu) // To allow using Menu in interface functions in assemblies.
+#endif
+
+namespace mforms {
  
 #ifndef DOXYGEN_SHOULD_SKIP_THIS 
 #ifndef SWIG
@@ -53,10 +62,12 @@ namespace mforms {
     MenuImplPtrs *_menu_impl;
     boost::function<void (const std::string&)> _action_handler;
     boost::signals2::signal<void ()> _on_will_show;
+    boost::signals2::signal<void (const std::string&)> _on_action;
     std::map<const std::string, int> _item_map;
   public:
     Menu();
 
+    bool empty() const;
     void clear();
     void remove_item(int i);
     int add_item(const std::string &caption, const std::string &action);
@@ -76,6 +87,7 @@ namespace mforms {
     int get_item_index(const std::string &action);
     
     boost::signals2::signal<void ()>* signal_will_show() { return &_on_will_show; }
+    boost::signals2::signal<void (const std::string&)>* signal_on_action() { return &_on_action; }
   };
 };
 
