@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +17,9 @@
  * 02110-1301  USA
  */
  
-#include "stdafx.h"
-#include "../stub_utilities.h"
+#include "tut_stdafx.h"
 
+#include "../stub_utilities.h"
 
 namespace mforms {
 namespace stub {
@@ -31,7 +31,9 @@ int __stdcall UtilitiesImpl::show_message(const std::string &title, const std::s
                         const std::string &ok, const std::string &cancel,
                         const std::string &other)
 {
-  return 0;
+  if (other == "Don't Save")
+      return mforms::ResultOther;
+  return mforms::ResultOk;
 }
 
 
@@ -69,7 +71,7 @@ bool __stdcall UtilitiesImpl::hide_wait_message()
 }
 
 bool __stdcall UtilitiesImpl::run_cancelable_wait_message(const std::string &title, const std::string &text,
-                      const sigc::slot<bool> &cancel_task)
+                      const boost::function<void ()> &start_task, const boost::function<bool ()> &cancel_task)
 {
   return true;
 }
@@ -98,25 +100,64 @@ std::string __stdcall UtilitiesImpl::get_special_folder(mforms::FolderType type)
   return "./";
 }
 
-void __stdcall UtilitiesImpl::add_timeout(float interval, const sigc::slot<bool> &slot)
+void __stdcall UtilitiesImpl::add_timeout(float interval, const boost::function<bool ()> &slot)
 {
 }
 
 void __stdcall UtilitiesImpl::store_password(const std::string &service, const std::string &account, const std::string &password)
 {
-  passwords[account] = password;
+  passwords[service+":"+account] = password;
 }
 
 //------------------------------------------------------------------------------
 bool __stdcall UtilitiesImpl::find_password(const std::string &service, const std::string &account, std::string &password)
 {
+  static bool loaded_passwords = false;
   bool ret_val = false;
-  if (passwords.count(account))
+
+  if (!loaded_passwords)
   {
-    password = passwords[account];
-    ret_val = true;
+    char *password_file = getenv("TEST_PASSWORD_FILE");
+    if (!password_file)
+    {
+      g_message("Specify a password file for tests with the TEST_PASSWORD_FILE env variable.");
+    }
+    else
+    {
+      FILE *f = fopen(password_file, "r");
+      if (f)
+      {
+        if (!getenv("QUIET"))
+          g_message("Loading %s", password_file);
+        char line[1024];
+        while ((fgets(line, sizeof(line), f)))
+        {
+          char *tmp = strrchr(line, '=');
+          if (tmp)
+          {
+            char *e = strchr(tmp, '\r');
+            if (e) *e = 0;
+            e = strchr(tmp, '\n');
+            if (e) *e = 0;
+            passwords[std::string(line, tmp-line)] = tmp+1;
+            if (!getenv("QUIET"))
+              g_message("%s=%s", std::string(line, tmp-line).c_str(), tmp+1);
+          }
+        }
+      }
+    }
+    loaded_passwords = true;
   }
 
+  if (passwords.count(service+":"+account))
+  {
+    password = passwords[service+":"+account];
+    ret_val = true;
+  }
+  else
+  {
+    g_message("Unknown password for %s : %s", service.c_str(), account.c_str());
+  }
   return ret_val;
 }
 
@@ -139,6 +180,11 @@ static const char* gnome_keyring_results[Gnome_keyring_results_size] = {"OK",
                                              "GNOME_KEYRING_RESULT_ALREADY_EXISTS",
                                              ""
                                             };
+//------------------------------------------------------------------------------
+void* __stdcall UtilitiesImpl::perform_from_main_thread(const boost::function<void* ()>& slot, bool wait)
+{
+  return slot();
+};
 
 //------------------------------------------------------------------------------
 void UtilitiesImpl::init()
@@ -162,9 +208,10 @@ void UtilitiesImpl::init()
   f->_utilities_impl.show_message_with_checkbox= &UtilitiesImpl::show_message_with_checkbox;
   f->_utilities_impl.show_wait_message= &UtilitiesImpl::show_wait_message;
   f->_utilities_impl.stop_cancelable_wait_message= &UtilitiesImpl::stop_cancelable_wait_message;
+  f->_utilities_impl.perform_from_main_thread= &UtilitiesImpl::perform_from_main_thread;
 }
 
-void UtilitiesImpl::set_open_url_slot(const sigc::slot<void,std::string> &slot)
+void UtilitiesImpl::set_open_url_slot(const boost::function<void (std::string)> &slot)
 {
 }
 
