@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,8 +23,9 @@
 
 #include "stdafx.h"
 
+#include "mforms/mforms.h"
+
 using namespace mforms;
-using namespace MySQL;
 
 //--------------------------------------------------------------------------------------------------
 
@@ -40,26 +41,11 @@ View::View()
 
 View::~View()
 {
-  App::get()->view_destroyed(this);
-  
   set_destroying();
-  if (_parent)
-    _parent->remove_subview(this);
+  if (_parent && !_parent->is_destroying())
+    _parent->remove(this);
   
-  std::list<std::pair<View*,bool> >::iterator iter;
-  
-  while ((iter = _subviews.begin()) != _subviews.end())
-  {
-    bool managed = iter->second;
-    View *sv = iter->first;
-    sv->_parent = NULL;
-    
-    _subviews.erase(iter);
-    if (managed)
-    {
-      sv->release();
-    }
-  }
+  clear_subviews();
 
   // Let the frontend delete all resources it allocated.
   if (_view_impl->destroy)
@@ -68,7 +54,19 @@ View::~View()
 
 //--------------------------------------------------------------------------------------------------
 
-// XXX: no longer needed, objects can be queried if they are managed.
+void View::clear_subviews()
+{
+  std::list<std::pair<View*,bool> >::iterator iter;
+
+  while ((iter = _subviews.begin()) != _subviews.end())
+  {
+    View *sv = iter->first;
+    remove(sv); // Let descendants adjust their child lists. This will also release the object if necessary.
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void View::set_managed()
 {
   Object::set_managed();
@@ -87,26 +85,32 @@ void View::set_managed()
 
 //--------------------------------------------------------------------------------------------------
 
-// XXX: add and remove should retain and release. Note: some subclasses already do this on their own.
-// This needs to be consolidated here.
 void View::add_subview(View *sv)
 {
+  if (sv->get_parent() != NULL)
+    throw std::logic_error("mforms: attempt to add a subview already contained somewhere");
+
+  if (sv == this)
+    throw std::logic_error("mforms: Can't add a view inside itself");
+
+  sv->set_parent(this);
+  if (!sv->_release_on_add) // Means: don't increase the ref count, the caller retained already, but won't release.
+    sv->retain();
   _subviews.push_back(std::make_pair(sv, sv->_managed));
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void View::remove_subview(View *sv)
+void View::remove(View *sv)
 {
-  if (!is_destroying())
+  sv->_parent = NULL;
+  for (std::list<std::pair<View*,bool> > ::iterator iter= _subviews.begin(); iter != _subviews.end(); ++iter)
   {
-    for (std::list<std::pair<View*,bool> > ::iterator iter= _subviews.begin(); iter != _subviews.end(); ++iter)
+    if (iter->first == sv)
     {
-      if (iter->first == sv)
-      {
-        _subviews.erase(iter);
-        break;
-      }
+      _subviews.erase(iter);
+      sv->release();
+      break;
     }
   }
 }
@@ -163,7 +167,7 @@ View *View::get_subview_at_index(int index)
  */
 bool View::contains_subview(View* subview)
 {
-  return get_subview_index(subview) >= 0;
+  return subview->get_parent() == this;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -411,9 +415,9 @@ std::string View::get_back_color()
 
 //--------------------------------------------------------------------------------------------------
 
-void View::set_back_image(const std::string &path, ImageLayout layout)
+void View::set_back_image(const std::string &path, Alignment align)
 {
-  _view_impl->set_back_image(this, path, layout);
+  _view_impl->set_back_image(this, path, align);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -439,3 +443,11 @@ void View::flush_events()
 
 //--------------------------------------------------------------------------------------------------
 
+void View::focus()
+{
+  if (_view_impl->focus)//TODO Windows
+    _view_impl->focus(this);
+  else
+    g_warning("mforms::View::focus not implemented");
+}
+
