@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,7 +18,6 @@
  */
 
 #include "stdafx.h"
-#include "wf_box.h"
 
 using namespace MySQL;
 using namespace MySQL::Forms;
@@ -63,7 +62,7 @@ void maximize_children(ControlList% list, bool horizontal, Size containerSize, i
       entry->bounds.X= padding;
 
       // Determine the preferred size based on the new width (for labels, as they can be in auto wrap mode).
-      if (entry->control->GetType()->IsSubclassOf(Label::typeid))
+      if (entry->control->GetType()->IsSubclassOf(Windows::Forms::Label::typeid))
         entry->bounds.Size= entry->control->GetPreferredSize(Size(containerSize.Width, 0));
       else
         entry->bounds.Width= containerSize.Width;
@@ -274,11 +273,23 @@ System::Drawing::Size HorizontalGtkBoxLayout::ComputeLayout(Box^ box, System::Dr
     entry->control= control;
     entry->fills= box->GetControlFills(control);
 
-    entry->bounds= System::Drawing::Rectangle(Point(0, 0), control->GetPreferredSize(Size(0, proposedSize.Height)));
-    if (ViewImpl::use_min_width_for_layout(control))
-      entry->bounds.Width= control->MinimumSize.Width;
-    if (ViewImpl::use_min_height_for_layout(control))
-      entry->bounds.Height= control->MinimumSize.Height;
+    bool use_min_width = ViewImpl::use_min_width_for_layout(control);
+    bool use_min_height = ViewImpl::use_min_height_for_layout(control);
+    if (use_min_width && use_min_height)
+      entry->bounds.Size = control->MinimumSize;
+    else
+    {
+      entry->bounds= System::Drawing::Rectangle(Point(0, 0), control->GetPreferredSize(Size(0, proposedSize.Height)));
+      if (use_min_width)
+        entry->bounds.Width= control->MinimumSize.Width;
+      if (use_min_height)
+        entry->bounds.Height= control->MinimumSize.Height;
+    }
+
+    // Weird behavior of combo boxes. They report a one pixel too small height (you cannot set them
+    // to this smaller height, though). So this wrong value is messing up our computed overall height.
+    if (entry->control->GetType() == ComboBox::typeid)
+      entry->bounds.Height++;
 
     // Sort control into the proper alignment list.
     if (control->Dock == DockStyle::Right)
@@ -481,11 +492,24 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
 
     // Keep track of the widest control, so we can adjust the container's width properly.
     ViewImpl::set_full_auto_resize(control);
-    entry->bounds= System::Drawing::Rectangle(Point(0, 0), control->GetPreferredSize(System::Drawing::Size(proposedSize.Width, 0)));
-    if (ViewImpl::use_min_width_for_layout(control))
-      entry->bounds.Width= control->MinimumSize.Width;
-    if (ViewImpl::use_min_height_for_layout(control))
-      entry->bounds.Height= control->MinimumSize.Height;
+
+    bool use_min_width = ViewImpl::use_min_width_for_layout(control);
+    bool use_min_height = ViewImpl::use_min_height_for_layout(control);
+    if (use_min_width && use_min_height)
+      entry->bounds.Size = control->MinimumSize;
+    else
+    {
+      entry->bounds= System::Drawing::Rectangle(Point(0, 0), control->GetPreferredSize(System::Drawing::Size(proposedSize.Width, 0)));
+      if (use_min_width)
+        entry->bounds.Width= control->MinimumSize.Width;
+      if (use_min_height)
+        entry->bounds.Height= control->MinimumSize.Height;
+    }
+
+    // Weird behavior of combo boxes. They report a one pixel too small height (you cannot set them
+    // to this smaller height, though). So this wrong value is messing up our computed overall height.
+    if (entry->control->GetType() == ComboBox::typeid)
+      entry->bounds.Height++;
 
     if (max_width < entry->bounds.Width)
       max_width= entry->bounds.Width;
@@ -609,6 +633,8 @@ System::Drawing::Size VerticalGtkBoxLayout::ComputeLayout(Box^ box, System::Draw
 bool VerticalGtkBoxLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
 {
   Box^ box= (Box^) container;
+  if (!box->IsHandleCreated)
+    return false;
 
   ControlUtilities::SuspendDrawing(box);
 
@@ -715,19 +741,6 @@ void BoxImpl::set_spacing(mforms::Box *self, int space)
 
 //-------------------------------------------------------------------------------------------------
 
-void BoxImpl::set_padding(mforms::Box *self, int padding)
-{
-  BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (box != nullptr)
-  {
-    box->set_padding(padding);
-    self->set_layout_dirty(true);
-  }
-}
-
-//-------------------------------------------------------------------------------------------------
-
 void BoxImpl::set_homogeneous(mforms::Box *self, bool value)
 {
   BoxImpl^ box= (BoxImpl^)ObjectImpl::FromUnmanaged(self);
@@ -791,11 +804,3 @@ void BoxImpl::set_spacing(int space)
 }
 
 //-------------------------------------------------------------------------------------------------
-
-void BoxImpl::set_padding(int padding)
-{
-  get_control<Box>()->Padding= Padding(padding);
-}
-
-//-------------------------------------------------------------------------------------------------
-
