@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,11 +17,7 @@
  * 02110-1301  USA
  */
 
-
 #include "stdafx.h"
-#include "wf_drawbox.h"
-
-#include <cairo/cairo-win32.h>
 
 using namespace MySQL;
 using namespace MySQL::Forms;
@@ -122,6 +118,7 @@ bool DrawBoxImpl::create(mforms::DrawBox *self)
     canvas->MouseDoubleClick += gcnew MouseEventHandler(drawbox, &DrawBoxImpl::drawbox_mouse_double_click);
     canvas->MouseEnter += gcnew EventHandler(drawbox, &DrawBoxImpl::drawbox_mouse_enter);
     canvas->MouseLeave += gcnew EventHandler(drawbox, &DrawBoxImpl::drawbox_mouse_leave);
+
     return true;
   }
   return false;
@@ -132,6 +129,7 @@ bool DrawBoxImpl::create(mforms::DrawBox *self)
 void DrawBoxImpl::drawbox_mouse_down(System::Object ^sender, MouseEventArgs ^e)
 {
   CanvasControl^ canvas= (CanvasControl^) sender;
+  canvas->Focus();
 
   if (canvas->Tag != nullptr)
   {
