@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,14 +17,8 @@
  * 02110-1301  USA
  */
 
-/**
- * Contains the implementation of the wrapper for a .NET form used by the mforms backend.
- */
-
 #include "stdafx.h"
 
-#include "wf_form.h"
-
 using namespace System;
 using namespace System::IO;
 using namespace Drawing;
@@ -146,7 +140,7 @@ bool FormImpl::create(::mforms::Form *self, ::mforms::Form *owner, ::mforms::For
 
   if (form != nullptr)
   {
-    Windows::Forms::Form ^f= ViewImpl::create<FillForm>(self, form);
+    Windows::Forms::Form ^f = ViewImpl::create<FillForm>(self, form);
     if (owner != NULL)
       f->StartPosition = FormStartPosition::CenterParent;
     else
@@ -178,7 +172,9 @@ bool FormImpl::create(::mforms::Form *self, ::mforms::Form *owner, ::mforms::For
 
     f->MinimizeBox= (flag & mforms::FormMinimizable) != 0;
     f->MaximizeBox= (flag & mforms::FormMinimizable) != 0;
-    f->TopMost= (flag & mforms::FormStayOnTop) != 0;
+    //f->TopMost=
+    if ((flag & mforms::FormStayOnTop) != 0)
+      f->Owner = UtilitiesImpl::get_mainform();
 
     form->_hide_on_close= (flag & mforms::FormHideOnClose) != 0;
     ViewImpl::remove_auto_resize(f, mforms::ResizeBoth);
@@ -195,7 +191,7 @@ void FormImpl::set_title(::mforms::Form *self, const std::string &title)
   FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);
 
   if (form != nullptr)
-    form->get_control<Form>()->Text= gcnew String(title.c_str());
+    form->get_control<Windows::Forms::Form>()->Text= gcnew String(title.c_str());
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -206,20 +202,21 @@ void FormImpl::show_modal(::mforms::Form *self, ::mforms::Button *accept,::mform
 
   if (form != nullptr)
   {
-    Form ^f= form->get_control<Form>();
+    Windows::Forms::Form ^f= form->get_control<Windows::Forms::Form>();
 
     if (accept)
     {
-      f->AcceptButton= ((ViewImpl^)ObjectImpl::FromUnmanaged(accept))->get_control<Button>();
-      f->AcceptButton->DialogResult= DialogResult::OK;
+      f->AcceptButton= ((ViewImpl^)ObjectImpl::FromUnmanaged(accept))->get_control<Windows::Forms::Button>();
+      f->AcceptButton->DialogResult = Windows::Forms::DialogResult::OK;
     }
     if (cancel)
     {
-      f->CancelButton= ((ViewImpl^)ObjectImpl::FromUnmanaged(cancel))->get_control<Button>();
-      f->CancelButton->DialogResult= DialogResult::Cancel;
+      f->CancelButton= ((ViewImpl^)ObjectImpl::FromUnmanaged(cancel))->get_control<Windows::Forms::Button>();
+      f->CancelButton->DialogResult = Windows::Forms::DialogResult::Cancel;
     }
     // Make the window top most (so it stays above all others), but non-blocking.
-    f->TopMost= true;
+    //f->TopMost= true;
+    f->Owner = UtilitiesImpl::get_mainform();
     if (f->InvokeRequired)
       f->BeginInvoke(gcnew MethodInvoker(f, &Control::Show));
     else
@@ -235,34 +232,42 @@ bool FormImpl::run_modal(::mforms::Form *self, ::mforms::Button *accept,::mforms
 
   if (form != nullptr)
   {
-    Form ^f= form->get_control<Form>();
-
+    Windows::Forms::Form ^f= form->get_control<Windows::Forms::Form>();
+    Windows::Forms::Button^ acceptButton = accept == NULL ? nullptr :
+      ((ViewImpl^)ObjectImpl::FromUnmanaged(accept))->get_control<Windows::Forms::Button>();
     if (accept)
     {
-      f->AcceptButton= ((ViewImpl^)ObjectImpl::FromUnmanaged(accept))->get_control<Button>();
-      f->AcceptButton->DialogResult= DialogResult::OK;
+      f->AcceptButton = acceptButton;
+      f->AcceptButton->DialogResult = Windows::Forms::DialogResult::OK;
     }
     if (cancel)
     {
-      f->CancelButton= ((ViewImpl^)ObjectImpl::FromUnmanaged(cancel))->get_control<Button>();
-      f->CancelButton->DialogResult= DialogResult::Cancel;
+      f->CancelButton= ((ViewImpl^)ObjectImpl::FromUnmanaged(cancel))->get_control<Windows::Forms::Button>();
+      f->CancelButton->DialogResult = Windows::Forms::DialogResult::Cancel;
     }
 
-    mforms::Utilities::enter_modal_loop();
-
-    DialogResult dialog_result;
+    Windows::Forms::DialogResult dialog_result;
     if (f->InvokeRequired)
     {
-      Object^ invocation_result= f->Invoke(gcnew FillForm::ShowModalDelegate(f, &Form::ShowDialog),
-        gcnew array<Object^> {form->parent});
-      dialog_result= *(DialogResult^) (invocation_result);
+      Object^ invocation_result= f->Invoke(gcnew FillForm::ShowModalDelegate(f, &Windows::Forms::Form::ShowDialog),
+        gcnew array<Object^> {form->_owner});
+      dialog_result= *(Windows::Forms::DialogResult^) (invocation_result);
     }
     else
-      dialog_result= f->ShowDialog(form->parent);
+    {
+      // If there is currently no active form then we are being called while the application
+      // is not active. Focus the accept button if possible in that case to avoid having any
+      // text field with keyboard focus, which might appear as if they were active (but are not in fact).
+      Windows::Forms::Form^ active_form = Windows::Forms::Form::ActiveForm;
+      if (active_form == nullptr && acceptButton != nullptr)
+        f->ActiveControl = acceptButton;
 
-    bool result = (dialog_result == DialogResult::OK);
+      mforms::Utilities::enter_modal_loop();
+      dialog_result= f->ShowDialog(form->_owner);
+      mforms::Utilities::leave_modal_loop();
+    }
 
-    mforms::Utilities::leave_modal_loop();
+    bool result = (dialog_result == Windows::Forms::DialogResult::OK);
 
     f->Hide();
 
@@ -278,8 +283,8 @@ void FormImpl::end_modal(::mforms::Form *self, bool result)
   FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);
   if (form != nullptr)
   {
-    Form ^f= form->get_control<Form>();
-    f->DialogResult = result ? DialogResult::OK : DialogResult::Cancel;
+    Windows::Forms::Form ^f= form->get_control<Windows::Forms::Form>();
+    f->DialogResult = result ? Windows::Forms::DialogResult::OK : Windows::Forms::DialogResult::Cancel;
   }
 }
 
@@ -293,7 +298,7 @@ void FormImpl::close(::mforms::Form *self)
   FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);
 
   if (form != nullptr)
-    form->get_control<Form>()->Close();
+    form->get_control<Windows::Forms::Form>()->Close();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -306,7 +311,7 @@ void FormImpl::set_content(::mforms::Form *self, ::mforms::View *view)
   {
     ViewImpl^ child= (ViewImpl^)ObjectImpl::FromUnmanaged(view);
     Control^ ctl= child->get_control<Control>();
-    form->get_control<Form>()->Controls->Add(ctl);
+    form->get_control<Windows::Forms::Form>()->Controls->Add(ctl);
   }
 }
 
@@ -338,15 +343,15 @@ FormImpl::FormImpl(::mforms::Form *form, ::mforms::Form *owner, ::mforms::FormFl
   {
     // Meant is the window parent here, not the real owner.
     if (owner == mforms::Form::main_form())
-      parent = Application::OpenForms[0];
+      _owner = Application::OpenForms[0];
     else
     {
       FormImpl^ parent_form= (FormImpl^)ObjectImpl::FromUnmanaged(owner);
-      parent = parent_form->get_control<Form>();
+      _owner = parent_form->get_control<Windows::Forms::Form>();
     }
   }
   else
-    parent = nullptr;
+    _owner = nullptr;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -357,7 +362,7 @@ FormImpl::FormImpl(::mforms::Form *form, ::mforms::Form *owner, ::mforms::FormFl
  */
 void FormImpl::center()
 {
-  Form^ form= get_control<Form>();
+  Windows::Forms::Form^ form= get_control<Windows::Forms::Form>();
   form->StartPosition= FormStartPosition::CenterParent;
 }
 
