@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,8 +22,6 @@
  */
 
 #include "stdafx.h"
-#include "wf_grttreeview.h"
-#include "wf_menu.h"
 
 using namespace System::Drawing;
 using namespace System::Windows::Forms;
@@ -152,7 +150,7 @@ bool GRTTreeViewImpl::create(GRTTreeView *self, TreeOptions options)
     TreeViewAdv^ t= ViewImpl::create<TreeViewAdv>(self, treeview);
     t->LoadOnDemand= true;
     t->AutoRowHeight= true;
-    t->AsyncExpanding = true;
+    t->AsyncExpanding = false;
     t->BackColor = SystemColors::Window;
     t->FullRowSelect = true;
     t->GridColor = SystemColors::Control;
@@ -219,14 +217,16 @@ void GRTTreeViewImpl::refresh(mforms::GRTTreeView* self, const bec::NodeId &node
   GRTTreeViewImpl^ treeview= (GRTTreeViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (treeview != nullptr)
   {
-    TreeViewAdv^ t= treeview->get_control<TreeViewAdv>();
-    for each (TreeNodeAdv^ current_node in t->AllNodes)
-      if ((*((GrtTreeNode^) current_node->Tag)->NodeId->get_unmanaged_object()) == node)
-      {
-        treeview->_model->RefreshModel(t->GetPath(current_node));
-        return;
-      }
-
+    if (node.is_valid())
+    {
+      TreeViewAdv^ t= treeview->get_control<TreeViewAdv>();
+      for each (TreeNodeAdv^ current_node in t->AllNodes)
+        if ((*((GrtTreeNode^) current_node->Tag)->NodeId->get_unmanaged_object()) == node)
+        {
+          treeview->_model->RefreshModel(t->GetPath(current_node));
+          return;
+        }
+    }
     if (treeview->_model != nullptr)
       treeview->_model->RefreshModel();
   }
@@ -274,8 +274,9 @@ int GRTTreeViewImpl::get_selection(mforms::GRTTreeView *self, std::vector<bec::N
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Sets the given node to the specified expand state. The entire path to that node
- * is collapsed/expanded as well.
+ * Sets the given node to the specified expand state. 
+ * When expanding, the entire path to that node is expanded
+ * When collapsing, only the target node is collapsed
  */
 void GRTTreeViewImpl::set_expanded(mforms::GRTTreeView* self, const bec::NodeId& node, bool expanded)
 {
@@ -294,12 +295,29 @@ void GRTTreeViewImpl::set_expanded(mforms::GRTTreeView* self, const bec::NodeId&
         break;
 
       walker = walker->Children[node[level++]];
-      walker->Expand(expanded);
+
+      if (expanded)
+        walker->Expand();
+      else if (node.depth() == level)
+        walker->Collapse();
     } while (node.depth() > level);
     
   }
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void GRTTreeViewImpl::enable_asynch_loading(mforms::GRTTreeView* self, bool enabled)
+{
+  GRTTreeViewImpl^ treeview= (GRTTreeViewImpl^)ObjectImpl::FromUnmanaged(self);
+  if (treeview != nullptr)
+  {
+    TreeViewAdv^ t= treeview->get_control<TreeViewAdv>();
+    t->AsyncExpanding = enabled;
+  }
+}
+
+
 //--------------------------------------------------------------------------------------------------
 
 bool GRTTreeViewImpl::get_selected_node(GRTTreeView *self, bec::NodeId &node)
@@ -326,7 +344,7 @@ void GRTTreeViewImpl::set_allow_multi_selection(GRTTreeView *self, bool flag)
   if (treeview != nullptr)
   {
     TreeViewAdv^ t= treeview->get_control<TreeViewAdv>();
-    t->SelectionMode= flag ? TreeSelectionMode::Multi : TreeSelectionMode::Single;
+    t->SelectionMode= flag ? Aga::Controls::Tree::TreeSelectionMode::Multi : Aga::Controls::Tree::TreeSelectionMode::Single;
   }
 }
 
@@ -355,11 +373,10 @@ void GRTTreeViewImpl::set_model(bec::TreeModel* model)
     if (model == NULL)
       return;
   }
-  else
-    if (_tree_model->get_unmanaged_object() == model)
-      return;
 
   TreeViewAdv^ t= get_control<TreeViewAdv>();
+  bool wasAsyncExpanding = t->AsyncExpanding;
+  t->AsyncExpanding = false; // Switch off async expanding to avoid creating background tasks when switching models.
   if (_model != nullptr)
   {
     _model->DetachEvents();
@@ -391,6 +408,8 @@ void GRTTreeViewImpl::set_model(bec::TreeModel* model)
     // Finally set the model.
     t->Model= _model;
   }
+
+  t->AsyncExpanding = wasAsyncExpanding;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -424,7 +443,7 @@ int GRTTreeViewImpl::add_column(GRTTreeColumnType type, int model_column, String
   treeview->Columns->Add(column);
 
   // If this is the main column then add our expand node control first.
-  if (model_column == 0)
+  if (model_column == 0 && !treeview->ShowPlusMinus)
   {
     TriangleNodeControl^ control = gcnew TriangleNodeControl();
     control->ParentColumn= column;
@@ -518,11 +537,11 @@ void GRTTreeViewImpl::OnMouseDown(System::Object^ sender, MouseEventArgs^ e)
     // update the associated context menu
     if (t && t->get_context_menu())
     {
-      tree->ContextMenu = MenuImpl::get(t->get_context_menu());
+      tree->ContextMenuStrip = MenuImpl::GetNativeControl(t->get_context_menu());
       (*t->get_context_menu()->signal_will_show())();
     }
     else
-      tree->ContextMenu = nullptr;
+      tree->ContextMenuStrip = nullptr;
   }
 }
 
