@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,6 @@
  */
 
 #include "stdafx.h"
-#include "wf_label.h"
 
 using namespace System;
 using namespace Drawing;
@@ -29,6 +28,8 @@ using namespace Windows::Forms;
 
 using namespace MySQL::Forms;
 
+DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_WRAPPER)
+
 //--------------------------------------------------------------------------------------------------
 
 /**
@@ -69,6 +70,13 @@ WrapControlLabel::WrapControlLabel()
 
 //--------------------------------------------------------------------------------------------------
 
+WrapControlLabel::~WrapControlLabel()
+{
+
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Returns the preferred size of the label depending on its auto-wrapping mode.
  */
@@ -117,7 +125,7 @@ bool LabelImpl::create(::mforms::Label *self)
 
   if (label != nullptr)
   {
-    Label ^l= ViewImpl::create<WrapControlLabel>(self, label);
+    WrapControlLabel^ l= ViewImpl::create<WrapControlLabel>(self, label);
     l->TextAlign= ContentAlignment::MiddleLeft;
     l->BorderStyle= BorderStyle::None;
     l->ForeColor = Color::Black;
@@ -196,44 +204,52 @@ LabelImpl::LabelImpl(::mforms::Label *self)
 
 void LabelImpl::set_style(::mforms::LabelStyle style)
 {
-  Label ^label= get_control<Label>();
+  WrapControlLabel^ label= get_control<WrapControlLabel>();
 
-  switch (style)
+  try
   {
-  case ::mforms::NormalStyle:
-    label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE, FontStyle::Regular, GraphicsUnit::Pixel);
-    break;
-  case ::mforms::BoldStyle: 
-    label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE, FontStyle::Bold, GraphicsUnit::Pixel);
-    break;
-  case ::mforms::BigStyle:
-    label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE + 4, FontStyle::Regular, GraphicsUnit::Pixel);
-    break;
-  case ::mforms::BigBoldStyle:
-    label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE + 4, FontStyle::Bold, GraphicsUnit::Pixel);
-    break;
-  case ::mforms::SmallBoldStyle:
-    label->Font= gcnew Font(DEFAULT_FONT, 8, FontStyle::Bold, GraphicsUnit::Pixel);
-    break;
-  case ::mforms::SmallStyle:
-    label->Font= gcnew Font(DEFAULT_FONT, 8, FontStyle::Regular, GraphicsUnit::Point);
-    break;
-  case ::mforms::VerySmallStyle:
-    label->Font= gcnew Font(DEFAULT_SMALL_FONT, 7.5, FontStyle::Regular, GraphicsUnit::Point);
-    break;
-  case ::mforms::InfoCaptionStyle:
-    label->Font= gcnew Font(DEFAULT_FONT, label->Font->Size, FontStyle::Regular, GraphicsUnit::Pixel);
-    break;
-  case ::mforms::BoldInfoCaptionStyle:
-    label->Font= gcnew Font(DEFAULT_FONT, label->Font->Size, FontStyle::Bold, GraphicsUnit::Pixel);
-    break;
-  case ::mforms::WizardHeadingStyle:
-    label->Font= gcnew Font(DEFAULT_FONT, 12.0, FontStyle::Bold, GraphicsUnit::Pixel);
-    label->ForeColor= ColorTranslator::FromHtml("#003392");
-    break;
-  case ::mforms::SmallHelpTextStyle:
-    label->Font= gcnew Font(DEFAULT_FONT, 11, FontStyle::Regular, GraphicsUnit::Pixel);
-    break;
+    switch (style)
+    {
+    case ::mforms::NormalStyle:
+      label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE, FontStyle::Regular, GraphicsUnit::Pixel);
+      break;
+    case ::mforms::BoldStyle: 
+      label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE, FontStyle::Bold, GraphicsUnit::Pixel);
+      break;
+    case ::mforms::BigStyle:
+      label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE + 4, FontStyle::Regular, GraphicsUnit::Pixel);
+      break;
+    case ::mforms::BigBoldStyle:
+      label->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE + 4, FontStyle::Bold, GraphicsUnit::Pixel);
+      break;
+    case ::mforms::SmallBoldStyle:
+      label->Font= gcnew Font(DEFAULT_FONT, 8, FontStyle::Bold, GraphicsUnit::Pixel);
+      break;
+    case ::mforms::SmallStyle:
+      label->Font= gcnew Font(DEFAULT_FONT, 8, FontStyle::Regular, GraphicsUnit::Point);
+      break;
+    case ::mforms::VerySmallStyle:
+      label->Font= gcnew Font(DEFAULT_SMALL_FONT, 7.5, FontStyle::Regular, GraphicsUnit::Point);
+      break;
+    case ::mforms::InfoCaptionStyle:
+      label->Font= gcnew Font(DEFAULT_FONT, label->Font->Size, FontStyle::Regular, GraphicsUnit::Pixel);
+      break;
+    case ::mforms::BoldInfoCaptionStyle:
+      label->Font= gcnew Font(DEFAULT_FONT, label->Font->Size, FontStyle::Bold, GraphicsUnit::Pixel);
+      break;
+    case ::mforms::WizardHeadingStyle:
+      label->Font= gcnew Font(DEFAULT_FONT, 12.0, FontStyle::Bold, GraphicsUnit::Pixel);
+      label->ForeColor= ColorTranslator::FromHtml("#003392");
+      break;
+    case ::mforms::SmallHelpTextStyle:
+      label->Font= gcnew Font(DEFAULT_FONT, 11, FontStyle::Regular, GraphicsUnit::Pixel);
+      break;
+    }
+  }
+  catch (System::ArgumentException^ e)
+  {
+    // Argument exception pops up when the system cannot find the Regular font style (corrupt font).
+    log_error("LabelImpl::set_style failed. %s\n", e->Message);
   }
 }
 
@@ -241,16 +257,16 @@ void LabelImpl::set_style(::mforms::LabelStyle style)
 
 void LabelImpl::set_text(const std::string &text)
 {
-  String^ new_text= CppStringToNative(text);
-  WrapControlLabel^ label= get_control<WrapControlLabel>();
+  String^ new_text = CppStringToNative(text);
+  WrapControlLabel^ label = get_control<WrapControlLabel>();
   if (label->Text != new_text)
   {
-    label->Text= new_text;
+    label->Text = new_text;
 
-    Size newSize= label->PreferredSize;
+    Size newSize = label->PreferredSize;
     if (label->AutoWrapping)
-      newSize.Width= label->Size.Width;
-    label->Size= newSize;
+      newSize.Width = label->Size.Width;
+    label->Size = newSize;
   };
 }
 
@@ -258,21 +274,21 @@ void LabelImpl::set_text(const std::string &text)
 
 void LabelImpl::set_text_align(::mforms::Alignment align)
 {
-  get_control<Label>()->TextAlign= get_alignment(align);
+  get_control<WrapControlLabel>()->TextAlign = get_alignment(align);
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void LabelImpl::set_color(const std::string &color)
 {
-  get_control<Label>()->ForeColor= ColorTranslator::FromHtml(CppStringToNative(color));
+  get_control<WrapControlLabel>()->ForeColor = ColorTranslator::FromHtml(CppStringToNative(color));
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void LabelImpl::set_wrap_text(bool flag)
 {
-  get_control<WrapControlLabel>()->AutoWrapping= flag;
+  get_control<WrapControlLabel>()->AutoWrapping = flag;
 }
 
 //--------------------------------------------------------------------------------------------------
