@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2012 Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,7 +18,6 @@
  */
 
 #include "stdafx.h"
-#include "wf_menu.h"
 
 using namespace System::Windows::Forms;
 
@@ -42,16 +41,19 @@ MenuImpl::~MenuImpl()
 bool MenuImpl::create(mforms::Menu* self)
 {
   MenuImpl^ menu= gcnew MenuImpl(self);
-  ObjectImpl::create<ContextMenu>(self, menu);
+  ContextMenuStrip^ strip = ObjectImpl::create<ContextMenuStrip>(self, menu);
+  strip->AutoSize = true;
+  strip->Opening += gcnew System::ComponentModel::CancelEventHandler(&MenuImpl::MenuOpening);
+
   return true;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-ContextMenu ^MenuImpl::get(mforms::Menu *self)
+ContextMenuStrip^ MenuImpl::GetNativeControl(mforms::Menu *self)
 {
   MenuImpl^ menu= (MenuImpl^) ObjectImpl::FromUnmanaged(self);
-  return menu->get_control<ContextMenu>();
+  return menu->get_control<ContextMenuStrip>();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -59,8 +61,8 @@ ContextMenu ^MenuImpl::get(mforms::Menu *self)
 void MenuImpl::remove_item(mforms::Menu* self, int i)
 {
   MenuImpl^ menu= (MenuImpl^) ObjectImpl::FromUnmanaged(self);
-  ContextMenu^ native_menu= menu->get_control<ContextMenu>();
-  native_menu->MenuItems->RemoveAt(i);
+  ContextMenuStrip^ native_menu= menu->get_control<ContextMenuStrip>();
+  native_menu->Items->RemoveAt(i);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -68,11 +70,12 @@ void MenuImpl::remove_item(mforms::Menu* self, int i)
 int MenuImpl::add_item(mforms::Menu* self, const std::string& caption, const std::string& action)
 {
   MenuImpl^ menu= (MenuImpl^) ObjectImpl::FromUnmanaged(self);
-  Windows::Forms::MenuItem^ item= gcnew Windows::Forms::MenuItem(CppStringToNative(caption),
-    gcnew System::EventHandler(menu, &MenuImpl::menuItemClick));
+  ContextMenuStrip^ native_menu= menu->get_control<ContextMenuStrip>();
+  ToolStripItem^ item = native_menu->Items->Add(CppStringToNative(caption));
+  item->Click += gcnew System::EventHandler(menu, &MenuImpl::ItemClick);
   item->Tag= CppStringToNative(action);
-  ContextMenu^ native_menu= menu->get_control<ContextMenu>();
-  return native_menu->MenuItems->Add(item);
+
+  return native_menu->Items->Count - 1;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -80,22 +83,25 @@ int MenuImpl::add_item(mforms::Menu* self, const std::string& caption, const std
 int MenuImpl::add_separator(mforms::Menu* self)
 {
   MenuImpl^ menu= (MenuImpl^) ObjectImpl::FromUnmanaged(self);
-  Windows::Forms::MenuItem^ item= gcnew Windows::Forms::MenuItem("-");
-  ContextMenu^ native_menu= menu->get_control<ContextMenu>();
-  return native_menu->MenuItems->Add(item);
+  ContextMenuStrip^ native_menu= menu->get_control<ContextMenuStrip>();
+
+  native_menu->Items->Add(gcnew ToolStripSeparator());
+  return native_menu->Items->Count - 1;
 }
 
 //--------------------------------------------------------------------------------------------------
 
 int MenuImpl::add_submenu(mforms::Menu* self, const std::string& caption, mforms::Menu* submenu)
 {
-  MenuImpl^ menu= (MenuImpl^) ObjectImpl::FromUnmanaged(self);
-  MenuImpl^ child_menu= (MenuImpl^) ObjectImpl::FromUnmanaged(submenu);
-  Windows::Forms::MenuItem^ item= gcnew Windows::Forms::MenuItem(CppStringToNative(caption));
+  MenuImpl^ menu = (MenuImpl^) ObjectImpl::FromUnmanaged(self);
+  MenuImpl^ child_menu = (MenuImpl^) ObjectImpl::FromUnmanaged(submenu);
 
-  ContextMenu^ native_menu= menu->get_control<ContextMenu>();
-  item->MergeMenu(child_menu->get_control<ContextMenu>());
-  return native_menu->MenuItems->Add(item);
+  ContextMenuStrip^ native_menu = menu->get_control<ContextMenuStrip>();
+  ToolStripMenuItem^ item = gcnew ToolStripMenuItem(CppStringToNative(caption));
+  native_menu->Items->Add(item);
+  item->DropDown = child_menu->get_control<ContextMenuStrip>();
+
+  return native_menu->Items->Count - 1;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -103,8 +109,8 @@ int MenuImpl::add_submenu(mforms::Menu* self, const std::string& caption, mforms
 void MenuImpl::set_item_enabled(mforms::Menu* self, int i, bool flag)
 {
   MenuImpl^ menu= (MenuImpl^) ObjectImpl::FromUnmanaged(self);
-  ContextMenu^ native_menu= menu->get_control<ContextMenu>();
-  native_menu->MenuItems[i]->Enabled= flag;
+  ContextMenuStrip^ native_menu= menu->get_control<ContextMenuStrip>();
+  native_menu->Items[i]->Enabled= flag;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -134,7 +140,7 @@ void MenuImpl::popup_at(mforms::Menu* self, mforms::Object* control, int x, int
   }
   if (controller != nullptr)
   {
-    ContextMenu^ native_menu= menu->get_control<ContextMenu>();
+    ContextMenuStrip^ native_menu= menu->get_control<ContextMenuStrip>();
     native_menu->Show(controller, System::Drawing::Point(x, y));
   }
 }
@@ -144,21 +150,21 @@ void MenuImpl::popup_at(mforms::Menu* self, mforms::Object* control, int x, int
 void MySQL::Forms::MenuImpl::clear( mforms::Menu* self )
 {
   MenuImpl^ menu= (MenuImpl^) ObjectImpl::FromUnmanaged(self);
-  ContextMenu^ native_menu= menu->get_control<ContextMenu>();
-  native_menu->MenuItems->Clear();
+  ContextMenuStrip^ native_menu= menu->get_control<ContextMenuStrip>();
+  native_menu->Items->Clear();
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void MenuImpl::menuItemClick(Object^ sender, System::EventArgs^ e)
+void MenuImpl::ItemClick(Object^ sender, System::EventArgs^ e)
 {
-  Windows::Forms::MenuItem^ item= dynamic_cast<Windows::Forms::MenuItem^>(sender);
+  ToolStripItem^ item = dynamic_cast<ToolStripItem^>(sender);
   if (item != nullptr)
   {
-    String^ action= dynamic_cast<String^>(item->Tag);
+    String^ action = dynamic_cast<String^>(item->Tag);
     if (action != nullptr)
     {
-      mforms::Menu* menu= ObjectImpl::get_backend_control<mforms::Menu>(item->Parent);
+      mforms::Menu* menu = ObjectImpl::get_backend_control<mforms::Menu>(item->Owner);
       if (menu != NULL)
         menu->handle_action(NativeToCppString(action));
     }
@@ -166,3 +172,16 @@ void MenuImpl::menuItemClick(Object^ sender, System::EventArgs^ e)
 }
 
 //--------------------------------------------------------------------------------------------------
+
+void MenuImpl::MenuOpening(Object^ sender, System::ComponentModel::CancelEventArgs^ e)
+{
+  ContextMenuStrip^ native_menu = dynamic_cast<ContextMenuStrip^>(sender);
+  if (native_menu != nullptr)
+  {
+    mforms::Menu* menu = ObjectImpl::get_backend_control<mforms::Menu>(native_menu);
+    if (menu != NULL)
+      (*(menu->signal_will_show()))();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
