@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,7 +18,6 @@
  */
 
 #include "stdafx.h"
-#include "wf_panel.h"
 
 using namespace System;
 
@@ -80,7 +79,7 @@ System::Drawing::Size FillLayout::ComputeLayout(Control^ control, System::Drawin
       {
         // The value for the content can be asynchronously only if a handle is created on the parent panel.
         ApplyBoundsDelegate^ applyBoundsDelegate= gcnew ApplyBoundsDelegate(valueSetter, &ValueSetter::ApplyContentBounds);
-        control->Invoke(applyBoundsDelegate, gcnew array<Object^>{ newBounds });
+        control->BeginInvoke(applyBoundsDelegate, gcnew array<Object^>{ newBounds });
       }
     }
 
@@ -132,6 +131,16 @@ System::Drawing::Size FillGroupBox::GetPreferredSize(System::Drawing::Size propo
   return size;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void FillGroupBox::OnPaintBackground(PaintEventArgs^ args)
+{
+  GroupBox::OnPaintBackground(args);
+
+  ViewImpl ^view = ObjectImpl::get_wrapper<ViewImpl>(this);
+  view->DrawBackground(args);
+}
+
 //----------------- FillPanel ---------------------------------------------------------------------
 
 System::Drawing::Size FillPanel::GetPreferredSize(System::Drawing::Size proposedSize)
@@ -144,6 +153,16 @@ System::Drawing::Size FillPanel::GetPreferredSize(System::Drawing::Size proposed
   return size;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void FillPanel::OnPaintBackground(PaintEventArgs^ args)
+{
+  Panel::OnPaintBackground(args);
+
+  ViewImpl ^view = ObjectImpl::get_wrapper<ViewImpl>(this);
+  view->DrawBackground(args);
+}
+
 //----------------- HeaderFillPanel ----------------------------------------------------------------
 
 System::Drawing::Size HeaderFillPanel::GetPreferredSize(System::Drawing::Size proposedSize)
@@ -194,7 +213,7 @@ bool PanelImpl::create(mforms::Panel *self, mforms::PanelType panelType)
     case mforms::FilledHeaderPanel:
     case mforms::StyledHeaderPanel: // Panel with header (rounded corners)
       control= ViewImpl::create<HeaderFillPanel>(self, panel);
-      control->Padding = Padding(0, 24, 0, 0);
+      control->Padding = Padding(0, 19, 0, 0);
       control->BackColor = Color::FromArgb(40, 55, 82);
       break;
 
@@ -272,19 +291,6 @@ bool PanelImpl::get_active(mforms::Panel* self)
 
 //-------------------------------------------------------------------------------------------------
 
-void PanelImpl::set_padding(mforms::Panel* self, int value)
-{
-  PanelImpl^ panel= (PanelImpl^)ObjectImpl::FromUnmanaged(self);
-
-  if (panel != nullptr)
-  {
-    panel->set_padding(value);
-    self->set_layout_dirty(true);
-  }
-}
-
-//-------------------------------------------------------------------------------------------------
-
 void PanelImpl::remove(mforms::Panel *self, mforms::View* view)
 {
   PanelImpl^ panel= (PanelImpl^)ObjectImpl::FromUnmanaged(self);
@@ -327,7 +333,7 @@ void PanelImpl::add(mforms::View *view)
 {
   ViewImpl^ child= (ViewImpl^)ObjectImpl::FromUnmanaged(view);
   Control^ ctl= child->get_control<Control>();
-  ctl->Dock = DockStyle::Fill; // For HeaderPanel.
+  //ctl->Dock = DockStyle::Fill; // Fill dock style defeats the purpose of doing our own layout.
 
   _child= view;
   Control^ panel_ctl= get_control<Control>();
@@ -351,13 +357,6 @@ bool PanelImpl::get_active()
 
 //-------------------------------------------------------------------------------------------------
 
-void PanelImpl::set_padding(int value)
-{
-  get_control<Control>()->Padding= Padding(value);
-}
-
-//-------------------------------------------------------------------------------------------------
-
 void PanelImpl::remove(mforms::View* view)
 {
   ViewImpl^ child= (ViewImpl^)ObjectImpl::FromUnmanaged(view);
