@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,10 +18,6 @@
  */
 
 #include "stdafx.h"
-#include "wf_popup.h"
-#include "wf_view.h"
-
-#include <cairo/cairo-Win32.h>
 
 using namespace System::Drawing;
 using namespace System::Windows::Forms;
@@ -436,7 +432,7 @@ void TransparentPopupControl::PrepareBitmap()
 
   // Let the backend draw its part.
   cairo_t* cr= cairo_create(surface);
-  MySQL::Geometry::Rect displayRect= DisplayRect;
+  base::Rect displayRect= DisplayRect;
 
   mforms::Popup* popup= ObjectImpl::get_backend_control<mforms::Popup>(this);
   if (popup != NULL)
@@ -553,7 +549,7 @@ int PopupImpl::show(mforms::Popup *self, int spot_x, int spot_y)
 
 //-------------------------------------------------------------------------------------------------
 
-MySQL::Geometry::Rect PopupImpl::get_content_rect(mforms::Popup *self)
+base::Rect PopupImpl::get_content_rect(mforms::Popup *self)
 {
   PopupImpl^ popup= (PopupImpl^)ObjectImpl::FromUnmanaged(self);
 
