@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,8 +18,6 @@
  */
 
 #include "stdafx.h"
-#include "wf_table.h"
-#include "wf_label.h"
 
 using namespace MySQL;
 using namespace MySQL::Forms;
@@ -124,6 +122,8 @@ int CompareRowSpan(CellEntry^ entry1, CellEntry^ entry2)
 bool GtkTableLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
 {
   Table^ table= (Table^) container;
+  if (!table->IsHandleCreated)
+    return false;
 
   if (table->RowCount > 0 && table->ColumnCount > 0)
   {
@@ -226,12 +226,24 @@ System::Drawing::Size Table::ComputeLayout(System::Drawing::Size proposedSize, b
         horizontalExpandState[i]= true;
     }
 
-    entry->bounds= System::Drawing::Rectangle(Point(0, 0),
-      entry->control->GetPreferredSize(System::Drawing::Size(currentWidth, 0)));
-    if (ViewImpl::use_min_width_for_layout(entry->control))
-      entry->bounds.Width= entry->control->MinimumSize.Width;
-    if (ViewImpl::use_min_height_for_layout(entry->control))
-      entry->bounds.Height= entry->control->MinimumSize.Height;
+    bool use_min_width = ViewImpl::use_min_width_for_layout(entry->control);
+    bool use_min_height = ViewImpl::use_min_height_for_layout(entry->control);
+    if (use_min_width && use_min_height)
+      entry->bounds.Size = entry->control->MinimumSize;
+    else
+    {
+      entry->bounds= System::Drawing::Rectangle(Point(0, 0),
+        entry->control->GetPreferredSize(System::Drawing::Size(currentWidth, 0)));
+      if (use_min_width)
+        entry->bounds.Width= entry->control->MinimumSize.Width;
+      if (use_min_height)
+        entry->bounds.Height= entry->control->MinimumSize.Height;
+    }
+
+    // Weird behavior of combo boxes. They report a one pixel too small height (you cannot set them
+    // to this smaller height, though). So this wrong value is messing up our computed overall height.
+    if (entry->control->GetType() == ComboBox::typeid)
+      entry->bounds.Height++;
 
     // Set all cells to the computed partial size if it is larger than what was found so far.
     // On the way apply the expand flag to all cells that are covered by that entry.
@@ -603,18 +615,6 @@ void TableImpl::set_column_spacing(mforms::Table *self, int space)
 
 //-------------------------------------------------------------------------------------------------
 
-void TableImpl::set_padding(mforms::Table *self, int padding)
-{
-  TableImpl^ table= (TableImpl^)ObjectImpl::FromUnmanaged(self);
-  if (table != nullptr)
-  {
-    table->set_padding(padding);
-    self->set_layout_dirty(true);
-  }
-}
-
-//-------------------------------------------------------------------------------------------------
-
 void TableImpl::set_homogeneous(mforms::Table *self, bool value)
 {
   TableImpl^ table= (TableImpl^)ObjectImpl::FromUnmanaged(self);
@@ -677,10 +677,3 @@ void TableImpl::set_column_count(int count)
 }
 
 //-------------------------------------------------------------------------------------------------
-
-void TableImpl::set_padding(int padding)
-{
-  get_control<Table>()->Padding= Padding(padding);
-}
-
-//-------------------------------------------------------------------------------------------------
