@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,15 +19,167 @@
 
 #include "stdafx.h"
 
-#include "wf_toolbar.h"
-#include "wf_view.h"
-
 using namespace System::Windows::Forms;
 using namespace System::IO;
 using namespace System::Drawing;
 
 using namespace MySQL::Forms;
 using namespace MySQL::Controls;
+using namespace MySQL::Utilities;
+
+DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_WRAPPER)
+
+//----------------- MformsToolStripLayout ----------------------------------------------------------
+
+/**
+ * Computes the entire layout of the toolstrip (position of items).
+ * 
+ * @param toolstrip What we are computing the layout for.
+ * @param proposedSize The size to start from layouting. Since super ordinated controls may impose
+ *                     a layout size we need to honor that (especially important for auto wrapping
+ *                     labels).
+ * @param preferredSizeOnly If true arrange compute only the minimal size needed, otherwise do a full layout.
+ * @return The resulting size of the box.
+ */
+System::Drawing::Size MformsToolStripLayout::ComputeLayout(MformsToolStrip^ toolstrip,
+  System::Drawing::Size proposedSize, bool preferredSizeOnly)
+{
+  int itemsTotalWidth= 0;
+  int maxHeight = 0;
+  int expandCount= 0;
+
+  // Consider extra pixels for the height if we have a main tool strip (it renders an inner border).
+  int borderOffset = 0;
+  if (toolstrip->Renderer->GetType() == FlatMainToolStripRenderer::typeid)
+    borderOffset = 2;
+
+  if (!preferredSizeOnly)
+  {
+    proposedSize.Width -= toolstrip->Padding.Horizontal + toolstrip->GripMargin.Horizontal;
+    proposedSize.Height = toolstrip->Size.Height - toolstrip->Padding.Vertical - borderOffset;
+  }
+
+  // The Margin property of a ToolStripItem has a special meaning. The horizontal values determine
+  // the distances to neighbor items. The vertical values reduce the available vertical space into
+  // which an item can be placed. An item is always vertically centered in its remaining space.
+  // Hence the horizontal margin extends the overall width, while the vertical distance does not.
+
+  // Determine total width of all items so we can distribute any remaining space over
+  // all expander items.
+  for each (ToolStripItem^ item in toolstrip->Items)
+  {
+    if (!item->Visible)
+      continue;
+
+    if (item->GetType() != ToolStripSegmentedButton::typeid)
+      item->Size = item->GetPreferredSize(proposedSize);
+    itemsTotalWidth += item->Width+ item->Margin.Horizontal;
+    if (item->Height > maxHeight)
+      maxHeight = item->Height;
+    if (item->GetType() == ToolStripExpander::typeid)
+      expandCount++;
+  }
+  
+  // Adjust width of the container if it is too small.
+  if (proposedSize.Width < itemsTotalWidth)
+    proposedSize.Width = itemsTotalWidth;
+
+  // Adjust height of the container if it is too small.
+  if (proposedSize.Height < maxHeight)
+    proposedSize.Height = maxHeight;
+
+  if (!preferredSizeOnly)
+  {
+    // Fraction of remaining space per expander. Due to integral pixel bounds + locations
+    // we might end up with a number of unused pixel. These will be individually distributed over
+    // all expanders as well.
+    int expanderFraction = 0;
+    int remainingSpace = 0;
+    if (expandCount > 0)
+    {
+      expanderFraction = (proposedSize.Width - itemsTotalWidth) / expandCount;
+      remainingSpace = proposedSize.Width - itemsTotalWidth - (expanderFraction * expandCount);
+    }
+
+    int offset = toolstrip->Padding.Left + toolstrip->GripMargin.Horizontal;
+
+    for each (ToolStripItem^ item in toolstrip->Items)
+    {
+      if (item->GetType() == ToolStripExpander::typeid)
+      {
+        offset += expanderFraction;
+        if (remainingSpace > 0)
+        {
+          offset++;
+          remainingSpace--;
+        }
+      }
+      else
+      {
+        System::Drawing::Point itemLocation;
+        int remainingHeight = proposedSize.Height - item->Margin.Vertical;
+
+        // Vertically center the item within the bounds that remain when we take off its margin.
+        itemLocation.Y = toolstrip->Padding.Top + item->Margin.Top + borderOffset / 2 +
+          (int) Math::Ceiling((remainingHeight - item->Height) / 2.0);
+        itemLocation.X = offset;
+        toolstrip->ApplyLocation(item, itemLocation);
+        offset += item->Width;
+
+        // No spacing between segmented buttons, but still spacing around the first and last button.
+        if (item->GetType() == ToolStripSegmentedButton::typeid)
+        {
+          // Check if this item is surrounded by other segmented buttons.
+          offset += 0;
+        }
+        else
+          offset += item->Margin.Horizontal;
+      }
+    }
+  }
+
+  proposedSize.Width += toolstrip->Padding.Horizontal + toolstrip->GripMargin.Horizontal;
+  proposedSize.Height += toolstrip->Padding.Vertical + borderOffset;
+
+  return proposedSize;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool MformsToolStripLayout::Layout(Object^ container, LayoutEventArgs^ arguments)
+{
+  MformsToolStrip^ toolstrip= (MformsToolStrip^) container;
+
+  ControlUtilities::SuspendDrawing(toolstrip);
+
+  System::Drawing::Size toolStripSize = ComputeLayout(toolstrip, toolstrip->Size, false);
+
+  // Finally resize container if necessary.
+  bool parentLayoutNeeded= !toolstrip->Size.Equals(toolStripSize);
+  if (parentLayoutNeeded && toolstrip->AutoSize)
+    toolstrip->Size = toolStripSize;
+
+  ControlUtilities::ResumeDrawing(toolstrip);
+  toolstrip->Refresh();
+
+  return parentLayoutNeeded;
+}
+
+//----------------- MformsToolStrip ----------------------------------------------------------------
+
+void MformsToolStrip::ApplyLocation(ToolStripItem^ item, Point location)
+{
+  // SetItemLocation is protected and its visibility cannot be increased nor is it possible
+  // to allow access to it via friend class or using clauses.
+  SetItemLocation(item, location);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+System::Drawing::Size MformsToolStrip::GetPreferredSize(System::Drawing::Size proposedSize)
+{
+  return layoutEngine->ComputeLayout(this, proposedSize, true);
+}
 
 //----------------- ToolBarItemImpl ----------------------------------------------------------------
 
@@ -38,22 +190,66 @@ ToolBarItemImpl::ToolBarItemImpl(mforms::ToolBarItem* item)
 
 //--------------------------------------------------------------------------------------------------
 
-void ToolBarItemImpl::set_normal_icon(const std::string& path)
+ToolBarItemImpl::~ToolBarItemImpl()
 {
-  _normal_icon = CppStringToNative(path);
-  if (File::Exists(_normal_icon))
-  {
-    ToolStripItem^ item = get_control<ToolStripItem>();
-    if (item != nullptr)
-      item->Image = Image::FromFile(_normal_icon);
-  }
+  delete normalImage;
+  delete activeImage;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void ToolBarItemImpl::set_active_icon(const std::string& path)
+void ToolBarItemImpl::UpdateItemImage()
 {
-  _active_icon = CppStringToNative(path);
+  ToolStripItem^ item = get_control<ToolStripItem>();
+  ToolStripButton^ button = dynamic_cast<ToolStripButton^>(item);
+  bool isChecked = false;
+  if (button != nullptr)
+    isChecked = dynamic_cast<ToolStripButton^>(item)->Checked;
+
+  static Size borderSize(2, 2);
+  if (item->GetType() == ToolStripSegmentedButton::typeid)
+  {
+    // For segmented buttons we use a trick to avoid the implicit border given by the default
+    // implementation. Using a background image avoids it. We have to size the item
+    // explicitly, though. Auto size must be off to make this work.
+    if (isChecked)
+    {
+      item->BackgroundImage = activeImage;
+      if (activeImage != nullptr)
+      {
+        item->AutoSize = false;
+        item->Size = activeImage->Size + button->Margin.Size + borderSize;
+      }
+    }
+    else
+    {
+      item->BackgroundImage = normalImage;
+      if (normalImage != nullptr)
+      {
+        item->AutoSize = false;
+        item->Size = normalImage->Size + button->Margin.Size + borderSize;
+      }
+    }
+  }
+  else
+  {
+    // Use the normal image also for checked items if no active image exists.
+    if (isChecked && activeImage != nullptr)
+    {
+      item->Image = activeImage;
+      item->AutoSize = false;
+      item->Size = activeImage->Size + button->Margin.Size + borderSize;
+    }
+    else
+    {
+      item->Image = normalImage;
+      if (normalImage != nullptr)
+      {
+        item->AutoSize = false;
+        item->Size = normalImage->Size + button->Margin.Size + borderSize;
+      }
+    }
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -64,11 +260,7 @@ void ToolBarItemImpl::set_item_checked(bool state)
   if (button != nullptr && button->Checked != state)
   {
     button->Checked = state;
-    if (state && File::Exists(_active_icon))
-      button->Image = Image::FromFile(_active_icon);
-    else
-      if (File::Exists(_normal_icon))
-        button->Image = Image::FromFile(_normal_icon);
+    UpdateItemImage();
   }
 }
 
@@ -76,13 +268,16 @@ void ToolBarItemImpl::set_item_checked(bool state)
 
 void ToolBarItemImpl::OnItemActivation(Object^ sender, System::EventArgs^ e)
 {
-  ToolStripItem^ native_item = (ToolStripItem^) sender;
+  ToolStripItem^ native_item = dynamic_cast<ToolStripItem^>(sender);
 
-  if (native_item->Tag != nullptr)
+  if (native_item != nullptr && native_item->Tag != nullptr)
   {
     mforms::ToolBarItem* item = ViewImpl::get_backend_control<mforms::ToolBarItem>(native_item);
     if (item != NULL)
+    {
       item->callback();
+      UpdateItemImage();
+    }
   }
 }
 
@@ -103,6 +298,37 @@ void ToolBarItemImpl::OnColorItemActivation(Object^ sender, System::EventArgs^ e
   }
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void ToolBarItemImpl::OnKeyPress(Object^ sender, KeyPressEventArgs^ e)
+{
+  if (e->KeyChar == '\r')
+  {
+    e->Handled = true;
+    ToolStripTextBox^ textField = dynamic_cast<ToolStripTextBox^>(sender);
+
+    if (textField->Tag != nullptr)
+    {
+      mforms::ToolBarItem* item = ViewImpl::get_backend_control<mforms::ToolBarItem>(textField);
+
+      // XXX: temporary, will later be rewired to search().
+      if (item != NULL)
+        //item->search(NativeToCppString(textField->Text));
+        item->callback();
+    }
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void ToolBarItemImpl::focus()
+{
+  ToolStripItem^ item = get_control<ToolStripItem>();
+  ToolStripTextBox^ textbox = dynamic_cast<ToolStripTextBox ^>(item);
+  if (textbox != nullptr)
+    textbox->Focus();
+}
+
 //----------------- ToolBarImpl --------------------------------------------------------------------
 
 ToolBarImpl::ToolBarImpl(mforms::ToolBar* toolbar)
@@ -128,35 +354,63 @@ bool ToolBarImpl::create_tool_bar(mforms::ToolBar* item, mforms::ToolBarType typ
   switch (type)
   {
     case mforms::MainToolBar:
-      native_toolbar = ViewImpl::create<ToolStrip>(item, toolbar);
-      native_toolbar->Name = "ToolBar";
+      native_toolbar = ViewImpl::create<MformsToolStrip>(item, toolbar);
+      native_toolbar->Name = "MainToolBar";
       native_toolbar->Renderer = gcnew FlatMainToolStripRenderer();
+      native_toolbar->Padding = Padding(0);
       native_toolbar->GripMargin = Padding(4, 6, 4, 6);
       native_toolbar->Margin = Padding(4, 8, 2, 8);
+      native_toolbar->AutoSize = true;
+      break;
+
+    case mforms::SecondaryToolBar:
+    case mforms::PaletteToolBar:
+      native_toolbar = ViewImpl::create<MformsToolStrip>(item, toolbar);
+      native_toolbar->Name = "SecondaryToolBar";
+      native_toolbar->Renderer = gcnew FlatSubToolStripRenderer();
+      native_toolbar->GripStyle = ToolStripGripStyle::Hidden;
+      native_toolbar->GripMargin = Padding(2, 4, 2, 5);
+      native_toolbar->Padding = Padding(5, 0, 0, 0);
+      native_toolbar->AutoSize = false;
+      native_toolbar->Size = Size(100, 28);
+      try
+      {
+        native_toolbar->Font = ControlUtilities::GetFont("Microsoft Sans Serif", 8.25f);
+      }
+      catch (System::ArgumentException^ e)
+      {
+        // Argument exception pops up when the system cannot find the Regular font style (corrupt font).
+        log_error("ToolBarImpl::create_tool_bar failed. %s\n", e->Message);
+      }
+
+      native_toolbar->BackColor = ColorTranslator::FromHtml("#bcc7d8");
       break;
 
     case mforms::OptionsToolBar:
-      native_toolbar = ViewImpl::create<ToolStrip>(item, toolbar);
-      native_toolbar->Name = "ToolBar";
+      native_toolbar = ViewImpl::create<MformsToolStrip>(item, toolbar);
+      native_toolbar->Name = "OptionsToolBar";
       native_toolbar->GripMargin = Padding(4, 6, 4, 6);
       native_toolbar->Margin = Padding(4, 8, 2, 8);
       native_toolbar->RenderMode = ToolStripRenderMode::ManagerRenderMode;
+      native_toolbar->AutoSize = true;
       break;
 
     case mforms::ToolPickerToolBar:
       native_toolbar = ViewImpl::create<ToolStrip>(item, toolbar);
-      native_toolbar->Name = "ToolBar";
+      native_toolbar->Name = "ToolPickerToolBar";
       native_toolbar->BackColor = SystemColors::ButtonFace;
       native_toolbar->GripStyle = ToolStripGripStyle::Hidden;
-      native_toolbar->ImageScalingSize = System::Drawing::Size(24, 22);
+      //native_toolbar->ImageScalingSize = System::Drawing::Size(24, 22);
       native_toolbar->LayoutStyle = ToolStripLayoutStyle::VerticalStackWithOverflow;
-      native_toolbar->Padding = Padding(2, 0, 0, 0);
+      native_toolbar->Padding = Padding(0, 0, 0, 0);
       native_toolbar->RenderMode = ToolStripRenderMode::ManagerRenderMode;
+      native_toolbar->AutoSize = true;
       break;
+
+    default:
+      throw std::runtime_error(base::strfmt("Internal error: unimplemented toolbar type requested (%i).", (int) type));
   }
 
-  if (native_toolbar != nullptr)
-    native_toolbar->AutoSize = true;
   return (native_toolbar != nullptr);
 }
 
@@ -178,30 +432,6 @@ void ToolBarImpl::insert_item(mforms::ToolBar* toolbar, int index, mforms::ToolB
     }
     else
       native_toolbar->Items->Insert(index, native_item);
-
-    // To simulate expander items we align all entries after such an entry on the right hand side.
-    bool hasExpander = false;
-    for (int i = index - 1; i >= 0; i--)
-      if (native_toolbar->Items[i]->GetType() == ExpandableToolStripSeparator::typeid)
-      {
-        hasExpander = true;
-        break;
-      }
-
-    // If there is already an expander left to the new entry then we don't need to handle the case
-    // that the new entry is itself an expander.
-    if (hasExpander)
-      native_item->Alignment = ToolStripItemAlignment::Right;
-    if (!hasExpander && item->get_type() == mforms::ExpanderItem)
-    {
-      // This is the first expander so right-align all following entries up to the next expander.
-      for (int i = index + 1; i < native_toolbar->Items->Count; i++)
-      {
-        native_toolbar->Items[i]->Alignment = ToolStripItemAlignment::Right;
-        if (native_toolbar->Items[i]->GetType() == ExpandableToolStripSeparator::typeid)
-          break;
-      }
-    }
   }
 }
 
@@ -216,36 +446,9 @@ void ToolBarImpl::remove_item(mforms::ToolBar* toolbar, mforms::ToolBarItem* ite
   {
     ToolStrip^ native_toolbar = toolbar_impl->get_control<ToolStrip>();
     ToolStripItem^ native_item = toolbar_item->get_control<ToolStripItem>();
-    int index = native_toolbar->Items->IndexOf(native_item);
-
-    // Early out if the given item is not in this toolbar.
-    if (index < 0)
-      return;
-
-    // Check if we have to reset the right alignment of certain controls.
-    // Is there an expander item left to what we remove?
-    bool hasExpander = false;
-    for (int i = index - 1; i >= 0; i--)
-      if (native_toolbar->Items[i]->GetType() == ExpandableToolStripSeparator::typeid)
-      {
-        hasExpander = true;
-        break;
-      }
-
-      // If there is already an expander left to the new entry then we don't need to handle the case
-      // that the entry to remove is itself an expander.
-      if (!hasExpander && item->get_type() == mforms::ExpanderItem)
-      {
-        // This is the first expander so set all following items to left align until the next expander.
-        for (int i = index + 1; i < native_toolbar->Items->Count; i++)
-        {
-          native_toolbar->Items[i]->Alignment = ToolStripItemAlignment::Left;
-          if (native_toolbar->Items[i]->GetType() == ExpandableToolStripSeparator::typeid)
-            break;
-        }
-      }
-
     native_toolbar->Items->Remove(native_item);
+
+    delete native_item;
   }
 }
 
@@ -258,6 +461,18 @@ bool ToolBarImpl::create_tool_item(mforms::ToolBarItem* item, mforms::ToolBarIte
   ToolStripItem^ native_item;
   switch (type)
   {
+  case mforms::ActionItem:
+    native_item = ViewImpl::create<ToolStripButton>(item, toolbar_item, native_item);
+    native_item->Click += gcnew System::EventHandler(toolbar_item, &ToolBarItemImpl::OnItemActivation);
+    native_item->Margin = Padding(1, 2, 1, 2);
+    break;
+
+  case mforms::TextActionItem:
+    native_item = ViewImpl::create<ToolStripButton>(item, toolbar_item, native_item);
+    native_item->Click += gcnew System::EventHandler(toolbar_item, &ToolBarItemImpl::OnItemActivation); 
+    native_item->ForeColor = ColorTranslator::FromHtml("#4a6184");
+    break;
+
   case mforms::LabelItem:
     native_item = ViewImpl::create<ToolStripLabel>(item, toolbar_item, native_item);
     native_item->ForeColor = Color::Black;
@@ -267,6 +482,17 @@ bool ToolBarImpl::create_tool_item(mforms::ToolBarItem* item, mforms::ToolBarIte
     native_item = ViewImpl::create<ToolStripButton>(item, toolbar_item, native_item);
     native_item->DisplayStyle = ToolStripItemDisplayStyle::Image;
     native_item->Click += gcnew System::EventHandler(toolbar_item, &ToolBarItemImpl::OnItemActivation); 
+    native_item->Margin = Padding(1, 2, 1, 2);
+    ((ToolStripButton^) native_item)->CheckOnClick = true;
+    break;
+
+  case mforms::SegmentedToggleItem:
+    native_item = ViewImpl::create<ToolStripSegmentedButton>(item, toolbar_item, native_item);
+    native_item->DisplayStyle = ToolStripItemDisplayStyle::Image;
+    native_item->Click += gcnew System::EventHandler(toolbar_item, &ToolBarItemImpl::OnItemActivation); 
+    native_item->Margin = Padding(0, 0, 0, 0);
+    native_item->AutoSize = false;
+    ((ToolStripSegmentedButton^) native_item)->CheckOnClick = true;
     break;
 
   case mforms::SearchFieldItem:
@@ -276,7 +502,7 @@ bool ToolBarImpl::create_tool_item(mforms::ToolBarItem* item, mforms::ToolBarIte
       box->AutoSize = false;
       box->Size = Size(180, 20);
       box->TextBox->MaximumSize = Size(180, 20);
-      //box->KeyPress += new KeyPressEventHandler(tb_KeyPress);
+      box->KeyPress += gcnew KeyPressEventHandler(toolbar_item, &ToolBarItemImpl::OnKeyPress);
 
       native_item = box;
       break;
@@ -286,6 +512,9 @@ bool ToolBarImpl::create_tool_item(mforms::ToolBarItem* item, mforms::ToolBarIte
     {
       ToolStripComboBox^ selector = ViewImpl::create<ToolStripComboBox>(item, toolbar_item, native_item);
       selector->DropDownStyle = ComboBoxStyle::DropDownList;
+      selector->FlatStyle = FlatStyle::Popup;
+      selector->Font = ControlUtilities::GetFont("Microsoft Sans Serif", 8.25f);
+      selector->Margin = Padding(1, 3, 1, 3);
       selector->SelectedIndexChanged += gcnew EventHandler(toolbar_item, &ToolBarItemImpl::OnItemActivation);
       native_item = selector;
       break;
@@ -311,12 +540,11 @@ bool ToolBarImpl::create_tool_item(mforms::ToolBarItem* item, mforms::ToolBarIte
     break;
 
   case mforms::ExpanderItem:
-    native_item = ViewImpl::create<ExpandableToolStripSeparator>(item, toolbar_item, native_item);
+    native_item = ViewImpl::create<ToolStripExpander>(item, toolbar_item, native_item);
     break;
 
-  default: // mforms::ActionItem
-    native_item = ViewImpl::create<ToolStripButton>(item, toolbar_item, native_item);
-    native_item->Click += gcnew System::EventHandler(toolbar_item, &ToolBarItemImpl::OnItemActivation); 
+  default:
+    throw std::runtime_error(base::strfmt("Internal error: unimplemented toolbaritem type requested (%i).", (int) type));
   }
 
   native_item->ImageScaling = ToolStripItemImageScaling::None;
@@ -330,8 +558,11 @@ void ToolBarImpl::set_item_icon(mforms::ToolBarItem* item, const std::string& pa
 {
   ToolBarItemImpl^ toolbar_item = (ToolBarItemImpl^) ViewImpl::FromUnmanaged(item);
   if (toolbar_item != nullptr)
-    toolbar_item->set_normal_icon(path);
-  // TODO: set the icon depending on the item state.
+  {
+    String^ iconPath = CppStringToNative(path);
+    if (File::Exists(iconPath))
+      toolbar_item->NormalImage = Image::FromFile(iconPath);
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -340,8 +571,11 @@ void ToolBarImpl::set_item_alt_icon(mforms::ToolBarItem* item, const std::string
 {
   ToolBarItemImpl^ toolbar_item = (ToolBarItemImpl^) ViewImpl::FromUnmanaged(item);
   if (toolbar_item != nullptr)
-    toolbar_item->set_active_icon(path);
-  // TODO: set the icon depending on the item state.
+  {
+    String^ iconPath = CppStringToNative(path);
+    if (File::Exists(iconPath))
+      toolbar_item->ActiveImage = Image::FromFile(iconPath);
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
