@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,21 +22,8 @@
  */
 
 #include "stdafx.h"
-
-#include <sstream>
-#include <commctrl.h>
-#include <ShellAPI.h>
 #include <StrSafe.h>
 
-#include "base/string_utilities.h"
-#include "wf_Utilities.h"
-
-using namespace Windows::Forms;
-using namespace System::Text;
-using namespace System::Drawing;
-using namespace System::Threading;
-using namespace System::Diagnostics;
-
 using namespace MySQL::Forms;
 using namespace MySQL::Utilities;
 
@@ -104,26 +91,14 @@ CustomMessageBox::CustomMessageBox()
   this->HelpButton= false;
   this->MinimizeBox= false;
   this->MaximizeBox= false;
-  this->TopMost= true;
+  //this->TopMost= true;
+  this->Owner = UtilitiesImpl::get_mainform();
 
   ResumeLayout(false);
 }
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Returns the window handle of the main form. If the main form handle is not yet created or we
- * are in a background thread return 0.
- */
-HWND get_mainform_handle()
-{
-  if (!Application::OpenForms[0]->IsHandleCreated || Application::OpenForms[0]->InvokeRequired)
-    return 0;
-  else
-    return (HWND) Application::OpenForms[0]->Handle.ToPointer();
-}
-//--------------------------------------------------------------------------------------------------
-
 // Some more task dialog icons, which are not defined in commctrl.h.
 #define TD_SHIELD_BLUE_ICON      MAKEINTRESOURCE(-5)
 #define TD_SECURITY_WARNING_ICON MAKEINTRESOURCE(-6)
@@ -153,19 +128,28 @@ mforms::DialogResult CustomMessageBox::ShowVistaStyle(const std::string& title,
   TASKDIALOG_BUTTON buttons[2];
 
   std::wstring do_text = base::string_to_wstring(buttonOK);
-  if (do_text.size() > 0)
+  if (!do_text.empty())
   {
     buttons[buttonCount].nButtonID = IDOK;
     buttons[buttonCount].pszButtonText = do_text.c_str();
     buttonCount++;
   }
 
-  // For cancel we show the in-built button, instead creating our own.
-  // Additionally, we enable dialog cancellation.
-  if (buttonCancel.size() > 0)
+  // Enable dialog cancellation via red-cross button if we have a cancel button.
+  // If the text of the button is the standard one ("Cancel") then use the built-in
+  // button, otherwise create an own command link.
+  std::wstring cancel_text = base::string_to_wstring(buttonCancel);
+  if (!cancel_text.empty())
   {
     config.dwFlags |= TDF_ALLOW_DIALOG_CANCELLATION;
-    config.dwCommonButtons = TDCBF_CANCEL_BUTTON;
+    if (base::tolower(buttonCancel) == "cancel")
+      config.dwCommonButtons = TDCBF_CANCEL_BUTTON;
+    else
+    {
+      buttons[buttonCount].nButtonID = IDCANCEL;
+      buttons[buttonCount].pszButtonText = cancel_text.c_str();
+      buttonCount++;
+    }
   }
 
   std::wstring other_text = base::string_to_wstring(buttonOther);
@@ -176,7 +160,7 @@ mforms::DialogResult CustomMessageBox::ShowVistaStyle(const std::string& title,
     buttonCount++;
   }
 
-  // If we have more than one button to show then enable command links.
+  // If we have more than one normal alternatives to show then enable command links.
   if (buttonCount > 1)
     config.dwFlags |= TDF_USE_COMMAND_LINKS;
 
@@ -493,8 +477,6 @@ mforms::DialogResult CustomMessageBox::Show(const std::string& title, const std:
       else
         image= Image::FromFile("images/ui/message_confirm.png");
     
-    bool checked;
-
     mforms::Utilities::enter_modal_loop();
     mforms::DialogResult result = CustomMessageBox::ShowTraditionalStyle(boxTitle, boxText, image,
       ok_text, cancel_text, other_text, checkbox_text, checked);
@@ -506,7 +488,7 @@ mforms::DialogResult CustomMessageBox::Show(const std::string& title, const std:
 //--------------------------------------------------------------------------------------------------
 
 Windows::Forms::DialogResult CustomMessageBox::Show(MessageType type, String^ title, String^ text,  
-  String^ buttonOK, String^ buttonCancel, String^ buttonOther, String^ checkbox, bool& checked)
+  String^ buttonOK, String^ buttonCancel, String^ buttonOther, String^ checkbox, [Out] bool% checked)
 {
   mforms::DialogResult result;
 
@@ -527,11 +509,13 @@ Windows::Forms::DialogResult CustomMessageBox::Show(MessageType type, String^ ti
       mainIcon = TD_INFORMATION_ICON;
       break;
     }
+    bool isChecked = false;
     result = ShowVistaStyle(
       NativeToCppString(title), NativeToCppString(text), mainIcon, NativeToCppString(buttonOK),
       NativeToCppString(buttonCancel), NativeToCppString(buttonOther), NativeToCppString(checkbox),
-      checked
+      isChecked
     );
+    checked = isChecked;
     mforms::Utilities::leave_modal_loop();
   }
   else
@@ -550,11 +534,11 @@ Windows::Forms::DialogResult CustomMessageBox::Show(MessageType type, String^ ti
       break;
     }
 
-    bool checked;
-
     mforms::Utilities::enter_modal_loop();
+    bool isChecked = false;
     result = CustomMessageBox::ShowTraditionalStyle(title, text, image, buttonOK, buttonCancel,
-      buttonOther, checkbox, checked);
+      buttonOther, checkbox, isChecked);
+    checked = isChecked;
     mforms::Utilities::leave_modal_loop();
   }
 
@@ -579,7 +563,7 @@ Windows::Forms::DialogResult CustomMessageBox::Show(MessageType type, String^ ti
 Windows::Forms::DialogResult CustomMessageBox::Show(MessageType type, String^ title, String^ text,
   String^ buttonOK)
 {
-  bool checked;
+  bool checked = false;
   return Show(type, title, text, buttonOK, "", "", "", checked);
 }
 
@@ -587,19 +571,35 @@ Windows::Forms::DialogResult CustomMessageBox::Show(MessageType type, String^ ti
 
 delegate InvokationResult^ RunSlotDelegate();
 
-void* DispatchControl::RunOnMainThread(const boost::function<void* ()>& slot)
+void* DispatchControl::RunOnMainThread(const boost::function<void* ()>& slot, bool wait)
 {
   if (InvokeRequired)
   {
-    _slot = &slot;
-    InvokationResult^ result = (InvokationResult^) Invoke(
-      gcnew RunSlotDelegate(this, &DispatchControl::RunSlot));
-    return result->Result;
+    _slot = new boost::function<void* ()>(slot);
+    if (wait)
+    {
+      InvokationResult^ result =
+        (InvokationResult^) Invoke(gcnew RunSlotDelegate(this, &DispatchControl::RunSlot));
+      return result->Result;
+    }
+    else
+    {
+      BeginInvoke( gcnew RunSlotDelegate(this, &DispatchControl::RunSlot));
+
+      // If we don't wait for the result we cannot return it.
+      return NULL;
+    }
   }
 
   return slot();
 }
 
+
+DispatchControl::~DispatchControl()
+{
+  delete _slot; 
+}
+
 //--------------------------------------------------------------------------------------------------
 
 /**
@@ -621,6 +621,8 @@ UtilitiesImpl::UtilitiesImpl()
 int UtilitiesImpl::show_message(const string &title, const string &text, const string &ok, 
   const string &cancel, const string &other)
 {
+  hide_wait_message();
+
   bool checked;
   return CustomMessageBox::Show(title, text, TD_INFORMATION_ICON, ok, cancel, other, "", checked);
 }
@@ -630,6 +632,8 @@ int UtilitiesImpl::show_message(const string &title, const string &text, const s
 int UtilitiesImpl::show_error(const string &title, const string &text, const string &ok, 
   const string &cancel, const string &other)
 {
+  hide_wait_message();
+
   bool checked;
   return CustomMessageBox::Show(title, text, TD_ERROR_ICON, ok, cancel, other, "", checked);
 }
@@ -639,6 +643,8 @@ int UtilitiesImpl::show_error(const string &title, const string &text, const str
 int UtilitiesImpl::show_warning(const string &title, const string &text, const string &ok, 
   const string &cancel, const string &other)
 {
+  hide_wait_message();
+
   bool checked;
   return CustomMessageBox::Show(title, text, TD_WARNING_ICON, ok, cancel, other, "", checked);
 }
@@ -649,10 +655,12 @@ int UtilitiesImpl::show_message_with_checkbox(const string &title, const string
   const string &ok, const string &cancel, const string &other, const std::string &checkbox_text,
   bool &isChecked)
 {
+  hide_wait_message();
+
   std::string checkboxText = (checkbox_text.size() > 0) ? checkbox_text :
     _("Don't show this message again");
   return CustomMessageBox::Show(title, text, TD_INFORMATION_ICON, ok, cancel, other, checkboxText,
- isChecked);
+    isChecked);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -662,10 +670,7 @@ int UtilitiesImpl::show_message_with_checkbox(const string &title, const string
  */
 void UtilitiesImpl::show_wait_message(const string &title, const string &text)
 {
-  if (Thread::CurrentThread->Name != "mainthread") // This name was set in Program.cs.
-    throw std::runtime_error("Internal error: wait box creation must be done on the main thread.");
-
-  HUDForm::Instance->Show(CppStringToNative(title), CppStringToNative(text), true);
+  HUDForm::Show(CppStringToNative(title), CppStringToNative(text), true);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -675,13 +680,14 @@ void UtilitiesImpl::show_wait_message(const string &title, const string &text)
  */
 bool UtilitiesImpl::hide_wait_message()
 {
-  bool result= HUDForm::Instance->Visible;
+  bool result= HUDForm::IsVisible;
   if (result)
-    HUDForm::Instance->Hide();
+    HUDForm::Finished();
   return result;
 }
 
 //--------------------------------------------------------------------------------------------------
+
 ref class CallSlotDelegate
 {
 public:
@@ -696,13 +702,25 @@ public:
 //-------------------------------------------------------------------------------------------------
 
 bool UtilitiesImpl::run_cancelable_wait_message(const string &title, const string &text, 
-                                                const boost::function<void ()> &start_task, const boost::function<bool ()> &cancel_slot)
+                                                const boost::function<void ()> &start_task,
+                                                const boost::function<bool ()> &cancel_slot)
 {
   CallSlotDelegate ^caller = gcnew CallSlotDelegate(&cancel_slot);
 
-  BusyForm::CancelDelegate ^deleg = gcnew BusyForm::CancelDelegate(caller, &CallSlotDelegate::call_slot);
-  start_task();
-  return BusyForm::ShowModal(CppStringToNative(title), CppStringToNative(text), deleg) == Windows::Forms::DialogResult::OK;
+  HUDForm::CancelDelegate ^deleg = gcnew HUDForm::CancelDelegate(caller, &CallSlotDelegate::call_slot);
+
+  // There is a possible race condition here
+  // if the task executed by start_task() finishes fast enough, it could
+  // result in a call to stop_cancelable_wait_message() before the below
+  // ShowModal. That will cause the modal window will stay up forever.
+
+  if (start_task)
+    start_task();
+
+  Windows::Forms::DialogResult result = HUDForm::ShowModal(CppStringToNative(title), CppStringToNative(text), true, deleg);
+
+  // Abort is used if the window was forcibly closed (e.g. when showing another dialog, like password query).
+  return (result == Windows::Forms::DialogResult::OK) || (result == Windows::Forms::DialogResult::Abort);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -713,7 +731,12 @@ bool UtilitiesImpl::run_cancelable_wait_message(const string &title, const strin
  */
 void UtilitiesImpl::stop_cancelable_wait_message()
 {
-  BusyForm::Finished();
+  // Wait until the HUDForm is visible (ie ShowModal is called) before closing it
+  // to avoid race condition. This could deadlock only if start_task() is not a 
+  // thread, which would be a problem in itself.
+  while (!HUDForm::IsVisible)
+    ;
+  HUDForm::Finished();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -726,7 +749,37 @@ void UtilitiesImpl::stop_cancelable_wait_message()
 void UtilitiesImpl::set_clipboard_text(const string &content)
 {
   if (!content.empty())
-    Clipboard::SetText(CppStringToNative(content));
+  {
+    String^ text = CppStringToNative(content);
+    try
+    {
+      Clipboard::Clear();
+
+      // If the clipboard cannot be opened because another application uses it at this very moment
+      // we try a second time with built-in retries. If that still fails we inform the user.
+      Clipboard::SetText(text);
+    }
+    catch (ExternalException ^)
+    {
+      try
+      {
+        Clipboard::SetDataObject(text, false, 5, 200);
+      }
+      catch (ExternalException ^e)
+      {
+        String^ msg = e->Message + Environment::NewLine + Environment::NewLine;
+        msg += "Caused by:" + Environment::NewLine;
+
+        HWND window = GetOpenClipboardWindow();
+        TCHAR window_text[501];
+        GetWindowText(window, window_text, 500);
+        String ^converted_text = gcnew String(window_text);
+        msg += converted_text;
+
+        CustomMessageBox::Show(MessageType::MessageError, _("Error while copying to clipboard"), msg, "Close");
+      }
+    }
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -751,21 +804,21 @@ string UtilitiesImpl::get_clipboard_text()
 /**
  * Returns platform specific user folders, e.g. for the desktop, the user's documents etc.
  */
-string UtilitiesImpl::get_special_folder(FolderType type)
+string UtilitiesImpl::get_special_folder(mforms::FolderType type)
 {
   Environment::SpecialFolder special_folder;
   switch (type)
   {
-  case Desktop:
+  case mforms::Desktop:
     special_folder = Environment::SpecialFolder::DesktopDirectory;
     break;
-  case ApplicationData:
+  case mforms::ApplicationData:
     special_folder = Environment::SpecialFolder::ApplicationData;
     break;
-  case WinProgramFiles:
+  case mforms::WinProgramFiles:
     special_folder = Environment::SpecialFolder::ProgramFiles;
     break;
-  case WinProgramFilesX86:
+  case mforms::WinProgramFilesX86:
     special_folder = Environment::SpecialFolder::ProgramFilesX86;
     break;
   default: // Documents
@@ -836,7 +889,7 @@ typedef std::map<std::string, std::string>::iterator PasswordIterator;
 void UtilitiesImpl::load_passwords()
 {
   // Load password cache from disk. Don't throw an error if the cache file doesn't exist yet, though.
-  std::string file= get_special_folder(ApplicationData) + "/MySQL/Workbench/workbench_user_data.dat";
+  std::string file= get_special_folder(mforms::ApplicationData) + "/MySQL/Workbench/workbench_user_data.dat";
   if (g_file_test(file.c_str(), G_FILE_TEST_EXISTS))
   {
     gchar* content;
@@ -915,7 +968,7 @@ void UtilitiesImpl::unload_passwords(bool store)
       }
 
       // Now write the encrypted data to file.
-      std::string file= get_special_folder(ApplicationData) + "/MySQL/Workbench/workbench_user_data.dat";
+      std::string file= get_special_folder(mforms::ApplicationData) + "/MySQL/Workbench/workbench_user_data.dat";
       GError* error= NULL;
       bool result= g_file_set_contents(file.c_str(), (gchar*) data_out.pbData, data_out.cbData, &error) == TRUE;
 
@@ -1017,9 +1070,19 @@ void UtilitiesImpl::forget_password(const std::string &service, const std::strin
 
 //--------------------------------------------------------------------------------------------------
 
-void* UtilitiesImpl::perform_from_main_thread(const boost::function<void* ()>& slot)
+void* UtilitiesImpl::perform_from_main_thread(const boost::function<void* ()>& slot, bool wait)
 {
-  return _dispatcher->RunOnMainThread(slot);
+  return _dispatcher->RunOnMainThread(slot, wait);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Returns the main form of the application.
+ */
+Windows::Forms::Form^ UtilitiesImpl::get_mainform()
+{
+  return Application::OpenForms["MainForm"];
 }
 
 //--------------------------------------------------------------------------------------------------
