@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,10 +18,10 @@
  */
 
 #include "stdafx.h"
-#include "wf_view.h"
 
 using namespace Windows::Forms;
 using namespace System::Drawing;
+using namespace System::IO;
 
 using namespace MySQL;
 using namespace MySQL::Forms;
@@ -29,14 +29,16 @@ using namespace MySQL::Utilities;
 using namespace MySQL::Controls;
 
 using namespace Aga::Controls::Tree;
-using namespace ScintillaNet;
+
+DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_WRAPPER)
 
 //----------------- ViewImpl ----------------------------------------------------------------------
 
-ViewImpl::ViewImpl(::mforms::View *view)
+ViewImpl::ViewImpl(mforms::View *view)
   : ObjectImpl(view)
 {
-  tooltip= nullptr;
+  tooltip = nullptr;
+  backgroundImage = nullptr;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -65,7 +67,8 @@ bool ViewImpl::use_min_width_for_layout(Control^ control)
     System::Type^ type = control->GetType();
     needsMinWidth= (type == TabControl::typeid) || (type == Panel::typeid) || (type == ComboBox::typeid)
       || (type == TreeViewAdv::typeid) || (type == ListBox::typeid) || (type == TextBox::typeid)
-      || (type == SplitContainer::typeid) || (type == Scintilla::typeid)
+      || (type == SplitContainer::typeid) || (type == ScintillaControl::typeid)
+      || (type == Windows::Forms::WebBrowser::typeid)
       
       // TODO: investigate if we can remove this and override GetPreferredSize in FlatTabControl instead.
       || (type == FlatTabControl::typeid);
@@ -84,7 +87,8 @@ bool ViewImpl::use_min_height_for_layout(Control^ control)
   {
     System::Type^ type = control->GetType();
     needsMinHeight= (type == TabControl::typeid) || (type == Panel::typeid) || (type == ListBox::typeid)
-      || (type == TreeViewAdv::typeid) || (type == SplitContainer::typeid) || (type == Scintilla::typeid)
+      || (type == TreeViewAdv::typeid) || (type == SplitContainer::typeid) || (type == ScintillaControl::typeid)
+      || (type == Windows::Forms::WebBrowser::typeid)
 
     // TODO: investigate if we can remove this and override GetPreferredSize in FlatTabControl instead.
     || (type == FlatTabControl::typeid);
@@ -197,7 +201,7 @@ void ViewImpl::set_layout_dirty(Control^ control, bool value)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::destroy(::mforms::View *self)
+void ViewImpl::destroy(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   delete view;
@@ -205,7 +209,7 @@ void ViewImpl::destroy(::mforms::View *self)
 
 //--------------------------------------------------------------------------------------------------
 
-void ViewImpl::show(::mforms::View *self, bool show)
+void ViewImpl::show(mforms::View *self, bool show)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
 
@@ -226,7 +230,7 @@ void ViewImpl::show(::mforms::View *self, bool show)
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_width(::mforms::View *self)
+int ViewImpl::get_width(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -236,7 +240,7 @@ int ViewImpl::get_width(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_height(::mforms::View *self)
+int ViewImpl::get_height(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -246,7 +250,7 @@ int ViewImpl::get_height(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_preferred_width(::mforms::View *self)
+int ViewImpl::get_preferred_width(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -258,7 +262,7 @@ int ViewImpl::get_preferred_width(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_preferred_height(::mforms::View *self)
+int ViewImpl::get_preferred_height(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -270,7 +274,7 @@ int ViewImpl::get_preferred_height(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_x(::mforms::View *self)
+int ViewImpl::get_x(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -282,7 +286,7 @@ int ViewImpl::get_x(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-int ViewImpl::get_y(::mforms::View *self)
+int ViewImpl::get_y(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -294,7 +298,7 @@ int ViewImpl::get_y(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_size(::mforms::View *self, int w, int h)
+void ViewImpl::set_size(mforms::View *self, int w, int h)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -303,19 +307,19 @@ void ViewImpl::set_size(::mforms::View *self, int w, int h)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_padding(::mforms::View *self, const MySQL::Geometry::Padding& padding)
+void ViewImpl::set_padding(mforms::View *self, int left, int top, int right, int bottom)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
   {
     self->set_layout_dirty(true);
-    view->set_padding(padding);
+    view->set_padding(left, top, right, bottom);
   }
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_position(::mforms::View *self, int x, int y)
+void ViewImpl::set_position(mforms::View *self, int x, int y)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -328,15 +332,15 @@ void ViewImpl::set_position(::mforms::View *self, int x, int y)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::client_to_screen(::mforms::View *self, MySQL::Geometry::Point& position)
+void ViewImpl::client_to_screen(mforms::View *self, int& x, int& y)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
   {
-    System::Drawing::Point location= System::Drawing::Point((int) position.x, (int) position.y);
+    System::Drawing::Point location= System::Drawing::Point(x, y);
     location= view->get_control<Control>()->PointToScreen(location);
-    position.x= location.X;
-    position.y= location.Y;
+    x = location.X;
+    y = location.Y;
   }
 }
 
@@ -364,7 +368,7 @@ void ViewImpl::relayout()
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_enabled(::mforms::View *self, bool flag)
+void ViewImpl::set_enabled(mforms::View *self, bool flag)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -373,7 +377,7 @@ void ViewImpl::set_enabled(::mforms::View *self, bool flag)
 
 //-------------------------------------------------------------------------------------------------
 
-bool ViewImpl::is_enabled(::mforms::View *self)
+bool ViewImpl::is_enabled(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -383,14 +387,14 @@ bool ViewImpl::is_enabled(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-View *ViewImpl::find_subview(::mforms::View *self, std::string &name)
+View *ViewImpl::find_subview(mforms::View *self, std::string &name)
 {
   return 0;
 }
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_name(::mforms::View *self, const std::string& text)
+void ViewImpl::set_name(mforms::View *self, const std::string& text)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -399,7 +403,7 @@ void ViewImpl::set_name(::mforms::View *self, const std::string& text)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::relayout(::mforms::View *self)
+void ViewImpl::relayout(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -408,7 +412,7 @@ void ViewImpl::relayout(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_needs_repaint(::mforms::View *self)
+void ViewImpl::set_needs_repaint(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -417,7 +421,7 @@ void ViewImpl::set_needs_repaint(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::suspend_layout(::mforms::View *self, bool flag)
+void ViewImpl::suspend_layout(mforms::View *self, bool flag)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -473,17 +477,30 @@ std::string ViewImpl::get_back_color(mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_back_image(mforms::View *self, const std::string &path, mforms::ImageLayout layout)
+void ViewImpl::set_back_image(mforms::View *self, const std::string &path, mforms::Alignment align)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
-    view->set_back_image(CppStringToNative(path), layout);
+    view->set_back_image(CppStringToNative(path), align);
 }
 
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_tooltip(::mforms::View *self, const std::string& text)
+void ViewImpl::focus(mforms::View *self)
+{
+  ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
+  if (view != nullptr)
+  {
+    Control^ control = view->get_control();
+    if (control->CanFocus)
+      control->Focus();
+  }
+}
+
+//-------------------------------------------------------------------------------------------------
+
+void ViewImpl::set_tooltip(mforms::View *self, const std::string& text)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -492,7 +509,7 @@ void ViewImpl::set_tooltip(::mforms::View *self, const std::string& text)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_font(::mforms::View *self, const std::string& fontDescription)
+void ViewImpl::set_font(mforms::View *self, const std::string& fontDescription)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -501,7 +518,7 @@ void ViewImpl::set_font(::mforms::View *self, const std::string& fontDescription
 
 //-------------------------------------------------------------------------------------------------
 
-bool ViewImpl::is_shown(::mforms::View *self)
+bool ViewImpl::is_shown(mforms::View *self)
 {
   ViewImpl^ view= (ViewImpl^)ObjectImpl::FromUnmanaged(self);
   if (view != nullptr)
@@ -511,6 +528,17 @@ bool ViewImpl::is_shown(::mforms::View *self)
 
 //-------------------------------------------------------------------------------------------------
 
+void ViewImpl::initialize()
+{
+  Control^ control= get_control<Control>();
+
+  // Can be null, e.g. for non-control objects like dialogs.
+  if (control != nullptr)
+    control->Resize += gcnew EventHandler(this, &ViewImpl::Resize);
+}
+
+//-------------------------------------------------------------------------------------------------
+
 void ViewImpl::set_size(int width, int height)
 {
   Control^ control= get_control<Control>();
@@ -527,10 +555,10 @@ void ViewImpl::set_size(int width, int height)
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_padding(const Geometry::Padding& padding)
+void ViewImpl::set_padding(int left, int top, int right, int bottom)
 {
   Control^ control= get_control<Control>();
-  control->Padding = Windows::Forms::Padding(padding.left, padding.top, padding.right, padding.bottom);
+  control->Padding = Windows::Forms::Padding(left, top, right, bottom);
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -570,15 +598,17 @@ void ViewImpl::set_font(const std::string& fontDescription)
     style = (FontStyle) (style | FontStyle::Bold);
   if (italic)
     style = (FontStyle) (style | FontStyle::Italic);
-  get_control<Control>()->Font = ControlUtilities::GetFont(CppStringToNativeRaw(font),
-    (float) size, style);
-}
 
-//-------------------------------------------------------------------------------------------------
-
-Windows::Forms::Control^ ViewImpl::GetControl()
-{
-  return get_control<Control>();
+  try
+  {
+    get_control<Control>()->Font = ControlUtilities::GetFont(CppStringToNativeRaw(font),
+      (float) size, style);
+  }
+  catch (System::ArgumentException^ e)
+  {
+    // Argument exception pops up when the system cannot find the Regular font style (corrupt font).
+    log_error("ViewImpl::set_font failed. %s\n", e->Message);
+  }
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -611,31 +641,14 @@ String^ ViewImpl::get_back_color()
 
 //-------------------------------------------------------------------------------------------------
 
-void ViewImpl::set_back_image(String ^path, mforms::ImageLayout layout)
+void ViewImpl::set_back_image(String ^path, mforms::Alignment layout)
 {
-  Windows::Forms::ImageLayout native_layout;
-  switch (layout)
+  path = AppImpl::get_image_path(path);
+  if (File::Exists(path))
   {
-    case mforms::LayoutTile:
-      native_layout= Windows::Forms::ImageLayout::Tile;
-      break;
-    case mforms::LayoutCenter:
-      native_layout= Windows::Forms::ImageLayout::Center;
-      break;
-    case mforms::LayoutStretch:
-      native_layout= Windows::Forms::ImageLayout::Stretch;
-      break;
-    case mforms::LayoutZoom:
-      native_layout= Windows::Forms::ImageLayout::Zoom;
-      break;
-    default:
-        native_layout= Windows::Forms::ImageLayout::None;
+    backgroundImage = Image::FromFile(path, true);
+    backgroundImageAlignment = layout;
   }
-
-  // An exception is raised if the image cannot be found.
-  Image^ image= Image::FromFile(path, true);
-  get_control<Control>()->BackgroundImage= image;
-  get_control<Control>()->BackgroundImageLayout= native_layout;
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -659,6 +672,7 @@ void ViewImpl::resize_with_docking(Control^ control, System::Drawing::Size& size
       size.Width= control->Size.Width;
 
   control->Size= size;
+  set_layout_dirty(control, true);
 }
 
 //-------------------------------------------------------------------------------------------------
@@ -686,3 +700,66 @@ void ViewImpl::flush_events(::mforms::View *)
 }
 
 //-------------------------------------------------------------------------------------------------
+
+void ViewImpl::Resize(System::Object ^sender, EventArgs ^e)
+{
+  Control^ control = (Control^)sender;
+
+  if (control->Tag != nullptr)
+  {
+    mforms::View* view = ViewImpl::get_backend_control<mforms::View>(control);
+    if (!view->is_destroying())
+      (*view->signal_resized())();
+  }
+}
+
+//-------------------------------------------------------------------------------------------------
+
+/**
+ * Draws the background image with the earlier set layout, if there's one.
+ */
+void ViewImpl::DrawBackground(PaintEventArgs ^args)
+{
+  if (backgroundImage != nullptr)
+  {
+    int left = 0;
+    int top = 0;
+    Control ^control = get_control();
+
+    // Horizontal alignment.
+    switch (backgroundImageAlignment)
+    {
+    case mforms::BottomCenter:
+    case mforms::MiddleCenter:
+    case mforms::TopCenter:
+      left = (control->Width - backgroundImage->Width) / 2;
+      break;
+
+    case mforms::BottomRight:
+    case mforms::MiddleRight:
+    case mforms::TopRight:
+      left = control->Width - backgroundImage->Width;
+      break;
+    }
+
+    // Vertical alignment.
+    switch (backgroundImageAlignment)
+    {
+    case mforms::MiddleLeft:
+    case mforms::MiddleCenter:
+    case mforms::MiddleRight:
+      top = (control->Height - backgroundImage->Height) / 2;
+      break;
+
+    case mforms::BottomLeft:
+    case mforms::BottomCenter:
+    case mforms::BottomRight:
+      top = control->Height - backgroundImage->Height;
+      break;
+    }
+
+    args->Graphics->DrawImage(backgroundImage, Point(left, top));
+  }
+}
+
+//-------------------------------------------------------------------------------------------------
