@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,13 +24,14 @@
 #include "workbench/wb_context.h"
 
 #include "wf_appview.h"
+#include "wf_dockingpoint.h"
 
 #pragma make_public(mforms::App)
 #pragma make_public(wb::WBFrontendCallbacks)
 
 using namespace System;
 using namespace System::Collections::Generic;
-using namespace System::Runtime::InteropServices;
+using namespace System::Runtime::InteropServices; // Needed for the [Out] keyword.
 
 using namespace MySQL::Controls;
 using namespace MySQL::Grt;
@@ -41,10 +42,6 @@ namespace MySQL {
 
     public enum class AppCommand
     {
-      AppDockView,
-      AppSelectView,
-      AppUndockView,
-      AppSetViewTitle,
       AppQuit,
       AppGetResourcePath,
       AppSetStatusText,
@@ -58,6 +55,7 @@ namespace MySQL {
       RefreshNothing              = wb::RefreshNothing,
       RefreshSchemaList           = wb::RefreshSchemaList,
       RefreshSchema               = wb::RefreshSchema,
+      RefreshSchemaNoReload       = wb::RefreshSchemaNoReload,
       RefreshNewDiagram           = wb::RefreshNewDiagram,
       RefreshSelection            = wb::RefreshSelection,
       RefreshCloseEditor          = wb::RefreshCloseEditor,
@@ -65,7 +63,6 @@ namespace MySQL {
       RefreshNewModel             = wb::RefreshNewModel,
       RefreshOverviewNodeInfo     = wb::RefreshOverviewNodeInfo,
       RefreshOverviewNodeChildren = wb::RefreshOverviewNodeChildren,
-      RefreshViewName             = wb::RefreshViewName,
       RefreshDocument             = wb::RefreshDocument,
       RefreshCloseDocument        = wb::RefreshCloseDocument,
       RefreshZoom                 = wb::RefreshZoom,
@@ -99,17 +96,17 @@ namespace MySQL {
       delegate void VoidIntPtrDelegate(IntPtr p2);
       delegate void VoidRefreshTypeStringIntPtrDelegate(RefreshType refresh, String^ str, IntPtr ptr);
       delegate String^ StringAppCommandAppViewStringDelegate(AppCommand, MySQL::Forms::AppViewImpl^ view, String^ str);
-
+      
     private:
       ::wb::WBFrontendCallbacks* _callbacks;
       GCHandle _cb_handle; // Keeps an unmanaged reference to this class for mforms::App callbacks.
+      DockingPointDelegateWrapper^ _docking_delegate;
 
       // ----------------------------------------------------------------------------
       // Wrapper delegates used by the C++.net wrapper
 
       void set_show_file_dialog(StrStrStrStrDelegate^ dt);
       void set_show_status_text(VoidStrDelegate^ dt);
-      void set_show_progress(BoolStrStrFloatDelegate^ dt);
       void set_request_input(BoolStrIntStrPtrDelegate^ dt);
       void set_create_diagram(CanvasViewStringStringIntPtrDelegate^ dt);
       void set_destroy_view(VoidCanvasViewDelegate^ dt);
@@ -273,17 +270,13 @@ namespace MySQL {
     protected:
       static Object^ GetFromFixedId(IntPtr ip);
 
-      static void dock_view(mforms::App *app, mforms::AppView *view, const std::string &position);
-      static bool select_view(mforms::App *app, const std::string &identifier);
-      static void undock_view(mforms::App *app, mforms::AppView *view);
-      static void set_view_title(mforms::App *app, mforms::AppView *view, const std::string &title);
       static std::string get_resource_path(mforms::App *app, const std::string &file);
       static void set_status_text(mforms::App *app, const std::string &text);
       static std::string get_bounds(mforms::App *app);
       static int enter_event_loop(mforms::App *app, float max_wait_time);
       static void exit_event_loop(mforms::App *app, int ret_code);
 
-      static MySQL::Drawing::Color get_system_color(mforms::SystemColor type);
+      static base::Color get_system_color(mforms::SystemColor type);
 
     public:
       // TODO: historically grown separation of certain application commands and callbacks.
@@ -304,27 +297,25 @@ namespace MySQL {
         VoidBoolDelegate^ lock_gui,
         VoidStrDelegate^ perform_command,
         VoidDelegate^ quit_application,
-        StringAppCommandAppViewStringDelegate^ app_command);
+        StringAppCommandAppViewStringDelegate^ app_command,
+        DockingPointDelegateWrapper^ docking_delegate);
 
       ~AppImpl();
 
       ::wb::WBFrontendCallbacks* get_callbacks();
+      static String^ get_image_path(String ^path);
 
       static void init(Manager ^mgr)
       {
         ::mforms::ControlFactory *f= ::mforms::ControlFactory::get_instance();
 
-        DEF_CALLBACK3(void, ::mforms::App*, ::mforms::AppView*, const std::string&, mgr, f->_app_impl, AppImpl, dock_view);
-        DEF_CALLBACK2(bool, ::mforms::App*, const std::string&, mgr, f->_app_impl, AppImpl, select_view);
-        DEF_CALLBACK2(void, ::mforms::App*, ::mforms::AppView*, mgr, f->_app_impl, AppImpl, undock_view);
-        DEF_CALLBACK3(void, ::mforms::App*, ::mforms::AppView*, const std::string&, mgr, f->_app_impl, AppImpl, set_view_title);
         DEF_CALLBACK2(std::string, ::mforms::App*, const std::string&, mgr, f->_app_impl, AppImpl, get_resource_path);
         DEF_CALLBACK2(void, ::mforms::App*, const std::string&, mgr, f->_app_impl, AppImpl, set_status_text);
         DEF_CALLBACK1(std::string, ::mforms::App*, mgr, f->_app_impl, AppImpl, get_bounds);
         DEF_CALLBACK2(int, ::mforms::App*, float, mgr, f->_app_impl, AppImpl, enter_event_loop);
         DEF_CALLBACK2(void, ::mforms::App*, int, mgr, f->_app_impl, AppImpl, exit_event_loop);
 
-        DEF_CALLBACK1(MySQL::Drawing::Color, mforms::SystemColor, mgr, f->_app_impl, AppImpl, get_system_color);
+        DEF_CALLBACK1(base::Color, mforms::SystemColor, mgr, f->_app_impl, AppImpl, get_system_color);
       }
     };
   };
