@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -51,8 +51,9 @@ namespace MySQL {
       TabDocument^ GetHost();
       MenuStrip^ GetMenuBar();
       ToolStrip^ GetToolBar();
+      String^ GetTitle();
 
-      void DocumentClosing(System::Object^ sender, TabClosingEventArgs^ e);
+      bool DocumentClosing();
 
       static void init(Manager ^mgr)
       {
@@ -74,6 +75,7 @@ namespace MySQL {
 
       AppViewImpl^ GetAppView();
       String^ GetAppViewIdentifier();
+
       mforms::AppView *get_unmanaged_object();
 
     };
