@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -383,7 +383,7 @@ namespace Forms {
   public ref class ObjectImpl
   {
   private:
-    GCHandle m_gch;
+    GCHandle _gc_handle;
 
     // These members represent the .NET class for which this wrapper class was created.
     // Neither of these classes have a common ancestor which has a Tag property (which we need
@@ -395,39 +395,22 @@ namespace Forms {
     // Returns a fixed pointer to this object that will not be modified by the GC
     IntPtr GetFixedId()
     {
-      if (!m_gch.IsAllocated)
-         m_gch = GCHandle::Alloc(this);
-       return GCHandle::ToIntPtr(m_gch);
+      if (!_gc_handle.IsAllocated)
+         _gc_handle = GCHandle::Alloc(this);
+       return GCHandle::ToIntPtr(_gc_handle);
     }
 
     // Needs to be called when destroying the object
     void ReleaseHandle()
     {
-      m_gch.Free();
+      _gc_handle.Free();
     }
 
   protected:
-    ObjectImpl(::mforms::Object *object)
-    {
-      control= nullptr;
-      dialog= nullptr;
-      toolStripItem = nullptr;
+    ObjectImpl(::mforms::Object *object);
+    virtual ~ObjectImpl();
 
-      // Get a fixed pointer to this object.
-      IntPtr ip = this->GetFixedId();
-
-      // Store this pointer in the backend object as mapping from backend to wrapper.
-      object->set_data((void*)(intptr_t) ip);
-    }
-
-    virtual ~ObjectImpl()
-    {
-      ReleaseHandle();
-
-      delete control;
-      delete dialog;
-      delete toolStripItem;
-    }
+    virtual void initialize() {};
 
     // Returns the object based on the fixed pointer retrieved by GetFixedId()
     static ObjectImpl^ GetFromFixedId(IntPtr ip)
@@ -462,6 +445,8 @@ namespace Forms {
       control->Size= control->MinimumSize;
       control->Font= gcnew Font(DEFAULT_FONT, DEFAULT_FONT_SIZE, FontStyle::Regular, GraphicsUnit::Pixel);
 
+      wrapper->initialize();
+
       return control;
     }
 
@@ -474,6 +459,7 @@ namespace Forms {
       ToolStripItem^ item= gcnew T();
       item->Tag= gcnew IntPtr(backend);
       wrapper->toolStripItem= item;
+      wrapper->initialize();
 
       return (T^) item;
     }
@@ -491,6 +477,7 @@ namespace Forms {
       T^ dialog= gcnew T();
       dialog->Tag= gcnew IntPtr(backend);
       wrapper->dialog= dialog;
+      wrapper->initialize();
 
       return dialog;
     }
@@ -536,6 +523,34 @@ namespace Forms {
         }
       }
     }
+
+    template<class T>
+    static T^ get_wrapper(Object ^object)
+    {
+      IntPtr ^reference = nullptr;
+      Control ^control= dynamic_cast<Control^>(object);
+      if (control != nullptr)
+        reference = (IntPtr^)control->Tag;
+      else
+      {
+        CommonDialog^ dialog= dynamic_cast<CommonDialog^>(object);
+        if (dialog != nullptr)
+          reference = (IntPtr^)dialog->Tag;
+        else
+        {
+          ToolStripItem^ item = dynamic_cast<ToolStripItem^>(object);
+          if (item != nullptr)
+            reference = (IntPtr^)item->Tag;
+        }
+      }
+
+      if (reference != nullptr)
+      {
+        mforms::Object *object = (mforms::Object*)reference->ToPointer();
+        return dynamic_cast<T^>(GetFromFixedId((IntPtr)(object->get_data_ptr())));
+      }
+      return nullptr;
+    }
   };
 };
 
