@@ -25,7 +25,6 @@
 #include "grt/tree_model.h"
 
 #include "mforms/grttreeview.h"
-#include "mforms/mforms.h"
 
 #include "wf_view.h"
 
@@ -41,6 +40,10 @@ using namespace Aga::Controls::Tree::NodeControls;
 namespace MySQL {
   namespace Forms {
 
+    /**
+     * The ColumnEntry struct is used as temporary cache for columns being added before a model
+     * was set.
+     */
     private ref struct ColumnEntry
     {
       NodeControl^ nodeControl;
@@ -125,6 +128,7 @@ namespace MySQL {
       static bool get_selected_node(mforms::GRTTreeView* self, bec::NodeId& node);
       static int get_selection(mforms::GRTTreeView* self, std::vector<bec::NodeId>& nodes);
       static void set_expanded(mforms::GRTTreeView* self, const bec::NodeId& node, bool expanded);
+      static void enable_asynch_loading(mforms::GRTTreeView* self, bool enabled);
 
       static void set_allow_multi_selection(mforms::GRTTreeView *self, bool flag);
       static void row_count_changed(mforms::GRTTreeView *self, const bec::NodeId& node, int old_count);
@@ -151,6 +155,7 @@ namespace MySQL {
         DEF_CALLBACK2(void, ::mforms::GRTTreeView*, bool, mgr, f->_grttreeview_impl, GRTTreeViewImpl, set_allow_multi_selection);
         DEF_CALLBACK3(void, ::mforms::GRTTreeView*, const bec::NodeId&, int, mgr, f->_grttreeview_impl, GRTTreeViewImpl, row_count_changed);
         DEF_CALLBACK3(void, ::mforms::GRTTreeView*, const bec::NodeId&, bool, mgr, f->_grttreeview_impl, GRTTreeViewImpl, set_expanded);
+        DEF_CALLBACK2(void, ::mforms::GRTTreeView*, bool, mgr, f->_grttreeview_impl, GRTTreeViewImpl, enable_asynch_loading);
       }
     };
   }
