@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,6 +36,7 @@ namespace MySQL {
       bool _auto_wrapping;
     public:
       WrapControlLabel();
+      ~WrapControlLabel();
 
       property bool AutoWrapping
       {
@@ -43,7 +44,7 @@ namespace MySQL {
         void set(bool value) { _auto_wrapping= value; };
       }
 
-      virtual Drawing::Size GetPreferredSize(Drawing::Size proposedSize) override;
+      virtual System::Drawing::Size GetPreferredSize(System::Drawing::Size proposedSize) override;
     };
 
     /**
