@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,8 +20,6 @@
 #ifndef _WF_TEXTENTRY_H_
 #define _WF_TEXTENTRY_H_
 
-#include <mforms/mforms.h>
-
 #include "wf_view.h"
 
 /**
@@ -35,14 +33,21 @@ namespace MySQL {
 
     public ref class TextEntryImpl : public ViewImpl
     {
+      String ^_placeholder;
+      bool _has_real_text;
+      bool _changing_text;
+
       static bool create(::mforms::TextEntry *self, ::mforms::TextEntryType type);
       static void set_text(::mforms::TextEntry *self, const std::string &text);
+      static void set_placeholder_text(::mforms::TextEntry *self, const std::string &text);
       static std::string get_text(::mforms::TextEntry *self);
       static void set_max_length(::mforms::TextEntry *self, int length);
       static void set_read_only(::mforms::TextEntry *self, bool flag);
 
-      static void change(Object^ sender, EventArgs^ args);
-      static void key_pressed(Object^ sender, KeyEventArgs^ args);
+      void change(Object^ sender, EventArgs^ args);
+      void key_pressed(Object^ sender, KeyEventArgs^ args);
+      void focus_in(Object^ sender, EventArgs^ args);
+      void focus_out(Object^ sender, EventArgs^ args);
     public:
       TextEntryImpl(::mforms::TextEntry *text);
 
@@ -52,6 +57,7 @@ namespace MySQL {
 
         DEF_CALLBACK2(bool, ::mforms::TextEntry*, ::mforms::TextEntryType, mgr, f->_textentry_impl, TextEntryImpl, create);
         DEF_CALLBACK2(void, ::mforms::TextEntry*, const std::string&, mgr, f->_textentry_impl, TextEntryImpl, set_text);
+        DEF_CALLBACK2(void, ::mforms::TextEntry*, const std::string&, mgr, f->_textentry_impl, TextEntryImpl, set_placeholder_text);
         DEF_CALLBACK1(std::string, ::mforms::TextEntry*, mgr, f->_textentry_impl, TextEntryImpl, get_text);
         DEF_CALLBACK2(void, ::mforms::TextEntry*, int, mgr, f->_textentry_impl, TextEntryImpl, set_max_length);
         DEF_CALLBACK2(void, ::mforms::TextEntry*, bool, mgr, f->_textentry_impl, TextEntryImpl, set_read_only);
