@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,9 +20,6 @@
 #ifndef _WF_VIEW_H_
 #define _WF_VIEW_H_
 
-#include <mforms/mforms.h>
-
-#include "wf_base.h"
 #include "wf_mforms.h"
 
 using namespace System;
@@ -38,6 +35,8 @@ namespace MySQL {
     {
     private:
       ToolTip^ tooltip;
+      Image^ backgroundImage;
+      mforms::Alignment backgroundImageAlignment;
     protected:
       ViewImpl(::mforms::View *view);
       ~ViewImpl();
@@ -51,9 +50,9 @@ namespace MySQL {
       static int get_x(::mforms::View *self);
       static int get_y(::mforms::View *self);
       static void set_size(::mforms::View *self, int w, int h);
-      static void set_padding(::mforms::View *self, const MySQL::Geometry::Padding& padding);
+      static void set_padding(::mforms::View *self, int left, int top, int right, int bottom);
       static void set_position(::mforms::View *self, int x, int y);
-      static void client_to_screen(::mforms::View *self, MySQL::Geometry::Point& position);
+      static void client_to_screen(::mforms::View *self, int& x, int& y);
 
       static void set_enabled(::mforms::View *self, bool flag);
       static bool is_enabled(::mforms::View *self);
@@ -69,11 +68,14 @@ namespace MySQL {
       static std::string get_front_color(mforms::View *self);
       static void set_back_color(mforms::View *self, const std::string &color);
       static std::string get_back_color(mforms::View *self);
-      static void set_back_image(mforms::View *self, const std::string &path, mforms::ImageLayout layout);
+      static void set_back_image(mforms::View *self, const std::string &path, mforms::Alignment alignment);
       static void flush_events(::mforms::View *self);
+      static void focus(::mforms::View *self);
+
+      virtual void initialize() override;
 
       virtual void set_size(int width, int height);
-      virtual void set_padding(const MySQL::Geometry::Padding& padding);
+      virtual void set_padding(int left, int top, int right, int bottom);
       virtual void set_position(int x, int y);
       virtual void set_tooltip(const std::string& text);
       virtual void set_font(const std::string& text);
@@ -83,10 +85,13 @@ namespace MySQL {
       String^ get_front_color();
       void set_back_color(String ^color);
       String^ get_back_color();
-      void set_back_image(String ^path, mforms::ImageLayout layout);
+      virtual void set_back_image(String ^path, mforms::Alignment alignment);
       
+      void Resize(System::Object^ sender, System::EventArgs^ e);
     public:
 
+      void DrawBackground(PaintEventArgs ^args);
+
       static void init(Manager ^mgr)
       {
         ::mforms::ControlFactory *f= ::mforms::ControlFactory::get_instance();
@@ -100,8 +105,8 @@ namespace MySQL {
         DEF_CALLBACK1(int, ::mforms::View*, mgr, f->_view_impl, ViewImpl, get_preferred_height);
         DEF_CALLBACK3(void, ::mforms::View*, int, int, mgr, f->_view_impl, ViewImpl, set_position);
         DEF_CALLBACK3(void, ::mforms::View*, int, int, mgr, f->_view_impl, ViewImpl, set_size);
-        DEF_CALLBACK2(void, ::mforms::View*, const MySQL::Geometry::Padding&, mgr, f->_view_impl, ViewImpl, set_padding);
-        DEF_CALLBACK2(void, ::mforms::View*, MySQL::Geometry::Point&, mgr, f->_view_impl, ViewImpl, client_to_screen);
+        DEF_CALLBACK5(void, ::mforms::View*, int, int, int, int, mgr, f->_view_impl, ViewImpl, set_padding);
+        DEF_CALLBACK3(void, ::mforms::View*, int&, int&, mgr, f->_view_impl, ViewImpl, client_to_screen);
 
         DEF_CALLBACK2(void, ::mforms::View*, bool, mgr, f->_view_impl, ViewImpl, show);
         DEF_CALLBACK2(void, ::mforms::View*, bool, mgr, f->_view_impl, ViewImpl, set_enabled);
@@ -117,13 +122,11 @@ namespace MySQL {
         DEF_CALLBACK1(std::string, ::mforms::View*, mgr, f->_view_impl, ViewImpl, get_front_color);
         DEF_CALLBACK2(void, ::mforms::View*, const std::string&, mgr, f->_view_impl, ViewImpl, set_back_color);
         DEF_CALLBACK1(std::string, ::mforms::View*, mgr, f->_view_impl, ViewImpl, get_back_color);
-        DEF_CALLBACK3(void, ::mforms::View*, const std::string&, ::mforms::ImageLayout, mgr, f->_view_impl, ViewImpl, set_back_image);
+        DEF_CALLBACK3(void, ::mforms::View*, const std::string&, ::mforms::Alignment, mgr, f->_view_impl, ViewImpl, set_back_image);
         DEF_CALLBACK1(void, ::mforms::View*, mgr, f->_view_impl, ViewImpl, flush_events);
+        DEF_CALLBACK1(void, ::mforms::View*, mgr, f->_view_impl, ViewImpl, focus);
       }
 
-      // For integration with native code.
-      Windows::Forms::Control^ GetControl();
-
       // Some utility functions.
       static bool use_min_width_for_layout(Control^ control);
       static bool use_min_height_for_layout(Control^ control);
