@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,8 +20,6 @@
 #ifndef _WF_WIZARD_H_
 #define _WF_WIZARD_H_
 
-#include <mforms/mforms.h>
-
 #include "mforms/wizard.h"
 #include "wf_base.h"
 #include "wf_mforms.h"
@@ -48,11 +46,9 @@ namespace MySQL {
     /**
      * Wrapper for a native .NET wizard skeleton.
      */
-    public ref class WizardImpl : public ObjectImpl
+    public ref class WizardImpl : public FormImpl
     {
     private:
-      Form^ _main;
-      Form^ _parent;
       FlowLayoutPanel^ _sidebar;
       Label^ _title;
       Button^ _extra_button;
@@ -80,13 +76,15 @@ namespace MySQL {
       static void set_next_caption(::mforms::Wizard *self, const string& caption);
 
       void sidebarPaint(Object^ sender, PaintEventArgs^ arguments);
+      void setupLayout(mforms::Wizard* wizard);
+
       static void nextClick(Object^ sender, EventArgs^ arguments);
       static void backClick(Object^ sender, EventArgs^ arguments);
       static void cancelClick(Object^ sender, EventArgs^ arguments);
       static void extraClick(Object^ sender, EventArgs^ arguments);
       static void resize(Object^ sender, EventArgs^ arguments);
     public:
-      WizardImpl(::mforms::Wizard *self);
+      WizardImpl(mforms::Wizard* self, mforms::Form* owner);
 
       void set_title(const string &title);
       void run_modal();
