@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,35 +34,7 @@ namespace grt
 
 //typedef ListDifference<ValueRef,internal::List::raw_iterator,internal::List::raw_iterator> GrtListDifference;
 
-static bool pless(const ValueRef &l, const ValueRef &r)
-{
-  if ((l.type() == r.type() && l.type() == ObjectType)
-    && ObjectRef::can_wrap(l) && ObjectRef::can_wrap(r))
-  {
-    ObjectRef left = ObjectRef::cast_from(l);
-    ObjectRef right = ObjectRef::cast_from(r);
-    if(left->has_member("name"))
-      return left->get_string_member("name") < right->get_string_member("name");
-  }
-  return l < r;
-}
-
-
-
-static bool peq(const ValueRef &l, const ValueRef &r)
-{
-  if ((l.type() == r.type() && l.type() == ObjectType)
-    && ObjectRef::can_wrap(l) && ObjectRef::can_wrap(r))
-  {
-    ObjectRef left = ObjectRef::cast_from(l);
-    ObjectRef right = ObjectRef::cast_from(r);
-    if(left->has_member("name"))
-      return left->get_string_member("name") == right->get_string_member("name");
-  }
-  return l == r;
-}
-
-
+//TODO: check if this is till used
 bool pless_struct::operator()(const ValueRef &_Left, const ValueRef &_Right) const
 { // apply operator< to operands
   return _Left < _Right;
@@ -90,11 +62,6 @@ struct omf_lt
   }
 };
 
-struct default_omf : public Omf
-{
-  virtual bool less(const ValueRef& l, const ValueRef& r)const {return pless(l,r);};
-  virtual bool equal(const ValueRef& l, const ValueRef& r)const  {return peq(l,r);};
-};
 
 struct OmfEqPred : public std::binary_function<ValueRef, ValueRef, bool>
 { // functor for operator<
@@ -164,11 +131,11 @@ void reversed_LIS(const InputContainerType& src, OutputContainerType& res)
 //  std::reverse(res.begin(),res.end());
 }
 
-bool diffPred(const ListItemChange* a, const ListItemChange* b)
+bool diffPred(const boost::shared_ptr<ListItemChange>& a, const boost::shared_ptr<ListItemChange>& b)
 {
   if (a->get_change_type() == grt::ListItemRemoved)
     if (b->get_change_type() == grt::ListItemRemoved)
-      return a->get_index() > b->get_index();//removals should be processed left to right to keep still existing items at the same indexes
+      return a->get_index() > b->get_index();//removals should be processed right to left to keep still existing items at the same indexes
     else
       return false;
   else
@@ -178,11 +145,11 @@ bool diffPred(const ListItemChange* a, const ListItemChange* b)
       return a->get_index() < b->get_index();
 }
 
-DiffChange *GrtListDiff::diff(const BaseListRef &source, const BaseListRef &target, const Omf *omf, const TSlotNormalizerSlot sqlDefinitionCmp)
+boost::shared_ptr<MultiChange> GrtListDiff::diff(const BaseListRef &source, const BaseListRef &target, const Omf *omf)
 {
   typedef std::vector<size_t> TIndexContainer;
   default_omf def_omf;
-  std::vector<ListItemChange*> changes;
+  std::vector<boost::shared_ptr<ListItemChange> > changes;
   const Omf *comparer = omf?omf:&def_omf;
   ValueRef prev_value;
   //This is indexes of source's elements that exist in both target and source
@@ -199,7 +166,7 @@ DiffChange *GrtListDiff::diff(const BaseListRef &source, const BaseListRef &targ
       continue;
     internal::List::raw_const_iterator It = find_if(source.content().raw_begin(), source.content().raw_end(),std::bind2nd(OmfEqPred(comparer),v));
     if (It == source.content().raw_end())
-      changes.push_back(new ListItemAddedChange(v, prev_value,target_idx));
+      changes.push_back(boost::shared_ptr<ListItemChange> (new ListItemAddedChange(v, prev_value,target_idx)));
     else//item exists in both target and source, save indexes
       source_indexes.push_back(source.get_index(*It));
     prev_value = v;
@@ -218,7 +185,7 @@ DiffChange *GrtListDiff::diff(const BaseListRef &source, const BaseListRef &targ
 
     internal::List::raw_const_iterator It = find_if(target.content().raw_begin(), target.content().raw_end(),std::bind2nd(OmfEqPred(comparer),v));
     if (It == target.content().raw_end())
-      changes.push_back(new ListItemRemovedChange(v,source_idx));
+      changes.push_back(boost::shared_ptr<ListItemChange> (new ListItemRemovedChange(v,source_idx)));
     else
       ordered_indexes.push_back(source_idx);
   };
@@ -233,43 +200,42 @@ DiffChange *GrtListDiff::diff(const BaseListRef &source, const BaseListRef &targ
   {
     internal::List::raw_const_iterator It_target = find_if(target.content().raw_begin(), target.content().raw_end(),std::bind2nd(OmfEqPred(comparer),source.get(*It)));
     prev_value = It_target == target.content().raw_begin()?ValueRef():*(It_target-1);
-      ListItemOrderChange* orderchange = 
-        new ListItemOrderChange(source.get(*It), *It_target, omf, sqlDefinitionCmp, prev_value, target.get_index(*It_target));
+      boost::shared_ptr<ListItemOrderChange> orderchange(
+        new ListItemOrderChange(source.get(*It), *It_target, omf, prev_value, target.get_index(*It_target)));
 //    if (!orderchange->subchanges()->empty())
       changes.push_back(orderchange); 
   }
 
   for (TIndexContainer::iterator It = stable_elements.begin(); It != stable_elements.end(); ++It)
   {
-    internal::List::raw_const_iterator It_target = find_if(target.content().raw_begin(), target.content().raw_end(),std::bind2nd(OmfEqPred(comparer),source.get(*It)));
+    internal::List::raw_const_iterator It_target = find_if(target.content().raw_begin(), target.content().raw_end(), std::bind2nd(OmfEqPred(comparer), source.get(*It)));
     if (It_target != target.content().raw_end())
     {
-      ListItemChange* change = create_item_modified_change(source.get(*It), *It_target, omf, sqlDefinitionCmp, target.get_index(*It_target));
+      boost::shared_ptr<ListItemChange> change = create_item_modified_change(source.get(*It), *It_target, omf, target.get_index(*It_target));
       if (change)
         changes.push_back(change);
     }
   }
   ChangeSet retval;
   std::sort(changes.begin(),changes.end(),diffPred);
-  for(std::vector<ListItemChange*>::const_iterator It = changes.begin(); It != changes.end(); ++It)
+  for(std::vector<boost::shared_ptr<ListItemChange> >::const_iterator It = changes.begin(); It != changes.end(); ++It)
     retval.append(*It);
-  return retval.empty()? NULL : new MultiChange(ListModified,retval);
+  return retval.empty()? boost::shared_ptr<MultiChange>() : boost::shared_ptr<MultiChange>(new MultiChange(ListModified,retval));
 
 }
 
 ////////////////////////////////////////////////////////////////////////////
-ListItemModifiedChange *create_item_modified_change(
+boost::shared_ptr<ListItemModifiedChange> create_item_modified_change(
   const ValueRef &source, 
   const ValueRef &target,
   const Omf* omf,
-  const TSlotNormalizerSlot sqlDefinitionCmp,
   const size_t index)
 {
-  DiffChange *subchange= diff_make(source, target, omf, sqlDefinitionCmp);
+  boost::shared_ptr<DiffChange> subchange= diff_make(source, target, omf);
   if (!subchange)
-    return NULL;
+    return boost::shared_ptr<ListItemModifiedChange>();
   //    diff_make(source, target, omf, sqlDefinitionCmp);
-  return new ListItemModifiedChange(source, target, subchange, index);
+  return boost::shared_ptr<ListItemModifiedChange> (new ListItemModifiedChange(source, target, subchange, index));
 }
 
 }
