@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,6 +20,7 @@
 #include "stdafx.h"
 
 #include "base/string_utilities.h"
+#include "base/threading.h"
 #include "base/log.h"
 
 #include "grtpp_shell_python.h"
@@ -28,6 +29,7 @@
 #include "grtpp_shell.h"
 #include "grtpp_module_cpp.h"
 #include "grtpp_undo_manager.h"
+#include "grtpp_notifications.h"
 
 #include <algorithm>
 
@@ -38,7 +40,7 @@
 #include "serializer.h"
 #include "unserializer.h"
 
-ENABLE_LOG("")
+DEFAULT_LOG_DOMAIN(DOMAIN_GRT)
 
 using namespace grt;
 using namespace base;
@@ -172,6 +174,8 @@ GRT::GRT()
   
   if (getenv("GRT_VERBOSE"))
     _verbose= true;
+  
+  GRTNotificationCenter::setup();
 
   _default_undo_manager= new UndoManager(this);
 
@@ -453,7 +457,7 @@ void GRT::end_loading_metaclasses(bool check_class_binding)
          iter != _metaclasses.end(); ++iter)
     {
       if (!iter->second->is_bound())
-        g_warning("Allocation function of '%s' is unbound, which probably means the implementing C++ class was not registered",
+        g_warning("Allocation function of '%s' is unbound, which probably means the implementing C++ class was not registered\n",
                   iter->second->name().c_str());
     }
   }
@@ -483,17 +487,18 @@ void GRT::add_metaclass(MetaClass *stru)
 
 void GRT::set_root(const ValueRef &root)
 {
-  AutoLock(this);
+  AutoLock lock(this);
   _root= root;
-  if (_root.is_valid())
-    _root.valueptr()->mark_global();
+// only nodes starting from /wb/doc (ie, in a model) should be marked global for undo tracking
+//  if (_root.is_valid())
+//    _root.mark_global();
 }
 
 
 
 ValueRef GRT::get(const std::string &path) const
 {
-  AutoLock(this);
+  AutoLock lock(this);
  
   return get_value_by_path(_root, path);
 }
@@ -501,7 +506,7 @@ ValueRef GRT::get(const std::string &path) const
 
 void GRT::set(const std::string &path, const ValueRef &value)
 {
-  AutoLock(this);
+  AutoLock lock(this);
   
   if (!set_value_by_path(_root, path, value))
     throw grt::bad_item("Invalid path "+path);
@@ -815,7 +820,6 @@ std::vector<Module*> GRT::find_modules_matching(const std::string &interface_nam
 
   for (std::vector<Module*>::const_iterator module= _modules.begin(); 
        module != _modules.end(); ++module)
-       
   {
     bool ok= true;
     if (!interface_name.empty())
@@ -974,28 +978,73 @@ Shell *GRT::get_shell()
 
 //--------------------------------------------------------------------------------
 
-MessageSlot GRT::set_message_handler(const MessageSlot &slot)
+void GRT::push_message_handler(const MessageSlot &slot)
 {
-  g_static_rec_mutex_lock(&_message_mutex);
-  MessageSlot old(_message_slot);
-  
-  _message_slot= slot;
-  g_static_rec_mutex_unlock(&_message_mutex);
-  
-  return old;
+  GStaticRecMutexLock lock(_message_mutex);  
+  _message_slot_stack.push_back(slot);
 }
 
 
-StatusQuerySlot GRT::set_status_query_handler(const StatusQuerySlot &slot)
+void GRT::pop_message_handler()
 {
-  StatusQuerySlot old(_status_query_slot);
-  
-  _status_query_slot= slot;
-  
-  return old;
+  GStaticRecMutexLock lock(_message_mutex);
+  if (_message_slot_stack.empty())
+    log_error("pop_message_handler() called on empty handler stack");
+  else
+    _message_slot_stack.pop_back();
 }
 
 
+bool GRT::handle_message(const Message &msg, void *sender)
+{
+  if (_message_slot_stack.empty())
+    log_error("Unhandled message: %s\n", msg.format().c_str());
+  else
+  {
+    int i = 0;
+    MessageSlot slot;
+    for (;;)
+    {
+      {
+        GStaticRecMutexLock lock(_message_mutex);
+        if ((int)_message_slot_stack.size() - i - 1 >= 0)
+        {
+          slot = _message_slot_stack[_message_slot_stack.size() - i - 1];
+          ++i;
+        }
+        else
+          break;
+      }
+      if (slot(msg, sender))
+        return true;
+    }
+  }
+  log_error("Unhandled message (%lu): %s\n", _message_slot_stack.size(), msg.format().c_str());
+  return false;
+}
+
+
+void GRT::push_status_query_handler(const StatusQuerySlot &slot)
+{
+  _status_query_slot_stack.push_back(slot);
+}
+
+
+void GRT::pop_status_query_handler()
+{
+  _status_query_slot_stack.pop_back();
+}
+
+
+bool GRT::query_status()
+{
+  if (_status_query_slot_stack.empty())
+    return false;
+  return _status_query_slot_stack.back()();
+}
+
+
+
 void GRT::make_output_visible(void *sender)
 {
   g_static_rec_mutex_lock(&_message_mutex);
@@ -1005,11 +1054,11 @@ void GRT::make_output_visible(void *sender)
   msg.detail= "";
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
-  _message_slot(msg, sender);
+  handle_message(msg, sender);
   g_static_rec_mutex_unlock(&_message_mutex);
 }
 
-
+// XXX: these handlers should go and be replaced by pure log_* calls.
 void GRT::send_error(const std::string &message, const std::string &details, void *sender)
 {
   g_static_rec_mutex_lock(&_message_mutex);
@@ -1019,12 +1068,10 @@ void GRT::send_error(const std::string &message, const std::string &details, voi
   msg.detail= details;
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
-  _message_slot(msg, sender);
+  handle_message(msg, sender);
   g_static_rec_mutex_unlock(&_message_mutex);
   
-  if (verbose())
-    g_message("ERROR: %s    %s", message.c_str(), details.c_str());
-  log_error("%s    %s", message.c_str(), details.c_str());
+  log_error("%s\t%s\n", message.c_str(), details.c_str());
 }
 
 
@@ -1037,12 +1084,10 @@ void GRT::send_warning(const std::string &message, const std::string &details, v
   msg.detail= details;
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
-  _message_slot(msg, sender);
+  handle_message(msg, sender);
   g_static_rec_mutex_unlock(&_message_mutex);
   
-  if (verbose())
-    g_message("WARNING: %s    %s", message.c_str(), details.c_str());
-  log_warning("%s    %s", message.c_str(), details.c_str());
+  log_warning("%s\t%s", message.c_str(), details.c_str());
 }
 
 
@@ -1055,10 +1100,40 @@ void GRT::send_info(const std::string &message, const std::string &details, void
   msg.detail= details;
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
-  _message_slot(msg, sender);
+  handle_message(msg, sender);
   g_static_rec_mutex_unlock(&_message_mutex);
   
-  log_info("%s    %s", message.c_str(), details.c_str());
+  log_info("%s\t%s\n", message.c_str(), details.c_str());
+}
+
+
+void GRT::reset_progress_steps()
+{
+  _progress_step_stack.clear();
+}
+
+
+/**
+ Used to begin tracking progress in a sub-task from a larger task.
+ A large task can be broken down in smaller sub-tasks.
+ The progress of the task as a whole will be tracked from a 0.0 to 1.0, and the sub-tasks
+ can also be individually reported to be ranging from 0.0 to 1.0. As long as begin_progress_step()
+ is called with correct values for the range of each sub-task in the larger task, the progress
+ of the task as a whole will be computed and reported as the user would expect. This has the advantage
+ that the sub-tasks don't need to know their relative progress in the big task, only their own progress.
+ 
+ begin - from 0.0 to 1.0, initial value of the progress for the sub-task
+ end - from 0.0 to 1.0, the final value of the progress for the sub-task once its completed
+ */
+void GRT::begin_progress_step(float from, float to)
+{
+  _progress_step_stack.push_back(std::make_pair(from, to));
+}
+
+
+void GRT::end_progress_step()
+{
+  _progress_step_stack.pop_back();
 }
 
 
@@ -1070,9 +1145,23 @@ void GRT::send_progress(float percentage, const std::string &message, const std:
   msg.text= message;
   msg.detail= details;
   msg.timestamp= time(NULL);
+  
+  // calculate the actual progress percentage
+  if (!_progress_step_stack.empty())
+  {
+    for (int i = _progress_step_stack.size()-1; i >= 0; --i)
+    {
+      percentage = _progress_step_stack[i].first + (_progress_step_stack[i].second - _progress_step_stack[i].first) * percentage;
+    } 
+  }
   msg.progress= percentage;
-  _message_slot(msg, sender);
+  handle_message(msg, sender);
   g_static_rec_mutex_unlock(&_message_mutex);
+
+  // Progress messages are often sent without a real message, just to trigger some actions.
+  // Don't log empty lines.
+//  if (message.size() > 0)
+//    log_debug3("%s\t%s", message.c_str(), details.c_str());
 }
 
 
@@ -1085,22 +1174,24 @@ void GRT::send_verbose(const std::string &message, void *sender)
   msg.detail= "";
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
-  _message_slot(msg, sender);
+  handle_message(msg, sender);
   g_static_rec_mutex_unlock(&_message_mutex);
-  
-  log_info("%s", message.c_str());
+
+  log_debug2("%s", message.c_str());
 }
 
 
-void GRT::send_output(const std::string &text, void *sender)
+void GRT::send_output(const std::string &message, void *sender)
 {
   g_static_rec_mutex_lock(&_message_mutex);
   Message msg;
   msg.type= OutputMsg;
-  msg.text= text;
+  msg.text= message;
   msg.detail= "";
   msg.timestamp= time(NULL);
   msg.progress= 0.0;
-  _message_slot(msg, sender);
+  handle_message(msg, sender);
   g_static_rec_mutex_unlock(&_message_mutex);
+  
+  log_debug("%s", message.c_str());
 }
