@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,9 @@
 #include "grtpp_util.h"
 #include "base/string_utilities.h"
 #include "base/file_functions.h"
+#include "base/log.h"
+
+DEFAULT_LOG_DOMAIN("ModulePython");
 
 #ifdef ENABLE_PYTHON_MODULES
 
@@ -55,17 +58,24 @@ static TypeSpec parse_type(PyObject *type)
   {
     TypeSpec s;
     PyObject *b, *c;
+    
+    if (PyTuple_Size(type) != 2)
+    {
+      PythonContext::log_python_error("Invalid type specification for Python module function. Tuple must be in form (<container type>, <content type>)");
+      throw std::runtime_error("Invalid type specification. Tuple must be in form (<container type>, <content type>)");
+    }
+    
     b= PyTuple_GetItem(type, 0);
     if (!b)
     {
-      PyErr_Print();
-      throw std::runtime_error("Invalid type specification");
+      PythonContext::log_python_error("Invalid type specification for Python module function");
+      throw std::runtime_error("Invalid type specification 0");
     }
     c= PyTuple_GetItem(type, 1);
     if (!c)
     {
-      PyErr_Print();
-      throw std::runtime_error("Invalid type specification");
+      PythonContext::log_python_error("Invalid type specification for Python module function");
+      throw std::runtime_error("Invalid type specification 1");
     }
 
     s.base.type= str_to_type(PyString_AsString(b));
@@ -105,9 +115,10 @@ void PythonModule::add_parse_function(const std::string &name, PyObject *return_
   {
     func.ret_type= parse_type(return_type);
   }
-  catch (std::exception)
+  catch (std::exception &exc)
   {
-    throw std::runtime_error(strfmt("Invalid return type specification in %s.%s", _name.c_str(), name.c_str()));
+    log_error("Invalid return type specification in %s.%s: %s\n", _name.c_str(), name.c_str(), exc.what());
+    throw std::runtime_error(strfmt("Invalid return type specification in %s.%s: %s", _name.c_str(), name.c_str(), exc.what()));
   }
   
   for (Py_ssize_t c= PyList_Size(arguments), i= 0; i < c; i++)
@@ -118,14 +129,14 @@ void PythonModule::add_parse_function(const std::string &name, PyObject *return_
     
     if (!PyTuple_Check(spec))
     {
-      PyErr_Print();
+      PythonContext::log_python_error("Invalid argument specification for Python module function (not a tuple)");
       throw std::runtime_error("Invalid argument specification (argument spec must be list of tuples)");
     }
   
     tmp= PyTuple_GetItem(spec, 0);
-    if (!tmp && !PyString_Check(tmp))
+    if (!tmp || !PyString_Check(tmp))
     {
-      PyErr_Print();
+      PythonContext::log_python_error("Invalid argument name specification for Python module function");
       throw std::runtime_error("Invalid argument name specification");
     }
     arg.name= PyString_AsString(tmp);
@@ -133,26 +144,52 @@ void PythonModule::add_parse_function(const std::string &name, PyObject *return_
     tmp= PyTuple_GetItem(spec, 1);
     if (!tmp)
     {
-      PyErr_Print();
+      PythonContext::log_python_error("Invalid argument type specification for Python module function");
       throw std::runtime_error("Invalid argument type specification");
     }
     try
     {
       arg.type= parse_type(tmp);
     }
-    catch (std::exception)
+    catch (std::exception &exc)
     {
+      log_error("Invalid argument type specification in %s.%s: %s\n", _name.c_str(), name.c_str(), exc.what());
       throw std::runtime_error(strfmt("Invalid argument type specification in %s.%s", _name.c_str(), name.c_str()));
     }
     func.arg_types.push_back(arg);
   }
   
-  func.call= sigc::bind(sigc::mem_fun(this, &PythonModule::call_function), callable, func);
+  PyObject *doc = PyObject_GetAttrString(callable, "__doc__");
+  if (doc && doc != Py_None)
+    func.description = PyString_AsString(doc);
+  
+  func.call= boost::bind(&PythonModule::call_function, this, _1, callable, func);
   
   add_function(func);
 }
 
 
+static std::string exception_detail()
+{
+  PyObject *exc_class = NULL, *exc = NULL, *exc_tb = NULL;
+  PyErr_Fetch(&exc_class, &exc, &exc_tb);
+  if (exc)
+  {
+    PyObject *str = PyObject_Str(exc);
+    if (str)
+    {
+      char *s = PyString_AsString(str);
+      if (s)
+        return s;
+      Py_DECREF(str);
+    }
+    Py_DECREF(exc);
+  }
+  Py_XDECREF(exc_class);
+  Py_XDECREF(exc_tb);
+  return "";
+}
+
 ValueRef PythonModule::call_function(const BaseListRef &args, PyObject *function, const Function &funcdef)
 {
   WillEnterPython lock;
@@ -180,23 +217,47 @@ ValueRef PythonModule::call_function(const BaseListRef &args, PyObject *function
 
   if (!ret || PyErr_Occurred())
   {
-    g_message("function call error");
-    PyErr_Print();
-    /* dont know how to fetch formatted traceback
-    PyObject *exception, *v, *tb= NULL;
-    PyErr_Fetch(&exception, &v, &tb);
-    PyErr_NormalizeException(&exception, &v, &tb);
- 
-    PyObject *s = PyObject_Str(v);
-    std::string error;
-    if (s != NULL && PyString_Check(s))
-      error= PyString_AsString(s);
-  
-    Py_XDECREF(exception);
-    Py_XDECREF(v);
-    Py_XDECREF(tb);
-    */
-    throw grt::module_error(strfmt("error calling %s.%s: see output for details", name().c_str(), funcdef.name.c_str()));
+    if (PyErr_ExceptionMatches(ctx->user_interrupted_error()))
+    {
+      std::string what = exception_detail();
+      if (what.empty())
+        what = "Interrupted by user";
+      
+      throw grt::user_cancelled(what);
+    }
+    if (PyErr_ExceptionMatches(ctx->db_access_denied_error()))
+    {
+      std::string what = exception_detail();
+      if (what.empty())
+        what = "Access denied";
+      throw grt::db_access_denied(what);
+    }
+    if (PyErr_ExceptionMatches(ctx->db_login_error()))
+    {
+      std::string what = exception_detail();
+      if (what.empty())
+        what = "Login error, check username and password";
+      throw grt::db_login_error(what);
+    }
+    PyObject *etype;
+    PyObject *evalue;
+    PyObject *etrace;
+    PyErr_Fetch(&etype, &evalue, &etrace);
+
+    PyObject *s = PyObject_GetAttrString(etype, "__name__");
+    std::string ename, exc;
+    if (!ctx->pystring_to_string(s, ename))
+      ename = "???";
+    s = PyObject_Str(evalue);
+    if (!ctx->pystring_to_string(s, exc))
+      exc = "???";
+    if (s)
+      Py_DECREF(s);
+    
+    PyErr_Restore(etype, evalue, etrace);
+    PythonContext::log_python_error(strfmt("error calling %s.%s", name().c_str(), funcdef.name.c_str()).c_str());
+    throw grt::module_error(strfmt("error calling Python module function %s.%s", name().c_str(), funcdef.name.c_str()),
+                            strfmt("%s(\"%s\")", ename.c_str(), exc.c_str()));
   }
 
   ValueRef result= ctx->from_pyobject(ret, funcdef.ret_type);
