@@ -72,7 +72,7 @@ std::string grt::get_guid()
   str= CFUUIDCreateString(NULL, uid);
   
 #ifdef ENABLE_DEBUG
-  assert(sizeof(data) > CFStringGetLength(str));
+  assert((int)sizeof(data) > CFStringGetLength(str));
 #endif
   //data= (char*)g_malloc(sizeof(gchar)*(CFStringGetLength(str)+1));
   
@@ -914,21 +914,6 @@ void grt::merge_contents(ObjectRef target, ObjectRef source)
   while (metac != 0);
 }
 
-
-
-ObjectRef grt::copy_object(GRT *grt, ObjectRef object, std::set<std::string> skip_members)
-{
-  CopyContext copier(grt);
-  ObjectRef copy;
-  
-  copy= copier.copy(object, skip_members);
-  copier.finish();
-
-  return copy;
-}
-
-
-
 ObjectRef CopyContext::copy(const ObjectRef &object, std::set<std::string> skip_members)
 {
   ObjectRef copy= duplicate_object(object, skip_members);
@@ -1072,6 +1057,6 @@ void grt::add_python_module_dir(grt::GRT *grt, const std::string &python_module_
 #ifdef ENABLE_PYTHON_MODULES
   PythonModuleLoader *loader= dynamic_cast<PythonModuleLoader *>(grt->get_module_loader("python"));
   if (loader && !python_module_path.empty())
-    loader->get_python_context()->add_module_path(python_module_path);
+    loader->get_python_context()->add_module_path(python_module_path, true);
 #endif
 }
