@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,31 +25,23 @@
 #include "grtpp_util.h"
 #include "base/string_utilities.h"
 #include "base/file_functions.h"
+#include "base/wb_memory.h"
 
 #ifdef ENABLE_PYTHON_MODULES
 
 // python internals
-#ifdef __APPLE__
-#include <Python/node.h>
-#include <Python/grammar.h>
-#include <Python/parsetok.h>
-#include <Python/errcode.h>
-#include <Python/token.h>
-#else
 #include <node.h>
 #include <grammar.h>
 #include <parsetok.h>
 #include <errcode.h>
 #include <token.h>
-#endif
-
 
 #include "python_grtobject.h"
 #include "python_grtlist.h"
 #include "python_grtdict.h"
 
 #include "base/log.h"
-ENABLE_LOG("grt")
+DEFAULT_LOG_DOMAIN("python context")
 
 using namespace grt;
 using namespace base;
@@ -104,8 +96,8 @@ PythonContext::PythonContext(GRT *grt, const std::string &module_path)
   }
 
   putenv(g_strdup_printf("PYTHONHOME=%s\\Python", module_path.c_str()));
-  putenv(g_strdup_printf("PYTHONPATH=%s\\Python;%s\\Python\\DLLs;%s\\Python\\Lib", 
-    module_path.c_str(), module_path.c_str(), module_path.c_str()));
+  putenv(g_strdup_printf("PYTHONPATH=%s\\Python;%s\\Python\\DLLs;%s\\Python\\Lib;%s\\Python\\mysql_libs.zip", 
+    module_path.c_str(), module_path.c_str(), module_path.c_str(), module_path.c_str()));
   //putenv("PYTHONHOME=C:\\nowhere");
 #endif
   Py_InitializeEx(0); // skips signal handler init
@@ -135,6 +127,14 @@ PythonContext::PythonContext(GRT *grt, const std::string &module_path)
   // set stdin to the GRT shell console
   PySys_SetObject((char*)"stdin", get_grt_module());
 
+  run_post_init_script();
+  
+  {
+    PyObject *path = from_grt(grt::StringRef(base::Logger::log_filename()));
+    PyDict_SetItemString(PyModule_GetDict(PyImport_AddModule("grt")), "logpath", path);
+    Py_DECREF(path);
+  }
+
   PyEval_SaveThread();
 }
 
@@ -178,6 +178,21 @@ void PythonContext::add_module_path(const std::string &modpath, bool prepend)
 }
 
 
+void PythonContext::set_user_interrupted(const grt::user_cancelled &exc)
+{
+  PyErr_SetString(PythonContext::get()->_grt_user_interrupt_error, exc.what());
+}
+
+void PythonContext::set_db_access_denied(const grt::db_access_denied &exc)
+{
+  PyErr_SetString(PythonContext::get()->_grt_db_access_denied_error, exc.what());
+}
+
+void PythonContext::set_db_login_error(const grt::db_login_error &exc)
+{
+  PyErr_SetString(PythonContext::get()->_grt_db_login_error, exc.what());
+}
+
 void PythonContext::set_python_error(const grt::type_error &exc, const std::string &location)
 {
   PyErr_SetString(PyExc_TypeError, (location.empty() ? exc.what() : location + ": " + exc.what()).c_str());
@@ -258,7 +273,7 @@ static PyObject *grt_print(PyObject *self, PyObject *args)
       return NULL;
   }  
   else
-    if (!ctx->pystring_to_string(o, text))
+    if (!ctx->pystring_to_string(o, text, true))
       return NULL;
   
 #ifdef _WIN32
@@ -272,6 +287,58 @@ static PyObject *grt_print(PyObject *self, PyObject *args)
   return Py_None;
 }
 
+static PyObject *pylog(base::Logger::LogLevel level, PyObject *args)
+{
+  PythonContext *ctx;
+  std::string text;
+  
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  char *domain;
+  PyObject *o;
+  if (!PyArg_ParseTuple(args, "sO", &domain, &o))
+    return NULL;
+  
+  if (!ctx->pystring_to_string(o, text, true))
+    return NULL;
+  
+  base::Logger::log(level, domain, "%s", text.c_str());
+
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+
+static PyObject *grt_log_error(PyObject *self, PyObject *args)
+{
+  return pylog(base::Logger::LogError, args);
+}
+
+static PyObject *grt_log_warning(PyObject *self, PyObject *args)
+{
+  return pylog(base::Logger::LogWarning, args);
+}
+
+static PyObject *grt_log_info(PyObject *self, PyObject *args)
+{
+  return pylog(base::Logger::LogInfo, args);
+}
+
+static PyObject *grt_log_debug(PyObject *self, PyObject *args)
+{
+  return pylog(base::Logger::LogDebug, args);
+}
+
+static PyObject *grt_log_debug2(PyObject *self, PyObject *args)
+{
+  return pylog(base::Logger::LogDebug2, args);
+}
+
+static PyObject *grt_log_debug3(PyObject *self, PyObject *args)
+{
+  return pylog(base::Logger::LogDebug3, args);
+}
 
 static PyObject *grt_send_output(PyObject *self, PyObject *args)
 {
@@ -293,7 +360,7 @@ static PyObject *grt_send_output(PyObject *self, PyObject *args)
       return NULL;
   }  
   else
-    if (!ctx->pystring_to_string(o, text))
+    if (!ctx->pystring_to_string(o, text, true))
       return NULL;
 
   ctx->get_grt()->send_output(text);
@@ -307,11 +374,12 @@ static PyObject *grt_send_warning(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
   std::string text;
+  std::string detail;
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
   
-  PyObject *o;
-  if (!PyArg_ParseTuple(args, "O", &o))
+  PyObject *o, *d = NULL;
+  if (!PyArg_ParseTuple(args, "O|O", &o, &d))
   {
     if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None)
     {
@@ -322,10 +390,14 @@ static PyObject *grt_send_warning(PyObject *self, PyObject *args)
       return NULL;
   }  
   else
-    if (!ctx->pystring_to_string(o, text))
+  {
+    if (!ctx->pystring_to_string(o, text, true))
       return NULL;
-
-  ctx->get_grt()->send_warning(text);
+    if (d && !ctx->pystring_to_string(d, detail, true))
+      return NULL;
+  }
+  
+  ctx->get_grt()->send_warning(text, detail);
   
   Py_INCREF(Py_None);
   return Py_None;
@@ -335,11 +407,12 @@ static PyObject *grt_send_info(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
   std::string text;
+  std::string detail;
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
   
-  PyObject *o;
-  if (!PyArg_ParseTuple(args, "O", &o))
+  PyObject *o, *d = NULL;
+  if (!PyArg_ParseTuple(args, "O|O", &o, &d))
   {
     if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None)
     {
@@ -350,12 +423,15 @@ static PyObject *grt_send_info(PyObject *self, PyObject *args)
       return NULL;
   }  
   else
-    if (!ctx->pystring_to_string(o, text))
+  {
+    if (!ctx->pystring_to_string(o, text, true))
       return NULL;
+    if (d && !ctx->pystring_to_string(d, detail, true))
+      return NULL;
+  }
 
-  ctx->get_grt()->send_info(text);
-  // vvvv as each log output ends with new-line the one-item-at-time prints consume too much vertical space
-  //base::Logger::log(base::Logger::LogInfo, "grt.python", text.c_str());
+  ctx->get_grt()->send_info(text, detail);
+//  log_debug2("grt.python", "%s: (%s)", text.c_str(), detail.c_str());
 
   Py_INCREF(Py_None);
   return Py_None;
@@ -365,11 +441,12 @@ static PyObject *grt_send_error(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
   std::string text;
+  std::string detail;
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
   
-  PyObject *o;
-  if (!PyArg_ParseTuple(args, "O", &o))
+  PyObject *o, *d = NULL;
+  if (!PyArg_ParseTuple(args, "O|O", &o, &d))
   {
     if (PyTuple_Size(args) == 1 && PyTuple_GetItem(args, 0) == Py_None)
     {
@@ -380,10 +457,14 @@ static PyObject *grt_send_error(PyObject *self, PyObject *args)
       return NULL;
   }  
   else
-    if (!ctx->pystring_to_string(o, text))
+  {
+    if (!ctx->pystring_to_string(o, text, true))
       return NULL;
-
-  ctx->get_grt()->send_error(text);
+    if (d && !ctx->pystring_to_string(d, detail, true))
+      return NULL;
+  }
+  
+  ctx->get_grt()->send_error(text, detail);
   
   Py_INCREF(Py_None);
   return Py_None;
@@ -397,15 +478,62 @@ static PyObject *grt_send_progress(PyObject *self, PyObject *args)
     return NULL;
   
   float pct;
-  char *text, *detail= NULL;
-  if (!PyArg_ParseTuple(args, "fs|s", &pct, &text, &detail))
+  PyObject *texto, *detailo= NULL;
+  if (!PyArg_ParseTuple(args, "fO|O", &pct, &texto, &detailo))
     return NULL;
   
-  ctx->get_grt()->send_progress(pct, text ? text : "", detail ? detail : "", NULL);
+  std::string text, detail;
+  if (!ctx->pystring_to_string(texto, text, true))
+    return NULL;
+  if (detailo && !ctx->pystring_to_string(detailo, detail, true))
+    return NULL;
+  ctx->get_grt()->send_progress(pct, text, detail, NULL);
   Py_INCREF(Py_None);
   return Py_None;
 }
 
+static PyObject *grt_begin_progress_step(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  float pct1, pct2;
+  if (!PyArg_ParseTuple(args, "ff", &pct1, &pct2))
+    return NULL;
+  
+  ctx->get_grt()->begin_progress_step(pct1, pct2);
+
+  Py_RETURN_NONE;
+}
+
+static PyObject *grt_end_progress_step(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  if (!PyArg_ParseTuple(args, ""))
+    return NULL;
+  
+  ctx->get_grt()->end_progress_step();
+  
+  Py_RETURN_NONE;
+}
+
+static PyObject *grt_reset_progress_steps(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  if (!PyArg_ParseTuple(args, ""))
+    return NULL;
+  
+  ctx->get_grt()->reset_progress_steps();
+  
+  Py_RETURN_NONE;
+}
 
 static PyObject *grt_get_by_path(PyObject *self, PyObject *args)
 {
@@ -471,20 +599,50 @@ static PyObject *grt_readline(PyObject *self, PyObject *args)
   return Py_BuildValue("s", line.c_str());
 }
 
+void decrement_int(int *i)
+{
+  (*i)--;
+}
 
-static bool call_handle_output(const std::string &text, PyObject *callable)
+static bool call_handle_message(const grt::Message &msg, void *sender, AutoPyObject callable)
 {
   WillEnterPython lock;
   PyObject *ret;
+  const char *type = NULL;
+  static int handling_message = 0;
   
-  if (!(ret = PyObject_Call(callable, Py_BuildValue("s", text.c_str()), NULL)))
+  if (handling_message > 10)
   {
-    g_warning("Error calling Python output handler:");
-    PyErr_Print();
+    log_warning("Force-breaking apparent recursion of GRT message handlers\n");
+    return false;
+  }
+  handling_message++;
+  base::scope_ptr<int, decrement_int> guard(&handling_message);
+  
+  std::string text = msg.text;
+  switch (msg.type)
+  {
+    case grt::ErrorMsg: type = "ERROR"; break;
+    case grt::WarningMsg: type = "WARNING"; break;
+    case grt::InfoMsg: type = "INFO"; break;
+    case grt::OutputMsg: type = "OUTPUT"; break;
+    case grt::VerboseMsg: type = "VERBOSE"; break;
+    case grt::ProgressMsg: type = "PROGRESS"; text = strfmt("%f:%s", msg.progress, msg.text.c_str()); break;
+    default:
+      type = "???"; 
+      break;
+  }
+  
+  PyObject *args = Py_BuildValue("(sss)", type, text.c_str(), msg.detail.c_str());
+  if (!(ret = PyObject_Call(callable, args, NULL)))
+  {
+    Py_DECREF(args);
+    PythonContext::log_python_error("Error calling Python output handler:");
     PyErr_Clear();
     return false;
   }
-  
+  Py_DECREF(args);
+
   if (ret == Py_None || ret == Py_False || PyInt_AsLong(ret) == 0)
   {
     Py_DECREF(ret);
@@ -495,7 +653,7 @@ static bool call_handle_output(const std::string &text, PyObject *callable)
 }
 
 
-static PyObject *grt_push_output_handler(PyObject *self, PyObject *args)
+static PyObject *grt_push_message_handler(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
   if (!(ctx= PythonContext::get_and_check()))
@@ -507,24 +665,98 @@ static PyObject *grt_push_output_handler(PyObject *self, PyObject *args)
   
   if (!PyCallable_Check(o))
     return NULL;
+  
+  ctx->get_grt()->push_message_handler(boost::bind(&call_handle_message, _1, _2, AutoPyObject(o)));
 
-  ctx->push_output_handler(boost::bind(&call_handle_output,_1, AutoPyObject(o)));
-  Py_INCREF(Py_None);
-  return Py_None;
+  return Py_BuildValue("i", ctx->get_grt()->message_handler_count());
 }
 
-static PyObject *grt_pop_output_handler(PyObject *self, PyObject *args)
+
+static PyObject *grt_pop_message_handler(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
   if (!(ctx= PythonContext::get_and_check()))
     return NULL;
-
-  ctx->pop_output_handler();
   
-  Py_INCREF(Py_None);
-  return Py_None;
+  ctx->get_grt()->pop_message_handler();
+  
+  return Py_BuildValue("i", ctx->get_grt()->message_handler_count());
 }
 
+//
+
+static bool call_status_query(AutoPyObject callable)
+{
+  WillEnterPython lock;
+
+  PyObject *ret;
+  PyObject *args = Py_BuildValue("()");
+  if (!(ret = PyObject_Call(callable, args, NULL)))
+  {
+    Py_DECREF(args);
+    PythonContext::log_python_error("Error calling Python status handler:");
+    PyErr_Clear();
+    return false;
+  }
+  Py_DECREF(args);
+  
+  if (ret == Py_None || ret == Py_False || PyInt_AsLong(ret) == 0)
+  {
+    Py_DECREF(ret);
+    return false;
+  }
+  Py_DECREF(ret);  
+  return true;
+}
+
+
+static PyObject *grt_push_status_query_handler(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  PyObject *o;
+  if (!PyArg_ParseTuple(args, "O", &o))
+    return NULL;
+  
+  if (!PyCallable_Check(o))
+    return NULL;
+  
+  ctx->get_grt()->push_status_query_handler(boost::bind(&call_status_query, AutoPyObject(o)));
+  
+  Py_RETURN_NONE;
+}
+
+
+static PyObject *grt_pop_status_query_handler(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  ctx->get_grt()->pop_status_query_handler();
+  
+  Py_RETURN_NONE;
+}
+
+static PyObject *grt_query_status(PyObject *self, PyObject *args)
+{
+  PythonContext *ctx;
+  if (!(ctx= PythonContext::get_and_check()))
+    return NULL;
+  
+  if (args && !PyArg_ParseTuple(args, ""))
+    return NULL;
+  
+  if (ctx->get_grt()->query_status())
+    Py_RETURN_TRUE;
+  else
+    Py_RETURN_FALSE;
+}
+
+//
+
 static PyObject *grt_serialize(PyObject *self, PyObject *args)
 {
   PythonContext *ctx;
@@ -583,6 +815,7 @@ static PyObject *grt_unserialize(PyObject *self, PyObject *args)
   try
   {
     grt::ValueRef value = ctx->get_grt()->unserialize(path);
+    return ctx->from_grt(value);
   }
   catch (const std::exception &exc)
   {
@@ -615,12 +848,38 @@ static PyMethodDef GrtModuleMethods[] = {
 "Write a info message to the GRT shell."},
 {"send_progress",  grt_send_progress, METH_VARARGS,
 "Write a progress message."},
+{"begin_progress_step",  grt_begin_progress_step, METH_VARARGS,
+  "Starts a new step in a larger task where progress is being tracked."},
+{"end_progress_step",  grt_end_progress_step, METH_VARARGS,
+  "Ends a step."},
+{"reset_progress_steps", grt_reset_progress_steps, METH_VARARGS,
+  "Clear the progress steps stack."},
 
-{"push_output_handler", grt_push_output_handler, METH_VARARGS,
-"Pushes a callback of the form function(text) to be called when a plugin outputs text."},
-{"pop_output_handler", grt_push_output_handler, METH_NOARGS,
- "Pops previously pushed handler."},
+{"log_error", grt_log_error, METH_VARARGS,
+"Logs an error to the log file, in the specified context ex: log_error('myplugin', 'cannot open file')"},
+{"log_warning", grt_log_warning, METH_VARARGS,
+"Logs a warning to the log file, in the specified context ex: log_warning('myplugin', 'cannot open file very well')"},
+{"log_debug", grt_log_debug, METH_VARARGS,
+"Logs a debug message to the log file, in the specified context ex: log_debug('myplugin', 'trying to open file with ')"},
+{"log_debug2", grt_log_debug2, METH_VARARGS,
+ "Logs a verbose debug message to the log file, in the specified context ex: log_debug2('myplugin', 'reading from file')"},
+{"log_debug3", grt_log_debug3, METH_VARARGS,
+ "Logs a very verbose debug message to the log file, in the specified context ex: log_debug3('myplugin', 'processing file')"},
+{"log_info", grt_log_info, METH_VARARGS,
+"Logs an informational message to the log file, in the specified context ex: log_info('myplugin', 'file opened')"},
 
+{"push_message_handler", grt_push_message_handler, METH_VARARGS,
+  "Pushes a callback of the form function((type, text, detail)) to be called when a plugin outputs text. Return value must be True if the message was handled, False if it should be handled by a previously installed handler."},
+{"pop_message_handler", grt_pop_message_handler, METH_NOARGS,
+  "Pops previously pushed handler."},
+  
+{"push_status_query_handler", grt_push_status_query_handler, METH_VARARGS,
+  "Pushes a callback that returns a boolean value. True should be returned for aborting running tasks that call grt.query_status()"},
+{"pop_status_query_handler", grt_pop_status_query_handler, METH_NOARGS,
+  "Pops status_query callback from the stack."},
+{"query_status", grt_query_status, METH_NOARGS,
+  "Queries the current status query handler whether currently running task should be cancelled. If True is returned, the task should be cancelled."},
+  
 {"readline", grt_readline, METH_VARARGS,
 "Waits for a line of text to be input to the scripting shell prompt."},
 
@@ -661,6 +920,19 @@ void PythonContext::register_grt_module()
   init_grt_list_type();
   init_grt_dict_type();
   init_grt_object_type();
+
+  {
+    _grt_user_interrupt_error = PyErr_NewException((char*)"grt.UserInterrupt", NULL, NULL);
+    PyModule_AddObject(_grt_module, "UserInterrupt", _grt_user_interrupt_error);
+  }
+  {
+    _grt_db_access_denied_error = PyErr_NewException((char*)"grt.DBAccessDenied", NULL, NULL);
+    PyModule_AddObject(_grt_module, "DBAccessDenied", _grt_db_access_denied_error);
+  }
+  {
+    _grt_db_login_error = PyErr_NewException((char*)"grt.DBLoginError", NULL, NULL);
+    PyModule_AddObject(_grt_module, "DBLoginError", _grt_db_login_error);
+  }
   
   _grt_modules_module = Py_InitModule("grt.modules", NULL);
   if (!_grt_modules_module)
@@ -696,7 +968,7 @@ bool PythonContext::import_module(const std::string &name)
   PyObject *module = PyImport_ImportModule((char*)name.c_str());
   if (!main || !module)
   {
-    PyErr_Print();
+    PythonContext::log_python_error(base::strfmt("Error importing %s", name.c_str()).c_str());
     return false;
   }
   PyDict_SetItemString(PyModule_GetDict(main), name.c_str(), module);
@@ -720,7 +992,7 @@ PyObject *PythonContext::eval_string(const std::string &expression)
   {
     PyObject *result= PyRun_String(expression.c_str(), Py_eval_input, globals, globals);
     if (!result)
-      PyErr_Print();
+      PythonContext::log_python_error(base::strfmt("Error running expr: %s", expression.c_str()).c_str());
     return result;
   }
   PyErr_Clear();
@@ -750,14 +1022,14 @@ bool PythonContext::set_global(const std::string &name, PyObject *value)
   PyObject *mainmod= PyImport_AddModule("__main__");
   if (!mainmod)
   {
-    PyErr_Print();
+    PythonContext::log_python_error("Error getting __main__");
     PyErr_Clear();
     return false;
   }
   PyObject *globals= PyModule_GetDict(mainmod);
   if (!globals)
   {
-    PyErr_Print();
+    PythonContext::log_python_error("Error getting __main__ dict");
     PyErr_Clear();
     return false;
   }
@@ -821,21 +1093,25 @@ PyObject *PythonContext::from_grt(const ValueRef &value)
       {
         // maybe this should start returning unicode data, but before that all python code
         // should be tested if it can handle the unicode type. For now we just return utf8 strings.
-        //return PyUnicode_EncodeUTF8();
         std::string data= *StringRef::cast_from(value);
+        //return PyUnicode_DecodeUTF8(data.data(), data.size(), NULL);
         return PyString_FromStringAndSize(data.data(), data.size());
       }
       case ListType:
       {
         PyObject *content= PythonContext::internal_cobject_from_value(value);
-        PyObject *r= PyObject_Call(_grt_list_class, Py_BuildValue("(ssO)", "", "", content), NULL);
+        PyObject *args = Py_BuildValue("(ssO)", "", "", content);
+        PyObject *r= PyObject_Call(_grt_list_class, args, NULL);
+        Py_XDECREF(args);
         Py_XDECREF(content);
         return r;
       }
       case DictType:
       {
         PyObject *content= PythonContext::internal_cobject_from_value(value);
-        PyObject *r= PyObject_Call(_grt_dict_class, Py_BuildValue("(ssO)", "", "", content), NULL);
+        PyObject *args = Py_BuildValue("(ssO)", "", "", content);
+        PyObject *r= PyObject_Call(_grt_dict_class, args, NULL);
+        Py_XDECREF(args);
         Py_XDECREF(content);
         return r;
       }
@@ -844,7 +1120,9 @@ PyObject *PythonContext::from_grt(const ValueRef &value)
         std::string class_name= grt::ObjectRef::cast_from(value).class_name();
         PyObject *content= PythonContext::internal_cobject_from_value(value);
         PyObject *theclass= _grt_class_wrappers[class_name];
-        PyObject *r= PyObject_Call(theclass ? theclass : (PyObject*)_grt_object_class, Py_BuildValue("(sO)", "", content), NULL);
+        PyObject *args = Py_BuildValue("(sO)", "", content);
+        PyObject *r= PyObject_Call(theclass ? theclass : (PyObject*)_grt_object_class, args, NULL);
+        Py_XDECREF(args);
         Py_XDECREF(content);
         
         return r;
@@ -857,7 +1135,7 @@ PyObject *PythonContext::from_grt(const ValueRef &value)
   return Py_None;
 }
 
-bool PythonContext::pystring_to_string(PyObject *strobject, std::string &ret_string)
+bool PythonContext::pystring_to_string(PyObject *strobject, std::string &ret_string, bool convert)
 {
   if (PyUnicode_Check(strobject))
   {
@@ -888,6 +1166,18 @@ bool PythonContext::pystring_to_string(PyObject *strobject, std::string &ret_str
       ret_string = "";
     return true;
   }
+  
+  if (convert)
+  {
+    PyObject *str = PyObject_Str(strobject);
+    if (str)
+    {
+      bool ret = pystring_to_string(str, ret_string, false);
+      Py_DECREF(str);
+      return ret;
+    }
+  }
+  
   return false;
 }
 
@@ -1055,7 +1345,7 @@ ValueRef PythonContext::from_pyobject(PyObject *object, const grt::TypeSpec &exp
               list.ginsert(from_pyobject(item));
               break;
             default:
-              g_warning("invalid type spec requested");
+              log_warning("invalid type spec requested\n");
               break;
           }
         }
@@ -1090,7 +1380,7 @@ ValueRef PythonContext::from_pyobject(PyObject *object, const grt::TypeSpec &exp
               dict.set(PyString_AsString(key), from_pyobject(value));
               break;
             default:
-              g_warning("invalid type spec requested");
+              log_warning("invalid type spec requested\n");
               break;
           }
         }
@@ -1117,16 +1407,15 @@ int PythonContext::run_file(const std::string &file, bool interactive)
   PyObject *f = PyFile_FromString((char*)base::string_to_path_for_open(file).c_str(), (char*)"r");
   if (!f)
   {
-    log_error("Could not open file %s", file.c_str());
-    PyErr_Print();
+    PythonContext::log_python_error(base::strfmt("Could not open file %s\n", file.c_str()).c_str());
     return -1;
   }
 
-  log_debug2("About to pyrun '%s'", file.c_str());
+  log_debug2("About to pyrun '%s'\n", file.c_str());
   if (PyRun_SimpleFile(PyFile_AsFile(f), file.c_str()) != 0)
   {
     Py_DECREF(f);
-    PyErr_Print();
+    PythonContext::log_python_error(base::strfmt("Error running file %s\n", file.c_str()).c_str());
     return -1;
   }
   Py_DECREF(f);
@@ -1186,7 +1475,7 @@ int PythonContext::run_buffer(const std::string &buffer, std::string *line_buffe
   
   if (!n)
   {
-    PyErr_Print();  
+    PythonContext::log_python_error("Error running buffer");
     
     if (line_buffer)
       line_buffer->clear();
@@ -1218,7 +1507,7 @@ int PythonContext::run_buffer(const std::string &buffer, std::string *line_buffe
   if (!result)
   {
     if (PyErr_Occurred())
-      PyErr_Print();
+      PythonContext::log_python_error("Error running buffer");
     
     return -1;
   }
@@ -1234,11 +1523,13 @@ int PythonContext::run_buffer(const std::string &buffer, std::string *line_buffe
 // template of a python function to create a wrapper class for a GRT class
 static const char *create_class_template=
 "class %s(grt.Object):\n"
+"  __grtclassname__ = \"%s\"\n"
 "  def __init__(self, classname = None, wrapobj = None):\n"
 "    grt.Object.__init__(self, classname, wrapobj)";
 
 static const char *create_class_template_sub=
 "class %s(%s):\n"
+"  __grtclassname__ = \"%s\"\n"
 "  def __init__(self, classname = '%s', wrapobj = None):\n"
 "    %s.__init__(self, classname, wrapobj)";
 
@@ -1257,16 +1548,18 @@ static void create_class_wrapper(grt::MetaClass *meta, PyObject *locals)
     script= strfmt(create_class_template_sub, 
                    flatten_class_name(meta->name()).c_str(), parname.c_str(), 
                    meta->name().c_str(),
+                   meta->name().c_str(),
                    parname.c_str());
   }
   else
   {
     script= strfmt(create_class_template, 
-                   flatten_class_name(meta->name()).c_str());
+                   flatten_class_name(meta->name()).c_str(),
+                   meta->name().c_str());
   }
 
   if (!PyRun_String(script.c_str(), Py_single_input, locals, locals))
-    PyErr_Print();
+    PythonContext::log_python_error(NULL);
 }
 
 /** Refresh Python environment with GRT information.
@@ -1299,13 +1592,62 @@ int PythonContext::refresh()
     PyObject *r= PyObject_Call(_grt_module_class, Py_BuildValue("(s)", (*iter)->name().c_str()), NULL);
 
     if (!r)
-      PyErr_Print();
+      PythonContext::log_python_error("Error refreshing grt modules");
     else
       if (PyModule_AddObject(_grt_modules_module, (char*)(*iter)->name().c_str(), r) < 0)
-        PyErr_Print();
+        PythonContext::log_python_error("Error refreshing grt modules");
   }
 
   return 0;
 }
 
+
+void PythonContext::log_python_error(const char *message)
+{
+  PythonContext *ctx = PythonContext::get();
+  if (ctx)
+  {
+    if (message)
+      base::Logger::log(base::Logger::LogError, "python", "%s", message);
+    PyObject *grt_dict = PyModule_GetDict(ctx->get_grt_module());
+    PyObject *_stderr = PyDict_GetItemString(grt_dict, "_log_stderr"); 
+    PyObject *old_stderr = PySys_GetObject((char*)"stderr");
+    Py_INCREF(old_stderr);
+    if (_stderr)
+      PySys_SetObject((char*)"stderr", _stderr);
+    PyErr_Print();
+    
+    if (_stderr)
+      PySys_SetObject((char*)"stderr", old_stderr);
+    Py_DECREF(old_stderr);
+  }
+}
+
+
+
+// script to be executed once GRT is initialized
+// executed in the grt module namespace
+static const char *post_init_script = 
+"import grt\n"
+"class _grtFileRedirector:\n"
+"    def __init__(self, logger):\n"
+"        self.logger = logger\n"
+"    def write(self, text):\n"
+"        if type(text) not in (str, unicode):\n"
+"            text = str(text)\n"
+"        grt.send_output(text)\n"
+"        self.logger(grt._log_domain, text)\n"
+"grt._log_domain = 'python'\n"
+"grt._log_stdout = _grtFileRedirector(grt.log_info)\n"
+"grt._log_stderr = _grtFileRedirector(grt.log_error)\n";
+
+void PythonContext::run_post_init_script()
+{
+  WillEnterPython lock;
+
+  if (PyRun_SimpleString((char*)post_init_script) < 0)
+    PythonContext::log_python_error("Error running post-init script:");
+}
+
+
 #endif // ENABLE_PYTHON_MODULES
