@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,12 +22,16 @@
 #define _GRT_PYTHON_CONTEXT_H_
 
 
-#if defined(_WIN32) || defined(__APPLE__)
+#if defined(_WIN32)
 # include <Python/Python.h>
 #else
 # include <Python.h>
 #endif
 
+// Undef junk #defined by Python headers
+#undef tolower
+#undef toupper
+
 #include "grtpp.h"
 
 #if (PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION < 5)
@@ -53,6 +57,12 @@ namespace grt {
     {
     }
 
+    AutoPyObject(const AutoPyObject &other)
+    : object(other.object)
+    {
+      Py_XINCREF(object);
+    }
+    
     AutoPyObject(PyObject *py, bool retain=true)
     : object(py)
     {
@@ -108,7 +118,7 @@ namespace grt {
     PyObject *from_grt(const ValueRef &value);
     grt::ValueRef from_pyobject(PyObject *object);
     grt::ValueRef from_pyobject(PyObject *object, const grt::TypeSpec &expected_type);
-    bool pystring_to_string(PyObject *str, std::string &ret_string);
+    bool pystring_to_string(PyObject *str, std::string &ret_string, bool convert = false);
 
     int run_file(const std::string &file, bool interactive);
     int run_buffer(const std::string &buffer, std::string *line_buffer=0);
@@ -131,15 +141,20 @@ namespace grt {
     std::string get_cwd() const { return _cwd; }
 
     boost::function<std::string ()> stdin_readline_slot;
-    
-    // slots to be assigned by GRTManager to allow python modules to register output handlers
-    boost::function<void (boost::function<bool (std::string)>) > push_output_handler;
-    boost::function<void ()> pop_output_handler;
-    
+
+    static void set_user_interrupted(const grt::user_cancelled &exc);
+    static void set_db_access_denied(const grt::db_access_denied &exc);
+    static void set_db_login_error(const grt::db_login_error &exc);
     static void set_python_error(const grt::type_error &exc, const std::string &location="");
     static void set_python_error(const grt::bad_item &exc, const std::string &location="");
     static void set_python_error(const std::exception &exc, const std::string &location="");
     
+    static void log_python_error(const char *message);
+    
+    PyObject *user_interrupted_error() { return _grt_user_interrupt_error; }
+    PyObject *db_access_denied_error() { return _grt_db_access_denied_error; }
+    PyObject *db_login_error() { return _grt_db_login_error; }
+    
   protected:
     GRT *_grt;    
     std::string _cwd;
@@ -155,6 +170,10 @@ namespace grt {
     AutoPyObject _grt_object_class;
     AutoPyObject _grt_method_class;
     
+    AutoPyObject _grt_user_interrupt_error;
+    AutoPyObject _grt_db_access_denied_error;
+    AutoPyObject _grt_db_login_error;
+    
     std::map<std::string, AutoPyObject> _grt_class_wrappers;
 
   private:
@@ -169,6 +188,8 @@ namespace grt {
     void init_grt_dict_type();
     void init_grt_object_type();
 
+    void run_post_init_script();
+    
     PyThreadState*  _main_thread_state;
 
   };
