@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -63,140 +63,6 @@ using namespace tut;
 #define TRIGGER_TESTS  1
 #define CREATE_TESTS   1
 
-sql::ConnectionWrapper create_connection_for_import(GRT *grt)
-{
-  // init database connection
-  db_mgmt_ConnectionRef connectionProperties(grt);
-
-  grt::DictRef conn_params(grt);
-  conn_params.set("host", grt::StringRef(test_params->get_host_name()));
-  conn_params.set("port", grt::IntegerRef(test_params->get_port()));
-  conn_params.set("username", grt::StringRef(test_params->get_user_name()));
-  conn_params.set("password", grt::StringRef(test_params->get_password()));
-  grt::replace_contents(connectionProperties->parameterValues(), conn_params);
-
-  db_mgmt_DriverRef driverProperties= db_mgmt_DriverRef::cast_from(grt->get("/rdbms/drivers/0/"));
-  connectionProperties->driver(driverProperties);
-
-  sql::DriverManager *dm= sql::DriverManager::getDriverManager();
-  return dm->getConnection(connectionProperties);
-}
-
-void populate_grt(GRT *grt, const char *rdbms_info_path)
-{
-  if(rdbms_info_path == NULL)
-    rdbms_info_path= "../../modules/db.mysql/res/mysql_rdbms_info.xml";
-
-  workbench_WorkbenchRef wb(grt);
-  workbench_DocumentRef doc(grt);
-  doc->owner(wb);
-  wb->doc(doc);
-
-  grt->set("/wb", wb);
-
-  db_mgmt_ManagementRef mgmt(grt); 
-  wb->rdbmsMgmt(mgmt);
-  mgmt->owner(wb);
-
-  ListRef<db_DatatypeGroup> grouplist= 
-    ListRef<db_DatatypeGroup>::cast_from(grt->unserialize("../../res/grtdata/db_datatype_groups.xml"));
-  grt::replace_contents(mgmt->datatypeGroups(), grouplist);
-
-  db_mgmt_RdbmsRef rdbms= db_mgmt_RdbmsRef::cast_from(grt->unserialize(rdbms_info_path));
-  ensure("db_mgmt_RdbmsRef initialization", rdbms.is_valid());
-  grt->set("/rdbms", rdbms);
-
-  mgmt->rdbms().insert(rdbms);
-  rdbms->owner(mgmt);
-
-  app_OptionsRef options(grt);
-  options->owner(wb);
-  wb->options(options);
-}
-
-db_mysql_CatalogRef create_empty_catalog_for_import(grt::GRT *grt)
-{
-  db_mgmt_RdbmsRef rdbms= db_mgmt_RdbmsRef::cast_from(grt->get("/rdbms"));
-
-  db_mysql_CatalogRef cat(grt);
-  cat->version(rdbms->version());
-  cat->name("default");
-  cat->oldName("default");
-  grt::replace_contents(cat->simpleDatatypes(), rdbms->simpleDatatypes());
-
-  return cat;
-}
-
-db_mysql_CatalogRef db_rev_eng_schema(bec::GRTManager *grtm, const std::list<std::string>& schema_names)
-{
-  sql::ConnectionWrapper conn= create_connection_for_import(grtm->get_grt());
-
-  Db_rev_eng db_rev_eng;
-  db_rev_eng.grtm(grtm);
-  ///
-  db_mgmt_ConnectionRef connectionProperties(grtm->get_grt());
-
-  grt::DictRef conn_params(grtm->get_grt());
-  conn_params.set("host", grt::StringRef(test_params->get_host_name()));
-  conn_params.set("port", grt::IntegerRef(test_params->get_port()));
-  conn_params.set("username", grt::StringRef(test_params->get_user_name()));
-  conn_params.set("password", grt::StringRef(test_params->get_password()));
-  grt::replace_contents(connectionProperties->parameterValues(), conn_params);
-
-  db_mgmt_DriverRef driverProperties= db_mgmt_DriverRef::cast_from(grtm->get_grt()->get("/rdbms/drivers/0/"));
-  connectionProperties->driver(driverProperties);
-////
-  db_rev_eng.db_conn()->set_connection(connectionProperties);
-  db_rev_eng.db_conn()->init(db_mgmt_ManagementRef::cast_from(grtm->get_grt()->get("/wb/rdbmsMgmt")),false);
-  db_rev_eng.db_conn()->set_active_rdbms(0);
-  db_rev_eng.db_conn()->set_active_driver(0);
-
-  DbDriverParams *dp= db_rev_eng.db_conn()->get_db_driver_param_handles();
-  dp->get("ctrl__hostName")->set_value(grt::StringRef(test_params->get_host_name()));
-  dp->get("ctrl__port")->set_value(grt::IntegerRef(test_params->get_port()));
-  dp->get("ctrl__userName")->set_value(grt::StringRef(test_params->get_user_name()));
-//  dp->get("ctrl__password")->set_value(grt::StringRef(test_params->get_password()));
-
-  {
-    std::vector<std::string> schemata;
-    db_rev_eng.load_schemata(schemata);
-  }
-
-  {
-    std::vector<std::string> selection;
-    std::copy(schema_names.begin(), schema_names.end(), std::back_inserter(selection));
-    db_rev_eng.schemata_selection(selection, false);
-  }
-
-  db_rev_eng.load_db_objects(Db_rev_eng::dbotTable);
-  db_rev_eng.load_db_objects(Db_rev_eng::dbotView);
-  db_rev_eng.load_db_objects(Db_rev_eng::dbotRoutine);
-  db_rev_eng.load_db_objects(Db_rev_eng::dbotTrigger);
-
-  db_mysql_CatalogRef cat= create_empty_catalog_for_import(grtm->get_grt());
-
-  db_rev_eng.model_catalog(cat);
-  db_rev_eng.exec_task(true);
-
-  return cat;
-}
-
-int execute_script_callback(const mysql_parser::MyxStatementParser *splitter, const char *sql, void *user_data);
-/*{
-  sql::Statement *stmt= static_cast<sql::Statement *>(user_data);
-  stmt->execute(sql);
-  return 1;
-}
-*/
-
-void execute_script(sql::Statement *stmt, const std::string& script,GRTManager* grtm);/*
-{
-  mysql_parser::CHARSET_INFO *cs= mysql_parser::get_charset_by_name("utf8_bin", MYF(0));
-  mysql_parser::MyxStatementParser(cs).process(std::istringstream(script), execute_script_callback, stmt, MYX_SPM_NORMAL_MODE);
-  Mysql_sql_parser_fe sql_parser_fe(tester.grt);
-}
-*/
-
 BEGIN_TEST_DATA_CLASS(grtdiff_alter_test)
 protected:
   WBTester tester;
@@ -205,89 +71,37 @@ protected:
   grt::DbObjectMatchAlterOmf omf;
   sql::ConnectionWrapper connection;
 
-//  void execute_alter(std::auto_ptr<sql::Statement>& stmt, grt::DictRef alter_map);
-//  void execute_alter(std::auto_ptr<sql::Statement>& stmt, grt::StringListRef alter_map);
-END_TEST_DATA_CLASS
+  TEST_DATA_CONSTRUCTOR(grtdiff_alter_test)
+  {
+      omf.dontdiff_mask = 3;
+      diffsql_module= tester.grt->get_native_module<DbMySQLImpl>();
+      ensure("DiffSQLGen module initialization", NULL != diffsql_module);
+
+      // init datatypes
+      populate_grt(tester.grt, NULL, tester);
+
+      // init database connection
+      connection= tester.create_connection_for_import();
+
+      sql_parser= SqlFacade::instance_for_rdbms_name(tester.grt, "Mysql");
+      ensure("failed to get sqlparser module", (NULL != sql_parser));
+  }
+
+  END_TEST_DATA_CLASS
 
 TEST_MODULE(grtdiff_alter_test, "GRT: diff alter");
 
-TEST_FUNCTION(1)
-{
-  omf.dontdiff_mask = 3;
-  diffsql_module= tester.grt->get_native_module<DbMySQLImpl>();
-  ensure("DiffSQLGen module initialization", NULL != diffsql_module);
-
-  // init datatypes
-  populate_grt(tester.grt, NULL);
-
-  // init database connection
-  connection= create_connection_for_import(tester.grt);
-
-  sql_parser= SqlFacade::instance_for_rdbms_name(tester.grt, "Mysql");
-  ensure("failed to get sqlparser module", (NULL != sql_parser));
-}
-
-void execute_alter(std::auto_ptr<sql::Statement>& stmt, grt::StringListRef alter_map)
-{
-  for(size_t sz= alter_map.count(), i= 0; i < sz; i++)
-  {
-    grt::StringRef str= alter_map.get(i);
-#if VERBOSE_TESTING
-    std::cout << "Q: " << str.c_str() << std::endl;
-#endif
-    stmt->execute(str.c_str());
-  }
-}
-
 // db rev-eng test
 TEST_FUNCTION(3)
 {
 
   std::list<std::string> schemata;
   schemata.push_back("grtdiff_alter_test");
-  db_mysql_CatalogRef cat= db_rev_eng_schema(tester.wb->get_grt_manager(), schemata);
+  tester.grt->get_undo_manager()->disable();
+  db_mysql_CatalogRef cat= tester.db_rev_eng_schema(schemata);
+  tester.wb->close_document();
+  tester.wb->close_document_finish();
 }
-/*
-void tut::Test_object_base<grtdiff_alter_test>::execute_alter(std::auto_ptr<sql::Statement>& stmt, grt::DictRef alter_map)
-{
-  for (grt::DictRef::const_iterator iterator= alter_map.begin(); iterator != alter_map.end(); iterator++)
-  {
-    std::string key= iterator->first;
-    grt::ValueRef value= iterator->second;
-
-    if(grt::StringListRef::can_wrap(value))
-    {
-      grt::StringListRef list= grt::StringListRef::cast_from(value);
-      for(size_t listcount= list.count(), j= 0; j < listcount; j++)
-      {
-#if VERBOSE_TESTING
-        std::cout << "Q: " << list.get(j).c_str() << std::endl;
-#endif
-        stmt->execute(list.get(j).c_str());
-      }
-    }
-    else if(grt::StringRef::can_wrap(value))
-    {
-#if VERBOSE_TESTING
-      std::cout << "Q: " << grt::StringRef::cast_from(value).c_str() << std::endl;
-#endif
-      stmt->execute(grt::StringRef::cast_from(value).c_str());
-    }
-  }
-}
-
-void tut::Test_object_base<grtdiff_alter_test>::execute_alter(std::auto_ptr<sql::Statement>& stmt, grt::StringListRef alter_map)
-{
-  for(size_t sz= alter_map.count(), i= 0; i < sz; i++)
-  {
-    grt::StringRef str= alter_map.get(i);
-#if VERBOSE_TESTING
-    std::cout << "Q: " << str.c_str() << std::endl;
-#endif
-    stmt->execute(str.c_str());
-  }
-}
-*/
 
   static struct 
   {
@@ -657,13 +471,13 @@ void tut::Test_object_base<grtdiff_alter_test>::execute_alter(std::auto_ptr<sql:
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) DEFAULT NULL, `id2` int(11) DEFAULT NULL) ENGINE=InnoDB DEFAULT CHARSET=latin1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id2` int(11) DEFAULT NULL, `id` int(11) DEFAULT NULL, t TEXT) ENGINE=InnoDB DEFAULT CHARSET=latin1"
-    }, {
+    },{
       "C< C+ C",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) DEFAULT NULL, `id2` int(11) DEFAULT NULL) ENGINE=InnoDB DEFAULT CHARSET=latin1",
       "CREATE TABLE grtdiff_alter_test.t1 (`id2` int(11) DEFAULT NULL, t TEXT, `id` int(11) DEFAULT NULL) ENGINE=InnoDB DEFAULT CHARSET=latin1"
-    }, {
+    },{
       "C> C- C",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1",
@@ -930,7 +744,7 @@ void tut::Test_object_base<grtdiff_alter_test>::execute_alter(std::auto_ptr<sql:
       "Change COLLATE attribute",
       "grtdiff_alter_test.t1",
       "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
-      "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci",
+      "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci",
       "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_bin"
     }, {
       "Change PACK_KEYS attribute",
@@ -1085,7 +899,8 @@ void tut::Test_object_base<grtdiff_alter_test>::execute_alter(std::auto_ptr<sql:
       " PARTITION p1 VALUES LESS THAN (1991) ENGINE = InnoDB,"
       " PARTITION p2 VALUES LESS THAN (1996) ENGINE = InnoDB,"
       " PARTITION p3 VALUES LESS THAN (2001) ENGINE = InnoDB,"
-      " PARTITION p4 VALUES LESS THAN (2005) ENGINE = InnoDB)"
+      " PARTITION p4 VALUES LESS THAN (2005) ENGINE = InnoDB,"
+      " PARTITION p5 VALUES LESS THAN (MAXVALUE) ENGINE = InnoDB)"
     }, {
       "Remove range partitioning ",
       "grtdiff_alter_test.t1",
@@ -1368,6 +1183,21 @@ void tut::Test_object_base<grtdiff_alter_test>::execute_alter(std::auto_ptr<sql:
       "CREATE TABLE grtdiff_alter_test.frn_keys_check_t1( `id` int(11) NOT NULL,  PRIMARY KEY (`id`) USING BTREE) ENGINE=InnoDB DEFAULT CHARSET=latin1; CREATE TABLE `grtdiff_alter_test`.`frn_keys_check_t2` (`id` int(11) NOT NULL,  KEY `fid` (`id`),  CONSTRAINT `fid` FOREIGN KEY (`id`) REFERENCES `grtdiff_alter_test`.`frn_keys_check_t1` (`id`) ON DELETE CASCADE ON UPDATE CASCADE) ENGINE=InnoDB DEFAULT CHARSET=latin1;"
     },
 #endif
+      {
+      "Change CHARSET attribute to server default",
+      "grtdiff_alter_test.t1",
+      "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
+      "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=latin2",
+      "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT"
+      },
+
+      {
+      "Change COLLATE attribute to server default",
+      "grtdiff_alter_test.t1",
+      "DROP TABLE IF EXISTS grtdiff_alter_test.t1;",
+      "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=latin1 COLLATE=latin1_german1_ci",
+      "CREATE TABLE grtdiff_alter_test.t1 (`id` int(11) NOT NULL DEFAULT '0') ENGINE=MyISAM DEFAULT CHARSET=latin1"
+    },
     {
       NULL, NULL, NULL
     }
@@ -1434,7 +1264,7 @@ TEST_FUNCTION(6)
     grt::NormalizedComparer normalizer(tester.grt);
     normalizer.init_omf(&omf);
     alter_change= diff_make(org_cat, mod_cat, &omf);
-    assert(alter_change);
+    ensure("Empty alter:",alter_change);
 
 #if VERBOSE_TESTING
     alter_change->dump_log(0);
@@ -1442,14 +1272,16 @@ TEST_FUNCTION(6)
 
     // 1. generate alter
     grt::StringListRef alter_map(tester.grt);
+    grt::ListRef<GrtNamedObject> alter_object_list(tester.grt);
     grt::DictRef options(tester.grt);
     options.set("UseFilteredLists", grt::IntegerRef(0));
     options.set("OutputContainer", alter_map);
+    options.set("OutputObjectContainer", alter_object_list);
+    options.set("CaseSensitive", grt::IntegerRef(omf.case_sensitive));
 
-    char buf1[128];
-    sprintf(buf1, "%p", alter_change.get());
-
-    diffsql_module->generateSQL(org_cat, options, std::string(buf1));
+    diffsql_module->generateSQL(mod_cat, options, alter_change);
+    diffsql_module->makeSQLSyncScript(options, alter_map, alter_object_list);
+    std::string export_sql_script= options.get_string("OutputScript");
 
     // 2. apply it to server
     std::auto_ptr<sql::Statement> stmt(connection->createStatement());
@@ -1458,7 +1290,7 @@ TEST_FUNCTION(6)
     {
       execute_script(stmt.get(), data[i].cleanup,tester.wb->get_grt_manager());
       execute_script(stmt.get(), data[i].org,tester.wb->get_grt_manager());
-      execute_alter(stmt, alter_map);
+      execute_script(stmt.get(), export_sql_script,tester.wb->get_grt_manager());
     }
     catch(sql::SQLException &ex)
     {
@@ -1470,7 +1302,10 @@ TEST_FUNCTION(6)
     std::list<std::string> schemata;
     schemata.push_back("grtdiff_alter_test");
     schemata.push_back("grtdiff_alter_test2");
-    db_mysql_CatalogRef cat= db_rev_eng_schema(tester.wb->get_grt_manager(), schemata);
+    db_mysql_CatalogRef cat= tester.db_rev_eng_schema(schemata);
+    if((cat->schemata().get(0).is_valid()) && (cat->schemata().get(0)->name() == "mydb"))
+      cat->schemata().remove(0);
+    mod_cat->oldName("");
 
     // 3a. cleanup the server
     execute_script(stmt.get(), data[i].cleanup,tester.wb->get_grt_manager());
@@ -1493,11 +1328,21 @@ TEST_FUNCTION(6)
 
     normalizer.init_omf(&omf);
     empty_change= diff_make(cat, mod_cat, &omf);
+    tester.wb->close_document();
+    tester.wb->close_document_finish();
 
     if (empty_change)
-      empty_change->dump_log(0);
-
-    ensure(data[i].description, empty_change == NULL);
+    {
+        empty_change->dump_log(0);
+        alter_map.clear();
+        diffsql_module->generateSQL(mod_cat, options, alter_change);
+        diffsql_module->makeSQLSyncScript(options, alter_map, alter_object_list);
+        std::cout<<"Iteration:"<<i<<std::endl;
+        std::cout<<export_sql_script<<std::endl;
+        std::string export_sql_script= options.get_string("OutputScript");
+        std::cout<<export_sql_script;
+        fail(data[i].description);
+    }
   }
 }
 
@@ -1519,7 +1364,7 @@ TEST_FUNCTION(7)
     execute_script(stmt.get(), "CREATE DATABASE IF NOT EXISTS grtdiff_alter_test /*!40100 DEFAULT CHARACTER SET latin1 */",tester.wb->get_grt_manager());
   }
 
-  for(int i= 0; data[i].description != NULL; i++)
+  for(int i= 2; data[i].description != NULL; i++)
 	  for (int j = 0; j <= 1; ++j)
   {
 #if VERBOSE_TESTING
@@ -1564,14 +1409,14 @@ TEST_FUNCTION(7)
     grt::NormalizedComparer normalizer(tester.grt);
     normalizer.init_omf(&omf);
     alter_change= diff_make(org_cat, mod_cat, &omf);
-    assert(alter_change);
+    ensure("Empty alter:",alter_change);
 
 #if VERBOSE_TESTING
     alter_change->dump_log(0);
 #endif
 
 	const char TemplateFile[] = //"E:\\projects\\mysql\\WB 5.1\\workbench\\testing\\tut\\"
-		"modules\\db.mysql\\res\\reporting\\Basic_Text.tpl\\basic_text_report.txt.tpl";
+		"data\\reporting\\Basic_Text.tpl\\basic_text_report.txt.tpl";
     // 1. generate alter
     grt::StringListRef alter_map(tester.grt);
     grt::DictRef options(tester.grt);
@@ -1580,22 +1425,25 @@ TEST_FUNCTION(7)
 	options.set("TemplateFile", grt::StringRef(TemplateFile));
 	options.set("UseShortNames", grt::IntegerRef(j));
 
-    char buf1[256];
+    
+	grt::StringRef report = diffsql_module->generateReport(org_cat, options, alter_change);
 
-	sprintf(buf1, "%p", alter_change.get());
-
-
-
-	grt::StringRef report = diffsql_module->generateReport(org_cat, options, std::string(buf1));
-
-	sprintf(buf1, "modules\\db.mysql\\res\\reporting\\Basic_Text.tpl\\reports\\testres%s%d.txt",j?"_longname":"_shortname", i);
+    char buf1[1000];
+	g_snprintf(buf1, sizeof(buf1), "data\\reporting\\Basic_Text.tpl\\reports\\testres%s%d.txt",j?"_longname":"_shortname", i);
 	std::ifstream rep;
 	rep.open(buf1);
 	std::string str((std::istreambuf_iterator<char>(rep)), std::istreambuf_iterator<char>());
-	ensure(buf1,report == str);
 
-
-	// Test Data generation
+    if (report != str)
+    {
+        alter_change->dump_log(0);
+        int pp = strcmp(report.c_str(),str.c_str());
+        std::cout<<report.c_str()<<std::endl;
+        std::cout<<"=======================================================================================================================\n";
+        std::cout<<str<<std::endl;
+        fail(buf1);
+    }
+// Test Data generation
 /*
 	sprintf(buf1, "testres%s%d.txt",j?"_longname":"_shortname", i);
 	std::ofstream out;
