@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,12 +27,11 @@
 #include "base/string_utilities.h"
 #include <sstream>
 #include <iterator>
-#include <boost/lambda/lambda.hpp>
+#include <boost/lambda/bind.hpp>
 
 using namespace grt;
 using namespace base;
 
-
 class TableStorageEngines
 {
 public:
@@ -70,9 +69,8 @@ static TableStorageEngines __table_storage_engines;
 Mysql_sql_parser::Null_state_keeper::~Null_state_keeper()
 {
   _sql_parser->_fk_refs.clear();
-  static const Parse_result pr = pr_irrelevant;
-  boost::function<Parse_result ()> f = boost::lambda::var(pr);
-  _sql_parser->_process_specific_create_statement.clear();
+  boost::function<Parse_result ()> f = boost::lambda::constant(pr_irrelevant);
+  _sql_parser->_process_specific_create_statement.clear(); //= boost::bind(f);
   _sql_parser->_datatype_cache= grt::DictRef();
   _sql_parser->_created_objects= grt::ListRef<GrtObject>();
   _sql_parser->_processing_create_statements= true;
@@ -170,7 +168,7 @@ int Mysql_sql_parser::parse_sql_script(db_CatalogRef &catalog, const std::string
 
   set_options(options);
 
-  add_log_message("Started parsing MySQL SQL script.", 0);
+  add_log_message("Started parsing MySQL SQL script.", 3);
 
   set_progress_state(0.f, _("Parsing MySQL SQL Script..."));
 
@@ -229,7 +227,8 @@ int Mysql_sql_parser::parse_sql_script(db_CatalogRef &catalog, const std::string
   if (from_file)
     res= Mysql_sql_parser_base::parse_sql_script_file(sql_parser_fe, *sql_script);
   else
-    res= Mysql_sql_parser_base::parse_sql_script(sql_parser_fe, *sql_script);
+    // TODO: optimize to avoid copies of the input (especially for large scripts).
+    res= Mysql_sql_parser_base::parse_sql_script(sql_parser_fe, sql_script->c_str());
 
   set_progress_state(0.9f, _("Creating foreign key references..."));
 
@@ -251,7 +250,7 @@ int Mysql_sql_parser::parse_sql_script(db_CatalogRef &catalog, const std::string
     oss << "Finished parsing MySQL SQL script. Totally processed statements: successful (" << _processed_obj_count <<
       "), errors (" << _err_count <<
        "), warnings (" << _warn_count << ").";
-    add_log_message(oss.str(), 0);
+    add_log_message(oss.str(), 3);
   }
 
   return res;
@@ -437,7 +436,7 @@ void Mysql_sql_parser::log_db_obj_operation(const std::string &op_name, const Gr
   if (obj1.is_valid()) text.append(obj1->name());
   if (obj2.is_valid()) text.append(".").append(obj2->name());
   if (obj3.is_valid()) text.append(".").append(obj3->name());
-  add_log_message(text, 0);
+  add_log_message(text, 3);
 }
 
 
@@ -450,6 +449,7 @@ void Mysql_sql_parser::set_fk_references()
     // referred table
     db_mysql_SchemaRef ref_schema= find_named_object_in_list(schemata, i->ref_schema_name(), _case_sensitive_identifiers);
     db_mysql_TableRef ref_table;
+    if (ref_schema.is_valid())
     {
       ref_table= find_named_object_in_list(ref_schema->tables(), i->ref_table_name(), _case_sensitive_identifiers);
       if (!ref_table.is_valid())
@@ -470,6 +470,8 @@ void Mysql_sql_parser::set_fk_references()
       }
       (*i).operator db_ForeignKeyRef &()->referencedTable(ref_table);
     }
+    else
+      add_log_message(std::string("Could not find refschema ") + i->ref_schema_name() + " for reftable " + i->ref_table_name(), 1);
 
     // ref columns
     int column_index= 0;
@@ -727,7 +729,12 @@ void Mysql_sql_parser::process_field_type_item(const SqlAstNode *item, db_mysql_
     {
       std::string scale= "";
       std::string precision= "";
-      process_float_options_item(item->subitem(sql::_float_options), &precision, &scale);
+      const SqlAstNode* float_item = item->subitem(sql::_float_options);
+      if (float_item)
+        process_float_options_item(float_item, &precision, &scale);
+      const SqlAstNode* real_item = item->subitem(sql::_real_type);
+      if (real_item)
+        process_float_options_item(real_item, &precision, &scale);
       if (!scale.empty())
         SET_INT(column->scale, scale)
       if (!precision.empty())
@@ -797,7 +804,7 @@ void Mysql_sql_parser::process_field_attributes_item(const SqlAstNode *item, db_
         if (subitem->subitem(sql::_AUTO_INC))
         {
           // set auto_increment only for numeric types
-          if (column->simpleType().is_valid() &&
+          if (column->simpleType().is_valid() && column->simpleType()->group().is_valid() &&
             are_strings_eq_ci(column->simpleType()->group()->name(), "NUMERIC"))
             column->autoIncrement(1);
         }
@@ -843,7 +850,7 @@ void Mysql_sql_parser::process_field_attributes_item(const SqlAstNode *item, db_
         {
           std::string text;
           // it was decided that 'ON UPDATE CURRENT_TIMESTAMP' should be stored in defaultValue to not create new rarely-used field
-          if (column->defaultValue().valueptr())
+          if (column->defaultValue().is_valid())
           {
             text= *column->defaultValue();
             text.append(" ");
@@ -1227,6 +1234,11 @@ void Mysql_sql_parser::process_fk_references_item(const SqlAstNode *tree, db_mys
   db_mysql_SchemaRef ref_schema;
   db_mysql_TableRef ref_table;
   {
+    // reset reuse_existing that could've been set for the owner table object, but would
+    // prevent a stub schema to be created by process_obj_full_name_item() in case its necessary
+    Val_keeper<bool> reusing_existing_objects_keeper(&_reusing_existing_obj);
+    _reusing_existing_obj= false;
+    
     std::string ref_obj_name= process_obj_full_name_item(tree->subitem(sql::_table_ident), &ref_schema);
     fk_ref.ref_schema_name(ref_schema->name());
     fk_ref.ref_table_name(ref_obj_name);
@@ -1276,11 +1288,13 @@ void Mysql_sql_parser::process_index_options_item(db_mysql_IndexRef &obj, const
       case sql::_spatial_key_opt:
         if (const SqlAstNode *aux_item= item->subseq(sql::_key_using_alg))
           process_index_kind_item(obj, aux_item->subitem(sql::_btree_or_rtree));
-        else if (const SqlAstNode *aux_item= item->subitem(sql::_all_key_opt, sql::_KEY_BLOCK_SIZE))
+        else if (/*const SqlAstNode *aux_item= */item->subitem(sql::_all_key_opt, sql::_KEY_BLOCK_SIZE))
           SET_INT_SI(obj->keyBlockSize, item, sql::_all_key_opt, sql::_ulong_num)
         else if (item->subseq(sql::_WITH, sql::_PARSER_SYM))
           SET_STR_SI(obj->withParser, item, sql::_IDENT_sys);
         break;
+      default:
+        break;
       }
     }
   }
@@ -1536,7 +1550,7 @@ Mysql_sql_parser::process_create_table_statement(const SqlAstNode *tree)
       }
     };  // class Partition_definition
 
-    const SqlAstNode *partition= create2_item->subitem(sql::_create2a, sql::_opt_partitioning, sql::_partitioning, sql::_partition);
+    const SqlAstNode *partition= create2_item->subitem(sql::_create2a, sql::_opt_create_partitioning, sql::_partitioning, sql::_partition);
     if (partition)
     {
       bool is_range_or_list_partition= (partition->subitem(sql::_part_type_def, sql::_RANGE_SYM) != NULL)
@@ -1815,12 +1829,19 @@ Mysql_sql_parser::process_create_routine_statement(const SqlAstNode *tree)
   if (!routine_tail)
     return pr_irrelevant;
 
+  std::string routine_type;
+  if (routine_tail->subitem(sql::_FUNCTION_SYM))
+    routine_type = "function";
+  else if (routine_tail->subitem(sql::_PROCEDURE_SYM))
+    routine_type = "procedure";
+  
   db_mysql_SchemaRef schema;
   const SqlAstNode *routine_ident_item= routine_tail->subitem(sql::_sp_name);
   std::string obj_name= process_obj_full_name_item(routine_ident_item, &schema);
   step_progress(obj_name);
 
-  db_mysql_RoutineRef obj= create_or_find_named_obj(schema->routines(), obj_name, false, schema); // mysql routines are always case-insensitive
+  db_mysql_RoutineRef obj= create_or_find_named_routine(schema->routines(), obj_name, false, routine_type,
+                                                        schema); // mysql routines are always case-insensitive
 
   // name
   std::string name= process_obj_full_name_item(routine_ident_item, NULL);
@@ -1830,14 +1851,13 @@ Mysql_sql_parser::process_create_routine_statement(const SqlAstNode *tree)
   {
     const SqlAstNode *param_list_item= NULL;
 
-    if (routine_tail->subitem(sql::_PROCEDURE_SYM))
+    obj->routineType(routine_type);
+    if (routine_type == "procedure")
     {
-      obj->routineType("procedure");
       param_list_item= routine_tail->subitem(sql::_sp_pdparam_list, sql::_sp_pdparams);
     }
-    else if (routine_tail->subitem(sql::_FUNCTION_SYM))
+    else if (routine_type == "function")
     {
-      obj->routineType("function");
       param_list_item= routine_tail->subitem(sql::_sp_fdparam_list, sql::_sp_fdparams);
 
       // return datatype
@@ -2605,3 +2625,52 @@ grt::Ref<T> Mysql_sql_parser::create_or_find_named_obj(const grt::ListRef<T>& ob
 
   return obj;
 }
+
+
+template <typename T>
+grt::Ref<T> Mysql_sql_parser::create_or_find_named_routine(const grt::ListRef<T>& obj_list, const std::string &obj_name, bool case_sensitive, const std::string &routine_type, const GrtNamedObjectRef &container1, const GrtNamedObjectRef &container2)
+{
+  std::string time= bec::fmttime(0, DATETIME_FMT);
+  
+  grt::Ref<T> obj;
+  if (grt::Ref<T>::can_wrap(get_active_object()))
+  {
+    obj= grt::Ref<T>::cast_from(get_active_object());
+    _reusing_existing_obj= true;
+  }
+  else
+  {
+    for (size_t c = obj_list.count(), i = 0; i < c; i++)
+    {
+      grt::Ref<T> item(obj_list[i]);
+      if (*item->routineType() == routine_type)
+      {
+        if (case_sensitive && item->name() == obj_name)
+        {
+          obj = item;
+          break;
+        }
+        else if (!case_sensitive && g_strcasecmp(item->name().c_str(), obj_name.c_str()) == 0)
+        {
+          obj = item;
+          break;
+        }
+      }
+    }
+    if (obj.is_valid())
+    {
+      blame_existing_obj(true, obj, container1, container2);
+      _reusing_existing_obj= true;
+    }
+    else
+    {
+      obj= grt::Ref<T>(_grt);
+      obj->owner(container2.is_valid() ? container2 : (container1.is_valid() ? container1 : GrtNamedObjectRef(_catalog)));
+      try { obj.set_member("createDate", StringRef(time)); } catch (std::exception&) {}
+    }
+  }
+  
+  try { obj.set_member("lastChangeDate", StringRef(time)); } catch (std::exception&) {}
+  
+  return obj;
+}
\ No newline at end of file
