@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -81,13 +81,11 @@ protected:
 
   Parse_result process_select_statement(const SqlAstNode *tree)
   {
-    // look for existing limit clause
+    // Look for an existing limit clause in a normal SELECT.
     {
       static sql::symbol path1[]= { sql::_select_init, sql::_select_init2, sql::_select_part2, sql::_select_into, sql::_select_from, sql::_opt_limit_clause, sql::_limit_clause, sql::_ };
       static sql::symbol path2[]= { sql::_select_init, sql::_select_init2, sql::_select_part2, sql::_select_into, sql::_opt_limit_clause, sql::_limit_clause, sql::_ };
-      static sql::symbol path3[]= { sql::_select_init, sql::_union_opt, sql::_union_order_or_limit, sql::_order_or_limit, sql::_opt_limit_clause_init, sql::_limit_clause, sql::_ };
-      static sql::symbol path4[]= { sql::_select_init, sql::_union_opt, sql::_union_order_or_limit, sql::_order_or_limit, sql::_limit_clause, sql::_ };
-      static sql::symbol * paths[]= { path1, path2, path3, path4 };
+      static sql::symbol * paths[]= { path1, path2 };
 
       const SqlAstNode *limit_clause= tree->search_by_paths(paths, ARR_CAPACITY(paths));
       if (limit_clause)
@@ -123,11 +121,40 @@ protected:
     if (*_contains_limit_clause)
       return pr_processed;
 
+    // Look for an INTO clause, which signals us not to add a LIMIT clause either.
+    {
+      static sql::symbol path1[]= { sql::_select_init, sql::_select_init2, sql::_select_part2, sql::_select_into, sql::_into, sql::_ };
+      static sql::symbol * paths[]= { path1 };
+
+      const SqlAstNode *into_clause= tree->search_by_paths(paths, ARR_CAPACITY(paths));
+      if (into_clause != NULL)
+      {
+        *_contains_limit_clause = true; // Not entirely true but does the job.
+        return pr_processed;
+      }
+    }
+    
+    // Now look if this is a UNION statement (UNION at top level only, not sure if nested UNIONS are even possible).
+    // We don't add a LIMIT clause for such queries as we would have to handle each individual select
+    // statement then.
+    {
+      static sql::symbol path1[]= { sql::_select_init, sql::_select_init2, sql::_union_clause, sql::_ };
+      static sql::symbol * paths[]= { path1 };
+
+      const SqlAstNode *union_clause= tree->search_by_paths(paths, ARR_CAPACITY(paths));
+      if (union_clause != NULL)
+      {
+        *_contains_limit_clause = true;
+        return pr_processed;
+      }
+    }
+
     // look for clauses that go after limit clause
     {
       static sql::symbol path1[]= { sql::_select_init, sql::_select_init2, sql::_select_part2, sql::_select_into, sql::_into, sql::_ };
       static sql::symbol path2[]= { sql::_select_init, sql::_select_init2, sql::_select_part2, sql::_select_lock_type, sql::_ };
-      static sql::symbol * paths[]= { path1, path2 };
+      static sql::symbol path3[]= { sql::_select_init, sql::_select_init2, sql::_select_part2, sql::_select_into, sql::_select_from, sql::_procedure_clause, sql::_ };
+      static sql::symbol * paths[]= { path1, path2, path3 };
 
       const SqlAstNode *afterlimit_clause= tree->search_by_paths(paths, ARR_CAPACITY(paths));
       if (afterlimit_clause)
