@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,6 +24,7 @@
 #include "mysql_sql_parser_fe.h"
 #include "grtdb/charset_utils.h"
 #include "base/string_utilities.h"
+#include <boost/lambda/bind.hpp>
 
 
 using namespace grt;
@@ -35,16 +36,14 @@ static size_t MAX_INSERT_SQL_LENGTH= 8 * 1024; // determines max length of passe
 
 Mysql_sql_syntax_check::Null_state_keeper::~Null_state_keeper()
 {
-  _sql_parser->_check_sql_statement.disconnect();
+  boost::function<Parse_result ()> f = boost::lambda::constant(pr_irrelevant);
+  _sql_parser->_check_sql_statement = boost::bind(f);
 }
 #define NULL_STATE_KEEPER Null_state_keeper _nsk(this);
 
 
 Mysql_sql_syntax_check::Mysql_sql_syntax_check(grt::GRT *grt)
-:
-Sql_parser_base(grt),
-Mysql_sql_parser_base(grt),
-Sql_syntax_check(grt)
+  : Sql_parser_base(grt), Mysql_sql_parser_base(grt), Sql_syntax_check(grt)
 {
   NULL_STATE_KEEPER
 }
@@ -52,36 +51,95 @@ Sql_syntax_check(grt)
 
 Mysql_sql_syntax_check::Statement_type Mysql_sql_syntax_check::determine_statement_type(const std::string &sql)
 {
-  NULL_STATE_KEEPER
-
   typedef std::map<std::string, Statement_type> KnownStatementTypes;
   static KnownStatementTypes known_statement_types;
-  struct StaticInitializer
+  if (known_statement_types.size() == 0)
   {
-    StaticInitializer()
-    {
-      known_statement_types[""]= sql_empty;
-      known_statement_types["CREATE"]= sql_create;
-      known_statement_types["ALTER"]= sql_alter;
-      known_statement_types["DROP"]= sql_drop;
-      known_statement_types["INSERT"]= sql_insert;
-      known_statement_types["DELETE"]= sql_delete;
-      known_statement_types["UPDATE"]= sql_update;
-      known_statement_types["SELECT"]= sql_select;
-      known_statement_types["DESC"]= sql_describe;
-      known_statement_types["DESCRIBE"]= sql_describe;
-      known_statement_types["SHOW"]= sql_show;
-      known_statement_types["USE"]= sql_use;
-      known_statement_types["LOAD"]= sql_load;
-      known_statement_types["EDIT"]= sql_edit;
-      known_statement_types["SET"]= sql_set;
-    }
+    known_statement_types[""]= sql_empty;
+    known_statement_types["CREATE"]= sql_create;
+    known_statement_types["ALTER"]= sql_alter;
+    known_statement_types["DROP"]= sql_drop;
+    known_statement_types["INSERT"]= sql_insert;
+    known_statement_types["DELETE"]= sql_delete;
+    known_statement_types["UPDATE"]= sql_update;
+    known_statement_types["SELECT"]= sql_select;
+    known_statement_types["DESC"]= sql_describe;
+    known_statement_types["DESCRIBE"]= sql_describe;
+    known_statement_types["SHOW"]= sql_show;
+    known_statement_types["USE"]= sql_use;
+    known_statement_types["LOAD"]= sql_load;
+    known_statement_types["EDIT"]= sql_edit;
+    known_statement_types["SET"]= sql_set;
   };
-  static StaticInitializer static_initializer;
 
-  Mysql_sql_parser_fe sql_parser_fe(_grtm->get_grt());
-  std::string token= sql_parser_fe.get_first_sql_token(sql, "UNKNOWN");
-  KnownStatementTypes::iterator statement_type= known_statement_types.find(token);
+  // Scan for the first normal text (ignoring any leading comment).
+  std::string token;
+  const char *head = sql.c_str();
+  bool found = false;
+  while (*head != '\0')
+  {
+    switch (*head)
+    {
+      case '-': // Possible single line comment.
+        if (*(head + 1) == '-' && *(head + 2) == ' ')
+        {
+          // Skip everything until the end of the line.
+          head += 3;
+          while (*head != '\0' && *head != '\n' && *head != '\r')
+            head++;
+          head--;
+        }
+        break;
+
+      case '#': // MySQL single line comment.
+        while (*head != '\0' && *head != '\n' && *head != '\r')
+          head++;
+        head--;
+        break;
+
+      case '/':
+        {
+          // Multiline comment? If so, skip it.
+          if (*(head + 1) == '*')
+          {
+            head += 2;
+            while (true)
+            {
+              while (*head != '\0' && *head != '*')
+                head++;
+              if (*head == '\0') // Unfinished comment.
+              {
+                head--;
+                break;
+              }
+              else
+              {
+                if (*++head == '/')
+                  break;
+              }
+            }
+          }
+        }
+      default:
+        if (*head > ' ')
+          found = true;
+        break;
+    }
+    if (found)
+      break;
+    head++;
+  }
+
+  if ((*head >= 'A' && *head <= 'Z') || (*head >= 'a' && *head <= 'z'))
+  {
+    // Found start of a token. Scan for the end.
+    const char *tail = head + 1;
+    while ((*tail >= 'A' && *tail <= 'Z') || (*tail >= 'a' && *tail <= 'z'))
+      tail++;
+    token = base::toupper(std::string(head, tail - head));
+  }
+
+  KnownStatementTypes::iterator statement_type = known_statement_types.find(token);
   return (known_statement_types.end() == statement_type) ? sql_unknown : statement_type->second;
 }
 
@@ -93,14 +151,14 @@ bool Mysql_sql_syntax_check::parse_edit_statement(const std::string &sql,
   _messages_enabled= false;
   _use_delimiter= false;
   Check_sql_statement do_check_slot=
-    sigc::bind(sigc::mem_fun(this, &Mysql_sql_syntax_check::do_parse_edit_statement),
-      sigc::ref(schema_name), sigc::ref(table_name), sigc::ref(statement_tail));
-  int err_count= check_sql_statement(sql, do_check_slot, ot_none);
+    boost::bind(&Mysql_sql_syntax_check::do_parse_edit_statement, this, _1,
+                boost::ref(schema_name), boost::ref(table_name), boost::ref(statement_tail));
+  int err_count= check_sql_statement(sql.c_str(), do_check_slot, ot_none);
   return (0 == err_count);
 }
 
 
-int Mysql_sql_syntax_check::check_sql(const std::string &sql)
+int Mysql_sql_syntax_check::check_sql(const char* sql)
 {
   NULL_STATE_KEEPER
   _messages_enabled= false;
@@ -110,16 +168,16 @@ int Mysql_sql_syntax_check::check_sql(const std::string &sql)
   switch (_object_type)
   {
   case ot_trigger:
-    do_check_slot= sigc::mem_fun(this, &Mysql_sql_syntax_check::do_check_trigger);
+    do_check_slot= boost::bind(&Mysql_sql_syntax_check::do_check_trigger, this, _1);
     break;
   case ot_view:
-    do_check_slot= sigc::mem_fun(this, &Mysql_sql_syntax_check::do_check_view);
+    do_check_slot= boost::bind(&Mysql_sql_syntax_check::do_check_view, this, _1);
     break;
   case ot_routine:
-    do_check_slot= sigc::mem_fun(this, &Mysql_sql_syntax_check::do_check_routine);
+    do_check_slot= boost::bind(&Mysql_sql_syntax_check::do_check_routine, this, _1);
     break;
   default:
-    do_check_slot= sigc::mem_fun(this, &Mysql_sql_syntax_check::do_check_sql);
+    do_check_slot= boost::bind(&Mysql_sql_syntax_check::do_check_sql, this, _1);
     break;
   }
 
@@ -128,43 +186,43 @@ int Mysql_sql_syntax_check::check_sql(const std::string &sql)
 }
 
 
-int Mysql_sql_syntax_check::check_trigger(const std::string &sql)
+int Mysql_sql_syntax_check::check_trigger(const char* sql)
 {
   NULL_STATE_KEEPER
   _messages_enabled= false;
   _use_delimiter= true;
-  int err_count= check_sql_statement(sql, sigc::mem_fun(this,
-    &Mysql_sql_syntax_check::do_check_trigger), ot_trigger);
+  int err_count= check_sql_statement(sql, boost::bind(
+    &Mysql_sql_syntax_check::do_check_trigger, this, _1), ot_trigger);
   return (err_count ? 0 : 1);
 }
 
 
-int Mysql_sql_syntax_check::check_view(const std::string &sql)
+int Mysql_sql_syntax_check::check_view(const char* sql)
 {
   NULL_STATE_KEEPER
   _messages_enabled= false;
   _use_delimiter= true;
-  int err_count= check_sql_statement(sql, sigc::mem_fun(this,
-    &Mysql_sql_syntax_check::do_check_view), ot_view);
+  int err_count= check_sql_statement(sql, boost::bind(
+    &Mysql_sql_syntax_check::do_check_view, this, _1), ot_view);
   return (err_count ? 0 : 1);
 }
 
 
-int Mysql_sql_syntax_check::check_routine(const std::string &sql)
+int Mysql_sql_syntax_check::check_routine(const char* sql)
 {
   NULL_STATE_KEEPER
   _messages_enabled= false;
   _use_delimiter= true;
-  int err_count= check_sql_statement(sql, sigc::mem_fun(this,
-    &Mysql_sql_syntax_check::do_check_routine), ot_routine);
+  int err_count= check_sql_statement(sql, boost::bind(
+    &Mysql_sql_syntax_check::do_check_routine, this, _1), ot_routine);
   return (err_count ? 0 : 1);
 }
 
 
-int Mysql_sql_syntax_check::check_sql_statement(const std::string &sql, Check_sql_statement check_sql_statement, ObjectType object_type)
+int Mysql_sql_syntax_check::check_sql_statement(const char* sql, Check_sql_statement check_sql_statement, ObjectType object_type)
 {
   _check_sql_statement= check_sql_statement;
-  _process_sql_statement= sigc::bind(sigc::mem_fun(this, &Mysql_sql_syntax_check::process_sql_statement), object_type);
+  _process_sql_statement= boost::bind(&Mysql_sql_syntax_check::process_sql_statement, this, _1, object_type);
 
   Mysql_sql_parser_fe sql_parser_fe(_grtm->get_grt());
   sql_parser_fe.is_ast_generation_enabled= _is_ast_generation_enabled;
@@ -175,23 +233,27 @@ int Mysql_sql_syntax_check::check_sql_statement(const std::string &sql, Check_sq
     sql_parser_fe.max_err_count= options.get_int("SqlEditor::SyntaxCheck::MaxErrCount", 100);    
   }
 
-  const std::string *sql_ptr= &sql;
+  const char* actual_sql = sql;
   
-  // set delimiter for sql script if needed
-  std::string sql_;
+  // Set delimiter for sql script if needed.
+  std::string compound_sql;
   if (_use_delimiter)
   {
-    sql_= "DELIMITER " + _non_std_sql_delimiter + EOL + sql + EOL + _non_std_sql_delimiter;
-    sql_ptr= &sql_;
+    // We are going to make a duplicate of the passed in SQL, which might lead to massive
+    // memory problems if the script is large. The use of _use_delimiter should be prohibited.
+    // Pass in the correct text instead.
+    compound_sql= "DELIMITER " + _non_std_sql_delimiter + EOL + sql + EOL + _non_std_sql_delimiter;
+    actual_sql = compound_sql.c_str();
   }
 
-  return parse_sql_script(sql_parser_fe, *sql_ptr);
+  return parse_sql_script(sql_parser_fe, actual_sql);
 }
 
 
 int Mysql_sql_syntax_check::process_sql_statement(const SqlAstNode *tree, ObjectType object_type)
 {
-  do_report_sql_statement_border(_stmt_begin_lineno, _stmt_begin_line_pos, _stmt_end_lineno, _stmt_end_line_pos);
+  if (report_sql_statement_border)
+    do_report_sql_statement_border(_stmt_begin_lineno, _stmt_begin_line_pos, _stmt_end_lineno, _stmt_end_line_pos);
 
   if (!_is_ast_generation_enabled && !_err_tok_len)
     return 0;
