@@ -116,6 +116,8 @@ class DiffSQLGeneratorBE
   bool _use_filtered_lists;
   bool _skip_foreign_keys;
   bool _skip_fk_indexes;
+  bool _case_sensitive;
+  bool _use_oid_as_dict_key;
   std::set<std::string> _filtered_schemata, _filtered_tables, _filtered_views, _filtered_routines, _filtered_triggers, _filtered_users;
 
   // to be removed
