@@ -1,9 +1,27 @@
+/* 
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
 
-#include "stdafx.h"
+#include "tut_stdafx.h"
 
 #include "testgrt.h"
 #include "grt_test_utility.h"
-#include "syntetic_mysql_model.h"
+#include "synthetic_mysql_model.h"
 #include "../src/db_mysql_diffsqlgen_grant.h"
 #include "wb_helpers.h"
 
