@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,15 +21,20 @@
 
 #include <glib/gstdio.h>
 
-#include "base/util_functions.h"
 #include "wb_model.h"
 #include "reporting.h"
 
-#include "grti/sqlgenerator.h"
+#include "interfaces/sqlgenerator.h"
 #include "grts/structs.workbench.h"
 
 #include "reporting_template_variables.h"
+
+#include "base/util_functions.h"
 #include "base/string_utilities.h"
+#include "base/geometry.h"
+#include "base/log.h"
+
+#include "mforms/code_editor.h"
 
 // Support for syntax highlighting in SQL output.
 #ifdef _WIN32
@@ -47,14 +52,14 @@
 #endif
 #include "SciLexer.h"
 
-#include "mysql_sql_editor_be.h"
-
 using namespace std;
 using namespace base;
 
 using ctemplate::Template;
 using ctemplate::TemplateDictionary;
 
+DEFAULT_LOG_DOMAIN("Reporting")
+
 //----------------- LexerDocument ------------------------------------------------------------------
 
 LexerDocument::LexerDocument(const std::string& text)
@@ -76,14 +81,18 @@ LexerDocument::~LexerDocument()
 // IDocument implementation.
 int LexerDocument::Version() const
 {
-  throw std::logic_error("Internal error. Unexpected use of unimplemented function in LexerDocument.");
+  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
+    append(__FUNCTION__).append(" in LexerDocument (").
+    append(__FILE__).append(")."));
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void LexerDocument::SetErrorStatus(int status)
 {
-  throw std::logic_error("Internal error. Unexpected use of unimplemented function in LexerDocument.");
+  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
+    append(__FUNCTION__).append(" in LexerDocument (").
+    append(__FILE__).append(")."));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -111,14 +120,25 @@ char LexerDocument::StyleAt(int position) const
 
 int LexerDocument::LineFromPosition(int position) const
 {
-  throw std::logic_error("Internal error. Unexpected use of unimplemented function in LexerDocument.");
+  // Since we always style the full text starting from position 0 this should be the normal case.
+  if (position == 0)
+    return 0;
+
+  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
+    append(__FUNCTION__).append(" in LexerDocument (").
+    append(__FILE__).append(")."));
 }
 
 //--------------------------------------------------------------------------------------------------
 
 int LexerDocument::LineStart(int line) const
 {
-  throw std::logic_error("Internal error. Unexpected use of unimplemented function in LexerDocument.");
+  if (line == 0)
+    return 0;
+
+  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
+    append(__FUNCTION__).append(" in LexerDocument (").
+    append(__FILE__).append(")."));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -156,14 +176,18 @@ int LexerDocument::SetLevel(int line, int level)
 
 int LexerDocument::GetLineState(int line) const
 {
-  throw std::logic_error("Internal error. Unexpected use of unimplemented function in LexerDocument.");
+  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
+    append(__FUNCTION__).append(" in LexerDocument (").
+    append(__FILE__).append(")."));
 }
 
 //--------------------------------------------------------------------------------------------------
 
 int LexerDocument::SetLineState(int line, int state)
 {
-  throw std::logic_error("Internal error. Unexpected use of unimplemented function in LexerDocument.");
+  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
+    append(__FUNCTION__).append(" in LexerDocument (").
+    append(__FILE__).append(")."));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -210,21 +234,27 @@ bool LexerDocument::SetStyles(int length, const char *styles)
 
 void LexerDocument::DecorationSetCurrentIndicator(int indicator)
 {
-  throw std::logic_error("Internal error. Unexpected use of unimplemented function in LexerDocument.");
+  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
+    append(__FUNCTION__).append(" in LexerDocument (").
+    append(__FILE__).append(")."));
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void LexerDocument::DecorationFillRange(int position, int value, int fillLength)
 {
-  throw std::logic_error("Internal error. Unexpected use of unimplemented function in LexerDocument.");
+  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
+    append(__FUNCTION__).append(" in LexerDocument (").
+    append(__FILE__).append(")."));
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void LexerDocument::ChangeLexerState(int start, int end)
 {
-  throw std::logic_error("Internal error. Unexpected use of unimplemented function in LexerDocument.");
+  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
+    append(__FUNCTION__).append(" in LexerDocument (").
+    append(__FILE__).append(")."));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -241,6 +271,24 @@ bool LexerDocument::IsDBCSLeadByte(char ch) const
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+const char* LexerDocument::BufferPointer()
+{
+  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
+    append(__FUNCTION__).append(" in LexerDocument (").
+    append(__FILE__).append(")."));
+}
+
+//--------------------------------------------------------------------------------------------------
+
+int LexerDocument::GetLineIndentation(int line)
+{
+  throw std::logic_error(std::string("Internal error. Unexpected use of unimplemented function ").
+    append(__FUNCTION__).append(" in LexerDocument (").
+    append(__FILE__).append(")."));
+}
+
 //----------------- WbModelImpl --------------------------------------------------------------------
 
 /**
@@ -631,18 +679,10 @@ const Scintilla::LexerModule* setup_syntax_highlighter(db_mgmt_RdbmsRef rdbms)
 {
   const Scintilla::LexerModule* result= SCI_WRAPPER_NS Catalogue::Find("mysql");
 
-  // Due to the way Scintilla lexer initialization works we might have caught Scintilla uninitialized
-  // here (indicated by a NULL result for the lexer module. In that case initialize the lexer catalog and
-  // try again to find a lexer module.
-  if (result == NULL)
-  {
-    Scintilla_LinkLexers();
-    result= SCI_WRAPPER_NS Catalogue::Find("mysql");
-  }
-
   if (result != NULL)
   {
-    Mysql_sql_editor editor(rdbms);
+    mforms::CodeEditorConfig config(mforms::LanguageMySQL);
+    std::map<std::string, const char*> keywords = config.get_keywords();
     
     // Create the keyword lists used for lexing.
 	  for (int i = 0; i <= KEYWORDSET_MAX; i++)
@@ -651,11 +691,12 @@ const Scintilla::LexerModule* setup_syntax_highlighter(db_mgmt_RdbmsRef rdbms)
     
     // There are no predefined constants for the indices below, but the occupancy of the list array
     // can be seen in LexMySQL.cxx.
-    ((SCI_WRAPPER_NS WordList*) keywordLists[0])->Set(editor.get_keywords(Sql_editor::General_Keywords));
-    ((SCI_WRAPPER_NS WordList*) keywordLists[3])->Set(editor.get_function_names());
-    ((SCI_WRAPPER_NS WordList*) keywordLists[5])->Set(editor.get_keywords(Sql_editor::Procedure_Keywords));
-    ((SCI_WRAPPER_NS WordList*) keywordLists[6])->Set(editor.get_keywords(Sql_editor::Client_Keywords));
-    ((SCI_WRAPPER_NS WordList*) keywordLists[7])->Set(editor.get_keywords(Sql_editor::User_Keywords));
+    ((SCI_WRAPPER_NS WordList*) keywordLists[0])->Set(keywords["Major Keywords"]);
+    ((SCI_WRAPPER_NS WordList*) keywordLists[3])->Set(keywords["Functions"]);
+    ((SCI_WRAPPER_NS WordList*) keywordLists[5])->Set(keywords["Procedure keywords"]);
+    ((SCI_WRAPPER_NS WordList*) keywordLists[6])->Set(keywords["User Keywords 1"]);
+
+    // TODO: adjust CSS files so the color values correspond to what the user has set currently.
   }
 
   return result;
@@ -754,7 +795,7 @@ const string markupFromStyle(int style)
 
 //--------------------------------------------------------------------------------------------------
 
-void set_ddl(TemplateDictionary *target, SQLGeneratorInterfaceWrapper* sqlgenModule, 
+void set_ddl(TemplateDictionary *target, SQLGeneratorInterfaceImpl* sqlgenModule, 
              const GrtNamedObjectRef& object, const Scintilla::LexerModule* lexer,
              bool ddl_enabled)
 {
@@ -982,11 +1023,12 @@ int WbModelImpl::generateReport(workbench_physical_ModelRef model, const grt::Di
   int total_sp_count= 0;
   int total_trigger_count= 0;
 
-  SQLGeneratorInterfaceWrapper *sqlgenModule= NULL;
+  SQLGeneratorInterfaceImpl *sqlgenModule= NULL;
 
   if (show_ddl)
   {
-    vector<SQLGeneratorInterfaceWrapper*> genmodules= grt->get_implementing_modules<SQLGeneratorInterfaceWrapper>();
+    /*
+    vector<SQLGeneratorInterfaceImpl*> genmodules= grt->get_implementing_modules<SQLGeneratorInterfaceWrapper>();
     for (vector<SQLGeneratorInterfaceWrapper*>::const_iterator iter= genmodules.begin();
          iter != genmodules.end(); ++iter)
     {
@@ -995,7 +1037,8 @@ int WbModelImpl::generateReport(workbench_physical_ModelRef model, const grt::Di
         sqlgenModule= *iter;
         break;
       }
-    }
+    }*/
+    sqlgenModule = dynamic_cast<SQLGeneratorInterfaceImpl*>(grt->get_module("DbMySQL"));
     if (!sqlgenModule)
       throw logic_error("could not find SQL generation module for mysql");
   }
