@@ -1,8 +1,24 @@
-#  Copyright (c) 2009 Sun Microsystems Inc. All rights reserved.
+# Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
 #
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as
+# published by the Free Software Foundation; version 2 of the
+# License.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+# 02110-1301  USA
 
+import sys
 import subprocess
 import os
+import threading
 
 # import the wb module
 from wb import *
@@ -116,13 +132,35 @@ def startCommandLineClientForConnection(conn):
         subprocess.call(["/bin/bash", "-c", "%s -e \"%s\" &" % (get_linux_terminal_program(), command)])
 
 
+if sys.platform == "linux2":
+    odbcadmin = os.getenv("MWB_MODULE_DIR")+"/../iodbcadm-gtk"
+    @ModuleInfo.export(grt.INT)
+    def startODBCAdmin():
+        if os.path.exists(odbcadmin):
+            subprocess.call(odbcadmin + " &", shell=True)
+            return 1
+        else:
+            return 0
+elif sys.platform == "darwin":
+    @ModuleInfo.export(grt.INT)
+    def startODBCAdmin():
+        ret = subprocess.call("open -a 'ODBC Administrator'", shell=True)
+        if ret == 1:
+            ret = subprocess.call("open -a 'ODBC Manager'", shell=True)
+        return 0 if ret == 1 else 1
+elif sys.platform == "win32":
+    @ModuleInfo.export(grt.INT)
+    def startODBCAdmin():
+        subprocess.call("odbcad32.exe", shell=True)
+        return 1
+
+
 # Utilities only work in Py 2.6
 disable_utilities = False
-import sys
 # find out version of default python interpreter
 if sys.platform != "win32":
     pyversion, junk = subprocess.Popen("python --version", stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True).communicate()
-    if [int(i) for i in pyversion.split(" ")[1].strip().split(".")[:2]] < [2, 6]:
+    if pyversion < '2.6':
         disable_utilities = True
 
 if not disable_utilities:
@@ -169,8 +207,72 @@ if not disable_utilities:
                     subprocess.call(["/bin/bash", "-c", "%s -e %s &" % (term, setup_script)])
 
 
-#@ModuleInfo.plugin("wb.tools.utilitiesShellFor", caption="Start MySQL Utilities Shell for Connection", input= [wbinputs.selectedConnection()], pluginMenu="Home/Connections")
-#@ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection)
-#def startUtilitiesShellFor(conn):
-#    return 0
+class CheckForUpdateThread(threading.Thread):
+    def __init__(self):
+        self.is_running = False
+        self.finished = False
+        super(CheckForUpdateThread, self).__init__()
+    
+    def run(self):
+        if self.is_running:
+            return
+        
+        self.is_running = True
+        try:
+            import xml.dom.minidom
+            import urllib2
+            
+            self.dom = xml.dom.minidom.parse(urllib2.urlopen('http://wb.mysql.com/installer/products.xml'))
+        except Exception, error:
+            self.dom = None
+            self.error = str(error)        
+    
+    def checkForUpdatesCallback(self):
+        if self.isAlive():
+            return True  # Don't do anything until the dom is built
+        
+        if not self.dom:
+            if hasattr(self, 'error'):
+                mforms.Utilities.show_error("Check for updates failed", str(self.error), "OK", "", "")
+        else:
+            try:
+                current_version = (grt.root.wb.info.version.majorNumber, grt.root.wb.info.version.minorNumber, grt.root.wb.info.version.releaseNumber)
+                edition = '' if grt.root.wb.info.license == 'GPL' else '-commercial'
+                
+                packages = ( package for package in self.dom.getElementsByTagName('Package') if package.parentNode.parentNode.attributes['name'].nodeValue == u'workbench-win32' + edition)
+                version_strings = ( node.attributes['thisVersion'].nodeValue for node in packages if node.attributes['type'].nodeValue == 'MSI' )
+                versions = ( tuple( int(num) for num in version_string.split('.') ) for version_string in version_strings )
+                newest_version = max( versions ) if versions else current_version
+                
+                if newest_version > current_version:
+                    if mforms.Utilities.show_message('New Version Available', 'The new MySQL Workbench %s has been released.\nYou can download the latest version from\nhttp://www.mysql.com/downloads/workbench.' % '.'.join( [str(num) for num in newest_version] ),
+                                                  'OK', 'Cancel', 'Get it Now') == mforms.ResultOther:
+                        mforms.Utilities.open_url('http://www.mysql.com/downloads/workbench')
+                else:
+                    mforms.Utilities.show_message('MySQL Workbench is Up to Date', 'You are already using the latest version of MySQL Workbench.', 'OK', '', '')
+        
+            except Exception, error:
+                mforms.Utilities.show_error("Check for updates failed", str(error), "OK", "", "")
 
+        mforms.App.get().set_status_text('Ready.')
+        self.is_running = False
+        self.finished = True
+        return False
+
+
+# Global variable:
+thread = CheckForUpdateThread()
+
+@ModuleInfo.plugin("wb.tools.checkForUpdates", caption="Check for Updates")
+@ModuleInfo.export(grt.INT)
+def checkForUpdates():
+    global thread
+    
+    if thread.is_running:
+        return 0
+    
+    if thread.finished:
+        thread = CheckForUpdateThread()
+    thread.start()
+    mforms.App.get().set_status_text('Checking for updates...')
+    mforms.Utilities.add_timeout(1.0, thread.checkForUpdatesCallback)
