@@ -16,10 +16,9 @@ using namespace grt;
 #include "grtdb/diff_dbobjectmatch.h"
 
 #include "grtsqlparser/sql_facade.h"
-#include "grti/sqlgenerator.h"
+#include "interfaces/sqlgenerator.h"
 #include "db.mysql/backend/db_mysql_public_interface.h"
 
-#include "db.mysql/src/module_db_mysql_shared_code.h"
 #include "grtdb/charset_utils.h"
 #include "grtdb/db_object_helpers.h"
 
@@ -132,34 +131,24 @@ std::string DbMySQLDiffReporting::generate_report(const std::string& left_file,
   bec::apply_user_datatypes(left_cat_copy, rdbms);
 
 
-  grt::DbObjectMatchAlterOmf omf;
-  omf.dontdiff_mask = 3;
-  DiffChange *alter_change= diff_make(left_cat_copy, right_cat_copy, &omf, grt::NormalizedComparer(manager_->get_grt()));
+  SQLGeneratorInterfaceImpl *diffsql_module= dynamic_cast<SQLGeneratorInterfaceImpl*>(manager_->get_grt()->get_module("DbMySQL"));
 
-  SQLGeneratorInterfaceWrapper *diffsql_module= 
-    manager_->get_grt()->get_module_wrapper<SQLGeneratorInterfaceWrapper>("DbMySQL");
-
-  if(diffsql_module == NULL)
-    throw DbMySQLDiffReportingException("error loading module MySQLModuleDbMySQL");
+  if (diffsql_module == NULL)
+    throw DbMySQLDiffReportingException("error loading module DbMySQL");
 
   std::string tpath;
   tpath.append("modules").append(G_DIR_SEPARATOR_S).append("data").append(G_DIR_SEPARATOR_S).append("db_mysql_catalog_reporting")
     .append(G_DIR_SEPARATOR_S).append("Basic_Text.tpl").append(G_DIR_SEPARATOR_S).append("basic_text_report.txt.tpl");
 
   grt::DictRef options(manager_->get_grt());
+  options.set("OMFDontDiffMask", grt::IntegerRef(3));
   options.set("UseFilteredLists", grt::IntegerRef(0));
   options.set("KeepOrder", grt::IntegerRef(1));
   options.set("TemplateFile", 
     grt::StringRef(manager_->get_data_file_path(tpath).c_str()));
 
-  char buf1[128];
-  sprintf(buf1, "%p", alter_change);
-
-  //if(alter_change)
-  //  alter_change->dump_log(0); 
-
   grt::StringRef output_string(
-    diffsql_module->generateReport(left_cat_copy, options, std::string(buf1)));
+    diffsql_module->generateReportForDifferences(left_cat_copy, right_cat_copy, options));
 
-  return std::string(output_string.c_str());
+  return *output_string;
 }
