@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,8 @@
 
 #include "base/string_utilities.h"
 
+#include "mforms/code_editor.h"
+
 using namespace bec;
 using namespace base;
 
@@ -64,7 +66,8 @@ bool MySQLTableColumnsListBE::set_field(const NodeId &node, int column, const st
 
         return result;
       }
-
+      break;
+    default:
       break;
     }
   }
@@ -166,32 +169,31 @@ bool MySQLTableColumnsListBE::get_field_grt(const ::bec::NodeId &node, int colum
   {
     if (node[0] < real_count())
       col= static_cast<MySQLTableEditorBE*>(_owner)->table()->columns().get(node[0]);
-
-    switch ((MySQLColumnListColumns)column)
+    if (col.is_valid())
     {
-    case IsAutoIncrement:
-      if (col.is_valid())
+      switch (column)
+      {
+      case IsAutoIncrement:
         value= col->autoIncrement();
-      else
+        return true;
+      case IsAutoIncrementable:
+        value = grt::IntegerRef(0);
+        if (col->simpleType().is_valid() && col->simpleType()->group().is_valid())
+        {
+          if (col->simpleType()->group()->name() == "numeric")
+            value= grt::IntegerRef(1);
+        }
+        return true;
+      case HasCharset:
         value= grt::IntegerRef(0);
-      return true;
-    case IsAutoIncrementable:
-      value= grt::IntegerRef(0);
-      if (col.is_valid() && col->simpleType().is_valid() && col->simpleType()->group().is_valid())
-      {
-        if (col->simpleType()->group()->name() == "numeric")
-          value= grt::IntegerRef(1);
-      }
-      return true;
-    case HasCharset:
-      value= grt::IntegerRef(0);
-      if (col.is_valid() && col->simpleType().is_valid())
-      {
-        if (col->simpleType()->group()->name() == "string" || col->simpleType()->group()->name() == "text"
+        if (col->simpleType().is_valid())
+        {
+          if (col->simpleType()->group()->name() == "string" || col->simpleType()->group()->name() == "text"
             || col->simpleType()->name() == "ENUM")
-          value= grt::IntegerRef(1);
+            value= grt::IntegerRef(1);
+        }
+        return true;          
       }
-      return true;          
     }
   }
   return TableColumnsListBE::get_field_grt(node, column, value);
@@ -334,12 +336,22 @@ bool MySQLTableColumnsListBE::activate_popup_item_for_nodes(const std::string &n
   return TableColumnsListBE::activate_popup_item_for_nodes(name, orig_nodes);
 }  
 
-//--------------------------------------------------------------------------------
+//--------------------------------------------------------------------------------------------------
 
-
-MySQLTableEditorBE::MySQLTableEditorBE(::bec::GRTManager *grtm, const db_mysql_TableRef &table, const db_mgmt_RdbmsRef &rdbms)
+MySQLTableEditorBE::MySQLTableEditorBE(::bec::GRTManager *grtm, db_mysql_TableRef table, const db_mgmt_RdbmsRef &rdbms)
   : TableEditorBE(grtm, table, rdbms), _table(table), _columns(this), _partitions(this), _indexes(this)
 {
+  if (_table->isStub() == 1)
+  {
+    if (mforms::Utilities::show_warning(
+      _("Stub table edit"),
+      _("The table you are trying to edit is a model-only stub, created during reverse "
+        "engineering, to represent missing tables referenced by foreign keys. Such tables are "
+        "ignored by forward engineering and synchronization.\n\n"
+        "You may convert this table to a real one that appears also in the generated SQL or keep it as stub."),
+        _("Convert to real table"), _("Edit as is")) == mforms::ResultOk)
+        db_mysql_TableRef(table)->isStub(0);
+  }
 }
 
 
@@ -396,6 +408,21 @@ std::vector<std::string> MySQLTableEditorBE::get_engines_list()
   return engines;
 }
 
+/**
+ * Determines if the currently set engine supports foreign keys and reports the outcome to the caller.
+ */
+bool MySQLTableEditorBE::engine_supports_foreign_keys()
+{
+  grt::StringRef name = _table->tableEngine();
+  if (name == "") // No engine set. Assume db default allows FKs.
+    return true;
+  
+  db_mysql_StorageEngineRef engine = bec::TableHelper::get_engine_by_name(get_grt(), name);
+  if (engine.is_valid())
+    return engine->supportsForeignKeys() == 1;
+  
+  return false; // Don't know anything about this engine, so assume it doesn't support FKs.
+}
 
 static struct TableOption
 {
@@ -575,8 +602,35 @@ std::string MySQLTableEditorBE::get_all_triggers_sql() const
   return retval;
 }
 
+//--------------------------------------------------------------------------------------------------
 
+/**
+ * Loads the current trigger sql text into the editor control and marks that as not dirty.
+ */
+void MySQLTableEditorBE::load_trigger_sql()
+{
+  mforms::CodeEditor* editor = get_sql_editor()->get_editor_control();
+  std::string sql = get_all_triggers_sql();
+  editor->set_text_keeping_state(sql.c_str());
+  editor->reset_dirty();
+}
 
+//--------------------------------------------------------------------------------------------------
+
+void MySQLTableEditorBE::commit_changes()
+{
+  mforms::CodeEditor* editor = get_sql_editor()->get_editor_control();
+  if (editor->is_dirty())
+  {
+    const std::string sql = editor->get_text(false);
+    set_triggers_sql(sql, true);
+    editor->reset_dirty();
+  }
+
+  // TODO: anything else to store?
+}
+
+//--------------------------------------------------------------------------------------------------
 
 bool MySQLTableEditorBE::set_partition_type(const std::string &type)
 {
@@ -877,7 +931,6 @@ bool MySQLTableEditorBE::check_column_referenceable_by_fk(const db_ColumnRef &co
 
 //--------------------------------------------------------------------------------
 
-
 MySQLTablePartitionTreeBE::MySQLTablePartitionTreeBE(MySQLTableEditorBE *owner)
 : _owner(owner)
 {
