@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -107,7 +107,7 @@ protected:
 class MYSQLWBMYSQLSUPPORTBACKEND_PUBLIC_FUNC MySQLTableEditorBE : public ::bec::TableEditorBE
 {
 public:
-  MySQLTableEditorBE(::bec::GRTManager *grtm, const db_mysql_TableRef &table, const db_mgmt_RdbmsRef &rdbms);
+  MySQLTableEditorBE(::bec::GRTManager *grtm, db_mysql_TableRef table, const db_mgmt_RdbmsRef &rdbms);
 
   virtual MySQLTableColumnsListBE *get_columns() { return &_columns; }
   virtual MySQLTableIndexListBE *get_indexes() { return &_indexes; }
@@ -124,11 +124,15 @@ public:
   virtual void set_table_option_by_name(const std::string& name, const std::string& value);
   virtual std::string get_table_option_by_name(const std::string& name);
   std::vector<std::string> get_engines_list();
+  bool engine_supports_foreign_keys();
   
   virtual bool check_column_referenceable_by_fk(const db_ColumnRef &column1, const db_ColumnRef &column2);
 
   // interactive functions
-  std::string get_all_triggers_sql() const;
+  std::string get_all_triggers_sql() const; // TODO: only used internally and in tests, should be protected.
+  void load_trigger_sql();
+
+  virtual void commit_changes();
 
   // partitioning
   bool set_partition_type(const std::string &type);
