@@ -39,6 +39,7 @@ class DbMySQLRoleEditor : public PluginEditorBase
   void set_name(const std::string& nm)
   {
     _be->set_name(nm);
+    _signal_title_changed.emit(_be->get_title());
   }
   
  public:
@@ -49,7 +50,6 @@ class DbMySQLRoleEditor : public PluginEditorBase
   }
 
   virtual void do_refresh_form_data();
-  virtual std::string get_title();
 
   void on_object_drop(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y,
                        const Gtk::SelectionData& selection_data, guint info, guint time);
@@ -156,12 +156,6 @@ bool DbMySQLRoleEditor::switch_edited_object(bec::GRTManager *grtm, const grt::B
   return true;
 }
 
-//------------------------------------------------------------------------------
-std::string DbMySQLRoleEditor::get_title()
-{
-  return _be->get_name();
-}
-
 //------------------------------------------------------------------------------
 bec::BaseEditor *DbMySQLRoleEditor::get_be()
 {
@@ -217,6 +211,8 @@ void DbMySQLRoleEditor::do_refresh_form_data()
   xml()->get_widget("name_entry", entry);
   entry->set_text(_be->get_name());
 
+  _signal_title_changed.emit(_be->get_title());
+
   std::vector<std::string> c= _be->get_role_list();  
   _parent_combo->set_model(model_from_string_list(c, &_parent_combo_model));
   _parent_combo->set_row_span_column(0);
