@@ -3,7 +3,7 @@
 #include "../backend/mysql_routinegroup_editor.h"
 #include "grtdb/db_object_helpers.h"
 #include "treemodel_wrapper.h"
-#include "sql_editor_fe.h"
+#include "gtk/mforms_gtk.h"
 #include "errors_list.h"
 #include <sstream>
 #include "text_list_columns_model.h"
@@ -14,16 +14,13 @@
 class DbMySQLRoutineGroupEditor : public PluginEditorBase
 {
   MySQLRoutineGroupEditorBE                *_be;
-  SqlEditorFE                               _code;
   Glib::RefPtr<Gtk::ListStore>              _routines_model;
   Gtk::TreeView                            *_rg_list;
   TextListColumnsModel                     *_routines_columns;
+  Gtk::Menu				    _context_menu;
   
   virtual bec::BaseEditor *get_be();
   
-  void parse_sql(const std::string& sql);
-  void error_selected(const int lineno, const std::string& msg);
-
   bool process_event(GdkEvent* event);
   void menu_action_on_node(const std::string &item_name, const Gtk::TreePath path);
 
@@ -39,7 +36,6 @@ class DbMySQLRoutineGroupEditor : public PluginEditorBase
   
   virtual ~DbMySQLRoutineGroupEditor();
   virtual void do_refresh_form_data();
-  virtual std::string get_title();
 
   void on_routine_drop(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y,
                        const Gtk::SelectionData& selection_data, guint info, guint time);
@@ -71,14 +67,10 @@ DbMySQLRoutineGroupEditor::DbMySQLRoutineGroupEditor(grt::Module *m, bec::GRTMan
   xml()->get_widget("rg_comment", tv);
   add_text_change_timer(tv, sigc::mem_fun(this, &DbMySQLRoutineGroupEditor::set_comment));
   
-  _code.be(_be->get_sql_editor());
-
   Gtk::VBox* code_win;
   xml()->get_widget("rg_code_holder", code_win);
-  _code.widget().set_size_request(-1, 100);
-  add_sqleditor_text_change_timer(&_code, sigc::mem_fun(this, &DbMySQLRoutineGroupEditor::parse_sql));
-  code_win->add(_code.widget());
-  code_win->resize_children();
+  embed_code_editor(_be->get_sql_editor()->get_editor_control(), code_win);
+  _be->load_routines_sql();
   
   refresh_form_data();
   
@@ -95,8 +87,6 @@ DbMySQLRoutineGroupEditor::DbMySQLRoutineGroupEditor(grt::Module *m, bec::GRTMan
   _rg_list->signal_drag_data_received().connect(sigc::mem_fun(this, &DbMySQLRoutineGroupEditor::on_routine_drop));
   _rg_list->signal_event().connect(sigc::mem_fun(*this, &DbMySQLRoutineGroupEditor::process_event));
 
-  _code.set_text(_be->get_routines_sql());
-
   show_all();
 }
 
@@ -109,31 +99,26 @@ DbMySQLRoutineGroupEditor::~DbMySQLRoutineGroupEditor()
 //------------------------------------------------------------------------------
 bool DbMySQLRoutineGroupEditor::switch_edited_object(bec::GRTManager *grtm, const grt::BaseListRef &args)
 {
-  MySQLRoutineGroupEditorBE *old_be = _be;
-  
+  Gtk::VBox* code_win;
+  xml()->get_widget("rg_code_holder", code_win);
+  delete _be;
+ 
   _be = new MySQLRoutineGroupEditorBE(grtm, db_mysql_RoutineGroupRef::cast_from(args[0]), get_rdbms_for_db_object(args[0]));
+  embed_code_editor(_be->get_sql_editor()->get_editor_control(), code_win);
+  _be->load_routines_sql();
   _be->set_refresh_ui_slot(sigc::mem_fun(this, &DbMySQLRoutineGroupEditor::refresh_form_data));
 
-  _code.be(_be->get_sql_editor());
-  
   refresh_form_data();
   
-  delete old_be;
-
   return true;
 }
 
 //------------------------------------------------------------------------------
-std::string DbMySQLRoutineGroupEditor::get_title()
-{
-  return strfmt(_("Group: %s"), _be->get_name().c_str());
-}
 
-//------------------------------------------------------------------------------
 void DbMySQLRoutineGroupEditor::set_group_name(const std::string &name)
 {
   _be->set_name(name);
-  _signal_title_changed.emit(get_title());
+  _signal_title_changed.emit(_be->get_title());
 }
 
 //------------------------------------------------------------------------------
@@ -150,35 +135,17 @@ void DbMySQLRoutineGroupEditor::do_refresh_form_data()
   if (entry->get_text() != _be->get_name())
   {
     entry->set_text(_be->get_name());
-    _signal_title_changed.emit(get_title());
+    _signal_title_changed.emit(_be->get_title());
   }
   Gtk::TextView* tv;
   xml()->get_widget("rg_comment", tv);
   tv->get_buffer()->set_text(_be->get_comment());
 
-  _code.check_sql();
+  _be->load_routines_sql();
 
-  if (_be->get_sql_editor()->is_refresh_enabled())
-  {
-    _be->get_sql_editor()->is_refresh_enabled(false);
-    _code.set_text(_be->get_routines_sql());
-  }
-  
   recreate_model_from_string_list(_routines_model, _be->get_routines_names());
 }
 
-//------------------------------------------------------------------------------
-void DbMySQLRoutineGroupEditor::parse_sql(const std::string& sql)
-{
-  _code.reset_sql_check_state();
-  _be->set_routines_sql(sql, false);
-  recreate_model_from_string_list(_routines_model, _be->get_routines_names());
-}
-
-//------------------------------------------------------------------------------
-void DbMySQLRoutineGroupEditor::error_selected(const int lineno, const std::string& msg)
-{}
-
 //------------------------------------------------------------------------------
 bool DbMySQLRoutineGroupEditor::process_event(GdkEvent* event)
 {
@@ -200,7 +167,7 @@ bool DbMySQLRoutineGroupEditor::process_event(GdkEvent* event)
       run_popup_menu(menuitems
                 ,event->button.time
                 ,sigc::bind(sigc::mem_fun(this, &DbMySQLRoutineGroupEditor::menu_action_on_node), path)
-                );
+                ,&_context_menu);
     }
   }
   return false;
@@ -214,7 +181,6 @@ void DbMySQLRoutineGroupEditor::menu_action_on_node(const std::string &item_name
     const std::string name = (*(_routines_model->get_iter(path)))[_routines_columns->item];
     _be->delete_routine_with_name(name);
     do_refresh_form_data();
-    _code.set_text(_be->get_routines_sql());
   }
 }
 
@@ -249,7 +215,6 @@ void DbMySQLRoutineGroupEditor::on_routine_drop(const Glib::RefPtr<Gdk::DragCont
     }
 
     recreate_model_from_string_list(_routines_model, _be->get_routines_names());
-    _code.set_text(_be->get_routines_sql());
 
     dnd_status = true;
   }
