@@ -16,21 +16,25 @@
 
 //------------------------------------------------------------------------------
 DbMySQLTableEditor::DbMySQLTableEditor(grt::Module *m, bec::GRTManager *grtm, const grt::BaseListRef &args)
-    : PluginEditorBase(m, grtm, args, "modules/data/editor_mysql_table.glade")
+//    : PluginEditorBase(m, grtm, args, "modules/data/editor_mysql_table.glade")
+    : PluginEditorBase(m, grtm, args, 0)
     , _be(new MySQLTableEditorBE(grtm, db_mysql_TableRef::cast_from(args[0]), get_rdbms_for_db_object(args[0])))
     , _part_page(0)
+    , _inserts_page(0)
 {
-  xml()->get_widget("mysql_editor_notebook", _editor_notebook);
+  load_glade((_be->is_editing_live_object()) ? "modules/data/editor_mysql_table_live.glade" : "modules/data/editor_mysql_table.glade");
 
+  xml()->get_widget("mysql_editor_notebook", _editor_notebook);
   _editor_notebook->signal_switch_page().connect(sigc::mem_fun(this, &DbMySQLTableEditor::page_changed));
 
-  Gtk::Image *image;
+  Gtk::Image *image(0);
   xml()->get_widget("table_editor_image", image);
   image->set(ImageCache::get_instance()->image_from_filename("db.Table.editor.48x48.png", false));
+  image->set_data("is_large", (void*)1);
+
 
   set_border_width(0);
 
-
   _columns_page   = new DbMySQLTableEditorColumnPage(this, _be, xml());
   _indexes_page   = new DbMySQLTableEditorIndexPage(this, _be, xml());
   _fks_page       = new DbMySQLTableEditorFKPage(this, _be, xml());
@@ -54,6 +58,14 @@ DbMySQLTableEditor::DbMySQLTableEditor(grt::Module *m, bec::GRTManager *grtm, co
     _editor_notebook->remove_page(*placeholder);
     //_inserts_page= NULL;
     _privs_page= NULL;
+
+    Gtk::ComboBox *cbox = 0;
+    xml()->get_widget("schema_combo", cbox);
+    if (cbox)
+    {
+      setup_combo_for_string_list(cbox);
+      //fill_combo_from_string_list(cbox, _be->get_all_schema_names());
+    }
   }
   
   create_table_page();
@@ -73,6 +85,10 @@ DbMySQLTableEditor::DbMySQLTableEditor(grt::Module *m, bec::GRTManager *grtm, co
 
   _be->set_refresh_ui_slot(sigc::mem_fun(this, &DbMySQLTableEditor::refresh_form_data));
   _be->set_partial_refresh_ui_slot(sigc::mem_fun(this, &DbMySQLTableEditor::partial_refresh));
+
+  //Gtk::Paned* table_page_paned = 0;
+  //xml()->get_widget("table_page_paned", table_page_paned);
+  //gtk_paned_set_pos_ratio(table_page_paned, 0.2);
 }
 
 //------------------------------------------------------------------------------
@@ -89,6 +105,78 @@ DbMySQLTableEditor::~DbMySQLTableEditor()
   delete _be;
 }
 
+//------------------------------------------------------------------------------
+void DbMySQLTableEditor::decorate_object_editor()
+{
+  if (is_editing_live_object())
+  {
+    PluginEditorBase::decorate_object_editor();
+    Gtk::HBox* header_part = 0;
+    xml()->get_widget("header_part", header_part);
+
+    if (header_part->get_parent() == NULL)
+    {
+      decorator_control()->pack_start(*header_part, false, true);
+      decorator_control()->reorder_child(*header_part, 0);
+
+      Gtk::Button *hide_button = 0;
+      xml()->get_widget("hide_button", hide_button);
+      Gtk::Image* hide_image = Gtk::manage(new Gtk::Image(ImageCache::get_instance()->image_from_filename("EditorExpanded.png", false)));
+      Gtk::Image* show_image = Gtk::manage(new Gtk::Image(ImageCache::get_instance()->image_from_filename("EditorCollapsed.png", false)));
+      hide_image->show();
+      Gtk::VBox* box = Gtk::manage(new Gtk::VBox());
+      box->pack_start(*hide_image, false, false);
+      box->pack_start(*show_image, false, false);
+      box->show();
+      show_image->hide();
+      hide_button->set_image(*box);
+      hide_button->signal_clicked().connect(sigc::mem_fun(this, &DbMySQLTableEditor::toggle_header_part));
+      toggle_header_part();
+    }
+  }
+}
+
+//------------------------------------------------------------------------------
+void DbMySQLTableEditor::toggle_header_part()
+{
+  Gtk::Button *hide_button = 0;
+  xml()->get_widget("hide_button", hide_button);
+
+  Gtk::Image *image = 0;
+  xml()->get_widget("table_editor_image", image);
+  const bool make_image_small = image->get_data("is_large");
+  image->set(ImageCache::get_instance()->image_from_filename(make_image_small ? "db.Table.editor.24x24.png" : "db.Table.editor.48x48.png", false));
+  image->set_data("is_large", (void*)(!make_image_small));
+
+  Gtk::VBox* image_box = dynamic_cast<Gtk::VBox*>(hide_button->get_image());
+  if (image_box)
+  {
+    const std::vector<Gtk::Widget*> images = image_box->get_children();
+    for (int i = ((int)images.size()) - 1; i >= 0; --i)
+    {
+      if (images[i]->is_visible())
+        images[i]->hide();
+      else
+        images[i]->show();
+    }
+
+    const char* const names[] = {"collation_label", "collation_combo", "engine_label", "engine_combo", "comment_box"};
+    const int names_size = sizeof(names) / sizeof(const char**);
+    for (int i = 0; i < names_size; ++i)
+    {
+      Gtk::Widget* w = 0;
+      xml()->get_widget(names[i], w);
+      if (w)
+      {
+        if (w->is_visible())
+          w->hide();
+        else
+          w->show();
+      }
+    }
+  }
+}
+
 //------------------------------------------------------------------------------
 bool DbMySQLTableEditor::switch_edited_object(bec::GRTManager *grtm, const grt::BaseListRef &args)
 {
@@ -121,12 +209,6 @@ bool DbMySQLTableEditor::switch_edited_object(bec::GRTManager *grtm, const grt::
   return true;
 }
 
-//------------------------------------------------------------------------------
-std::string DbMySQLTableEditor::get_title()
-{
-  return strfmt(_("Table: %s"), _be->get_name().c_str());
-}
-
 //------------------------------------------------------------------------------
 bec::BaseEditor *DbMySQLTableEditor::get_be()
 {
@@ -138,7 +220,7 @@ bec::BaseEditor *DbMySQLTableEditor::get_be()
 void DbMySQLTableEditor::set_table_name(const std::string &name)
 {
   _be->set_name(name);
-  _signal_title_changed.emit(get_title());
+  _signal_title_changed.emit(_be->get_title());
 }
 
 //------------------------------------------------------------------------------
@@ -171,17 +253,18 @@ void DbMySQLTableEditor::create_table_page()
   fill_combo_from_string_list(combo, collations);
   add_option_combo_change_handler(combo, "CHARACTER SET - COLLATE", sigc::mem_fun(this, &DbMySQLTableEditor::set_table_option_by_name));
 
-  Gtk::TextView *tview;
+  Gtk::TextView *tview = 0;
   xml()->get_widget("table_comments", tview);
 
   add_text_change_timer(tview, sigc::mem_fun(this, &DbMySQLTableEditor::set_comment));
+
 }
 
 //------------------------------------------------------------------------------
 
 bool DbMySQLTableEditor::can_close()
 {
-  if (_inserts_page->has_changes())
+  if (_inserts_page && _inserts_page->has_changes())
   {
     Gtk::MessageDialog dlg("<b>There are unsaved changes in the INSERTS editor</b>\nPlease Apply or Revert these changes before closing.",
                            true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
@@ -225,7 +308,7 @@ void DbMySQLTableEditor::refresh_table_page()
   if (_be->get_name() != entry->get_text())
   {
     entry->set_text(_be->get_name());
-    _signal_title_changed.emit(get_title());
+    _signal_title_changed.emit(_be->get_title());
   }
 
   Gtk::TextView *tview;
@@ -254,6 +337,7 @@ void DbMySQLTableEditor::do_refresh_form_data()
   _fks_page->refresh();
   _triggers_page->refresh();
   _part_page->refresh();
+  _opts_page->refresh();
 
   if (!is_editing_live_object())
   {
@@ -266,6 +350,16 @@ void DbMySQLTableEditor::do_refresh_form_data()
 
     _privs_page->refresh();
   }
+  else
+  {
+    Gtk::ComboBox *cbox = 0;
+    xml()->get_widget("schema_combo", cbox);
+    if (cbox)
+    {
+      fill_combo_from_string_list(cbox, _be->get_all_schema_names());
+      cbox->set_active(0);
+    }
+  }
 }
 
 //TESTING
@@ -292,7 +386,7 @@ void DbMySQLTableEditor::page_changed(GtkNotebookPage* page, guint page_num)
     break;
 
   case 3: // fks
-    _fks_page->refresh();
+    _fks_page->refresh(); // Note! Currently the call to refresh is mandatory to show/hide FK page content depending on engine capabilities
     break;
 
   case 4: // triggers
