@@ -1,28 +1,28 @@
-
-/*!
- Copyright 2009 Sun Microsystems, Inc.
- 
- @author
- jak
- 
- @class
- MacTableEditorFKColumnsInformationSource
- 
- @abstract
- The purpose of this class is to insulate the C++ data structures needed by the MySQL Table Editor.
- 
- @ingroup
- MySQL Table Editor
+/* 
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  */
-
-
+#include "base/geometry.h"
+#include "base/string_utilities.h"
 
 #import "MacTableEditorFKColumnsInformationSource.h"
 
 #include "mysql_table_editor.h"
 
-
-
 @implementation MacTableEditorFKColumnsInformationSource
 
 
