@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,11 +17,13 @@
  * 02110-1301  USA
  */
 
+#include "base/geometry.h"
+#include "base/string_utilities.h"
+
 #import "RoleEditor.h"
 #import "MCPPUtilities.h"
 #import "GRTTreeDataSource.h"
 #include "grtdb/db_object_helpers.h" // get_rdbms_for_db_object()
-#include "base/string_utilities.h"
 
 @implementation RolePrivilegeObjectListDataSource
 
@@ -149,15 +151,13 @@ static void call_refresh(DbMysqlRoleEditor *self)
 
 - (id)initWithModule:(grt::Module*)module GRTManager:(bec::GRTManager*)grtm arguments:(const grt::BaseListRef&)args
 {
-  self= [super initWithModule:module GRTManager:grtm arguments:args];
+  self= [super initWithNibName: @"RoleEditor" bundle: [NSBundle bundleForClass:[self class]]];
   if (self != nil)
   {
+    _grtm = grtm;
     // load GUI. Top level view in the nib is the NSTabView that will be docked to the main window
-    if (![[NSBundle bundleForClass:[self class]] loadNibFile:@"RoleEditor"
-                                           externalNameTable:[NSDictionary dictionaryWithObject:self forKey:NSNibOwner] 
-                                                    withZone:nil])
-      NSLog(@"Could not load RoleEditor.xib");
-    
+    [self loadView];
+
     [self setMinimumSize: [tabView frame].size];
 
     [roleOutline registerForDraggedTypes:[NSArray arrayWithObject:@"x-mysql-wb/controlprivate"]];
@@ -192,7 +192,8 @@ static void call_refresh(DbMysqlRoleEditor *self)
     [privilegeListDS setRoleEditor: self];
     
     bec::NodeId roleNode= mBackEnd->get_role_tree()->node_id_for_role(mBackEnd->get_role());
-    [roleOutline selectRow: [roleOutline rowForItem:[roleTreeDS itemForNodeId: roleNode]] byExtendingSelection:NO];
+    [roleOutline selectRowIndexes: [NSIndexSet indexSetWithIndex: [roleOutline rowForItem:[roleTreeDS itemForNodeId: roleNode]]]
+             byExtendingSelection: NO];
 
     // update the UI
     [self refresh];
@@ -201,10 +202,6 @@ static void call_refresh(DbMysqlRoleEditor *self)
 
 - (void)dealloc
 {
-  [roleTreeDS release];
-  [objectListDS release];
-  [privilegeListDS release];
-
   delete mBackEnd;
   [super dealloc];
 }
@@ -247,20 +244,6 @@ static void call_refresh(DbMysqlRoleEditor *self)
 }
 
 
-- (NSString*)title
-{
-  // the title for the editor
-  return [NSString stringWithCPPString: base::strfmt("%s - Role", mBackEnd->get_name().c_str())];
-}
-
-
-- (NSView*)dockableView
-{
-  // the view to be docked to the main window
-  return [tabView superview];
-}
-
-
 - (void)controlTextDidEndEditing:(NSNotification *)aNotification
 {
   if ([aNotification object] == nameText)
@@ -279,4 +262,9 @@ static void call_refresh(DbMysqlRoleEditor *self)
 }
 
 
+- (bec::BaseEditor*)editorBE
+{
+  return mBackEnd;
+}
+
 @end
