@@ -1,14 +1,33 @@
+/* 
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 using System;
 using System.Collections.Generic;
 using System.Windows.Forms;
+
 using Aga.Controls.Tree;
 using Aga.Controls.Tree.NodeControls;
 
 using MySQL.Forms;
-using MySQL.Grt;
 using MySQL.Utilities;
 
-namespace MySQL.GUI.Workbench.Plugins
+namespace MySQL.Grt.Db
 {
 	public class DbMysqlTableColumnsListModel : GrtListModel
 	{
@@ -24,13 +43,16 @@ namespace MySQL.GUI.Workbench.Plugins
     private AdvNodeTextBox defaultNodeControl;
     private MySQLTableEditorBE mySQLTableEditorBE;
 
+    private IModelChangeListener listener;
+
 		/// <summary>
 		/// Constructor that initializes the model with the given objects
 		/// </summary>
 		/// <param name="TreeView">The TreeViewAdv control this model belongs to</param>
 		/// <param name="GrtTree">The GRT tree this model belongs to</param>
 		/// <param name="NodeStateIcon">The NodeStateIcon NodeControl that displays the icon</param>
-		public DbMysqlTableColumnsListModel(TreeViewAdv ListView, MySQLTableColumnsListBE GrtList,
+		public DbMysqlTableColumnsListModel(IModelChangeListener listener, TreeViewAdv ListView,
+      MySQLTableColumnsListBE GrtList,
 			NodeIcon columnIconNodeControl, AdvNodeTextBox NameNodeControl, AdvNodeComboBox DatatypeComboBoxNodeControl,
       NodeCheckBox pkNodeControl_, NodeCheckBox nnNodeControl_, NodeCheckBox uqNodeControl_, NodeCheckBox binNodeControl_, 
       NodeCheckBox unNodeControl_, NodeCheckBox zfNodeControl_, NodeCheckBox aiNodeControl_,
@@ -38,6 +60,8 @@ namespace MySQL.GUI.Workbench.Plugins
       MySQLTableEditorBE MySQLTableEditorBE)
 			: base(ListView, GrtList, columnIconNodeControl, true)
 		{
+      this.listener = listener;
+
 			nameNodeControl = NameNodeControl;
       datatypeComboBoxNodeControl = DatatypeComboBoxNodeControl;
       pkNodeControl = pkNodeControl_;
@@ -154,57 +178,57 @@ namespace MySQL.GUI.Workbench.Plugins
 					{
 						string caption;
 
-						grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Name, out caption);
-						e.Value = caption;
+						if (grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Name, out caption))
+              e.Value = caption;
 					}
           else if (sender == datatypeComboBoxNodeControl)
           {
             string caption;
 
-            grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Type, out caption);
-            e.Value = caption;
+            if (grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Type, out caption))
+              e.Value = caption;
           }
 					else if (sender == pkNodeControl)
 					{
 						int value;
 
-						grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK, out value);
-						e.Value = (value == 1);
+						if (grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK, out value))
+              e.Value = (value == 1);
 					}
           else if (sender == nnNodeControl)
           {
             int notNull;
 
-            grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsNotNull, out notNull);
-            e.Value = (notNull == 1);
+            if (grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsNotNull, out notNull))
+              e.Value = (notNull == 1);
           }
           else if (sender == uqNodeControl)
           {
             int unique;
 
-            grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnique, out unique);
-            e.Value = (unique == 1);
+            if (grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnique, out unique))
+              e.Value = (unique == 1);
           }
           else if (sender == binNodeControl)
           {
             int value;
 
-            grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsBinary, out value);
-            e.Value = (value == 1);
+            if (grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsBinary, out value))
+              e.Value = (value == 1);
           }
           else if (sender == unNodeControl)
           {
             int value;
 
-            grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnsigned, out value);
-            e.Value = (value == 1);
+            if (grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnsigned, out value))
+              e.Value = (value == 1);
           }
           else if (sender == zfNodeControl)
           {
             int value;
 
-            grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsZerofill, out value);
-            e.Value = (value == 1);
+            if (grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsZerofill, out value))
+              e.Value = (value == 1);
           }
           else if (sender == aiNodeControl)
 					{
@@ -214,15 +238,15 @@ namespace MySQL.GUI.Workbench.Plugins
             // While editing allow to reset the auto inc value but prevent setting it if the
             // column data type does not allow it.
 
-            grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsAutoIncrement, out autoInc);
-						e.Value = (autoInc == 1);
+            if (grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsAutoIncrement, out autoInc))
+              e.Value = (autoInc == 1);
 					}
 					else if (sender == defaultNodeControl)
 					{
 						string caption;
 
-						grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Default, out caption);
-						e.Value = caption;
+						if (grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Default, out caption))
+              e.Value = caption;
 					}
 				}
 			}
@@ -233,7 +257,7 @@ namespace MySQL.GUI.Workbench.Plugins
     /// </summary>
     void activateColumnPlaceholder(NodeId node)
     {
-      // The following code is a bit involved, but it makes the table view
+      // The following code is a bit involved, but it makes the table grid
       // properly display the default PK column name and all its other settings.
       
       // Tell the backend we are editing now the placeholder row.
@@ -333,6 +357,9 @@ namespace MySQL.GUI.Workbench.Plugins
 
           if (selIndex > -1)
             this.ListView.SelectedNode = this.ListView.Root.Children[selIndex];
+
+          if (listener != null)
+            listener.ValueChanged(sender as BindableControl, e.Value);
 				}
 			}
 		}
@@ -344,20 +371,23 @@ namespace MySQL.GUI.Workbench.Plugins
 				TextBox textBox = e.Editor as TextBox;
         if (textBox != null)
         {
-          if (ListView.CurrentNode.Index == 0)
+          // Only do for the placeholder row.
+          if (ListView.CurrentNode.Index == grtList.count() - 1)
           {
             if (e.Node != null && e.Node.Tag != null)
             {
               GrtListNode node = e.Node.Tag as GrtListNode;
 
               String value;
-              grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Type, out value);
+              grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Name, out value);
 
               if (value.Equals(""))
               {
-                activateColumnPlaceholder(node.NodeId);
+                // Mark this row as placeholder. This activates special handling in the backend
+                // (e.g. default value generation).
+                grtList.set_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Name, 1);
 
-                // Read the value the activation code just set and initialize the editor control with it.
+                // Read the default value and initialize the editor control with it.
                 grtList.get_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Name, out value);
                 textBox.Text = value;
               }
@@ -382,7 +412,7 @@ namespace MySQL.GUI.Workbench.Plugins
 
             if (value.Equals(""))
             {
-              activateColumnPlaceholder(node.NodeId);
+              grtList.set_field(node.NodeId, (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Name, 1);
               comboBox.Text = value;
             }
             else
