@@ -19,13 +19,14 @@
 
 using System;
 using System.Collections.Generic;
-using System.ComponentModel;
 using System.Windows.Forms;
 
 using Aga.Controls.Tree;
+using Aga.Controls.Tree.NodeControls;
 
 using MySQL.Forms;
 using MySQL.Grt;
+using MySQL.Grt.Db;
 using MySQL.GUI.Workbench.Plugins.Properties;
 using MySQL.Utilities;
 
@@ -33,12 +34,11 @@ namespace MySQL.GUI.Workbench.Plugins
 {
   // TODO: remove all stored NodeIds . This is just nonsense.
   //       NodeIds should be created on demand (they are temporary anyway) from tree node indices.
-  public partial class DbMysqlTableEditor : ObjectEditorPlugin
+  public partial class DbMysqlTableEditor : ObjectEditorPlugin, IModelChangeListener
   {
     #region Member Variables
 
     private MySQLTableEditorBE tableEditorBE { get { return Backend as MySQLTableEditorBE; } } 
-    private GrtListNode activeNode = null;
     private bool dragInProgress = false;
 
     private DbMysqlTableColumnsListModel columnsListModel;
@@ -58,34 +58,14 @@ namespace MySQL.GUI.Workbench.Plugins
 
     #region Constructors
 
-    public DbMysqlTableEditor(GrtManager GrtManager, GrtValue GrtList)
-      : base(new MySQLTableEditorBE(GrtManager, GrtList))
+    public DbMysqlTableEditor(GrtManager manager, GrtValue value)
+      : base(manager)
     {
       InitializeComponent();
+      ReinitWithArguments(value);
 
-      grtManager = GrtManager;
-
-      sqlEd = new SqlEditorHelper(GrtManager, triggersTabPage, triggersEditorParseLogCallback);
-
-      dbObjectEditorPages = new DbObjectEditorPages(grtManager, tableEditorBE);
-
-      InitializingControls = true;
-      try
-      {
-        InitFormData();
-
-        if (!IsEditingLiveObject)
-          AdjustToSmallerLayout();
-
-        CallRefreshFormData();
-      }
-      finally
-      {
-        InitializingControls = false;
-      }
-
-      tableEditorBE.add_refresh_ui_handler(RefreshFormDataInvoke);
-      tableEditorBE.add_refresh_partial_ui_handler(RefreshPartialFormDataInvoke);
+      if (!IsEditingLiveObject)
+        AdjustToSmallerLayout();
     }
 
     #endregion
@@ -100,25 +80,19 @@ namespace MySQL.GUI.Workbench.Plugins
         topPanel.Parent = mainTabControl.Parent;
     }
 
-    public override bool ChangeGrtList(GrtManager GrtManager, GrtValue GrtList)
+    public override bool ReinitWithArguments(GrtValue value)
     {
-      // Make sure the changes are stored
-      // must be done synchronously otherwise the actual parsing will happen at a random time, after
-      // the editor obj is already deleted
-      sqlEd.Parse(true);
-
       InitializingControls = true;
       SuspendLayout();
 
       try
       {
-        grtManager = GrtManager;
-        grtList = GrtList;
-
-        Backend = new MySQLTableEditorBE(GrtManager, GrtList);
+        Backend = new MySQLTableEditorBE(GrtManager, value);
+        SetupEditorOnHost(triggersTabPage, true);
+        tableEditorBE.load_trigger_sql();
+        dbObjectEditorPages = new DbObjectEditorPages(GrtManager, tableEditorBE);
 
         InitFormData();
-
         CallRefreshFormData();
       }
       finally
@@ -127,8 +101,6 @@ namespace MySQL.GUI.Workbench.Plugins
         InitializingControls = false;
       }
 
-      tableEditorBE.add_refresh_ui_handler(RefreshFormDataInvoke);
-
       Invalidate();
 
       return true;
@@ -149,15 +121,18 @@ namespace MySQL.GUI.Workbench.Plugins
       if (columnsTreeView.Model != null && columnsTreeView.Model is DbMysqlTableColumnsListModel)
         (columnsTreeView.Model as DbMysqlTableColumnsListModel).DetachEvents();
 
-      columnsListModel = new DbMysqlTableColumnsListModel(columnsTreeView, tableEditorBE.get_columns(),
-        columnIconNodeControl, nameNodeControl, datatypeComboBoxNodeControl,
-        pkNodeControl, nnNodeControl, uqNodeControl, binNodeControl, unNodeControl, zfNodeControl, aiNodeControl, defaultNodeControl,
-        tableEditorBE);
+      columnsListModel = new DbMysqlTableColumnsListModel(this, columnsTreeView, tableEditorBE.get_columns(),
+        columnIconNodeControl, nameNodeControl, datatypeComboBoxNodeControl, pkNodeControl,
+        nnNodeControl, uqNodeControl, binNodeControl, unNodeControl, zfNodeControl, aiNodeControl,
+        defaultNodeControl, tableEditorBE);
       
       nameNodeControl.IsEditEnabledValueNeeded += new EventHandler<Aga.Controls.Tree.NodeControls.NodeControlValueEventArgs>(canEdit);
       datatypeComboBoxNodeControl.IsEditEnabledValueNeeded += new EventHandler<Aga.Controls.Tree.NodeControls.NodeControlValueEventArgs>(canEdit);
       defaultNodeControl.IsEditEnabledValueNeeded += new EventHandler<Aga.Controls.Tree.NodeControls.NodeControlValueEventArgs>(canEdit);
 
+      columnCommentTextBox.Text = "";
+      columnCommentTextBox.Enabled = false;
+
       columnsTreeView.Model = columnsListModel;
       columnsTreeView.Refresh();
 
@@ -228,7 +203,7 @@ namespace MySQL.GUI.Workbench.Plugins
       optCollation.Items.Clear();
       optCollation.Items.Add("Schema Default");
 
-      columnCollationComboBox.Enabled = true;
+      columnCollationComboBox.Enabled = false;
       columnCollationComboBox.Items.Clear();
       columnCollationComboBox.Items.Add("Table Default");
 
@@ -251,10 +226,11 @@ namespace MySQL.GUI.Workbench.Plugins
           mainTabControl.TabPages.Add(insertsTabPage);
       }
 
-
-      if (mainTabControl.TabPages.Contains(dbObjectEditorPages.PrivilegesTabPage))
+      // Same as for the inserts page, however the privileges tab page is recreated when
+      // re-initializing the editor, so the Contains check cannot work for this case.
+      if (mainTabControl.TabPages.ContainsKey(dbObjectEditorPages.PrivilegesTabPage.Name))
       {
-        if (IsEditingLiveObject)
+        if (!IsEditingLiveObject)
           mainTabControl.TabPages.Remove(dbObjectEditorPages.PrivilegesTabPage);
       }
       else
@@ -301,6 +277,7 @@ namespace MySQL.GUI.Workbench.Plugins
       oldTableLayoutPanel.SetCellPosition(label12, new TableLayoutPanelCellPosition(0, 1));
       columnCommentTextBox.Parent = oldTableLayoutPanel;
       oldTableLayoutPanel.SetCellPosition(columnCommentTextBox, new TableLayoutPanelCellPosition(1, 1));
+      columnCommentTextBox.AcceptsReturn = true;
 
       // Finally hide the entire lower part (column details).
       tableLayoutPanel1.Hide();
@@ -329,16 +306,6 @@ namespace MySQL.GUI.Workbench.Plugins
       // Sets all the table options.
       refreshTableOptUi();
 
-      if (sqlEd.sqlEditor.IsRefreshEnabled || sqlEd.EditorBE != tableEditorBE)
-      {
-        sqlEd.SetEditorBackend(tableEditorBE, tableEditorBE.set_triggers_sql);
-        sqlEd.sqlEditor.SqlText = tableEditorBE.get_all_triggers_sql();
-      }
-      else if (sqlEd.sqlEditor.IsSqlCheckEnabled)
-      {
-        sqlEd.sqlEditor.CheckSql(false);
-      }
-
       nameTextBox.Text = tableEditorBE.get_name();
       optComments.Text = tableEditorBE.get_comment();
 
@@ -447,7 +414,7 @@ namespace MySQL.GUI.Workbench.Plugins
         idx++;
       }
 
-      TabText = nameTextBox.Text;
+      TabText = tableEditorBE.get_title();
 
       columnsListModel.RefreshModel();
       indicesListModel.RefreshModel();
@@ -480,6 +447,8 @@ namespace MySQL.GUI.Workbench.Plugins
         insertsRecordsetView.Model = tableEditorBE.get_inserts_model();
         insertsRecordsetView.Model.refresh();
       }
+
+      tableEditorBE.load_trigger_sql();
     }
 
     private void nameTextBox_TextChanged(object sender, EventArgs e)
@@ -487,7 +456,7 @@ namespace MySQL.GUI.Workbench.Plugins
       if (!InitializingControls && !nameTextBox.Text.Equals(tableEditorBE.get_name()))
         tableEditorBE.set_name(nameTextBox.Text);
 
-      TabText = nameTextBox.Text;
+      TabText = tableEditorBE.get_title();
     }
 
     private void commentsTextBox_TextChanged(object sender, EventArgs e)
@@ -621,23 +590,28 @@ namespace MySQL.GUI.Workbench.Plugins
       if (InsideInitializationOrRefresh())
         return;
 
-      activeNode = null; // Make sure the following changes won't be committed.
-
       if (columnsTreeView.SelectedNode != null)
       {
         NodeId nodeId = (columnsTreeView.SelectedNode.Tag as GrtListNode).NodeId;
 
         string stringValue;
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Comment, out stringValue);
-        columnCommentTextBox.Text = stringValue;
-        columnCommentTextBox.Enabled = true;
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Comment, out stringValue))
+        {
+          columnCommentTextBox.Text = stringValue;
+          columnCommentTextBox.Enabled = true;
+        }
+        else
+        {
+          columnCommentTextBox.Text = "";
+          columnCommentTextBox.Enabled = false;
+        }
 
         // charset/collation
         String hasCharset = null;
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.HasCharset, out hasCharset);
-        if (hasCharset == "1")
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.HasCharset, out hasCharset) &&
+          (hasCharset == "1"))
         {
           String column_cscoll;
           columnsListModel.GrtList.get_field(nodeId,
@@ -664,62 +638,129 @@ namespace MySQL.GUI.Workbench.Plugins
           columnCollationComboBox.Enabled = false;
         }
 
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Name, out stringValue);
-        columnNameTextBox.Text = stringValue;
-        columnNameTextBox.Enabled = true;
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Name, out stringValue))
+        {
+          columnNameTextBox.Text = stringValue;
+          columnNameTextBox.Enabled = true;
+        }
+        else
+        {
+          columnNameTextBox.Text = "";
+          columnNameTextBox.Enabled = false;
+        }
 
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Type, out stringValue);
-        columnDataTypeTextBox.Text = stringValue;
-        columnDataTypeTextBox.Enabled = true;
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Type, out stringValue))
+        {
+          columnDataTypeTextBox.Text = stringValue;
+          columnDataTypeTextBox.Enabled = true;
+        }
+        else
+        {
+          columnDataTypeTextBox.Text = "";
+          columnDataTypeTextBox.Enabled = false;
+        }
 
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Default, out stringValue);
-        columnDefaultTextBox.Text = stringValue;
-        columnDefaultTextBox.Enabled = true;
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.Default, out stringValue))
+        {
+          columnDefaultTextBox.Text = stringValue;
+          columnDefaultTextBox.Enabled = true;
+        }
+        else
+        {
+          columnDefaultTextBox.Text = "";
+          columnDefaultTextBox.Enabled = false;
+        }
 
         int flag;
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK, out flag);
-        pkCheckBox.Checked = flag != 0;
-        pkCheckBox.Enabled = true;
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK, out flag))
+        {
+          pkCheckBox.Checked = flag != 0;
+          pkCheckBox.Enabled = true;
+        }
+        else
+        {
+          pkCheckBox.Checked = false;
+          pkCheckBox.Enabled = false;
+        }
 
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsNotNull, out flag);
-        nnCheckBox.Checked = flag != 0;
-        nnCheckBox.Enabled = true;
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsNotNull, out flag))
+        {
+          nnCheckBox.Checked = flag != 0;
+          nnCheckBox.Enabled = true;
+        }
+        else
+        {
+          nnCheckBox.Checked = false;
+          nnCheckBox.Enabled = false;
+        }
 
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnique, out flag);
-        uniqueCheckBox.Checked = flag != 0;
-        uniqueCheckBox.Enabled = true;
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnique, out flag))
+        {
+          uniqueCheckBox.Checked = flag != 0;
+          uniqueCheckBox.Enabled = true;
+        }
+        else
+        {
+          uniqueCheckBox.Checked = false;
+          uniqueCheckBox.Enabled = false;
+        }
 
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsBinary, out flag);
-        binaryCheckBox.Checked = flag != 0;
-        binaryCheckBox.Enabled = true;
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsBinary, out flag))
+        {
+          binaryCheckBox.Checked = flag != 0;
+          binaryCheckBox.Enabled = true;
+        }
+        else
+        {
+          binaryCheckBox.Checked = false;
+          binaryCheckBox.Enabled = false;
+        }
 
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnsigned, out flag);
-        unsignedCheckBox.Checked = flag != 0;
-        unsignedCheckBox.Enabled = true;
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsUnsigned, out flag))
+        {
+          unsignedCheckBox.Checked = flag != 0;
+          unsignedCheckBox.Enabled = true;
+        }
+        else
+        {
+          unsignedCheckBox.Checked = false;
+          unsignedCheckBox.Enabled = false;
+        }
 
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsZerofill, out flag);
-        zeroFillCheckBox.Checked = flag != 0;
-        zeroFillCheckBox.Enabled = true;
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsZerofill, out flag))
+        {
+          zeroFillCheckBox.Checked = flag != 0;
+          zeroFillCheckBox.Enabled = true;
+        }
+        else
+        {
+          zeroFillCheckBox.Checked = false;
+          zeroFillCheckBox.Enabled = false;
+        }
 
-        columnsListModel.GrtList.get_field(nodeId,
-          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsAutoIncrement, out flag);
-        aiCheckBox.Checked = flag != 0;
-        aiCheckBox.Enabled = true;
-
-        activeNode = columnsTreeView.SelectedNode.Tag as GrtListNode;
+        if (columnsListModel.GrtList.get_field(nodeId,
+          (int)MySQLTableColumnsListBE.MySQLColumnListColumns.IsAutoIncrement, out flag))
+        {
+          aiCheckBox.Checked = flag != 0;
+          aiCheckBox.Enabled = true;
+        }
+        else
+        {
+          aiCheckBox.Checked = false;
+          aiCheckBox.Enabled = false;
+        }
       }
       else
       {
-        activeNode = null;
         columnCommentTextBox.Enabled = false;
         columnCommentTextBox.Text = "";
         columnCollationComboBox.Enabled = false;
@@ -749,16 +790,17 @@ namespace MySQL.GUI.Workbench.Plugins
     private void columnCollationComboBox_SelectedIndexChanged(object sender, EventArgs e)
     {
       // set charset/collation
-      if (activeNode != null)
+      if (columnsTreeView.SelectedNode != null)
       {
+        NodeId nodeId = new NodeId(columnsTreeView.SelectedNode.Index);
         if (columnCollationComboBox.SelectedIndex == 0)
         {
-          columnsListModel.GrtList.set_field(activeNode.NodeId,
+          columnsListModel.GrtList.set_field(nodeId,
             (int)MySQLTableColumnsListBE.MySQLColumnListColumns.CharsetCollation, "");
         }
         else
         {
-          columnsListModel.GrtList.set_field(activeNode.NodeId,
+          columnsListModel.GrtList.set_field(nodeId,
             (int)MySQLTableColumnsListBE.MySQLColumnListColumns.CharsetCollation,
               columnCollationComboBox.Text);
         }
@@ -767,12 +809,12 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private void updateColumnTextField(MySQLTableColumnsListBE.MySQLColumnListColumns column, TextBox box)
     {
-      if (activeNode != null)
+      if (columnsTreeView.SelectedNode != null)
       {
         // Change an attribute triggers a whole chain of events which ultimately cause the
         // table editor to refresh completely (including all nodes in the columns tree etc.).
         // So we have to take measure to restore what was active when the user changed the value.
-        NodeId nodeId = activeNode.NodeId;
+        NodeId nodeId = new NodeId(columnsTreeView.SelectedNode.Index);
         if (!columnsListModel.GrtList.set_field(nodeId, (int)column, box.Text))
         {
           CustomMessageBox.Show(MessageType.MessageError, "Could not set new value",
@@ -790,7 +832,6 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private void applyColumnText(TextBox box)
     {
-      NodeId nodeId = activeNode.NodeId;
       MySQLTableColumnsListBE.MySQLColumnListColumns column = MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK;
       switch (box.Tag.ToString())
       {
@@ -810,34 +851,22 @@ namespace MySQL.GUI.Workbench.Plugins
       if (column != MySQLTableColumnsListBE.MySQLColumnListColumns.IsPK)
       {
         updateColumnTextField(column, box);
-        //box.Focus();
         box.SelectAll();
       }
     }
 
     private void columnTextBox_Leave(object sender, EventArgs e)
     {
-      if (activeNode != null && !IsDisposed)
+      if (columnsTreeView.SelectedNode != null && !IsDisposed)
         applyColumnText(sender as TextBox);
     }
 
-    private void columnTextBox_KeyDown(object sender, KeyEventArgs e)
-    {
-      if (activeNode != null && (e.KeyCode == Keys.Enter || e.KeyCode == Keys.Return))
-      {
-        applyColumnText(sender as TextBox);
-
-        e.SuppressKeyPress = true;
-        e.Handled = true;
-      }
-    }
-
     private void columnFlagsChanged(object sender, EventArgs e)
     {
-      if (activeNode == null)
+      if (columnsTreeView.SelectedNode == null)
         return;
 
-      NodeId nodeId = activeNode.NodeId;
+      NodeId nodeId = new NodeId(columnsTreeView.SelectedNode.Index);
       CheckBox box = sender as CheckBox;
       int value = box.Checked ? 1 : 0;
       MySQLTableColumnsListBE.MySQLColumnListColumns columnValue = MySQLTableColumnsListBE.MySQLColumnListColumns.Name;
@@ -868,10 +897,10 @@ namespace MySQL.GUI.Workbench.Plugins
 
       if (columnValue != MySQLTableColumnsListBE.MySQLColumnListColumns.Name)
       {
-        if (!columnsListModel.GrtList.set_field(activeNode.NodeId, (int)columnValue, value))
+        if (!columnsListModel.GrtList.set_field(nodeId, (int)columnValue, value))
         {
           // Value not accepted. Restore previous one.
-          columnsListModel.GrtList.get_field(activeNode.NodeId, (int)columnValue, out value);
+          columnsListModel.GrtList.get_field(nodeId, (int)columnValue, out value);
           box.Checked = value != 0;
         }
       }
@@ -990,11 +1019,11 @@ namespace MySQL.GUI.Workbench.Plugins
         {
           String text = null;
           tableEditorBE.get_indexes().get_field(node.NodeId, 
-            (int)MySQL.GUI.Workbench.Plugins.MySQLIndexListBE.Columns.Parser, out text);
+            (int)MySQLIndexListBE.Columns.Parser, out text);
           if (indexParserText.Text != text)
           {
             tableEditorBE.get_indexes().set_field(node.NodeId, 
-              (int)MySQL.GUI.Workbench.Plugins.MySQLIndexListBE.Columns.Parser, indexParserText.Text);
+              (int)MySQLIndexListBE.Columns.Parser, indexParserText.Text);
           }
         }
       }
@@ -1010,11 +1039,11 @@ namespace MySQL.GUI.Workbench.Plugins
         {
           String text = null;
           tableEditorBE.get_indexes().get_field(node.NodeId, 
-            (int)MySQL.GUI.Workbench.Plugins.MySQLIndexListBE.Columns.RowBlockSize, out text);
+            (int)MySQLIndexListBE.Columns.RowBlockSize, out text);
           if (indexRowBlockSizeText.Text != text)
           {
             tableEditorBE.get_indexes().set_field(node.NodeId, 
-              (int)MySQL.GUI.Workbench.Plugins.MySQLIndexListBE.Columns.RowBlockSize, indexRowBlockSizeText.Text);
+              (int)MySQLIndexListBE.Columns.RowBlockSize, indexRowBlockSizeText.Text);
           }
         }
       }
@@ -1030,19 +1059,17 @@ namespace MySQL.GUI.Workbench.Plugins
         {
           String text = null;
           tableEditorBE.get_indexes().get_field(node.NodeId,
-            (int)MySQL.GUI.Workbench.Plugins.MySQLIndexListBE.Columns.StorageType, out text);
+            (int)MySQLIndexListBE.Columns.StorageType, out text);
 
           if (indexStorageTypeComboBox.SelectedItem == null)
           {
             tableEditorBE.get_indexes().set_field(node.NodeId,
-              (int)MySQL.GUI.Workbench.Plugins.MySQLIndexListBE.Columns.StorageType,
-              "");
+              (int)MySQLIndexListBE.Columns.StorageType, "");
           }
           else if (indexStorageTypeComboBox.SelectedItem.ToString() != text)
           {
             tableEditorBE.get_indexes().set_field(node.NodeId,
-              (int)MySQL.GUI.Workbench.Plugins.MySQLIndexListBE.Columns.StorageType,
-              indexStorageTypeComboBox.SelectedItem.ToString());
+              (int)MySQLIndexListBE.Columns.StorageType, indexStorageTypeComboBox.SelectedItem.ToString());
           }
         }
       }
@@ -1230,14 +1257,6 @@ namespace MySQL.GUI.Workbench.Plugins
       nameTextBox.Focus();
     }
 
-    private bool forceClosing = false;
-    public override void Close(bool force)
-    {
-      forceClosing = force;
-      base.Close(force);
-      forceClosing = false;
-    }
-
     #endregion
 
     #region Partitioning
@@ -1440,7 +1459,6 @@ namespace MySQL.GUI.Workbench.Plugins
           fkCommentText.Text = text;
 
           tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListBE.FKConstraintListColumns.Index, out text);
-          //fkIndexLabel.Text = text;
 
           int value= 0;
           tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListBE.FKConstraintListColumns.ModelOnly, out value);
@@ -1568,7 +1586,7 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private void triggersTabPage_Enter(object sender, EventArgs e)
     {
-      ActiveControl = sqlEd.sqlEditor;
+      ActivateEditor();
     }
 
     private void triggersEditorParseLogCallback(List<String> strlist)
@@ -1588,7 +1606,7 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private void insertsTabPage_Enter(object sender, EventArgs e)
     {
-      if (null == insertsRecordsetView)
+      if (insertsRecordsetView == null)
       {
         insertsRecordsetView = new MySQL.Grt.Db.RecordsetView();
         insertsRecordsetView.Embed(insertsTabPage, tableEditorBE.get_inserts_model());
@@ -1596,19 +1614,6 @@ namespace MySQL.GUI.Workbench.Plugins
       insertsRecordsetView.Model.refresh();
     }
 
-    private void insertsTabPage_Validating(object sender, CancelEventArgs e)
-    {
-      if (!forceClosing)
-      {
-        e.Cancel = insertsRecordsetView.Model.has_pending_changes();
-        if (e.Cancel)
-        {
-          // provoke message about pending changes
-          insertsRecordsetView.Model.limit_rows(insertsRecordsetView.Model.limit_rows());
-        }
-      }
-    }
-
     #endregion
 
     #region Event handling
@@ -1630,11 +1635,6 @@ namespace MySQL.GUI.Workbench.Plugins
       }
     }
 
-    private void fkColumnsTreeView_Click(object sender, EventArgs e)
-    {
-      targetColumnFkNodeControl.BeginEdit();
-    }
-
     private void collapsePictureBox_Click(object sender, EventArgs e)
     {
       headingCollapsed = !headingCollapsed;
@@ -1653,47 +1653,6 @@ namespace MySQL.GUI.Workbench.Plugins
       }
     }
 
-    private void undoItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.UndoRedo.Undo();
-    }
-
-    private void cutItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.Clipboard.Cut();
-    }
-
-    private void copyItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.Clipboard.Copy();
-    }
-
-    private void pasteItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.Clipboard.Paste();
-    }
-
-    private void selectAllItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.Selection.SelectAll();
-    }
-
-    private void redoItem_Click(object sender, EventArgs e)
-    {
-      sqlEd.sqlEditor.UndoRedo.Redo();
-    }
-
-    private void editorContextMenu_Opening(object sender, CancelEventArgs e)
-    {
-      // Enable/Disable menu items depending on the edit content.
-      undoItem.Enabled = sqlEd.sqlEditor.UndoRedo.CanUndo;
-      redoItem.Enabled = sqlEd.sqlEditor.UndoRedo.CanRedo;
-      cutItem.Enabled = sqlEd.sqlEditor.Selection.Length > 0;
-      copyItem.Enabled = sqlEd.sqlEditor.Selection.Length > 0;
-      pasteItem.Enabled = sqlEd.sqlEditor.Clipboard.CanPaste;
-      selectAllItem.Enabled = sqlEd.sqlEditor.TextLength > 0;
-    }
-
     #endregion
 
     #region Drag & Drop
@@ -1800,5 +1759,63 @@ namespace MySQL.GUI.Workbench.Plugins
 
     #endregion
 
+    #region IWorkbenchDocument Interface
+
+    public override bool CanCloseDocument()
+    {
+      if (insertsRecordsetView != null && insertsRecordsetView.Model.has_pending_changes())
+      {
+        CustomMessageBox.Show(MessageType.MessageWarning, "Inserts Data Changed", "There are pending changes that " +
+          "must be applied first before you can close the editor. Alternatively you can revert to the previous state. ", "OK");
+        return false;
+      }
+      return base.CanCloseDocument();
+    }
+    
+    #endregion
+
+    #region IModelChangeListener interface
+
+    /// <summary>
+    /// Called by the model if something was changed there. We use this notification to update
+    /// our separate input elements (especially the checkboxes). No need to also update the text fields
+    /// as they are already adjusted in the selection change event handler.
+    /// </summary>
+    /// <param name="control">The control that caused the change.</param>
+    public void ValueChanged(BindableControl control, Object value)
+    {
+      if (value == null || !(control is NodeCheckBox))
+        return;
+
+      bool boolValue = Convert.ToBoolean(value);
+      if (control == pkNodeControl)
+        pkCheckBox.Checked = boolValue;
+      else
+        if (control == nnNodeControl)
+          nnCheckBox.Checked = boolValue;
+        else
+          if (control == uqNodeControl)
+            uniqueCheckBox.Checked = boolValue;
+          else
+            if (control == binNodeControl)
+              binaryCheckBox.Checked = boolValue;
+            else
+              if (control == unNodeControl)
+                unsignedCheckBox.Checked = boolValue;
+              else
+                if (control == zfNodeControl)
+                  zeroFillCheckBox.Checked = boolValue;
+                else
+                  if (control == aiNodeControl)
+                    aiCheckBox.Checked = boolValue;
+    }
+
+    #endregion
+
+    public override void EditorLostFocus()
+    {
+      tableEditorBE.commit_changes();
+    }
+
   }
 }
