@@ -41,21 +41,25 @@ namespace MySQL.GUI.Workbench.Plugins
 
     #region Constructors
 
-    public DbMysqlUserEditor()
+    public DbMysqlUserEditor(GrtManager manager, GrtValue value)
+      : base(manager)
     {
       InitializeComponent();
+      ReinitWithArguments(value);
     }
 
-    public DbMysqlUserEditor(GrtManager GrtManager, GrtValue GrtObject)
-      : base(new UserEditorBE(GrtManager, GrtObject))
+    #endregion
+
+    #region ObjectEditorPlugin Overrides
+
+    public override bool ReinitWithArguments(GrtValue value)
     {
-      InitializeComponent();
-
-      // Init BE
-
       InitializingControls = true;
+
       try
       {
+        Backend = new UserEditorBE(GrtManager, value);
+
         nameTextBox.Text = userEditorBE.get_name();
         passwordTextBox.Text = userEditorBE.get_password();
         commentTextBox.Text = userEditorBE.get_comment();
@@ -66,15 +70,17 @@ namespace MySQL.GUI.Workbench.Plugins
         roleTreeModel = new DifferenceByNameGrtTreeModel(roleTreeView, new List<String>(), roleTreeBE, false);
         roleTreeModel.AddColumn(roleTreeNodeText, (int)RoleTreeBE.Columns.Name, false);
         roleTreeView.Model = roleTreeModel;
-
-        RefreshAssignedRoles();
+        
+        RefreshFormData();
       }
       finally
       {
         InitializingControls = false;
       }
 
-      userEditorBE.add_refresh_ui_handler(RefreshFormDataInvoke);
+      Invalidate();
+
+      return true;
     }
 
     #endregion
@@ -127,7 +133,7 @@ namespace MySQL.GUI.Workbench.Plugins
       if (!InitializingControls)
         userEditorBE.set_name(nameTextBox.Text);
 
-      TabText = nameTextBox.Text;
+      TabText = userEditorBE.get_title();
     }
 
     private void passwordTextBox_TextChanged(object sender, EventArgs e)
@@ -145,7 +151,7 @@ namespace MySQL.GUI.Workbench.Plugins
     protected override void RefreshFormData() 
     {
       RefreshAssignedRoles();
-      TabText = userEditorBE.get_name() + " - User";
+      TabText = userEditorBE.get_title();
       nameTextBox.Text = userEditorBE.get_name();
     }
 
