@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,16 +24,13 @@
 #include "grtpp.h"
 #include "GrtTemplates.h"
 
-#ifdef _MSC_VER
 using namespace MySQL::Grt;
 using namespace System;
 using namespace System::Collections::Generic;
-#endif
 
 namespace MySQL {
-namespace GUI {
-namespace Workbench {
-namespace Plugins {
+namespace Grt {
+namespace Db {
 
 ref class MySQLTableEditorBE;
 
@@ -159,7 +156,9 @@ public:
 
   String^ get_all_triggers_sql()
   { return CppStringToNative(get_unmanaged_object()->get_all_triggers_sql()); }
-  //
+  
+  void load_trigger_sql();
+  void commit_changes();
 
   bool set_partition_type(String^ type)
   { return get_unmanaged_object()->set_partition_type(NativeToCppString(type)); }
@@ -214,10 +213,8 @@ public:
   { return get_unmanaged_object()->get_explicit_subpartitions(); }
 };
 
-
-};  // namespace Plugins
-};  // namespace Workbench
-};  // namespace GUI
+};  // namespace Db
+};  // namespace Grt
 };  // namespace MySQL
 
 #endif // __MYSQL_TABLE_EDITOR_BE_H__
\ No newline at end of file
