@@ -7,14 +7,14 @@
 #include "grts/structs.db.mysql.h"
 
 #include "grtpp.h"
+#include "grt/grt_manager.h"
 #include "grt/grt_reporter.h"
 #include "grtdb/diff_dbobjectmatch.h"
+#include "base/log.h"
 
 using namespace grt;
 
-#include "grti/sqlgenerator.h"
-//QQQ#include "grti/wbvalidation.h"
-//QQQ#include "grti/wbvalidationmysql.h"
+#include "interfaces/sqlgenerator.h"
 
 #include "db_mysql_sql_export.h"
 #include "diff_tree.h"
@@ -23,6 +23,8 @@ using namespace grt;
 #include "db.mysql/src/module_db_mysql_shared_code.h"
 #include "grtsqlparser/sql_facade.h"
 
+DEFAULT_LOG_DOMAIN("DbMySQLSQLExport");
+
 void DbMySQLSQLExport::init_from_ctor(bec::GRTManager *grtm, db_mysql_CatalogRef catalog)
 {
   //_manager= grtm;
@@ -119,19 +121,19 @@ grt::StringListRef convert_string_vector_to_grt_list(grt::GRT *grt, const std::v
   return grt_list;
 }
 
-static void parse_string_from_filter(const std::string& source, 
-                                     std::string& out_prefix,
-                                     std::string& out_suffix)
-{
-  std::string::size_type period= source.find('.');
-  out_prefix.assign(source.substr(0, period));
-  if(period != std::string::npos)
-    out_suffix.assign(source.substr(period+1));
-}
+//static void parse_string_from_filter(const std::string& source, 
+//                                     std::string& out_prefix,
+//                                     std::string& out_suffix)
+//{
+//  std::string::size_type period= source.find('.');
+//  out_prefix.assign(source.substr(0, period));
+//  if(period != std::string::npos)
+//    out_suffix.assign(source.substr(period+1));
+//}
 
 typedef std::map<std::string, std::list<std::string> > StringListMap;
 
-std::vector<std::string> get_names(const bec::GrtStringListModel* list,const std::map<std::string,GrtNamedObjectRef>& obj_map,std::set<db_mysql_SchemaRef>& schemas)
+std::vector<std::string> get_names(const bec::GrtStringListModel* list,const std::map<std::string,GrtNamedObjectRef>& obj_map,std::set<db_mysql_SchemaRef>& schemas, const bool case_sensitive)
 {
   std::vector<std::string> work_vector;
   const std::vector<std::string>& list_items = list->items();
@@ -140,7 +142,7 @@ std::vector<std::string> get_names(const bec::GrtStringListModel* list,const std
     std::map<std::string,GrtNamedObjectRef>::const_iterator ItTableName = obj_map.find(*It);
     if (ItTableName != obj_map.end())
     {
-      work_vector.push_back(get_old_object_name_for_key(ItTableName->second));
+      work_vector.push_back(get_old_object_name_for_key(ItTableName->second, case_sensitive));
       if(db_mysql_TriggerRef::can_wrap(ItTableName->second))
         schemas.insert(db_mysql_SchemaRef::cast_from(ItTableName->second->owner()->owner()));
       else if (db_mysql_SchemaRef::can_wrap(ItTableName->second->owner()))
@@ -171,7 +173,7 @@ grt::DictRef DbMySQLSQLExport::get_options_as_dict(grt::GRT *grt)
   db_mysql_CatalogRef cat(get_model_catalog());
 
   for(size_t i= 0; i < cat->schemata().count(); i++)
-    schemata_names.push_back(get_old_object_name_for_key(cat->schemata().get(i)));
+    schemata_names.push_back(get_old_object_name_for_key(cat->schemata().get(i), _case_sensitive));
 
   std::set<db_mysql_SchemaRef> schemas;
 
@@ -179,26 +181,28 @@ grt::DictRef DbMySQLSQLExport::get_options_as_dict(grt::GRT *grt)
 
   // filtering options
   options.set("TableFilterList", _tables_are_selected ? 
-    convert_string_vector_to_grt_list(grt,get_names(_tables_model,_tables_map, schemas)) : grt::StringListRef(grt));
+    convert_string_vector_to_grt_list(grt,get_names(_tables_model,_tables_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
 
   options.set("ViewFilterList", _views_are_selected ? 
-    convert_string_vector_to_grt_list(grt, get_names(_views_model,_views_map, schemas)) : grt::StringListRef(grt));
+    convert_string_vector_to_grt_list(grt, get_names(_views_model,_views_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
 
   options.set("RoutineFilterList", _routines_are_selected ? 
-    convert_string_vector_to_grt_list(grt, get_names(_routines_model,_routines_map, schemas)) : grt::StringListRef(grt));
+    convert_string_vector_to_grt_list(grt, get_names(_routines_model,_routines_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
 
   options.set("TriggerFilterList", _triggers_are_selected ? 
-    convert_string_vector_to_grt_list(grt, get_names(_triggers_model,_triggers_map, schemas)) : grt::StringListRef(grt));
+    convert_string_vector_to_grt_list(grt, get_names(_triggers_model,_triggers_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
 
   options.set("UserFilterList", _users_are_selected ? 
-    convert_string_vector_to_grt_list(grt, get_names(_users_model,_users_map, schemas)) : grt::StringListRef(grt));
+    convert_string_vector_to_grt_list(grt, get_names(_users_model,_users_map, schemas, _case_sensitive)) : grt::StringListRef(grt));
 
   grt::StringListRef schema_names_list(grt);
   for (std::set<db_mysql_SchemaRef>::const_iterator It = schemas.begin(); It != schemas.end(); ++It)
-    schema_names_list.insert(get_old_object_name_for_key(*It));
+    schema_names_list.insert(get_old_object_name_for_key(*It, _case_sensitive));
   
   options.set("SchemaFilterList", schema_names_list);
 
+  options.set("CaseSensitive", grt::IntegerRef(_case_sensitive));
+
   return options;
 }
 
@@ -221,7 +225,7 @@ void DbMySQLSQLExport::export_finished(grt::ValueRef res)
 {
   CatalogMap cmap;
   update_all_old_names(get_model_catalog(), false, cmap);
-  _manager->get_grt()->send_output(grt::StringRef::cast_from(res).c_str());
+  _manager->get_grt()->send_output(*grt::StringRef::cast_from(res) + '\n');
   if (_task_finish_cb)
     _task_finish_cb();
 }
@@ -232,54 +236,41 @@ ValueRef DbMySQLSQLExport::export_task(grt::GRT* grt, grt::StringRef)
 
   try
   {
-    SQLGeneratorInterfaceWrapper *diffsql_module= 
-      //grt->get_module_wrapper<SQLGeneratorInterfaceWrapper>(grt->get_module("MySQLModuleDbMySQL"));
-      grt->get_module_wrapper<SQLGeneratorInterfaceWrapper>(grt->get_module("DbMySQL"));
+    SQLGeneratorInterfaceImpl *diffsql_module= dynamic_cast<SQLGeneratorInterfaceImpl*>(grt->get_module("DbMySQL"));
   
     if(diffsql_module == NULL)
-      return grt::StringRef("\nSQL Script Export Error: Not able to load 'MySQLModuleDbMySQL' module");
+      return grt::StringRef("\nSQL Script Export Error: Not able to load 'DbMySQL' module");
 
-    grt::ValueRef e(NULL);
-
-//  return diff_make(cat, org_cat, &omf, boost::bind(&NormalizedComparer<grt::GRT*>::normalizedComparison, cmp));
-
-    DiffChange* create_change= diff_make(e, _catalog, NULL, grt::NormalizedComparer(grt));
-    DiffChange* drop_change= diff_make(_catalog, e, NULL, grt::NormalizedComparer(grt));
-    DictRef create_map(grt);
-    DictRef drop_map(grt);
-
-    char sprintf_buf[32];
-    
-    std::string change_ptr;
-    std::string log_ptr;
-
-    sprintf(sprintf_buf, "%p", create_change);
-    change_ptr.assign(sprintf_buf);
+    DictRef create_map;
+    DictRef drop_map;
 
     grt::DictRef options= get_options_as_dict(grt);
 
-    options.set("OutputContainer", create_map);
-
-    diffsql_module->generateSQL(GrtNamedObjectRef::cast_from(ValueRef(NULL)), options, change_ptr);
-    //delete create_change;
-
-    sprintf(sprintf_buf, "%p", drop_change);
-    change_ptr.assign(sprintf_buf);
-
-    if(_gen_drops)
+    options.set("SQL_MODE", _manager->get_app_option("SqlGenerator.Mysql:SQL_MODE"));
+    if(_db_options.is_valid())
     {
-      options.set("OutputContainer", drop_map);
-      diffsql_module->generateSQL(GrtNamedObjectRef::cast_from(_catalog), options, change_ptr);
-      //delete drop_change;
+        _db_options.set("CaseSensitive", grt::IntegerRef(1));
+        options.set("DBSettings",_db_options);
     }
 
+    create_map = diffsql_module->generateSQLForDifferences(GrtNamedObjectRef(), _catalog, options);
+
+    if (_gen_drops)
+      drop_map = diffsql_module->generateSQLForDifferences(_catalog, GrtNamedObjectRef(), options);
+    if (!drop_map.is_valid())
+      drop_map = grt::DictRef(grt);
+    
     grt::StringListRef strlist= grt::StringListRef::cast_from(options.get("ViewFilterList"));
 
-    if(diffsql_module->makeSQLExportScript(_catalog, options, create_map, drop_map))
+    // generateSQLForDifferences() will set DiffCaseSensitiveness to the used value
+    _case_sensitive = options.get_int("DiffCaseSensitiveness", _case_sensitive);
+    options.set("CaseSensitive", grt::IntegerRef(_case_sensitive));
+
+    if (diffsql_module->makeSQLExportScript(_catalog, options, create_map, drop_map))
     {
       return grt::StringRef("\nSQL Script Export Error: SQL Script Export Module Returned Error");
     }
-
+    
     _export_sql_script= options.get_string("OutputScript");
     if (!_output_filename.empty())
     {
