@@ -27,6 +27,7 @@ class WBPLUGINDBMYSQLBE_PUBLIC_FUNC DbMySQLSQLExport : public DbMySQLValidationP
   bool _generate_use;
   bool _skip_foreign_keys;
   bool _skip_fk_indexes;
+  bool _case_sensitive;
 
   bec::GrtStringListModel *_users_model;
   bec::GrtStringListModel *_users_exc_model;
@@ -44,6 +45,7 @@ class WBPLUGINDBMYSQLBE_PUBLIC_FUNC DbMySQLSQLExport : public DbMySQLValidationP
   std::map<std::string,GrtNamedObjectRef> _views_map;
   std::map<std::string,GrtNamedObjectRef> _routines_map;
   std::map<std::string,GrtNamedObjectRef> _triggers_map;
+  grt::DictRef _db_options;
 
   std::string get_q_name(const char *part1, const char *part2)
   {
@@ -71,6 +73,7 @@ public:
   
   void set_option(const std::string& name, bool value);
   void set_option(const std::string& name, const std::string& value);
+  void set_db_options(const grt::DictRef& db_options){_db_options = db_options;};
 
   void start_export(bool wait_finish);
   //void run_validation();
