@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,8 +30,9 @@
 #include "grt/icon_manager.h"
 #include "grts/structs.db.h"
 #include "base/string_utilities.h"
+#include "db.mysql/src/module_db_mysql.h"
 
-#include <glib/gunicode.h>
+#include <glib.h>
 
 void Db_plugin::grtm(bec::GRTManager *grtm)
 {
@@ -44,7 +45,7 @@ void Db_plugin::grtm(bec::GRTManager *grtm)
     _doc= workbench_DocumentRef::cast_from(_grtm->get_grt()->get("/wb/doc"));
 
     db_mgmt_ManagementRef mgmt= workbench_WorkbenchRef::cast_from(_doc->owner())->rdbmsMgmt();
-    _db_conn.init(mgmt, true);
+    _db_conn = new DbConnection(mgmt, db_mgmt_DriverRef(), false);
 
     _tables.icon_id(table_icon_id(bec::Icon16));
     _views.icon_id(view_icon_id(bec::Icon16));
@@ -63,15 +64,14 @@ std::string Db_plugin::task_desc()
 }
 
 
-void Db_plugin::model_catalog(db_CatalogRef catalog)
+db_mgmt_RdbmsRef Db_plugin::selected_rdbms()
 {
-  _catalog= catalog;
+  return db_mgmt_RdbmsRef::cast_from(_db_conn->get_connection()->driver()->owner());
 }
 
-
 db_CatalogRef Db_plugin::model_catalog()
 {
-  db_mgmt_RdbmsRef rdbms= _db_conn.get_rdbms();
+  db_mgmt_RdbmsRef rdbms= selected_rdbms();
 
   // find first appropriate model catalog for selected rdbms
   grt::ListRef<workbench_physical_Model> physicalModels= _doc->physicalModels();
@@ -129,18 +129,25 @@ std::string Db_plugin::db_objects_struct_name_by_type(Db_object_type db_object_t
   return obj.get_metaclass()->get_member_type(attr_name).content.object_class;
 }
 
-
 void Db_plugin::load_schemata(std::vector<std::string> &schemata)
 {
   _schemata.clear();
   _schemata_ddl.clear();
 
-  sql::ConnectionWrapper dbc_conn= _db_conn.get_dbc_connection();
+  sql::ConnectionWrapper dbc_conn= _db_conn->get_dbc_connection();
   sql::DatabaseMetaData *dbc_meta(dbc_conn->getMetaData());
 
   _grtm->get_grt()->send_info(_("Fetching schema list."));
   _grtm->get_grt()->send_progress(0.0, _("Fetching schema list..."));
 
+  const unsigned int major = dbc_meta->getDatabaseMajorVersion();
+  const unsigned int minor = dbc_meta->getDatabaseMinorVersion();
+  const unsigned int revision = dbc_meta->getDatabasePatchVersion();
+
+  DbMySQLImpl *diffsql_module= _grtm->get_grt()->find_native_module<DbMySQLImpl>("DbMySQL");
+  _db_options = diffsql_module->getTraitsForServerVersion(major, minor, revision);
+  _db_options.set("CaseSensitive", grt::IntegerRef(dbc_meta->storesMixedCaseIdentifiers()));
+
   std::auto_ptr<sql::ResultSet> rset(dbc_meta->getSchemaObjects("", "", "schema"));
   _schemata.reserve(rset->rowsCount());
   float total= (float)rset->rowsCount();
@@ -159,26 +166,26 @@ void Db_plugin::load_schemata(std::vector<std::string> &schemata)
   schemata= _schemata;
 }
 
-
-void Db_plugin::default_schemata_selection(std::vector<std::string> &selection)
-{
-  grt::ListRef<db_Schema> model_schemata= model_catalog()->schemata();
-
-  for (grt::ListRef<db_Schema>::const_iterator It= model_schemata.begin(); It != model_schemata.end(); ++It)
-  {
-    selection.push_back((*It)->name());  
-  }
-  return;
-/*
-//select only schemas thet exists both in  DB and model
-  for (size_t n= 0, count= _schemata.size(); n < count; ++n)
-  {
-    db_SchemaRef model_schema= grt::find_named_object_in_list(model_schemata, _schemata[n].c_str());
-    if (model_schema.is_valid())
-      selection.push_back(_schemata[n]);
-  }
-  */
-}
+//
+//void Db_plugin::default_schemata_selection(std::vector<std::string> &selection)
+//{
+//  grt::ListRef<db_Schema> model_schemata= model_catalog()->schemata();
+//
+//  for (grt::ListRef<db_Schema>::const_iterator It= model_schemata.begin(); It != model_schemata.end(); ++It)
+//  {
+//    selection.push_back((*It)->name());  
+//  }
+//  return;
+///*
+////select only schemas thet exists both in  DB and model
+//  for (size_t n= 0, count= _schemata.size(); n < count; ++n)
+//  {
+//    db_SchemaRef model_schema= grt::find_named_object_in_list(model_schemata, _schemata[n].c_str());
+//    if (model_schema.is_valid())
+//      selection.push_back(_schemata[n]);
+//  }
+//  */
+//}
 
 
 void Db_plugin::schemata_selection(const std::vector<std::string> &selection, bool sel_none_means_sel_all)
@@ -199,7 +206,7 @@ void Db_plugin::load_db_objects(Db_object_type db_object_type)
   
   _grtm->get_grt()->send_progress(0.0, std::string("Fetching ").append(db_objects_type_to_string(db_object_type)).append(" list."));
   
-  sql::ConnectionWrapper dbc_conn= _db_conn.get_dbc_connection();
+  sql::ConnectionWrapper dbc_conn= _db_conn->get_dbc_connection();
   sql::DatabaseMetaData *dbc_meta(dbc_conn->getMetaData());
   std::string db_object_type_name= db_objects_type_to_string(db_object_type);
   std::list<Db_obj_handle> db_objects;
@@ -218,23 +225,26 @@ void Db_plugin::load_db_objects(Db_object_type db_object_type)
     _grtm->get_grt()->send_progress((current_schema / total_schemas), 
                                     std::string("Fetch ").append(db_objects_type_to_string(db_object_type)).append(" objects from ").append(schema_name));
 
-    std::auto_ptr<sql::ResultSet> rset(dbc_meta->getSchemaObjects("", schema_name, db_object_type_name));
-    total_objects= (float)rset->rowsCount();
-    while (rset->next())
+    if (!schema_name.empty())
     {
-      Db_obj_handle db_obj;
-      db_obj.schema= schema_name;
-      db_obj.name= rset->getString("name");
-      db_obj.ddl= rset->getString("ddl");
-      setup->all.push_back(db_obj);
-
-      // prefixed by schema name
-      db_obj_names.push_back(std::string(schema_name).append(".").append(db_obj.name));
-
-      _grtm->get_grt()->send_progress((current_schema / total_schemas) + (count / total_objects)/total_schemas,
-                                      db_obj_names.back());
-      
-      count++;
+      std::auto_ptr<sql::ResultSet> rset(dbc_meta->getSchemaObjects("", schema_name, db_object_type_name));
+      total_objects= (float)rset->rowsCount();
+      while (rset->next())
+      {
+        Db_obj_handle db_obj;
+        db_obj.schema= schema_name;
+        db_obj.name= rset->getString("name");
+        db_obj.ddl= rset->getString("ddl");
+        setup->all.push_back(db_obj);
+        
+        // prefixed by schema name
+        db_obj_names.push_back(std::string(schema_name).append(".").append(db_obj.name));
+        
+        _grtm->get_grt()->send_progress((current_schema / total_schemas) + (count / total_objects)/total_schemas,
+                                        db_obj_names.back());
+        
+        count++;
+      }
     }
 
     current_schema++;
@@ -305,7 +315,7 @@ void Db_plugin::dump_ddl(Db_object_type db_object_type, std::string &sql_script)
 {
   std::string _non_std_sql_delimiter;
   {
-    SqlFacade::Ref sql_facade= SqlFacade::instance_for_rdbms(_db_conn.get_rdbms());
+    SqlFacade::Ref sql_facade= SqlFacade::instance_for_rdbms(selected_rdbms());
     Sql_specifics::Ref sql_specifics= sql_facade->sqlSpecifics();
     _non_std_sql_delimiter= sql_specifics->non_std_sql_delimiter();
   }
@@ -388,7 +398,9 @@ db_CatalogRef Db_plugin::db_catalog()
   catalog->oldName(catalog->name());
 
   SqlFacade::Ref sql_parser= SqlFacade::instance_for_rdbms(pm->rdbms());
-  sql_parser->parseSqlScriptString(catalog, sql_input_script);
+  grt::DictRef parse_options(_grtm->get_grt());
+  parse_options.set("case_sensitive_identifiers", _db_options.get("CaseSensitive", grt::IntegerRef(1)));
+  sql_parser->parseSqlScriptStringEx(catalog, sql_input_script, parse_options);
 
   return catalog;
 }
@@ -396,7 +408,7 @@ db_CatalogRef Db_plugin::db_catalog()
 
 void Db_plugin::set_task_proc()
 {
-  _task_proc_cb= sigc::mem_fun(this, &Db_plugin::apply_script_to_db);
+  _task_proc_cb= boost::bind(&Db_plugin::apply_script_to_db, this, _1);
 }
 
 
@@ -408,14 +420,14 @@ grt::StringRef Db_plugin::apply_script_to_db(grt::GRT *grt)
   grt->send_info(_("Executing SQL script in server"));
   
   std::list<std::string> statements;
-  SqlFacade::Ref sql_splitter= SqlFacade::instance_for_rdbms(_db_conn.get_rdbms());
+  SqlFacade::Ref sql_splitter= SqlFacade::instance_for_rdbms(selected_rdbms());
   sql_splitter->splitSqlScript(_sql_script, statements);
 
   sql::SqlBatchExec sql_batch_exec;
 
-  sql_batch_exec.error_cb(sigc::mem_fun(this, &Db_plugin::process_sql_script_error));
-  sql_batch_exec.batch_exec_progress_cb(sigc::mem_fun(this, &Db_plugin::process_sql_script_progress));
-  sql_batch_exec.batch_exec_stat_cb(sigc::mem_fun(this, &Db_plugin::process_sql_script_statistics));
+  sql_batch_exec.error_cb(boost::bind(&Db_plugin::process_sql_script_error, this, _1, _2, _3));
+  sql_batch_exec.batch_exec_progress_cb(boost::bind(&Db_plugin::process_sql_script_progress, this, _1));
+  sql_batch_exec.batch_exec_stat_cb(boost::bind(&Db_plugin::process_sql_script_statistics, this, _1, _2));
 
   sql_batch_exec(stmt.get(), statements);
 
@@ -426,7 +438,7 @@ grt::StringRef Db_plugin::apply_script_to_db(grt::GRT *grt)
 int Db_plugin::process_sql_script_error(long long err_no, const std::string &err_msg, const std::string &statement)
 {
   std::ostringstream oss;
-  oss << _("Error ") << err_no << _(": ") << err_msg << std::endl << statement << std::endl;
+  oss << _("Error ") << err_no << ": " << err_msg << std::endl << statement << std::endl;
   _grtm->get_grt()->send_error(oss.str());
   return 0;
 }
@@ -434,7 +446,7 @@ int Db_plugin::process_sql_script_error(long long err_no, const std::string &err
 
 int Db_plugin::process_sql_script_progress(float progress_state)
 {
-  _grtm->get_grt()->send_progress(progress_state, _(""));
+  _grtm->get_grt()->send_progress(progress_state, "");
   return 0;
 }
 
@@ -444,7 +456,7 @@ int Db_plugin::process_sql_script_statistics(long success_count, long err_count)
   std::ostringstream oss;
   oss << _("SQL script execution finished: statements: ") << success_count << _(" succeeded, ")
     << err_count << _(" failed") << std::endl;
-  _grtm->get_grt()->send_progress(1.f, _(""));
+  _grtm->get_grt()->send_progress(1.f, "");
   _grtm->get_grt()->send_info(oss.str());
   return 0;
 }
