@@ -1,4 +1,23 @@
-#include "stdafx.h"
+/* 
+ * Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
+#include "tut_stdafx.h"
 
 #include "testgrt.h"
 #include "grt_test_utility.h"
@@ -26,12 +45,14 @@
 #include "grtdb/diff_dbobjectmatch.h"
 
 #include "myx_statement_parser.h"
-#include "test_import_helpers.h"
+//#include "test_import_helpers.h"
 #include "db_mysql_sql_script_sync.h"
 #include "db_mysql_sql_export.h"
 #include "module_db_mysql.h"
 #include "wb_helpers.h"
 
+#include "module_db_mysql_shared_code.h"
+
 /*
   We override get_model_catalog() method of the original plugin,
   to make testing setup easier, in real life the model catalog
@@ -65,15 +86,25 @@ public:
   void set_options_as_dict(grt::DictRef options_dict) { options= options_dict; }
 };
 
+struct all_objects_mwb
+{
+    db_SchemaRef schema;
+    db_TableRef t1;
+    db_TableRef t2;
+    db_ViewRef view;
+    db_RoutineRef routine;
+    db_ForeignKeyRef FK;
+    db_TriggerRef trigger;
+};
+
 BEGIN_TEST_DATA_CLASS(db_mysql_plugin_test)
 protected:
   WBTester tester;
   std::auto_ptr<DbMySQLScriptSync> sync_plugin;
   std::auto_ptr<DbMySQLSQLExport> fwdeng_plugin;
   SqlFacade::Ref sql_parser;
-  std::auto_ptr<sql::Connection> connection;
+  sql::ConnectionWrapper connection;
   grt::DbObjectMatchAlterOmf omf;
-  omf.dontdiff_mask = 3;
 
   db_mysql_CatalogRef create_catalog_from_script(const std::string& sql);
   db_mysql_CatalogRef create_catalog_from_script(const std::string& sql, grt::GRT *grt);
@@ -90,8 +121,23 @@ protected:
 
   std::string run_fwdeng_plugin_generate_script(db_mysql_CatalogRef cat, 
                                                 DbMySQLSQLExportTest *plugin);
-  DiffChange *compare_catalog_to_server_schema(db_mysql_CatalogRef org_cat, 
+  boost::shared_ptr<DiffChange> compare_catalog_to_server_schema(db_mysql_CatalogRef org_cat, 
                                                const std::string& schema_name);
+  void apply_sql_to_model(const std::string& sql);
+  all_objects_mwb get_model_objects();
+  TEST_DATA_CONSTRUCTOR(db_mysql_plugin_test)
+  {
+  // init datatypes
+  populate_grt(tester.grt, NULL, tester);
+
+  omf.dontdiff_mask = 3;
+
+  // init database connection
+  connection= tester.create_connection_for_import();
+
+  sql_parser= SqlFacade::instance_for_rdbms_name(tester.grt, "Mysql");
+  ensure("failed to get sqlparser module", (NULL != sql_parser));
+  }
 
 END_TEST_DATA_CLASS
 
@@ -104,18 +150,6 @@ static int process_sql_statement_callback(const MyxStatementParser *splitter, co
   return 1;
 }
 
-TEST_FUNCTION(1)
-{
-  // init datatypes
-  populate_grt(tester.grt, NULL);
-
-  // init database connection
-  connection= create_connection_for_import(tester.grt);
-
-  sql_parser= SqlFacade::instance_for_rdbms_name(tester.grt, "Mysql");
-  ensure("failed to get sqlparser module", (NULL != sql_parser));
-}
-
 db_mysql_CatalogRef tut::Test_object_base<db_mysql_plugin_test>::create_catalog_from_script(
   const std::string& sql)
 {
@@ -125,7 +159,7 @@ db_mysql_CatalogRef tut::Test_object_base<db_mysql_plugin_test>::create_catalog_
 db_mysql_CatalogRef tut::Test_object_base<db_mysql_plugin_test>::create_catalog_from_script(
   const std::string& sql, grt::GRT *grt)
 {
-  db_mysql_CatalogRef cat= create_empty_catalog_for_import(grt);
+  db_mysql_CatalogRef cat= create_empty_catalog_for_import(tester.grt);
   sql_parser->parseSqlScriptString(cat, sql);
   return cat;
 }
@@ -137,7 +171,7 @@ std::string tut::Test_object_base<db_mysql_plugin_test>::run_sync_plugin_generat
 {
   sync_plugin.reset(new DbMySQLScriptSyncTest(tester.wb->get_grt_manager()));
   static_cast<DbMySQLScriptSyncTest *>(sync_plugin.get())->set_model_catalog(mod_cat);
-  sync_plugin->init_diff_tree(org_cat, mod_cat);
+  sync_plugin->init_diff_tree(std::vector<std::string>(), org_cat, mod_cat, grt::StringListRef());
   return sync_plugin->generate_diff_tree_script();
 }
 
@@ -148,7 +182,7 @@ void tut::Test_object_base<db_mysql_plugin_test>::run_sync_plugin_apply_to_model
 {
   sync_plugin.reset(new DbMySQLScriptSyncTest(tester.wb->get_grt_manager()));
   static_cast<DbMySQLScriptSyncTest *>(sync_plugin.get())->set_model_catalog(mod_cat);
-  sync_plugin->init_diff_tree(org_cat, ValueRef());
+  sync_plugin->init_diff_tree(std::vector<std::string>(), org_cat, ValueRef(), grt::StringListRef());
   sync_plugin->apply_changes_to_model();
 }
 
@@ -160,18 +194,65 @@ std::string tut::Test_object_base<db_mysql_plugin_test>::run_fwdeng_plugin_gener
   return fwdeng_plugin->export_sql_script();
 }
 
-DiffChange *tut::Test_object_base<db_mysql_plugin_test>::compare_catalog_to_server_schema(db_mysql_CatalogRef org_cat, 
+boost::shared_ptr<DiffChange> tut::Test_object_base<db_mysql_plugin_test>::compare_catalog_to_server_schema(db_mysql_CatalogRef org_cat, 
                                                                                           const std::string& schema_name)
 {
   sync_plugin.reset(new DbMySQLScriptSyncTest(tester.wb->get_grt_manager()));
   std::list<std::string> schemata;
   schemata.push_back("db_mysql_plugin_test");
-  db_mysql_CatalogRef cat= db_rev_eng_schema(tester.wb->get_grt_manager(), schemata);
-  bec::CatalogHelper::apply_defaults(cat);
-  bec::CatalogHelper::apply_defaults(org_cat);
-  return diff_make(cat, org_cat, &omf,grt::NormalizedComparer<grt::GRT*>(tester.grt));
+  db_mysql_CatalogRef cat= tester.db_rev_eng_schema(schemata);
+  if((cat->schemata().get(0).is_valid()) && (cat->schemata().get(0)->name() == "mydb"))
+      cat->schemata().remove(0);
+  org_cat->oldName("");
+
+  grt::ValueRef default_engine = tester.wb->get_grt_manager()->get_app_option("db.mysql.Table:tableEngine");
+  std::string default_engine_name;
+  if(grt::StringRef::can_wrap(default_engine))
+    default_engine_name = grt::StringRef::cast_from(default_engine);
+
+  bec::CatalogHelper::apply_defaults(cat, default_engine_name);
+  bec::CatalogHelper::apply_defaults(org_cat, default_engine_name);
+
+  grt::NormalizedComparer comparer(tester.grt,grt::DictRef(tester.grt));
+  comparer.init_omf(&omf);
+
+  return diff_make(cat, org_cat, &omf);
 }
 
+void tut::Test_object_base<db_mysql_plugin_test>::apply_sql_to_model(const std::string& sql)
+{
+
+  db_mysql_CatalogRef org_cat= create_catalog_from_script(sql, tester.grt);
+
+  std::vector<std::string> schemata;
+  schemata.push_back("mydb");
+
+  db_mysql_CatalogRef mod_cat= db_mysql_CatalogRef::cast_from(tester.get_catalog());
+
+  DbMySQLSQLExportTest *plugin= new DbMySQLSQLExportTest(
+    tester.wb->get_grt_manager(), mod_cat);
+  
+  grt::DictRef options(tester.grt);
+  options.set("UseFilteredLists", grt::IntegerRef(0));
+  plugin->set_options_as_dict(options);
+
+  std::string value;
+
+  DbMySQLScriptSyncTest p(tester.wb->get_grt_manager());
+  p.set_model_catalog(mod_cat);
+  boost::shared_ptr<DiffTreeBE> tree= p.init_diff_tree(std::vector<std::string>(), org_cat, ValueRef(), grt::StringListRef());
+  
+  // apply everything back to model
+  tree->set_apply_direction(tree->get_root(), DiffNode::ApplyToModel, true);
+  bec::NodeId mydb_node= tree->get_child(NodeId(), 0);
+  bec::NodeId table1_node= tree->get_child(mydb_node, 0);
+  tree->get_field(table1_node, DiffTreeBE::ModelObjectName, value);
+
+  p.apply_changes_to_model();
+
+}
+
+
 TEST_FUNCTION(2)
 {
   // Bug #32327
@@ -197,10 +278,10 @@ TEST_FUNCTION(2)
   std::string script= run_sync_plugin_generate_script(schemata, org_cat, mod_cat);
 
   std::auto_ptr<sql::Statement> stmt(connection->createStatement());
-  execute_script(stmt.get(), sql1);
-  execute_script(stmt.get(), script);
+  execute_script(stmt.get(), sql1, tester.wb->get_grt_manager());
+  execute_script(stmt.get(), script, tester.wb->get_grt_manager());
 
-  DiffChange *empty_change= compare_catalog_to_server_schema(org_cat, "db_mysql_plugin_test");
+  boost::shared_ptr<DiffChange> empty_change= compare_catalog_to_server_schema(org_cat, "db_mysql_plugin_test");
 
   if (empty_change)
     empty_change->dump_log(0);
@@ -230,10 +311,10 @@ TEST_FUNCTION(3)
   std::string script= run_sync_plugin_generate_script(schemata, org_cat, mod_cat);
 
   std::auto_ptr<sql::Statement> stmt(connection->createStatement());
-  execute_script(stmt.get(), sql1);
-  execute_script(stmt.get(), script);
+  execute_script(stmt.get(), sql1, tester.wb->get_grt_manager());
+  execute_script(stmt.get(), script, tester.wb->get_grt_manager());
 
-  DiffChange *empty_change= compare_catalog_to_server_schema(mod_cat, "db_mysql_plugin_test");
+  boost::shared_ptr<DiffChange> empty_change = compare_catalog_to_server_schema(mod_cat, "db_mysql_plugin_test");
 
   if(empty_change)
     empty_change->dump_log(0);
@@ -277,8 +358,8 @@ TEST_FUNCTION(4)
   std::string script= run_sync_plugin_generate_script(schemata, org_cat, mod_cat);
 
   std::auto_ptr<sql::Statement> stmt(connection->createStatement());
-  execute_script(stmt.get(), sql1);
-  execute_script(stmt.get(), script);
+  execute_script(stmt.get(), sql1, tester.wb->get_grt_manager());
+  execute_script(stmt.get(), script, tester.wb->get_grt_manager());
 
   // TODO: this check doesnt work because of a rev-eng problem http://bugs.mysql.com/bug.php?id=32491
 
@@ -315,11 +396,11 @@ TEST_FUNCTION(5)
   std::string script= run_sync_plugin_generate_script(schemata, org_cat, mod_cat);
 
   std::auto_ptr<sql::Statement> stmt(connection->createStatement());
-  execute_script(stmt.get(), sql1);
+  execute_script(stmt.get(), sql1, tester.wb->get_grt_manager());
 
   try
   {
-    execute_script(stmt.get(), script);
+    execute_script(stmt.get(), script, tester.wb->get_grt_manager());
   }
   catch(sql::SQLException &)
   {
@@ -364,8 +445,11 @@ TEST_FUNCTION(6)
     db_mysql_CatalogRef(mod_cat.get_grt()), mod_cat);
 
   std::auto_ptr<sql::Statement> stmt(connection->createStatement());
-  execute_script(stmt.get(), sql2);
-  execute_script(stmt.get(), script);
+  execute_script(stmt.get(), sql2, tester.wb->get_grt_manager());
+  //Self referencing keys are no longer supported, but as testcase itself checks for
+  //crash it is better to this test case since there shouldn't be any crashes even
+  //in case of invalid sqll being produced
+//  execute_script(stmt.get(), script, tester.wb->get_grt_manager());
 }
 
 TEST_FUNCTION(7)
@@ -388,17 +472,16 @@ TEST_FUNCTION(7)
     "INSERT INTO t1(col_char) VALUES ('a'), ('b'), ('c');";
 
   std::auto_ptr<sql::Statement> stmt(connection->createStatement());
-  execute_script(stmt.get(), sql1);
+  execute_script(stmt.get(), sql1, tester.wb->get_grt_manager());
 
   std::list<std::string> schemata_list;
   schemata_list.push_back("db_mysql_plugin_test");
-  db_mysql_CatalogRef mod_cat= db_rev_eng_schema(tester.wb->get_grt_manager(), schemata_list);
-  db_mysql_CatalogRef org_cat= db_rev_eng_schema(tester.wb->get_grt_manager(), schemata_list);
+  db_mysql_CatalogRef mod_cat= tester.db_rev_eng_schema(schemata_list);
+  db_mysql_CatalogRef org_cat= tester.db_rev_eng_schema(schemata_list);
 
-  ensure("bug_32367 - invalid test input", 
-    (mod_cat->schemata().get(0)->tables().count() == 3) &&
-    (mod_cat->schemata().get(0)->tables().get(0)->triggers().count() == 1) &&
-    (mod_cat->schemata().get(0)->routines().count() == 1));
+  ensure("bug_32367 - invalid test input wrong table count",  (mod_cat->schemata().get(0)->tables().count() == 3));
+  ensure("bug_32367 - invalid test input wrong trigger count",  (mod_cat->schemata().get(0)->tables().get(0)->triggers().count() == 1));
+  ensure("bug_32367 - invalid test input wrong routines count",  (mod_cat->schemata().get(0)->routines().count() == 1));
   
   // delete a table, a routine and a trigger
   mod_cat->schemata().get(0)->tables().remove(2);
@@ -409,9 +492,9 @@ TEST_FUNCTION(7)
   schemata.push_back("db_mysql_plugin_test");
 
   std::string script= run_sync_plugin_generate_script(schemata, org_cat, mod_cat);
-  execute_script(stmt.get(), script);
+  execute_script(stmt.get(), script, tester.wb->get_grt_manager());
 
-  db_mysql_CatalogRef new_cat= db_rev_eng_schema(tester.wb->get_grt_manager(), schemata_list);
+  db_mysql_CatalogRef new_cat= tester.db_rev_eng_schema(schemata_list);
 
   ensure_equals("Table count mismatch", new_cat->schemata().get(0)->tables().count(), 2);
   ensure_equals("Trigger count mismatch", new_cat->schemata().get(0)->tables().get(0)->triggers().count(), 0);
@@ -442,18 +525,18 @@ TEST_FUNCTION(8)
   // part1 - check that unmodified procedure is not updated
 
   std::auto_ptr<sql::Statement> stmt(connection->createStatement());
-  execute_script(stmt.get(), sql1);
+  execute_script(stmt.get(), sql1, tester.wb->get_grt_manager());
 
   std::list<std::string> schemata_list;
   schemata_list.push_back("db_mysql_plugin_test");
-  db_mysql_CatalogRef mod_cat= db_rev_eng_schema(tester.wb->get_grt_manager(), schemata_list);
-  db_mysql_CatalogRef org_cat= db_rev_eng_schema(tester.wb->get_grt_manager(), schemata_list);
+  db_mysql_CatalogRef mod_cat= tester.db_rev_eng_schema(schemata_list);
+  db_mysql_CatalogRef org_cat= tester.db_rev_eng_schema(schemata_list);
 
   ensure("bug_32371 - invalid test input", 
     (mod_cat->schemata().get(0)->tables().count() == 1) &&
     (mod_cat->schemata().get(0)->routines().count() == 2));
 
-  DiffChange *empty_change= compare_catalog_to_server_schema(mod_cat, "db_mysql_plugin_test");
+  boost::shared_ptr<DiffChange> empty_change= compare_catalog_to_server_schema(mod_cat, "db_mysql_plugin_test");
 
   if(empty_change)
     empty_change->dump_log(0);
@@ -483,7 +566,7 @@ TEST_FUNCTION(8)
   schemata.push_back("db_mysql_plugin_test");
 
   std::string script= run_sync_plugin_generate_script(schemata, org_cat, mod_cat);
-  execute_script(stmt.get(), script);
+  execute_script(stmt.get(), script, tester.wb->get_grt_manager());
 }
 
 TEST_FUNCTION(9)
@@ -510,10 +593,10 @@ TEST_FUNCTION(9)
   std::string script= run_sync_plugin_generate_script(schemata, org_cat, mod_cat);
 
   std::auto_ptr<sql::Statement> stmt(connection->createStatement());
-  execute_script(stmt.get(), sql1);
-  execute_script(stmt.get(), script);
+  execute_script(stmt.get(), sql1, tester.wb->get_grt_manager());
+  execute_script(stmt.get(), script, tester.wb->get_grt_manager());
 
-  DiffChange *empty_change= compare_catalog_to_server_schema(mod_cat, "db_mysql_plugin_test");
+  boost::shared_ptr<DiffChange> empty_change= compare_catalog_to_server_schema(mod_cat, "db_mysql_plugin_test");
 
   if (empty_change)
     empty_change->dump_log(0);
@@ -548,10 +631,10 @@ TEST_FUNCTION(10)
   std::string script= run_sync_plugin_generate_script(schemata, org_cat, mod_cat);
 
   std::auto_ptr<sql::Statement> stmt(connection->createStatement());
-  execute_script(stmt.get(), sql1);
-  execute_script(stmt.get(), script);
+  execute_script(stmt.get(), sql1, tester.wb->get_grt_manager());
+  execute_script(stmt.get(), script, tester.wb->get_grt_manager());
 
-  DiffChange *empty_change= compare_catalog_to_server_schema(mod_cat, "db_mysql_plugin_test");
+  boost::shared_ptr<DiffChange> empty_change= compare_catalog_to_server_schema(mod_cat, "db_mysql_plugin_test");
 
   if(empty_change)
     empty_change->dump_log(0);
@@ -580,15 +663,15 @@ TEST_FUNCTION(11)
   
   grt::DictRef options(tester.grt);
   grt::StringListRef views(tester.grt);
-  views.insert(get_old_object_name_for_key(mod_cat->schemata().get(0)->views().get(0)));
-  views.insert(get_old_object_name_for_key(mod_cat->schemata().get(0)->views().get(1)));
+  views.insert(get_old_object_name_for_key(mod_cat->schemata().get(0)->views().get(0), false), false);
+  views.insert(get_old_object_name_for_key(mod_cat->schemata().get(0)->views().get(1), false), false);
   options.set("ViewFilterList", views);
   plugin->set_options_as_dict(options);
   
   std::string script= run_fwdeng_plugin_generate_script(mod_cat, plugin);
 
   std::auto_ptr<sql::Statement> stmt(connection->createStatement());
-  execute_script(stmt.get(), script);
+  execute_script(stmt.get(), script, tester.wb->get_grt_manager());
 
   std::vector<std::string> schemata;
   schemata.push_back("db_mysql_plugin_test");
@@ -598,8 +681,8 @@ TEST_FUNCTION(11)
     db_mysql_CatalogRef(mod_cat.get_grt()), mod_cat));
 
   std::auto_ptr<sql::Statement> stmt2(connection->createStatement());
-  execute_script(stmt2.get(), "DROP DATABASE IF EXISTS `db_mysql_plugin_test`");
-  execute_script(stmt2.get(), script);
+  execute_script(stmt2.get(), "DROP DATABASE IF EXISTS `db_mysql_plugin_test`", tester.wb->get_grt_manager());
+  execute_script(stmt2.get(), script, tester.wb->get_grt_manager());
 }
 
 // bug #37634
@@ -617,73 +700,99 @@ TEST_FUNCTION(12)
     "ENGINE = InnoDB;"
     ;
 
-  WBTester wb;
-  wb.wb->open_document("data/workbench/diff_table_replace_test.mwb");
+  tester.wb->open_document("data/workbench/diff_table_replace_test.mwb");
 
-  db_mgmt_ManagementRef mgmt(db_mgmt_ManagementRef::cast_from(wb.grt->get("/wb/rdbmsMgmt")));
+  db_mgmt_ManagementRef mgmt(db_mgmt_ManagementRef::cast_from(tester.grt->get("/wb/rdbmsMgmt")));
 
   ListRef<db_DatatypeGroup> grouplist= 
-    ListRef<db_DatatypeGroup>::cast_from(wb.grt->unserialize("../../bin/debug/data/db_datatype_groups.xml"));
+    ListRef<db_DatatypeGroup>::cast_from(tester.grt->unserialize("../../bin/debug/data/db_datatype_groups.xml"));
   grt::replace_contents(mgmt->datatypeGroups(), grouplist);
 
   db_mgmt_RdbmsRef rdbms= 
-    db_mgmt_RdbmsRef::cast_from(wb.grt->unserialize("../../bin/debug/modules/data/mysql_rdbms_info.xml"));
+    db_mgmt_RdbmsRef::cast_from(tester.grt->unserialize("../../bin/debug/modules/data/mysql_rdbms_info.xml"));
   ensure("db_mgmt_Rdbms initialization", rdbms.is_valid());
-  wb.grt->set("/rdbms", rdbms);
+  tester.grt->set("/rdbms", rdbms);
 
   mgmt->rdbms().insert(rdbms);
   rdbms->owner(mgmt);
 
-  db_TableRef t1= wb.get_catalog()->schemata().get(0)->tables().get(0);
+  db_TableRef t1= tester.get_catalog()->schemata().get(0)->tables().get(0);
 
   ensure("before update table is referenced from figure 0", 
-    wb.grt->get("/wb/doc/physicalModels/0/diagrams/0/figures/0/table")
+    tester.grt->get("/wb/doc/physicalModels/0/diagrams/0/figures/0/table")
     == t1);
 
   ensure("before update table is referenced from figure 1", 
-    wb.grt->get("/wb/doc/physicalModels/0/diagrams/1/figures/0/table")
+    tester.grt->get("/wb/doc/physicalModels/0/diagrams/1/figures/0/table")
     == t1);
 
-  db_mysql_CatalogRef org_cat= create_catalog_from_script(sql1, wb.grt);
+  db_mysql_CatalogRef org_cat= create_catalog_from_script(sql1, tester.grt);
 
   std::vector<std::string> schemata;
   schemata.push_back("mydb");
 
-  db_mysql_CatalogRef mod_cat= db_mysql_CatalogRef::cast_from(wb.get_catalog());
+  db_mysql_CatalogRef mod_cat= db_mysql_CatalogRef::cast_from(tester.get_catalog());
 
   DbMySQLSQLExportTest *plugin= new DbMySQLSQLExportTest(
-    wb.wb->get_grt_manager(), mod_cat);
+    tester.wb->get_grt_manager(), mod_cat);
   
-  grt::DictRef options(wb.grt);
+  grt::DictRef options(tester.grt);
   options.set("UseFilteredLists", grt::IntegerRef(0));
   plugin->set_options_as_dict(options);
 
   std::string value;
 
-  DbMySQLScriptSyncTest p(wb.wb->get_grt_manager());
+  DbMySQLScriptSyncTest p(tester.wb->get_grt_manager());
   p.set_model_catalog(mod_cat);
-  DiffTreeBE *tree= p.init_diff_tree(org_cat, ValueRef());
+  boost::shared_ptr<DiffTreeBE> tree= p.init_diff_tree(std::vector<std::string>(), org_cat, ValueRef(), grt::StringListRef());
   
   // change apply direction for table table1
   bec::NodeId mydb_node= tree->get_child(NodeId(), 0);
   bec::NodeId table1_node= tree->get_child(mydb_node, 0);
   tree->get_field(table1_node, DiffTreeBE::ModelObjectName, value);
+
   p.set_next_apply_direction(table1_node);
   p.set_next_apply_direction(table1_node);
 
   p.apply_changes_to_model();
 
-  db_TableRef t2= wb.get_catalog()->schemata().get(0)->tables().get(0);
-
-  ensure("table was replaced", t1 != t2);
+  db_TableRef t2= tester.get_catalog()->schemata().get(0)->tables().get(0);
 
   ensure("before update table is referenced from figure 0", 
-    wb.grt->get("/wb/doc/physicalModels/0/diagrams/0/figures/0/table")
+    tester.grt->get("/wb/doc/physicalModels/0/diagrams/0/figures/0/table")
     == t2);
 
   ensure("before update table is referenced from figure 1", 
-    wb.grt->get("/wb/doc/physicalModels/0/diagrams/1/figures/0/table")
+    tester.grt->get("/wb/doc/physicalModels/0/diagrams/1/figures/0/table")
     == t2);  
 }
 
+TEST_FUNCTION(13)
+{
+  static const char *sql1=
+    "CREATE SCHEMA IF NOT EXISTS `mydb` DEFAULT CHARACTER SET latin1 COLLATE latin1_swedish_ci;\n"
+    "USE `mydb`;\n"
+    "CREATE  TABLE IF NOT EXISTS `mydb`.`table1` (\n"
+    "  `idtable1` TINYINT NOT NULL ,\n"
+    "  PRIMARY KEY (`idtable1`) )\n"
+    "ENGINE = InnoDB;"
+    ;
+  tester.wb->open_document("data/workbench/diff_table_replace_test.mwb");
+  apply_sql_to_model(sql1);
+
+
+  db_TableRef t2= tester.get_catalog()->schemata().get(0)->tables().get(0);
+  db_ColumnRef col = t2->columns().get(0);
+  db_SimpleDatatypeRef dtype = col->simpleType();
+  ensure("Column type not changed", dtype->name() == "TINYINT");
+
+}
+
+TEST_FUNCTION(14)
+{
+  static const char *sql1= "CREATE SCHEMA IF NOT EXISTS `mydb` DEFAULT CHARACTER SET latin1 COLLATE latin1_swedish_ci;";
+  tester.wb->open_document("data/workbench/diff_table_replace_test.mwb");
+  apply_sql_to_model(sql1);
+  ensure ("drop table in model",tester.get_catalog()->schemata().get(0)->tables().count() == 0);
+}
 END_TESTS
