@@ -1,5 +1,5 @@
 /* 
-* Copyright (c) 2009, 2010, 2011 Oracle and/or its affiliates. All rights reserved.
+* Copyright (c) 2009, 2012 Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
@@ -36,7 +36,7 @@ using namespace mforms;
 using namespace base;
 
 #include "grtui/connection_page.h"
-#include "schema_selection_page.h"
+#include "schema_matching_page.h"
 #include "fetch_schema_names_page.h"
 #include "fetch_schema_contents_page.h"
 #include "synchronize_differences_page.h"
@@ -225,10 +225,10 @@ public:
 };
 
 
-class ViewResultPage : public ViewTextPage
+class AlterViewResultPage : public ViewTextPage
 {
 public:
-  ViewResultPage(WizardForm *form)
+  AlterViewResultPage(WizardForm *form)
     : ViewTextPage(form, "viewdiff", (ViewTextPage::Buttons)(ViewTextPage::SaveButton|ViewTextPage::CopyButton), "SQL Files (*.sql)|*.sql")
   {
     set_short_title(_("Detected Changes"));
@@ -262,7 +262,12 @@ public:
     return true;  
   }
 
-  virtual bool allow_cancel()const  { return false; }
+  virtual std::string next_button_caption()
+  {
+    return execute_caption();
+  }
+
+  virtual bool allow_cancel() const  { return false; }
   virtual bool next_closes_wizard() { return values().get_int("result") != DataSourceSelector::ServerSource; }
 
 protected:  
@@ -323,9 +328,8 @@ public:
 
   bool do_connect()
   {
-    static const grt::ValueRef vref;
-    boost::function<grt::ValueRef (bool)> task = boost::lambda::var(vref);
-    execute_grt_task(boost::bind(task,boost::bind(&DbConnection::test_connection,_dbplugin->db_conn())), false);
+    execute_grt_task(boost::bind(boost::function<grt::ValueRef (bool)> (boost::lambda::constant(grt::ValueRef())),
+        boost::bind(&DbConnection::test_connection,_dbplugin->db_conn())), false);
     return true;
   }  
 
@@ -412,8 +416,8 @@ public:
       fetch_names_page->set_load_schemata_slot(boost::bind(&WbPluginDiffAlter::load_schemata, this, dbplugin[i]));
       fetch_names_page->set_title(std::string(title_prefix[i]).append(fetch_names_page->get_title()));
 
-      SchemaSelectionPage *schema_page;
-      add_page(mforms::manage(schema_page= new SchemaSelectionPage(this, strfmt("pickSchemata%i", i).c_str())));
+      SchemaMatchingPage *schema_page;
+      add_page(mforms::manage(schema_page= new SchemaMatchingPage(this, strfmt("pickSchemata%i", i).c_str(), "Model Schema", "Script Schema")));
       schema_page->set_db_plugin(dbplugin[i]);
       schema_page->set_title(std::string(title_prefix[i]).append(schema_page->get_title()));
 
@@ -431,8 +435,8 @@ public:
     _diffs_page->set_title(_("Differences Found"));
     add_page(mforms::manage(_diffs_page));
 
-    ViewResultPage *page;
-    add_page(mforms::manage(page= new ViewResultPage(this)));
+    AlterViewResultPage *page;
+    add_page(mforms::manage(page= new AlterViewResultPage(this)));
     page->set_generate_text_slot(boost::bind(&WbPluginDiffAlter::generate_alter, this));
     add_page(mforms::manage(_apply_page = new AlterApplyProgressPage(this)));
     _apply_page->set_db_plugin(dbplugin[1]);
@@ -505,6 +509,8 @@ public:
         right_catalog = _right_db.db_catalog();
       else if (_source_page->get_right_source() == DataSourceSelector::FileSource)
         right_catalog = _import_destination_file->get_catalog();
+      else if (_source_page->get_right_source() == DataSourceSelector::ModelSource)
+        right_catalog = _be.get_model_catalog();
 
       _diffs_page->set_src(right_catalog);
       _diffs_page->set_dst(left_catalog);
@@ -529,6 +535,7 @@ protected:
   {
     std::vector<std::string> names;
     db->load_schemata(names);
+    _be.set_db_options(db->load_db_options());
     return names;
   }
 };
@@ -539,3 +546,7 @@ WizardPlugin *createWbPluginDiffAlter(grt::Module *module, db_CatalogRef catalog
   return new WbPluginDiffAlter(module);
 }
 
+void deleteWbPluginDiffAlter(WizardPlugin *plugin)
+{
+    delete plugin;
+}
