@@ -1,6 +1,30 @@
+/* 
+* Copyright (c) 2009, 2012 Oracle and/or its affiliates. All rights reserved.
+*
+* This program is free software; you can redistribute it and/or
+* modify it under the terms of the GNU General Public License as
+* published by the Free Software Foundation; version 2 of the
+* License.
+* 
+* This program is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+* GNU General Public License for more details.
+* 
+* You should have received a copy of the GNU General Public License
+* along with this program; if not, write to the Free Software
+* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+* 02110-1301  USA
+*/
+
+#include "db_mysql_sql_script_sync.h"
 
 #include "mforms/grttreeview.h"
-#include "db_mysql_sql_script_sync.h"
+#include "mforms/splitter.h"
+#include "mforms/label.h"
+#include "mforms/textbox.h"
+#include "mforms/box.h"
+#include "mforms/button.h"
 
 class SynchronizeDifferencesPage : public WizardPage
 {
@@ -70,12 +94,13 @@ public:
     std::string sql;
     if (_tree.get_selected_node(node))
     {
-    ValueRef obj = _be->get_model_object(node);
-    if (GrtNamedObjectRef::can_wrap(obj))
-      sql.append(_be->get_sql_for_object(GrtNamedObjectRef::cast_from(obj)));
-    obj = _be->get_db_object(node);
-    if (GrtNamedObjectRef::can_wrap(obj))
-      sql.append(_be->get_sql_for_object(GrtNamedObjectRef::cast_from(obj)));
+        ValueRef obj = _be->get_db_object(node);
+        if (GrtNamedObjectRef::can_wrap(obj))
+            sql.append(_be->get_sql_for_object(GrtNamedObjectRef::cast_from(obj)));
+
+        obj = _be->get_model_object(node);
+        if (GrtNamedObjectRef::can_wrap(obj))
+            sql.append(_be->get_sql_for_object(GrtNamedObjectRef::cast_from(obj)));
     }
     _diff_sql_text.set_value(sql);
   }
@@ -102,10 +127,10 @@ public:
     grt::StringListRef schemas_to_skip(grt::StringListRef::cast_from(values().get("unSelectedSchemata")));
     if (get_source_catalog)
       _src = get_source_catalog();
-    bec::TreeModel *diff_tree= _be->init_diff_tree(std::vector<std::string>(),_src,_dst,schemas_to_skip);
+    _diff_tree= _be->init_diff_tree(std::vector<std::string>(),_src,_dst,schemas_to_skip);
 
-    _tree.set_model(diff_tree);
-    for (int i= 0, c= diff_tree->count_children(bec::NodeId()); i < c; i++)
+    _tree.set_model(_diff_tree.get());
+    for (int i= 0, c= _diff_tree->count_children(bec::NodeId()); i < c; i++)
          _tree.set_expanded(bec::NodeId(i), true);
     
     return true;
@@ -168,9 +193,10 @@ public:
   
 protected:
   GRTTreeView _tree;
+  boost::shared_ptr<DiffTreeBE> _diff_tree;
   Label _heading;
-  mforms::TextBox _diff_sql_text;
-  mforms::Splitter _splitter;
+  ::mforms::TextBox _diff_sql_text;
+  ::mforms::Splitter _splitter;
   
   Box _bottom_box;
   Button _select_all;
