@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -15,11 +15,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 # 02110-1301  USA
 
-import grt
-from grt import modules
-from db_utils import QueryError, escape_sql_string
+from workbench.db_utils import QueryError, escape_sql_string
 from wb_common import PermissionDeniedError
-from grt import modules;
 
 import hashlib
 def make_scrambled_password(pwd):
@@ -56,6 +53,8 @@ FLUSH_PRIVILEGES = "FLUSH PRIVILEGES"
 REVOKE_ALL = "REVOKE ALL PRIVILEGES, GRANT OPTION FROM '%(user)s'@'%(host)s'"
 REMOVE_USER = "DROP USER '%(user)s'@'%(host)s'"
 
+# Map of user table's column name to privilege and its description.
+# It has the form { 'TableColumnPrivName': ('PrettyName', 'ADescriptionOfThePrivilege'), ... }
 PrivilegeInfo = {
 "Select_priv": ("SELECT", "The SELECT privilege enables you to select rows from tables in a database.\nSELECT statements require the SELECT privilege only if they actually retrieve rows from a table. Some SELECT statements do not access tables and can be executed without permission for any database"),
 "Insert_priv": ("INSERT", "The INSERT privilege enables rows to be inserted into tables in a database.\nINSERT is also required for the ANALYZE TABLE, OPTIMIZE TABLE, and REPAIR TABLE table-maintenance statements."),
@@ -67,13 +66,12 @@ Beginning with MySQL 5.1.10, the DROP privilege is also required in order to use
 Beginning with MySQL 5.1.16, the DROP privilege is required for TRUNCATE TABLE (before that, TRUNCATE TABLE requires the DELETE privilege).
 If you grant the DROP privilege for the mysql database to a user, that user can drop the database in which the MySQL access privileges are stored."""),
 "Reload_priv": ("RELOAD", "The RELOAD privilege enables use of the FLUSH statement.\nIt also enables mysqladmin commands that are equivalent to FLUSH operations:\nflush-hosts, flush-logs, flush-privileges, flush-status, flush-tables, flush-threads, refresh, and reload."),
-"Event_priv": ("EVENT", "The EVENT privilege is required to create, alter, or drop events for the Event Scheduler. This privilege was added in MySQL 5.1.6."),
 "Shutdown_priv": ("SHUTDOWN", "The SHUTDOWN privilege enables use of the mysqladmin shutdown command. There is no corresponding SQL statement."),
 "Process_priv": ("PROCESS", "The PROCESS privilege pertains to display of information about the threads executing within the server\n(that is, information about the statements being executed by sessions).\nThe privilege enables use of SHOW PROCESSLIST or mysqladmin processlist to see threads belonging to other accounts; you can always see your own threads."),
-"File_priv": ("FILE", """The FILE privilege gives you permission to read and write files on the server host using the 
+"File_priv": ("FILE", """The FILE privilege gives you permission to read and write files on the server host using the
 LOAD DATA INFILE and SELECT ... INTO OUTFILE statements and the LOAD_FILE() function. A user who has the FILE privilege
-can read any file on the server host that is either world-readable or readable by the MySQL server. 
-(This implies the user can read any file in any database directory, because the server can access any of those files.) 
+can read any file on the server host that is either world-readable or readable by the MySQL server.
+(This implies the user can read any file in any database directory, because the server can access any of those files.)
 The FILE privilege also enables the user to create new files in any directory where the MySQL server has write access.
 As a security measure, the server will not overwrite existing files."""),
 "Grant_priv": ("GRANT OPTION", "The GRANT OPTION privilege enables you to give to other users or remove from other users those privileges that you yourself possess."),
@@ -84,13 +82,13 @@ As a security measure, the server will not overwrite existing files."""),
 SHOW DATABASE statement. Accounts that do not have this privilege see only databases for which
 they have some privileges, and cannot use the statement at all if the server was started with
 the --skip-show-database option. Note that any global privilege is a privilege for the database."""),
-"Trigger_priv": ("TRIGGER", "The TRIGGER privilege enables you to create and drop triggers.\nYou must have this privilege for a table to create or drop triggers for that table. This privilege was added in MySQL 5.1.6. (Prior to MySQL 5.1.6, trigger operations required the SUPER privilege.)"),
 "Super_priv": ("SUPER", """The SUPER privilege enables an account to use CHANGE MASTER TO, KILL or
 mysqladmin kill to kill threads belonging to other accounts (you can always kill your own threads),
-PURGE BINARY LOGS, configuration changes via SET GLOBAL to modify global system variables, 
+PURGE BINARY LOGS, configuration changes via SET GLOBAL to modify global system variables,
 the mysqladmin debug command, enabling or disabling logging, performing updates even if the read_only
 system variable is enabled, starting and stopping replication on slave servers, and allows you to
 connect (once) even if the connection limit controlled by the max_connections system variable is reached."""),
+
 "Create_tmp_table_priv": ("CREATE TEMPORARY TABLES", "The CREATE TEMPORARY TABLES privilege enables the use of the keyword TEMPORARY in CREATE TABLE statements."),
 "Lock_tables_priv": ("LOCK TABLES", "The LOCK TABLES privilege enables the use of explicit LOCK TABLES statements to lock tables for which you have the SELECT privilege.\nThis includes the use of write locks, which prevents other sessions from reading the locked table."),
 "Execute_priv": ("EXECUTE", "The EXECUTE privilege is required to execute stored routines (procedures and functions)."),
@@ -101,6 +99,8 @@ connect (once) even if the connection limit controlled by the max_connections sy
 "Create_routine_priv": ("CREATE ROUTINE", "The CREATE ROUTINE privilege is needed to create stored routines (procedures and functions)."),
 "Alter_routine_priv": ("ALTER ROUTINE", "The ALTER ROUTINE privilege is needed to alter or drop stored routines (procedures and functions)."),
 "Create_user_priv": ("CREATE USER", "The CREATE USER privilege enables use of CREATE USER, DROP USER, RENAME USER, and REVOKE ALL PRIVILEGES."),
+"Event_priv": ("EVENT", "The EVENT privilege is required to create, alter, or drop events for the Event Scheduler. This privilege was added in MySQL 5.1.6."),
+"Trigger_priv": ("TRIGGER", "The TRIGGER privilege enables you to create and drop triggers.\nYou must have this privilege for a table to create or drop triggers for that table. This privilege was added in MySQL 5.1.6. (Prior to MySQL 5.1.6, trigger operations required the SUPER privilege.)"),
 "Create_tablespace_priv": ("CREATE TABLESPACE", "The CREATE TABLESPACE privilege is needed to create, alter, or drop tablespaces and log file groups."),
 
 # custom stuff
@@ -111,6 +111,7 @@ connect (once) even if the connection limit controlled by the max_connections sy
 #"IS_monitor_innodb_attr" : ("* InnoDB monitoring in INFO SCHEMA", "")
 }
 
+PrivilegeReverseDict = dict( (val[0], key) for key, val in PrivilegeInfo.iteritems() )
 
 ## Special Tables
 
@@ -118,55 +119,55 @@ connect (once) even if the connection limit controlled by the max_connections sy
 
 AdminAttributes = {
 #             Db, Tables, Db Privileges
-#"IS_monitor_attr" : ("information_schema", 
+#"IS_monitor_attr" : ("information_schema",
 #              ["statistics", "engines", "plugins", "partitions", "files", "processlist", "global_status",
-#              "session_status", "global_variables", "session_variables", "profiling"], 
+#              "session_status", "global_variables", "session_variables", "profiling"],
 #              ["Select_priv"],  None, None),
 "Routine_manage_attr" : ("mysql",
               ["proc", "func", "event"],
               ["Insert", "Select", "Update", "Delete"],
               "GRANT INSERT, SELECT, UPDATE, DELETE ON TABLE mysql.%(table)s TO '%(user)s'@'%(host)s'",
               "REVOKE INSERT, SELECT, UPDATE, DELETE ON TABLE mysql.%(table)s FROM '%(user)s'@'%(host)s'"),
-#"IS_read_all_attr" : ("information_schema", 
-#              ["*"], 
+#"IS_read_all_attr" : ("information_schema",
+#              ["*"],
 #              ["Select_priv"]),
-"User_manage_attr" : ("mysql", 
+"User_manage_attr" : ("mysql",
               ["columns_priv", "db", "host", "procs_priv", "tables_priv", "user"],
               ["Select", "Insert", "Update", "Delete"],
               "GRANT INSERT, SELECT, UPDATE, DELETE ON TABLE mysql.%(table)s TO '%(user)s'@'%(host)s'",
               "REVOKE INSERT, SELECT, UPDATE, DELETE ON TABLE mysql.%(table)s FROM '%(user)s'@'%(host)s'"),
-#"IS_monitor_innodb_attr" : ("information_schema", 
+#"IS_monitor_innodb_attr" : ("information_schema",
 #              [],  # InnoDB_* (5.4+ only)
 #              ["Select_priv"]),
 }
 
 
 # Administrative Roles available for selection. Add your custom roles here.
-# Format is ("NameOfTheRole", "description", [list,of,privileges]) 
+# Format is ("NameOfTheRole", "description", [list,of,privileges])
 # For a list of available privileges, look at the mysql.user table
 SecurityAdminRoles = [
-("DBA", "grants the rights to perform all tasks", 
+("DBA", "grants the rights to perform all tasks",
     PrivilegeInfo.keys()),
-("MaintenanceAdmin", "grants rights needed to maintain server", 
+("MaintenanceAdmin", "grants rights needed to maintain server",
     ["Event_priv", "Reload_priv", "Show_db_priv", "Shutdown_priv", "Super_priv"]), # , "IS_monitor_attr" File_priv?
 ("ProcessAdmin", "rights needed to assess, monitor, and kill any user process running in server",
     ["Reload_priv", "Super_priv"]),#, "IS_monitor_attr"
-("UserAdmin", "grants rights to create users logins and reset passwords", 
+("UserAdmin", "grants rights to create users logins and reset passwords",
     ["Create_user_priv", "Reload_priv"]),
-("SecurityAdmin", "rights to manage logins and grant and revoke server and database level permission", 
+("SecurityAdmin", "rights to manage logins and grant and revoke server and database level permission",
     ["Grant_priv", "Create_user_priv", "Reload_priv", "Show_db_priv", "User_manage_attr"]),
-("MonitorAdmin", "minimum set of rights needed to monitor server", 
+("MonitorAdmin", "minimum set of rights needed to monitor server",
     ["Process_priv"]), #"IS_monitor_attr", "IS_monitor_innodb_attr"
 ("DBManager", "grants full rights on all databases",
-    ["Create_priv", "Drop_priv", "Grant_priv", "Event_priv", "Alter_priv", "Delete_priv", 
+    ["Create_priv", "Drop_priv", "Grant_priv", "Event_priv", "Alter_priv", "Delete_priv",
     "Index_priv", "Insert_priv", "Select_priv", "Update_priv", "Create_tmp_table_priv", "Lock_tables_priv",
     "Trigger_priv", "Create_view_priv", "Show_view_priv", "Create_routine_priv", "Alter_routine_priv", "Show_db_priv"]),
-("DBDesigner", "rights to create and reverse engineer any database schema", 
+("DBDesigner", "rights to create and reverse engineer any database schema",
     ["Create_priv", "Alter_priv", "Index_priv", "Trigger_priv", "Create_view_priv",
     "Show_view_priv", "Create_routine_priv", "Alter_routine_priv", "Show_db_priv", "Routine_manage_attr"]),#, "IS_read_all_attr"
-("ReplicationAdmin", "rights needed to setup and manage replication", 
+("ReplicationAdmin", "rights needed to setup and manage replication",
     ["Repl_client_priv", "Repl_slave_priv", "Super_priv"]),
-("BackupAdmin", "minimal rights needed to backup any database", 
+("BackupAdmin", "minimal rights needed to backup any database",
     ["Event_priv", "Select_priv", "Lock_tables_priv", "Show_db_priv"])
 ]
 
@@ -197,20 +198,33 @@ def acl_compare(t1, t2):
     return get_acl_sort(t1) - get_acl_sort(t2)
 
 
+def escape_schema_name(s):
+    return s.replace("\\", "\\\\").replace("_", "\_").replace("%", "\%")
+
+
 class AdminSecurity(object):
     def __init__(self, ctrl_be):
         self.ctrl_be = ctrl_be
         self._accounts = []
         self._schema_names = []
-        self._account_info_cache = {}
+
+        self._account_info_cache = {}  # A mapping with the form 'user@host' -> AdminAccount instance for that user
         self._schema_privileges_cache = {}
 
         self.has_plugin = False
         self.has_authentication_string = False
         self.has_max_user_connections = False
-        self.schema_privilege_names = []
-        self.global_privilege_names = []
-        self.user_table_fields = []
+
+        # Supported schema specific privilege list:
+        self.schema_privilege_names = []    # This list will be filled in self.async_refresh() with the column names
+                                            # that represent specific privileges in the mysql.db table. These column
+                                            # names are queried so that only the supported ones end up here.
+        # Supported user global privilege list:
+        self.global_privilege_names = []    # This list will be filled in self.async_refresh() with the column names
+                                            # that represent specific privileges in the mysql.user table. These column
+                                            # names are queried so that only the supported ones end up here.
+
+        self.user_table_fields = []         # To be filled with all the fields in the mysql.user table.
 
 
     def get_valid_privileges(self):
@@ -225,73 +239,73 @@ class AdminSecurity(object):
         return privs
 
     def async_refresh(self, callback):
-        # get list of privileges that actually exist
+        # Get the list of privileges supported by the version of MySQL Server we are connected to:
         if not self.schema_privilege_names:
             try:
-              result = self.ctrl_be.exec_query("DESCRIBE mysql.db")
+                result = self.ctrl_be.exec_query("DESCRIBE mysql.db")
             except QueryError, e:
-              if e.error == 1142:
-                raise PermissionDeniedError("Please make sure the used account has rights to the MySQL grant tables.\n%s"%e)
-              raise e
+                if e.error == 1142:
+                    raise PermissionDeniedError("Please make sure the used account has rights to the MySQL grant tables.\n%s" % e)
+                raise e
 
             if result is not None:
-              while result.nextRow():
-                field= result.stringByName("Field")
-                if field.endswith("_priv"):
-                  self.schema_privilege_names.append(field)
+                while result.nextRow():
+                    field= result.stringByName("Field")
+                    if field.endswith("_priv"):
+                        self.schema_privilege_names.append(field)
 
         if not self.user_table_fields:
             try:
-              result = self.ctrl_be.exec_query("DESCRIBE mysql.user")
+                result = self.ctrl_be.exec_query("DESCRIBE mysql.user")
             except QueryError, e:
-              if e.error == 1142:
-                raise PermissionDeniedError("Please make sure the used account has rights to the MySQL grant tables.\n%s"%e)
-              raise e
+                if e.error == 1142:
+                    raise PermissionDeniedError("Please make sure the used account has rights to the MySQL grant tables.\n%s" % e)
+                raise e
 
             if result is not None:
-              while result.nextRow():
-                field= result.stringByName("Field")
-                self.user_table_fields.append(field)
-                if field.endswith("_priv"):
-                  self.global_privilege_names.append(field)
-                if field == "max_user_connections":
-                  self.has_max_user_connections = True
-                if field == "plugin":
-                  self.has_plugin = True
-                if field == "authentication_string":
-                  self.has_authentication_string = True
+                while result.nextRow():
+                    field= result.stringByName("Field")
+                    self.user_table_fields.append(field)
+                    if field.endswith("_priv"):
+                        self.global_privilege_names.append(field)
+                    elif field == "max_user_connections":
+                        self.has_max_user_connections = True
+                    elif field == "plugin":
+                        self.has_plugin = True
+                    elif field == "authentication_string":
+                        self.has_authentication_string = True
 
         # get list of schema names
         schema_names = []
         try:
-          result = self.ctrl_be.exec_query(LIST_SCHEMAS_QUERY)
+            result = self.ctrl_be.exec_query(LIST_SCHEMAS_QUERY)
         except QueryError, e:
-          if e.error == 1142:
-            raise PermissionDeniedError("Please make sure the used account has rights to the MySQL grant tables.\n%s"%e)
-          raise e
+            if e.error == 1142:
+                raise PermissionDeniedError("Please make sure the used account has rights to the MySQL grant tables.\n%s" % e)
+            raise e
         except Exception, e:
-          raise Exception("Error querying security information: %s" % e)
+            raise Exception("Error querying privilege information: %s" % e)
 
         if result is not None:
-          while result.nextRow():
-            name = result.stringByName("Database")
-            schema_names.append(name)
+            while result.nextRow():
+                name = result.stringByName("Database")
+                schema_names.append(name)
 
         schema_names.sort()
         self._schema_names = schema_names
 
-        # get list of account names from User table
+        # Get a list of the account names from the mysql.user table:
         accounts = []
         try:
-          result = self.ctrl_be.exec_query(LIST_ACCOUNTS_QUERY)
+            result = self.ctrl_be.exec_query(LIST_ACCOUNTS_QUERY)
         except Exception, e:
-          raise Exception("Error querying security information: %s" % e)
+            raise Exception("Error querying privilege information: %s" % e)
 
         if result:
-          while result.nextRow():
-            user = result.stringByName("User")
-            host = result.stringByName("Host")
-            accounts.append((user, host))
+            while result.nextRow():
+                user = result.stringByName("User")
+                host = result.stringByName("Host")
+                accounts.append((user, host))
 
         # Sort list of accounts by User and Host
         # User is sorted alphabetically, Host is sorted by specificity in addition to alpha
@@ -305,25 +319,30 @@ class AdminSecurity(object):
 
         callback()
 
-    
+
     @property
     def schema_names(self):
         return self._schema_names
 
+    @property
+    def escaped_schema_names(self):
+        return [escape_schema_name(s) for s in self._schema_names]
+
     @property
     def account_names(self):
         return self._accounts
 
 
-    def delete_account(self, acct):
-        if acct.is_commited:
-            query = REMOVE_USER % {"user":escape_sql_string(acct.username), "host":escape_sql_string(acct.host)}
+    def delete_account(self, account):
+        if account.is_commited:
+            query = REMOVE_USER % {"user":escape_sql_string(account.username), "host":escape_sql_string(account.host)}
+            self.ctrl_be.exec_sql("USE mysql")
             self.ctrl_be.exec_sql(query)
 
             self.ctrl_be.exec_sql(FLUSH_PRIVILEGES)
-    
-        del self._account_info_cache[acct.username+"@"+acct.host]
-        self._accounts.remove((acct.username, acct.host))
+
+        del self._account_info_cache[account.username+"@"+account.host]
+        self._accounts.remove((account.username, account.host))
 
 
     def revert_account(self, account, backup):
@@ -331,7 +350,7 @@ class AdminSecurity(object):
         self._accounts[i] = ((backup.username, backup.host))
         del self._account_info_cache[account.username+"@"+account.host]
         self._account_info_cache[backup.username+"@"+backup.host] = backup
-        
+
         return backup
 
 
@@ -350,7 +369,7 @@ class AdminSecurity(object):
             if (name, host) in self._accounts:
                 name = unique_name(user, host, counter+1 if isinstance(counter, int) else 1)
             return name
-            
+
         acct = AdminAccount(self)
         acct.host = "%"
         acct.username = unique_name('newuser', acct.host)
@@ -365,12 +384,12 @@ class AdminSecurity(object):
         if self._account_info_cache.has_key(key):
             callback(self._account_info_cache[key])
             return
-        
+
         account = AdminAccount(self)
         account.load(name, host)
 
         self._account_info_cache[key] = account
-  
+
         callback(account)
 
 
@@ -378,7 +397,7 @@ class AdminSecurity(object):
     def revert_user_schema_privs(self, privs, backup):
         del self._schema_privileges_cache[privs.username]
         self._schema_privileges_cache[backup.username] = backup
-        
+
         return backup
 
 
@@ -389,11 +408,11 @@ class AdminSecurity(object):
 
         privileges = AdminUserDbPrivs(self)
         privileges.load(user)
-        
+
         self._schema_privileges_cache[user] = privileges
-        
+
         callback(privileges)
-    
+
 
 
 
@@ -402,7 +421,7 @@ class AdminUserDbPrivEntry(object):
     host = None
     db = None
     privileges = set()
-    
+
     def __init__(self, host, db, privileges):
         assert type(privileges) == set
         self.host = host
@@ -412,13 +431,13 @@ class AdminUserDbPrivEntry(object):
 
     def copy(self):
         return AdminUserDbPrivEntry(self.host, self.db, self.privileges.copy())
-    
+
 
 
 
 class AdminUserDbPrivs(object):
     _owner = None
-    username = None    
+    username = None
     entries = []
     _orig_entries = []
 
@@ -428,10 +447,10 @@ class AdminUserDbPrivs(object):
 
     def copy(self):
         copy = AdminUserDbPrivs(self._owner)
-        
+
         copy.username = self.username
         copy.entries = [e.copy() for e in self.entries]
-        
+
         return copy
 
     def snapshot_for_revert(self):
@@ -467,18 +486,18 @@ class AdminUserDbPrivs(object):
 
             schema = result.stringByName("Db")
             host = result.stringByName("Host")
-            
+
             self.entries.append(AdminUserDbPrivEntry(host, schema, privs))
 
         self.entries.sort(lambda a, b: acl_compare((a.host, a.db), (b.host, b.db)))
         self._orig_entries = [e.copy() for e in self.entries]
 
-    
+
     def save(self):
         ##XXX begin transaction
-
+        self._owner.ctrl_be.exec_sql("USE mysql")
         self._owner.ctrl_be.exec_sql(DELETE_SCHEMA_PRIV % {"user":escape_sql_string(self.username) if self.username else self.username})
-        
+
         for entry in self.entries:
             fields = {"user":escape_sql_string(self.username), "host":entry.host, "db":entry.db}
             priv_names = []
@@ -486,14 +505,14 @@ class AdminUserDbPrivs(object):
             for priv in self._owner.schema_privilege_names:
                 priv_names.append("%s" % priv)
                 if priv in entry.privileges:
-                  priv_values.append("'Y'")
+                    priv_values.append("'Y'")
                 else:
-                  priv_values.append("'N'")
+                    priv_values.append("'N'")
                 #priv_values.append("'Y'" if priv in entry.privileges else "'N'")
             fields["priv_names"] = ", ".join(priv_names)
             fields["priv_values"] = ", ".join(priv_values)
             self._owner.ctrl_be.exec_sql(INSERT_SCHEMA_PRIV % fields)
-        
+
         ##XXX commit transaction
 
         self._owner.ctrl_be.exec_sql(FLUSH_PRIVILEGES)
@@ -502,7 +521,7 @@ class AdminUserDbPrivs(object):
 
 class AdminAccount(object):
     _owner = None
-    
+
     _orig_username = None
     _orig_host = None
     _orig_password = None
@@ -511,19 +530,21 @@ class AdminAccount(object):
     password = None
     confirm_password = None
     host = None
-    
+
     max_questions = 0
     max_updates = 0
     max_connections = 0
     max_user_connections = 0
     auth_plugin = None
     auth_string = None
-    
+
     _global_privs = set()
+    _custom_checked_privs = set()
+    _remembered_custom_privs = set()
     _orig_global_privs = set()
 
     is_commited = False # False means that account was not pushed to mysql server
-    
+
     def __init__(self, owner):
         self._owner = owner
 
@@ -547,8 +568,9 @@ class AdminAccount(object):
         copy.auth_string = self.auth_string
         copy.is_commited = self.is_commited
         copy._global_privs = self._global_privs.copy()
+        copy._custom_checked_privs = self._custom_checked_privs.copy()
         return copy
-    
+
     def snapshot_for_revert(self):
         copy = self.copy()
         copy._orig_username = self._orig_username
@@ -557,34 +579,50 @@ class AdminAccount(object):
         copy._orig_host = self._orig_host
         return copy
 
+    def toggle_priv(self, priv, flag):
+        if flag:
+            self._global_privs.update([priv])
+        else:
+            self._global_privs.difference_update([priv])
+            self._remembered_custom_privs.difference_update([priv])
+
+    @property
+    def is_custom_role_needed(self):
+        return bool(self._remembered_custom_privs or self._custom_checked_privs)
+
+    def forget_custom_privs(self):
+        self._remembered_custom_privs = set()
+
     def toggle_role(self, role, flag):
         def privs_for_role(role):
+            if role == 'Custom':
+                return self._remembered_custom_privs
             for rname, rdesc, rprivs in SecurityAdminRoles:
                 if rname == role:
-                    return rprivs
-            return []
+                    return set(rprivs).intersection(self._owner.global_privilege_names)
+            return set()
 
         privs = privs_for_role(role)
         if privs:
             if flag:
-                self._global_privs = self._global_privs.union(set(privs))
+                self._global_privs.update(privs)
             else:
-                self._global_privs = self._global_privs.difference(set(privs))
-
-            # check what roles are left and then reapply them so that there are no leftover privs
-            admin_roles = self.admin_roles
-            self._global_privs = set()
-            for role in admin_roles:
-                self._global_privs = self._global_privs.union(set(privs_for_role(role)))
-
+                self._global_privs.difference_update(privs)
 
     @property
     def admin_roles(self):
         roles = []
+        privs_from_roles = set()
         for rname, rdesc, rprivs in SecurityAdminRoles:
-            if set(rprivs).issubset(self._global_privs):
+            # The role is active if the subset of its privileges that are supported by the current server
+            # version are fully contained in the privileges that the user has:
+            if set(rprivs).intersection(self._owner.global_privilege_names).issubset(self._global_privs):
                 roles.append(rname)
-
+                privs_from_roles.update(rprivs)
+        self._custom_checked_privs = self._global_privs - privs_from_roles
+        if self._custom_checked_privs:
+            roles.append('Custom')
+            self._remembered_custom_privs.update(self._custom_checked_privs)
         return roles
 
 
@@ -594,6 +632,7 @@ class AdminAccount(object):
 
 
     def revoke_all(self):
+        self._owner.ctrl_be.exec_sql("USE mysql")
         command = REVOKE_ALL % {"user":escape_sql_string(self.username),"host":self.host}
         self._owner.ctrl_be.exec_sql(command)
 
@@ -626,14 +665,16 @@ class AdminAccount(object):
             "auth_string" : escape_sql_string(self.auth_string) if self.auth_string else None
         }
 
+        self._owner.ctrl_be.exec_sql("USE mysql")
+
         if self.is_commited:
             assert self._orig_username is not None and self._orig_host is not None
             assignments = []
             for priv in self._owner.global_privilege_names:
                 if priv in self._global_privs:
-                  assignments.append("%s='%s'" % (priv, 'Y'))
+                    assignments.append("%s='%s'" % (priv, 'Y'))
                 else:
-                  assignments.append("%s='%s'" % (priv, 'N'))
+                    assignments.append("%s='%s'" % (priv, 'N'))
 
             for limit in ["max_questions", "max_updates", "max_connections"] + (self._owner.has_max_user_connections and ["max_user_connections"] or []):
                 assignments.append("%s='%s'" % (limit, getattr(self, limit)))
@@ -654,18 +695,18 @@ class AdminAccount(object):
             query = UPDATE_ACCOUNT_QUERY % fields
 
             try:
-                result = self._owner.ctrl_be.exec_sql(query)
+                self._owner.ctrl_be.exec_sql(query)
             except QueryError, e:
                 if e.error == 1142:
-                    raise Exception("Error updating account %s@%s: Insufficient rights to perform operation"%(self.username, self.host))
+                    raise PermissionDeniedError("Error updating account %s@%s: Insufficient rights to perform operation" % (self.username, self.host) )
                 else:
-                    raise Exception("Error updating account %s@%s: %s"%(self.username, self.host, e))
+                    raise Exception("Error updating account %s@%s: %s" % (self.username, self.host, e) )
             except Exception, e:
-                raise Exception("Error updating account %s@%s: %s"%(self.username, self.host, e))
+                raise Exception("Error updating account %s@%s: %s" % (self.username, self.host, e) )
 
             self._owner.ctrl_be.exec_sql(FLUSH_PRIVILEGES)
         else:
-            # Account was nost saved yet
+            # Account was not saved yet
             def run_query(query):
                 result = None
                 try:
@@ -678,7 +719,7 @@ class AdminAccount(object):
                 except Exception, e:
                     raise Exception("Error creating account %s@%s: %s"%(self.username, self.host, e))
                 return result
-
+  
             raw_pwd = fields['password']
             fields['password'] = make_scrambled_password(raw_pwd)
             if self.auth_plugin:
@@ -699,22 +740,22 @@ class AdminAccount(object):
                 (grant_priv_type, description) = PrivilegeInfo.get(priv_field_name, (None,None))
                 if grant_priv_type == 'GRANT OPTION':
                     # these will be glued to 'GRANT OPTION' below at 'for name, value...
-                    names.append("GRANT") 
+                    names.append("GRANT")
                     values.append("OPTION")
                 elif grant_priv_type and (grant_priv_type[0] != '*'):
                     if grant_priv_type in valid_privs:
                         grants.append(grant_priv_type)
 
             if len(grants) == 0:
-              grants.append("USAGE")
+                grants.append("USAGE")
 
             fields["grants"] = ", ".join(grants)
 
             grant_query = "GRANT %(grants)s ON *.* TO '%(user)s'@'%(host)s' with " % fields
 
             for name, value in zip(names, values):
-              if value:
-                grant_query += "%s %s "%(name, str(value))
+                if value:
+                    grant_query += "%s %s "%(name, str(value))
 
             run_query(grant_query)
 
@@ -769,12 +810,12 @@ class AdminAccount(object):
 
         self.username = result.stringByName("User")
         self.host = result.stringByName("Host")
-        
+
         self._orig_password = result.stringByName("Password")
         self._orig_username = self.username
         self._orig_host = self.host
         self._orig_auth_string = self.auth_string
-        
+
         self.max_questions = result.intByName("max_questions")
         self.max_updates = result.intByName("max_updates")
         self.max_connections = result.intByName("max_connections")
@@ -784,12 +825,13 @@ class AdminAccount(object):
             self.auth_plugin = result.stringByName("plugin")
         if self._owner.has_authentication_string:
             self.auth_string = result.stringByName("authentication_string")
-        
+
         self._global_privs = set()
         for priv in self._owner.global_privilege_names:
             if result.stringByName(priv) == 'Y':
                 self._global_privs.add(priv)
 
+        self.forget_custom_privs()
 
         """ not necessary, IS is accessible to all
         # privs from information_schema tables
@@ -836,7 +878,3 @@ class AdminAccount(object):
                     self._global_privs.add(name)
 
         self._orig_global_privs = self._global_privs.copy()
-
-
-
-
