@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -25,7 +25,7 @@ import grt
 import mforms
 
 from wb_server_management import wbaOS
-from wb_common import OperationCancelledError, InvalidPasswordError, dprint_ex
+from wb_common import OperationCancelledError, InvalidPasswordError
 
 import re
 import socket
@@ -43,48 +43,48 @@ def get_local_ip_list():
 
     out = None
     try:
-      out = os.popen(cmd)
+        out = os.popen(cmd)
     except Exception, e:
-      # We do not care if execution failed, the list will just be empty
-      #print e
-      out = None
+        # We do not care if execution failed, the list will just be empty
+        #print e
+        out = None
     except:
-      print "Unknown exception while running '%s'"%cmd
+        print "Unknown exception while running '%s'"%cmd
 
     try:
-      ret = []
-      if out:
-          result = out.read()
-          m = re.findall(regexp, result)
+        ret = []
+        if out:
+            result = out.read()
+            m = re.findall(regexp, result)
 
-          for ip in m:
-              try:
-                  t_ip = socket.gethostbyname(ip)
-                  ret.append(ip)
-              except socket.gaierror,e:
-                  print ip, str(e)
+            for ip in m:
+                try:
+                    t_ip = socket.gethostbyname(ip)
+                    ret.append(ip)
+                except socket.gaierror,e:
+                    print ip, str(e)
     except Exception, e:
-      # We do not care if execution failed, the list will just be empty
-      #print e
-      ret = []
+        # We do not care if execution failed, the list will just be empty
+        #print e
+        ret = []
     except:
-      print "Unknown exception while running '%s'"%cmd
-      ret = []
+        print "Unknown exception while running '%s'"%cmd
+        ret = []
 
     return ret
 
 local_ip_list = get_local_ip_list()
 
 #===============================================================================
-class ServerProfile:
+class ServerProfile(object):
     def __init__(self, instance_profile, force_remote_admin_off=False):
         self.disable_remote_admin = force_remote_admin_off
         self.__settings = instance_profile
         self.expanded_config_file_path = None
-    
+
     def get_settings_object(self):
         return self.__settings
-    
+
     def _int_server_info(self, name, default_value=0):
         # always returns an int, a None value will be replaced with default_value
         try:
@@ -99,7 +99,7 @@ class ServerProfile:
         if value is None:
             return default_value
         return value
-    
+
     def _int_login_info(self, name, default_value=0):
         # always returns an int, a None value will be replaced with default_value
         try:
@@ -149,43 +149,43 @@ class ServerProfile:
 
     @property
     def host_os(self):
-      if hasattr(sys, 'getwindowsversion'):
-        return wbaOS.windows
-      elif ('inux' in sys.platform):
-        return wbaOS.linux
-      elif ('arwin' in sys.platform):
-        return wbaOS.darwin
-      return wbaOS.unknown
+        if hasattr(sys, 'getwindowsversion'):
+            return wbaOS.windows
+        elif ('inux' in sys.platform):
+            return wbaOS.linux
+        elif ('arwin' in sys.platform):
+            return wbaOS.darwin
+        return wbaOS.unknown
 
     # Must return smth from wbaOS
     @property
     def target_os(self):
-      # This call must return either value from profile for remote server
-      # If remote server is not set, which means local server, then returned value
-      # must be equal to self.host_os
-      ret = wbaOS.unknown
-      
-      try:
-        system = self.__settings.serverInfo['sys.system']
-      except:
-        system = None
-      if system is not None and type(system) is str or type(system) is unicode:
-        system = system.strip(" \r\t\n").lower()
-        if system == 'windows':
-          ret = wbaOS.windows
-        elif system in ('linux', 'freebsd', 'opensolaris'):
-          ret = wbaOS.linux
-        elif ('macos' in system):
-          ret = wbaOS.darwin
-      else:
-        ret = self.host_os
-      
-      return ret
+        # This call must return either value from profile for remote server
+        # If remote server is not set, which means local server, then returned value
+        # must be equal to self.host_os
+        ret = wbaOS.unknown
+
+        try:
+            system = self.__settings.serverInfo['sys.system']
+        except:
+            system = None
+        if system is not None and type(system) is str or type(system) is unicode:
+            system = system.strip(" \r\t\n").lower()
+            if system == 'windows':
+                ret = wbaOS.windows
+            elif system in ('linux', 'freebsd', 'opensolaris'):
+                ret = wbaOS.linux
+            elif ('macos' in system):
+                ret = wbaOS.darwin
+        else:
+            ret = self.host_os
+
+        return ret
 
     @property
     def host_is_windows(self):
         return self.host_os == wbaOS.windows
-    
+
     @property
     def target_is_windows(self):
         return self.target_os == wbaOS.windows
@@ -194,11 +194,11 @@ class ServerProfile:
     @property
     def remote_admin_enabled(self):
         return (self.uses_ssh or (not self.is_local and self.uses_wmi)) and not self.disable_remote_admin
-    
+
     @property
     def admin_enabled(self):
         return self.is_local or self.remote_admin_enabled
-        
+
     @property
     def is_local(self):
         local_addrs = ["localhost", "", "0", "127.0.0.1"] + local_ip_list
@@ -210,30 +210,36 @@ class ServerProfile:
             if self.db_connection_params:
                 params = self.db_connection_params.parameterValues
                 if params:
-                    if self.db_connection_params.driver.name == "MysqlNative":
-                        return params["hostName"] in local_addrs
-                    elif self.db_connection_params.driver.name == "MysqlNativeSocket":
-                        return True
-                    elif self.db_connection_params.driver.name == "MysqlNativeSSH":
-                        return False
+                    if self.db_connection_params.driver:
+                        if self.db_connection_params.driver.name == "MysqlNative":
+                            return params["hostName"] in local_addrs
+                        elif self.db_connection_params.driver.name == "MysqlNativeSocket":
+                            return True
+                        elif self.db_connection_params.driver.name == "MysqlNativeSSH":
+                            return False
+                    else:
+                        from wb_common import NoDriverInConnection
+                        raise NoDriverInConnection, """Workbench has not found a driver for the connection
+that is being used by this server instance.
+Please edit your connection settings and try again."""
             return False
 
 
     @property
     def connect_method(self):
-      # choices are ['local', 'ssh', 'wmi', 'none'] 
-      # 'none' is for remote server ip with "No Remote Management" selected
-      ret = None
-      if self.uses_wmi:
-        ret = 'wmi'
-      elif self.uses_ssh:
-        ret = 'ssh'
-      else:
-        if self.is_local:
-          ret = 'local'
+        # choices are ['local', 'ssh', 'wmi', 'none']
+        # 'none' is for remote server ip with "No Remote Management" selected
+        ret = None
+        if self.uses_wmi:
+            ret = 'wmi'
+        elif self.uses_ssh:
+            ret = 'ssh'
         else:
-          ret = 'none'
-      return ret
+            if self.is_local:
+                ret = 'local'
+            else:
+                ret = 'none'
+        return ret
 
 
     @property
@@ -264,11 +270,11 @@ class ServerProfile:
     @property
     def ssh_key(self):
         return self._str_login_info("ssh.key")
-    
+
     @property
     def wmi_hostname(self):
         return self._str_login_info("wmi.hostName")
-    
+
     @property
     def wmi_username(self):
         return self._str_login_info("wmi.userName")
@@ -276,19 +282,29 @@ class ServerProfile:
     @property
     def wmi_service_name(self):
         return self._str_server_info("sys.mysqld.service_name")
-        
+
     @property
     def start_server_cmd(self):
-        return self._str_server_info("sys.mysqld.start")
-    
+        ret = None
+        if self.uses_wmi:
+            ret = "sc start " + self.wmi_service_name
+        else:
+            ret = self._str_server_info("sys.mysqld.start")
+        return ret
+
     @property
     def stop_server_cmd(self):
-        return self._str_server_info("sys.mysqld.stop")
-        
+        ret = None
+        if self.uses_wmi:
+            ret = "sc stop " + self.wmi_service_name
+        else:
+            ret = self._str_server_info("sys.mysqld.stop")
+        return ret
+
     @property
     def check_server_status_cmd(self):
         return self._str_server_info("sys.mysqld.status")
-        
+
     @property
     def use_sudo(self):
         return self._int_server_info("sys.usesudo")
@@ -300,8 +316,8 @@ class ServerProfile:
     @property
     def use_sudo_for_status(self):
         return self._int_server_info("sys.usesudostatus")
-        
-    
+
+
     @property
     def config_file_path(self):
         """ path to configuration file (eg my.cnf)
@@ -320,13 +336,88 @@ class ServerProfile:
     def sys_script(self):
         return self._str_server_info('sys.script', '')
 
+    #-------------------------------------------------------
+    def server_version(self):
+        return self._str_server_info("serverVersion", None)
+
+    def set_server_version(self, version):
+        self.__settings.serverInfo["serverVersion"] = version
+
+    server_version = property(server_version, set_server_version)
+    #-------------------------------------------------------
+    @property
+    def server_version_tuple(self):
+        if not self.server_version:
+            return ()
+        import re
+        version_tokens = self.server_version.split('.')
+        ver = []
+        for token in version_tokens:
+            match = re.match(r'(\d+).*', token)
+            if match:
+                ver.append(int(match.group(1)))
+        return tuple(ver)
+
+    #-------------------------------------------------------
+    def log_output(self):
+        return self._str_server_info("logOutput")
+
+    def set_log_output(self, output):
+        self.__settings.serverInfo['logOutput'] = output
+
+    log_output = property(log_output, set_log_output)
+
+    #-------------------------------------------------------
+    def general_log_enabled(self):
+        return self._int_server_info("generalLogEnabled")
+
+    def set_general_log_enabled(self, value):
+        self.__settings.serverInfo['generalLogEnabled'] = value
+
+    general_log_enabled = property(general_log_enabled, set_general_log_enabled)
+
+    #-------------------------------------------------------
+    def slow_log_enabled(self):
+        return self._int_server_info("slowLogEnabled")
+
+    def set_slow_log_enabled(self, value):
+        self.__settings.serverInfo['slowLogEnabled'] = value
+
+    slow_log_enabled = property(slow_log_enabled, set_slow_log_enabled)
+
+    #-------------------------------------------------------
+    def general_log_file_path(self):
+        return self._str_server_info("generalLogFilePath")
+
+    def set_general_log_file_path(self, path):
+        self.__settings.serverInfo['generalLogFilePath'] = path
+
+    general_log_file_path = property(general_log_file_path, set_general_log_file_path)
+
+    #-------------------------------------------------------
+    def slow_log_file_path(self):
+        return self._str_server_info("slowLogFilePath")
+
+    def set_slow_log_file_path(self, path):
+        self.__settings.serverInfo['slowLogFilePath'] = path
+
+    slow_log_file_path = property(slow_log_file_path, set_slow_log_file_path)
+
+    #-------------------------------------------------------
+    def error_log_file_path(self):
+        return self._str_server_info("errorLogFilePath")
+
+    def set_error_log_file_path(self, path):
+        self.__settings.serverInfo['errorLogFilePath'] = path
+
+    error_log_file_path = property(error_log_file_path, set_error_log_file_path)
 
 
-class PasswordHandler:
+class PasswordHandler(object):
     def __init__(self, server_profile):
         self.server_profile = server_profile
         self.pwd_store = {}
-    
+
     def get_password_parameters(self, service_type):
         # known service types: remoteshell, ssh, sshkey, file, service.startstop, service.status
         # these are mapped to
@@ -409,23 +500,23 @@ class PasswordHandler:
 
         elif password_type == "UAC":
             return "UAC"
-        
+
         elif password_type == "ssh":
             return ("SSH Login Password Required", "ssh@%s" % profile.ssh_hostname, profile.ssh_username)
-        
+
         elif password_type == "sshkey":
             return ("SSH Private Key Password Required", "ssh_keyfile@%s" % profile.ssh_key, profile.ssh_username)
-        
+
         else:
             return None
-        
+
     # call this if a InvalidPasswordError is caught
     def reset_password_for(self, service_type):
         if self.pwd_store.has_key(service_type):
             # None means the stored password was bad
             self.pwd_store[service_type] = None
 
-    def get_password_for(self, service_type): # Raises OperationCancelledError if user cancels pwd dlg. 
+    def get_password_for(self, service_type): # Raises OperationCancelledError if user cancels pwd dlg.
         force_reset = False
         if self.pwd_store.has_key(service_type):
             if self.pwd_store[service_type] is None:
@@ -436,14 +527,14 @@ class PasswordHandler:
         details = self.get_password_parameters(service_type)
         if not details: # No password needed for this service_type
             return None
-        
+
         # special case for UAC used in windows
         if details == "UAC":
             return "UAC"
 
         title, service, account = details
 
-        dprint_ex(2, "request password for %s => %s, %s, %s" % (service_type, title, service, account))
+        grt.log_debug2('%s:get_password_for()' % self.__class__.__name__, 'request password for "%s" => "%s", "%s", "%s"\n' % (service_type, title, service, account))
 
         accepted, password = mforms.Utilities.find_or_ask_for_password(title, service, account, force_reset)
         if accepted:
@@ -455,7 +546,7 @@ class PasswordHandler:
             raise OperationCancelledError("Password input cancelled")
 
 
-class ServerControlBase:
+class ServerControlBase(object):
     def __init__(self, serverProfile, helper, password_delegate):
         self.profile = serverProfile
         self.helper = helper
@@ -491,7 +582,7 @@ class ServerControlBase:
         thread = threading.Thread(target=self.worker_thread, args=(self.start, password, finish_callback))
         thread.run()
         return True
-    
+
     def worker_thread(self, action, password, finish_callback):
         try:
             action(password)
@@ -506,7 +597,7 @@ class ServerControlBase:
 
     def stop(self, password): # overriden by concrete subclasses, must be non-interactive
         assert 0
-    
+
     def stop_async(self, finish_callback):
         try:
             password = self.get_password()
@@ -526,11 +617,11 @@ class ServerControlShell(ServerControlBase):
         ServerControlBase.__init__(self, profile, helper, password_delegate)
 
         # Ask for password here, as this class is instantiated from the main thread
-        # The other methods may be instantiated from other threads, which will not work 
+        # The other methods may be instantiated from other threads, which will not work
         # with the password dialogs
-        
+
         # Ensure sudo password for status checks is cached. For other sudo uses,
-        # the password should be requested when the action is started from the UI (ie, in the code called 
+        # the password should be requested when the action is started from the UI (ie, in the code called
         # by the Start/Stop button in Server startup tab)
         if self.profile.use_sudo_for_status:
             password_delegate.get_password_for("service.status")
@@ -552,15 +643,15 @@ class ServerControlShell(ServerControlBase):
         """ Can also throw InvalidPasswordError"""
         self.info("Starting server...") # TODO: This does not come to info output
         try:
-            r = self.helper.execute_command(self.profile.start_server_cmd, 
-                                    as_admin=self.profile.use_sudo, 
+            r = self.helper.execute_command(self.profile.start_server_cmd,
+                                    as_admin=self.profile.use_sudo,
                                     admin_password=password,
                                     output_handler=lambda s:self.info("Start server: %s"%(s if ((type(s) is str) or (type(s) is unicode)) else "").replace(password, "")))
         except InvalidPasswordError, exc:
             # forget password, so that next time it will be requested
             self.password_delegate.reset_password_for("service.startstop")
             raise exc
-            
+
         if r:
             return True
         return False
@@ -578,8 +669,8 @@ class ServerControlShell(ServerControlBase):
                     output = ""
                 self.info("Stop server: %s"%str(output))
 
-            r = self.helper.execute_command(self.profile.stop_server_cmd, 
-                                    as_admin=self.profile.use_sudo, 
+            r = self.helper.execute_command(self.profile.stop_server_cmd,
+                                    as_admin=self.profile.use_sudo,
                                     admin_password=password,
                                     output_handler=strip_pwd)
         except InvalidPasswordError, exc:
@@ -624,9 +715,9 @@ class ServerControlShell(ServerControlBase):
             if dump:
                 dump(line)
         try:
-            result = self.helper.execute_command(raw_script, 
-                                    as_admin=self.profile.use_sudo_for_status, 
-                                    admin_password=password, 
+            result = self.helper.execute_command(raw_script,
+                                    as_admin=self.profile.use_sudo_for_status,
+                                    admin_password=password,
                                     output_handler=collect_output)
         except InvalidPasswordError, exc:
             self.info("Invalid password")
@@ -643,21 +734,21 @@ class ServerControlShell(ServerControlBase):
         else:
             #output_text = "\n".join(lines)
             pass
-        
-    
+
+
         if verbose > 1:
             self.info("Server check returned %s" % result)
 
         status = "unknown"
         if result == 0:
             if verbose:
-                self.info("Checked server status: Server is running.")    
+                self.info("Checked server status: Server is running.")
             status = "running"
         elif result == 1:
             if verbose:
                 self.info("Checked server status: Server is stopped.")
             status = "stopped"
-    
+
         return status
 
 
@@ -716,9 +807,9 @@ class ServerControlWMI(ServerControlBase):
 
     def set_filter_functions(self, prepare_f, apply_f):
         if self.profile.is_local and self.shell:
-          print "WMIShell: Set filters for local windows cli"
-          self.shell.prepare_filter = prepare_f
-          self.shell.apply_filter = apply_f
+            print "WMIShell: Set filters for local windows cli"
+            self.shell.prepare_filter = prepare_f
+            self.shell.apply_filter = apply_f
 
     def get_password(self):
         if self.profile.is_local:
@@ -796,10 +887,3 @@ class ServerControlWMI(ServerControlBase):
             result = "starting"
 
         return result
-
-
-
-
-
-
-
