@@ -1,15 +1,15 @@
-# Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
 # published by the Free Software Foundation; version 2 of the
 # License.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
@@ -31,7 +31,7 @@ import wb_admin_ssh
 import wb_admin_control
 from wb_server_control import PasswordHandler
 from wb_server_control import ServerProfile
-from db_utils import MySQLConnection, MySQLError
+from workbench.db_utils import MySQLConnection, MySQLError
 from wb_common import OperationCancelledError, InvalidPasswordError, NoDriverInConnection
 from wb_server_management import local_run_cmd, local_get_cmd_output
 
@@ -62,19 +62,19 @@ class AdministratorTab(AppView):
 
         server_profile = ServerProfile(server_instance_settings, False)
 
-        self.ctrl_be = wb_admin_control.WbAdminControl(server_profile, 
+        self.ctrl_be = wb_admin_control.WbAdminControl(server_profile,
                         connect_sql=True)
 
         self.ctrl_be.init()
 
         version = self.ctrl_be.get_server_version()
         if type(version) is tuple:
-          valid_versions = ((4,0), (4,1), (5,0), (5,1), (5,2), (5,4), (5,5), (5,6), (6,0))
-          if version[:2] not in valid_versions:
-            print version, "UNSUPPORTED"
-            log_warning(_this_file, "%s: Server version %s is NOT supported\n" % (self.__class__.__name__, str(version)) )
-          else:
-            log_info(_this_file, "%s: Server version %s is supported\n" % (self.__class__.__name__, str(version)) )
+            valid_versions = ((4,0), (4,1), (5,0), (5,1), (5,2), (5,4), (5,5), (5,6), (6,0))
+            if version[:2] not in valid_versions:
+                print version, "UNSUPPORTED"
+                log_warning(_this_file, "%s: Server version %s is NOT supported\n" % (self.__class__.__name__, str(version)) )
+            else:
+                log_info(_this_file, "%s: Server version %s is supported\n" % (self.__class__.__name__, str(version)) )
 
         self.on_close(wb_admin_utils.weakcb(self, "handle_on_close"))
 
@@ -88,8 +88,11 @@ class AdministratorTab(AppView):
     def handle_on_close(self):
         App.get().set_status_text("Closing Administator.")
         self.configuration.shutdown()
+        if not self.configuration.closing:
+            return False
         self.ctrl_be.shutdown()
         tab_references.remove(self)
+        self.release()
         return True
 
     #---------------------------------------------------------------------------
@@ -113,29 +116,29 @@ def do_open_administrator(server_instance):
     validate_setting(server_instance.serverInfo, "sys.usesudostatus", norm_to_switch, None)
 
     if server_instance.serverInfo["sys.system"] != "Windows":
-      #validate_setting(server_instance.serverInfo, "sys.sudo", make_str_existing, "Server profile has no privileges elevation command defined")
+        #validate_setting(server_instance.serverInfo, "sys.sudo", make_str_existing, "Server profile has no privileges elevation command defined")
 
-      #if server_instance.serverInfo.has_key("sys.sudo") and server_instance.serverInfo["sys.sudo"].strip(" \r\t\n") == "":
-      #  Utilities.show_warning("WB Administrator", "Server profile has empty privileges elevation command defined. Some functionality maybe unavailable", "OK", "", "")
-      try:
-        if not server_instance.serverInfo["sys.sudo"]:
-          # don't break settings that were working perfectly before, assume a valid default
-          server_instance.serverInfo["sys.sudo"] = "/usr/bin/sudo -p EnterPasswordHere /bin/bash -c"
-      except:
-        server_instance.serverInfo["sys.sudo"] = "/usr/bin/sudo -p EnterPasswordHere /bin/bash -c"
+        #if server_instance.serverInfo.has_key("sys.sudo") and server_instance.serverInfo["sys.sudo"].strip(" \r\t\n") == "":
+        #  Utilities.show_warning("WB Administrator", "Server profile has empty privileges elevation command defined. Some functionality maybe unavailable", "OK", "", "")
+        try:
+            if not server_instance.serverInfo["sys.sudo"]:
+            # don't break settings that were working perfectly before, assume a valid default
+                server_instance.serverInfo["sys.sudo"] = "/usr/bin/sudo -p EnterPasswordHere /bin/bash -c"
+        except:
+            server_instance.serverInfo["sys.sudo"] = "/usr/bin/sudo -p EnterPasswordHere /bin/bash -c"
 
     app = App.get()
     try:
-      adminTab = AdministratorTab(server_instance)
+        adminTab = AdministratorTab(server_instance)
     except wb_admin_ssh.ConnectionError, exc:
-      Utilities.show_error("Error Connecting to Server (%s@%s)" % (server_instance.loginInfo["ssh.userName"], server_instance.loginInfo["ssh.hostName"]), str(exc), "OK", "", "")
-      app.set_status_text("Could not Open WB Admin")
-      return None
+        Utilities.show_error("Error Connecting to Server (%s@%s)" % (server_instance.loginInfo["ssh.userName"], server_instance.loginInfo["ssh.hostName"]), str(exc), "OK", "", "")
+        app.set_status_text("Could not Open WB Admin")
+        return None
     except MySQLError, exc:
-      if exc.message:
-        Utilities.show_error("Error Connecting to MySQL Server (%s)" % exc.location, str(exc), "OK", "", "")
-      app.set_status_text("Could not Open WB Admin")
-      return None
+        if exc.message:
+            Utilities.show_error("Error Connecting to MySQL Server (%s)" % exc.location, str(exc), "OK", "", "")
+        app.set_status_text("Could not Open WB Admin")
+        return None
     except OperationCancelledError, exc:
         app.set_status_text("Cancelled (%s)"%exc)
         return None
@@ -149,14 +152,14 @@ def do_open_administrator(server_instance):
         Utilities.show_error("Error Starting Workbench Administrator", "%s: %s" % (type(exc).__name__, exc), "OK", "", "")
         app.set_status_text("Could not Open WB Admin")
         return None
-    
+
     version = adminTab.ctrl_be.get_server_version()
     if version and version[0] < 5:
-      Utilities.show_error("Unsupported Server Version", "The version of the server you're trying to connect to is %i.%i, which is not supported by Workbench."%version[:2],
-                          "Close", "Ignore", "")
-      app.set_status_text("Could not Open WB Admin")
-      return None
-    
+        Utilities.show_error("Unsupported Server Version", "The version of the server you're trying to connect to is %i.%i, which is not supported by Workbench."%version[:2],
+                            "Close", "Ignore", "")
+        app.set_status_text("Could not Open WB Admin")
+        return None
+
     app.dock_view(adminTab, "maintab")
     app.set_view_title(adminTab, "Admin (%s)" % (server_instance.name))
 
@@ -170,30 +173,30 @@ def do_open_administrator(server_instance):
 
 #-------------------------------------------------------------------------------
 def validate_setting(settings, option, norm_cb, msg):
-  if settings.has_key(option):
-    if norm_cb is not None:
-      norm_cb(settings, option)
-  else:
-    if msg is not None:
-      Utilities.show_warning("WB Administartor", msg, "OK", "", "")
-    norm_cb(settings, option)
+    if settings.has_key(option):
+        if norm_cb is not None:
+            norm_cb(settings, option)
+    else:
+        if msg is not None:
+            Utilities.show_warning("WB Administartor", msg, "OK", "", "")
+        norm_cb(settings, option)
 
 #-------------------------------------------------------------------------------
 def norm_to_switch(settings, option):
-  value = 0
-  if settings.has_key(option):
-    value = settings[option]
-    if value > 0:
-      value = 1
-    else:
-      value = 0
+    value = 0
+    if settings.has_key(option):
+        value = settings[option]
+        if value > 0:
+            value = 1
+        else:
+            value = 0
 
-  settings[option] = value
+    settings[option] = value
 
 #-------------------------------------------------------------------------------
 def make_str_existing(settings, option):
-  if not settings.has_key(option):
-    settings[option] = ""
+    if not settings.has_key(option):
+        settings[option] = ""
 
 #-------------------------------------------------------------------------------
 @ModuleInfo.plugin("wb.admin.open", type="standalone", caption= "Initialize WB Admin",  pluginMenu= "Administrator")
@@ -231,36 +234,35 @@ def selectServer(title):
     window.set_title(title)
     box = mforms.newBox(False)
     window.set_content(box)
-    
+
     box.set_padding(12)
     box.set_spacing(12)
-    
-    label = mforms.newLabel()
-    label.set_text("Select Server to Connect to:")
+
+    label = mforms.newLabel("Select Server to Connect to:")
     box.add(label, False, True)
-    
+
     listbox = mforms.newListBox(False)
     box.add(listbox, True, True)
     listbox.show();
 
     for inst in grt.root.wb.rdbmsMgmt.storedInstances:
-      listbox.add_item(inst.name)
-      
+        listbox.add_item(inst.name)
+
     if len(grt.root.wb.rdbmsMgmt.storedInstances) > 0:
         listbox.set_selected(0)
     else:
-        Utilities.show_warning("No Database Server Instances", 
+        Utilities.show_warning("No Database Server Instances",
                     '''You have not defined a database server instance. At least one
-    server instance is needed. Please define one by clicking in 
+    server instance is needed. Please define one by clicking in
     "New Server Instance" and retry.''',
                     "OK", "", "")
         return None
-    
+
     bbox = mforms.newBox(True)
     box.add(bbox, False, True)
-    
+
     bbox.set_spacing(8)
-    
+
     ok = mforms.newButton()
     ok.set_text("OK")
     bbox.add_end(ok, False, True)
@@ -268,16 +270,16 @@ def selectServer(title):
     cancel = mforms.newButton()
     cancel.set_text("Cancel")
     bbox.add_end(cancel, False, True)
-    
+
     window.set_size(400, 300)
     window.center()
-    
+
     if window.run_modal(ok, cancel):
-      i = listbox.get_selected_index()
-      if i >= 0:
-          return grt.root.wb.rdbmsMgmt.storedInstances[i]
+        i = listbox.get_selected_index()
+        if i >= 0:
+            return grt.root.wb.rdbmsMgmt.storedInstances[i]
     return None
-        
+
 
 
 @ModuleInfo.plugin("wb.admin.dumpManager", type="standalone", caption= "Open Database Export/Import Manager",  pluginMenu= "Administrator")
@@ -286,7 +288,7 @@ def openExportImport():
     server = selectServer("Import/Export MySQL Data")
     if server:
         tab = do_open_administrator(server)
-        if tab:   
+        if tab:
             tab.wait_server_check(4)
             tab.configuration.switch_to("DATA EXPORT / RESTORE", "Data Export")
             return 1
@@ -317,17 +319,17 @@ def tmpAdministratorShortcut():
 #-------------------------------------------------------------------------------
 @ModuleInfo.export(grt.INT, grt.STRING)
 def testAdministrator(what):
-  import wb_admin_test
-  wb_admin_test.run()
-  import sys
-  sys.exit(0) # TODO return code here
-  return 1
+    import wb_admin_test
+    wb_admin_test.run()
+    import sys
+    sys.exit(0) # TODO return code here
+    return 1
 
 
 def check_if_config_file_has_section(config_file, section):
     for line in config_file:
         if line.strip() == "[%s]"%section:
-	    return True
+            return True
     return False
 
 
@@ -351,12 +353,12 @@ def testInstanceSettingByName(what, server_instance):
             test_ssh_connection.init()
             grt.send_info("connected.")
         except Exception, exc:
-          import traceback
-          traceback.print_exc()
-          return "ERROR "+str(exc)
+            import traceback
+            traceback.print_exc()
+            return "ERROR "+str(exc)
         except:
-          print "Unknown error"
-          return "ERROR"
+            print "Unknown error"
+            return "ERROR"
 
         if not test_ssh_connection.is_ssh_connected():
             test_ssh_connection = None
@@ -379,32 +381,32 @@ def testInstanceSettingByName(what, server_instance):
         if test_ssh_connection:
             test_ssh_connection = None
         return "OK"
-    
+
     elif what == "check_privileges":
         return "ERROR"
-    
+
     elif what in ("find_config_file", "check_config_path", "check_config_section"):
         config_file = profile.config_file_path
         print "Check if %s exists in remote host" % config_file
         if not test_ssh_connection.ssh.file_exists(config_file):
-          return "ERROR File %s doesn't exist" % config_file
+            return "ERROR File %s doesn't exist" % config_file
         else:
-          print "File was found in expected location"
-        
+            print "File was found in expected location"
+
         if what == "check_config_path":
-          return "OK"
+            return "OK"
 
         section = profile.config_file_section
         cfg_file_content = ""
         print "Check if %s section exists in %s" % (section, config_file)
         try:
-          #local_file = test_ssh_connection.fetch_file(config_file)
-          cfg_file_content = test_ssh_connection.server_helper.get_file_content(path=config_file, as_admin=False, admin_password=False)
+            #local_file = test_ssh_connection.fetch_file(config_file)
+            cfg_file_content = test_ssh_connection.server_helper.get_file_content(path=config_file, as_admin=False, admin_password=False)
         except Exception, exc:
-          return "ERROR "+str(exc)
+            return "ERROR "+str(exc)
 
         if ("[" + section + "]") in cfg_file_content:
-          return "OK"
+            return "OK"
         return "ERROR Couldn't find section %s in the remote config file %s" % (section, config_file)
 
     elif what in ("find_config_file/local", "check_config_path/local", "check_config_section/local"):
@@ -412,23 +414,23 @@ def testInstanceSettingByName(what, server_instance):
         config_file = wb_admin_control.WbAdminControl(profile, connect_sql=False).expand_path_variables(config_file)
         print "Check if %s can be accessed" % config_file
         if os.path.exists(config_file):
-          print "File was found at the expected location"
+            print "File was found at the expected location"
         else:
-          return "ERROR File %s doesn't exist" % config_file
+            return "ERROR File %s doesn't exist" % config_file
 
         if what == "check_config_path/local":
-          return "OK"
+            return "OK"
 
         section = profile.config_file_section
         print "Check if section for instance %s exists in %s" % (section, config_file)
         if check_if_config_file_has_section(open(config_file, "r"), section):
-          print "[%s] section found in configuration file" % section
-          return "OK"
+            print "[%s] section found in configuration file" % section
+            return "OK"
         return "ERROR Couldn't find section [%s] in the config file %s" % (section, config_file)
 
     elif what == "find_error_files":
         return "ERROR"
-      
+
     elif what == "check_admin_commands":
         path = profile.start_server_cmd
         cmd_start= None
@@ -447,7 +449,7 @@ def testInstanceSettingByName(what, server_instance):
         print "Checking command '%s'" % command
         #rc = test_ssh_connection.is_running()
         rc = test_ssh_connection.server_control.get_status()
-        print "Server detected as %s" % (rc and "running" or "stopped"), 
+        print "Server detected as %s" % (rc and "running" or "stopped"),
 
         return "OK"
 
@@ -476,11 +478,11 @@ def testInstanceSettingByName(what, server_instance):
 
 
 @ModuleInfo.export(grt.DICT, grt.classes.db_mgmt_ServerInstance)
-def detectInstanceSettings(server_instance):    
+def detectInstanceSettings(server_instance):
     #form = Form()
-    
+
     #form.run(None, None)
-    
+
     return {}
 
 
@@ -488,4 +490,3 @@ def detectInstanceSettings(server_instance):
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_ServerInstance)
 def testInstanceSettings(server_instance):
     return 0
-
