@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -15,15 +15,19 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 # 02110-1301  USA
 
+from __future__ import with_statement
+
 import platform
 import threading
 import time
+import ConfigParser
 
 from mforms import App, Utilities, newBox, newPanel, newButton, newLabel, newTabView, newTabSwitcher, newTextEntry, newSelector
 from mforms import newTaskSidebar
 import mforms
 
 import wb_admin_monitor
+import wb_admin_config_file_be
 
 from wb_admin_configuration_startup import WbAdminConfigurationStartup
 from wb_admin_config_file_ui import WbAdminConfigFileUI
@@ -31,13 +35,14 @@ from wb_admin_connections import WbAdminConnections
 from wb_admin_variables import WbAdminVariables
 from wb_admin_security import WbAdminSecurity
 from wb_admin_logs import WbAdminLogs
-from wb_admin_export import WbAdminExport
+from wb_admin_export import WbAdminExport, WbAdminImport
 from wb_admin_utils import MySQLConnection, MySQLError, weakcb, no_remote_admin_warning_label, dprint_ex
 from wb_server_management import wbaOS
 from wb_admin_ui_profile import UIProfile
 
 import new
 
+
 #-------------------------------------------------------------------------------
 #def add_top_spacer(panel):
 #    box = newBox(False)
@@ -71,6 +76,7 @@ class ServerStatusPage(mforms.Box):
         mforms.Box.__init__(self, False) # Make vertical box
         main_view.ui_profile.apply_style(self, 'page')
         self.set_managed()
+        self.set_release_on_add()
 
         self.ctrl_be = ctrl_be
         self.main_view = main_view
@@ -96,7 +102,7 @@ class ServerStatusPage(mforms.Box):
     def refresh(self, status):
         self.status.refresh_status(status)
         if status and self.connections:
-          self.connections.refresh_mt(self.ctrl_be)
+            self.connections.refresh_mt(self.ctrl_be)
 
     #---------------------------------------------------------------------------
     def page_activated(self):
@@ -119,7 +125,7 @@ class WbAdminMainView(mforms.Box):
         self.tasks_side                 = newTaskSidebar()
         self.content_box                = newBox(False)
 
-        self.tasks_side.set_selection_color(mforms.SystemHighlight)
+        self.tasks_side.set_selection_color(mforms.SystemColorHighlight)
 
         self.tabs                       = []
         self.name2page                  = {}
@@ -136,6 +142,7 @@ class WbAdminMainView(mforms.Box):
 
         # Setup self
         self.set_managed()
+        self.set_release_on_add()
         self.ui_profile.apply_style(self, "main")
 
         if server_profile.host_os == wbaOS.windows:
@@ -152,7 +159,7 @@ class WbAdminMainView(mforms.Box):
             vbox  = newBox(False)
             vbox.add(self.tasks_side, True, True)
             self.add(vbox, False, False)
-            
+
             vbox  = newBox(False)
             self.content_label = newHeaderLabel("")
             self.ui_profile.apply_style(self.content_label, 'content-label')
@@ -172,6 +179,36 @@ class WbAdminMainView(mforms.Box):
         # Setup content box
         self.content_box.add(self.tabview, True, True)
 
+        # Retrieve from server the log file paths if exist
+        status = self.ctrl_be.is_server_running(verbose=0)
+        if status in ['stopped', 'unknown'] and not all(
+                [ self.server_profile.general_log_file_path,  # only proceed to parse the config file if
+                  self.server_profile.slow_log_file_path,     # any of these is missing
+                  self.server_profile.error_log_file_path,
+                  self.server_profile.log_output ] ):
+
+            cfg_be = wb_admin_config_file_be.WbAdminConfigFileBE(self.server_profile, self.ctrl_be)
+            cfg_be.open_configuration_file(self.server_profile.config_file_path)
+            options = dict(cfg_be.get_options(self.server_profile.config_file_section))
+
+            if not self.server_profile.log_output and options.has_key('log-output'):
+                self.server_profile.log_ouput = options['log-output']
+
+            if not self.server_profile.general_log_file_path:
+                path = options['general_log_file'] if options.has_key('general_log_file') else (
+                       options['log'] if options.has_key('log') else '')  # the 'log' option is deprecated but still allowed
+                if path:
+                    self.server_profile.general_log_file_path = path.strip('"')
+
+            if not self.server_profile.slow_log_file_path:
+                path = options['slow_query_log_file'] if options.has_key('slow_query_log_file') else (
+                       options['log-slow-queries'] if options.has_key('log-slow-queries') else '')  # the 'log-slow-queries' option is deprecated but still allowed
+                if path:
+                    self.server_profile.slow_log_file_path = path.strip('"')
+
+            if not self.server_profile.error_log_file_path and options.has_key('log-error'):
+                self.server_profile.error_log_file_path = options['log-error'].strip('"')
+
         # Create content pages
         self.server_status_page = ServerStatusPage(server_profile, ctrl_be, self)
         self.config_ui = WbAdminConfigFileUI(server_profile = server_profile, ctrl_be = ctrl_be, main_view = self)
@@ -180,6 +217,10 @@ class WbAdminMainView(mforms.Box):
         self.variables = WbAdminVariables(self.ctrl_be, server_profile, self)
         self.logs = WbAdminLogs(self.ctrl_be, server_profile, self)
         self.dump = WbAdminExport(server_profile, self.ctrl_be, self)
+        self.restore = WbAdminImport(server_profile, self.ctrl_be, self)
+
+        #from wb_admin_meb import WbAdminEnterpriseBackup
+        #self.meb = WbAdminEnterpriseBackup(server_profile, self.ctrl_be, self)
 
         Utilities.add_timeout(0.5, weakcb(self, "timeout"))
         self.timeout_thread = threading.Thread(target = self.refresh_tasks_thread)
@@ -214,39 +255,42 @@ class WbAdminMainView(mforms.Box):
         self.add_taskbar_section("CONFIGURATION")
         self.add_taskbar_section("SECURITY")
         self.add_taskbar_section("DATA EXPORT / RESTORE")
+        #self.add_taskbar_section("ENTERPRISE BACKUP")
 
     #---------------------------------------------------------------------------
     def section_clicked(self, section):
-      i, content = self.name2page.get(section, (None, None))
-      #print section, i, content
-      if i is not None:
-        self.tabview.set_active_tab(i)
-        self.tab_changed()
+        i, content = self.name2page.get(section, (None, None))
+        #print section, i, content
+        if i is not None:
+            self.tabview.set_active_tab(i)
+            #set_active_tab() already triggers a tab_changed event
+            #self.tab_changed()
 
     #---------------------------------------------------------------------------
     def switch_to(self, section, task):
-      self.tasks_side.select_entry(section, task)
-      self.tab_changed()
+        self.tasks_side.select_entry(section, task)
+        self.tab_changed()
 
     #---------------------------------------------------------------------------
     def tab_changed(self):
-      if self.old_active_tab and hasattr(self.old_active_tab, "page_deactivated"):
-        self.old_active_tab.page_deactivated()
+        if self.old_active_tab and hasattr(self.old_active_tab, "page_deactivated"):
+            self.old_active_tab.page_deactivated()
 
-      i = self.tabview.get_active_tab()
-      panel = self.tabs[i]
-      if panel is not None:
-        panel.page_activated()
-      self.old_active_tab = panel
+        i = self.tabview.get_active_tab()
+        panel = self.tabs[i]
+        if panel is not None:
+            panel.page_activated()
+        self.old_active_tab = panel
 
     #---------------------------------------------------------------------------
     def shutdown(self):
         dprint_ex(2, " closing")
+        self.closing = True
         for tab in self.tabs:
             if hasattr(tab, "shutdown"):
-                tab.shutdown()
-
-        self.closing = True
+                res = tab.shutdown()
+                if res is False:  # It has to explicitely return False to cancel shutdown
+                    self.closing = False
 
     #---------------------------------------------------------------------------
     def shutdown_event(self):
@@ -254,84 +298,85 @@ class WbAdminMainView(mforms.Box):
 
     #---------------------------------------------------------------------------
     def server_started_event(self):
-      dprint_ex(1, "Handling start event")
-      if len(self.tabs) > 0 and hasattr(self.tabs[0], 'print_output'):
-        self.ctrl_be.uitask(self.tabs[0].print_output, "Server is running")
-      ###self.connect_mysql()
-      #if None == self.connect_mysql():
-      self.refresh_tasks_sleep_time = 2
-      dprint_ex(1, "Done handling start event")
+        dprint_ex(1, "Handling start event")
+        if len(self.tabs) > 0 and hasattr(self.tabs[0], 'print_output'):
+            self.ctrl_be.uitask(self.tabs[0].print_output, "Server is running")
+        ###self.connect_mysql()
+        #if None == self.connect_mysql():
+        self.refresh_tasks_sleep_time = 2
+        dprint_ex(1, "Done handling start event")
 
     #---------------------------------------------------------------------------
     def server_stopped_event(self):
-      dprint_ex(1, "Handling stop event")
-      if len(self.tabs) > 0 and hasattr(self.tabs[0], "print_output"):
-        self.ctrl_be.uitask(self.tabs[0].print_output, "Server is stopped")
+        dprint_ex(1, "Handling stop event")
+        if len(self.tabs) > 0 and hasattr(self.tabs[0], "print_output"):
+            self.ctrl_be.uitask(self.tabs[0].print_output, "Server is stopped")
 
-      self.refresh_tasks_sleep_time = 3
-      dprint_ex(1, "Done handling stop event")
+        self.refresh_tasks_sleep_time = 3
+        dprint_ex(1, "Done handling stop event")
 
     #---------------------------------------------------------------------------
     def refresh_tasks_thread(self):
-      dprint_ex(2, "Enter")
-      number_of_wakes_between_refreshes = 3
-      cnt = 0
+        dprint_ex(2, "Entering refresh tasks thread")
+        number_of_wakes_between_refreshes = 3
+        cnt = 0
 
-      only_status_check = 0
+        only_status_check = 0
 
-      while not self.closing:
-        status = "unknown"
+        while not self.closing:
+            status = "unknown"
 
-        try:
-          status = self.ctrl_be.is_server_running(verbose=0)
-        except Exception, exc:
-          import traceback
-          traceback.print_exc()
-          print "exception getting server status: %s" % exc
+            try:
+                status = self.ctrl_be.is_server_running(verbose=0)
+            except Exception, exc:
+                import traceback
+                traceback.print_exc()
+                print "exception getting server status: %s" % exc
 
-        control_event = None
-        if self.last_server_status != status:
-          if status == "running":
-            control_event = "server_started"
-          elif status == "stopped":
-            control_event = "server_stopped"
+            control_event = None
+            if self.last_server_status != status:
+                if status == "running":
+                    control_event = "server_started"
+                elif status == "stopped":
+                    control_event = "server_stopped"
 
-        if control_event:
-          self.ctrl_be.event_from_main(control_event)
+            if control_event:
+                self.ctrl_be.event_from_main(control_event)
 
-        dprint_ex(3, "server running", status, ", self.closing =", self.closing)
-        if self.last_server_status != status or only_status_check == 0:
-          dprint_ex(2, "Performing extra actions")
-          if self.server_status_page:
-            self.server_status_page.refresh(status)
+            dprint_ex(3, "server running", status, ", self.closing =", self.closing)
+            if self.last_server_status != status or only_status_check == 0:
+                dprint_ex(2, "Performing extra actions")
+                if self.server_status_page:
+                    self.server_status_page.refresh(status)
 
-        self.last_server_status = status
+            self.last_server_status = status
 
 
-        time.sleep(self.refresh_tasks_sleep_time)
-        cnt += 1
-        only_status_check = cnt % number_of_wakes_between_refreshes
+            time.sleep(self.refresh_tasks_sleep_time)
+            cnt += 1
+            only_status_check = cnt % number_of_wakes_between_refreshes
 
-      dprint_ex(2, "Leave")
+        dprint_ex(2, "Leaving refresh tasks thread")
 
     #---------------------------------------------------------------------------
     def timeout(self):
-      if not self.closing:
-        self.ctrl_be.process_ui_task_queue()
-      return not self.closing
+        if not self.closing:
+            self.ctrl_be.process_ui_task_queue()
+        return not self.closing
 
 
     #---------------------------------------------------------------------------
     def add_content_page(self, content, section_name, item_name, icon_name_as_cmd, return_content_container = False):
-        # strip _win prefix for osx icons, because we do not install _win icons in osx
-        if self.server_profile.host_os == wbaOS.darwin:
+            # strip _win prefix for osx icons, because we do not install _win icons in osx
+        import sys
+        if sys.platform == "darwin":
             if icon_name_as_cmd[-4:] == "_win":
                 icon_name_as_cmd = icon_name_as_cmd[:-4]
 
         container = content
         if return_content_container:
-          container = newBox(False)
-          container.add_end(content, True, True)
+            container = newBox(False)
+            container.add_end(content, True, True)
 
         self.tasks_side.add_section_entry(section_name, item_name, icon_name_as_cmd + ".png", icon_name_as_cmd, False)
         i = self.tabview.add_page(container, "")
