@@ -20,7 +20,7 @@ from wb_server_management import wbaOS
 #===============================================================================
 #
 #===============================================================================
-class UIProfile:
+class UIProfile(object):
     def __init__(self, server_profile):
         self.server_profile = server_profile
         self.host_os = server_profile.host_os
@@ -39,10 +39,10 @@ class UIProfile:
                                                   lambda x: x.set_color("#ffffff"),
                                                   lambda x: x.set_size(-1, 25)
                                                  ),
-                               'subsection-label' : (lambda x: x.set_back_color("#d9e2ef"),
+                               'subsection-label' : (lambda x: x.set_back_color("#ffffff"),
                                                      lambda x: x.set_color("#000000")
                                                     ),
-                               'option-search-panel' : (lambda x: x.set_back_color("#bdc7de"), 
+                               'option-search-panel' : (lambda x: x.set_back_color("#bdc7de"),
                                                         lambda x: x.set_size(-1, 26)
                                                        )
                              },
@@ -69,7 +69,7 @@ class UIProfile:
                              },
                         wbaOS.darwin :
                              {
-                               'main' : (lambda x: x.set_spacing(1), 
+                               'main' : (lambda x: x.set_spacing(1),
                                          lambda x: x.set_back_color("#bbbbbb")),
                                'page' : (lambda x: None, ),
                                'sidebar-label' : (lambda x: x.set_back_color("#efefef"),
