@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,29 +22,34 @@
 
 #include "wb_editor_storednote.h"
 #include "GrtTemplates.h"
+#include "DelegateWrapper.h"
+
+#include "mforms/code_editor.h"
 #include "base/string_utilities.h"
 
 #pragma make_public(::StoredNoteEditorBE)
 
-#ifdef _MSC_VER
 using namespace MySQL::Grt;
 using namespace System;
 using namespace System::Collections::Generic;
-#endif
 
 namespace MySQL {
 namespace Grt {
 
+public delegate void TextChangeDelegate(int line, int linesAdded);
+
 public ref class StoredNoteEditorBE : public BaseEditor
 {
 private:
   bool _is_script;
+
 protected:
   StoredNoteEditorBE(::StoredNoteEditorBE *inn)
     : BaseEditor(inn)
   {}
 
 public:
+
   StoredNoteEditorBE::StoredNoteEditorBE(MySQL::Grt::GrtManager^ grtm, MySQL::Grt::GrtValue^ arglist)
   : BaseEditor(
       new ::StoredNoteEditorBE(grtm->get_unmanaged_object(), 
@@ -52,25 +57,25 @@ public:
           )
       )
   {
-    grt::ValueRef value= grt::BaseListRef::cast_from(arglist->get_unmanaged_object()).get(0);
-    _is_script= grt::ObjectRef::cast_from(value).is_instance("db.Script");
+    grt::ValueRef value = grt::BaseListRef::cast_from(arglist->get_unmanaged_object()).get(0);
+    _is_script = grt::ObjectRef::cast_from(value).is_instance("db.Script");
+  }
+
+  StoredNoteEditorBE::~StoredNoteEditorBE()
+  {
   }
 
   ::StoredNoteEditorBE *get_unmanaged_object()
   { return static_cast<::StoredNoteEditorBE *>(inner); }
 
-  void set_text(String ^text)
+  void commit_changes()
   {
-    get_unmanaged_object()->set_text(NativeToCppString(text));
+    get_unmanaged_object()->commit_changes();
   }
 
-  String^ get_text([Out] bool %isutf8)
+  void load_text()
   {
-    bool tmp;
-    String ^str;
-    str= CppStringToNative(get_unmanaged_object()->get_text(tmp));
-    isutf8= tmp;
-    return str;
+    get_unmanaged_object()->load_text();
   }
 
   void set_name(String ^name)
@@ -105,6 +110,16 @@ public:
   {
     return _is_script;
   }
+
+  /**
+   * Returns a flag showing if the editor text was changed since last time it was applied to the
+   * backend object. It does *not* indicate if the backend object is dirty.
+   * Usually used to enable/disable Apply/Cancel buttons.
+   */
+  bool is_editor_dirty()
+  {
+    return get_unmanaged_object()->is_editor_dirty();
+  }
 };
 
 } // namespace Grt
